@@ -19,6 +19,7 @@
 #define _TAU_MALLOC_H_
 #define _MALLOC_H 1 
 
+#include <stdlib.h>
 #include <sys/types.h>
 /* needed for Linux stdlib.h */
 #define __malloc_and_calloc_defined 
@@ -26,14 +27,18 @@
 #ifdef __cplusplus
 extern "C" {
 #endif /* __cplusplus */
-	/*
+
 void *malloc (size_t size);
+void *Tau_malloc_C( const char *file, int line, size_t size);
+void Tau_free_C(const char *file, int line, void *p);
+void *Tau_realloc_C(const char *file, int line, void *p, size_t size);
 void free(void *p);
 
 void *calloc(size_t nmemb, size_t size);
+void *Tau_calloc_C(const char *file, int line, size_t nmemb, size_t size);
 void *realloc(void *ptr, size_t size);
 
-*/
+
 #ifdef __cplusplus
 }
 #endif /* __cplusplus */
@@ -51,17 +56,23 @@ void *realloc(void *ptr, size_t size);
 /********************************************************************/
 
 #include <sys/types.h>
-#ifndef __cplusplus
+#ifndef TAU_USE_CXX_MALLOC_API
 /* For C */ 
 
-void *Tau_malloc_C( const char *file, int line, size_t size);
+/* void *Tau_malloc_C( const char *file, int line, size_t size);
+*/
 #define malloc(size) Tau_malloc_C(__FILE__, __LINE__, size)
 
-void free (void *);
-void Tau_free_C(const char *file, int line, void *p);
+/* void free (void *);
+  void Tau_free_C(const char *file, int line, void *p);
+*/
 #define free(p) Tau_free_C(__FILE__, __LINE__, p)
 
-#else
+#define realloc(p, s) Tau_realloc_C(__FILE__, __LINE__, p, s)
+
+#define calloc(n, s) Tau_calloc_C(__FILE__, __LINE__, n, s)
+
+#else /* TAU_USE_CXX_MALLOC_API */
 /* For C++ */
 
 class TauVoidPointer {
@@ -75,9 +86,10 @@ TauVoidPointer Tau_malloc(const char *file, int line, size_t size);
 void Tau_free(const char *file, int line, TauVoidPointer p);
 
 #define malloc(size) Tau_malloc(__FILE__, __LINE__, size)
+#define calloc(nmemb, size) Tau_calloc(__FILE__, __LINE__, nmemb, size)
 #define free(p) Tau_free(__FILE__, __LINE__, p)
 
-#endif /* __cplusplus */
+#endif /* TAU_USE_CXX_MALLOC_API */
 
 
 
