@@ -18,14 +18,17 @@
 
 #ifdef TAU_PAPI
 
-extern "C" {
-#include "papi.h"
-}
+
+#define TAU_PAPI_MAX_COMPONENTS 4
+
+#define MAX_PAPI_COUNTERS TAU_MAX_COUNTERS
 
 struct ThreadValue {
   int ThreadID;
-  int EventSet; 
+  int EventSet[TAU_PAPI_MAX_COMPONENTS]; 
+  int NumEvents[TAU_PAPI_MAX_COMPONENTS];
   long long *CounterValues;
+  int Comp2Metric[TAU_PAPI_MAX_COMPONENTS][MAX_PAPI_COUNTERS];
 };
 
 
@@ -46,13 +49,7 @@ private:
   static bool papiInitialized;
   static ThreadValue *ThreadList[TAU_MAX_THREADS];
   static int numCounters;
-#ifdef TAU_MULTIPLE_COUNTERS
-  static int counterList[MAX_TAU_COUNTERS];
-#else
-  static int counterList[1];
-#endif
-
-
+  static int counterList[TAU_MAX_COUNTERS];
 };
 
 #endif /* TAU_PAPI */
