@@ -1,6 +1,6 @@
 /****************************************************************************
 **			TAU Portable Profiling Package			   **
-**			http://www.acl.lanl.gov/tau		           **
+**			http://www.cs.uoregon.edu/research/tau	           **
 *****************************************************************************
 **    Copyright 1997  						   	   **
 **    Department of Computer and Information Science, University of Oregon **
@@ -9,21 +9,8 @@
 /***************************************************************************
 **	File 		: ProfileHeaders.h			          **
 **	Description 	: TAU Profiling Package include files	    	  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**			  -DSGI_TIMERS  for SGI fast nanosecs timer	  **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
-**	Documentation	: See http://www.acl.lanl.gov/tau	          **
+*	Contact		: tau-team@cs.uoregon.edu 		 	  **
+**	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 #ifndef _PROFILE_HEADERS_H_
 #define _PROFILE_HEADERS_H_
@@ -66,7 +53,7 @@ using std::type_info;
 
 #endif /* _PROFILE_HEADERS_H_ */
 /***************************************************************************
- * $RCSfile: ProfileHeaders.h,v $   $Author: sameer $
- * $Revision: 1.6 $   $Date: 2002/01/24 23:27:47 $
- * POOMA_VERSION_ID: $Id: ProfileHeaders.h,v 1.6 2002/01/24 23:27:47 sameer Exp $ 
+ * $RCSfile: ProfileHeaders.h,v $   $Author: amorris $
+ * $Revision: 1.8 $   $Date: 2009/01/16 00:46:32 $
+ * POOMA_VERSION_ID: $Id: ProfileHeaders.h,v 1.8 2009/01/16 00:46:32 amorris Exp $ 
  ***************************************************************************/
