@@ -11,6 +11,19 @@
 //-----------------------------------------------------------------------------
 //
 // $Log: PyDatabase.h,v $
+// Revision 1.4  2008/10/24 22:48:46  sameer
+// Added pytau.exit(msg) binding.
+//
+// Revision 1.3  2008/09/15 23:27:45  sameer
+// Added pytau_setNode(<node>) API call:
+// import pytau
+// pytau.setNode(353)
+//
+// 	creates profile.353.0.0 file.
+// Scott,
+// 	Would you please document this in the users guide?
+// 	Thanks!
+//
 // Revision 1.2  2007/03/02 02:35:37  amorris
 // Added snapshot API for python
 //
@@ -39,6 +52,11 @@ extern char pytau_snapshot__doc__[];
 extern "C"
 PyObject * pytau_snapshot(PyObject *, PyObject *);
 
+extern char pytau_exit__name__[];
+extern char pytau_exit__doc__[];
+extern "C"
+PyObject * pytau_exit(PyObject *, PyObject *);
+
 extern char pytau_dbDump__name__[];
 extern char pytau_dbDump__doc__[];
 extern "C"
@@ -84,9 +102,14 @@ extern char pytau_dumpFuncValsIncr__doc__[];
 extern "C"
 PyObject * pytau_dumpFuncValsIncr(PyObject *, PyObject *);
 
+extern char pytau_setNode__name__[];
+extern char pytau_setNode__doc__[];
+extern "C"
+PyObject * pytau_setNode(PyObject *, PyObject *);
+
 #endif // pytau_database_h
 
 // version
-// $Id: PyDatabase.h,v 1.2 2007/03/02 02:35:37 amorris Exp $
+// $Id: PyDatabase.h,v 1.4 2008/10/24 22:48:46 sameer Exp $
 
 // End of file
