@@ -2,27 +2,14 @@
 **			TAU Portable Profiling Package			   **
 **			http://www.cs.uoregon.edu/research/tau	           **
 *****************************************************************************
-**    Copyright 1997  						   	   **
+**    Copyright 1997-2009					   	   **
 **    Department of Computer and Information Science, University of Oregon **
 **    Advanced Computing Laboratory, Los Alamos National Laboratory        **
 ****************************************************************************/
 /***************************************************************************
 **	File 		: UserEvent.h					  **
 **	Description 	: TAU Profiling Package				  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+*	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -32,6 +19,7 @@
 
 #define TAU_EVENT_DATATYPE  double
 
+class TauContextUserEvent;
 class TauUserEvent {
     
   public: 
@@ -45,6 +33,7 @@ class TauUserEvent {
     bool DisableMin, DisableMax, DisableMean, DisableStdDev, MonotonicallyIncreasing;
     string EventName;
     long EventId;
+    TauContextUserEvent *ctxevt;
 
     void AddEventToDB();
     TauUserEvent();
@@ -52,6 +41,7 @@ class TauUserEvent {
     TauUserEvent(TauUserEvent& );
     TauUserEvent& operator= (const TauUserEvent& );
     void TriggerEvent(TAU_EVENT_DATATYPE data, int tid = RtsLayer::myThread());
+    void TriggerEvent(TAU_EVENT_DATATYPE data, int tid, double timestamp, int use_ts);
     ~TauUserEvent();
     TAU_EVENT_DATATYPE GetMin(int tid = RtsLayer::myThread());
     TAU_EVENT_DATATYPE GetMax(int tid = RtsLayer::myThread());
@@ -81,7 +71,10 @@ class TauContextUserEvent {
     TauContextUserEvent(const char * EName, bool MonoIncr=false);
     ~TauContextUserEvent(); 
     void SetDisableContext(bool value);
-    void TriggerEvent( TAU_EVENT_DATATYPE data, int tid = RtsLayer::myThread());
+    void TriggerEvent(TAU_EVENT_DATATYPE data, int tid = RtsLayer::myThread());
+    void TriggerEvent(TAU_EVENT_DATATYPE data, int tid, double timestamp, int use_ts);
+    const char *GetEventName(void); 
+    TauUserEvent *contextevent;
   private:
     bool DisableContext;
     TauUserEvent *uevent;
@@ -90,34 +83,9 @@ class TauContextUserEvent {
 
 
 TAU_STD_NAMESPACE vector<TauUserEvent*>& TheEventDB(void);
-/*    
-#ifdef PROFILING_ON
-#define TAU_REGISTER_EVENT(event, name)  	TauUserEvent event(name);
-#define TAU_EVENT(event, data) 		 	(event).TriggerEvent(data);
-#define TAU_EVENT_DISABLE_MIN(event) 		(event).SetDisableMin(true);
-#define TAU_EVENT_DISABLE_MAX(event) 		(event).SetDisableMax(true);
-#define TAU_EVENT_DISABLE_MEAN(event) 		(event).SetDisableMean(true);
-#define TAU_EVENT_DISABLE_STDDEV(event) 	(event).SetDisableStdDev(true);
-#define TAU_REPORT_STATISTICS()			TauUserEvent::ReportStatistics()
-#define TAU_REPORT_THREAD_STATISTICS()		TauUserEvent::ReportStatistics(true)
-
-#else // PROFILING is disabled
-#define TAU_REGISTER_EVENT(event, name)
-#define TAU_EVENT(event, data)
-#define TAU_EVENT_DISABLE_MIN(event)
-#define TAU_EVENT_DISABLE_MAX(event)
-#define TAU_EVENT_DISABLE_MEAN(event)
-#define TAU_EVENT_DISABLE_STDDEV(event)
-#define TAU_STORE_ALL_EVENTS
-#define TAU_REPORT_STATISTICS()
-#define TAU_REPORT_THREAD_STATISTICS()
-
-
-#endif // PROFILING_ON 
-*/
 
 /***************************************************************************
  * $RCSfile: UserEvent.h,v $   $Author: amorris $
- * $Revision: 1.12 $   $Date: 2005/11/11 03:46:48 $
- * POOMA_VERSION_ID: $Id: UserEvent.h,v 1.12 2005/11/11 03:46:48 amorris Exp $ 
+ * $Revision: 1.17 $   $Date: 2009/09/28 18:28:48 $
+ * POOMA_VERSION_ID: $Id: UserEvent.h,v 1.17 2009/09/28 18:28:48 amorris Exp $ 
  ***************************************************************************/
