@@ -9,20 +9,7 @@
 /***************************************************************************
 **	File 		: FunctionInfo.cpp				  **
 **	Description 	: TAU Profiling Package				  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**			  -DSGI_TIMERS  for SGI fast nanosecs timer	  **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+*	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -47,33 +34,24 @@ using namespace std;
 
 #if (!defined(TAU_WINDOWS))
  #include <unistd.h>
- #if (defined(POOMA_TFLOP) || !defined(TULIP_TIMERS))
-  #include <sys/time.h>
- #else
-  #ifdef TULIP_TIMERS 
-   #include "Profile/TulipTimers.h"
-  #endif //TULIP_TIMERS 
- #endif //POOMA_TFLOP
-
 #else
   #include <vector>
 #endif //TAU_WINDOWS
 
-#ifdef TRACING_ON
 #ifdef TAU_VAMPIRTRACE 
 #include <Profile/TauVampirTrace.h>
 #else /* TAU_VAMPIRTRACE */
 #ifdef TAU_EPILOG
 #include "elg_trc.h"
-#else /* TAU_EPILOG */
-#define PCXX_EVENT_SRC
-#include "Profile/pcxx_events.h"
 #endif /* TAU_EPILOG */
 #endif /* TAU_VAMPIRTRACE */
-#endif // TRACING_ON 
 
+#ifdef TAU_SILC
+#include <Profile/TauSilc.h>
+#endif
 
-bool Tau_snapshot_initialization();
+#include <Profile/TauTrace.h>
+#include <Profile/TauInit.h>
 
 //////////////////////////////////////////////////////////////////////
 // The purpose of this subclass of vector is to give us a chance to execute
@@ -83,11 +61,7 @@ bool Tau_snapshot_initialization();
 class FIvector : public vector<FunctionInfo*> {
 public: 
   ~FIvector() {
-    //printf ("FIvector destructor!\n");
-    if (TheUsingDyninst() && TheSafeToDumpData()) {
-      TAU_PROFILE_EXIT("FunctionDB destructor");
-      TheSafeToDumpData() = 0;
-    }
+    Tau_destructor_trigger();
   }
 };
 
@@ -99,13 +73,16 @@ public:
 vector<FunctionInfo*>& TheFunctionDB(void)
 { // FunctionDB contains pointers to each FunctionInfo static object
 
-  // we need the timestamp of the "start"
-  static bool flag = Tau_snapshot_initialization();
-
   // we now use the above FIvector, which subclasses vector
   //static vector<FunctionInfo*> FunctionDB;
   static FIvector FunctionDB;
 
+  static int flag = 1;
+  if (flag) {
+    flag = 0;
+    InitializeTAU();
+  }
+
   return FunctionDB;
 }
 
@@ -129,6 +106,15 @@ int& TheUsingDyninst()
   return UsingDyninst;
 }
 
+//////////////////////////////////////////////////////////////////////
+// Set when uning Compiler Instrumentation
+//////////////////////////////////////////////////////////////////////
+int& TheUsingCompInst()
+{ 
+  static int UsingCompInst=0;
+  return UsingCompInst;
+}
+
 #ifdef TAU_VAMPIRTRACE
 //////////////////////////////////////////////////////////////////////
 // Initialize VampirTrace Tracing package
@@ -145,10 +131,9 @@ int TauInitVampirTrace(void)
 //////////////////////////////////////////////////////////////////////
 // Initialize EPILOG Tracing package
 //////////////////////////////////////////////////////////////////////
-int TauInitEpilog(void)
-{
-  DEBUGPROFMSG("Calling elg_open"<<endl;);
-  elg_open();
+int TauInitEpilog(void) {
+  DEBUGPROFMSG("Calling esd_open"<<endl;);
+  esd_open();
   return 1;
 }
 #endif /* TAU_EPILOG */
@@ -157,178 +142,173 @@ int TauInitEpilog(void)
 // Member Function Definitions For class FunctionInfo
 //////////////////////////////////////////////////////////////////////
 
+
+static char *strip_tau_group(const char *ProfileGroupName) {
+  char *source = strdup(ProfileGroupName);
+  const char *find = "TAU_GROUP_";
+  char *ptr;
+
+  while (ptr = strstr(source,find)) {
+    char *endptr = ptr+strlen(find);
+    while (*endptr != '\0') {
+      *ptr++ = *endptr++;
+    }
+  }
+  return source;
+}
+
 //////////////////////////////////////////////////////////////////////
 // FunctionInfoInit is called by all four forms of FunctionInfo ctor
 //////////////////////////////////////////////////////////////////////
 void FunctionInfo::FunctionInfoInit(TauGroup_t ProfileGroup, 
-	const char *ProfileGroupName, bool InitData, int tid)
-{
+	const char *ProfileGroupName, bool InitData, int tid) {
+
+  /* Make sure TAU is initialized */
+  static int flag = 1;
+  if (flag) {
+    flag = 0;
+    InitializeTAU();
+  }
+
   //Need to keep track of all the groups this function is a member of.
-  AllGroups = ProfileGroupName;
+  AllGroups = strip_tau_group(ProfileGroupName);
+  GroupName = strdup(RtsLayer::PrimaryGroup(AllGroups).c_str());
 
-#ifdef TRACING_ON
-	GroupName = RtsLayer::PrimaryGroup(ProfileGroupName);
-#endif //TRACING_ON
+  // Since FunctionInfo constructor is called once for each function (static)
+  // we know that it couldn't be already on the call stack.
+  RtsLayer::LockDB();
+  // Use LockDB to avoid a possible race condition.
+  
+  //Add function name to the name list.
+  TauProfiler_theFunctionList(NULL, NULL, true, (const char *)GetName());
+  
+  if (InitData) {
+    for (int i=0; i < TAU_MAX_THREADS; i++) {
+      NumCalls[i] = 0;
+      SetAlreadyOnStack(false, i);
+      NumSubrs[i] = 0;
+      for(int j=0;j<Tau_Global_numCounters;j++){
+	ExclTime[i][j] = 0;
+	InclTime[i][j] = 0;
+      } 
+    }
+  }
+  
+  // Make this a ptr to a list so that ~FunctionInfo doesn't destroy it.
+  
+  for (int i=0; i<TAU_MAX_THREADS; i++) {
+    MyProfileGroup_[i] = ProfileGroup;
+  }
+  // While accessing the global function database, lock it to ensure
+  // an atomic operation in the push_back and size() operations. 
+  // Important in the presence of concurrent threads.
+  TheFunctionDB().push_back(this);
+  FunctionId = RtsLayer::GenerateUniqueId();
 
-// Since FunctionInfo constructor is called once for each function (static)
-// we know that it couldn't be already on the call stack.
-	RtsLayer::LockDB();
-// Use LockDB to avoid a possible race condition.
-
-	//Add function name to the name list.
-	Profiler::theFunctionList(NULL, NULL, true, (const char *)GetName());
-
-        if (InitData) 
-        {
-	  SetAlreadyOnStack(false, tid);
-      	  for (int i=0; i < TAU_MAX_THREADS; i++)
-   	  {
-// don't initialize NumCalls and AlreadyOnStack as there could be 
-// data corruption. Inspite of the lock, while one thread is being 
-// initialized, other thread may have started executing and setting 
-// these values? 
-     	    NumCalls[i] = 0;
-#ifdef JAVA
-	    SetAlreadyOnStack(false, i);
-#endif /* JAVA  */
-     	    NumSubrs[i] = 0;
-#ifndef TAU_MULTIPLE_COUNTERS
-	    ExclTime[i] = 0;
-       	    InclTime[i] = 0;
-#else //TAU_MULTIPLE_COUNTERS
-	    for(int j=0;j<MAX_TAU_COUNTERS;j++){
-	      ExclTime[i][j] = 0;
-	      InclTime[i][j] = 0;
-	    } 
-#endif//TAU_MULTIPLE_COUNTERS
- 	  }
-	}
-
-#ifdef PROFILE_STATS
-	SumExclSqr[tid] = 0;
-#endif //PROFILE_STATS
-
-#ifdef PROFILE_CALLS
-	ExclInclCallList = new list<pair<double, double> >();
-#endif //PROFILE_CALLS
-	// Make this a ptr to a list so that ~FunctionInfo doesn't destroy it.
-	
-	for (int i=0; i<TAU_MAX_THREADS; i++) {
-	  MyProfileGroup_[i] = ProfileGroup;
-	}
-	// While accessing the global function database, lock it to ensure
-	// an atomic operation in the push_back and size() operations. 
-	// Important in the presence of concurrent threads.
-	TheFunctionDB().push_back(this);
-#ifdef TRACING_ON
 #ifdef TAU_VAMPIRTRACE
-        static int tau_vt_init=TauInitVampirTrace();
-        string tau_vt_name(Name+" "+Type);
-	FunctionId = vt_def_region(tau_vt_name.c_str(), VT_NO_ID, VT_NO_LNO,
-		VT_NO_LNO, GroupName.c_str(), VT_FUNCTION);
-	DEBUGPROFMSG("vt_def_region: "<<tau_vt_name<<": returns "<<FunctionId<<endl;);
+  static int tau_vt_init=TauInitVampirTrace();
+  string tau_vt_name(string(Name)+" "+string(Type));
+  FunctionId = TAU_VT_DEF_REGION(tau_vt_name.c_str(), VT_NO_ID, VT_NO_LNO,
+			     VT_NO_LNO, GroupName, VT_FUNCTION);
+  DEBUGPROFMSG("vt_def_region: "<<tau_vt_name<<": returns "<<FunctionId<<endl;);
 #else /* TAU_VAMPIRTRACE */
 #ifdef TAU_EPILOG
-        static int tau_elg_init=TauInitEpilog();
-	string tau_elg_name(Name+" "+Type);
-	FunctionId = elg_def_region(tau_elg_name.c_str(), ELG_NO_ID, ELG_NO_LNO,
-		ELG_NO_LNO, GroupName.c_str(), ELG_FUNCTION);
-	DEBUGPROFMSG("elg_def_region: "<<tau_elg_name<<": returns "<<FunctionId<<endl;);
-#else /* TAU_EPILOG */
-	// FOR Tracing, we should make the two a single operation 
-	// when threads are supported for traces. There needs to be 
-	// a lock in RtsLayer that can be locked while the push_back
-	// and size operations are done (this should be atomic). 
-	// Function Id is the index into the DB vector
-	/* OLD:
-	 * FunctionId = TheFunctionDB().size();
-	 */
-	FunctionId = RtsLayer::GenerateUniqueId();
-	SetFlushEvents(tid);
+  static int tau_elg_init=TauInitEpilog();
+  string tau_elg_name(string(Name)+" "+string(Type));
+  FunctionId = esd_def_region(tau_elg_name.c_str(), ELG_NO_ID, ELG_NO_LNO,
+			      ELG_NO_LNO, GroupName, ELG_FUNCTION);
+  DEBUGPROFMSG("elg_def_region: "<<tau_elg_name<<": returns "<<FunctionId<<endl;);
+#else
+#ifdef TAU_SILC
+  static int tau_silc_init=0;
+  if (tau_silc_init == 0) {
+    SILC_InitMeasurement();
+    tau_silc_init = 1;
+  }
+  string tau_silc_name(string(Name)+" "+string(Type));
+  FunctionId =  SILC_DefineRegion( tau_silc_name.c_str(),
+				   SILC_INVALID_SOURCE_FILE,
+				   SILC_INVALID_LINE_NO,
+				   SILC_INVALID_LINE_NO,
+				   SILC_ADAPTER_COMPILER,
+				   SILC_REGION_FUNCTION
+				   );
+
+#endif /* TAU_SILC */
 #endif /* TAU_EPILOG */
 #endif /* TAU_VAMPIRTRACE */
-#endif //TRACING_ON
-	RtsLayer::UnLockDB();
-		
-        DEBUGPROFMSG("nct "<< RtsLayer::myNode() <<"," 
-	  << RtsLayer::myContext() << ", " << tid 
-          << " FunctionInfo::FunctionInfo(n,t) : Name : "<< GetName() 
-          << " Group :  " << GetProfileGroup()
-	  << " Type : " << GetType() << endl;);
-
+  TauTraceSetFlushEvents(1);
+  RtsLayer::UnLockDB();
+  
+  DEBUGPROFMSG("nct "<< RtsLayer::myNode() <<"," 
+	       << RtsLayer::myContext() << ", " << tid 
+	       << " FunctionInfo::FunctionInfo(n,t) : Name : "<< GetName() 
+	       << " Group :  " << GetProfileGroup()
+	       << " Type : " << GetType() << endl;);
+  
 #ifdef TAU_PROFILEMEMORY
-	MemoryEvent = new TauUserEvent(string(Name+" "+Type+" - Heap Memory Used (KB)").c_str());
+  MemoryEvent = new TauUserEvent(string(string(Name)+" "+Type+" - Heap Memory Used (KB)").c_str());
 #endif /* TAU_PROFILEMEMORY */
-
+  
 #ifdef TAU_PROFILEHEADROOM
-	HeadroomEvent = new TauUserEvent(string(Name+" "+Type+" - Memory Headroom Available (MB)").c_str());
+  HeadroomEvent = new TauUserEvent(string(string(Name)+" "+Type+" - Memory Headroom Available (MB)").c_str());
 #endif /* TAU_PROFILEHEADROOM */
-
+  
 #ifdef RENCI_STFF
-#ifdef TAU_MULTIPLE_COUNTERS
-    for (int t=0; t < TAU_MAX_THREADS; t++) {
-        for (int i=0; i < MAX_TAU_COUNTERS; i++) {
-            Signatures[t][i] = NULL;
-        }
+  for (int t=0; t < TAU_MAX_THREADS; t++) {
+    for (int i=0; i < TAU_MAX_COUNTERS; i++) {
+      Signatures[t][i] = NULL;
     }
- #else // TAU_MULTIPLE_COUNTERS
-    for (int t=0; t < TAU_MAX_THREADS; t++) {
-        Signatures[t] = NULL;
-    }
-#endif // TAU_MULTIPLE_COUNTERS
+  }
 #endif //RENCI_STFF
-
-	return;
+  
+  return;
 }
+
 //////////////////////////////////////////////////////////////////////
 FunctionInfo::FunctionInfo(const char *name, const char *type, 
 	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid)
-{
-      DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup 
-        << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
-
-      Name = name;
-      Type = type;
-
-      FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+	int tid) {
+  DEBUGPROFMSG("FunctionInfo::FunctionInfo: MyProfileGroup_ = " << ProfileGroup 
+	       << " Mask = " << RtsLayer::TheProfileMask() <<endl;);
+  Name = strdup(name);
+  Type = strdup(type);
+  
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const char *name, const string& type, 
-	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid)
-{
-      Name = name;
-      Type = type;
-
-      FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+			   TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
+			   int tid) {
+  Name = strdup(name);
+  Type = strdup(type.c_str());
+  
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const string& name, const char * type, 
 	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid)
-{
-      Name = name;
-      Type = type;
-
-      FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+	int tid) {
+  Name = strdup(name.c_str());
+  Type = strdup(type);
+  
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
 //////////////////////////////////////////////////////////////////////
 
 FunctionInfo::FunctionInfo(const string& name, const string& type, 
 	TauGroup_t ProfileGroup , const char *ProfileGroupName, bool InitData,
-	int tid)
-{
-
-      Name = name;
-      Type = type;
- 
-      FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
+	int tid) {
+  
+  Name = strdup(name.c_str());
+  Type = strdup(type.c_str());
+  
+  FunctionInfoInit(ProfileGroup, ProfileGroupName, InitData, tid);
 }
 
 //////////////////////////////////////////////////////////////////////
@@ -343,166 +323,166 @@ FunctionInfo::~FunctionInfo()
   TheSafeToDumpData() = 0;
 }
 
-#ifdef TAU_MULTIPLE_COUNTERS
 double * FunctionInfo::GetExclTime(int tid){
-  double * tmpCharPtr = (double *) malloc( sizeof(double) * MAX_TAU_COUNTERS);
-  for(int i=0;i<MAX_TAU_COUNTERS;i++){
+  double * tmpCharPtr = (double *) malloc( sizeof(double) * Tau_Global_numCounters);
+  for(int i=0;i<Tau_Global_numCounters;i++){
     tmpCharPtr[i] = ExclTime[tid][i];
   }
   return tmpCharPtr;
 }
 
 double * FunctionInfo::GetInclTime(int tid){
-  double * tmpCharPtr = (double *) malloc( sizeof(double) * MAX_TAU_COUNTERS);
-  for(int i=0;i<MAX_TAU_COUNTERS;i++){
+  double * tmpCharPtr = (double *) malloc( sizeof(double) * Tau_Global_numCounters);
+  for(int i=0;i<Tau_Global_numCounters;i++){
     tmpCharPtr[i] = InclTime[tid][i];
   }
   return tmpCharPtr;
 }
-#endif //TAU_MULTIPLE_COUNTERS
 
-#ifdef PROFILE_CALLS
-//////////////////////////////////////////////////////////////////////
 
-int FunctionInfo::AppendExclInclTimeThisCall(double ex, double in)
-{
-	ExclInclCallList->push_back(pair<double,double>(ex,in));
-	return 1;
+double *FunctionInfo::getInclusiveValues(int tid) {
+  printf ("TAU: Warning, potentially evil function called\n");
+  return InclTime[tid];
 }
 
-#endif //PROFILE_CALLS
+double *FunctionInfo::getExclusiveValues(int tid) {
+  printf ("TAU: Warning, potentially evil function called\n");
+  return ExclTime[tid];
+}
+
+void FunctionInfo::getInclusiveValues(int tid, double *values) {
+  for(int i=0; i<Tau_Global_numCounters; i++) {
+    values[i] = InclTime[tid][i];
+  }
+}
+
+void FunctionInfo::getExclusiveValues(int tid, double *values) {
+  for(int i=0; i<Tau_Global_numCounters; i++) {
+    values[i] = ExclTime[tid][i];
+  }
+}
+
+
 //////////////////////////////////////////////////////////////////////
-long FunctionInfo::GetFunctionId(void) 
-{
-   // To avoid data races, we use a lock if the id has not been created
-  	if (FunctionId == 0)
-	{
+long FunctionInfo::GetFunctionId(void) {
+  // To avoid data races, we use a lock if the id has not been created
+  if (FunctionId == 0) {
 #ifdef DEBUG_PROF
-  	  printf("Fid = 0! \n");
+    printf("Fid = 0! \n");
 #endif // DEBUG_PROF
-	  while (FunctionId ==0)
-	  {
-	    RtsLayer::LockDB();
-	    RtsLayer::UnLockDB();
-	  }
-	}
-	return FunctionId;
+    while (FunctionId ==0) {
+      RtsLayer::LockDB();
+      RtsLayer::UnLockDB();
+    }
+  }
+  return FunctionId;
 }
 	    
 
 //////////////////////////////////////////////////////////////////////
-void FunctionInfo::ResetExclTimeIfNegative(int tid) 
-{ /* if exclusive time is negative (at Stop) we set it to zero during
+void FunctionInfo::ResetExclTimeIfNegative(int tid) { 
+  /* if exclusive time is negative (at Stop) we set it to zero during
      compensation. This function is used to reset it to zero for single
      and multiple counters */
-  	int i;
-#ifndef TAU_MULTIPLE_COUNTERS
-	if (ExclTime[tid] < 0)
-        {
-          ExclTime[tid] = 0.0;
-        }
-#else /* TAU_MULTIPLE_COUNTERS */
-	for (i=0; i < MAX_TAU_COUNTERS; i++)
-	{
-	  if (ExclTime[tid][i] < 0)
-          {
-            ExclTime[tid][i] = 0.0; /* set each negative counter to zero */
-          }
-        }
-#endif /* TAU_MULTIPLE_COUNTERS */
-        return; 
+  int i;
+  for (i=0; i < Tau_Global_numCounters; i++) {
+    if (ExclTime[tid][i] < 0) {
+      ExclTime[tid][i] = 0.0; /* set each negative counter to zero */
+    }
+  }
+  return; 
 }
 
 
 
 //////////////////////////////////////////////////////////////////////
-void tauCreateFI(FunctionInfo **ptr, const char *name, const char *type, 
+void tauCreateFI(void **ptr, const char *name, const char *type, 
 		 TauGroup_t ProfileGroup , const char *ProfileGroupName) {
   if (*ptr == 0) {
 
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::LockEnv();
+      RtsLayer::LockDB();
 #else
-    RtsLayer::LockEnv();
+    RtsLayer::LockDB();
 #endif
     if (*ptr == 0) {
       *ptr = new FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
     }
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::UnLockEnv();
+      RtsLayer::UnLockDB();
 #else
-    RtsLayer::UnLockEnv();
+    RtsLayer::UnLockDB();
 #endif
 
   }
 }
 
-void tauCreateFI(FunctionInfo **ptr, const char *name, const string& type, 
+void tauCreateFI(void **ptr, const char *name, const string& type, 
 		 TauGroup_t ProfileGroup , const char *ProfileGroupName) {
   if (*ptr == 0) {
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::LockEnv();
+      RtsLayer::LockDB();
 #else
-    RtsLayer::LockEnv();
+    RtsLayer::LockDB();
 #endif
     if (*ptr == 0) {
       *ptr = new FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
     }
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::UnLockEnv();
+      RtsLayer::UnLockDB();
 #else
-    RtsLayer::UnLockEnv();
+    RtsLayer::UnLockDB();
 #endif
   }
 }
 
-void tauCreateFI(FunctionInfo **ptr, const string& name, const char *type, 
+void tauCreateFI(void **ptr, const string& name, const char *type, 
 		 TauGroup_t ProfileGroup , const char *ProfileGroupName) {
   if (*ptr == 0) {
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::LockEnv();
+      RtsLayer::LockDB();
 #else
-    RtsLayer::LockEnv();
+    RtsLayer::LockDB();
 #endif
     if (*ptr == 0) {
       *ptr = new FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
     }
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::UnLockEnv();
+      RtsLayer::UnLockDB();
 #else
-    RtsLayer::UnLockEnv();
+    RtsLayer::UnLockDB();
 #endif
   }
 }
 
-void tauCreateFI(FunctionInfo **ptr, const string& name, const string& type, 
+void tauCreateFI(void **ptr, const string& name, const string& type, 
 		 TauGroup_t ProfileGroup , const char *ProfileGroupName) {
   if (*ptr == 0) {
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::LockEnv();
+      RtsLayer::LockDB();
 #else
-    RtsLayer::LockEnv();
+    RtsLayer::LockDB();
 #endif
     if (*ptr == 0) {
       *ptr = new FunctionInfo(name, type, ProfileGroup, ProfileGroupName);
     }
 #ifdef TAU_CHARM
     if (RtsLayer::myNode() != -1)
-      RtsLayer::UnLockEnv();
+      RtsLayer::UnLockDB();
 #else
-    RtsLayer::UnLockEnv();
+    RtsLayer::UnLockDB();
 #endif
   }
 }
 /***************************************************************************
  * $RCSfile: FunctionInfo.cpp,v $   $Author: amorris $
- * $Revision: 1.48 $   $Date: 2007/04/20 15:35:39 $
- * POOMA_VERSION_ID: $Id: FunctionInfo.cpp,v 1.48 2007/04/20 15:35:39 amorris Exp $ 
+ * $Revision: 1.82 $   $Date: 2009/12/21 17:58:01 $
+ * VERSION_ID: $Id: FunctionInfo.cpp,v 1.82 2009/12/21 17:58:01 amorris Exp $ 
  ***************************************************************************/
