@@ -1,8 +1,8 @@
 /****************************************************************************
 **			TAU Portable Profiling Package			   **
-**			http://www.acl.lanl.gov/tau		           **
+**			http://www.cs.uoregon.edu/research/tau	           **
 *****************************************************************************
-**    Copyright 1997-2000 					      	   **
+**    Copyright 1997-2009 					      	   **
 **    Department of Computer and Information Science, University of Oregon **
 **    Advanced Computing Laboratory, Los Alamos National Laboratory        **
 ****************************************************************************/
@@ -10,21 +10,8 @@
 **	File 		: JavaThreadLayer.cpp				  **
 **	Description 	: TAU Profiling Package RTS Layer definitions     **
 **			  for supporting Java Threads 			  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**			  -DSGI_TIMERS  for SGI fast nanosecs timer	  **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
-**	Documentation	: See http://www.acl.lanl.gov/tau	          **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
+**	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
 
@@ -42,6 +29,7 @@ using namespace std;
 #endif /* TAU_DOT_H_LESS_HEADERS */
 #include <Profile/Profiler.h>
 #include <Profile/TauJava.h>
+#include <stdlib.h>
 
 
 
@@ -60,7 +48,7 @@ int 	  JavaThreadLayer::tauThreadCount = 0;
 JVMPI_RawMonitor JavaThreadLayer::tauNumThreadsLock ;
 JVMPI_RawMonitor JavaThreadLayer::tauDBMutex ;
 JVMPI_RawMonitor JavaThreadLayer::tauEnvMutex ;
-JVMPI_Interface  * JavaThreadLayer::tau_jvmpi_interface;
+JVMPI_Interface  * JavaThreadLayer::tau_jvmpi_interface = NULL;
 
 
 ////////////////////////////////////////////////////////////////////////
@@ -216,6 +204,11 @@ int JavaThreadLayer::UnLockDB(void)
 ////////////////////////////////////////////////////////////////////////
 int JavaThreadLayer::InitializeEnvMutexData(void)
 {
+  if (tau_jvmpi_interface == NULL) {
+    fprintf (stderr,"Error, TAU's jvmpi interface was not initialized properly (java -XrunTAU ...)\n");
+    fprintf (stderr,"When TAU is configured with -jdk=<dir>, it can only profile Java Programs!\n");
+    exit(-1);
+  }
   // For locking functionDB 
   tauEnvMutex =  tau_jvmpi_interface->RawMonitorCreate("Env lock");
   
@@ -273,9 +266,9 @@ jlong JavaThreadLayer::getCurrentThreadCpuTime(void) {
 
 
 /***************************************************************************
- * $RCSfile: JavaThreadLayer.cpp,v $   $Author: amorris $
- * $Revision: 1.4 $   $Date: 2005/05/20 20:30:36 $
- * TAU_VERSION_ID: $Id: JavaThreadLayer.cpp,v 1.4 2005/05/20 20:30:36 amorris Exp $
+ * $RCSfile: JavaThreadLayer.cpp,v $   $Author: khuck $
+ * $Revision: 1.8 $   $Date: 2009/03/13 00:46:56 $
+ * TAU_VERSION_ID: $Id: JavaThreadLayer.cpp,v 1.8 2009/03/13 00:46:56 khuck Exp $
  ***************************************************************************/
 
 
