@@ -14,7 +14,9 @@
 ****************************************************************************/
 
 
-#include "Profile/Profiler.h"
+#include <Profile/Profiler.h>
+#include <Profile/TauSampling.h>
+
 #ifdef TAU_DOT_H_LESS_HEADERS
 #include <iostream>
 using namespace std;
@@ -22,25 +24,41 @@ using namespace std;
 #include <iostream.h>
 #endif /* TAU_DOT_H_LESS_HEADERS */
 
+#include <stdlib.h>
+#include <stdio.h>
+
+extern "C" {
+#include "papi.h"
+}
+
+#ifdef PAPI_VERSION // 2.x doesn't have these macros
+#if (PAPI_VERSION_MAJOR(PAPI_VERSION) >= 3 && PAPI_VERSION_MINOR(PAPI_VERSION) >= 9)
+#define TAU_COMPONENT_PAPI
+#endif
+
+#if (PAPI_VERSION_MAJOR(PAPI_VERSION) >= 4)
+#define TAU_COMPONENT_PAPI
+#endif
+
+#endif
+
+#ifndef TAU_COMPONENT_PAPI
+// if there is no component papi, then we pretend that there is just one component
+#define PAPI_COMPONENT_INDEX(a) 0
+#endif
 
 //#define TAU_PAPI_DEBUG 
-#define TAU_PAPI_DEBUG_LEVEL 0
+#define TAU_PAPI_DEBUG_LEVEL 1
 // level 0 will perform backward running counter checking and output critical errors
 // level 1 will perform output diagnostic information
 // level 10 will output all counter values, for each retrieval
 
-
-#ifdef TAU_MULTIPLE_COUNTERS
-#define MAX_PAPI_COUNTERS MAX_TAU_COUNTERS
-#else
-#define MAX_PAPI_COUNTERS 1
-#endif
-
 bool PapiLayer::papiInitialized = false;
 ThreadValue *PapiLayer::ThreadList[TAU_MAX_THREADS];
 int PapiLayer::numCounters = 0;
 int PapiLayer::counterList[MAX_PAPI_COUNTERS];
 
+int tauSampEvent = 0;
 
 // Some versions of PAPI don't have these defined
 // so we'll define them to 0 and if the user tries to use them
@@ -93,76 +111,119 @@ int PapiLayer::addCounter(char *name) {
   int code, rc;
 
 #ifdef TAU_PAPI_DEBUG
-  dmesg(1, "PAPI: Adding counter %s\n", name);
+  dmesg(1, "TAU: PAPI: Adding counter %s\n", name);
 #endif
 
   rc = PAPI_event_name_to_code(name, &code);
+#ifndef TAU_COMPONENT_PAPI
+  // There is currently a bug in PAPI-C 3.9 that causes the return code to not
+  // be PAPI_OK, even if it has succeeded, for now, we will just not check.
   if (rc != PAPI_OK) {
-    fprintf (stderr, "Error: Couldn't Identify Counter '%s': %s\n", name, PAPI_strerror(rc));
+    fprintf (stderr, "TAU: Error: Couldn't Identify Counter '%s': %s\n", name, PAPI_strerror(rc));
     return -1;
   }
+#endif
   
   if ((PAPI_query_event(code) != PAPI_OK)) {
-    fprintf (stderr, "Error: Counter %s is not available!\n", name);
+    fprintf (stderr, "TAU: Error: Counter %s is not available!\n", name);
     return -1;
   }
 
   int counterID = numCounters++;
   counterList[counterID] = code;
-  return counterID;
 
+#ifdef TAU_EXP_SAMPLING
+  if (strcmp(name, TauEnv_get_ebs_source())==0) {
+    tauSampEvent = code;
+  }
+#endif
+
+
+  return counterID;
 }
 
 
 
 ////////////////////////////////////////////////////
 int PapiLayer::initializeThread(int tid) {
-  int rc;
+  int rc, i;
 
 #ifdef TAU_PAPI_DEBUG
-  dmesg(1, "PAPI: Initializing Thread Data for TID = %d\n", tid);
+  dmesg(1, "TAU: PAPI: Initializing Thread Data for TID = %d\n", tid);
 #endif
 
   if (tid >= TAU_MAX_THREADS) {
-    fprintf (stderr, "Exceeded max thread count of TAU_MAX_THREADS\n");
+    fprintf (stderr, "TAU: Exceeded max thread count of TAU_MAX_THREADS\n");
     return -1;
   }
   
   ThreadList[tid] = new ThreadValue;
   ThreadList[tid]->ThreadID = tid;
-  ThreadList[tid]->EventSet = PAPI_NULL;
   ThreadList[tid]->CounterValues = new long long[MAX_PAPI_COUNTERS];
 
-  for (int i=0; i<MAX_PAPI_COUNTERS; i++) {
+  for (i=0; i<MAX_PAPI_COUNTERS; i++) {
     ThreadList[tid]->CounterValues[i] = 0L;
   }
   
-  rc = PAPI_create_eventset(&(ThreadList[tid]->EventSet));
-  if (rc != PAPI_OK) {
-    fprintf (stderr, "Error creating PAPI event set: %s\n", PAPI_strerror(rc));
-    return -1;
+  for (i=0; i<TAU_PAPI_MAX_COMPONENTS; i++) {
+    ThreadList[tid]->NumEvents[i] = 0;
+    ThreadList[tid]->EventSet[i] = PAPI_NULL;
+    rc = PAPI_create_eventset(&(ThreadList[tid]->EventSet[i]));
+    if (rc != PAPI_OK) {
+      fprintf (stderr, "TAU: Error creating PAPI event set: %s\n", PAPI_strerror(rc));
+      return -1;
+    }
   }
-  
 
 #ifndef PAPI_VERSION
-/* PAPI 2 support goes here */
-  rc = PAPI_add_events(&(ThreadList[tid]->EventSet), counterList, numCounters);
-#elif (PAPI_VERSION_MAJOR(PAPI_VERSION) == 3)
-/* PAPI 3 support goes here */
-  rc = PAPI_add_events(ThreadList[tid]->EventSet, counterList, numCounters);
+  /* PAPI 2 support goes here */
+  rc = PAPI_add_events(&(ThreadList[tid]->EventSet[0]), counterList, numCounters);
+  if (rc != PAPI_OK) {
+    fprintf (stderr, "TAU: Error adding PAPI events: %s\n", PAPI_strerror(rc));
+    return -1;
+  }
+#elif (PAPI_VERSION_MAJOR(PAPI_VERSION) == 3) || (PAPI_VERSION_MAJOR(PAPI_VERSION) == 4)
+  /* PAPI 3 support goes here */
+  for (i=0; i<numCounters; i++) {
+    int comp = PAPI_COMPONENT_INDEX (counterList[i]);
+    rc = PAPI_add_event(ThreadList[tid]->EventSet[comp], counterList[i]);
+    if (rc != PAPI_OK) {
+      fprintf (stderr, "TAU: Error adding PAPI events: %s\n", PAPI_strerror(rc));
+      return -1;
+    }
+    
+    // this creates a mapping from 'component', and index in that component back
+    // to the original index, since we return just a single array of values
+    ThreadList[tid]->Comp2Metric[comp][ThreadList[tid]->NumEvents[comp]++] = i;
+  }
+
+
+#ifdef TAU_EXP_SAMPLING
+  int comp = PAPI_COMPONENT_INDEX (tauSampEvent);
+  int threshold = TauEnv_get_ebs_frequency();
+  TAU_VERBOSE("TAU: Setting PAPI overflow handler\n");
+  rc = PAPI_overflow(ThreadList[tid]->EventSet[comp], tauSampEvent, threshold, 0, Tau_sampling_papi_overflow_handler);
+  if (rc != PAPI_OK) {
+    fprintf (stderr, "TAU: Error adding PAPI overflow handler: %s\n", PAPI_strerror(rc));
+    return -1;
+  }
+#endif
+
+
+
 #else
 /* PAPI future support goes here */
 #error "TAU does not support this version of PAPI, please contact tau-bugs@cs.uoregon.edu"
 #endif 
-  if (rc != PAPI_OK) {
-    fprintf (stderr, "Error adding PAPI events: %s\n", PAPI_strerror(rc));
-    return -1;
+  
+  for (i=0; i<TAU_PAPI_MAX_COMPONENTS; i++) {
+    if (ThreadList[tid]->NumEvents[i] >= 1) { // if there were active counters for this component
+      rc = PAPI_start(ThreadList[tid]->EventSet[i]);
+    }
   }
   
-  rc = PAPI_start(ThreadList[tid]->EventSet);
-  
   if (rc != PAPI_OK) {
-    fprintf (stderr, "Error calling PAPI_start: %s\n", PAPI_strerror(rc));
+    fprintf (stderr, "TAU: Error calling PAPI_start: %s\n", PAPI_strerror(rc));
     return -1;
   }
   
@@ -171,63 +232,11 @@ int PapiLayer::initializeThread(int tid) {
 
 
 
-////////////////////////////////////////////////////
-long long PapiLayer::getSingleCounter(int tid) {
-
-  int rc;
-  if (!papiInitialized) {
-    rc = initializePAPI();
-    if (rc != 0) {
-      return rc;
-    }
-
-    rc = initializeSingleCounter();
-    if (rc != PAPI_OK) {
-      return rc;
-    }
-  }
-
-  if (numCounters == 0) {
-    // adding must have failed, just return
-    return 0;
-  }
-
-  if (ThreadList[tid] == NULL) {
-    rc = initializeThread(tid);
-    if (rc != 0) {
-      return rc;
-    }
-  }
-
-
-#ifdef TAU_PAPI_DEBUG
-  long long oldValue = ThreadList[tid]->CounterValues[0];
-#endif  
-
-  rc = PAPI_read(ThreadList[tid]->EventSet, ThreadList[tid]->CounterValues);
-
-#ifdef TAU_PAPI_DEBUG
-  dmesg(10, "PAPI: getSingleCounter<%d> = %lld\n", tid, ThreadList[tid]->CounterValues[0]);
-
-  long long difference = ThreadList[tid]->CounterValues[0] - oldValue;
-  dmesg(10, "PAPI: Difference = %lld\n", difference);
-  if (difference < 0) dmesg (0, "PAPI: Counter running backwards?\n");
-  dmesg(difference < 0 ? 0 : 10, "PAPI: Previous value = %lld\n", oldValue);
-  dmesg(difference < 0 ? 0 : 10, "PAPI: Current  value = %lld\n", ThreadList[tid]->CounterValues[0]);
-  dmesg(difference < 0 ? 0 : 10, "PAPI: Difference     = %lld\n", difference);
-#endif  
-
-  if (rc != PAPI_OK) {
-    fprintf (stderr, "Error reading PAPI counters: %s\n", PAPI_strerror(rc));
-    return -1;
-  }
-
-  return ThreadList[tid]->CounterValues[0];  
-}
 
 /////////////////////////////////////////////////
 long long *PapiLayer::getAllCounters(int tid, int *numValues) {
   int rc;
+  long long tmpCounters[MAX_PAPI_COUNTERS];
 
   if (!papiInitialized) {
     int rc = initializePAPI();
@@ -241,7 +250,6 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
     return NULL;
   }
 
-
   if (ThreadList[tid] == NULL) {
     rc = initializeThread(tid);
     if (rc != 0) {
@@ -251,7 +259,6 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
   
   *numValues = numCounters;
 
-
 #ifdef TAU_PAPI_DEBUG
   long long previousCounters[MAX_PAPI_COUNTERS];
   for (int i=0; i<numCounters; i++) {
@@ -259,23 +266,40 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
   }
 #endif
 
-  rc = PAPI_read(ThreadList[tid]->EventSet, ThreadList[tid]->CounterValues);
+#ifdef PTHREADS
+  if (tid != RtsLayer::myThread() ) {
+    //printf("Returning values for %d instead of %d\n", tid, RtsLayer::myThread());
+    return ThreadList[tid]->CounterValues;
+  }
+#endif /* PTHREADS */
+
+  for (int comp=0; comp<TAU_PAPI_MAX_COMPONENTS; comp++) {
+    if (ThreadList[tid]->NumEvents[comp] >= 1) { // if there were active counters for this component
+      // read eventset for this component
+      rc = PAPI_read(ThreadList[tid]->EventSet[comp], tmpCounters);  
+      // now map back to original indices
+      for (int j=0; j<ThreadList[tid]->NumEvents[comp]; j++) {
+	int index = ThreadList[tid]->Comp2Metric[comp][j];
+	ThreadList[tid]->CounterValues[index] = tmpCounters[j];
+      }
+    }
+  }
 
 #ifdef TAU_PAPI_DEBUG
   for (int i=0; i<numCounters; i++) {
     long long difference = ThreadList[tid]->CounterValues[i] - previousCounters[i];
-    dmesg(10, "PAPI: Difference[%d] = %lld\n", i, difference);
+    dmesg(10, "TAU: PAPI: Difference[%d] = %lld\n", i, difference);
     if (difference < 0) {
-      dmesg(0, "PAPI: Counter running backwards?\n");
-      dmesg(0, "PAPI: Previous value[%d] = %lld\n", i, previousCounters[i]);
-      dmesg(0, "PAPI: Current  value[%d] = %lld\n", i, ThreadList[tid]->CounterValues[i]);
-      dmesg(0, "PAPI: Difference    [%d] = %lld\n", i, difference);
+      dmesg(0, "TAU: PAPI: Counter running backwards?\n");
+      dmesg(0, "TAU: PAPI: Previous value[%d] = %lld\n", i, previousCounters[i]);
+      dmesg(0, "TAU: PAPI: Current  value[%d] = %lld\n", i, ThreadList[tid]->CounterValues[i]);
+      dmesg(0, "TAU: PAPI: Difference    [%d] = %lld\n", i, difference);
     }
   }
 #endif
 
   if (rc != PAPI_OK) {
-    fprintf (stderr, "Error reading PAPI counters: %s\n", PAPI_strerror(rc));
+    fprintf (stderr, "TAU: Error reading PAPI counters: %s\n", PAPI_strerror(rc));
     return NULL;
   }
   
@@ -287,7 +311,7 @@ long long *PapiLayer::getAllCounters(int tid, int *numValues) {
 /////////////////////////////////////////////////
 int PapiLayer::reinitializePAPI() {
 #ifdef TAU_PAPI_DEBUG
-  dmesg(1, "PapiLayer::reinitializePAPI\n");
+  dmesg(1, "TAU: PapiLayer::reinitializePAPI\n");
 #endif
   // This function is called from the fork() handler
   // We need to clean up the ThreadList and then reinitialize PAPI
@@ -305,28 +329,6 @@ int PapiLayer::reinitializePAPI() {
 }
 
 
-/////////////////////////////////////////////////
-int PapiLayer::initializeSingleCounter() {
-  
-  // This function may get called more than once if there is a fork
-  if (numCounters != 0) { 
-    return 0;
-  }
-
-  // Add the counter named by PAPI_EVENT
-  char *papi_event = getenv("PAPI_EVENT");
-  if (papi_event == NULL) {
-    fprintf (stderr, "Error - You must define the PAPI_EVENT environment variable.\n");
-    return -1;
-  }
-
-  int counterID = addCounter(papi_event);
-  if (counterID < 0) {
-    return -1;
-  }
-
-  return 0;
-}
 
 #ifdef TAU_PAPI_THREADS
 // note, this only works on linux
@@ -350,7 +352,7 @@ unsigned long papi_thread_gettid(void) {
 /////////////////////////////////////////////////
 void PapiLayer::checkDomain(int domain, char *domainstr) {
   if (domain == 0) {
-    fprintf (stderr, "Warning: PAPI domain \"%s\" is not available with this version of PAPI\n", domainstr);
+    fprintf (stderr, "TAU: Warning: PAPI domain \"%s\" is not available with this version of PAPI\n", domainstr);
   }
 }
 
@@ -358,7 +360,7 @@ void PapiLayer::checkDomain(int domain, char *domainstr) {
 /////////////////////////////////////////////////
 int PapiLayer::initializePAPI() {
 #ifdef TAU_PAPI_DEBUG
-  dmesg(1, "PapiLayer::initializePAPI\n");
+  dmesg(1, "TAU: PapiLayer::initializePAPI\n");
 #endif
 
   papiInitialized = true;
@@ -371,9 +373,9 @@ int PapiLayer::initializePAPI() {
   int papi_ver = PAPI_library_init(PAPI_VER_CURRENT);
   if (papi_ver != PAPI_VER_CURRENT) {
     if (papi_ver > 0) {
-      fprintf(stderr, "Error initializing PAPI: version mismatch: %d\n", papi_ver);
+      fprintf(stderr, "TAU: Error initializing PAPI: version mismatch: %d\n", papi_ver);
     } else {
-      fprintf(stderr, "Error initializing PAPI: %s\n", PAPI_strerror(papi_ver));
+      fprintf(stderr, "TAU: Error initializing PAPI: %s\n", PAPI_strerror(papi_ver));
     }
     return -1;
   }
@@ -390,7 +392,7 @@ int PapiLayer::initializePAPI() {
 #ifndef PAPI_VERSION
   /* PAPI 2 support goes here */
   rc = PAPI_thread_init((unsigned long (*)(void))(RtsLayer::myThread),0);
-#elif (PAPI_VERSION_MAJOR(PAPI_VERSION) == 3)
+#elif (PAPI_VERSION_MAJOR(PAPI_VERSION) == 3) || (PAPI_VERSION_MAJOR(PAPI_VERSION) == 4)
   /* PAPI 3 support goes here */
   rc = PAPI_thread_init((unsigned long (*)(void))(RtsLayer::myThread));
 #else
@@ -402,7 +404,7 @@ int PapiLayer::initializePAPI() {
 
   
   if (rc != PAPI_OK) {
-    fprintf(stderr, "Error Initializing PAPI: %s\n", PAPI_strerror(rc));
+    fprintf(stderr, "TAU: Error Initializing PAPI: %s\n", PAPI_strerror(rc));
     return -1;
   }
 #endif /* __alpha */
@@ -426,7 +428,7 @@ int PapiLayer::initializePAPI() {
       } else if (!strcmp(token,"PAPI_DOM_ALL")) {
 	thisDomain |= PAPI_DOM_ALL;
       } else {
-	fprintf (stderr, "Warning: Unknown PAPI domain, \"%s\"\n", token);
+	fprintf (stderr, "TAU: Warning: Unknown PAPI domain, \"%s\"\n", token);
       }
 
       checkDomain(thisDomain, token);
@@ -435,11 +437,11 @@ int PapiLayer::initializePAPI() {
     }
     
     if (domain == 0) {
-      fprintf (stderr, "Warning, No valid PAPI domains specified\n");
+      fprintf (stderr, "TAU: Warning, No valid PAPI domains specified\n");
     }
     rc = PAPI_set_domain(domain);
     if (rc != PAPI_OK) {
-      fprintf(stderr, "Error setting PAPI domain: %s\n", PAPI_strerror(rc));
+      fprintf(stderr, "TAU: Error setting PAPI domain: %s\n", PAPI_strerror(rc));
       return -1;
     }
   }
@@ -457,6 +459,10 @@ int PapiLayer::initializePapiLayer(bool lock) {
     return 0;
   }
 
+#ifdef TAU_PAPI_DEBUG
+  dmesg(1, "TAU: PAPI: Initializing PAPI Layer");
+#endif
+
   if (lock) RtsLayer::LockDB();
   int rc = initializePAPI();
   if (lock) RtsLayer::UnLockDB();
@@ -475,9 +481,9 @@ long long PapiLayer::getWallClockTime(void) {
   newvalue = PAPI_get_real_usec();
   if (newvalue < oldvalue) {
     offset += UINT_MAX;
-    DEBUGPROFMSG("WARNING: papi counter overflow. Fixed in TAU! new = "
+    DEBUGPROFMSG("TAU: WARNING: papi counter overflow. Fixed in TAU! new = "
 		 <<newvalue <<" old = " <<oldvalue<<" offset = "<<offset <<endl;);
-    DEBUGPROFMSG("Returning "<<newvalue + offset<<endl;);
+    DEBUGPROFMSG("TAU: Returning "<<newvalue + offset<<endl;);
   }
   oldvalue = newvalue;
   return (newvalue + offset);
