@@ -9,20 +9,7 @@
 /***************************************************************************
 **	File 		: ProfileParam.cpp				  **
 **	Description 	: TAU Profiling Package				  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -41,14 +28,7 @@
 
 #include "Profile/Profiler.h"
 
-
-#ifdef TAU_WINDOWS
-  typedef __int64 x_int64;
-  typedef unsigned __int64 x_uint64;
-#else
-  typedef long long x_int64;
-  typedef unsigned long long x_uint64;
-#endif
+#include "tau_internal.h"
 
 
 #include <stdio.h>
@@ -75,17 +55,18 @@ using namespace std;
  * they're equal. If they two arrays have the same depth, then we iterate
  * through the array and compare each array element till the end */
 /////////////////////////////////////////////////////////////////////////
-struct TaultProfileParamLong
-{
-  bool operator() (const long *l1, const long *l2) const
- {
+struct TaultProfileParamLong {
+  bool operator() (const long *l1, const long *l2) const {
    int i;
    /* first check 0th index (size) */
-   if (l1[0] != l2[0]) return (l1[0] < l2[0]);
+   if (l1[0] != l2[0]) {
+     return (l1[0] < l2[0]);
+   }
    /* they're equal, see the size and iterate */
-   for (i = 0; i < l1[0] ; i++)
-   {
-     if (l1[i] != l2[i]) return l1[i] < l2[i];
+   for (i = 1; i < l1[0] ; i++) {
+     if (l1[i] != l2[i]) {
+       return l1[i] < l2[i];
+     }
    }
    return (l1[i] < l2[i]);
  }
@@ -95,16 +76,15 @@ struct TaultProfileParamLong
 /////////////////////////////////////////////////////////////////////////
 // We use one global map to store the callpath information
 /////////////////////////////////////////////////////////////////////////
-map<TAU_PROFILE_PARAM_TYPE >& TheTimerProfileParamMap(void)
-{ // to avoid initialization problems of non-local static variables
+map<TAU_PROFILE_PARAM_TYPE >& TheTimerProfileParamMap(void) { 
+  // to avoid initialization problems of non-local static variables
   static map<TAU_PROFILE_PARAM_TYPE > timerappdatamap;
 
   return timerappdatamap;
 }
 
-long * TauCreateProfileParamArray(long FuncId, long key)
-{
-  int depth = 3; 
+long * TauCreateProfileParamArray(long FuncId, long key) {
+  int depth = 2; 
   long *retary = new long[depth+1]; 
   retary[0] = depth; /* encode the depth first */
   retary[1] = FuncId; /* the id of the current timer */
@@ -112,55 +92,50 @@ long * TauCreateProfileParamArray(long FuncId, long key)
   return retary;
 }
 
-FunctionInfo * TauGetProfileParamFI(int tid, long key, string& keyname)
-{
+FunctionInfo * TauGetProfileParamFI(int tid, long key, string& keyname) {
   /* Get the FunctionInfo Object of the current Profiler */
-  Profiler *current = Profiler::CurrentProfiler[tid];
+  Profiler *current = TauInternal_CurrentProfiler(tid);
   if (!current) return NULL; /* not in a valid profiler */
   FunctionInfo *f = current->ThisFunction; 
   if (!f) return NULL;  /* proceed if we are in a valid function */
-
+  
   /* we have a timer definition. We need to examine the key and see if
    * it has appeared before. If not, we need to create a new functionInfo 
    * and a mapping between the key and the newly created functionInfo */
   
   long *ary = TauCreateProfileParamArray((long) f, key);
-
-   /* We've set the key */
-   map<TAU_PROFILE_PARAM_TYPE >::iterator it = TheTimerProfileParamMap().find(ary);
-
-  if (it == TheTimerProfileParamMap().end())
-  {
+  
+  /* We've set the key */
+  map<TAU_PROFILE_PARAM_TYPE >::iterator it = TheTimerProfileParamMap().find(ary);
+  
+  if (it == TheTimerProfileParamMap().end()) {
     /* Couldn't find it */
     char keystr[256]; 
     sprintf(keystr, "%ld", key); 
-
+    
     string name ( f->GetName() + string(" ") + f->GetType()+ string(" [ <")
-	    +keyname+ string("> = <")+ keystr + string("> ]")); 
+		  +keyname+ string("> = <")+ keystr + string("> ]")); 
     DEBUGPROFMSG("Name created = "<<name<<endl;);
     string grname = string("TAU_PARAM | ") + RtsLayer::PrimaryGroup(f->GetAllGroups());
-
+    
     FunctionInfo *fnew = new FunctionInfo(name, " ", 
-		    f->GetProfileGroup(),
-		    (const char *)grname.c_str(), true, tid); 
+					  f->GetProfileGroup(),
+					  (const char *)grname.c_str(), true, tid); 
     TheTimerProfileParamMap().insert(map<TAU_PROFILE_PARAM_TYPE >::value_type(ary, fnew)); /* Add it to the map */
     return fnew; 
-  }
-  else
-  { /* found it. (*it).second refers to the functionInfo object corresponding
+  } else { 
+    /* found it. (*it).second refers to the functionInfo object corresponding
        to our particular instance */
     DEBUGPROFMSG("Found name = "<<(*it).second->GetName()<<endl;);
     return (*it).second; 
-
   }
-
 }
 	
-void Profiler::AddProfileParamData(long key, string& keyname)
-{
+void TauProfiler_AddProfileParamData(long key, const char *keyname) {
+  string keystring(keyname);
   int tid = RtsLayer::myThread();
-  FunctionInfo *f = TauGetProfileParamFI(tid, key, keyname);
-  Profiler *current = CurrentProfiler[tid];
+  FunctionInfo *f = TauGetProfileParamFI(tid, key, keystring);
+  Profiler *current = TauInternal_CurrentProfiler(tid);
   if (!current) return; 
   current->ProfileParamFunction = f; 
 
@@ -168,24 +143,16 @@ void Profiler::AddProfileParamData(long key, string& keyname)
   if (f->GetAlreadyOnStack(tid) == false) { /* is it on the callstack? */
     current->AddInclProfileParamFlag = true; 
     f->SetAlreadyOnStack(true, tid); // it is on callstack now 
-  }
-  else
-  {
-   current->AddInclProfileParamFlag = false; // no need to add incl time
+  } else {
+    current->AddInclProfileParamFlag = false; // no need to add incl time
   }
  
   return;  
 }
 	
 
-#ifdef TAU_MULTIPLE_COUNTERS
-void Profiler::ProfileParamStop(double* TotalTime, int tid)
-#else // single counter
-void Profiler::ProfileParamStop(double TotalTime, int tid)
-#endif // TAU_MULTIPLE_COUNTERS
-{
-  if (ProfileParamFunction)
-  {
+void Profiler::ProfileParamStop(double* TotalTime, int tid) {
+  if (ProfileParamFunction) {
     DEBUGPROFMSG("Inside ProfileParamStop "<<ThisFunction->GetName()<<endl;);
     if (AddInclProfileParamFlag == true) { // The first time it came on call stack
       ProfileParamFunction->SetAlreadyOnStack(false, tid); // while exiting
@@ -200,12 +167,11 @@ void Profiler::ProfileParamStop(double TotalTime, int tid)
     }
 
     ProfileParamFunction->AddExclTime(TotalTime, tid);  
-
   }
 }
   
 /***************************************************************************
- * $RCSfile: ProfileParam.cpp,v $   $Author: sameer $
- * $Revision: 1.2 $   $Date: 2006/04/14 04:42:00 $
- * TAU_VERSION_ID: $Id: ProfileParam.cpp,v 1.2 2006/04/14 04:42:00 sameer Exp $ 
+ * $RCSfile: ProfileParam.cpp,v $   $Author: amorris $
+ * $Revision: 1.8 $   $Date: 2009/08/12 17:35:22 $
+ * TAU_VERSION_ID: $Id: ProfileParam.cpp,v 1.8 2009/08/12 17:35:22 amorris Exp $ 
  ***************************************************************************/
