@@ -11,6 +11,15 @@
 //-----------------------------------------------------------------------------
 //
 // $Log: PyBindings.cpp,v $
+// Revision 1.6  2008/10/24 22:48:18  sameer
+// Added a pytau.exit("message") binding for TAU_PROFILE_EXIT(msg).
+//
+// Revision 1.5  2008/09/18 23:50:29  sameer
+// Added memory tracking extensions.
+//
+// Revision 1.4  2008/09/15 23:25:49  sameer
+// Added pytau_setNode(<nodeid>) number for the Python API.
+//
 // Revision 1.3  2007/03/02 02:36:51  amorris
 // Added snapshot API for python.
 //
@@ -46,6 +55,7 @@
 #include "Profile/PyDatabase.h"
 #include "Profile/PyGroups.h"
 #include "Profile/PyTimer.h"
+#include "Profile/PyMemory.h"
 
 
 // method table
@@ -55,6 +65,7 @@ struct PyMethodDef pytau_methods[] = {
 // database
 
     {pytau_snapshot__name__, pytau_snapshot, METH_VARARGS, pytau_snapshot__doc__},
+    {pytau_exit__name__, pytau_exit, METH_VARARGS, pytau_exit__doc__},
     {pytau_dbDump__name__, pytau_dbDump, METH_VARARGS, pytau_dbDump__doc__},
     {pytau_dbDumpIncr__name__, pytau_dbDumpIncr, METH_VARARGS, pytau_dbDumpIncr__doc__},
     {pytau_dbPurge__name__, pytau_dbPurge, METH_VARARGS, pytau_dbPurge__doc__},
@@ -81,12 +92,25 @@ struct PyMethodDef pytau_methods[] = {
     {pytau_profileTimer__name__, pytau_profileTimer, METH_VARARGS | METH_KEYWORDS, pytau_profileTimer__doc__}, 
     {pytau_start__name__, pytau_start, METH_VARARGS, pytau_start__doc__}, 
     {pytau_stop__name__, pytau_stop, METH_VARARGS, pytau_stop__doc__}, 
+// runtime system
+    {pytau_setNode__name__, pytau_setNode, METH_VARARGS, pytau_setNode__doc__}, 
+
+// memory events
+    {pytau_trackMemory__name__, pytau_trackMemory, METH_VARARGS, pytau_trackMemory__doc__}, 
+    {pytau_trackMemoryHeadroom__name__, pytau_trackMemoryHeadroom, METH_VARARGS, pytau_trackMemoryHeadroom__doc__}, 
+    {pytau_trackMemoryHere__name__, pytau_trackMemoryHere, METH_VARARGS, pytau_trackMemoryHere__doc__}, 
+    {pytau_trackMemoryHeadroomHere__name__, pytau_trackMemoryHeadroomHere, METH_VARARGS, pytau_trackMemoryHeadroomHere__doc__}, 
+    {pytau_enableTrackingMemory__name__, pytau_enableTrackingMemory, METH_VARARGS, pytau_enableTrackingMemory__doc__}, 
+    {pytau_disableTrackingMemory__name__, pytau_disableTrackingMemory, METH_VARARGS, pytau_disableTrackingMemory__doc__}, 
+    {pytau_enableTrackingMemoryHeadroom__name__, pytau_enableTrackingMemoryHeadroom, METH_VARARGS, pytau_enableTrackingMemoryHeadroom__doc__}, 
+    {pytau_disableTrackingMemoryHeadroom__name__, pytau_disableTrackingMemoryHeadroom, METH_VARARGS, pytau_disableTrackingMemoryHeadroom__doc__}, 
+    {pytau_setInterruptInterval__name__, pytau_setInterruptInterval, METH_VARARGS, pytau_setInterruptInterval__doc__}, 
 
 // Sentinel
     {0, 0}
 };
 
 // version
-// $Id: PyBindings.cpp,v 1.3 2007/03/02 02:36:51 amorris Exp $
+// $Id: PyBindings.cpp,v 1.6 2008/10/24 22:48:18 sameer Exp $
 
 // End of file
