@@ -11,6 +11,12 @@
 //-----------------------------------------------------------------------------
 //
 // $Log: PyDatabase.cpp,v $
+// Revision 1.8  2008/10/24 22:48:18  sameer
+// Added a pytau.exit("message") binding for TAU_PROFILE_EXIT(msg).
+//
+// Revision 1.7  2008/09/15 23:25:49  sameer
+// Added pytau_setNode(<nodeid>) number for the Python API.
+//
 // Revision 1.6  2007/03/02 02:36:51  amorris
 // Added snapshot API for python.
 //
@@ -131,6 +137,28 @@ PyObject * pytau_dbPurge(PyObject *, PyObject *)
     return Py_None;
 }
 
+char pytau_exit__name__[] = "exit";
+char pytau_exit__doc__[] = "close the callstack and flush the Tau statistics";
+PyObject * pytau_exit(PyObject *self, PyObject *args)
+{
+    char *message = "pythonexit";
+    int len = 10;
+
+    // Check to see if a prefix is specified
+    if (PyArg_ParseTuple(args, "|s", &message, &len))
+    {
+      // extracted the prefix, call dump routine
+#ifdef DEBUG
+      printf("dbDump: extracted message = %s, len = %d\n", message, len);
+#endif /* DEBUG */
+    }
+    TAU_PROFILE_EXIT(message);
+
+    // return
+    Py_INCREF(Py_None);
+    return Py_None;
+}
+
 char pytau_getFuncNames__name__[] = "getFuncNames";
 char pytau_getFuncNames__doc__[] = "get list of profiled functions";
 PyObject * pytau_getFuncNames(PyObject *, PyObject *)
@@ -328,9 +356,32 @@ PyObject * pytau_dumpFuncValsIncr(PyObject *, PyObject * args)
     return Py_None;
 }
 
+char pytau_setNode__name__[] = "setNode";
+char pytau_setNode__doc__[] = "set node number for genrating profile.<node>.0.0";
+PyObject * pytau_setNode(PyObject *self, PyObject *args)
+{
+    int node_number;
+
+    node_number = 0;
+
+    // extract function list from Python args
+    int ok = PyArg_ParseTuple(args, "i:nodeNumber", &node_number);
+    if (!ok) {
+        return 0;
+    }
+
+#ifdef DEBUG_PROF
+    printf("pytau_setnode: %d\n", node_number);
+#endif /* DEBUG_PROF */
+    TAU_PROFILE_SET_NODE(node_number);
+
+
+    Py_INCREF(Py_None);
+    return Py_None;
+}
 
 // version
-// $Id: PyDatabase.cpp,v 1.6 2007/03/02 02:36:51 amorris Exp $
+// $Id: PyDatabase.cpp,v 1.8 2008/10/24 22:48:18 sameer Exp $
 
 // End of file
   
