@@ -12,6 +12,36 @@
 //-----------------------------------------------------------------------------
 //
 // $Log: PyTimer.cpp,v $
+// Revision 1.11  2009/03/26 19:15:39  sameer
+// Updated the start/stop calls with the additional tid argument. Before:
+//
+// Tau_start_timer(void *timer, int phase)
+// {
+//   int tid = RtsLayer::myThread();
+//
+//   p->Start();  // Start internally calls tid = RtsLayer::myThread()
+// }
+//
+// After:
+//
+// Tau_start_timer(void *timer, int phase, int tid)
+// {
+//   // int tid  = RtsLayer::myThread();
+//   p->Start(tid);
+// }
+//
+// This is used extensively in the new TASK API. See examples/profilercreate/taskc++.
+//
+// Revision 1.10  2009/02/24 21:30:23  amorris
+// Getting rid of Profiler::CurrentProfiler, it doesn't make sense to maintain
+// this linked list if we have an explicit data structure for the callstack.  It's
+// replaced with a routine: TauInternal_CurrentProfiler(tid)
+//
+// Revision 1.9  2009/01/15 02:47:23  amorris
+// Changes for C++ measurement API
+//
+// Revision 1.8  2008/10/16 22:58:24  amorris
+//
 // Revision 1.7  2007/03/02 02:36:24  amorris
 // Made explicit the phase calls, true and false.
 //
@@ -156,23 +186,15 @@ PyObject * pytau_start(PyObject *self, PyObject *args)
 #endif /* DEBUG */
     int tid = RtsLayer::myThread();
     /* Get the FunctionInfo object */
-    Profiler *p = new Profiler(f,  f != (FunctionInfo *) 0 ? f->GetProfileGroup() : 
-      TAU_DEFAULT, true, tid);
-    if (p == (Profiler *) NULL) { 
-      printf("ERROR: Out of Memory in pytau_start! new returns NULL!\n");
-      return NULL;
-    }
 
-#ifdef TAU_PROFILEPHASE
     bool isPhase = phaseMap[id];
+    int phase = 0;
     if (isPhase) {
-      p->SetPhase(true);
-    } else {
-      p->SetPhase(false);
+      phase = 1;
     }
-#endif
 
-    p->Start(tid);
+    Tau_start_timer(f, phase, Tau_get_tid());
+
     Py_INCREF(Py_None);
     return Py_None;
 
@@ -197,19 +219,13 @@ PyObject * pytau_stop(PyObject *self, PyObject *args)
     int tid = RtsLayer::myThread();
     static int taunode = tau_check_and_set_nodeid();
 
-    Profiler *p = Profiler::CurrentProfiler[tid];
-    if (p != (Profiler *) NULL)
-    {
-#ifdef DEBUG
-      printf("Looking at function %s\n", p->ThisFunction->GetName());
-#endif /* DEBUG */
-      p->Stop();
-      // It was stopped properly
+    Profiler *p = TauInternal_CurrentProfiler(tid);
+
+    if (p != (Profiler *) NULL) {
+      Tau_stop_timer(p->ThisFunction, Tau_get_tid());
       Py_INCREF(Py_None);
       return Py_None;
-    }
-    else
-    {
+    } else {
       printf("pytau_stop: Stack error. Profiler is NULL!");
       return NULL;
     }
@@ -219,7 +235,7 @@ PyObject * pytau_stop(PyObject *self, PyObject *args)
 }
 
 // version
-// $Id: PyTimer.cpp,v 1.7 2007/03/02 02:36:24 amorris Exp $
+// $Id: PyTimer.cpp,v 1.11 2009/03/26 19:15:39 sameer Exp $
 
 // End of file
   
