@@ -7,23 +7,10 @@
 **    Advanced Computing Laboratory, Los Alamos National Laboratory        **
 ****************************************************************************/
 /***************************************************************************
-**	File 		: RtsThread.cpp				  **
+**	File 		: RtsThread.cpp				          **
 **	Description 	: TAU Profiling Package RTS Layer definitions     **
 **			  for supporting threads 			  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**			  -DSGI_TIMERS  for SGI fast nanosecs timer	  **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -41,11 +28,8 @@ using namespace std;
 #include "Profile/Profiler.h"
 #include "Profile/OpenMPLayer.h"
 
-#ifdef TRACING_ON
-#define PCXX_EVENT_SRC
-#include "Profile/pcxx_events.h"
+#include <Profile/TauTrace.h>
 void TraceCallStack(int tid, Profiler *current);
-#endif // TRACING_ON
 
 #include <stdio.h>
 
@@ -59,6 +43,12 @@ void TraceCallStack(int tid, Profiler *current);
 #endif //TAUKTAU_SHCTR
 #endif //TAUKTAU
 
+#include <Profile/TauSampling.h>
+
+
+int RtsLayer::lockDBcount[TAU_MAX_THREADS];
+
+
 //////////////////////////////////////////////////////////////////////
 // myNode() returns the current node id (0..N-1)
 //////////////////////////////////////////////////////////////////////
@@ -79,6 +69,11 @@ int RtsLayer::myContext(void)
   return TheContext(); 
 }
 
+extern "C" int Tau_RtsLayer_myThread(void) {
+  return RtsLayer::myThread();
+}
+
+
 //////////////////////////////////////////////////////////////////////
 // myNode() returns the current node id (0..N-1)
 //////////////////////////////////////////////////////////////////////
@@ -104,21 +99,30 @@ int RtsLayer::myThread(void)
 #endif // PTHREADS
 }
 
+int RtsLayer::setMyThread(int tid) {
+#ifdef PTHREADS
+  PthreadLayer::SetThreadId(tid);
+#endif // PTHREADS
+  return 0;
+}
 
 //////////////////////////////////////////////////////////////////////
 // RegisterThread is called before any other profiling function in a 
 // thread that is spawned off
 //////////////////////////////////////////////////////////////////////
-void RtsLayer::RegisterThread()
-{ /* Check the size of threads */
+int RtsLayer::RegisterThread() {
+  /* Check the size of threads */
   LockEnv();
   static int numthreads = 1;
   numthreads ++;
-  if (numthreads >= TAU_MAX_THREADS)
-  {
+  if (numthreads >= TAU_MAX_THREADS) {
     fprintf(stderr, "TAU: RtsLayer: Max thread limit (%d) exceeded. Please re-configure TAU with -useropt=-DTAU_MAX_THREADS=<higher limit>\n", numthreads);
   }
   UnLockEnv();
+  
+#ifdef TAU_EXP_SAMPLING
+  Tau_sampling_init(numthreads-1);
+#endif /* TAU_EXP_SAMPLING */
 
 #ifdef PTHREADS
   PthreadLayer::RegisterThread();
@@ -134,7 +138,7 @@ void RtsLayer::RegisterThread()
   PapiThreadLayer::RegisterThread();
 #endif // PTHREADS
 // Note: Java thread registration is done at the VM layer in TauJava.cpp
-  return;
+  return numthreads;
 }
 
 
@@ -142,8 +146,7 @@ void RtsLayer::RegisterThread()
 // RegisterFork is called before any other profiling function in a 
 // process that is forked off (child process)
 //////////////////////////////////////////////////////////////////////
-void RtsLayer::RegisterFork(int nodeid, enum TauFork_t opcode)
-{
+void RtsLayer::RegisterFork(int nodeid, enum TauFork_t opcode) {
 #ifdef PROFILING_ON
   vector<FunctionInfo*>::iterator it;
   Profiler *current;
@@ -170,44 +173,31 @@ void RtsLayer::RegisterFork(int nodeid, enum TauFork_t opcode)
 
   TAU_PROFILE_SET_NODE(nodeid);
   // First, set the new id 
-  if (opcode == TAU_EXCLUDE_PARENT_DATA)
-  {
+  if (opcode == TAU_EXCLUDE_PARENT_DATA) {
   // If opcode is TAU_EXCLUDE_PARENT_DATA then we clear out the 
   // previous values in the TheFunctionDB()
 
   // Get the current time
-#ifndef TAU_MULTIPLE_COUNTERS
-     double CurrentTimeOrCounts = getUSecD(myThread());
-#else //TAU_MULTIPLE_COUNTERS
-     double CurrentTimeOrCounts[MAX_TAU_COUNTERS];
-     for(int i=0;i<MAX_TAU_COUNTERS;i++){
+     double CurrentTimeOrCounts[TAU_MAX_COUNTERS];
+     for(int i=0;i<Tau_Global_numCounters;i++){
        CurrentTimeOrCounts[i]=0;
      }
      getUSecD(myThread(), CurrentTimeOrCounts);
-#endif//TAU_MULTIPLE_COUNTERS
-     for (int tid = 0; tid < TAU_MAX_THREADS; tid++)
-     { // For each thread of execution 
+     for (int tid = 0; tid < TAU_MAX_THREADS; tid++) { 
+       // For each thread of execution 
 #ifdef PROFILING_ON
-       for(it=TheFunctionDB().begin(); it!=TheFunctionDB().end(); it++)
-       { // Iterate through each FunctionDB item 
+       for(it=TheFunctionDB().begin(); it!=TheFunctionDB().end(); it++) { 
+	 // Iterate through each FunctionDB item 
 	 // Clear all values 
 	 (*it)->SetCalls(tid, 0);
 	 (*it)->SetSubrs(tid, 0);
-#ifndef TAU_MULTIPLE_COUNTERS
-	 (*it)->SetExclTime(tid, 0);
-	 (*it)->SetInclTime(tid, 0);
-#else //TAU_MULTIPLE_COUNTERS
          (*it)->SetExclTimeZero(tid);
          (*it)->SetInclTimeZero(tid);
-#endif//TAU_MULTIPLE_COUNTERS
-#ifdef PROFILE_STATS
-	 (*it)->SetSumExclSqr(tid, 0);
-#endif // PROFILE_STATS 
 	/* Do we need to change AlreadyOnStack? No*/
 	DEBUGPROFMSG("FI Zap: Inside "<< (*it)->GetName() <<endl;);
 #ifdef TAUKTAU_MERGE
-         DEBUGPROFMSG("RtsLayer::RegisterFork: GetKtauFuncInfo(tid)->ResetAllCounters(tid): Func:"<< (*it)->GetName() <<endl;);
-	 (*it)->GetKtauFuncInfo(tid)->ResetAllCounters(tid);
+	DEBUGPROFMSG("RtsLayer::RegisterFork: GetKtauFuncInfo(tid)->ResetAllCounters(tid): Func:"<< (*it)->GetName() <<endl;);
+	(*it)->GetKtauFuncInfo(tid)->ResetAllCounters(tid);
 #endif //TAUKTAU_MERGE
        }
 #ifdef TAUKTAU_MERGE
@@ -217,40 +207,35 @@ void RtsLayer::RegisterFork(int nodeid, enum TauFork_t opcode)
        DEBUGPROFMSG("RtsLayer::RegisterFork: Running-Up Stack\n");
        // Now that the FunctionDB is cleared, we need to add values to it 
        //	corresponding to the present state.
-       current = Profiler::CurrentProfiler[tid];
-       while (current != 0) 
-       { // Iterate through each profiler on the callstack and 
+       current = TauInternal_CurrentProfiler(tid);
+       while (current != 0) { 
+	 // Iterate through each profiler on the callstack and 
 	 // fill Values in it 
 	 DEBUGPROFMSG("P Correct: Inside "<< current->ThisFunction->GetName() 
-			<<endl;);
+		      <<endl;);
 	 current->ThisFunction->IncrNumCalls(tid);
-	 if (current->ParentProfiler != 0)
-	 { // Increment the number of called functions in its parent
+	 if (current->ParentProfiler != 0) { 
+	   // Increment the number of called functions in its parent
 	   current->ParentProfiler->ThisFunction->IncrNumSubrs(tid);
 	 }
-#ifndef TAU_MULTIPLE_COUNTERS
-	 current->StartTime = CurrentTimeOrCounts;
-#else //TAU_MULTIPLE_COUNTERS
-	 for(int j=0;j<MAX_TAU_COUNTERS;j++){
-	     current->StartTime[j] = CurrentTimeOrCounts[j];
+	 for(int j=0;j<Tau_Global_numCounters;j++){
+	   current->StartTime[j] = CurrentTimeOrCounts[j];
 	 }
-#endif//TAU_MULTIPLE_COUNTERS
-#if ( defined(PROFILE_CALLS) || defined(PROFILE_STATS) || defined(PROFILE_CALLSTACK) )
-	 current->ExclTimeThisCall = 0;
-#endif   //  PROFILE_CALLS || PROFILE_STATS || PROFILE_CALLSTACK
 	 current = current->ParentProfiler;
        } // Until the top of the stack
 #endif   // PROFILING_ON
-#ifdef TRACING_ON    
-       DEBUGPROFMSG("Tracing Correct: "<<endl;);
-       TraceUnInitialize(tid); // Zap the earlier contents of the trace buffer  
-       TraceCallStack(tid, Profiler::CurrentProfiler[tid]); 
-#endif   // TRACING_ON
-
+       
+       
+       if (TauEnv_get_tracing()) {
+	 DEBUGPROFMSG("Tracing Correct: "<<endl;);
+	 TauTraceUnInitialize(tid); // Zap the earlier contents of the trace buffer  
+	 TraceCallStack(tid, TauInternal_CurrentProfiler(tid)); 
+       }
+       
 #ifdef TAUKTAU
-       DEBUGPROFMSG("RtsLayer::RegisterFork: CurrentProfiler:"<<Profiler::CurrentProfiler[tid]<<endl;);
-       if(Profiler::CurrentProfiler[tid] != NULL) {
-	       Profiler::CurrentProfiler[tid]->ThisKtauProfiler->RegisterFork(Profiler::CurrentProfiler[tid], tid, nodeid, opcode);
+       DEBUGPROFMSG("RtsLayer::RegisterFork: CurrentProfiler:"<<TauInternal_CurrentProfiler(tid)<<endl;);
+       if (TauInternal_CurrentProfiler(tid) != NULL) {
+	 TauInternal_CurrentProfiler(tid)->ThisKtauProfiler->RegisterFork(TauInternal_CurrentProfiler(tid), tid, nodeid, opcode);
        }
 #endif //TAUKTAU
 
@@ -260,15 +245,47 @@ void RtsLayer::RegisterFork(int nodeid, enum TauFork_t opcode)
    // If it is TAU_INCLUDE_PARENT_DATA then there's no need to do anything.
    // fork would copy over all the parent data as it is. 
 }
-	 
-	
-     
+
+bool RtsLayer::initLocks(void) {
+  threadLockDB();
+  for (int i=0; i<TAU_MAX_THREADS; i++) {
+    lockDBcount[i] = 0;
+  }
+  threadUnLockDB();
+  return true;
+}
+
 //////////////////////////////////////////////////////////////////////
 // This ensure that the FunctionDB (global) is locked while updating
 //////////////////////////////////////////////////////////////////////
 
-void RtsLayer::LockDB(void)
-{
+extern "C" void Tau_RtsLayer_LockDB() {
+  RtsLayer::LockDB();
+}
+
+extern "C" void Tau_RtsLayer_UnLockDB() {
+  RtsLayer::UnLockDB();
+}
+
+void RtsLayer::LockDB(void) {
+  static bool init = initLocks();
+  int tid=myThread();
+  if (lockDBcount[tid] == 0) {
+    threadLockDB();
+  }
+  lockDBcount[tid]++;
+  return;
+}
+
+void RtsLayer::UnLockDB(void) {
+  int tid=myThread();
+  lockDBcount[tid]--;
+  if (lockDBcount[tid] == 0) {
+    threadUnLockDB();
+  }
+}
+
+void RtsLayer::threadLockDB(void) {
 #ifdef PTHREADS
   PthreadLayer::LockDB();
 #elif TAU_SPROC
@@ -283,16 +300,16 @@ void RtsLayer::LockDB(void)
   OpenMPLayer::LockDB();
 #elif TAU_PAPI_THREADS
   PapiThreadLayer::LockDB();
-#endif // PTHREADS
+#endif
   return ; // do nothing if threads are not used
 }
 
 
+
 //////////////////////////////////////////////////////////////////////
 // This ensure that the FunctionDB (global) is locked while updating
 //////////////////////////////////////////////////////////////////////
-void RtsLayer::UnLockDB(void)
-{
+void RtsLayer::threadUnLockDB(void) {
 #ifdef PTHREADS
   PthreadLayer::UnLockDB();
 #elif TAU_SPROC
@@ -307,7 +324,7 @@ void RtsLayer::UnLockDB(void)
   OpenMPLayer::UnLockDB();
 #elif TAU_PAPI_THREADS
   PapiThreadLayer::UnLockDB();
-#endif // PTHREADS
+#endif
   return;
 }
 
@@ -361,9 +378,9 @@ void RtsLayer::UnLockEnv(void)
 
 
 /***************************************************************************
- * $RCSfile: RtsThread.cpp,v $   $Author: anataraj $
- * $Revision: 1.25 $   $Date: 2007/04/19 03:21:45 $
- * VERSION: $Id: RtsThread.cpp,v 1.25 2007/04/19 03:21:45 anataraj Exp $
+ * $RCSfile: RtsThread.cpp,v $   $Author: amorris $
+ * $Revision: 1.39 $   $Date: 2009/12/19 04:58:57 $
+ * VERSION: $Id: RtsThread.cpp,v 1.39 2009/12/19 04:58:57 amorris Exp $
  ***************************************************************************/
 
 
