@@ -9,20 +9,7 @@
 /***************************************************************************
 **	File 		: TauFAPI.cpp					  **
 **	Description 	: TAU Profiling Package wrapper for F77/F90	  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -168,6 +155,21 @@ void tau_phase_stop_(void **profiler)
 {
 }
 
+#ifdef linux
+#include <pthread.h>
+#pragma weak pthread_create
+#pragma weak pthread_exit
+int tau_pthread_create (pthread_t * threadp,
+		    const pthread_attr_t *attr,
+		    void *(*start_routine) (void *),
+		    void *arg) {
+  return pthread_create(threadp, attr, start_routine, arg);
+}
+
+void tau_pthread_exit (void *value_ptr) {
+  pthread_exit(value_ptr);
+}
+#endif
 
 /* Cray F90 specific extensions */
 #ifdef CRAYKAI
@@ -266,12 +268,12 @@ void * tau_get_profiler(char *fname, char *type, TauGroup_t group)
 }
 
 ///////////////////////////////////////////////////////////////////////////
-void tau_start_timer(void *profiler)
+void tau_start_timer(void *profiler, int tid)
 {
 }
 
 ///////////////////////////////////////////////////////////////////////////
-void tau_stop_timer(void *profiler)
+void tau_stop_timer(void *profiler, int tid)
 {
 }
 
@@ -580,11 +582,11 @@ void TAU_SET_INTERRUPT_INTERVAL(int value)
 {
 } 
 
-void Tau_start_timer(void * timer, int phase) 
+void Tau_start_timer(void * timer, int phase, int tid) 
 {
 }
 
-void Tau_stop_timer(void *) 
+void Tau_stop_timer(void *, int tid) 
 {
 }
  
@@ -687,13 +689,19 @@ void tau_dealloc__(void ** ptr, int* line, char *name, int slen)
 {
 }
 
-void Tau_init_ref(int* argc, char ***argv) {}
+void Tau_init_ref(int* argc, char ***argv) 
+{
+}
+
+void Tau_exit(char *)
+{
+}
 
 ////////////////////////////////////////////////////////////////////////////
 } /* extern "C" */
 
 /***************************************************************************
- * $RCSfile: TauDisable.cpp,v $   $Author: amorris $
- * $Revision: 1.10 $   $Date: 2007/10/31 23:33:19 $
- * POOMA_VERSION_ID: $Id: TauDisable.cpp,v 1.10 2007/10/31 23:33:19 amorris Exp $ 
+ * $RCSfile: TauDisable.cpp,v $   $Author: sameer $
+ * $Revision: 1.16 $   $Date: 2009/03/26 19:15:39 $
+ * POOMA_VERSION_ID: $Id: TauDisable.cpp,v 1.16 2009/03/26 19:15:39 sameer Exp $ 
  ***************************************************************************/
