@@ -9,20 +9,7 @@
 /***************************************************************************
 **	File 		: TauFAPI.cpp					  **
 **	Description 	: TAU Profiling Package wrapper for F77/F90	  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -37,8 +24,8 @@
 
 
 
-extern "C" void Tau_pure_start(char *name);
-extern "C" void Tau_pure_stop(char *name);
+extern "C" void Tau_pure_start(const char *name);
+extern "C" void Tau_pure_stop(const char *name);
 
 /* Utility function to retrieve fortran strings */
 static char *getFortranName(char *name, int slen) {
@@ -56,8 +43,8 @@ static char *getFortranName(char *name, int slen) {
 
 extern "C" {
 void * Tau_get_profiler(char *, char *, TauGroup_t, char *gr_name);
-void Tau_start_timer(void * timer, int phase);
-void Tau_stop_timer(void *);
+void Tau_start_timer(void * timer, int phase, int tid);
+void Tau_stop_timer(void *, int tid);
 void Tau_exit(char *);
 void Tau_init(int, char **);
 void Tau_enable_group(TauGroup_t group);
@@ -70,12 +57,13 @@ void Tau_disable_instrumentation(void);
 void Tau_trace_sendmsg(int type, int destination, int length);
 void Tau_trace_recvmsg(int type, int source, int length);
 void * Tau_get_userevent(char *name);
-void * Tau_get_context_userevent(char *name);
+void Tau_get_context_userevent(void **ptr, char *name);
 void Tau_userevent(void *ue, double data);
 void Tau_context_userevent(void *ue, double data);
 void Tau_report_statistics(void);
 void Tau_report_thread_statistics(void);
 void Tau_dump(void);
+void Tau_purge(void);
 void Tau_dump_prefix(char *prefix);
 void tau_extract_groupinfo(char *& fname, TauGroup_t & gr, char *& gr_name);
 TauGroup_t Tau_get_profile_group(char * group) ; 
@@ -85,26 +73,25 @@ TauGroup_t Tau_enable_group_name(char *group_name);
 TauGroup_t Tau_disable_group_name(char *group_name);
 void Tau_track_memory(void);
 void Tau_track_memory_here(void);
-void Tau_track_muse_events(void);
 void Tau_enable_tracking_memory(void);
 void Tau_disable_tracking_memory(void);
-void Tau_enable_tracking_muse_events(void);
-void Tau_disable_tracking_muse_events(void);
 void Tau_set_interrupt_interval(int value);
 void Tau_track_memory_headroom(void);
 void Tau_track_memory_headroom_here(void); 
 void Tau_enable_tracking_memory_headroom(void);
 void Tau_disable_tracking_memory_headroom(void);
-void Tau_mark_group_as_phase(void **ptr);
+void Tau_mark_group_as_phase(void *ptr);
 void Tau_profile_callstack(void );
 void Tau_profile_snapshot(char *name);
 void Tau_profile_snapshot_1l(char *name, int number);
 void Tau_metadata(char *name, char *value);
 void Tau_static_phase_start(char *name);
 void Tau_static_phase_stop(char *name);
-void Tau_dynamic_start(char *name, int iteration, int isPhase);
-void Tau_dynamic_stop(char *name, int iteration, int isPhase);
+void Tau_dynamic_start(char *name, int isPhase);
+void Tau_dynamic_stop(char *name, int isPhase);
 char * Tau_append_iteration_to_name(int iteration, char *name);
+int Tau_get_tid(void);
+void Tau_profile_param1l(long data, const char *dataname);
   
 
 
@@ -125,7 +112,7 @@ void tau_profile_timer_group_(void **ptr, char *infname, int *group, int slen)
     printf("Inside tau_profile_timer_group_ fname=%s\n", fname);
 #endif /* DEBUG_PROF */
     
-    *ptr = Tau_get_profiler(fname, (char *)" ", *group, fname);
+    *ptr = Tau_get_profiler(fname, "", *group, fname);
     free(fname);
   }
   
@@ -161,25 +148,20 @@ void tau_extract_groupinfo(char *& fname, TauGroup_t & gr, char *& gr_name)
   /* See if a > appears in the function name. If it does, it separates the
      group name from the function name. Separate these two */
   char *first, *second;
+
   first = strtok(fname, ">"); 
-  if (first != 0)
-  {
+  if (first != 0) {
     second = strtok(NULL,  ">");
-    if (second == NULL) 
-    {
+    if (second == NULL) {
       fname = first; 
       gr = TAU_USER;
-      gr_name = fname; 
-    }
-    else
-    {
+      gr_name = "TAU_DEFAULT"; 
+    } else {
       gr = Tau_get_profile_group(first); 
       gr_name = first;
       fname = second;
     }
   }
-  
-
 }
 
 
@@ -285,7 +267,9 @@ void tau_static_phase_stop(char *fname, int flen) {
     }
   }
 
+#ifdef DEBUG_PROF 
   printf("tau_static_phase_stop: %s\n", localname);
+#endif
   Tau_static_phase_stop(localname);
   free(localname);
 }
@@ -302,7 +286,7 @@ void TAU_STATIC_PHASE_STOP(char *fname, int flen) {
   tau_static_phase_stop(fname, flen);
 }
 
-void tau_dynamic_phase_start(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_start(void *iteration, char *fname, int flen) {
   // make a copy so that we can null terminate it
   char *localname = (char *) malloc((size_t)flen+1);
   strncpy(localname, fname, flen);
@@ -316,23 +300,23 @@ void tau_dynamic_phase_start(int *iteration, char *fname, int flen) {
     }
   }
 
-  Tau_dynamic_start(localname, *iteration, 1); /* 1 is isPhase */
+  Tau_dynamic_start(localname, 1); /* 1 is isPhase */
   free(localname);
 }
 
-void tau_dynamic_phase_start_(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_start_(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_start(iteration, fname, flen);
 }
 
-void tau_dynamic_phase_start__(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_start__(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_start(iteration, fname, flen);
 }
 
-void TAU_DYNAMIC_PHASE_START(int *iteration, char *fname, int flen) {
+void TAU_DYNAMIC_PHASE_START(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_start(iteration, fname, flen);
 }
 
-void tau_dynamic_phase_stop(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_stop(void *iteration, char *fname, int flen) {
   // make a copy so that we can null terminate it
   char *localname = (char *) malloc((size_t)flen+1);
   strncpy(localname, fname, flen);
@@ -346,24 +330,24 @@ void tau_dynamic_phase_stop(int *iteration, char *fname, int flen) {
     }
   }
 
-  Tau_dynamic_stop(localname, *iteration, 1); /* 1 is isPhase */
+  Tau_dynamic_stop(localname, 1); /* 1 is isPhase */
   free(localname);
 }
 
-void tau_dynamic_phase_stop_(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_stop_(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_stop(iteration, fname, flen);
 }
   
-void tau_dynamic_phase_stop__(int *iteration, char *fname, int flen) {
+void tau_dynamic_phase_stop__(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_stop(iteration, fname, flen);
 }
 
-void TAU_DYNAMIC_PHASE_STOP(int *iteration, char *fname, int flen) {
+void TAU_DYNAMIC_PHASE_STOP(void *iteration, char *fname, int flen) {
   tau_dynamic_phase_stop(iteration, fname, flen);
 }
 
 /* TAU_DYNAMIC_TIMER_START/STOP are similar to TAU_DYNAMIC_PHASE_START/STOP */
-void tau_dynamic_timer_start(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_start(void *iteration, char *fname, int flen) {
   // make a copy so that we can null terminate it
   char *localname = (char *) malloc((size_t)flen+1);
   strncpy(localname, fname, flen);
@@ -377,19 +361,19 @@ void tau_dynamic_timer_start(int *iteration, char *fname, int flen) {
     }
   }
 
-  Tau_dynamic_start(localname, *iteration, 0); /* isPhase=0 implies a timer */
+  Tau_dynamic_start(localname, 0); /* isPhase=0 implies a timer */
   free(localname);
 }
 
-void tau_dynamic_timer_start_(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_start_(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_start(iteration, fname, flen);
 }
 
-void tau_dynamic_timer_start__(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_start__(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_start(iteration, fname, flen);
 }
 
-void TAU_DYNAMIC_TIMER_START(int *iteration, char *fname, int flen) {
+void TAU_DYNAMIC_TIMER_START(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_start(iteration, fname, flen);
 }
 
@@ -417,7 +401,7 @@ void TAU_STATIC_TIMER_START(char *fname, int flen)
 
 
 /* TAU_DYNAMIC_TIMER_STOP */
-void tau_dynamic_timer_stop(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_stop(void *iteration, char *fname, int flen) {
   // make a copy so that we can null terminate it
   char *localname = (char *) malloc((size_t)flen+1);
   strncpy(localname, fname, flen);
@@ -431,19 +415,19 @@ void tau_dynamic_timer_stop(int *iteration, char *fname, int flen) {
     }
   }
 
-  Tau_dynamic_stop(localname, *iteration, 0); /* isPhase = 0 implies timer */
+  Tau_dynamic_stop(localname, 0); /* isPhase = 0 implies timer */
   free(localname);
 }
 
-void tau_dynamic_timer_stop_(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_stop_(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_stop(iteration, fname, flen);
 }
 
-void tau_dynamic_timer_stop__(int *iteration, char *fname, int flen) {
+void tau_dynamic_timer_stop__(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_stop(iteration, fname, flen);
 }
 
-void TAU_DYNAMIC_TIMER_STOP(int *iteration, char *fname, int flen) {
+void TAU_DYNAMIC_TIMER_STOP(void *iteration, char *fname, int flen) {
   tau_dynamic_timer_stop(iteration, fname, flen);
 }
 
@@ -470,9 +454,9 @@ void TAU_STATIC_TIMER_STOP(char *fname, int flen)
 }
 
 /* C API */
-void Tau_start(char *name)
+void Tau_start(const char *name)
 {
-  tau_pure_start(name, strlen(name));
+  Tau_pure_start(name);
 }
 
 
@@ -517,9 +501,9 @@ void tau_stop__(char *fname, int flen)
 }
 
 /* C API */
-void Tau_stop(char *name)
+void Tau_stop(const char *name)
 {
-  tau_pure_stop(name, strlen(name));
+  Tau_pure_stop(name);
 }
 
 void tau_profile_timer_(void **ptr, char *fname, int flen)
@@ -533,7 +517,7 @@ void tau_profile_timer_(void **ptr, char *fname, int flen)
   {
 
 #ifdef TAU_OPENMP
-#pragma omp critical
+#pragma omp critical (tau_profile_timer)
     {
       if (*ptr == 0) {
 #endif /* TAU_OPENMP */
@@ -576,7 +560,7 @@ void tau_profile_timer_(void **ptr, char *fname, int flen)
 
     EXTRACT_GROUP(localname, flen, gr, gr_name);
 
-    *ptr = Tau_get_profiler(localname, (char *)" ", gr, gr_name);
+    *ptr = Tau_get_profiler(localname, "", gr, gr_name);
     free(tmp); 
     free(tmp2);
 #ifdef TAU_OPENMP
@@ -605,14 +589,14 @@ void tau_phase_create_static_(void **ptr, char *infname, int slen)
   /* we know the FunctionInfo pointer in ptr. If its here the first time 
      set the group name to be | TAU_PHASE */
   if (firsttime)
-    Tau_mark_group_as_phase(ptr);
+    Tau_mark_group_as_phase(*ptr);
 }
 
 void tau_phase_create_dynamic_(void **ptr, char *infname, int slen)
 {
   *ptr = 0;  /* reset it each time so it creates a new timer */
   tau_profile_timer_(ptr, infname, slen);
-  Tau_mark_group_as_phase(ptr);
+  Tau_mark_group_as_phase(*ptr);
 }
 
 void tau_profile_timer_dynamic_(void **ptr, char *infname, int slen)
@@ -627,25 +611,25 @@ void tau_profile_start_(void **profiler)
   printf("start_timer gets %lx\n", *profiler);
 #endif /* DEBUG_PROF */
 
-  Tau_start_timer(*profiler, 0);
+  Tau_start_timer(*profiler, 0, Tau_get_tid());
   return;
 }
 
 void tau_profile_stop_(void **profiler)
 {
-  Tau_stop_timer(*profiler);
+  Tau_stop_timer(*profiler, Tau_get_tid());
   return;
 }
 
 void tau_phase_start_(void **profiler)
 {
-  Tau_start_timer(*profiler, 1); /* 1 indicates phase based profiling */
+  Tau_start_timer(*profiler, 1, Tau_get_tid()); /* 1 indicates phase based profiling */
   return;
 }
 
 void tau_phase_stop_(void **profiler)
 {
-  Tau_stop_timer(*profiler);
+  Tau_stop_timer(*profiler, Tau_get_tid());
   return;
 }
 
@@ -722,6 +706,12 @@ void tau_db_dump_(void)
   return;
 }
 
+void tau_db_purge_(void)
+{
+  Tau_purge();
+  return;
+}
+
 void tau_db_dump_prefix_(char *prefix)
 {
   Tau_dump_prefix(prefix);
@@ -781,8 +771,9 @@ void tau_disable_group_name(char * group_name, int len)
 }
 
 //////////////////////////////////////////////////////
-// MEMORY, MUSE events API
+// MEMORY API
 //////////////////////////////////////////////////////
+
 void tau_track_memory(void)
 {
   Tau_track_memory();
@@ -803,10 +794,6 @@ void tau_track_memory_headroom_here(void)
   Tau_track_memory_headroom_here();
 } 
 
-void tau_track_muse_events(void)
-{
-  Tau_track_muse_events();
-} 
 
 void tau_enable_tracking_memory(void)
 {
@@ -828,15 +815,6 @@ void tau_disable_tracking_memory_headroom(void)
   Tau_disable_tracking_memory_headroom();
 } 
 
-void tau_enable_tracking_muse_events(void)
-{
-  Tau_enable_tracking_muse_events();
-} 
-
-void tau_disable_tracking_muse_events(void)
-{
-  Tau_disable_tracking_muse_events();
-} 
 
 void tau_set_interrupt_interval(int* value)
 {
@@ -1026,7 +1004,7 @@ void tau_register_context_event_(void **ptr, char *name, int slen)
 #ifdef DEBUG_PROF
     printf("Tau_get_context_userevent(%s) \n", localname);
 #endif /* DEBUG_PROF */
-    *ptr = Tau_get_context_userevent(localname);
+    Tau_get_context_userevent(ptr, localname);
     free(tmp);
     free(tmp2);
   }
@@ -1055,7 +1033,7 @@ void tau_report_thread_statistics_(void)
 }
 
 //////////////////////////////////////////////////////
-// MEMORY, MUSE events API
+// MEMORY API
 //////////////////////////////////////////////////////
 void tau_track_memory_(void)
 {
@@ -1077,10 +1055,6 @@ void tau_track_memory_headroom_here_(void)
   Tau_track_memory_headroom_here();
 } 
 
-void tau_track_muse_events_(void)
-{
-  Tau_track_muse_events();
-} 
 
 void tau_enable_tracking_memory_(void)
 {
@@ -1102,15 +1076,6 @@ void tau_disable_tracking_memory_headroom_(void)
   Tau_disable_tracking_memory_headroom();
 } 
 
-void tau_enable_tracking_muse_events_(void)
-{
-  Tau_enable_tracking_muse_events();
-} 
-
-void tau_disable_tracking_muse_events_(void)
-{
-  Tau_disable_tracking_muse_events();
-} 
 
 void tau_set_interrupt_interval_(int* value)
 {
@@ -1129,27 +1094,55 @@ void TAU_PROFILE_TIMER(void **ptr, char *fname, int flen)
   return;
 }
 
+void TAU_PROFILE_TIMER_(void **ptr, char *fname, int flen)
+{
+  if (*ptr == 0) {
+    tau_profile_timer_(ptr, fname, flen);
+  }
+  return;
+}
+
 void TAU_PROFILE_START(void **profiler)
 {
   tau_profile_start_(profiler);
 }
 
+void TAU_PROFILE_START_(void **profiler)
+{
+  tau_profile_start_(profiler);
+}
+
 void TAU_PROFILE_STOP(void **profiler)
 {
   tau_profile_stop_(profiler);
 }
 
+void TAU_PROFILE_STOP_(void **profiler)
+{
+  tau_profile_stop_(profiler);
+}
+
 void TAU_PROFILE_EXIT(char *msg)
 {
   tau_profile_exit_(msg);
 }
 
+void TAU_PROFILE_EXIT_(char *msg)
+{
+  tau_profile_exit_(msg);
+}
+
 void TAU_DB_DUMP(void)
 {
   Tau_dump();
   return;
 }
 
+void TAU_DB_PURGE(void)
+{
+  Tau_purge();
+  return;
+}
 void TAU_DB_DUMP_PREFIX(char *prefix)
 {
   Tau_dump_prefix(prefix);
@@ -1169,6 +1162,15 @@ void TAU_PROFILE_INIT()
 #endif /* TAU_MPI */
 }
 
+void TAU_PROFILE_INIT_()
+{
+#ifndef TAU_MPI
+#ifndef TAU_SHMEM
+  Tau_set_node(0); 
+#endif /* TAU_SHMEM */
+#endif /* TAU_MPI */
+}
+
 void TAU_PROFILE_SET_NODE(int *node)
 {
   tau_profile_set_node_(node);
@@ -1293,7 +1295,7 @@ void tau_profile_timer(void **ptr, char *fname, int flen)
 }
 
 //////////////////////////////////////////////////////
-// MEMORY, MUSE events API
+// MEMORY API
 //////////////////////////////////////////////////////
 void TAU_TRACK_MEMORY(void)
 {
@@ -1315,11 +1317,6 @@ void TAU_TRACK_MEMORY_HEADROOM_HERE(void)
   Tau_track_memory_headroom_here();
 } 
 
-void TAU_TRACK_MUSE_EVENTS(void)
-{
-  Tau_track_muse_events();
-} 
-
 void TAU_ENABLE_TRACKING_MEMORY(void)
 {
   Tau_enable_tracking_memory();
@@ -1340,15 +1337,6 @@ void TAU_DISABLE_TRACKING_MEMORY_HEADROOM(void)
   Tau_disable_tracking_memory_headroom();
 } 
 
-void TAU_ENABLE_TRACKING_MUSE_EVENTS(void)
-{
-  Tau_enable_tracking_muse_events();
-} 
-
-void TAU_DISABLE_TRACKING_MUSE_EVENTS(void)
-{
-  Tau_disable_tracking_muse_events();
-} 
 
 void TAU_SET_INTERRUPT_INTERVAL(int* value)
 {
@@ -1357,12 +1345,12 @@ void TAU_SET_INTERRUPT_INTERVAL(int* value)
 
 void tau_profile_start(int **profiler)
 {
-  Tau_start_timer((void *)*profiler, 0);
+  Tau_start_timer((void *)*profiler, 0, Tau_get_tid());
 }
 
 void tau_profile_stop(int **profiler)
 {
-  Tau_stop_timer((void *)*profiler);
+  Tau_stop_timer((void *)*profiler, Tau_get_tid());
 }
 
 void tau_profile_init(void)
@@ -1394,6 +1382,11 @@ void tau_db_dump(void)
   Tau_dump();
 }
 
+void tau_db_purge(void)
+{
+  Tau_purge();
+}
+
 void tau_db_dump_prefix(char *prefix)
 {
   Tau_dump_prefix(prefix);
@@ -1493,6 +1486,12 @@ void tau_db_dump__(void)
   return;
 }
 
+void tau_db_purge__(void)
+{
+  Tau_purge();
+  return;
+}
+
 void tau_db_dump_prefix__(char *prefix)
 {
   Tau_dump_prefix(prefix);
@@ -1630,7 +1629,7 @@ void tau_disable_group_name__(char * group_name, int len)
 }
 
 //////////////////////////////////////////////////////
-// MEMORY, MUSE events API
+// MEMORY API
 //////////////////////////////////////////////////////
 void tau_track_memory__(void)
 {
@@ -1652,10 +1651,6 @@ void tau_track_memory_headroom_here__(void)
   Tau_track_memory_headroom_here();
 } 
 
-void tau_track_muse_events__(void)
-{
-  Tau_track_muse_events();
-} 
 
 void tau_enable_tracking_memory__(void)
 {
@@ -1677,15 +1672,6 @@ void tau_disable_tracking_memory_headroom__(void)
   Tau_disable_tracking_memory_headroom();
 } 
 
-void tau_enable_tracking_muse_events__(void)
-{
-  Tau_enable_tracking_muse_events();
-} 
-
-void tau_disable_tracking_muse_events__(void)
-{
-  Tau_disable_tracking_muse_events();
-} 
 
 void tau_set_interrupt_interval__(int* value)
 {
@@ -1832,6 +1818,29 @@ void TAU_PROFILE_SNAPSHOT(char *name, int slen) {
   tau_profile_snapshot_(name, slen);
 }
 
+//////////////////////////////////////////////////////////////////////
+// Parameter Profiling
+//////////////////////////////////////////////////////////////////////
+void tau_profile_param_1l_(char *name, int *number, int slen) {
+  char *fname = getFortranName(name, slen);
+  Tau_profile_param1l(*number, fname);
+  free (fname);
+}
+
+void tau_profile_param_1l(char *name, int *number, int slen) {
+  tau_profile_param_1l_(name, number, slen);
+}
+
+void tau_profile_param_1l__(char *name, int *number, int slen) {
+  tau_profile_param_1l_(name, number, slen);
+}
+
+void TAU_PROFILE_PARAM_1L(char *name, int *number, int slen) {
+  tau_profile_param_1l_(name, number, slen);
+}
+
+void Tau_profile_param1l(long data, const char *dataname);
+
 
 //////////////////////////////////////////////////////////////////////
 // Metadata routines
@@ -1960,26 +1969,39 @@ void tau_dealloc_(void ** ptr, int* line, char *name, int slen)
   free(tmp2);
 }
 
-void tau_dealloc(void ** ptr, int* line, char *name, int slen) 
-{
+void tau_dealloc(void ** ptr, int* line, char *name, int slen) {
   tau_dealloc_(ptr, line, name, slen);
 }
 
-void tau_dealloc__(void ** ptr, int* line, char *name, int slen) 
-{
+void tau_dealloc__(void ** ptr, int* line, char *name, int slen) {
   tau_dealloc_(ptr, line, name, slen);
 }
 
-void TAU_DEALLOC(void ** ptr, int* line, char *name, int slen) 
-{
+void TAU_DEALLOC(void ** ptr, int* line, char *name, int slen) {
   tau_dealloc_(ptr, line, name, slen);
 }
 
 } /* extern "C" */
 
 
+/* Empty MUSE events for compatibility */
+void tau_track_muse_events(void) {}
+void tau_enable_tracking_muse_events(void) {}
+void tau_disable_tracking_muse_events(void) {}
+void tau_track_muse_events__(void) {}
+void tau_enable_tracking_muse_events__(void) {}
+void tau_disable_tracking_muse_events__(void) {}
+void tau_track_muse_events_(void) {}
+void tau_enable_tracking_muse_events_(void) {}
+void tau_disable_tracking_muse_events_(void) {}
+void TAU_TRACK_MUSE_EVENTS(void) {}
+void TAU_ENABLE_TRACKING_MUSE_EVENTS(void) {}
+void TAU_DISABLE_TRACKING_MUSE_EVENTS(void) {}
+
+
+
 /***************************************************************************
  * $RCSfile: TauFAPI.cpp,v $   $Author: amorris $
- * $Revision: 1.66 $   $Date: 2007/09/19 20:56:08 $
- * POOMA_VERSION_ID: $Id: TauFAPI.cpp,v 1.66 2007/09/19 20:56:08 amorris Exp $ 
+ * $Revision: 1.81 $   $Date: 2010/02/23 07:42:19 $
+ * POOMA_VERSION_ID: $Id: TauFAPI.cpp,v 1.81 2010/02/23 07:42:19 amorris Exp $ 
  ***************************************************************************/
