@@ -9,20 +9,7 @@
 /***************************************************************************
 **	File 		: TauHooks.cpp					  **
 **	Description 	: TAU hooks for DynInst (Dynamic Instrumentation) **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -40,18 +27,23 @@
 using namespace std;
 #endif
 
+#include <stdlib.h>
+
 //#define DEBUG_PROF
 //int debugPrint = 0;
 // control debug printf statements
 //#define dprintf if (debugPrint) printf
+//#define dprintf printf
 #ifdef DEBUG_PROF
 #define dprintf printf
 #else // DEBUG_PROF 
-#define dprintf if (0) printf
+#define dprintf TAU_VERBOSE
 #endif
 
 //#define ORIGINAL_HEAVY_IMPLEMENTATION_USING_MAP 1
 #define TAUDYNVEC 1
+//extern "C" void Tau_get_func_name(long addr, char * fname, char *filename);
+
 
 #ifndef __ia64
 int TheFlag[TAU_MAX_THREADS] ;
@@ -70,6 +62,12 @@ vector<FunctionInfo*>& TheTauDynFI(void)
 
   return FuncTauDynFI;
 }
+vector<void*>& TheTauBinDynFI(void)
+{ // FunctionDB contains pointers to each FunctionInfo static object
+  static vector<void*> FuncTauDynFI;
+
+  return FuncTauDynFI;
+}
 /* global FI vector */
 // Initialization procedure. Should be called before invoking 
 // other TAU routines.
@@ -78,20 +76,37 @@ void TauInitCode(char *arg, int isMPI)
 {
   // Register that we are using dyninst so that the FIvector destructor will
   // perform cleanup for us
+  // dprintf("TauInitCode: arg=%s, isMPI=%d\n", arg, isMPI);
   TheUsingDyninst() = 1;
   char *name;
   int tid = 0;
   TAU_MONITOR_ENTER(0);
   int functionId = 0;
+  char funcname[1024];
+  char *saveptr;
 
+  int j;
+  char *str1;
   /* iterate for each routine name */
-  name = strtok(arg, "|");
-  while (name != (char *)NULL)
+/*
+  for (j=1, str1 = arg; ; j++, str1 = NULL) {
+    name = strtok_r(str1, "|", &saveptr);
+    if (name == NULL) 
+      break;
+    printf("Extracted token = %s\n", name);
+  }
+*/
+  for (j=1, str1 = arg; ; j++, str1 = NULL) 
   { 
+    name = strtok_r(str1, "|", &saveptr);
+    if (name == NULL) 
+      break;
+    dprintf("After loop: name = %s\n", name);
+
     functionId ++; 
 #ifdef ORIGINAL_HEAVY_IMPLEMENTATION_USING_MAP
     dprintf("Extracted : %s :id = %d\n", name, functionId);
-    TAU_MAPPING_CREATE(name, " ", functionId, "TAU_DEFAULT", tid);
+    TAU_MAPPING_CREATE(funcname, " ", functionId, "TAU_DEFAULT", tid);
 #else
     dprintf("Extracted : %s :id = %d\n", name, functionId-1);
     /* Create a new FunctionInfo object with the given name and add it to 
@@ -113,7 +128,6 @@ void TauInitCode(char *arg, int isMPI)
 #endif /* TAUDYNVEC */
 #endif /* ORIGINAL_HEAVY_IMPLEMENTATION_USING_MAP */
     
-    name = strtok(NULL, "|");
   }
   dprintf("Inside TauInitCode Initializations to be done here!\n");
   if (!isMPI)
@@ -271,11 +285,171 @@ int TauRenameTimer(char *oldName, char *newName)
   dprintf("Didn't find the routine!\n");
   return 0; /* didn't find it! */
 }
-} // extern "C"
+
+
+static int tauFiniID; 
+static int tauDyninstEnabled[TAU_MAX_THREADS];
+void trace_register_func(char *func, int id)
+{
+  static int invocations = 0;
+  int tid = RtsLayer::myThread();
+  if (!tauDyninstEnabled[tid]) return;
+  dprintf("trace_register_func: func = %s, id = %d\n", func, id);
+
+  void *taufi;
+  TAU_PROFILER_CREATE(taufi, func, " ", TAU_DEFAULT);
+
+ 
+  if (strncmp(func, "_fini", 5) == 0) {
+    dprintf("FOUND FINI id = %d\n", id);
+    tauFiniID = id;
+  } 
+  if (func[0] == 't' && func[1] == 'a' && func[2] == 'r' && func[3] == 'g') {
+    if (isdigit(func[4])) {
+      long addr;
+      dprintf("trace_register_func: Routine name is targN...\n");
+      ((FunctionInfo *)taufi)->SetProfileGroup(TAU_GROUP_31);
+
+    // TAU_GROUP_31 is special. It indicates that the routine is called targ...
+    // This routine should be exited prior to the beginning of the next routine
+    // Extract the name from the address:
+/*
+      sscanf(func, "targ%lx", &addr);
+      dprintf("ADDR=%lx, name =%s\n", addr, func);
+      char name[256];
+      char filename[256];
+      Tau_get_func_name(addr, (char *)name, (char *)filename); 
+      printf("GOT: name = %s, filename = %s, addr = %lx\n", name, filename, addr);
+*/
+    }
+  }
+  dprintf("TAU FI = %lx\n", taufi);
+  dprintf("id = %d, invocations = %d\n", id , invocations);
+  if (id == invocations)
+    TheTauBinDynFI().push_back(taufi);
+  else {
+    printf("WARNING: trace_register_func: id does not match invocations\n");
+    TheTauBinDynFI()[id] = taufi;
+  } 
+  invocations ++;
+}
+
+void traceEntry(int id)
+{
+  int tid = RtsLayer::myThread();
+  if (!tauDyninstEnabled[tid]) return;
+  void *fi = TheTauBinDynFI()[id];
+
+  // Additional sanity checks
+  if (fi == NULL) { 
+    dprintf("ERROR?: ENTRY: id = null!\n");
+    return;
+  }
+  TAU_QUERY_DECLARE_EVENT(curr);
+  TAU_QUERY_GET_CURRENT_EVENT(curr);
+
+  if ( curr && ((Profiler *)curr)->ThisFunction && 
+     ((Profiler *)curr)->ThisFunction->GetProfileGroup() == TAU_GROUP_31) {
+    dprintf("TARG on the stack \n");
+    TAU_PROFILER_STOP(((Profiler *)curr)->ThisFunction);
+  }
+
+  dprintf("Inside traceEntry: id = %d fi = %lx\n", id, fi);
+  dprintf("Name = %s\n", ((FunctionInfo *)fi)->GetName());
+  if (id == tauFiniID) { 
+    TAU_DISABLE_INSTRUMENTATION();
+    dprintf("Disabling instrumentation found id = %d\n", id);
+  } 
+  else {
+    if (fi != NULL) {
+      TAU_PROFILER_START(fi);
+    } else {
+      dprintf("ERROR?: traceEntry: fi = null!\n");
+    }
+  }
+  
+
+}
+
+void traceExit(int id)
+{
+  const char *strcurr;
+  const char *strbin;
+  int tid = RtsLayer::myThread();
+  if (!tauDyninstEnabled[tid]) return;
+  void *fi = TheTauBinDynFI()[id];
+  dprintf("traceExit: Name = %s, %lx\n", ((FunctionInfo *)fi)->GetName(), fi);
+
+  TAU_QUERY_DECLARE_EVENT(curr);
+  TAU_QUERY_GET_CURRENT_EVENT(curr);
+
+  // Additional sanity checks: Stop profiling after main exits
+  bool disableinstr = false;
+  if ( curr && ((Profiler *)curr)->ParentProfiler == (Profiler *) NULL)
+  {
+    if (strncmp(((FunctionInfo *)fi)->GetName(), "main",4)== 0) {
+      disableinstr = true;
+      dprintf("Disabling instrumentation!\n");
+    }
+  }
+  if (fi != NULL) {
+    TAU_PROFILER_STOP(fi);
+  } else {
+    printf("ERROR: traceExit: fi = null!\n");
+  }
+  if(disableinstr) {
+    tauDyninstEnabled[tid] = false;
+  }
+
+}
+
+void my_otf_init(int isMPI)
+{
+  dprintf("Inside my otf_init\n");
+  dprintf("isMPI = %d\n", isMPI);
+  if (!isMPI)
+  {
+    dprintf("Calling SET NODE 0\n");
+    TAU_PROFILE_SET_NODE(0);
+  }
+  int tid = RtsLayer::myThread();
+  if (!tauDyninstEnabled[tid]) {
+    tauDyninstEnabled[tid] = 1;
+  }
+}
+
+void my_otf_cleanup()
+{
+  dprintf("Inside my otf_cleanup\n");
+}
+
+void tau_dyninst_init(int isMPI)
+{
+  dprintf("Inside tau_dyninst_init \n");
+  dprintf("isMPI = %d\n", isMPI);
+  if (!isMPI)
+  {
+    dprintf("Calling SET NODE 0\n");
+    TAU_PROFILE_SET_NODE(0);
+  }
+  int tid = RtsLayer::myThread();
+  if (!tauDyninstEnabled[tid]) {
+    tauDyninstEnabled[tid] = 1;
+  }
+}
+
+void tau_dyninst_cleanup()
+{
+  dprintf("Inside tau_dyninst_cleanup\n");
+}
+
+
+}
+// extern "C"
 
 // EOF TauHooks.cpp
 /***************************************************************************
- * $RCSfile: TauHooks.cpp,v $   $Author: sameer $
- * $Revision: 1.24 $   $Date: 2007/03/24 01:32:54 $
- * TAU_VERSION_ID: $Id: TauHooks.cpp,v 1.24 2007/03/24 01:32:54 sameer Exp $ 
+ * $RCSfile: TauHooks.cpp,v $   $Author: amorris $
+ * $Revision: 1.33 $   $Date: 2009/11/03 17:34:17 $
+ * TAU_VERSION_ID: $Id: TauHooks.cpp,v 1.33 2009/11/03 17:34:17 amorris Exp $ 
  ***************************************************************************/
