@@ -9,20 +9,7 @@
 /***************************************************************************
 **      File            : TauJava.cpp                                     **
 **      Description     : TAU interface for JVMPI                         **
-**      Author          : Sameer Shende                                   **
-**      Contact         : sameer@cs.uoregon.edu sameer@acl.lanl.gov       **
-**      Flags           : Compile with                                    **
-**                        -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**                        -DPROFILE_STATS for Std. Deviation of Excl Time **
-**                        -DSGI_HW_COUNTERS for using SGI counters        **
-**                        -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**                        -DTULIP_TIMERS for non-sgi Platform             **
-**                        -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**                        -DPOOMA_TFLOP for Intel Teraflop at SNL/NM      **
-**                        -DPOOMA_KAI for KCC compiler                    **
-**                        -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
 **      Documentation   : See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -36,6 +23,7 @@
 #include <Profile/TauJava.h>
 #if (!defined(TAU_WINDOWS))
 #include <unistd.h>
+#include <stdlib.h>
 #endif //TAU_WINDOWS
 
 
@@ -50,8 +38,7 @@ bool& TheTauExcludeMethodsFlag()
 
 // profiler agent entry point
 extern "C" {
-  JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *jvm, char *options, void *reserved)
-{
+  JNIEXPORT jint JNICALL JVM_OnLoad(JavaVM *jvm, char *options, void *reserved) {
 #ifdef DEBUG_PROF
     fprintf(stdout, "TAU> initializing ..... \n");
 #endif /* DEBUG_PROF */
@@ -72,8 +59,7 @@ extern "C" {
     TauJavaLayer::Init(options);
 
     // enabling class load event notification
-    if (!TheTauExcludeMethodsFlag())
-    {
+    if (!TheTauExcludeMethodsFlag()) {
       CALL(EnableEvent)(JVMPI_EVENT_CLASS_LOAD, NULL);
       CALL(EnableEvent)(JVMPI_EVENT_METHOD_ENTRY, NULL);
       CALL(EnableEvent)(JVMPI_EVENT_METHOD_EXIT, NULL);
@@ -109,33 +95,29 @@ extern "C" {
 
 static char **TauExcludeList=NULL;
 static int TauExcludeListSize = 0;
-void TauJavaLayer::Init(char *options){
-char *token;
-static int num_tokens = 0; /* This can be called more than once! TauJAPI */
-int token_len;
-char *s1;
-char *s2;
-
+void TauJavaLayer::Init(char *options) {
+  char *token;
+  static int num_tokens = 0; /* This can be called more than once! TauJAPI */
+  int token_len;
+  char *s1;
+  char *s2;
+  
 #ifdef DEBUG_PROF
   printf("Inside TauJavaLayer::Init options = %s\n",options);
 #endif // DEBUG_PROF
-
-  if(options && strlen(options))
-  {
+  
+  if(options && strlen(options)) {
 #ifdef DEBUG_PROF
     fprintf(stdout,"Init: options = %s, len=%d\n", options, strlen(options));
 #endif // DEBUG_PROF
 
     // There are problems with strtok. Since this is called twice,
     // we allocate new strings before using with strtok. Crashes otherwise 	
-    if (num_tokens == 0)
-    {
+    if (num_tokens == 0) {
       s1 = new char [strlen (options) + 1];
       strcpy(s1, options);
       token=strtok(s1, "=,");
-    }
-    else
-    {
+    } else {
       s2 = new char [strlen (options) + 1];
       strcpy(s2, options);
       token=strtok(s2, "=,");
@@ -150,11 +132,7 @@ char *s2;
         TauExcludeList = (char **) malloc(256*sizeof(char*)); // 256 items max in exclude list
 	// This ensures that if it is coming here again then the list is not blank
       }
-      while(token=strtok(NULL,"=,")) 
-      {
-#ifdef DEBUG_PROF
-        printf("token=%s\n",token);
-#endif // DEBUG_PROF
+      while(token=strtok(NULL,"=,")) {
 	token_len = strlen(token);
 	TauExcludeList[num_tokens]=(char *)malloc(token_len+1);
 	strcpy(TauExcludeList[num_tokens], token);
@@ -162,14 +140,12 @@ char *s2;
       }
       TauExcludeListSize = num_tokens;
     }
-
+    
 #ifdef DEBUG_PROF
-    for(int i = 0; i < TauExcludeListSize; i++)
-    {
+    for(int i = 0; i < TauExcludeListSize; i++) {
       printf("ExcludeList[%d] = %s\n", i, TauExcludeList[i]);
     }
 #endif // DEBUG_PROF
-    
   }
 }
 
@@ -177,7 +153,11 @@ char *s2;
 void TauJavaLayer::NotifyEvent(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
 #ifndef TAU_MPI
-  static int j = TAU_MAPPING_PROFILE_SET_NODE(0, tid);
+  static int j = 0;
+  if (j == 0) {
+    j=1;
+    TAU_PROFILE_SET_NODE(0);
+  }
 #else  /* TAU_MPI */
   //static int j = TAU_MAPPING_PROFILE_SET_NODE(getpid(), tid);
 #endif /* TAU_MPI */
@@ -227,8 +207,7 @@ void TauJavaLayer::NotifyEvent(JVMPI_Event *event) {
   }
 }
 
-void TauJavaLayer::ClassLoad(JVMPI_Event *event)
-{
+void TauJavaLayer::ClassLoad(JVMPI_Event *event) {
   char funcname[2048], classname[1024];
   char *groupname;
   int i;
@@ -240,13 +219,10 @@ void TauJavaLayer::ClassLoad(JVMPI_Event *event)
 
 int origkey = 1;
   
-  if (TauExcludeListSize) 
-  {
-    for (i=0; i < TauExcludeListSize; i++)
-    {
+  if (TauExcludeListSize) {
+    for (i=0; i < TauExcludeListSize; i++) {
       if (strncmp(event->u.class_load.class_name, 
-	  TauExcludeList[i], strlen(TauExcludeList[i])) == 0) 
-      {
+		  TauExcludeList[i], strlen(TauExcludeList[i])) == 0) {
 #ifdef DEBUG_PROF
         printf("Excluding %s\n", event->u.class_load.class_name);
 #endif // DEBUG_PROF
@@ -254,19 +230,18 @@ int origkey = 1;
       }
     }
   }
-  for (i = 0; i < event->u.class_load.num_methods; i++)
-  {
+  for (i = 0; i < event->u.class_load.num_methods; i++) {
     /* Create FunctionInfo objects for each of these methods */
 
-#ifdef TRACING_ON
-    sprintf(funcname, "%s  %s", event->u.class_load.class_name, 
-	event->u.class_load.methods[i].method_name); 
-#else
-    sprintf(funcname, "%s  %s %s", event->u.class_load.class_name, 
-	event->u.class_load.methods[i].method_name, 
-	event->u.class_load.methods[i].method_signature); 
-#endif /* signature is too much for tau_convert to handle */
-
+    if (TauEnv_get_tracing()) {
+      sprintf(funcname, "%s  %s", event->u.class_load.class_name, 
+	      event->u.class_load.methods[i].method_name); 
+    } else {
+      sprintf(funcname, "%s  %s %s", event->u.class_load.class_name, 
+	      event->u.class_load.methods[i].method_name, 
+	      event->u.class_load.methods[i].method_signature); 
+    }
+    
     sprintf(classname, "%s", event->u.class_load.class_name); 
     groupname = strtok(classname, " /=");
 
@@ -275,38 +250,35 @@ int origkey = 1;
 	  (long)  event->u.class_load.methods[i].method_id, 
 		     groupname, tid); 
 */
-    if (origkey == 1)
-    {
+    if (origkey == 1) {
       TAU_MAPPING_CREATE(funcname, " ",
-	  (long)  event->u.class_load.methods[i].method_id, 
-		     groupname, tid); 
+			 (long)  event->u.class_load.methods[i].method_id, 
+			 groupname, tid); 
     } else {
-/* NO NEED TO CREATE THIS OBJECT! */ 
-/*
-      TAU_MAPPING_CREATE1(funcname, " ",
-	  (long)  event->u.class_load.methods[i].method_id, origkey,
-		     groupname, tid); 
-*/
+      /* NO NEED TO CREATE THIS OBJECT! */ 
+      /*
+	TAU_MAPPING_CREATE1(funcname, " ",
+	(long)  event->u.class_load.methods[i].method_id, origkey,
+	groupname, tid); 
+      */
     }
-   
-
+    
+    
 #ifdef DEBUG_PROF 
     printf("TAU> %s, id: %ld group:  %s\n", funcname,
-		event->u.class_load.methods[i].method_id, groupname);
+	   event->u.class_load.methods[i].method_id, groupname);
 #endif /* DEBUG_PROF */
     /* name, type, key, group name  are the four arguments above */
   }
-	
 }
 
-void TauJavaLayer::MethodEntry(JVMPI_Event *event)
-{
+void TauJavaLayer::MethodEntry(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
   TAU_MAPPING_OBJECT(TauMethodName=NULL);
   TAU_MAPPING_LINK(TauMethodName, (long) event->u.method.method_id);
   
   // If the method is not masked, (has non-zero group id), execute it
-  if(TauMethodName && TauMethodName->GetProfileGroup()) { 
+  if (TauMethodName) { 
     TAU_MAPPING_PROFILE_TIMER(TauTimer, TauMethodName, tid);
     TAU_MAPPING_PROFILE_START(TauTimer, tid);
 
@@ -318,32 +290,30 @@ void TauJavaLayer::MethodEntry(JVMPI_Event *event)
   }
 }
 
-void TauJavaLayer::MethodExit(JVMPI_Event *event)
-{
+void TauJavaLayer::MethodExit(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
 
   TAU_MAPPING_OBJECT(TauMethodName=NULL);
   TAU_MAPPING_LINK(TauMethodName, (long) event->u.method.method_id);
   
-  if(TauMethodName && TauMethodName->GetProfileGroup()) { // stop only if it has a finite group 
+  if (TauMethodName) { // stop only if it has a finite group 
     TAU_MAPPING_PROFILE_STOP(tid);
 #ifdef DEBUG_PROF
-  fprintf(stdout, "TAU> Method Exit : %ld, TID = %d\n",
-	 	(long) event->u.method.method_id, tid);
+    fprintf(stdout, "TAU> Method Exit : %ld, TID = %d\n",
+	    (long) event->u.method.method_id, tid);
 #endif /* DEBUG_PROF */
   }
 }
 
 void TauJavaLayer::CreateTopLevelRoutine(char *name, char *type, char *groupname, 
-			int tid)
-{
+			int tid) {
 #ifdef DEBUG_PROF
   fprintf(stdout, "Inside CreateTopLevelRoutine: name = %s, type = %s, group = %s, tid = %d\n",
-	name, type, groupname, tid); 
+	  name, type, groupname, tid); 
 #endif
   /* Create a top-level routine that is always called. Use the thread name in it */
   TAU_MAPPING_CREATE(name, type, 1, groupname, tid); 
-
+  
   TAU_MAPPING_OBJECT(TauMethodName);
   TAU_MAPPING_LINK(TauMethodName, (long) 1);
   
@@ -351,8 +321,7 @@ void TauJavaLayer::CreateTopLevelRoutine(char *name, char *type, char *groupname
   TAU_MAPPING_PROFILE_START(TauTimer, tid);
 }
 
-void TauJavaLayer::ThreadStart(JVMPI_Event *event)
-{
+void TauJavaLayer::ThreadStart(JVMPI_Event *event) {
   int * ptid = JavaThreadLayer::RegisterThread(event->env_id);
   int tid = *ptid;
 
@@ -367,8 +336,7 @@ void TauJavaLayer::ThreadStart(JVMPI_Event *event)
   CreateTopLevelRoutine(thread_name, " ", "THREAD", tid); 
 }
 
-void TauJavaLayer::ThreadEnd(JVMPI_Event *event)
-{
+void TauJavaLayer::ThreadEnd(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
 #ifdef DEBUG_PROF
   fprintf(stdout, "TAU> Thread End : id = %d \n", tid);
@@ -377,8 +345,7 @@ void TauJavaLayer::ThreadEnd(JVMPI_Event *event)
     TAU_MAPPING_PROFILE_EXIT("END...", tid);
 }
 
-void TauJavaLayer::ShutDown(JVMPI_Event *event)
-{
+void TauJavaLayer::ShutDown(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
   JVMPI_RawMonitor shutdown_lock = CALL(RawMonitorCreate)("Shutdown lock");
 #ifdef DEBUG_PROF
@@ -386,36 +353,29 @@ void TauJavaLayer::ShutDown(JVMPI_Event *event)
 #endif 
 
   CALL(RawMonitorEnter)(shutdown_lock);
-  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++)
-  {
+  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++) {
     TAU_MAPPING_PROFILE_EXIT("Forcing Shutdown of Performance Data",i);
   }
   CALL(RawMonitorExit)(shutdown_lock);
-
-  
 }
 
 
-void TauJavaLayer::DataDump(JVMPI_Event *event)
-{
+void TauJavaLayer::DataDump(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
   JVMPI_RawMonitor dump_lock = CALL(RawMonitorCreate)("Dump lock");
-
+  
 #ifdef DEBUG_PROF
   fprintf(stdout, "TAU> JVM DUMP : id = %d \n", tid);
 #endif 
 
   CALL(RawMonitorEnter)(dump_lock);
-  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++)
-  {
+  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++) {
     TAU_MAPPING_DB_DUMP(i);
   }
   CALL(RawMonitorExit)(dump_lock);
-  
 }
 
-void TauJavaLayer::DataPurge(JVMPI_Event *event)
-{
+void TauJavaLayer::DataPurge(JVMPI_Event *event) {
   int tid = JavaThreadLayer::GetThreadId(event->env_id);
   JVMPI_RawMonitor purge_lock = CALL(RawMonitorCreate)("Purge lock");
 
@@ -424,22 +384,20 @@ void TauJavaLayer::DataPurge(JVMPI_Event *event)
 #endif 
 
   CALL(RawMonitorEnter)(purge_lock);
-  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++)
-  {
-/* Disabled to retain profile DB integrity on Ctrl-\ (DUMP/PURGE)
-    TAU_MAPPING_DB_PURGE(i);
-*/
+  for(int i = 0; i < JavaThreadLayer::TotalThreads(); i++) {
+    /* Disabled to retain profile DB integrity on Ctrl-\ (DUMP/PURGE)
+       TAU_MAPPING_DB_PURGE(i);
+    */
   }
   CALL(RawMonitorExit)(purge_lock);
-  
 }
 
 /* EOF : TauJava.cpp */
 
 
 /***************************************************************************
- * $RCSfile: TauJava.cpp,v $   $Author: amorris $
- * $Revision: 1.28 $   $Date: 2005/11/11 03:46:50 $
- * TAU_VERSION_ID: $Id: TauJava.cpp,v 1.28 2005/11/11 03:46:50 amorris Exp $
+ * $RCSfile: TauJava.cpp,v $   $Author: khuck $
+ * $Revision: 1.33 $   $Date: 2009/03/13 00:46:56 $
+ * TAU_VERSION_ID: $Id: TauJava.cpp,v 1.33 2009/03/13 00:46:56 khuck Exp $
  ***************************************************************************/
 
