@@ -49,6 +49,31 @@ TAU_GLOBAL_TIMER(tworkshare, "workshare enter/exit", "[OpenMP]", OpenMP);
 TAU_GLOBAL_TIMER(tregion, "inst region begin/end", "[OpenMP]", OpenMP); 
 
 
+#define NUM_OMP_TYPES 15
+
+static char* omp_names[15] = {"atomic enter/exit", "barrier enter/exit", "critical begin/end", 
+			     "critical enter/exit", "for enter/exit", "master begin/end",
+			     "parallel begin/end", "parallel fork/join", "section begin/end",
+			     "sections enter/exit", "single begin/end", "single enter/exit",
+			      "workshare enter/exit", "inst region begin/end", "flush enter/exit" };
+
+
+#define TAU_OMP_ATOMIC      0
+#define TAU_OMP_BARRIER     1
+#define TAU_OMP_CRITICAL_BE 2
+#define TAU_OMP_CRITICAL_EE 3
+#define TAU_OMP_FOR_EE      4
+#define TAU_OMP_MASTER_BE   5
+#define TAU_OMP_PAR_BE      6
+#define TAU_OMP_PAR_FJ      7
+#define TAU_OMP_SECTION_BE  8
+#define TAU_OMP_SECTION_EE  9
+#define TAU_OMP_SINGLE_BE  10
+#define TAU_OMP_SINGLE_EE  11
+#define TAU_OMP_WORK_EE    12
+#define TAU_OMP_INST_BE    13
+#define TAU_OMP_FLUSH_EE   14
+
 static int omp_tracing    = 1;
 static int omp_fin_called = 0;
 
@@ -682,34 +707,72 @@ int tau_openmp_init(void)
 }
 
 
-void TauStartOpenMPRegionTimer(struct ompregdescr *r)
+void TauStartOpenMPRegionTimer(struct ompregdescr *r, int index)
 {
   static int tau_openmp_initialized = tau_openmp_init();
 /* For any region, create a mapping between a region r and timer t and
    start the timer. */
 
   omp_set_lock(&tau_ompregdescr_lock);
-  if (r->data)
-  {
-    Profiler *p = new Profiler((FunctionInfo *) r->data, OpenMP, true, RtsLayer::myThread());
-    p->Start();
-  }
-  else
-  {
-    char rname[256], rtype[1024];
+
+  if (!r->data) {
+#ifdef TAU_OPENMP_PARTITION_REGION
+    FunctionInfo **flist = new FunctionInfo*[NUM_OMP_TYPES];
+    for (int i=0; i < NUM_OMP_TYPES; i++) {
+      char rname[1024], rtype[1024];
+      sprintf(rname, "%s %s (%s)", r->name, r->sub_name, omp_names[i]);
+      sprintf(rtype, "[OpenMP location: file:%s <%d, %d>]",
+	      r->file_name, r->begin_first_line, r->end_last_line);
+      
+      FunctionInfo *f = new FunctionInfo(rname, rtype, OpenMP, "OpenMP");
+      flist[i] = f;
+    }
+    r->data = (void*)flist;
+#else
+    char rname[1024], rtype[1024];
     sprintf(rname, "%s %s", r->name, r->sub_name);
     sprintf(rtype, "[OpenMP location: file:%s <%d, %d>]",
-        r->file_name, r->begin_first_line, r->end_last_line);
-
+	    r->file_name, r->begin_first_line, r->end_last_line);
+    
     FunctionInfo *f = new FunctionInfo(rname, rtype, OpenMP, "OpenMP");
-    r->data = (void *) f;
-    Profiler *p = new Profiler (f, OpenMP, true, RtsLayer::myThread());
-    p->Start();
+    r->data = (void*)f;
+#endif
   }
+  
+#ifdef TAU_OPENMP_PARTITION_REGION
+  FunctionInfo *f = ((FunctionInfo **)r->data)[index];
+#else 
+  FunctionInfo *f = (FunctionInfo *)r->data;
+#endif
+  Tau_start_timer(f, 0, Tau_get_tid());
+  
   omp_unset_lock(&tau_ompregdescr_lock);
-
 }
 
+
+void TauStopOpenMPRegionTimer(struct ompregdescr *r, int index)
+{
+
+#ifdef TAU_OPENMP_PARTITION_REGION
+    FunctionInfo *f = ((FunctionInfo **)r->data)[index];
+#else
+    FunctionInfo *f = (FunctionInfo *)r->data;
+#endif
+    TauGroup_t gr = f->GetProfileGroup();
+
+    int tid = RtsLayer::myThread(); 
+    Profiler *p =TauInternal_CurrentProfiler(tid); 
+    if (p->ThisFunction == f) {
+      Tau_stop_timer(f, Tau_get_tid());
+    } else {
+      // nothing, it must have been disabled/throttled
+    }
+}
+
+
+
+
+
 /* pomp library calls */
 
 void POMP_Finalize() {
@@ -754,7 +817,7 @@ void POMP_Atomic_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r,TAU_OMP_ATOMIC); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -766,13 +829,13 @@ void POMP_Atomic_enter(struct ompregdescr* r) {
 
 void POMP_Atomic_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_ATOMIC);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
-  
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -788,7 +851,7 @@ void POMP_Barrier_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_BARRIER); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -804,13 +867,14 @@ void POMP_Barrier_enter(struct ompregdescr* r) {
 
 void POMP_Barrier_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_BARRIER);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -830,7 +894,7 @@ void POMP_Critical_begin(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_CRITICAL_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -843,13 +907,14 @@ void POMP_Critical_begin(struct ompregdescr* r) {
 
 void POMP_Critical_end(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_CRITICAL_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -866,7 +931,7 @@ void POMP_Critical_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_CRITICAL_EE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -879,13 +944,14 @@ void POMP_Critical_enter(struct ompregdescr* r) {
 
 void POMP_Critical_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_CRITICAL_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -903,7 +969,7 @@ void POMP_For_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_FOR_EE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -915,14 +981,15 @@ void POMP_For_enter(struct ompregdescr* r) {
 
 void POMP_For_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_FOR_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   // as in a stack. lifo
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -938,7 +1005,7 @@ void POMP_Master_begin(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_MASTER_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -950,13 +1017,14 @@ void POMP_Master_begin(struct ompregdescr* r) {
 
 void POMP_Master_end(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_MASTER_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -966,13 +1034,15 @@ void POMP_Master_end(struct ompregdescr* r) {
 }
 
 void POMP_Parallel_begin(struct ompregdescr* r) {
+/* if there is no top level timer, create it */
+  Tau_create_top_level_timer_if_necessary();
 
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_START(tparallelb);
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_PAR_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -984,13 +1054,14 @@ void POMP_Parallel_begin(struct ompregdescr* r) {
 
 void POMP_Parallel_end(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_PAR_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1006,9 +1077,10 @@ void POMP_Parallel_fork(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_PAR_FJ); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
+
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
     fprintf(stderr, "%3d: fork  parallel\n", omp_get_thread_num());
@@ -1018,13 +1090,14 @@ void POMP_Parallel_fork(struct ompregdescr* r) {
 
 void POMP_Parallel_join(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_PAR_FJ);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1040,7 +1113,7 @@ void POMP_Section_begin(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_SECTION_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1052,13 +1125,14 @@ void POMP_Section_begin(struct ompregdescr* r) {
 
 void POMP_Section_end(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_SECTION_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1074,7 +1148,7 @@ void POMP_Sections_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_SECTION_EE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1086,13 +1160,14 @@ void POMP_Sections_enter(struct ompregdescr* r) {
 
 void POMP_Sections_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_SECTION_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1108,7 +1183,7 @@ void POMP_Single_begin(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_SINGLE_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1120,13 +1195,14 @@ void POMP_Single_begin(struct ompregdescr* r) {
 
 void POMP_Single_end(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_SINGLE_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
   
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1142,7 +1218,7 @@ void POMP_Single_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_SINGLE_EE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1154,13 +1230,13 @@ void POMP_Single_enter(struct ompregdescr* r) {
 
 void POMP_Single_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_SINGLE_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
-  
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1176,7 +1252,7 @@ void POMP_Workshare_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_WORK_EE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1188,14 +1264,15 @@ void POMP_Workshare_enter(struct ompregdescr* r) {
 
 void POMP_Workshare_exit(struct ompregdescr* r) {
 
+  
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_WORK_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
-  
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
     fprintf(stderr, "%3d: exit  workshare\n", omp_get_thread_num());
@@ -1210,7 +1287,7 @@ void POMP_Begin(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r); 
+  TauStartOpenMPRegionTimer(r, TAU_OMP_INST_BE); 
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1223,13 +1300,14 @@ void POMP_Begin(struct ompregdescr* r) {
 
 void POMP_End(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_INST_BE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
   
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1247,7 +1325,7 @@ void POMP_Flush_enter(struct ompregdescr* r) {
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
 
 #ifdef TAU_OPENMP_REGION_VIEW
-  TauStartOpenMPRegionTimer(r);
+  TauStartOpenMPRegionTimer(r, TAU_OMP_FLUSH_EE);
 #endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
@@ -1260,13 +1338,14 @@ void POMP_Flush_enter(struct ompregdescr* r) {
 
 void POMP_Flush_exit(struct ompregdescr* r) {
 
+#ifdef TAU_OPENMP_REGION_VIEW
+  TauStopOpenMPRegionTimer(r, TAU_OMP_FLUSH_EE);
+#endif /* TAU_OPENMP_REGION_VIEW */
+
 #ifdef TAU_AGGREGATE_OPENMP_TIMINGS
   TAU_GLOBAL_TIMER_STOP(); /* global timer stop */
 #endif /* TAU_AGGREGATE_OPENMP_TIMINGS */
  
-#ifdef TAU_OPENMP_REGION_VIEW
-  TAU_GLOBAL_TIMER_STOP(); /* region timer stop */
-#endif /* TAU_OPENMP_REGION_VIEW */
 
 #ifdef DEBUG_PROF
   if ( omp_tracing ) {
@@ -1380,8 +1459,8 @@ int  POMP_Test_nest_lock(omp_nest_lock_t *s) {
 
 /***************************************************************************
  * $RCSfile: TauKojakOpari.cpp,v $   $Author: sameer $
- * $Revision: 1.2 $   $Date: 2003/07/19 21:47:19 $
- * POOMA_VERSION_ID: $Id: TauKojakOpari.cpp,v 1.2 2003/07/19 21:47:19 sameer Exp $
+ * $Revision: 1.9 $   $Date: 2009/03/26 19:15:39 $
+ * POOMA_VERSION_ID: $Id: TauKojakOpari.cpp,v 1.9 2009/03/26 19:15:39 sameer Exp $
  ***************************************************************************/
 
 
