@@ -1,6 +1,6 @@
 /****************************************************************************
 **			TAU Portable Profiling Package			   **
-**			http://www.acl.lanl.gov/tau		           **
+**			http://www.cs.uoregon.edu/research/tau	           **
 *****************************************************************************
 **    Copyright 1997  						   	   **
 **    Department of Computer and Information Science, University of Oregon **
@@ -9,21 +9,8 @@
 /***************************************************************************
 **	File 		: TauMapping.cpp				  **
 **	Description 	: TAU Profiling Package				  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**			  -DSGI_TIMERS  for SGI fast nanosecs timer	  **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
-**	Documentation	: See http://www.acl.lanl.gov/tau	          **
+**	Contact		: tau-team@cs.uoregon.edu 		 	  **
+**	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
 //////////////////////////////////////////////////////////////////////
@@ -36,10 +23,8 @@
 using namespace std;
 #endif
 
-struct lTauGroup
-{
-  bool operator()(const TauGroup_t s1, const TauGroup_t s2) const
-  {
+struct lTauGroup {
+  bool operator()(const TauGroup_t s1, const TauGroup_t s2) const {
     return s1 < s2;
   }
 };
@@ -51,11 +36,9 @@ struct lTauGroup
 // the FunctionInfo * pointer that contains the id of the function 
 // being mapped. The key is currently in the form of a profile group.
 //////////////////////////////////////////////////////////////////////
-FunctionInfo *& TheTauMapFI(TauGroup_t Pgroup )
-{ 
+void *& TheTauMapFI(TauGroup_t key) { 
   //static FunctionInfo *TauMapFI = (FunctionInfo *) NULL;
-  static map<TauGroup_t, FunctionInfo *, lTauGroup > TauMapGroups;
-
-  return TauMapGroups[Pgroup];
+  static map<TauGroup_t, void*, lTauGroup > TauMapGroups;
+  return TauMapGroups[key];
 }
 // EOF
