@@ -21,7 +21,7 @@
 
 #include <stdio.h>
 #include <Profile/Profiler.h>
-#if (defined(__APPLE_CC__) || defined(TAU_APPLE_XLC))
+#if (defined(__APPLE_CC__) || defined(TAU_APPLE_XLC) || defined(TAU_APPLE_PGI))
 #include <malloc/malloc.h>
 #else
 #ifdef TAU_FREEBSD
@@ -40,11 +40,17 @@ using namespace std;
 #endif /* TAU_DOT_H_LESS_HEADERS */
 
 #if (defined(__QK_USER__) || defined(__LIBCATAMOUNT__ ))
-#ifndef TAU_CATAMOUNT
 #define TAU_CATAMOUNT 
-#endif /* TAU_CATAMOUNT */
-#include <catamount/catmalloc.h>
 #endif /* __QK_USER__ || __LIBCATAMOUNT__ */
+#ifdef TAU_CATAMOUNT
+#include <catamount/catmalloc.h>
+#endif /* TAU_CATAMOUNT */
+
+#include <stdlib.h>
+
+#ifdef TAU_BGP
+#include <kernel_interface.h>
+#endif
 
 
 //////////////////////////////////////////////////////////////////////
@@ -273,6 +279,30 @@ TauVoidPointer Tau_malloc(const char *file, int line, size_t size)
   return ptr;  /* what was allocated */
 }
 
+//////////////////////////////////////////////////////////////////////
+// Tau_calloc calls the before and after routines and allocates memory
+//////////////////////////////////////////////////////////////////////
+TauVoidPointer Tau_calloc(const char *file, int line, size_t nmemb, size_t size)
+{
+  TAU_USER_EVENT_TYPE *e; 
+
+  /* We get the event that is created */
+  e = Tau_malloc_before(file, line, nmemb * size);
+
+  TauVoidPointer ptr = calloc(nmemb, size);
+
+#ifdef DEBUGPROF
+  printf("TAU_CALLOC<%d>: %s:%d ptr = %p size = %d\n", RtsLayer::myNode(), file, line, ptr, size);
+#endif /* DEBUGPROF */
+
+  /* associate the event generated and its size with the address of memory
+   * allocated by calloc. This is used later for memory leak detection and
+   * to evaluate the size of the memory freed in the Tau_free(ptr) routine. */
+
+  Tau_malloc_after(ptr, nmemb * size, e);
+  return ptr;  /* what was allocated */
+}
+
 //////////////////////////////////////////////////////////////////////
 // Tau_track_memory_allocation does everything that Tau_malloc does except
 // allocate memory
@@ -505,6 +535,19 @@ extern "C" void * Tau_realloc_C(const char *file, int line, void *p, size_t size
   return (void *) Tau_realloc(file, line, p, size);
 
 }
+//////////////////////////////////////////////////////////////////////
+// Tau_calloc for C++ has file and line information
+//////////////////////////////////////////////////////////////////////
+extern "C" void * Tau_calloc_C(const char *file, int line, size_t nmemb, size_t size)
+{
+#ifdef DEBUGPROF
+  printf("C: Tau_calloc: %s:%d\n", file, line);
+#endif /* DEBUGPROF */
+  /* realloc acts like a free and malloc */ 
+  return (void *) Tau_calloc(file, line, nmemb, size);
+
+}
+
 //////////////////////////////////////////////////////////////////////
 // The amount of memory available for use (in MB) 
 //////////////////////////////////////////////////////////////////////
@@ -526,6 +569,15 @@ int TauGetFreeMemory(void)
 #else /* TAU_CATAMOUNT */
 int TauGetFreeMemory(void)
 {
+
+#ifdef TAU_BGP
+  uint32_t available_heap;
+  Kernel_GetMemorySize( KERNEL_MEMSIZE_ESTHEAPAVAIL, &available_heap );
+  return available_heap / (1024 * 1024);
+#endif
+
+
+
   char* blocks[TAU_BLOCK_COUNT];
   char* ptr;
   int i,j;
@@ -559,6 +611,6 @@ int TauGetFreeMemory(void)
 
 /***************************************************************************
  * $RCSfile: TauMemory.cpp,v $   $Author: amorris $
- * $Revision: 1.28 $   $Date: 2007/05/31 18:41:03 $
- * TAU_VERSION_ID: $Id: TauMemory.cpp,v 1.28 2007/05/31 18:41:03 amorris Exp $ 
+ * $Revision: 1.33 $   $Date: 2010/01/27 00:47:51 $
+ * TAU_VERSION_ID: $Id: TauMemory.cpp,v 1.33 2010/01/27 00:47:51 amorris Exp $ 
  ***************************************************************************/
