@@ -1,13 +1,20 @@
 #include <Profile/Profiler.h>
+#include <Profile/TauEnv.h>
+
 #include <stdio.h>
 #include <mpi.h>
 #include <stdlib.h>
+
+#include <string.h>
+
 #define TAU_MAX_REQUESTS  4096
 
-#ifdef TAU_SYNCHRONIZE_CLOCKS
-extern void TauSyncClocks(int rank, int size);
-extern void TauSyncFinalClocks(int rank, int size);
-#endif
+void TauSyncClocks();
+void TauSyncFinalClocks();
+int TAUDECL Tau_write_snapshot(const char *name, int finalize);
+int Tau_mergeProfiles();
+void TAUDECL Tau_set_usesMPI(int value);
+int TAUDECL tau_totalnodes(int set_or_get, int value);
 
 
 /* This file uses the MPI Profiling Interface with TAU instrumentation.
@@ -25,12 +32,8 @@ extern void TauSyncFinalClocks(int rank, int size);
 
 /* Requests */
 
-#ifndef TAU_NOCOMM
-#define TAU_TRACK_MSG 1
-#endif /* TAU_NOCOMM */
 
 
-#ifdef TAU_TRACK_MSG
 typedef struct request_list_ {
     MPI_Request request; /* SSS request should be a pointer */
     int         status, size, tag, otherParty;
@@ -99,10 +102,53 @@ typedef struct request_list_ {
 #define rq_end( head_alloc ) {\
   request_list *rq; while (head_alloc) {\
 	rq = head_alloc->next;free(head_alloc);head_alloc=rq;}}
-static request_list *requests_head_0, *requests_tail_0;
-#endif /* TAU_TRACK_MSG */
+static request_list *requests_head_0=NULL, *requests_tail_0=NULL;
+
+
+
+
 static int procid_0;
 
+#define track_vector( call, counts, typesize ) { \
+    int typesize, commSize, commRank, sendcount = 0, i; \
+    PMPI_Comm_rank(comm, &commRank); \
+    PMPI_Comm_size(comm, &commSize); \
+    if ( commRank == root ) { \
+      PMPI_Type_size( sendtype, &typesize ); \
+      for (i = 0; i<commSize; i++) { \
+	sendcount += counts[i]; \
+      } \
+      call(typesize*sendcount); \
+    } \
+  } 
+
+
+static int sum_array (int *counts, MPI_Datatype type, MPI_Comm comm) {
+
+  int typesize, commSize, commRank, i;
+  int total = 0;
+  PMPI_Comm_rank(comm, &commRank);
+  PMPI_Comm_size(comm, &commSize);
+  PMPI_Type_size(type, &typesize );
+  
+  for (i = 0; i<commSize; i++) {
+    total += counts[i];
+  } 
+  return total;
+}
+
+#define track_allvector( call, counts, typesize ) { \
+    int typesize, commSize, commRank, sendcount = 0, i; \
+    PMPI_Comm_rank(comm, &commRank); \
+    PMPI_Comm_size(comm, &commSize); \
+    PMPI_Type_size( sendtype, &typesize ); \
+    for (i = 0; i<commSize; i++) { \
+      sendcount += counts[i]; \
+    } \
+    call(typesize*sendcount); \
+  } 
+
+
 /* This function translates a given rank in a given communicator to the proper
    rank in MPI_COMM_WORLD */
 static int translateRankToWorld(MPI_Comm comm, int rank) {
@@ -185,10 +231,8 @@ static int translateRankToWorld(MPI_Comm comm, int rank) {
 
   
 
-int TAUDECL tau_totalnodes(int set_or_get, int value);
 
 
-#ifdef TAU_TRACK_MSG
 
 void TauProcessRecv ( request, status, note )
 MPI_Request request;
@@ -390,7 +434,6 @@ char *note;
   return ; 
 }
 
-#endif /* TAU_TRACK_MSG */
 
 
 
@@ -449,8 +492,8 @@ MPI_Comm comm;
   
   returnVal = PMPI_Allgatherv( sendbuf, sendcount, sendtype, recvbuf, recvcounts, displs, recvtype, comm );
   PMPI_Type_size( recvtype, &typesize );
-  TAU_ALLGATHER_DATA(typesize*(*recvcounts));
-  
+
+  track_allvector(TAU_ALLGATHER_DATA, recvcounts, typesize);
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -495,6 +538,12 @@ MPI_Comm comm;
   TAU_PROFILE_TIMER(tautimer, "MPI_Alltoall()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
+#ifdef TAU_EXP_TRACK_COMM
+  void *commhandle;
+  commhandle = (void*)comm;
+  TAU_PROFILE_PARAM1L((long)commhandle, "comm");
+#endif /* TAU_EXP_TRACK_COMM */
+
   returnVal = PMPI_Alltoall( sendbuf, sendcount, sendtype, recvbuf, recvcnt, recvtype, comm );
   PMPI_Type_size( sendtype, &typesize );
   TAU_ALLTOALL_DATA(typesize*sendcount);
@@ -516,14 +565,18 @@ MPI_Datatype recvtype;
 MPI_Comm comm;
 {
   int   returnVal;
-  int   typesize;
+  int   sendtypesize, recvtypesize;
+  int tracksize = 0;
 
   TAU_PROFILE_TIMER(tautimer, "MPI_Alltoallv()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
   returnVal = PMPI_Alltoallv( sendbuf, sendcnts, sdispls, sendtype, recvbuf, recvcnts, rdispls, recvtype, comm );
-  PMPI_Type_size( sendtype, &typesize );
-  TAU_ALLTOALL_DATA(typesize*(*sendcnts));
+
+  tracksize = sum_array(sendcnts, sendtype, comm);
+  tracksize += sum_array(recvcnts, recvtype, comm);
+
+  TAU_ALLTOALL_DATA(tracksize);
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -618,11 +671,7 @@ MPI_Comm comm;
 
   returnVal = PMPI_Gatherv( sendbuf, sendcnt, sendtype, recvbuf, recvcnts, displs, recvtype, root, comm );
 
-  PMPI_Comm_rank ( comm, &rank );
-  if (rank == root) {
-    PMPI_Type_size( recvtype, &typesize );
-    TAU_GATHER_DATA(typesize*(*recvcnts));
-  }
+  track_vector(TAU_GATHER_DATA, recvcnts, recvtype);
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -767,15 +816,14 @@ MPI_Datatype recvtype;
 int root;
 MPI_Comm comm;
 {
-  int   returnVal;
-  int   typesize;
+  int returnVal;
 
   TAU_PROFILE_TIMER(tautimer, "MPI_Scatterv()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
   returnVal = PMPI_Scatterv( sendbuf, sendcnts, displs, sendtype, recvbuf, recvcnt, recvtype, root, comm );
-  PMPI_Type_size( sendtype, &typesize );
-  TAU_SCATTER_DATA(typesize*(*sendcnts));
+
+  track_vector(TAU_SCATTER_DATA, sendcnts, typesize);
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -928,8 +976,10 @@ int * rank;
   TAU_PROFILE_STOP(tautimer);
 
   /* Set the node as we did in MPI_Init */
-  if (comm == MPI_COMM_WORLD)
+  if (comm == MPI_COMM_WORLD) {
     TAU_PROFILE_SET_NODE(*rank);
+    Tau_set_usesMPI(1);
+  }
 
   return returnVal;
 }
@@ -982,6 +1032,44 @@ int * size;
   return returnVal;
 }
 
+/**************************************************************************
+ Experimental routine to track communicator splits in metadata 
+ This will create a metadata item such as:
+   Name : MPI_Comm 102140608
+   Value : 0 2 4 6
+***************************************************************************/
+#ifdef TAU_EXP_TRACK_COMM
+void tau_exp_track_comm_split (MPI_Comm oldcomm, MPI_Comm newcomm) {
+  int worldrank;
+  int newCommSize;
+  void *oldcommhandle, *newcommhandle;
+  int i;
+  char buffer[16384];
+  char catbuffer[2048];
+  char namebuffer[512];
+
+  oldcommhandle = (void*)oldcomm;
+  newcommhandle = (void*)newcomm;
+
+/*   printf ("comm %p split into %p for %d\n", oldcommhandle, newcommhandle, procid_0); */
+  MPI_Comm_size(newcomm, &newCommSize);
+/*   printf ("comm %p split into %p for %d, new size = %d\n", oldcommhandle, newcommhandle, procid_0, newCommSize); */
+
+  /* initialize to empty */
+  buffer[0] = 0;
+  for (i=0; i<newCommSize; i++) {
+    worldrank = translateRankToWorld(newcomm, i);
+/*     printf ("comm %p has world member %d\n", newcommhandle, worldrank); */
+    sprintf (catbuffer, "%d ", worldrank);
+    strcat(buffer, catbuffer);
+  }
+
+/*   printf ("buffer is %s\n", buffer); */
+  sprintf (namebuffer, "MPI_Comm %lu", newcommhandle);
+  TAU_METADATA(namebuffer, buffer);
+}
+#endif /* TAU_EXP_TRACK_COMM */
+
 int   MPI_Comm_split( comm, color, key, comm_out )
 MPI_Comm comm;
 int color;
@@ -995,6 +1083,10 @@ MPI_Comm * comm_out;
   
   returnVal = PMPI_Comm_split( comm, color, key, comm_out );
 
+#ifdef TAU_EXP_TRACK_COMM
+  tau_exp_track_comm_split(comm, *comm_out);
+#endif /* TAU_EXP_TRACK_COMM */
+
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -1420,15 +1512,26 @@ int  MPI_Finalize(  )
   TAU_PROFILE_TIMER(tautimer, "MPI_Finalize()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_SYNCHRONIZE_CLOCKS
-  PMPI_Comm_size( MPI_COMM_WORLD, &size );
-  TauSyncFinalClocks(procid_0, size);
-#endif
+
+  if (TauEnv_get_synchronize_clocks()) {
+    TauSyncFinalClocks();
+  }
 
   PMPI_Get_processor_name(procname, &procnamelength);
   TAU_METADATA("MPI Processor Name", procname);
 
-  returnVal = PMPI_Finalize(  );
+  /* Grab the node id, we don't always wrap mpi_init */
+  PMPI_Comm_rank( MPI_COMM_WORLD, &procid_0 );
+  TAU_PROFILE_SET_NODE(procid_0 ); 
+  Tau_set_usesMPI(1);
+
+  /* Create a merged profile if requested */
+  if (TauEnv_get_profile_format() == TAU_FORMAT_MERGED) {
+    Tau_write_snapshot("merge", 1);
+    Tau_mergeProfiles();
+  }
+
+  returnVal = PMPI_Finalize();
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -1471,6 +1574,7 @@ char *** argv;
 
   PMPI_Comm_rank( MPI_COMM_WORLD, &procid_0 );
   TAU_PROFILE_SET_NODE(procid_0 ); 
+  Tau_set_usesMPI(1);
 
   PMPI_Comm_size( MPI_COMM_WORLD, &size );
   tau_totalnodes(1, size); /* Set the totalnodes */
@@ -1478,13 +1582,11 @@ char *** argv;
   PMPI_Get_processor_name(procname, &procnamelength);
   TAU_METADATA("MPI Processor Name", procname);
 
-#ifdef TAU_SYNCHRONIZE_CLOCKS
-  TauSyncClocks(procid_0, size);
-#endif
+  if (TauEnv_get_synchronize_clocks()) {
+    TauSyncClocks();
+  }
 
-#ifdef TAU_TRACK_MSG
   requests_head_0 = requests_tail_0 = 0;
-#endif /* TAU_TRACK_MSG */
 
   return returnVal;
 }
@@ -1513,6 +1615,7 @@ int *provided;
 
   PMPI_Comm_rank( MPI_COMM_WORLD, &procid_0 );
   TAU_PROFILE_SET_NODE(procid_0 );
+  Tau_set_usesMPI(1);
 
   PMPI_Comm_size( MPI_COMM_WORLD, &size );
   tau_totalnodes(1, size); /* Set the totalnodes */
@@ -1520,13 +1623,11 @@ int *provided;
   PMPI_Get_processor_name(procname, &procnamelength);
   TAU_METADATA("MPI Processor Name", procname);
 
-#ifdef TAU_SYNCHRONIZE_CLOCKS
-  TauSyncClocks(procid_0, size);
-#endif
+  if (TauEnv_get_synchronize_clocks()) {
+    TauSyncClocks(procid_0, size);
+  }
 
-#ifdef TAU_TRACK_MSG
   requests_head_0 = requests_tail_0 = 0;
-#endif /* TAU_TRACK_MSG */
 
   return returnVal;
 }
@@ -1612,16 +1713,13 @@ MPI_Comm comm;
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Bsend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
-    /*
-    prof_send( procid_0, dest, tag, typesize*count,
-	       "MPI_Bsend" );
-    */
+
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Bsend( buf, count, datatype, dest, tag, comm );
 
@@ -1648,9 +1746,9 @@ MPI_Request * request;
 
   returnVal = PMPI_Bsend_init( buf, count, datatype, dest, tag, comm, request );
 
-#ifdef TAU_TRACK_MSG
-  TauAddRequest(RQ_SEND, count, datatype, dest, tag, comm, request, returnVal, 1);
-#endif /* TAU_TRACK_MSG */
+  if (TauEnv_get_track_message()) {
+    TauAddRequest(RQ_SEND, count, datatype, dest, tag, comm, request, returnVal, 1);
+  }
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -1697,15 +1795,12 @@ MPI_Request * request;
   TAU_PROFILE_TIMER(tautimer, "MPI_Cancel()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
-  TauRemoveRequest(*request, "MPI_Cancel");
-#endif /* TAU_TRACK_MSG */
+  if (TauEnv_get_track_message()) {
+    TauRemoveRequest(*request, "MPI_Cancel");
+  }
 
   returnVal = PMPI_Cancel( request );
-
   TAU_PROFILE_STOP(tautimer);
-
-
   return returnVal;
 }
 
@@ -1714,23 +1809,18 @@ MPI_Request * request;
 {
   int  returnVal;
 
-
   /* The request may have completed, may have not.  */
   /* We'll assume it didn't. */
   
-  
   TAU_PROFILE_TIMER(tautimer, "MPI_Request_free()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  TauRemoveRequest(*request, "MPI_Request_free");
-#endif /* TAU_TRACK_MSG */
+  if (TauEnv_get_track_message()) {
+    TauRemoveRequest(*request, "MPI_Request_free");
+  }
   
   returnVal = PMPI_Request_free( request );
-
   TAU_PROFILE_STOP(tautimer);
-
-
   return returnVal;
 }
 
@@ -1752,10 +1842,9 @@ MPI_Request * request;
 
   TAU_PROFILE_STOP(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  TauAddRequest(RQ_RECV, count, datatype, source, tag, comm, request, returnVal, 1);
-#endif /* TAU_TRACK_MSG */
-
+  if (TauEnv_get_track_message()) {
+    TauAddRequest(RQ_RECV, count, datatype, source, tag, comm, request, returnVal, 1);
+  }
   return returnVal;
 }
 
@@ -1781,9 +1870,9 @@ MPI_Request * request;
 
   /* we need to store the request and associate it with the size/tag so MPI_Start can 
      retrieve it and log the TAU_TRACE_SENDMSG */
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   TauAddRequest(RQ_SEND, count, datatype, dest, tag, comm, request, returnVal, 1);
-#endif /* TAU_TRACK_MSG */
+}
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -1834,29 +1923,20 @@ MPI_Comm comm;
 MPI_Request * request;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize;
-#endif /* TAU_TRACK_MSG */
-
-  
-  
-/* fprintf( stderr, "MPI_Ibsend call on %d\n", procid_0 ); */
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Ibsend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize);
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize);
+    }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   returnVal = PMPI_Ibsend( buf, count, datatype, dest, tag, comm, request );
-
-
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -1889,7 +1969,6 @@ MPI_Comm comm;
 MPI_Request * request;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   request_list *newrq1;
   int typesize;
 
@@ -1897,8 +1976,6 @@ MPI_Request * request;
   int myrank;
 #endif /* DEBUG */
 
-#endif /* TAU_TRACK_MSG */
-
   
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Irecv()",  " ", TAU_MESSAGE);
@@ -1916,23 +1993,23 @@ MPI_Request * request;
 
   TAU_PROFILE_STOP(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  if (source != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
-    if (newrq1 = (request_list*) malloc(sizeof( request_list ))) {
-      PMPI_Type_size( datatype, &typesize );
-      newrq1->request = *request;
-      newrq1->status = RQ_RECV;
-      newrq1->size = typesize * count;
-      newrq1->otherParty = source;
-      newrq1->comm = comm;
-      newrq1->tag = tag;
-      newrq1->is_persistent = 0;
-      newrq1->next = 0;
-      rq_add( requests_head_0, requests_tail_0, newrq1 );
+  if (TauEnv_get_track_message()) {
+    if (source != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
+      if (newrq1 = (request_list*) malloc(sizeof( request_list ))) {
+	PMPI_Type_size( datatype, &typesize );
+	newrq1->request = *request;
+	newrq1->status = RQ_RECV;
+	newrq1->size = typesize * count;
+	newrq1->otherParty = source;
+	newrq1->comm = comm;
+	newrq1->tag = tag;
+	newrq1->is_persistent = 0;
+	newrq1->next = 0;
+	rq_add( requests_head_0, requests_tail_0, newrq1 );
+      }
     }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   return returnVal;
 }
 
@@ -1946,23 +2023,17 @@ MPI_Comm comm;
 MPI_Request * request;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize3;
-#endif /* TAU_TRACK_MSG */
-
-  
-  
-/* fprintf( stderr, "MPI_Irsend call on %d\n", procid_0 ); */
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Irsend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   if (dest != MPI_PROC_NULL) {
     PMPI_Type_size( datatype, &typesize3 );
     TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize3);
   }
-#endif /* TAU_TRACK_MSG */
+}
 
   returnVal = PMPI_Irsend( buf, count, datatype, dest, tag, comm, request );
 
@@ -1982,29 +2053,20 @@ MPI_Comm comm;
 MPI_Request * request;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize3;
-#endif /* TAU_TRACK_MSG */
 
-  
-  
-/* fprintf( stderr, "MPI_Isend call on %d\n", procid_0 ); */
-  
   TAU_PROFILE_TIMER(tautimer, "MPI_Isend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize3 );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize3);
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize3 );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize3);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Isend( buf, count, datatype, dest, tag, comm, request );
-
-
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -2018,29 +2080,20 @@ MPI_Comm comm;
 MPI_Request * request;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize3;
-#endif /* TAU_TRACK_MSG */
 
-  
-  
-/* fprintf( stderr, "MPI_Issend call on %d\n", procid_0 ); */
-  
   TAU_PROFILE_TIMER(tautimer, "MPI_Issend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize3 );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize3);
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize3 );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), count * typesize3);
+    }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   returnVal = PMPI_Issend( buf, count, datatype, dest, tag, comm, request );
-
-
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -2117,15 +2170,15 @@ MPI_Status * status;
   TAU_PROFILE_TIMER(tautimer, "MPI_Recv()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   if (status == MPI_STATUS_IGNORE) {
     status = &local_status;
   }
-#endif /* TAU_TRACK_MSG */
+}
   
   returnVal = PMPI_Recv( buf, count, datatype, source, tag, comm, status );
 
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   if (source != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
     PMPI_Get_count( status, MPI_BYTE, &size );
 
@@ -2136,7 +2189,7 @@ MPI_Status * status;
 	       status->MPI_TAG, size, "MPI_Recv" );
     */
   }
-#endif /* TAU_TRACK_MSG */
+}
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -2151,28 +2204,19 @@ int tag;
 MPI_Comm comm;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize;
-#endif /* TAU_TRACK_MSG */
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Rsend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
-    /*
-    prof_send( procid_0, dest, tag, typesize*count,
-	       "MPI_Rsend" );
-    */
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Rsend( buf, count, datatype, dest, tag, comm );
-
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -2194,9 +2238,9 @@ MPI_Request * request;
   
   returnVal = PMPI_Rsend_init( buf, count, datatype, dest, tag, comm, request );
 
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   TauAddRequest(RQ_SEND, count, datatype, dest, tag, comm, request, returnVal, 1);
-#endif /* TAU_TRACK_MSG */
+}
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -2213,29 +2257,19 @@ int tag;
 MPI_Comm comm;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize;
-#endif /* TAU_TRACK_MSG */
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Send()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize );
-
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
-    /*
-    prof_send( procid_0, dest, tag, typesize*count,
-	       "MPI_Send" );
-    */
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Send( buf, count, datatype, dest, tag, comm );
-
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -2254,41 +2288,35 @@ MPI_Comm comm;
 MPI_Status * status;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   MPI_Status local_status;
   int typesize1;
   int count;
-#endif /* TAU_TRACK_MSG */
 
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Sendrecv()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( sendtype, &typesize1 );
-    TAU_TRACE_SENDMSG(sendtag, translateRankToWorld(comm, dest), typesize1*sendcount);
-    /*
-    prof_send( procid_0, dest, sendtag,
-               typesize1*sendcount, "MPI_Sendrecv" );
-    */
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( sendtype, &typesize1 );
+      TAU_TRACE_SENDMSG(sendtag, translateRankToWorld(comm, dest), typesize1*sendcount);
+    }
+    
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
   }
-
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
-  }
-#endif /* TAU_TRACK_MSG */
   
 
   returnVal = PMPI_Sendrecv( sendbuf, sendcount, sendtype, dest, sendtag, recvbuf, recvcount, recvtype, source, recvtag, comm, status );
 
-#ifdef TAU_TRACK_MSG
-  if (source != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
-    PMPI_Get_count( status, MPI_BYTE, &count );
-    TAU_TRACE_RECVMSG(status->MPI_TAG, translateRankToWorld(comm, status->MPI_SOURCE), count);
+  if (TauEnv_get_track_message()) {
+    if (source != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
+      PMPI_Get_count( status, MPI_BYTE, &count );
+      TAU_TRACE_RECVMSG(status->MPI_TAG, translateRankToWorld(comm, status->MPI_SOURCE), count);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   TAU_PROFILE_STOP(tautimer);
-
+  
   return returnVal;
 }
 
@@ -2304,44 +2332,34 @@ MPI_Comm comm;
 MPI_Status * status;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   MPI_Status local_status;
   int size1;
   int typesize2;
-#endif /* TAU_TRACK_MSG */
 
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Sendrecv_replace()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize2 );
-    TAU_TRACE_SENDMSG(sendtag, translateRankToWorld(comm, dest), typesize2*count);
-    /*
-    prof_send( procid_0, dest, sendtag,
-               typesize2*count, "MPI_Sendrecv_replace" );
-    */
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize2 );
+      TAU_TRACE_SENDMSG(sendtag, translateRankToWorld(comm, dest), typesize2*count);
+    }
+    
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
   }
-
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Sendrecv_replace( buf, count, datatype, dest, sendtag, source, recvtag, comm, status );
-
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
-    PMPI_Get_count( status, MPI_BYTE, &size1 );
-    TAU_TRACE_RECVMSG(status->MPI_TAG, translateRankToWorld(comm, status->MPI_SOURCE), size1);
-    /*
-    prof_recv( dest, procid_0, recvtag, size1,
-	       "MPI_Sendrecv_replace" );
-    */
+  
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL && returnVal == MPI_SUCCESS) {
+      PMPI_Get_count( status, MPI_BYTE, &size1 );
+      TAU_TRACE_RECVMSG(status->MPI_TAG, translateRankToWorld(comm, status->MPI_SOURCE), size1);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   TAU_PROFILE_STOP(tautimer);
-
+  
   return returnVal;
 }
 
@@ -2354,28 +2372,21 @@ int tag;
 MPI_Comm comm;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int typesize;
-#endif /* TAU_TRACK_MSG */
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Ssend()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  if (dest != MPI_PROC_NULL) {
-    PMPI_Type_size( datatype, &typesize );
-    TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
-    /*
-    prof_send( procid_0, dest, tag, typesize*count,
-	       "MPI_Ssend" );
-    */
+  if (TauEnv_get_track_message()) {
+    if (dest != MPI_PROC_NULL) {
+      PMPI_Type_size( datatype, &typesize );
+      TAU_TRACE_SENDMSG(tag, translateRankToWorld(comm, dest), typesize*count);
+    }
   }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Ssend( buf, count, datatype, dest, tag, comm );
-
+  
   TAU_PROFILE_STOP(tautimer);
-
+  
   return returnVal;
 }
 
@@ -2397,9 +2408,9 @@ MPI_Request * request;
   
   returnVal = PMPI_Ssend_init( buf, count, datatype, dest, tag, comm, request );
 
-#ifdef TAU_TRACK_MSG
+if (TauEnv_get_track_message()) {
   TauAddRequest(RQ_SEND, count, datatype, dest, tag, comm, request, returnVal, 1);
-#endif /* TAU_TRACK_MSG */
+}
 
   TAU_PROFILE_STOP(tautimer);
 
@@ -2409,36 +2420,25 @@ MPI_Request * request;
 int  MPI_Start( request )
 MPI_Request * request;
 {
-#ifdef TAU_TRACK_MSG
   request_list *rq;
-#endif /* TAU_TRACK_MSG */
   int  returnVal;
 
   TAU_PROFILE_TIMER(tautimer, "MPI_Start()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-
-
-
-#ifdef TAU_TRACK_MSG
-  rq = TauGetRequest(*request);
-  TauProcessSend(*request, "MPI_Start");
-#endif /* TAU_TRACK_MSG */
-
+  if (TauEnv_get_track_message()) {
+    rq = TauGetRequest(*request);
+    TauProcessSend(*request, "MPI_Start");
+  }
 
   returnVal = PMPI_Start( request );
 
-
-#ifdef TAU_TRACK_MSG
-  /* fix up the request since MPI_Start may (will) change it */
-  rq->request = *request;
-#endif /* TAU_TRACK_MSG */
-
-
-
-
+  if (TauEnv_get_track_message()) {
+    /* fix up the request since MPI_Start may (will) change it */
+    rq->request = *request;
+  }
+  
   TAU_PROFILE_STOP(tautimer);
-
   return returnVal;
 }
 
@@ -2464,28 +2464,27 @@ int * flag;
 MPI_Status * status;
 {
   int   returnVal;
-#ifdef TAU_TRACK_MSG
   MPI_Request saverequest;
   MPI_Status local_status;
-#endif /* TAU_TRACK_MSG */
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Test()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
-  saverequest = *request;
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
+  if (TauEnv_get_track_message()) {
+    saverequest = *request;
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   returnVal = PMPI_Test( request, flag, status );
-
-#ifdef TAU_TRACK_MSG
-  if (*flag)
-    TauProcessRecv(saverequest, status, "MPI_Test");
-#endif /* TAU_TRACK_MSG */
-
+  
+  if (TauEnv_get_track_message()) {
+    if (*flag) {
+      TauProcessRecv(saverequest, status, "MPI_Test");
+    }
+  }
+  
   TAU_PROFILE_STOP(tautimer);
   return returnVal;
 }
@@ -2496,39 +2495,38 @@ MPI_Request * array_of_requests;
 int * flag;
 MPI_Status * array_of_statuses;
 {
-  int  returnVal;
-#ifdef TAU_TRACK_MSG
+  int returnVal;
   int need_to_free = 0;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
 
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Testall()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < count; i++) {
-    saverequest[i] = array_of_requests[i];
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < count; i++) {
+      saverequest[i] = array_of_requests[i];
+    }
+    if (array_of_statuses == MPI_STATUSES_IGNORE) {
+      array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*count);
+      need_to_free = 1;
+    }
   }
-  if (array_of_statuses == MPI_STATUSES_IGNORE) {
-    array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*count);
-    need_to_free = 1;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Testall( count, array_of_requests, flag, array_of_statuses );
-
-#ifdef TAU_TRACK_MSG
-  if (*flag)
-  { /* at least one completed */
-    for(i=0; i < count; i++)
-      TauProcessRecv(saverequest[i], &array_of_statuses[i], "MPI_Testall");
+  
+  if (TauEnv_get_track_message()) {
+    if (*flag) { 
+      /* at least one completed */
+      for(i=0; i < count; i++) {
+	TauProcessRecv(saverequest[i], &array_of_statuses[i], "MPI_Testall");
+      }
+    }
+    if (need_to_free) {
+      free(array_of_statuses);
+    }
   }
-  if (need_to_free) {
-    free(array_of_statuses);
-  }
-#endif /* TAU_TRACK_MSG */
-
+  
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -2542,35 +2540,31 @@ int * flag;
 MPI_Status * status;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   MPI_Status local_status;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
-
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Testany()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < count; i++) {
-    saverequest[i] = array_of_requests[i];
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < count; i++) {
+      saverequest[i] = array_of_requests[i];
+    }
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
   }
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Testany( count, array_of_requests, index, flag, status );
-
-
-#ifdef TAU_TRACK_MSG
-  if (*flag)
-    TauProcessRecv( (saverequest[*index]),
-			        status, "MPI_Testany" );
-
-#endif /* TAU_TRACK_MSG */
+  
+  
+  if (TauEnv_get_track_message()) {
+    if (*flag && (*index != MPI_UNDEFINED)) {
+      TauProcessRecv(saverequest[*index], status, "MPI_Testany");
+    }
+    
+  }
   TAU_PROFILE_STOP(tautimer);
   return returnVal;
 }
@@ -2599,40 +2593,38 @@ int * array_of_indices;
 MPI_Status * array_of_statuses;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int need_to_free = 0;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Testsome()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
-
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < incount; i++){
-    saverequest[i] = array_of_requests[i];
+  
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < incount; i++){
+      saverequest[i] = array_of_requests[i];
+    }
+    if (array_of_statuses == MPI_STATUSES_IGNORE) {
+      array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*incount);
+      need_to_free = 1;
+    }
   }
-  if (array_of_statuses == MPI_STATUSES_IGNORE) {
-    array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*incount);
-    need_to_free = 1;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Testsome( incount, array_of_requests, outcount, array_of_indices, array_of_statuses );
-
-#ifdef TAU_TRACK_MSG
-  for (i=0; i < *outcount; i++) {
-    TauProcessRecv( (saverequest [array_of_indices[i]]),
-			        &(array_of_statuses[i]),
-			        "MPI_Testsome" );
-  }
-  if (need_to_free) {
+  
+  if (TauEnv_get_track_message()) {
+    for (i=0; i < *outcount; i++) {
+      TauProcessRecv( (saverequest [array_of_indices[i]]),
+		      &(array_of_statuses[i]),
+		      "MPI_Testsome" );
+    }
+    if (need_to_free) {
     free(array_of_statuses);
+    }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   TAU_PROFILE_STOP(tautimer);
-
+  
   return returnVal;
 }
 
@@ -2868,32 +2860,25 @@ MPI_Request * request;
 MPI_Status * status;
 {
   int   returnVal;
-
-#ifdef TAU_TRACK_MSG
   MPI_Status local_status;
   MPI_Request saverequest;
-#endif /* TAU_TRACK_MSG */
 
   TAU_PROFILE_TIMER(tautimer, "MPI_Wait()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
+  if (TauEnv_get_track_message()) {
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
+    saverequest = *request;
   }
-#endif /* TAU_TRACK_MSG */
-
-
-#ifdef TAU_TRACK_MSG
-  saverequest = *request;
-#endif /* TAU_TRACK_MSG */
-
+  
   returnVal = PMPI_Wait( request, status );
-
-#ifdef TAU_TRACK_MSG
-  TauProcessRecv(saverequest, status, "MPI_Wait");
-#endif /* TAU_TRACK_MSG */
-
+  
+  if (TauEnv_get_track_message()) {
+    TauProcessRecv(saverequest, status, "MPI_Wait");
+  }
+  
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -2905,38 +2890,36 @@ MPI_Request * array_of_requests;
 MPI_Status * array_of_statuses;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   int need_to_free = 0;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Waitall()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < count; i++) {
-    saverequest[i] = array_of_requests[i];
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < count; i++) {
+      saverequest[i] = array_of_requests[i];
+    }
+    
+    if (array_of_statuses == MPI_STATUSES_IGNORE) {
+      array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*count);
+      need_to_free = 1;
+    }
   }
-
-  if (array_of_statuses == MPI_STATUSES_IGNORE) {
-    array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*count);
-    need_to_free = 1;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Waitall( count, array_of_requests, array_of_statuses );
-
-#ifdef TAU_TRACK_MSG
-  for(i=0; i < count; i++)
-    TauProcessRecv(saverequest[i], &array_of_statuses[i], "MPI_Waitall");
-
-  if (need_to_free) {
-    free(array_of_statuses);
+  
+  if (TauEnv_get_track_message()) {
+    for(i=0; i < count; i++) {
+      TauProcessRecv(saverequest[i], &array_of_statuses[i], "MPI_Waitall");
+    }
+    
+    if (need_to_free) {
+      free(array_of_statuses);
+    }
   }
-#endif /* TAU_TRACK_MSG */
-
+  
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -2949,34 +2932,30 @@ int * index;
 MPI_Status * status;
 {
   int  returnVal;
-#ifdef TAU_TRACK_MSG
   MPI_Status local_status;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
-
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Waitany()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
   
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < count; i++){
-    saverequest[i] = array_of_requests[i];
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < count; i++){
+      saverequest[i] = array_of_requests[i];
+    }
+    if (status == MPI_STATUS_IGNORE) {
+      status = &local_status;
+    }
   }
-  if (status == MPI_STATUS_IGNORE) {
-    status = &local_status;
-  }
-#endif /* TAU_TRACK_MSG */
-
+  
   returnVal = PMPI_Waitany( count, array_of_requests, index, status );
 
 
-#ifdef TAU_TRACK_MSG
-  TauProcessRecv( (saverequest[*index]),
-			status, "MPI_Waitany" );
-#endif /* TAU_TRACK_MSG */
-
+  if (TauEnv_get_track_message()) {
+    TauProcessRecv( (saverequest[*index]),
+		    status, "MPI_Waitany" );
+  }
+  
   TAU_PROFILE_STOP(tautimer);
   return returnVal;
 }
@@ -2990,41 +2969,38 @@ MPI_Status * array_of_statuses;
 {
   int  returnVal;
 
-#ifdef TAU_TRACK_MSG
   int need_to_free = 0;
   int i;
   MPI_Request saverequest[TAU_MAX_REQUESTS];
-#endif /* TAU_TRACK_MSG */
-
   
   TAU_PROFILE_TIMER(tautimer, "MPI_Waitsome()",  " ", TAU_MESSAGE);
   TAU_PROFILE_START(tautimer);
 
-#ifdef TAU_TRACK_MSG
-  for (i = 0; i < incount; i++) {
-    saverequest[i] = array_of_requests[i];
+  if (TauEnv_get_track_message()) {
+    for (i = 0; i < incount; i++) {
+      saverequest[i] = array_of_requests[i];
+    }
+    
+    if (array_of_statuses == MPI_STATUSES_IGNORE) {
+      array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*incount);
+      need_to_free = 1;
+    }
   }
-
-  if (array_of_statuses == MPI_STATUSES_IGNORE) {
-    array_of_statuses = (MPI_Status*) malloc (sizeof(MPI_Status)*incount);
-    need_to_free = 1;
-  }
-#endif /* TAU_TRACK_MSG */
   
   returnVal = PMPI_Waitsome( incount, array_of_requests, outcount, array_of_indices, array_of_statuses );
 
-
-#ifdef TAU_TRACK_MSG
-  for (i=0; i < *outcount; i++) {
-    TauProcessRecv( (saverequest [array_of_indices[i]]),
-			        &(array_of_statuses[i]),
-			        "MPI_Waitsome" );
+  
+  if (TauEnv_get_track_message()) {
+    for (i=0; i < *outcount; i++) {
+      TauProcessRecv( (saverequest [array_of_indices[i]]),
+		      &(array_of_statuses[i]),
+		      "MPI_Waitsome" );
+    }
+    if (need_to_free) {
+      free(array_of_statuses);
+    }
+    
   }
-  if (need_to_free) {
-    free(array_of_statuses);
-  }
-
-#endif /* TAU_TRACK_MSG */
   TAU_PROFILE_STOP(tautimer);
 
   return returnVal;
@@ -3318,4 +3294,58 @@ int * top_type;
   return returnVal;
 }
 
+
+unsigned long hash(unsigned char *str) {
+  unsigned long hash = 5381;
+  int c;
+  
+  while (c = *str++) {
+    hash = ((hash << 5) + hash) + c; /* hash * 33 + c */
+  }
+  
+  return hash;
+}
+
+//For a given process, process is the unique MPI rank
+//Node n is the nth node in the allocation
+//Core m is the mth core on node n
+int TauGetCpuSite(unsigned int *node, unsigned int *core, unsigned int *rank) {
+  char host_name[MPI_MAX_PROCESSOR_NAME];
+  char (*host_names)[MPI_MAX_PROCESSOR_NAME];
+  MPI_Comm internode;
+  MPI_Comm intranode;
+  
+  int nprocs, namelen,n,bytes;
+  
+  MPI_Comm_rank(MPI_COMM_WORLD, rank);
+  MPI_Comm_size(MPI_COMM_WORLD, &nprocs);
+  MPI_Get_processor_name(host_name,&namelen);
+  bytes = nprocs * sizeof(char[MPI_MAX_PROCESSOR_NAME]);
+  
+  host_names = (char (*)[MPI_MAX_PROCESSOR_NAME]) malloc(bytes);
+  
+  strcpy(host_names[*rank], host_name);
+  for (n=0; n<nprocs; n++) {
+    MPI_Bcast(&(host_names[n]),MPI_MAX_PROCESSOR_NAME, MPI_CHAR, n, MPI_COMM_WORLD); 
+  }
+  
+  unsigned int color;
+  color = 0;
+  
+  for (n=1; n<nprocs; n++) {
+    if(strcmp(host_names[n-1], host_names[n])) color++;
+    if(strcmp(host_name, host_names[n]) == 0) break;
+  }
+  
+  MPI_Comm_split(MPI_COMM_WORLD, color, *rank, &internode);
+  MPI_Comm_rank(internode,core);
+  
+  MPI_Comm_split(MPI_COMM_WORLD, *core, *rank, &intranode);
+  
+  MPI_Comm_rank(intranode,node);
+  return 0;
+}
+
+
+
 /* EOF TauMpi.c */
