@@ -2,27 +2,14 @@
 **			TAU Portable Profiling Package			   **
 **			http://www.cs.uoregon.edu/research/tau	           **
 *****************************************************************************
-**    Copyright 1997  						   	   **
+**    Copyright 1997-2009	          			   	   **
 **    Department of Computer and Information Science, University of Oregon **
 **    Advanced Computing Laboratory, Los Alamos National Laboratory        **
 ****************************************************************************/
 /***************************************************************************
 **	File 		: UserEvent.cpp					  **
 **	Description 	: TAU Profiling Package				  **
-**	Author		: Sameer Shende					  **
-**	Contact		: sameer@cs.uoregon.edu sameer@acl.lanl.gov 	  **
-**	Flags		: Compile with				          **
-**			  -DPROFILING_ON to enable profiling (ESSENTIAL)  **
-**			  -DPROFILE_STATS for Std. Deviation of Excl Time **
-**			  -DSGI_HW_COUNTERS for using SGI counters 	  **
-**			  -DPROFILE_CALLS  for trace of each invocation   **
-**                        -DSGI_TIMERS  for SGI fast nanosecs timer       **
-**			  -DTULIP_TIMERS for non-sgi Platform	 	  **
-**			  -DPOOMA_STDSTL for using STD STL in POOMA src   **
-**			  -DPOOMA_TFLOP for Intel Teraflop at SNL/NM 	  **
-**			  -DPOOMA_KAI for KCC compiler 			  **
-**			  -DDEBUG_PROF  for internal debugging messages   **
-**                        -DPROFILE_CALLSTACK to enable callstack traces  **
+**	Contact		: tau-bugs@cs.uoregon.edu 		 	  **
 **	Documentation	: See http://www.cs.uoregon.edu/research/tau      **
 ***************************************************************************/
 
@@ -40,19 +27,11 @@
 //#define DEBUG_PROF
 
 #include "Profile/Profiler.h"
-
-
-#ifdef TAU_WINDOWS
-  typedef __int64 x_int64;
-  typedef unsigned __int64 x_uint64;
-#else
-  typedef long long x_int64;
-  typedef unsigned long long x_uint64;
-#endif
-
+#include <tau_internal.h>
 
 #include <stdio.h>
 #include <fcntl.h>
+#include <stdlib.h>
 
 //#include <math.h>
 #ifdef TAU_DOT_H_LESS_HEADERS
@@ -62,19 +41,16 @@ using namespace std;
 #include <iostream.h>
 #endif /* TAU_DOT_H_LESS_HEADERS */
 
-#ifdef TRACING_ON
 #ifdef TAU_VAMPIRTRACE
 #include <otf.h>
 #include "Profile/TauVampirTrace.h"
-#else /* TAU_VAMPIRTRACE */
+#endif /* TAU_VAMPIRTRACE */
+
 #ifdef TAU_EPILOG
 #include "elg_trc.h"
-#else /* TAU_EPILOG */
-#define PCXX_EVENT_SRC
-#include "Profile/pcxx_events.h"
 #endif /* TAU_EPILOG */
-#endif /* TAU_VAMPIRTRACE */
-#endif // TRACING_ON 
+
+#include <Profile/TauTrace.h>
 
 
 
@@ -84,16 +60,15 @@ template TauUserEvent** copy_backward(TauUserEvent**,TauUserEvent**,TauUserEvent
 template TauUserEvent** uninitialized_copy(TauUserEvent**,TauUserEvent**,TauUserEvent**);
 #endif // PGI
 
-vector<TauUserEvent*>& TheEventDB(void)
-{
-  static vector<TauUserEvent*> EventDB;
 
+
+vector<TauUserEvent*>& TheEventDB(void) {
+  static vector<TauUserEvent*> EventDB;
   return EventDB;
 }
 
 // Add User Event to the EventDB
-void TauUserEvent::AddEventToDB()
-{
+void TauUserEvent::AddEventToDB() {
   RtsLayer::LockDB();
   TheEventDB().push_back(this);
   DEBUGPROFMSG("Successfully registered event " << GetEventName() << endl;);
@@ -108,15 +83,15 @@ void TauUserEvent::AddEventToDB()
   return;
 }
 
-// Constructor 
-long TauUserEvent::GetEventId(void) 
-{
+long TauUserEvent::GetEventId(void) {
   return EventId;
 }
 
-// Constructor 
-TauUserEvent::TauUserEvent(const char * EName, bool increasing)
-{
+extern "C" long TauUserEvent_GetEventId(TauUserEvent *evt) {
+  return evt->GetEventId();
+}
+
+TauUserEvent::TauUserEvent(const char * EName, bool increasing) {
   DEBUGPROFMSG("Inside ctor of TauUserEvent EName = "<< EName << endl;);
 
   EventName 	= EName;
@@ -127,8 +102,7 @@ TauUserEvent::TauUserEvent(const char * EName, bool increasing)
   DisableStdDev = false; 	// StdDev   is calculated
   MonotonicallyIncreasing = increasing; // By default it is false 
 
-  for(int i=0; i < TAU_MAX_THREADS; i++) 
-  {
+  for(int i=0; i < TAU_MAX_THREADS; i++) {
     LastValueRecorded[i] = 0;  	// null to start with
     NumEvents[i] = 0L; 		// initialize
     MinValue[i]  = 9999999;  	// Least -ve value? limits.h
@@ -142,8 +116,7 @@ TauUserEvent::TauUserEvent(const char * EName, bool increasing)
 }
 
 // Copy Constructor 
-TauUserEvent::TauUserEvent(TauUserEvent& X)
-{
+TauUserEvent::TauUserEvent(TauUserEvent& X) {
   DEBUGPROFMSG("Inside copy ctor TauUserEvent::TauUserEvent()" << endl;);
 
   EventName 	= X.EventName;
@@ -166,8 +139,7 @@ TauUserEvent::TauUserEvent(TauUserEvent& X)
 }
 
 // Default constructor
-TauUserEvent::TauUserEvent()
-{
+TauUserEvent::TauUserEvent() {
   EventName 	= string("No Name");
   DisableMin 	= false; 	// Min 	    is calculated 
   DisableMax 	= false; 	// Max      is calculated 
@@ -175,8 +147,7 @@ TauUserEvent::TauUserEvent()
   DisableStdDev = false; 	// StdDev   is calculated
   MonotonicallyIncreasing = false; // By default it does not have any constraints
 
-  for (int i=0; i < TAU_MAX_THREADS; i++)
-  {
+  for (int i=0; i < TAU_MAX_THREADS; i++) {
     LastValueRecorded[i] = 0;  	// null to start with
     NumEvents[i] = 0L; 		// initialize
     MinValue[i]  = 9999999;  	// Least -ve value? limits.h
@@ -190,8 +161,7 @@ TauUserEvent::TauUserEvent()
 }
 
 // Assignment operator
-TauUserEvent& TauUserEvent::operator= (const TauUserEvent& X)
-{
+TauUserEvent& TauUserEvent::operator= (const TauUserEvent& X) {
 
   DEBUGPROFMSG("Inside TauUserEvent::operator= (const TauUserEvent& X)" << endl;);
 
@@ -215,16 +185,14 @@ TauUserEvent& TauUserEvent::operator= (const TauUserEvent& X)
 ///////////////////////////////////////////////////////////
 // GetMonotonicallyIncreasing
 ///////////////////////////////////////////////////////////
-bool TauUserEvent::GetMonotonicallyIncreasing(void)
-{
+bool TauUserEvent::GetMonotonicallyIncreasing(void) {
   return MonotonicallyIncreasing; 
 }
 
 ///////////////////////////////////////////////////////////
 // SetMonotonicallyIncreasing
 ///////////////////////////////////////////////////////////
-void TauUserEvent::SetMonotonicallyIncreasing(bool value)
-{
+void TauUserEvent::SetMonotonicallyIncreasing(bool value) {
   MonotonicallyIncreasing = value; 
 }
 
@@ -232,9 +200,11 @@ void TauUserEvent::SetMonotonicallyIncreasing(bool value)
 // TriggerEvent records the value of data in the UserEvent
 ///////////////////////////////////////////////////////////
 
-void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid)
-{ 
-#ifdef TRACING_ON
+void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid) { 
+  TriggerEvent(data, tid, 0, 0);
+}
+
+void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid, double timestamp, int use_ts) { 
 #ifdef TAU_VAMPIRTRACE
   uint64_t time;
   uint64_t cval;
@@ -248,13 +218,14 @@ void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid)
   vt_count(&time, id, 0);
 #else /* TAU_VAMPIRTRACE */
 #ifndef TAU_EPILOG
-  TraceEvent(GetEventId(), (x_uint64) 0, tid, 0, 0); 
-  TraceEvent(GetEventId(), (x_uint64) data, tid, 0, 0); 
-  TraceEvent(GetEventId(), (x_uint64) 0, tid, 0, 0); 
+  if (TauEnv_get_tracing()) {
+    TauTraceEvent(GetEventId(), (x_uint64) 0, tid, (x_uint64) timestamp, use_ts); 
+    TauTraceEvent(GetEventId(), (x_uint64) data, tid, (x_uint64) timestamp, use_ts); 
+    TauTraceEvent(GetEventId(), (x_uint64) 0, tid, (x_uint64) timestamp, use_ts); 
+  }
 #endif /* TAU_EPILOG */
   /* Timestamp is 0, and use_ts is 0, so tracing layer gets timestamp */
 #endif /* TAU_VAMPIRTRACE */
-#endif /* TRACING_ON */
 
 #ifdef PROFILING_ON
   // Record this value  
@@ -264,29 +235,31 @@ void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid)
   NumEvents[tid] ++;
 
   // Compute relevant statistics for the data 
-  if (!GetDisableMin()) 
-  {  // Min is not disabled
+  if (!GetDisableMin()) {  
+    // Min is not disabled
      if (NumEvents[tid] > 1) {
-     	MinValue[tid] = data < MinValue[tid] ? data : MinValue[tid];
-     } else
+       MinValue[tid] = data < MinValue[tid] ? data : MinValue[tid];
+     } else {
 	MinValue[tid] = data;
+     }
   }
   
-  if (!GetDisableMax())
-  {  // Max is not disabled
+  if (!GetDisableMax()) {  
+    // Max is not disabled
      if (NumEvents[tid] > 1) {
        MaxValue[tid] = MaxValue[tid] < data ? data : MaxValue[tid];
-     } else
+     } else {
        MaxValue[tid] = data;
+     }
   }
 
-  if (!GetDisableMean())
-  {  // Mean is not disabled 
+  if (!GetDisableMean()) {  
+    // Mean is not disabled 
      SumValue[tid] += data; 
   }
      
-  if (!GetDisableStdDev())
-  {  // Standard Deviation is not disabled
+  if (!GetDisableStdDev()) {  
+    // Standard Deviation is not disabled
      SumSqrValue[tid] += data*data; 
   }
 
@@ -295,126 +268,104 @@ void TauUserEvent::TriggerEvent(TAU_EVENT_DATATYPE data, int tid)
 }
 
 // Return the data stored in the class
-TAU_EVENT_DATATYPE TauUserEvent::GetMin(int tid)
-{ 
-  if (NumEvents[tid] != 0L)
-  { 
+TAU_EVENT_DATATYPE TauUserEvent::GetMin(int tid) { 
+  if (NumEvents[tid] != 0L) { 
     return MinValue[tid];
-  }
-  else
+  } else {
     return 0;
+  }
 }
 
-TAU_EVENT_DATATYPE TauUserEvent::GetMax(int tid)
-{
-  if (NumEvents[tid] != 0L)
-  {
+TAU_EVENT_DATATYPE TauUserEvent::GetMax(int tid) {
+  if (NumEvents[tid] != 0L) {
     return MaxValue[tid];
-  }
-  else
+  } else {
     return 0;
+  }
 }
 
-TAU_EVENT_DATATYPE TauUserEvent::GetSumValue(int tid)
-{  
-  if (NumEvents[tid] != 0L)
-  {
+TAU_EVENT_DATATYPE TauUserEvent::GetSumValue(int tid) {  
+  if (NumEvents[tid] != 0L) {
     return SumValue[tid];
+  } else {
+    return 0;
   }
-  else
-    return 0;
 }
 
-TAU_EVENT_DATATYPE TauUserEvent::GetMean(int tid)
-{
-  if (NumEvents[tid] != 0L) 
-  {
+TAU_EVENT_DATATYPE TauUserEvent::GetMean(int tid) {
+  if (NumEvents[tid] != 0L) {
     return (SumValue[tid]/NumEvents[tid]);
-  } 
-  else
+  } else {
     return 0;
+  }
 }
 
-double TauUserEvent::GetSumSqr(int tid)
-{
+double TauUserEvent::GetSumSqr(int tid) {
   return (SumSqrValue[tid]);
 }
 
-long TauUserEvent::GetNumEvents(int tid)
-{
+long TauUserEvent::GetNumEvents(int tid) {
   return NumEvents[tid];
 }
 
 // Get the event name
-const char * TauUserEvent::GetEventName (void) const
-{
+const char *TauUserEvent::GetEventName (void) const {
   return EventName.c_str();
 }
 
 // Set the event name
-void TauUserEvent::SetEventName (const char *newname)
-{
+void TauUserEvent::SetEventName (const char *newname) {
   EventName = newname;
 }
 
 // Set the event name
-void TauUserEvent::SetEventName (string newname)
-{
+void TauUserEvent::SetEventName (string newname) {
   EventName = newname;
 }
 
-bool TauUserEvent::GetDisableMin(void)
-{ 
+bool TauUserEvent::GetDisableMin(void) { 
   return DisableMin;
 }
 
-bool TauUserEvent::GetDisableMax(void)
-{
+bool TauUserEvent::GetDisableMax(void) {
   return DisableMax;
 }
 
-bool TauUserEvent::GetDisableMean(void)
-{
+bool TauUserEvent::GetDisableMean(void) {
   return DisableMean;
 }
 
-bool TauUserEvent::GetDisableStdDev(void)
-{
+bool TauUserEvent::GetDisableStdDev(void) {
   return DisableStdDev;
 }
 
 // Set Routines
-void TauUserEvent::SetDisableMin(bool value)
-{
+void TauUserEvent::SetDisableMin(bool value) {
   DisableMin = value;
   return;
 }
 
-void TauUserEvent::SetDisableMax(bool value)
-{
+void TauUserEvent::SetDisableMax(bool value) {
   DisableMax = value;
   return;
 }
 
-void TauUserEvent::SetDisableMean(bool value)
-{
+void TauUserEvent::SetDisableMean(bool value) {
   DisableMean = value;
   return;
 }
 
-void TauUserEvent::SetDisableStdDev(bool value)
-{
+void TauUserEvent::SetDisableStdDev(bool value) {
   DisableStdDev = value;
   return;
 }
 
-TauUserEvent::~TauUserEvent(void)
-{
+TauUserEvent::~TauUserEvent(void) {
   DEBUGPROFMSG(" DTOR CALLED for " << GetEventName() << endl;); 
+  Tau_destructor_trigger();
 }
 
-void TauUserEvent::ReportStatistics(bool ForEachThread)
-{
+void TauUserEvent::ReportStatistics(bool ForEachThread) {
   TAU_EVENT_DATATYPE TotalNumEvents, TotalSumValue, Minima, Maxima ;
   vector<TauUserEvent*>::iterator it;
 
@@ -422,8 +373,7 @@ void TauUserEvent::ReportStatistics(bool ForEachThread)
   cout << "TAU Runtime Statistics" <<endl;
   cout << "*************************************************************" << endl;
 
-  for(it  = TheEventDB().begin(); it != TheEventDB().end(); it++)
-  {
+  for(it  = TheEventDB().begin(); it != TheEventDB().end(); it++) {
     DEBUGPROFMSG("TauUserEvent "<< 
       (*it)->GetEventName() << "\n Min " << (*it)->GetMin() << "\n Max " <<
       (*it)->GetMax() << "\n Mean " << (*it)->GetMean() << "\n Sum Sqr " <<
@@ -431,42 +381,39 @@ void TauUserEvent::ReportStatistics(bool ForEachThread)
       
     TotalNumEvents = TotalSumValue = 0;
 
-    for (int tid = 0; tid < TAU_MAX_THREADS; tid++)
-    { 
-      if ((*it)->GetNumEvents(tid) > 0)
-      { // There were some events on this thread 
+    for (int tid = 0; tid < TAU_MAX_THREADS; tid++) { 
+      if ((*it)->GetNumEvents(tid) > 0) { 
+	// There were some events on this thread 
         TotalNumEvents += (*it)->GetNumEvents(tid); 
 	TotalSumValue  += (*it)->GetSumValue(tid);
 
-        if (!(*it)->GetDisableMin())
-        { // Min is not disabled
+        if (!(*it)->GetDisableMin()) { 
+	  // Min is not disabled
 	  // take the lesser of Minima and the min on that thread
-	  if (tid > 0) 
-	  { // more than one thread
+	  if (tid > 0) { 
+	    // more than one thread
 	    Minima = (*it)->GetMin(tid) < Minima ? (*it)->GetMin(tid) : Minima;
-	  } 
-	  else 
-	  { // this is the first thread. Initialize Minima to the min on it.
+	  } else { 
+	    // this is the first thread. Initialize Minima to the min on it.
 	    Minima = (*it)->GetMin(tid);
 	  }
 	} 
 
-	if (!(*it)->GetDisableMax())
-	{ // Max is not disabled
+	if (!(*it)->GetDisableMax()) {
+	  // Max is not disabled
 	  // take the maximum of Maxima and max on that thread
-	  if (tid > 0)
-	  { // more than one thread 
+	  if (tid > 0) {
+	    // more than one thread 
 	    Maxima = (*it)->GetMax(tid) > Maxima ? (*it)->GetMax(tid) : Maxima;
-	  } 
-	  else
-	  { // this is the first thread. Initialize Maxima to the max on it.
+	  } else { 
+	    // this is the first thread. Initialize Maxima to the max on it.
 	    Maxima = (*it)->GetMax(tid);
 	  }
 	}   
 	  
 
-	if (ForEachThread) 
-	{ // true, print statistics for this thread
+	if (ForEachThread) {
+	  // true, print statistics for this thread
 	  cout <<  "n,c,t "<<RtsLayer::myNode() <<"," <<RtsLayer::myContext()
 	       <<  "," << tid << " : Event : "<< (*it)->GetEventName() << endl
 	       <<  " Number : " << (*it)->GetNumEvents(tid) <<endl
@@ -515,16 +462,13 @@ void TauUserEvent::ReportStatistics(bool ForEachThread)
  * they're equal. If they two arrays have the same depth, then we iterate
  * through the array and compare each array element till the end */
 /////////////////////////////////////////////////////////////////////////
-struct TaultUserEventLong
-{
-  bool operator() (const long *l1, const long *l2) const
- {
+struct TaultUserEventLong {
+  bool operator() (const long *l1, const long *l2) const {
    int i;
    /* first check 0th index (size) */
    if (l1[0] != l2[0]) return (l1[0] < l2[0]);
    /* they're equal, see the size and iterate */
-   for (i = 0; i < l1[0] ; i++)
-   {
+   for (i = 1; i < l1[0] ; i++) {
      if (l1[i] != l2[i]) return l1[i] < l2[i];
    }
    return (l1[i] < l2[i]);
@@ -535,112 +479,61 @@ struct TaultUserEventLong
 /////////////////////////////////////////////////////////////////////////
 // We use one global map to store the callpath information
 /////////////////////////////////////////////////////////////////////////
-map<TAU_CONTEXT_MAP_TYPE >& TheContextMap(void)
-{ // to avoid initialization problems of non-local static variables
+map<TAU_CONTEXT_MAP_TYPE >& TheContextMap(void) { 
+  // to avoid initialization problems of non-local static variables
   static map<TAU_CONTEXT_MAP_TYPE > contextmap;
-
   return contextmap;
 }
 
-//////////////////////////////////////////////////////////////////////
-#define TAU_DEFAULT_CONTEXT_CALLPATH_DEPTH 2
-
-//////////////////////////////////////////////////////////////////////
-// How deep should the callpath be? The default value is 2
-//////////////////////////////////////////////////////////////////////
-int& TauGetContextCallPathDepth(void)
-{
-  char *depth;
-  static int value = 0;
-
-  
-  if (value == 0)
-  {
-    if ((depth = getenv("TAU_CALLPATH_DEPTH")) != NULL)
-    {
-      value = atoi(depth);
-      if (value > 1)
-      {
-        return value;
-      }
-      else
-      {
-        value = TAU_DEFAULT_CONTEXT_CALLPATH_DEPTH;
-        return value; /* default value */
-      }
-    }
-    else
-    {
-      value = TAU_DEFAULT_CONTEXT_CALLPATH_DEPTH;
-      return value;
-    }
-  }
-  else
-    return value;
-}
-
-
-    
-
 ////////////////////////////////////////////////////////////////////////////
 // Formulate Context Comparison Array
 //////////////////////////////////////////////////////////////////////
-long* TauFormulateContextComparisonArray(Profiler *p, TauUserEvent *uevent)
-{
-  int depth = TauGetContextCallPathDepth();
-  /* Create a long array with size depth+1. We need to put the depth
-   * in it as the 0th index */
+long* TauFormulateContextComparisonArray(Profiler *p, TauUserEvent *uevent) {
+  int depth = TauEnv_get_callpath_depth();
+  /* Create a long array with size depth+2. We need to put the depth
+   * in it as the 0th index, the user event goes as the tail element */
+
   long *ary = new long [depth+2];
-
-  int i = 0;
-  int j;
-  Profiler *current = p; /* argument */
-
+  if (!ary) return NULL;
   /* initialize the array */
-  for (j = 0; j < depth+1; j++)
-  {
+  for (int j = 0; j < depth+2; j++) {
     ary[j] = 0L;
   }
-  /* use the clean array now */
 
-  if (ary)
-  {
-    ary[0] = depth; /* this tells us how deep it is */
-    while (current != NULL && depth != 0)
-    {
-      i++; /* increment i */
-      ary[i] = (long) current->ThisFunction;
-      depth --;
-      current = current->ParentProfiler;
-    }
+  Profiler *current = p; /* argument */
+  int index = 1; /* start writing to index 1, we fill in the depth after */
+
+  while (current != NULL && depth != 0) {
+    ary[index++] = (long) current->ThisFunction;
+    depth--;
+    current = current->ParentProfiler;
   }
-  ary[i] = (long) uevent; 
+  
+  ary[index++] = (long) uevent;
+  ary[0] = index-1; /* set the depth */
   return ary;
 }
 
 ////////////////////////////////////////////////////////////////////////////
 // Formulate Context Callpath name string
 ////////////////////////////////////////////////////////////////////////////
-string * TauFormulateContextNameString(Profiler *p)
-{
-  DEBUGPROFMSG("Inside TauFormulateContextNameString()"<<endl;);
-  int depth = TauGetContextCallPathDepth();
+string * TauFormulateContextNameString(Profiler *p) {
+  int depth = TauEnv_get_callpath_depth();
   Profiler *current = p;
   string delimiter(" => ");
   string *name = new string("");
 
-  while (current != NULL && depth != 0)
-  {
-    if (current != p)
+  while (current != NULL && depth != 0) {
+    if (current != p) {
       *name =  current->ThisFunction->GetName() + string(" ") +
                current->ThisFunction->GetType() + delimiter + *name;
-    else
+    } else {
       *name =  current->ThisFunction->GetName() + string (" ") +
                current->ThisFunction->GetType();
+    }
     current = current->ParentProfiler;
     depth --;
   }
-  DEBUGPROFMSG("TauFormulateContextNameString:Name: "<<*name <<endl;);
   return name;
 }
 
@@ -649,19 +542,23 @@ string * TauFormulateContextNameString(Profiler *p)
 ////////////////////////////////////////////////////////////////////////////
 // Ctor for TauContextUserEvent 
 ////////////////////////////////////////////////////////////////////////////
-TauContextUserEvent::TauContextUserEvent(const char *EName, bool MonoIncr)
-{
+TauContextUserEvent::TauContextUserEvent(const char *EName, bool MonoIncr) {
   /* create the event */
   uevent = new TauUserEvent(EName, MonoIncr);
   DisableContext = false; /* context tracking is enabled by default */
+
+  int depth = TauEnv_get_callpath_depth();
+  if (depth == 0) {
+    DisableContext = true;
+  }
+
   MonotonicallyIncreasing = MonoIncr;
 }
 
 ////////////////////////////////////////////////////////////////////////////
 // Dtor for TauContextUserEvent 
 ////////////////////////////////////////////////////////////////////////////
-TauContextUserEvent::~TauContextUserEvent()
-{
+TauContextUserEvent::~TauContextUserEvent() {
   delete uevent; 
   delete contextevent; 
 }
@@ -669,65 +566,75 @@ TauContextUserEvent::~TauContextUserEvent()
 ////////////////////////////////////////////////////////////////////////////
 // SetDisableContext for TauContextUserEvent 
 ////////////////////////////////////////////////////////////////////////////
-void TauContextUserEvent::SetDisableContext(bool value)
-{
-  /* set it */
+void TauContextUserEvent::SetDisableContext(bool value) {
   DisableContext = value;
 }
 
 ////////////////////////////////////////////////////////////////////////////
 // GetEventName() returns the name of the context event 
 ////////////////////////////////////////////////////////////////////////////
-const char * TauContextUserEvent::GetEventName(void)
-{
-  if (contextevent)
+const char * TauContextUserEvent::GetEventName(void) {
+  if (contextevent) {
     return contextevent->GetEventName();
-  else 
+  } else {
     return (const char *)NULL;
+  }
 }
 ////////////////////////////////////////////////////////////////////////////
 // Trigger the context event
 ////////////////////////////////////////////////////////////////////////////
-void TauContextUserEvent::TriggerEvent( TAU_EVENT_DATATYPE data, int tid)
-{
-  if (!DisableContext)
-  {
+
+void TauContextUserEvent::TriggerEvent( TAU_EVENT_DATATYPE data, int tid) {
+  TriggerEvent(data, tid, 0, 0);
+}
+
+void TauContextUserEvent::TriggerEvent( TAU_EVENT_DATATYPE data, int tid, double timestamp, int use_ts) {
+  if (!DisableContext) {
     long *comparison = 0;
     TauUserEvent *ue;
     /* context tracking is enabled */
-    Profiler *current = Profiler::CurrentProfiler[tid];
+    Profiler *current = TauInternal_CurrentProfiler(tid);
     comparison = TauFormulateContextComparisonArray(current, uevent); 
 
     map<TAU_CONTEXT_MAP_TYPE>::iterator it = TheContextMap().find(comparison);
-    if (it == TheContextMap().end())
-    {
-      string *ctxname = TauFormulateContextNameString(current);
-      string contextname(uevent->EventName  + " : " + *ctxname);
-      DEBUGPROFMSG("Couldn't find string in map: "<<*comparison<<endl; );
-      ue = new TauUserEvent((const char *)(contextname.c_str()), MonotonicallyIncreasing);
-      TheContextMap().insert(map<TAU_CONTEXT_MAP_TYPE>::value_type(comparison, ue));
-      ue->ctxevt = this; /* store the current object in the user event */
-      delete ctxname; /* free up the string memory */
-    }
-    else
-    {
+
+    if (it == TheContextMap().end()) {
+      RtsLayer::LockEnv();
+      it = TheContextMap().find(comparison);
+      if (it == TheContextMap().end()) {
+	string *ctxname = TauFormulateContextNameString(current);
+	string contextname(uevent->EventName  + " : " + *ctxname);
+	
+	ue = new TauUserEvent((const char *)(contextname.c_str()), MonotonicallyIncreasing);
+	TheContextMap().insert(map<TAU_CONTEXT_MAP_TYPE>::value_type(comparison, ue));
+	
+	ue->ctxevt = this; /* store the current object in the user event */
+	delete ctxname; /* free up the string memory */
+      } else {
+	/* found it! Get the user defined event from the map */
+	ue = (*it).second;
+	delete[] comparison; // free up memory when name is found
+      }
+      RtsLayer::UnLockEnv();
+    } else {
       /* found it! Get the user defined event from the map */
       ue = (*it).second;
-      delete comparison; // free up memory when name is found
+      delete[] comparison; // free up memory when name is found
     }
+
     /* Now we trigger this event */
-    if (ue)
-    { /* it is not null, trigger it */
+    if (ue) { 
+      /* it is not null, trigger it */
       contextevent = ue;
       /* store this context event, so we can get its name */
-      contextevent->TriggerEvent(data, tid);
+      contextevent->TriggerEvent(data, tid, timestamp, use_ts);
     }
   }
-  uevent->TriggerEvent(data, tid);
+  uevent->TriggerEvent(data, tid, timestamp, use_ts);
 }
 
 /***************************************************************************
- * $RCSfile: UserEvent.cpp,v $   $Author: sameer $
- * $Revision: 1.21 $   $Date: 2007/11/06 18:53:33 $
- * POOMA_VERSION_ID: $Id: UserEvent.cpp,v 1.21 2007/11/06 18:53:33 sameer Exp $ 
+ * $RCSfile: UserEvent.cpp,v $   $Author: amorris $
+ * $Revision: 1.42 $   $Date: 2009/09/29 00:37:03 $
+ * POOMA_VERSION_ID: $Id: UserEvent.cpp,v 1.42 2009/09/29 00:37:03 amorris Exp $ 
  ***************************************************************************/
