@@ -18,9 +18,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
 /**
  * CallPathTextWindow: This window displays callpath data in a text format
  *   
- * <P>CVS $Id: CallPathTextWindow.java,v 1.29 2007/02/03 01:40:11 amorris Exp $</P>
+ * <P>CVS $Id: CallPathTextWindow.java,v 1.32 2009/09/10 00:13:44 amorris Exp $</P>
  * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.29 $
+ * @version	$Revision: 1.32 $
  * @see		CallPathDrawObject
  * @see		CallPathTextWindowPanel
  */
@@ -89,9 +89,6 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
             this.help(false);
         }
 
-        //####################################
-        //Create and add the components.
-        //####################################
         //Setting up the layout system for the main window.
         Container contentPane = getContentPane();
         GridBagLayout gbl = new GridBagLayout();
@@ -125,7 +122,6 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
         JMenuBar mainMenu = new JMenuBar();
         mainMenu.addKeyListener(this);
         JMenu subMenu = null;
-        JMenuItem menuItem = null;
 
         //Options menu.
         optionsMenu = new JMenu("Options");
@@ -294,11 +290,9 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
         }
     }
 
-    public void menuDeselected(MenuEvent evt) {
-    }
+    public void menuDeselected(MenuEvent evt) {}
 
-    public void menuCanceled(MenuEvent evt) {
-    }
+    public void menuCanceled(MenuEvent evt) {}
 
     public void update(Observable o, Object arg) {
         String tmpString = (String) arg;
@@ -320,8 +314,9 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
     public void help(boolean display) {
         //Show the ParaProf help window.
         ParaProf.getHelpWindow().clearText();
-        if (display)
-            ParaProf.getHelpWindow().show();
+        if (display) {
+            ParaProf.getHelpWindow().setVisible(true);
+        }
         ParaProf.getHelpWindow().writeText("Call path text window.");
         ParaProf.getHelpWindow().writeText("");
         ParaProf.getHelpWindow().writeText("This window displays call path relationships in two ways:");
@@ -353,7 +348,7 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
             dataSorter.setSortType(SortType.VALUE);
 
         dataSorter.setDescendingOrder(descendingOrder.isSelected());
-        dataSorter.setSelectedMetricID(ppTrial.getDefaultMetricID());
+        dataSorter.setSelectedMetric(ppTrial.getDefaultMetric());
 
         this.setHeader();
 
@@ -404,9 +399,6 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
         closeThisWindow();
     }
 
-    //######
-    //Panel header.
-    //######
     //This process is separated into two functionProfiles to provide the option
     //of obtaining the current header string being used for the panel
     //without resetting the actual header. Printing and image generation
@@ -428,15 +420,11 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
     }
 
     public String getHeaderString() {
-        return "Metric Name: " + (ppTrial.getMetricName(dataSorter.getSelectedMetricID())) + "\n" + "Sorted By: "
+        return "Metric Name: " + (dataSorter.getSelectedMetric().getName()) + "\n" + "Sorted By: "
                 + dataSorter.getValueType() + "\n" + "Units: "
                 + UtilFncs.getUnitsString(units, dataSorter.isTimeMetric(), dataSorter.isDerivedMetric()) + "\n";
     }
 
-    //######
-    //End - Panel header.
-    //######
-
     public void closeThisWindow() {
         setVisible(false);
         ppTrial.deleteObserver(this);
@@ -480,11 +468,9 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
         }
     }
 
-    public void keyReleased(KeyEvent e) {
-    }
+    public void keyReleased(KeyEvent e) {}
 
-    public void keyTyped(KeyEvent e) {
-    }
+    public void keyTyped(KeyEvent e) {}
 
     public void setUnits(int units) {
         this.units = units;
@@ -496,4 +482,7 @@ public class CallPathTextWindow extends JFrame implements ActionListener, MenuLi
     public JScrollPane getScrollPane() {
         return sp;
     }
+    public JFrame getFrame() {
+        return this;
+    }
 }
\ No newline at end of file
