@@ -17,9 +17,9 @@ import edu.uoregon.tau.perfdmf.Thread;
 /**
  * CallPathTextWindowPanel: This is the panel for the CallPathTextWindow
  *   
- * <P>CVS $Id: CallPathTextWindowPanel.java,v 1.43 2007/03/02 20:08:14 amorris Exp $</P>
+ * <P>CVS $Id: CallPathTextWindowPanel.java,v 1.45 2009/09/10 00:13:44 amorris Exp $</P>
  * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.43 $
+ * @version	$Revision: 1.45 $
  * @see		CallPathDrawObject
  * @see		CallPathTextWindow
  * 
@@ -130,8 +130,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
 
                     while (l3.hasNext()) {
                         FunctionProfile callPath = (FunctionProfile) l3.next();
-                        d1 = d1 + callPath.getExclusive(ppTrial.getDefaultMetricID());
-                        d2 = d2 + callPath.getInclusive(ppTrial.getDefaultMetricID());
+                        d1 = d1 + callPath.getExclusive(ppTrial.getDefaultMetric().getID());
+                        d2 = d2 + callPath.getInclusive(ppTrial.getDefaultMetric().getID());
                         d3 = d3 + callPath.getNumCalls();
                     }
                     CallPathDrawObject callPathDrawObject = new CallPathDrawObject(parent.getFunction(), true, false, false);
@@ -156,8 +156,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
                     double d3 = 0.0;
                     for (Iterator it3 = ppFunctionProfile.getFunctionProfile().getChildProfileCallPathIterator(child); it3.hasNext();) {
                         FunctionProfile callPath = (FunctionProfile) it3.next();
-                        d1 = d1 + callPath.getExclusive(ppTrial.getDefaultMetricID());
-                        d2 = d2 + callPath.getInclusive(ppTrial.getDefaultMetricID());
+                        d1 = d1 + callPath.getExclusive(ppTrial.getDefaultMetric().getID());
+                        d2 = d2 + callPath.getInclusive(ppTrial.getDefaultMetric().getID());
                         d3 = d3 + callPath.getNumCalls();
                     }
                     callPathDrawObject = new CallPathDrawObject(child.getFunction(), false, true, false);
@@ -269,8 +269,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
 
                     Function function = callPathDrawObject.getFunction();
 
-                    line = "--> " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11)
-                            + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11)
+                    line = "--> " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
+                            + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
                             + "      " + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCalls(), 7, false);
 
                     line = UtilFncs.pad(line, 58) + callPathDrawObject.getName();// + "[" + function.getID() + "]";
@@ -281,8 +281,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
 
                     Function function = callPathDrawObject.getFunction();
 
-                    line = "    " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11)
-                            + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11)
+                    line = "    " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
+                            + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
                             + "      "
                             + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCallsFromCallPathObjects(), 7, false) + "/"
                             + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCalls(), 7, false);
@@ -414,8 +414,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
                 g2D.drawString(dashString, base, yCoord);
             } else if (!callPathDrawObject.isParentChild() && !callPathDrawObject.isSpacer()) {
 
-                String stats = "--> " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11)
-                        + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11)
+                String stats = "--> " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
+                        + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
                         + "      " + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCalls(), 7, false);
                 g2D.drawString(stats, base, yCoord);
 
@@ -428,8 +428,8 @@ public class CallPathTextWindowPanel extends JPanel implements MouseListener, Pr
 
             } else if (callPathDrawObject.isSpacer()) {} else {
 
-                String stats = "    " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11)
-                        + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11)
+                String stats = "    " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getExclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
+                        + "      " + UtilFncs.getOutputString(window.units(), callPathDrawObject.getInclusiveValue(), 11, ppTrial.getDefaultMetric().isTimeDenominator())
                         + "      " + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCallsFromCallPathObjects(), 7, false)
                         + "/" + UtilFncs.formatDouble(callPathDrawObject.getNumberOfCalls(), 7, false);
 
