@@ -2,6 +2,7 @@ package edu.uoregon.tau.paraprof;
 
 import java.util.*;
 
+import edu.uoregon.tau.common.AlphanumComparator;
 import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.UserEventValueType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
@@ -14,22 +15,22 @@ import edu.uoregon.tau.perfdmf.Thread;
  * functions that are in groups supposed to be shown. 
  *  
  * 
- * <P>CVS $Id: DataSorter.java,v 1.9 2007/05/04 01:44:34 amorris Exp $</P>
+ * <P>CVS $Id: DataSorter.java,v 1.18 2009/10/27 00:13:48 amorris Exp $</P>
  * @author	Alan Morris, Robert Bell
- * @version	$Revision: 1.9 $
+ * @version	$Revision: 1.18 $
  */
 public class DataSorter implements Comparator {
 
     private ParaProfTrial ppTrial = null;
 
-    private int selectedMetricID;
+    private Metric selectedMetric;
+    private Metric sortMetric;
     private ValueType valueType;
+    private SortType sortType;
+    private ValueType sortValueType;
     private UserEventValueType userEventValueType = UserEventValueType.NUMSAMPLES;
 
     private boolean descendingOrder;
-    private SortType sortType;
-    private ValueType sortValueType;
-    private int sortMetric;
     private boolean sortByVisible = true;
 
     private Function phase;
@@ -37,10 +38,14 @@ public class DataSorter implements Comparator {
     private static SortType defaultSortType = SortType.MEAN_VALUE;
     private static ValueType defaultValueType = ValueType.EXCLUSIVE;
     private static boolean defaultSortOrder = true;
+    
+    private boolean selectedSnapshotOverride = false;
+    private int selectedSnapshot = -1;
 
     public DataSorter(ParaProfTrial ppTrial) {
         this.ppTrial = ppTrial;
-        this.selectedMetricID = ppTrial.getDefaultMetricID();
+        this.selectedMetric = ppTrial.getDefaultMetric();
+        this.sortMetric = ppTrial.getDefaultMetric();
 
         this.sortType = DataSorter.defaultSortType;
         this.valueType = DataSorter.defaultValueType;
@@ -49,13 +54,7 @@ public class DataSorter implements Comparator {
     }
 
     public boolean isTimeMetric() {
-        String metricName = ppTrial.getMetricName(this.getSelectedMetricID());
-        metricName = metricName.toUpperCase();
-        if (metricName.indexOf("TIME") == -1) {
-            return false;
-        } else {
-            return true;
-        }
+        return selectedMetric.isTimeMetric();
     }
 
     public boolean isDerivedMetric() {
@@ -64,17 +63,16 @@ public class DataSorter implements Comparator {
         //String metricName = this.getMetricName(this.getSelectedMetricID());
         //if (metricName.indexOf("*") != -1 || metricName.indexOf("/") != -1)
         //    return true;
-        return ppTrial.getMetric(this.getSelectedMetricID()).getDerivedMetric();
+        return selectedMetric.getDerivedMetric();
     }
 
     public List getUserEventProfiles(Thread thread) {
 
         UserEventProfile userEventProfile = null;
-        List list = thread.getUserEventProfiles();
 
         List newList = new ArrayList();
 
-        for (Iterator e1 = list.iterator(); e1.hasNext();) {
+        for (Iterator e1 = thread.getUserEventProfiles(); e1.hasNext();) {
             userEventProfile = (UserEventProfile) e1.next();
             if (userEventProfile != null) {
                 PPUserEventProfile ppUserEventProfile = new PPUserEventProfile(this, thread, userEventProfile);
@@ -129,7 +127,7 @@ public class DataSorter implements Comparator {
 
         for (int i = 0; i < functionList.size(); i++) {
             FunctionProfile functionProfile = (FunctionProfile) functionList.get(i);
-            if (functionProfile != null) {
+            if (functionProfile != null && ppTrial.displayFunction(functionProfile.getFunction())) {
                 newList.add(functionProfile);
             }
         }
@@ -410,6 +408,7 @@ public class DataSorter implements Comparator {
             }
         }
         Collections.sort(newList);
+        //Collections.sort(newList, new AlphanumComparator());
         return newList;
     }
 
@@ -501,8 +500,8 @@ public class DataSorter implements Comparator {
                     left.getFunction().getMeanProfile(), right.getFunction().getMeanProfile());
 
         } else if (sortType == SortType.VALUE) {
-            return compareToHelper(type.getValue(left, getSortMetric(), getSelectedSnapshot()), type.getValue(right, getSortMetric(),
-                    getSelectedSnapshot()));
+            return compareToHelper(type.getValue(left, getSortMetric(), getSelectedSnapshot()), type.getValue(right,
+                    getSortMetric(), getSelectedSnapshot()));
         } else {
             throw new ParaProfException("Unexpected sort type: " + sortType);
         }
@@ -574,15 +573,15 @@ public class DataSorter implements Comparator {
         this.ppTrial = ppTrial;
     }
 
-    public int getSortMetric() {
+    public Metric getSortMetric() {
         if (getSortByVisible()) {
-            return selectedMetricID;
+            return selectedMetric;
         } else {
             return sortMetric;
         }
     }
 
-    public void setSortMetric(int sortMetric) {
+    public void setSortMetric(Metric sortMetric) {
         this.sortMetric = sortMetric;
     }
 
@@ -606,20 +605,21 @@ public class DataSorter implements Comparator {
         this.sortByVisible = sortByVisible;
     }
 
-    public void setSelectedMetricID(int metric) {
-        this.selectedMetricID = metric;
+    public void setSelectedMetric(Metric metric) {
+        this.selectedMetric = metric;
     }
 
     public double getValue(FunctionProfile fp) {
-        return valueType.getValue(fp, selectedMetricID, getSelectedSnapshot());
+        return getValueType().getValue(fp, selectedMetric, getSelectedSnapshot());
     }
 
     public double getValue(FunctionProfile fp, int snapshot) {
-        return valueType.getValue(fp, selectedMetricID, snapshot);
+        return getValueType().getValue(fp, selectedMetric, snapshot);
     }
 
-    public int getSelectedMetricID() {
-        return selectedMetricID;
+    
+    public Metric getSelectedMetric() {
+        return selectedMetric;
     }
 
     public void setDescendingOrder(boolean descendingOrder) {
@@ -635,7 +635,7 @@ public class DataSorter implements Comparator {
     }
 
     public SortType getSortType() {
-        return this.sortType;
+        return sortType;
     }
 
     public void setValueType(ValueType valueType) {
@@ -643,7 +643,7 @@ public class DataSorter implements Comparator {
     }
 
     public ValueType getValueType() {
-        return this.valueType;
+        return valueType;
     }
 
     public void setPhase(Function phase) {
@@ -675,6 +675,22 @@ public class DataSorter implements Comparator {
     }
 
     public int getSelectedSnapshot() {
-        return ppTrial.getSelectedSnapshot();
+        if (selectedSnapshotOverride) {
+            return selectedSnapshot;
+        } else {
+            return ppTrial.getSelectedSnapshot();
+        }
+    }
+
+    public boolean getSelectedSnapshotOverride() {
+        return selectedSnapshotOverride;
+    }
+
+    public void setSelectedSnapshotOverride(boolean selectedSnapshotOverride) {
+        this.selectedSnapshotOverride = selectedSnapshotOverride;
+    }
+
+    public void setSelectedSnapshot(int selectedSnapshot) {
+        this.selectedSnapshot = selectedSnapshot;
     }
 }
\ No newline at end of file
