@@ -8,13 +8,13 @@ import java.util.List;
 import javax.swing.*;
 import javax.swing.event.*;
 
+import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.paraprof.barchart.*;
 import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.interfaces.*;
-import edu.uoregon.tau.perfdmf.Function;
+import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.Thread;
-import edu.uoregon.tau.perfdmf.UtilFncs;
 
 /**
  * The FunctionBarChartWindow displays performance data in many ways.  
@@ -24,9 +24,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
  * 1) Need to replace constructors with a factory, get rid of "changeToPhase..."
  * 2) Need to track all ppTrials (Observers) for comparisonChart 
  * 
- * <P>CVS $Id: FunctionBarChartWindow.java,v 1.17 2007/05/11 21:44:41 amorris Exp $</P>
+ * <P>CVS $Id: FunctionBarChartWindow.java,v 1.25 2010/01/14 01:31:39 amorris Exp $</P>
  * @author  Robert Bell, Alan Morris
- * @version $Revision: 1.17 $
+ * @version $Revision: 1.25 $
  * @see     FunctionBarChartModel
  * @see     ThreadBarChartModel
  */
@@ -71,25 +71,51 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
     private JTextArea jTextArea;
     private Component headerView;
 
-
     private FunctionBarChartWindow() {
     // disable default constructor
     }
 
     // Initializes Chart as a single function across threads
     public FunctionBarChartWindow(ParaProfTrial ppTrial, Function function, Component parent) {
+        this(ppTrial, function, parent, false, null);
+    }
+
+    public FunctionBarChartWindow(ParaProfTrial ppTrial, Function function, Component parent, boolean isPhase, Thread thread) {
         this.ppTrial = ppTrial;
         this.function = function;
         dataSorter = new DataSorter(ppTrial);
+        this.phaseDisplay = isPhase;
+
+        if (phaseDisplay) {
+            this.ppThread = new PPThread(thread, ppTrial);
+            // in case we were opened on "main => foo", switch to foo
+            this.function = ppTrial.getDataSource().getFunction(UtilFncs.getLeftSide(function.getName()));
+
+            // we're no longer in a phase
+            this.phase = null;
+            //sortByNCTCheckbox.setSelected(false);
+            //optionsMenu.remove(sortByNCTCheckbox);
+            //descendingOrderCheckBox.setSelected(true);
+
+            //sortLocalData();
+            setTitle("TAU: ParaProf: " + ParaProfUtils.getThreadLabel(thread) + " - Function Data: "
+                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+        } else {
+            setTitle("TAU: ParaProf: Function Data Window: "
+                    + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+        }
+        ParaProfUtils.setFrameIcon(this);
 
         model = new FunctionBarChartModel(this, dataSorter, function);
         panel = new BarChartPanel(model, null);
         initialize(parent);
 
-        setTitle("TAU: ParaProf: Function Data Window: "
-                + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
-        ParaProfUtils.setFrameIcon(this);
+    }
 
+    public static FunctionBarChartWindow CreateFunctionsOverPhaseDisplay(ParaProfTrial ppTrial, Function function, Thread thread,
+            Component parent) {
+        FunctionBarChartWindow fbcw = new FunctionBarChartWindow(ppTrial, function, parent, true, thread);
+        return fbcw;
     }
 
     // Initializes Chart as a single thread across functions
@@ -167,8 +193,8 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
         });
 
         // Set up the data sorter
-        dataSorter.setSelectedMetricID(ppTrial.getDefaultMetricID());
-        dataSorter.setSortMetric(ppTrial.getDefaultMetricID());
+        dataSorter.setSelectedMetric(ppTrial.getDefaultMetric());
+        dataSorter.setSortMetric(ppTrial.getDefaultMetric());
         dataSorter.setSortByVisible(true);
         dataSorter.setSortType(SortType.VALUE);
         //dataSorter.setValueType(ValueType.EXCLUSIVE_PERCENT);
@@ -216,36 +242,14 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
         ParaProf.incrementNumWindows();
     }
 
-    public void changeToPhaseDisplay(Thread thread) {
-
-        this.ppThread = new PPThread(thread, ppTrial);
-        phaseDisplay = true;
-
-        this.setTitle("TAU: ParaProf: " + ParaProfUtils.getThreadIdentifier(thread) + " - Function Data: "
-                + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
-        ParaProfUtils.setFrameIcon(this);
-
-        // in case we were opened on "main => foo", switch to foo
-        this.function = ppTrial.getDataSource().getFunction(UtilFncs.getLeftSide(function.getName()));
-
-        // we're no longer in a phase
-        this.phase = null;
-        setHeader();
-        //sortByNCTCheckbox.setSelected(false);
-        //optionsMenu.remove(sortByNCTCheckbox);
-        descendingOrderCheckBox.setSelected(true);
-
-        sortLocalData();
-    }
-
-    public void setMetricID(int id) {
-        dataSorter.setSelectedMetricID(id);
+    public void setMetric(Metric metric) {
+        dataSorter.setSelectedMetric(metric);
         sortLocalData();
         panel.repaint();
     }
 
-    public int getMetricID() {
-        return dataSorter.getSelectedMetricID();
+    public Metric getMetric() {
+        return dataSorter.getSelectedMetric();
     }
 
     public void setValueType(ValueType type) {
@@ -290,17 +294,27 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
         descendingOrderCheckBox.addActionListener(sortData);
         optionsMenu.add(descendingOrderCheckBox);
 
+        if (showValuesAsPercent == null) {
         showValuesAsPercent = new JCheckBoxMenuItem("Show Values as Percent", defaultPercentValue);
+        }
         showValuesAsPercent.addActionListener(sortData);
         optionsMenu.add(showValuesAsPercent);
 
         unitsSubMenu = ParaProfUtils.createUnitsMenu(this, units, true);
         optionsMenu.add(unitsSubMenu);
 
-        optionsMenu.add(ParaProfUtils.createMetricSelectionMenu(ppTrial, "Select Metric...", false, function != null, dataSorter,
-                this, true));
-        optionsMenu.add(ParaProfUtils.createMetricSelectionMenu(ppTrial, "Sort by...", true, function != null, dataSorter, this,
-                true));
+        boolean nct = false;
+        // if this is a function display (one function across all threads), we want 'sort by nct'
+        if (function != null) {
+            nct = true;
+        }
+        // if this is a phase display (all functions across one phase), we want 'sort by name'
+        if (phaseDisplay) {
+            nct = false;
+        }
+
+        optionsMenu.add(ParaProfUtils.createMetricSelectionMenu(ppTrial, "Select Metric...", false, nct, dataSorter, this, true));
+        optionsMenu.add(ParaProfUtils.createMetricSelectionMenu(ppTrial, "Sort by...", true, nct, dataSorter, this, true));
 
         optionsMenu.addMenuListener(this);
 
@@ -364,8 +378,7 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
                 if (showValuesAsPercent.isSelected()) {
                     unitsSubMenu.setEnabled(false);
                 } else {
-
-                    String metricName = ppTrial.getMetricName(dataSorter.getSelectedMetricID());
+                    String metricName = dataSorter.getSelectedMetric().getName();
                     metricName = metricName.toUpperCase();
                     if (dataSorter.isTimeMetric()) {
                         unitsSubMenu.setEnabled(true);
@@ -402,7 +415,7 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
         } else if (tmpString.equals("colorEvent")) {
             panel.repaint();
         } else if (tmpString.equals("dataEvent")) {
-            dataSorter.setSelectedMetricID(ppTrial.getDefaultMetricID());
+            dataSorter.setSelectedMetric(ppTrial.getDefaultMetric());
             setupMenus();
             validate(); // must call validate or the new JMenuBar won't work
             sortLocalData();
@@ -451,21 +464,6 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
     public void sortLocalData() {
         dataSorter.setDescendingOrder(descendingOrderCheckBox.isSelected());
 
-        //        if (function != null) { // function
-        //            if (sortByNCTCheckbox.isSelected()) {
-        //                dataSorter.setSortType(SortType.NCT);
-        //            } else {
-        //                dataSorter.setSortType(SortType.VALUE);
-        //            }
-        //
-        //        } else { // thread
-        //            if (sortByNameCheckBox.isSelected()) {
-        //                dataSorter.setSortType(SortType.NAME);
-        //            } else {
-        //                dataSorter.setSortType(SortType.VALUE);
-        //            }
-        //        }
-
         if (showValuesAsPercent.isSelected()) {
             if (dataSorter.getValueType() == ValueType.EXCLUSIVE) {
                 dataSorter.setValueType(ValueType.EXCLUSIVE_PERCENT);
@@ -485,15 +483,16 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
     }
 
     public int units() {
-        if (showValuesAsPercent.isEnabled() && showValuesAsPercent.isSelected())
+        if (showValuesAsPercent.isEnabled() && showValuesAsPercent.isSelected()) {
             return 0;
-
-        if (!dataSorter.isTimeMetric()) // we don't do units for non-time metrics
+        }
+        if (!dataSorter.isTimeMetric()) {
+            // we don't do units for non-time metrics
             return 0;
-
-        if (dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR)
+        }
+        if (dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR) {
             return 0;
-
+        }
         return units;
     }
 
@@ -514,7 +513,6 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
     // header string being used for the panel without resetting the actual header. 
     // Printing and image generation use this functionality.
 
-
     public void setHeader() {
         if (showMetaData.isSelected()) {
 
@@ -539,11 +537,11 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
                 gbc.anchor = GridBagConstraints.WEST;
                 gbc.weightx = 0;
                 gbc.weighty = 1;
-                ParaProfUtils.addCompItem(holder, jTextArea, gbc, 0, 0, 1, 1);
+                Utility.addCompItem(holder, jTextArea, gbc, 0, 0, 1, 1);
                 gbc.weightx = 1;
                 gbc.weighty = 1;
                 gbc.anchor = GridBagConstraints.EAST;
-                ParaProfUtils.addCompItem(holder, legendPanel, gbc, 1, 0, 1, 1);
+                Utility.addCompItem(holder, legendPanel, gbc, 1, 0, 1, 1);
 
                 panel.setColumnHeaderView(holder);
                 headerView = holder;
@@ -569,8 +567,7 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
                 header += "Name: " + function.getName();
             }
 
-            header += "\nMetric Name: " + ppTrial.getMetricName(dataSorter.getSelectedMetricID()) + "\nValue: "
-                    + dataSorter.getValueType();
+            header += "\nMetric Name: " + dataSorter.getSelectedMetric().getName() + "\nValue: " + dataSorter.getValueType();
 
             if ((dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR)
                     || showValuesAsPercent.isSelected()) {
@@ -583,11 +580,10 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
             String starter;
 
             if (phase != null) {
-                starter = "Phase: " + phase + "\nMetric: " + (ppTrial.getMetricName(dataSorter.getSelectedMetricID())) + "\n"
-                        + "Value: " + dataSorter.getValueType();
-            } else {
-                starter = "Metric: " + (ppTrial.getMetricName(dataSorter.getSelectedMetricID())) + "\n" + "Value: "
+                starter = "Phase: " + phase + "\nMetric: " + dataSorter.getSelectedMetric().getName() + "\n" + "Value: "
                         + dataSorter.getValueType();
+            } else {
+                starter = "Metric: " + dataSorter.getSelectedMetric().getName() + "\n" + "Value: " + dataSorter.getValueType();
             }
 
             if ((dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR)
@@ -636,7 +632,7 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
             addCompItem(panel, gbc, 0, 0, 1, 1);
         }
 
-        //Now call validate so that these componant changes are displayed.
+        //Now call validate so that these component changes are displayed.
         validate();
     }
 
@@ -753,4 +749,12 @@ public class FunctionBarChartWindow extends JFrame implements KeyListener, Searc
         panel.repaint();
     }
 
+    public BarChartPanel getPanel() {
+        return panel;
+    }
+
+    public JFrame getFrame() {
+        return this;
+    }
+
 }
\ No newline at end of file
