@@ -26,6 +26,7 @@ import org.jfree.data.statistics.HistogramDataset;
 import org.jfree.data.xy.XYDataset;
 
 import edu.uoregon.tau.common.ImageExport;
+import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.interfaces.ParaProfWindow;
 import edu.uoregon.tau.paraprof.interfaces.UnitListener;
@@ -36,9 +37,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
  * HistogramWindow
  * This is the histogram window
  *  
- * <P>CVS $Id: HistogramWindow.java,v 1.5 2006/11/08 23:17:58 amorris Exp $</P>
+ * <P>CVS $Id: HistogramWindow.java,v 1.10 2009/09/10 00:13:46 amorris Exp $</P>
  * @author  Robert Bell, Alan Morris
- * @version $Revision: 1.5 $
+ * @version $Revision: 1.10 $
  * @see     HistogramWindowPanel
  */
 public class HistogramWindow extends JFrame implements ActionListener, MenuListener, Observer, ChangeListener, ParaProfWindow,
@@ -271,11 +272,9 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
         }
     }
 
-    public void menuDeselected(MenuEvent evt) {
-    }
+    public void menuDeselected(MenuEvent evt) {}
 
-    public void menuCanceled(MenuEvent evt) {
-    }
+    public void menuCanceled(MenuEvent evt) {}
 
     // listener for the numBinsSlider
     public void stateChanged(ChangeEvent event) {
@@ -294,7 +293,7 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
         } else if (tmpString.equals("colorEvent")) {
             redraw();
         } else if (tmpString.equals("dataEvent")) {
-            dataSorter.setSelectedMetricID(ppTrial.getDefaultMetricID());
+            dataSorter.setSelectedMetric(ppTrial.getDefaultMetric());
             sortLocalData();
             redraw();
         } else if (tmpString.equals("subWindowCloseEvent")) {
@@ -304,8 +303,9 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
 
     public void help(boolean display) {
         ParaProf.getHelpWindow().clearText();
-        if (display)
-            ParaProf.getHelpWindow().show();
+        if (display) {
+            ParaProf.getHelpWindow().setVisible(true);
+        }
         ParaProf.getHelpWindow().writeText("This is the histogram window");
         ParaProf.getHelpWindow().writeText("");
         ParaProf.getHelpWindow().writeText("This window shows you a histogram of all of the values for this function.");
@@ -329,7 +329,7 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
                 starter = "Name: " + function.getName();
             }
 
-            starter = starter + "\nMetric: " + ppTrial.getMetricName(dataSorter.getSelectedMetricID()) + "\nValue: "
+            starter = starter + "\nMetric: " + dataSorter.getSelectedMetric().getName() + "\nValue: "
                     + dataSorter.getValueType();
 
             if ((dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR)) {
@@ -340,10 +340,10 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
             }
         } else {
             if (dataSorter.getValueType() == ValueType.NUMCALLS || dataSorter.getValueType() == ValueType.NUMSUBR) {
-                return "Metric Name: " + (ppTrial.getMetricName(ppTrial.getDefaultMetricID())) + "\n" + "Name: "
+                return "Metric Name: " + ppTrial.getDefaultMetric().getName() + "\n" + "Name: "
                         + ParaProfUtils.getDisplayName(function) + "\n" + "Value Type: " + dataSorter.getValueType() + "\n";
             } else {
-                return "Metric Name: " + (ppTrial.getMetricName(ppTrial.getDefaultMetricID())) + "\n" + "Name: "
+                return "Metric Name: " +ppTrial.getDefaultMetric().getName() + "\n" + "Name: "
                         + ParaProfUtils.getDisplayName(function) + "\n" + "Value Type: " + dataSorter.getValueType() + "\n"
                         + "Units: " + UtilFncs.getUnitsString(units, ppTrial.isTimeMetric(), ppTrial.isDerivedMetric()) + "\n";
             }
@@ -422,26 +422,27 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
         }
 
         dataset.addSeries(function.getName(), values, numBins, minValue, maxValue);
-        
+
         String xAxis = dataSorter.getValueType().toString();
 
         if ((dataSorter.getValueType() != ValueType.NUMCALLS && dataSorter.getValueType() != ValueType.NUMSUBR)) {
-            xAxis = xAxis + " " + ppTrial.getMetricName(dataSorter.getSelectedMetricID()) + " ("
+            xAxis = xAxis + " " + dataSorter.getSelectedMetric().getName() + " ("
                     + UtilFncs.getUnitsString(units, dataSorter.isTimeMetric(), dataSorter.isDerivedMetric()) + ")";
         }
 
         JFreeChart chart = ChartFactory.createHistogram(function.getName(), xAxis, "Threads", dataset, PlotOrientation.VERTICAL,
-                false,  // legend
-                true,   // tooltips
+                false, // legend
+                true, // tooltips
                 false); // urls
 
         chart.getXYPlot().getDomainAxis().setUpperBound(maxValue);
         chart.getXYPlot().getDomainAxis().setLowerBound(minValue);
 
-        NumberAxis numberAxis = (NumberAxis) chart.getXYPlot().getDomainAxis();
-        numberAxis.setNumberFormatOverride(ParaProfUtils.createNumberFormatter(units()));
-        numberAxis.setTickLabelsVisible(true);
+        Utility.applyDefaultChartTheme(chart);
 
+        NumberAxis numberAxis = (NumberAxis) chart.getXYPlot().getDomainAxis();
+        numberAxis.setNumberFormatOverride(ParaProfUtils.createNumberFormatter(units(), dataSorter.getSelectedMetric().isTimeDenominator()));
+        numberAxis.setTickLabelsVisible(true);
 
         numberAxis.setTickUnit(new NumberTickUnit((maxValue - minValue) / 10));
 
@@ -451,21 +452,20 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
         XYItemRenderer renderer = chart.getXYPlot().getRenderer();
         renderer.setToolTipGenerator(new XYToolTipGenerator() {
             public String generateToolTip(XYDataset dataset, int arg1, int arg2) {
-                String minString = UtilFncs.getOutputString(units(),dataset.getXValue(arg1, arg2) - (binWidth / 2), 5);
-                String maxString = UtilFncs.getOutputString(units(),dataset.getXValue(arg1, arg2) + (binWidth / 2), 5);
+                String minString = UtilFncs.getOutputString(units(), dataset.getXValue(arg1, arg2) - (binWidth / 2), 5, ppTrial.getDefaultMetric().isTimeDenominator());
+                String maxString = UtilFncs.getOutputString(units(), dataset.getXValue(arg1, arg2) + (binWidth / 2), 5, ppTrial.getDefaultMetric().isTimeDenominator());
 
-                return "<html>Number of threads: " + (int)dataset.getYValue(arg1, arg2) + "<br>Range minimum: " + minString + "<br>Range maximum: "
-                + maxString + "</html>";
+                return "<html>Number of threads: " + (int) dataset.getYValue(arg1, arg2) + "<br>Range minimum: " + minString
+                        + "<br>Range maximum: " + maxString + "</html>";
             }
         });
 
-
-        if (numBins < 25) { 
+        if (numBins < 25) {
             // it looks nicer with a margin, but only when we're at a low number of bars
-            ((XYBarRenderer)chart.getXYPlot().getRenderer()).setMargin(0.10);
+            ((XYBarRenderer) chart.getXYPlot().getRenderer()).setMargin(0.10);
         }
 
-        ((XYBarRenderer)chart.getXYPlot().getRenderer()).setOutlinePaint(Color.black); 
+        ((XYBarRenderer) chart.getXYPlot().getRenderer()).setOutlinePaint(Color.black);
 
         return chart;
     }
@@ -496,5 +496,8 @@ public class HistogramWindow extends JFrame implements ActionListener, MenuListe
         return chartPanel.getSize();
     }
 
+    public JFrame getFrame() {
+        return this;
+    }
 
 }
\ No newline at end of file
