@@ -27,8 +27,7 @@ import edu.uoregon.tau.common.ImageOptionsPanel;
 public class ParaProfImageOutput {
 
     // do not allow instantiation
-    private ParaProfImageOutput() {
-    }
+    private ParaProfImageOutput() {}
 
     public static void saveImage(ImageExport ref) throws IOException {
 
@@ -39,8 +38,9 @@ public class ParaProfImageOutput {
         fileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
         //Get the current file filters.
         javax.swing.filechooser.FileFilter fileFilters[] = fileChooser.getChoosableFileFilters();
-        for (int i = 0; i < fileFilters.length; i++)
+        for (int i = 0; i < fileFilters.length; i++) {
             fileChooser.removeChoosableFileFilter(fileFilters[i]);
+        }
         fileChooser.addChoosableFileFilter(new ImageFormatFileFilter(ImageFormatFileFilter.JPG));
         fileChooser.addChoosableFileFilter(new ImageFormatFileFilter(ImageFormatFileFilter.PNG));
         fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
@@ -52,9 +52,8 @@ public class ParaProfImageOutput {
         if (resultValue != JFileChooser.APPROVE_OPTION) {
             return;
         }
-
-        //Get both the file and FileFilter.
         File file = fileChooser.getSelectedFile();
+
         String path = file.getCanonicalPath();
 
         ImageFormatFileFilter paraProfImageFormatFileFilter = null;
@@ -63,10 +62,10 @@ public class ParaProfImageOutput {
             paraProfImageFormatFileFilter = (ImageFormatFileFilter) fileFilter;
         } else {
             throw new ParaProfException("Unknown format : " + fileFilter);
-            //???
         }
         String extension = ImageFormatFileFilter.getExtension(file);
-        if (extension == null || ((extension.toUpperCase().compareTo("JPG")!=0) && (extension.toUpperCase().compareTo("PNG")!=0)) ) {
+        if (extension == null
+                || ((extension.toUpperCase().compareTo("JPG") != 0) && (extension.toUpperCase().compareTo("PNG") != 0))) {
             extension = paraProfImageFormatFileFilter.getExtension();
             path = path + "." + extension;
             file = new File(path);
@@ -78,17 +77,36 @@ public class ParaProfImageOutput {
             if (response == JOptionPane.CANCEL_OPTION)
                 return;
         }
+        saveImage(ref, file, paraProfImageOptionsPanel.isFullScreen(), paraProfImageOptionsPanel.isPrependHeader(),
+                paraProfImageOptionsPanel.getImageQuality());
+    }
+
+    public static void saveImage(ImageExport ref, File file) throws IOException {
+        saveImage(ref, file, false, true, 100);
+    }
+
+    public static void saveImage(ImageExport ref, String file) throws IOException {
+        saveImage(ref, new File(file), false, true, 100);
+    }
+
+    public static void saveImage(ImageExport ref, File file, boolean fullScreen, boolean prependHeader, float imageQuality)
+            throws IOException {
+        String extension = ImageFormatFileFilter.getExtension(file);
 
         // I'm doing this twice right now because the getImageSize won't be correct until 
         // renderIt has been called with the appropriate settings.  Stupid, I know.
-        Dimension d = ref.getImageSize(paraProfImageOptionsPanel.isFullScreen(), paraProfImageOptionsPanel.isPrependHeader());
+        Dimension d = ref.getImageSize(fullScreen, prependHeader);
+        d.height = Math.max(d.height, 1);
+        d.width = Math.max(d.width, 1);
         BufferedImage bi = new BufferedImage((int) d.getWidth(), (int) d.getHeight(), BufferedImage.TYPE_INT_RGB);
         Graphics2D g2D = bi.createGraphics();
 
         //Draw to this graphics object.
-        ref.export(g2D, false, paraProfImageOptionsPanel.isFullScreen(), paraProfImageOptionsPanel.isPrependHeader());
+        ref.export(g2D, false, fullScreen, prependHeader);
 
-        d = ref.getImageSize(paraProfImageOptionsPanel.isFullScreen(), paraProfImageOptionsPanel.isPrependHeader());
+        d = ref.getImageSize(fullScreen, prependHeader);
+        d.height = Math.max(d.height, 1);
+        d.width = Math.max(d.width, 1);
         bi = new BufferedImage((int) d.getWidth(), (int) d.getHeight(), BufferedImage.TYPE_INT_RGB);
         g2D = bi.createGraphics();
 
@@ -100,7 +118,7 @@ public class ParaProfImageOutput {
         g2D.setColor(Color.black);
 
         //Draw to this graphics object.
-        ref.export(g2D, false, paraProfImageOptionsPanel.isFullScreen(), paraProfImageOptionsPanel.isPrependHeader());
+        ref.export(g2D, false, fullScreen, prependHeader);
 
         //Now write the image to file.
         ImageWriter writer = null;
@@ -109,7 +127,8 @@ public class ParaProfImageOutput {
             writer = (ImageWriter) iter.next();
         }
         if (writer == null) {
-            JOptionPane.showMessageDialog((Component)ref, "Couldn't find Image Writer for extension '." + extension.toUpperCase() + "'");
+            JOptionPane.showMessageDialog((Component) ref, "Couldn't find Image Writer for extension '."
+                    + extension.toUpperCase() + "'");
             return;
         }
         ImageOutputStream imageOut = ImageIO.createImageOutputStream(file);
@@ -120,14 +139,14 @@ public class ParaProfImageOutput {
         if (extension.toUpperCase().compareTo("JPG") == 0) {
             ImageWriteParam iwp = writer.getDefaultWriteParam();
             iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
-            iwp.setCompressionQuality(paraProfImageOptionsPanel.getImageQuality());
+            iwp.setCompressionQuality(imageQuality);
             writer.write(null, iioImage, iwp);
         } else {
             writer.write(iioImage);
         }
     }
 
-    public static void save3dImage(final ThreeDeeWindow ref) throws IOException {
+    public static void save3dImage(final ThreeDeeImageProvider ref) throws IOException {
 
         //Ask the user for a filename and location.
         JFileChooser fileChooser = new JFileChooser();
@@ -136,45 +155,66 @@ public class ParaProfImageOutput {
         fileChooser.setCurrentDirectory(new File(System.getProperty("user.dir")));
         //Get the current file filters.
         javax.swing.filechooser.FileFilter fileFilters[] = fileChooser.getChoosableFileFilters();
-        for (int i = 0; i < fileFilters.length; i++)
+        for (int i = 0; i < fileFilters.length; i++) {
             fileChooser.removeChoosableFileFilter(fileFilters[i]);
-        fileChooser.addChoosableFileFilter(new ImageFormatFileFilter(ImageFormatFileFilter.PNG));
+        }
         fileChooser.addChoosableFileFilter(new ImageFormatFileFilter(ImageFormatFileFilter.JPG));
+        fileChooser.addChoosableFileFilter(new ImageFormatFileFilter(ImageFormatFileFilter.PNG));
         fileChooser.setFileSelectionMode(JFileChooser.FILES_ONLY);
 
-        final ImageOptionsPanel paraProfImageOptionsPanel = new ImageOptionsPanel((Component) ref, false, false);
-        fileChooser.setAccessory(paraProfImageOptionsPanel); 
+        final ImageOptionsPanel paraProfImageOptionsPanel = new ImageOptionsPanel(ref.getComponent(), false, false);
+        fileChooser.setAccessory(paraProfImageOptionsPanel);
         fileChooser.addPropertyChangeListener(paraProfImageOptionsPanel);
-        int resultValue = fileChooser.showSaveDialog((Component) ref);
+        int resultValue = fileChooser.showSaveDialog(ref.getComponent());
         if (resultValue != JFileChooser.APPROVE_OPTION) {
             return;
         }
 
-        //Get both the file and FileFilter.
-        File f = fileChooser.getSelectedFile();
-        javax.swing.filechooser.FileFilter fileFilter = fileChooser.getFileFilter();
-        String path = f.getCanonicalPath();
-        //Append extension if required.
+        File file = fileChooser.getSelectedFile();
+        String path = file.getCanonicalPath();
 
+        ImageFormatFileFilter paraProfImageFormatFileFilter = null;
+        javax.swing.filechooser.FileFilter fileFilter = fileChooser.getFileFilter();
+        if (fileFilter instanceof ImageFormatFileFilter) {
+            paraProfImageFormatFileFilter = (ImageFormatFileFilter) fileFilter;
+        } else {
+            throw new ParaProfException("Unknown format : " + fileFilter);
+        }
+        
+        String extension = ImageFormatFileFilter.getExtension(file);
+        if (extension == null
+                || ((extension.toUpperCase().compareTo("JPG") != 0) && (extension.toUpperCase().compareTo("PNG") != 0))) {
+            extension = paraProfImageFormatFileFilter.getExtension();
+            path = path + "." + extension;
+            file = new File(path);
+        }
+        
+        if (file.exists()) {
+            int response = JOptionPane.showConfirmDialog((Component) ref, file + " already exists\nOverwrite existing file?",
+                    "Confirm Overwrite", JOptionPane.OK_CANCEL_OPTION, JOptionPane.QUESTION_MESSAGE);
+            if (response == JOptionPane.CANCEL_OPTION)
+                return;
+        }
+        
         //Only create if we recognize the format.
         ImageFormatFileFilter imageFormatFileFilter = null;
         if (fileFilter instanceof ImageFormatFileFilter) {
             imageFormatFileFilter = (ImageFormatFileFilter) fileFilter;
-            String extension = ImageFormatFileFilter.getExtension(f);
+
             //Could probably collapse this if/else based on the order of evaluation of arguments (ie, to make sure
             //the extension is not null before trying to call equals on it).  However, it is easier to understand
             //what is going on this way.
             if (extension == null) {
                 path = path + "." + imageFormatFileFilter.getExtension();
-                f = new File(path);
+                file = new File(path);
             } else if (!(extension.equals("png") || extension.equals("jpg"))) {
                 path = path + "." + imageFormatFileFilter.getExtension();
-                f = new File(path);
+                file = new File(path);
             }
 
             final String extensionString = imageFormatFileFilter.getExtension().toUpperCase();
 
-            final File filename = f;
+            final File filename = file;
 
             fileChooser.setVisible(false);
 
@@ -191,7 +231,7 @@ public class ParaProfImageOutput {
             IIOImage iioImage = new IIOImage(bi, null, null);
 
             //Try setting quality.
-            if (paraProfImageOptionsPanel.imageQualityEnabled()) {
+            if (extension.toUpperCase().compareTo("JPG") == 0) {
                 ImageWriteParam iwp = writer.getDefaultWriteParam();
                 iwp.setCompressionMode(ImageWriteParam.MODE_EXPLICIT);
                 iwp.setCompressionQuality(paraProfImageOptionsPanel.getImageQuality());
