@@ -8,18 +8,17 @@
 
 package edu.uoregon.tau.paraprof;
 
-import edu.uoregon.tau.perfdmf.*;
+import javax.swing.tree.DefaultMutableTreeNode;
+import javax.swing.tree.TreePath;
 
-import javax.swing.tree.*;
+import edu.uoregon.tau.perfdmf.Metric;
 
 public class ParaProfMetric extends Metric implements ParaProfTreeNodeUserObject {
     private ParaProfTrial ppTrial = null;
     private DefaultMutableTreeNode defaultMutableTreeNode = null;
     private TreePath treePath = null;
-    private boolean derivedMetric = false;
 
-    public ParaProfMetric() {
-    }
+    public ParaProfMetric() {}
 
     public void setPpTrial(ParaProfTrial ppTrial) {
         this.ppTrial = ppTrial;
@@ -45,14 +44,6 @@ public class ParaProfMetric extends Metric implements ParaProfTreeNodeUserObject
         return treePath;
     }
 
-    public void setDerivedMetric(boolean derivedMetric) {
-        this.derivedMetric = derivedMetric;
-    }
-
-    public boolean getDerivedMetric() {
-        return derivedMetric;
-    }
-
     public int getApplicationID() {
         return ppTrial.getApplicationID();
     }
@@ -81,4 +72,8 @@ public class ParaProfMetric extends Metric implements ParaProfTreeNodeUserObject
         this.setDMTN(null);
     }
 
+    public boolean dbMetric() {
+        return ppTrial.dBTrial();
+    }
+
 }
\ No newline at end of file
