@@ -48,7 +48,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
 
     private String path = null;
     private String pathReverse = null;
-    private int defaultMetricID = 0;
+    private Metric defaultMetric = null;
     private int selectedSnapshot = -1; // -1 = final snapshot
 
     private Group selectedGroup;
@@ -78,6 +78,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
 
     public ParaProfTrial(Trial trial) {
         this.trial = new Trial(trial);
+        assignDefaultMetric();
     }
 
     public Iterator getFunctions() {
@@ -173,10 +174,11 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
     }
 
     public String getIDString() {
-        if (experiment != null)
+        if (experiment != null) {
             return (experiment.getIDString()) + ":" + (trial.getID());
-        else
+        } else {
             return ":" + (trial.getID());
+        }
     }
 
     public ColorChooser getColorChooser() {
@@ -233,13 +235,10 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
 
     public void showMainWindow() {
         if (fullDataWindow == null) {
-            fullDataWindow = new GlobalDataWindow(this, trial.getDataSource().getTopLevelPhase());
-            fullDataWindow.setVisible(true);
             showSnapshotController();
-        } else {
-            ParaProf.incrementNumWindows();
-            fullDataWindow.setVisible(true);
         }
+        fullDataWindow = new GlobalDataWindow(this, trial.getDataSource().getTopLevelPhase());
+        fullDataWindow.setVisible(true);
     }
 
     public void showSnapshotController() {
@@ -249,44 +248,35 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         }
     }
 
-    public void setDefaultMetricID(int selectedMetricID) {
-        this.defaultMetricID = selectedMetricID;
+    public void setDefaultMetric(Metric metric) {
+        this.defaultMetric = metric;
     }
 
-    public int getDefaultMetricID() {
-        return defaultMetricID;
+    public Metric getDefaultMetric() {
+        return defaultMetric;
     }
 
     public boolean isTimeMetric() {
-        String metricName = this.getMetricName(this.getDefaultMetricID());
-        metricName = metricName.toUpperCase();
-        if (metricName.indexOf("TIME") == -1) {
-            return false;
-        } else {
-            return true;
-        }
+        return defaultMetric.isTimeMetric();
     }
 
-    //    public boolean isTimeMetric(int metricID) {
-    //        String metricName = this.getMetricName(metricID);
-    //        metricName = metricName.toUpperCase();
-    //        if (metricName.indexOf("TIME") == -1)
-    //            return false;
-    //        else
-    //            return true;
-    //    }
-
     public boolean isDerivedMetric() {
 
         // We can't do this, HPMToolkit stuff has /'s and -'s all over the place
         //String metricName = this.getMetricName(this.getSelectedMetricID());
         //if (metricName.indexOf("*") != -1 || metricName.indexOf("/") != -1)
         //    return true;
-        return this.getMetric(this.getDefaultMetricID()).getDerivedMetric();
+        return defaultMetric.getDerivedMetric();
     }
 
     //Override this function.
     public List getMetrics() {
+        if (trial == null) {
+            return null;
+        }
+        if (trial.getDataSource() == null) {
+            return null;
+        }
         return trial.getDataSource().getMetrics();
     }
 
@@ -307,6 +297,10 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         return (ParaProfMetric) trial.getDataSource().getMetric(metricID);
     }
 
+    public Metric getRegularMetric(int metricID) {
+        return trial.getDataSource().getMetric(metricID);
+    }
+
     public String getMetricName(int metricID) {
         return trial.getDataSource().getMetricName(metricID);
     }
@@ -474,6 +468,35 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         return selectedGroup;
     }
 
+    private void assignDefaultMetric() {
+        // set the default metric to be the first one
+        if (getMetrics() != null && getMetrics().size() > 0) {
+            setDefaultMetric((Metric) getMetrics().get(0));
+        }
+
+        if (getMetrics() == null) {
+            return;
+        }
+
+        // set the default metric to the first time based metric (if it exists)
+        for (Iterator it = getMetrics().iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            if (metric.isTimeMetric()) {
+                setDefaultMetric(metric);
+                break;
+            }
+        }
+
+        // set the default metric to the first metric named "Time" (if it exists), higher priority than above
+        for (Iterator it = getMetrics().iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            if (metric.getName().equalsIgnoreCase(("time"))) {
+                setDefaultMetric(metric);
+                break;
+            }
+        }
+    }
+
     public void finishLoad() {
 
         // assign the metadata from the datasource to the trial
@@ -489,30 +512,15 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
             ParaProfMetric ppMetric = new ParaProfMetric();
             ppMetric.setName(trial.getDataSource().getMetricName(i));
             ppMetric.setID(i);
+            ppMetric.setDbMetricID(trial.getDataSource().getMetric(i).getDbMetricID());
             ppMetric.setPpTrial(this);
             ppMetrics.add(ppMetric);
         }
 
-        // Now set the dataSource's metrics.
+        // mow set the dataSource's metrics.
         trial.getDataSource().setMetrics(ppMetrics);
 
-        // Set the default metric to the first time based metric (if it exists)
-        for (int i = 0; i < numberOfMetrics; i++) {
-            ParaProfMetric ppMetric = (ParaProfMetric) trial.getDataSource().getMetric(i);
-            if (ppMetric.isTimeMetric()) {
-                setDefaultMetricID(i);
-                break;
-            }
-        }
-
-        // Set the default metric to the first metric named "Time" (if it exists), higher priority than above
-        for (int i = 0; i < numberOfMetrics; i++) {
-            ParaProfMetric ppMetric = (ParaProfMetric) trial.getDataSource().getMetric(i);
-            if (ppMetric.getName().equalsIgnoreCase("Time")) {
-                setDefaultMetricID(i);
-                break;
-            }
-        }
+        assignDefaultMetric();
 
         // set the mask
         functionMask = new boolean[getDataSource().getNumFunctions()];
@@ -712,7 +720,7 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         this.selectedSnapshot = selectedSnapshot;
         updateRegisteredObjects("dataEvent");
     }
-    
+
     public Database getDatabase() {
         if (experiment == null) {
             return null;
@@ -720,5 +728,60 @@ public class ParaProfTrial extends Observable implements ParaProfTreeNodeUserObj
         return experiment.getDatabase();
     }
 
+    public List getThreads() {
+        return getDataSource().getAllThreads();
+    }
+
+    public List getThreadNames() {
+        List threadNames = new ArrayList();
+
+        for (Iterator it = getDataSource().getAllThreads().iterator(); it.hasNext();) {
+            Thread thread = (Thread) it.next();
+            if (getDataSource().getExecutionType() == DataSource.EXEC_TYPE_MPI) {
+                threadNames.add(Integer.toString(thread.getNodeID()));
+            } else if (getDataSource().getExecutionType() == DataSource.EXEC_TYPE_HYBRID) {
+                threadNames.add(thread.getNodeID() + ":" + thread.getThreadID());
+            } else {
+                threadNames.add(thread.getNodeID() + ":" + thread.getContextID() + ":" + thread.getThreadID());
+            }
+        }
+        return threadNames;
+    }
+
+    public void deleteMetric(Metric metric) {
+
+        trial.getDataSource().getMetrics().remove(metric);
+
+        if (metric == defaultMetric) {
+            assignDefaultMetric();
+        }
+    }
+
+    public Metric[] getMetricArray() {
+
+        int count = 0;
+
+        for (Iterator it = getMetrics().iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            if (metric == null) {
+                continue;
+            }
+            count++;
+        }
+
+        Metric[] items = new Metric[count];
+
+        int idx = 0;
+
+        for (Iterator it = getMetrics().iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            if (metric == null) {
+                continue;
+            }
+            items[idx++] = metric;
+        }
+
+        return items;
+    }
 
 }
\ No newline at end of file
