@@ -7,9 +7,7 @@ import java.io.File;
 import java.io.FileOutputStream;
 import java.io.PrintStream;
 import java.net.URL;
-import java.text.FieldPosition;
-import java.text.NumberFormat;
-import java.text.ParsePosition;
+import java.text.*;
 import java.util.*;
 import java.util.List;
 
@@ -17,30 +15,35 @@ import javax.swing.*;
 import javax.swing.event.MenuEvent;
 import javax.swing.event.MenuListener;
 
+import edu.uoregon.tau.common.ExternalTool;
 import edu.uoregon.tau.common.ImageExport;
 import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.common.VectorExport;
+import edu.uoregon.tau.paraprof.barchart.BarChart;
+import edu.uoregon.tau.paraprof.barchart.BarChartModel;
 import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.interfaces.*;
 import edu.uoregon.tau.paraprof.script.ParaProfFunctionScript;
 import edu.uoregon.tau.paraprof.script.ParaProfScript;
 import edu.uoregon.tau.paraprof.script.ParaProfTrialScript;
+import edu.uoregon.tau.paraprof.sourceview.SourceViewer;
 import edu.uoregon.tau.paraprof.treetable.ContextEventWindow;
 import edu.uoregon.tau.paraprof.treetable.TreeTableWindow;
 import edu.uoregon.tau.paraprof.util.MapViewer;
 import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.Thread;
+import edu.uoregon.tau.vis.HeatMapWindow;
 
 /**
  * Utility class for ParaProf
  * 
  * <P>
- * CVS $Id: ParaProfUtils.java,v 1.35 2007/06/01 00:06:48 amorris Exp $
+ * CVS $Id: ParaProfUtils.java,v 1.61 2009/12/18 04:21:48 amorris Exp $
  * </P>
  * 
  * @author Alan Morris
- * @version $Revision: 1.35 $
+ * @version $Revision: 1.61 $
  */
 public class ParaProfUtils {
 
@@ -133,29 +136,14 @@ public class ParaProfUtils {
         menu.add(item);
     }
 
-    public static void addCompItem(JFrame frame, Component c, GridBagConstraints gbc, int x, int y, int w, int h) {
-        gbc.gridx = x;
-        gbc.gridy = y;
-        gbc.gridwidth = w;
-        gbc.gridheight = h;
-        frame.getContentPane().add(c, gbc);
-    }
-
-    public static void addCompItem(Container container, Component c, GridBagConstraints gbc, int x, int y, int w, int h) {
-        gbc.gridx = x;
-        gbc.gridy = y;
-        gbc.gridwidth = w;
-        gbc.gridheight = h;
-        container.add(c, gbc);
-    }
-
     public static void print(Printable printable) {
         PrinterJob job = PrinterJob.getPrinterJob();
         PageFormat defaultFormat = job.defaultPage();
         PageFormat selectedFormat = job.pageDialog(defaultFormat);
+
         if (defaultFormat != selectedFormat) { // only proceed if the user did not select cancel
+
             job.setPrintable(printable, selectedFormat);
-            //if (job.getPrintService() != null) {
             if (job.printDialog()) { // only proceed if the user did not select cancel
                 try {
                     job.print();
@@ -163,9 +151,7 @@ public class ParaProfUtils {
                     ParaProfUtils.handleException(e);
                 }
             }
-            //}
         }
-
     }
 
     public static JMenu createHelpMenu(final JFrame owner, final ParaProfWindow ppWindow) {
@@ -229,24 +215,21 @@ public class ParaProfUtils {
 
                         if (panel instanceof ImageExport) {
                             ParaProfImageOutput.saveImage((ImageExport) panel);
-                        } else if (panel instanceof ThreeDeeWindow) {
-                            ThreeDeeWindow threeDeeWindow = (ThreeDeeWindow) panel;
-                            ParaProfImageOutput.save3dImage(threeDeeWindow);
+                        } else if (panel instanceof ThreeDeeImageProvider) {
+                            ParaProfImageOutput.save3dImage((ThreeDeeImageProvider) panel);
                         } else {
                             throw new ParaProfException("Don't know how to \"Save Image\" for " + panel.getClass());
                         }
 
                     } else if (arg.equals("Save as Vector Graphics")) {
-                        if (panel instanceof ImageExport) {
-
-                            if (JVMDependent.version.equals("1.4")) {
-                                VectorExport.promptForVectorExport((ImageExport) panel, "ParaProf");
-                            } else {
-                                JOptionPane.showMessageDialog((Component) panel,
-                                        "Can't save Vector Graphics with Java 1.3.  Reconfigure TAU with Java 1.4 or higher in the path.");
-                            }
-                        } else if (panel instanceof ThreeDeeWindow) {
-                            JOptionPane.showMessageDialog((JFrame) window, "Can't save 3D visualization as vector graphics");
+                        if (panel instanceof HeatMapWindow) {
+                            JOptionPane.showMessageDialog(window.getFrame(), "Can't save heat map as vector graphics");
+                        } else if (panel instanceof ThreeDeeImageProvider) {
+                            JOptionPane.showMessageDialog(window.getFrame(), "Can't save 3D visualization as vector graphics");
+                        } else if (panel instanceof SourceViewer) {
+                            JOptionPane.showMessageDialog(window.getFrame(), "Can't save Source Viewer as vector graphics");
+                        } else if (panel instanceof ImageExport) {
+                            VectorExport.promptForVectorExport((ImageExport) panel, "ParaProf");
                         } else {
                             throw new ParaProfException("Don't know how to \"Save as Vector Graphics\" for " + panel.getClass());
                         }
@@ -254,7 +237,17 @@ public class ParaProfUtils {
                         window.closeThisWindow();
                     } else if (arg.equals("Exit ParaProf!")) {
                         ParaProf.exitParaProf(0);
+                    } else if (arg.equals("Export Profile")) {
+                        GlobalDataWindow gdw = (GlobalDataWindow) window;
+                        ParaProfUtils.exportTrial(gdw.getPpTrial(), gdw);
+                    } else if (arg.equals("Convert to Phase Profile")) {
+                        GlobalDataWindow gdw = (GlobalDataWindow) window;
+                        ParaProfUtils.phaseConvertTrial(gdw.getPpTrial(), gdw);
+                    } else if (arg.equals("Create Selective Instrumentation File")) {
+                        GlobalDataWindow gdw = (GlobalDataWindow) window;
+                        SelectiveFileGenerator.showWindow(gdw.getPpTrial(), gdw);
                     }
+
                 } catch (Exception e) {
                     ParaProfUtils.handleException(e);
                 }
@@ -264,6 +257,23 @@ public class ParaProfUtils {
 
         JMenu fileMenu = new JMenu("File");
 
+        if (window instanceof GlobalDataWindow) {
+            JMenuItem jMenuItem;
+            jMenuItem = new JMenuItem("Export Profile");
+            jMenuItem.addActionListener(actionListener);
+            fileMenu.add(jMenuItem);
+            jMenuItem = new JMenuItem("Convert to Phase Profile");
+            jMenuItem.addActionListener(actionListener);
+            fileMenu.add(jMenuItem);
+            jMenuItem = new JMenuItem("Create Selective Instrumentation File");
+            jMenuItem.addActionListener(actionListener);
+            fileMenu.add(jMenuItem);
+            jMenuItem = new JMenuItem("Add Mean to Comparison Window");
+            jMenuItem.addActionListener(actionListener);
+            fileMenu.add(jMenuItem);
+            fileMenu.add(new JSeparator());
+        }
+
         JMenu subMenu = new JMenu("Save ...");
         subMenu.getPopupMenu().setLightWeightPopupEnabled(false);
 
@@ -351,6 +361,20 @@ public class ParaProfUtils {
         return menu;
     }
 
+    public static void showCommMatrix(ParaProfTrial ppTrial, JFrame parentFrame) {
+        JFrame window = CommunicationMatrixWindow.createCommunicationMatrixWindow(ppTrial, parentFrame);
+        if (window != null) {
+            window.setVisible(true);
+        }
+    }
+
+    public static void show3dCommMatrix(ParaProfTrial ppTrial, JFrame parentFrame) {
+        JFrame window = ThreeDeeCommMatrixWindow.createCommunicationMatrixWindow(ppTrial, parentFrame);
+        if (window != null) {
+            window.setVisible(true);
+        }
+    }
+
     public static JMenu createWindowsMenu(final ParaProfTrial ppTrial, final JFrame owner) {
 
         ActionListener actionListener = new ActionListener() {
@@ -361,9 +385,12 @@ public class ParaProfUtils {
                     String arg = evt.getActionCommand();
 
                     if (arg.equals("ParaProf Manager")) {
-                        (new ParaProfManagerWindow()).setVisible(true);
+			ParaProf.paraProfManagerWindow.setVisible(true);
                     } else if (arg.equals("Function Legend")) {
                         (new LedgerWindow(ppTrial, 0, owner)).setVisible(true);
+                    } else if (arg.equals("Group Changer")) {
+                        GroupChangerWindow gcw = GroupChangerWindow.createGroupChangerWindow(ppTrial, owner);
+                        gcw.setVisible(true);
                     } else if (arg.equals("Group Legend")) {
                         (new LedgerWindow(ppTrial, 1, owner)).setVisible(true);
                     } else if (arg.equals("User Event Legend")) {
@@ -372,6 +399,10 @@ public class ParaProfUtils {
                         (new LedgerWindow(ppTrial, 3, owner)).setVisible(true);
                     } else if (arg.equals("Snapshot Controller")) {
                         ppTrial.showSnapshotController();
+                    } else if (arg.equals("Communication Matrix")) {
+                        ParaProfUtils.showCommMatrix(ppTrial, owner);
+                    } else if (arg.equals("3D Communication Matrix")) {
+                        ParaProfUtils.show3dCommMatrix(ppTrial, owner);
                     } else if (arg.equals("3D Visualization")) {
 
                         if (JVMDependent.version.equals("1.3")) {
@@ -391,7 +422,7 @@ public class ParaProfUtils {
                             JOptionPane.showMessageDialog(owner, "Unable to load jogl library.  Possible reasons:\n"
                                     + "libjogl.so is not in your LD_LIBRARY_PATH.\n"
                                     + "Jogl is not built for this platform.\nOpenGL is not installed\n\n"
-                                    + "Jogl is available at jogl.dev.java.net");
+                                    + "Jogl is available at jogl.dev.java.net\n\n" + "Message : " + e.getMessage());
                         } catch (UnsupportedClassVersionError e) {
                             JOptionPane.showMessageDialog(owner,
                                     "Unsupported class version.  Are you sure you're using Java 1.4 or above?");
@@ -424,6 +455,15 @@ public class ParaProfUtils {
         menuItem.addActionListener(actionListener);
         windowsMenu.add(menuItem);
 
+        // still in development
+        menuItem = new JMenuItem("3D Communication Matrix");
+        menuItem.addActionListener(actionListener);
+        windowsMenu.add(menuItem);
+
+        menuItem = new JMenuItem("Communication Matrix");
+        menuItem.addActionListener(actionListener);
+        windowsMenu.add(menuItem);
+
         if (ppTrial.getDataSource().getWellBehavedSnapshots()) {
             menuItem = new JMenuItem("Snapshot Controller");
             menuItem.addActionListener(actionListener);
@@ -533,6 +573,10 @@ public class ParaProfUtils {
             windowsMenu.add(phaseLedger);
         }
 
+        final JMenuItem groupRemapper = new JMenuItem("Group Changer");
+        groupRemapper.addActionListener(actionListener);
+        windowsMenu.add(groupRemapper);
+
         windowsMenu.add(new JSeparator());
 
         menuItem = new JMenuItem("Close All Sub-Windows");
@@ -583,13 +627,6 @@ public class ParaProfUtils {
         g2.scale(scale, scale);
     }
 
-    public FunctionBarChartWindow createFunctionAcrossPhasesWindow(ParaProfTrial ppTrial, Thread thread, Function function,
-            Component owner) {
-        FunctionBarChartWindow functionDataWindow = new FunctionBarChartWindow(ppTrial, function, owner);
-        functionDataWindow.changeToPhaseDisplay(thread);
-        return functionDataWindow;
-    }
-
     public static JPopupMenu createFunctionClickPopUp(final ParaProfTrial ppTrial, final Function function, final Thread thread,
             final Component owner) {
         ActionListener actionListener = new ActionListener() {
@@ -602,9 +639,9 @@ public class ParaProfUtils {
                         FunctionBarChartWindow functionDataWindow = new FunctionBarChartWindow(ppTrial, function, owner);
                         functionDataWindow.setVisible(true);
                     } else if (arg.equals("Show Function Data over Phases")) {
-                        FunctionBarChartWindow functionDataWindow = new FunctionBarChartWindow(ppTrial, function, owner);
-                        functionDataWindow.changeToPhaseDisplay(thread);
-                        functionDataWindow.setVisible(true);
+                        FunctionBarChartWindow functionBarChartWindow = FunctionBarChartWindow.CreateFunctionsOverPhaseDisplay(
+                                ppTrial, function, thread, owner);
+                        functionBarChartWindow.setVisible(true);
                     } else if (arg.equals("Show Function Histogram")) {
                         HistogramWindow hw = new HistogramWindow(ppTrial, function, owner);
                         hw.setVisible(true);
@@ -619,12 +656,32 @@ public class ParaProfUtils {
                         ParaProf.incrementNumWindows();
                     } else if (arg.equals("Show Source Code")) {
 
-                        if (ParaProf.insideEclipse) {
+                        if (ParaProf.controlMode) {
+                            ExternalController.outputCommand("sourcecode " + function.getSourceLink());
+                        } else if (ParaProf.insideEclipse) {
                             ParaProf.eclipseHandler.openSourceLocation(ppTrial, function);
                         } else {
                             // use internal viewer
                             ParaProf.getDirectoryManager().showSourceCode(function.getSourceLink());
                         }
+                    } else if (arg.equals("Launch External Tool for this Function & Metric")) {
+                        String metricName = "TIME";
+                        if (owner instanceof BarChart) {
+                            BarChart tmp = (BarChart) owner;
+                            metricName = tmp.getBarChartModel().getDataSorter().getSelectedMetric().getName();
+                        }
+                        List tools = ExternalTool.findMatchingTools((String) ppTrial.getTrial().getMetaData().get(
+                                DataSource.FILE_TYPE_NAME), (String) ppTrial.getTrial().getName());
+                        ExternalTool.CommandParameters params = new ExternalTool.CommandParameters();
+                        params.function = function.getName();
+                        params.metric = metricName;
+                        params.nodeID = thread.getNodeID();
+                        params.threadID = thread.getThreadID();
+                        Map map = new TreeMap();
+                        map.putAll(thread.getMetaData());
+                        map.putAll(ppTrial.getDataSource().getMetaData());
+                        params.metadata = map;
+                        ExternalTool.launch(tools, params, owner);
                     }
 
                 } catch (Exception e) {
@@ -671,6 +728,14 @@ public class ParaProfUtils {
         jMenuItem.addActionListener(actionListener);
         functionPopup.add(jMenuItem);
 
+        if ((thread.getNodeID() >= 0)
+                && (ExternalTool.matchingToolExists((String) ppTrial.getTrial().getMetaData().get(DataSource.FILE_TYPE_NAME),
+                        (String) ppTrial.getTrial().getName()))) {
+            JMenuItem toolMenuItem = new JMenuItem("Launch External Tool for this Function & Metric");
+            toolMenuItem.addActionListener(actionListener);
+            functionPopup.add(toolMenuItem);
+        }
+
         // count function scripts
         int functionScripts = 0;
         for (int i = 0; i < ParaProf.scripts.size(); i++) {
@@ -1072,7 +1137,7 @@ public class ParaProfUtils {
             experiment.setName("New Experiment");
 
             ParaProf.paraProfManagerWindow.expandApplicationType(0, application.getID(), application);
-            ParaProf.paraProfManagerWindow.expandApplication(0, null, experiment);
+            ParaProf.paraProfManagerWindow.expandApplication(0, application, experiment);
 
             final ParaProfTrial ppTrial = new ParaProfTrial();
             ppTrial.getTrial().setDataSource(phaseDataSource);
@@ -1091,9 +1156,7 @@ public class ParaProfUtils {
                     ppTrial.showMainWindow();
                 }
             });
-
         }
-
     }
 
     public static void exportTrial(ParaProfTrial ppTrial, Component owner) {
@@ -1176,6 +1239,19 @@ public class ParaProfUtils {
         return str;
     }
 
+    // remove the source code location, if preferences are set for it
+    public static String removeSourceLocation(String str) {
+        if (str.startsWith("Loop:")) {
+            return str;
+        }
+        while (str.indexOf("[{") != -1) {
+            int a = str.indexOf("[{");
+            int b = str.indexOf("}]");
+            str = str.substring(0, a) + str.substring(b + 2);
+        }
+        return str;
+    }
+
     // handles reversed callpaths
     public static String getDisplayName(Function function) {
         if (ParaProf.preferences.getReversedCallPaths()) {
@@ -1203,7 +1279,7 @@ public class ParaProfUtils {
         return removeSource(name);
     }
 
-    public static String getThreadIdentifier(Thread thread) {
+    public static String getThreadLabel(Thread thread) {
 
         if (thread.getNodeID() == -1) {
             return "Mean";
@@ -1212,6 +1288,15 @@ public class ParaProfUtils {
         } else if (thread.getNodeID() == -3) {
             return "Std. Dev.";
         } else {
+            if (ParaProf.preferences.getAutoLabels()) {
+                DataSource dataSource = thread.getDataSource();
+                if (dataSource.getHasContexts() == false && dataSource.getHasThreads() == false) {
+                    return "node " + thread.getNodeID();
+                }
+                if (dataSource.getHasContexts() == false) {
+                    return "node " + thread.getNodeID() + ", thread " + thread.getThreadID();
+                }
+            }
             return "n,c,t " + thread.getNodeID() + "," + thread.getContextID() + "," + thread.getThreadID();
         }
 
@@ -1251,7 +1336,7 @@ public class ParaProfUtils {
         return new Dimension(width, height);
     }
 
-    public static NumberFormat createNumberFormatter(final int units) {
+    public static NumberFormat createNumberFormatter(final int units, final boolean timeDenominator) {
         return new NumberFormat() {
 
             public Number parse(String source, ParsePosition parsePosition) {
@@ -1260,11 +1345,11 @@ public class ParaProfUtils {
             }
 
             public StringBuffer format(double number, StringBuffer toAppendTo, FieldPosition pos) {
-                return toAppendTo.append(UtilFncs.getOutputString(units, number, 5));
+                return toAppendTo.append(UtilFncs.getOutputString(units, number, 5, timeDenominator));
             }
 
             public StringBuffer format(long number, StringBuffer toAppendTo, FieldPosition pos) {
-                return toAppendTo.append(UtilFncs.getOutputString(units, number, 5));
+                return toAppendTo.append(UtilFncs.getOutputString(units, number, 5, timeDenominator));
             }
         };
     }
@@ -1279,20 +1364,14 @@ public class ParaProfUtils {
     public static void createMetricToolbarItems(JToolBar bar, ParaProfTrial ppTrial, final DataSorter dataSorter,
             final ToolBarListener listener) {
 
-        Vector metricList = new Vector();
-
-        for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
-            metricList.add(ppTrial.getMetricName(i));
-        }
-
-        final JComboBox metricBox = new JComboBox(metricList);
+        final JComboBox metricBox = new JComboBox(ppTrial.getMetricArray());
         final JComboBox valueBox = new JComboBox(ValueType.VALUES);
         metricBox.setMaximumSize(metricBox.getPreferredSize());
         valueBox.setMaximumSize(valueBox.getPreferredSize());
 
         ActionListener actionListener = new ActionListener() {
             public void actionPerformed(ActionEvent e) {
-                dataSorter.setSelectedMetricID(metricBox.getSelectedIndex());
+                dataSorter.setSelectedMetric((Metric) metricBox.getSelectedItem());
                 dataSorter.setValueType((ValueType) valueBox.getSelectedItem());
                 listener.toolBarUsed();
             }
@@ -1330,14 +1409,21 @@ public class ParaProfUtils {
         } else {
             JMenu subSubMenu = new JMenu(valueType.toString() + "...");
             subSubMenu.getPopupMenu().setLightWeightPopupEnabled(false);
-            for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
-
-                if (i == dataSorter.getSelectedMetricID() && enabled) {
-                    button = new JRadioButtonMenuItem(ppTrial.getMetric(i).getName(), true);
-                } else {
-                    button = new JRadioButtonMenuItem(ppTrial.getMetric(i).getName());
+            for (Iterator it = ppTrial.getMetrics().iterator(); it.hasNext();) {
+                Metric metric = (Metric) it.next();
+                if (metric == null) {
+                    continue;
                 }
-                final int metricID = i;
+
+                int id = metric.getID();
+
+                if (id == dataSorter.getSelectedMetric().getID() && enabled) {
+                    button = new JRadioButtonMenuItem(metric.getName(), true);
+                } else {
+                    button = new JRadioButtonMenuItem(metric.getName());
+                }
+
+                final Metric useMetric = metric;
 
                 button.addActionListener(new ActionListener() {
 
@@ -1345,10 +1431,10 @@ public class ParaProfUtils {
                         if (sort) {
                             dataSorter.setSortByVisible(false);
                             dataSorter.setSortType(SortType.VALUE);
-                            dataSorter.setSortMetric(metricID);
+                            dataSorter.setSortMetric(useMetric);
                             dataSorter.setSortValueType(valueType);
                         } else {
-                            dataSorter.setSelectedMetricID(metricID);
+                            dataSorter.setSelectedMetric(useMetric);
                             dataSorter.setValueType(valueType);
                         }
                         sortListener.resort();
@@ -1403,7 +1489,6 @@ public class ParaProfUtils {
                 group.add(button);
                 subMenu.add(button);
             }
-
         }
 
         subMenu.add(createMetricMenu(ppTrial, ValueType.EXCLUSIVE, dataSorter.getValueType() == ValueType.EXCLUSIVE
