@@ -17,6 +17,7 @@ import java.util.*;
 
 import javax.swing.*;
 
+import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.perfdmf.DataSource;
 
 public class PreferencesWindow extends JFrame implements ActionListener, Observer {
@@ -40,16 +41,17 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
     private Font font;
 
     private JComboBox unitsBox;
+    private JCheckBox autoLabelsBox = new JCheckBox("Auto label node/context/threads");
     private JCheckBox showValuesAsPercentBox = new JCheckBox("Show Values as Percent");
     private JCheckBox showPathTitleInReverseBox = new JCheckBox("Show Path Title in Reverse");
     private JCheckBox reverseCallPathsBox = new JCheckBox("Reverse Call Paths");
     private JCheckBox meanIncludeNullBox = new JCheckBox(
             "<html>Interpret threads that do not call a given function as a 0 value for statistics computation</html>");
     private JCheckBox generateIntermediateCallPathDataBox = new JCheckBox(
-    "<html>Generate data for reverse calltree<br>(requires lots of memory)<br>(does not apply to currently loaded profiles)</html>");
+            "<html>Generate data for reverse calltree<br>(requires lots of memory)<br>(does not apply to currently loaded profiles)</html>");
 
     private JCheckBox showSourceLocationsBox = new JCheckBox("<html>Show Source Locations</html>");
-    
+
     public PreferencesWindow(Preferences preferences) {
         this.preferences = preferences;
 
@@ -63,20 +65,20 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
         meanIncludeNullBox.setToolTipText("<html>There are two methods for computing the mean for a given function over all threads:<br>1. Add up the values for that function for each thread and divide by the total number of threads.<br>2. Add up the values for that function for each thread and divide by the number of threads that actually called that function.<br><br>This has the effect that if a particular function is only called by 2 threads:<br>The first method will show the mean value as 1/N where N is the total number of threads.<br>The second method will only divide by 2.<br><br>This option also affects standard deviation computation.");
         reverseCallPathsBox.setToolTipText("<html>If this option is enabled, call path names will be shown in reverse<br>(e.g. \"C &lt;= B &lt;= A\" vs. \"A =&gt; B =&gt; C\")");
         generateIntermediateCallPathDataBox.setToolTipText("<html>If this option is enabled, then the reverse calltree will be available.<br>However, it requires additional memory and should be disabled if the JVM<br>runs out of memory on large callpath datasets.</html>");
+        autoLabelsBox.setToolTipText("<html>If this option is enabled, execution thread labels \"n,c,t 0,0,0\" will be shortened based on the execution type (MPI, threaded, hybrid)</html>");
 
-        if (preferences.getLoaded()) {
-            // Set preferences based on saved values.
-            fontName = preferences.getFontName();
-            fontStyle = preferences.getFontStyle();
-            fontSize = preferences.getFontSize();
-            unitsBox.setSelectedIndex(preferences.getUnits());
-            showValuesAsPercentBox.setSelected(preferences.getShowValuesAsPercent());
-            showPathTitleInReverseBox.setSelected(preferences.getShowPathTitleInReverse());
-            reverseCallPathsBox.setSelected(preferences.getReversedCallPaths());
-            meanIncludeNullBox.setSelected(!preferences.getComputeMeanWithoutNulls());
-            generateIntermediateCallPathDataBox.setSelected(preferences.getGenerateIntermediateCallPathData());
-            showSourceLocationsBox.setSelected(preferences.getShowSourceLocation());
-        }
+        // Set preferences based on saved values.
+        fontName = preferences.getFontName();
+        fontStyle = preferences.getFontStyle();
+        fontSize = preferences.getFontSize();
+        unitsBox.setSelectedIndex(preferences.getUnits());
+        showValuesAsPercentBox.setSelected(preferences.getShowValuesAsPercent());
+        showPathTitleInReverseBox.setSelected(preferences.getShowPathTitleInReverse());
+        reverseCallPathsBox.setSelected(preferences.getReversedCallPaths());
+        autoLabelsBox.setSelected(preferences.getAutoLabels());
+        meanIncludeNullBox.setSelected(!preferences.getComputeMeanWithoutNulls());
+        generateIntermediateCallPathDataBox.setSelected(preferences.getGenerateIntermediateCallPathData());
+        showSourceLocationsBox.setSelected(preferences.getShowSourceLocation());
 
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
@@ -98,7 +100,6 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
         setTitle("TAU: ParaProf: Preferences");
         ParaProfUtils.setFrameIcon(this);
 
-
         int windowWidth = 650;
         int windowHeight = 520;
         setSize(new java.awt.Dimension(windowWidth, windowHeight));
@@ -150,31 +151,31 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.weightx = 0;
         gbc.weighty = 0;
-        ParaProfUtils.addCompItem(fontPanel, fontComboBox, gbc, 0, 0, 2, 1);
+        Utility.addCompItem(fontPanel, fontComboBox, gbc, 0, 0, 2, 1);
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.weightx = 0;
         gbc.weighty = 0;
-        ParaProfUtils.addCompItem(fontPanel, bold, gbc, 0, 1, 1, 1);
+        Utility.addCompItem(fontPanel, bold, gbc, 0, 1, 1, 1);
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.WEST;
         gbc.weightx = 0;
         gbc.weighty = 0;
-        ParaProfUtils.addCompItem(fontPanel, italic, gbc, 0, 2, 1, 1);
+        Utility.addCompItem(fontPanel, italic, gbc, 0, 2, 1, 1);
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.weightx = 0;
         gbc.weighty = 0;
-        ParaProfUtils.addCompItem(fontPanel, barHeightLabel, gbc, 1, 1, 1, 1);
+        Utility.addCompItem(fontPanel, barHeightLabel, gbc, 1, 1, 1, 1);
 
         gbc.fill = GridBagConstraints.HORIZONTAL;
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.weightx = 0.25;
         gbc.weighty = 0.25;
-        ParaProfUtils.addCompItem(fontPanel, barHeightSlider, gbc, 1, 2, 1, 1);
+        Utility.addCompItem(fontPanel, barHeightSlider, gbc, 1, 2, 1, 1);
 
         JPanel defaultsPanel = new JPanel();
         defaultsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Window defaults"));
@@ -182,18 +183,19 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
 
         JLabel unitsLabel = new JLabel("Units");
 
-        ParaProfUtils.addCompItem(defaultsPanel, unitsLabel, gbc, 0, 0, 1, 1);
-        ParaProfUtils.addCompItem(defaultsPanel, unitsBox, gbc, 1, 0, 1, 1);
-        ParaProfUtils.addCompItem(defaultsPanel, showValuesAsPercentBox, gbc, 0, 1, 2, 1);
+        Utility.addCompItem(defaultsPanel, unitsLabel, gbc, 0, 0, 1, 1);
+        Utility.addCompItem(defaultsPanel, unitsBox, gbc, 1, 0, 1, 1);
+        Utility.addCompItem(defaultsPanel, showValuesAsPercentBox, gbc, 0, 1, 2, 1);
 
         JPanel settingsPanel = new JPanel();
         settingsPanel.setBorder(BorderFactory.createTitledBorder(BorderFactory.createEtchedBorder(), "Settings"));
         settingsPanel.setLayout(new GridBagLayout());
-        ParaProfUtils.addCompItem(settingsPanel, showPathTitleInReverseBox, gbc, 0, 2, 2, 1);
-        ParaProfUtils.addCompItem(settingsPanel, reverseCallPathsBox, gbc, 0, 3, 2, 1);
-        ParaProfUtils.addCompItem(settingsPanel, meanIncludeNullBox, gbc, 0, 4, 2, 1);
-        ParaProfUtils.addCompItem(settingsPanel, generateIntermediateCallPathDataBox, gbc, 0, 5, 2, 1);
-        ParaProfUtils.addCompItem(settingsPanel, showSourceLocationsBox, gbc, 0, 6, 2, 1);
+        Utility.addCompItem(settingsPanel, showPathTitleInReverseBox, gbc, 0, 2, 2, 1);
+        Utility.addCompItem(settingsPanel, reverseCallPathsBox, gbc, 0, 3, 2, 1);
+        Utility.addCompItem(settingsPanel, meanIncludeNullBox, gbc, 0, 4, 2, 1);
+        Utility.addCompItem(settingsPanel, generateIntermediateCallPathDataBox, gbc, 0, 5, 2, 1);
+        Utility.addCompItem(settingsPanel, showSourceLocationsBox, gbc, 0, 6, 2, 1);
+        Utility.addCompItem(settingsPanel, autoLabelsBox, gbc, 0, 7, 2, 1);
 
         gbc.fill = GridBagConstraints.BOTH;
 
@@ -307,18 +309,16 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
         //The path to data might have changed, therefore, reset the title.
         this.setTitle("ParaProf Preferences");
         this.setLocation(WindowPlacer.getNewLocation(this, invoker));
-        this.show();
+        this.setVisible(true);
     }
 
     public void loadSavedPreferences() {
         this.preferences = ParaProf.preferences;
-        if (preferences.getLoaded()) {
-            // Set preferences based on saved values.
-            fontName = preferences.getFontName();
-            fontStyle = preferences.getFontStyle();
-            fontSize = preferences.getFontSize();
-            font = null;
-        }
+        // Set preferences based on saved values.
+        fontName = preferences.getFontName();
+        fontStyle = preferences.getFontStyle();
+        fontSize = preferences.getFontSize();
+        font = null;
     }
 
     public Font getFont() {
@@ -336,6 +336,7 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
         ParaProf.preferences.setShowValuesAsPercent(showValuesAsPercentBox.isSelected());
         ParaProf.preferences.setShowPathTitleInReverse(showPathTitleInReverseBox.isSelected());
         ParaProf.preferences.setReversedCallPaths(reverseCallPathsBox.isSelected());
+        ParaProf.preferences.setAutoLabels(autoLabelsBox.isSelected());
         ParaProf.preferences.setComputeMeanWithoutNulls(!meanIncludeNullBox.isSelected());
         ParaProf.preferences.setGenerateIntermediateCallPathData(generateIntermediateCallPathDataBox.isSelected());
         ParaProf.preferences.setShowSourceLocation(showSourceLocationsBox.isSelected());
@@ -372,7 +373,7 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
                 if (arg.equals("Show Color Map")) {
                     ParaProf.colorMap.showColorMap(this);
                 } else if (arg.equals("Edit Source Directories")) {
-                	ParaProf.getDirectoryManager().display(this);
+                    ParaProf.getDirectoryManager().display(this);
                 } else if (arg.equals("Load Preferences...")) {
 
                     JFileChooser fileChooser = new JFileChooser();
@@ -479,6 +480,7 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
                     showPathTitleInReverseBox.setSelected(false);
                     reverseCallPathsBox.setSelected(false);
                     meanIncludeNullBox.setSelected(true);
+                    autoLabelsBox.setSelected(true);
                     generateIntermediateCallPathDataBox.setSelected(false);
                     showSourceLocationsBox.setSelected(true);
                     setControls();
@@ -496,6 +498,10 @@ public class PreferencesWindow extends JFrame implements ActionListener, Observe
             needDataEvent = true;
         }
 
+        if (autoLabelsBox.isSelected() != ParaProf.preferences.getAutoLabels()) {
+            needDataEvent = true;
+        }
+
         if (meanIncludeNullBox.isSelected() == ParaProf.preferences.getComputeMeanWithoutNulls()) {
             needDataEvent = true;
             DataSource.setMeanIncludeNulls(meanIncludeNullBox.isSelected());
