@@ -2,18 +2,21 @@ package edu.uoregon.tau.paraprof;
 
 import java.awt.*;
 import java.awt.event.*;
+import java.util.ArrayList;
 import java.util.List;
-import java.util.Vector;
 
 import javax.swing.*;
 import javax.swing.plaf.basic.BasicComboPopup;
 import javax.swing.plaf.basic.ComboPopup;
 import javax.swing.plaf.metal.MetalComboBoxUI;
 
+import edu.uoregon.tau.common.Utility;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.enums.VisType;
-import edu.uoregon.tau.perfdmf.Function;
+import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.perfdmf.Thread;
 import edu.uoregon.tau.vis.Plot;
+import edu.uoregon.tau.vis.SteppedComboBox;
 import edu.uoregon.tau.vis.VisRenderer;
 
 /**
@@ -21,9 +24,9 @@ import edu.uoregon.tau.vis.VisRenderer;
  *    
  * TODO : ...
  *
- * <P>CVS $Id: ThreeDeeControlPanel.java,v 1.7 2007/03/20 19:00:06 amorris Exp $</P>
+ * <P>CVS $Id: ThreeDeeControlPanel.java,v 1.19 2010/01/28 00:27:41 amorris Exp $</P>
  * @author	Alan Morris
- * @version	$Revision: 1.7 $
+ * @version	$Revision: 1.19 $
  */
 public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
@@ -43,6 +46,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
     private int selectedTab;
     private JTabbedPane tabbedPane; // keep a handle to remember the selected tab
+    private ThreeDeeScalePanel scalePanel;
 
     public class SliderComboBox extends JComboBox {
         public SliderComboBox() {
@@ -68,69 +72,6 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         }
     }
 
-    // From: http://forum.java.sun.com/thread.jspa?forumID=257&threadID=300107
-    class SteppedComboBoxUI extends MetalComboBoxUI {
-        protected ComboPopup createPopup() {
-            BasicComboPopup popup = new BasicComboPopup(comboBox) {
-
-                public void show() {
-                    Dimension popupSize = ((SteppedComboBox) comboBox).getPopupSize();
-                    popupSize.setSize(popupSize.width, getPopupHeightForRowCount(comboBox.getMaximumRowCount()));
-                    Rectangle popupBounds = computePopupBounds(0, comboBox.getBounds().height, popupSize.width, popupSize.height);
-                    scroller.setMaximumSize(popupBounds.getSize());
-                    scroller.setPreferredSize(popupBounds.getSize());
-                    scroller.setMinimumSize(popupBounds.getSize());
-                    list.invalidate();
-                    int selectedIndex = comboBox.getSelectedIndex();
-                    if (selectedIndex == -1) {
-                        list.clearSelection();
-                    } else {
-                        list.setSelectedIndex(selectedIndex);
-                    }
-                    list.ensureIndexIsVisible(list.getSelectedIndex());
-                    setLightWeightPopupEnabled(comboBox.isLightWeightPopupEnabled());
-
-                    show(comboBox, popupBounds.x, popupBounds.y);
-                }
-            };
-            popup.getAccessibleContext().setAccessibleParent(comboBox);
-            return popup;
-        }
-    }
-
-    public class SteppedComboBox extends JComboBox {
-        protected int popupWidth;
-
-        public SteppedComboBox(ComboBoxModel aModel) {
-            super(aModel);
-            setUI(new SteppedComboBoxUI());
-            popupWidth = 0;
-        }
-
-        public SteppedComboBox(final Object[] items) {
-            super(items);
-            setUI(new SteppedComboBoxUI());
-            popupWidth = 0;
-        }
-
-        public SteppedComboBox(Vector items) {
-            super(items);
-            setUI(new SteppedComboBoxUI());
-            popupWidth = 0;
-        }
-
-        public void setPopupWidth(int width) {
-            popupWidth = width;
-        }
-
-        public Dimension getPopupSize() {
-            Dimension size = getSize();
-            if (popupWidth < 1)
-                popupWidth = size.width;
-            return new Dimension(popupWidth, size.height);
-        }
-    }
-
     public ThreeDeeControlPanel(ThreeDeeWindow window, ThreeDeeSettings settings, ParaProfTrial ppTrial, VisRenderer visRenderer) {
         this.settings = settings;
         this.window = window;
@@ -163,6 +104,11 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         group.add(jrb);
         addCompItem(this, jrb, gbc, 0, 3, 1, 1);
 
+        //        jrb = new JRadioButton(VisType.CALLGRAPH.toString(), settings.getVisType() == VisType.CALLGRAPH);
+        //        jrb.addActionListener(this);
+        //        group.add(jrb);
+        //        addCompItem(this, jrb, gbc, 0, 4, 1, 1);
+
         createSubPanel();
 
     }
@@ -176,6 +122,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
         if (settings.getVisType() == VisType.SCATTER_PLOT) {
             subPanel = createScatterPanel();
+        } else if (settings.getVisType() == VisType.CALLGRAPH) {
+            subPanel = createCallGraphPanel();
         } else {
             subPanel = createFullDataPanel();
         }
@@ -214,30 +162,25 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         functionField.setBorder(BorderFactory.createLoweredBevelBorder());
         functionField.setCaretPosition(0);
 
-        String[] items = new String[ppTrial.getNumberOfMetrics()];
-        for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
-            items[i] = ppTrial.getMetric(i).getName();
-        }
-
         Dimension d;
         final SteppedComboBox valueBox = new SteppedComboBox(ValueType.VALUES);
         d = valueBox.getPreferredSize();
         valueBox.setMinimumSize(new Dimension(50, valueBox.getMinimumSize().height));
         valueBox.setPopupWidth(d.width);
 
-        final SteppedComboBox metricBox = new SteppedComboBox(items);
+        final SteppedComboBox metricBox = new SteppedComboBox(ppTrial.getMetricArray());
         d = metricBox.getPreferredSize();
         metricBox.setMinimumSize(new Dimension(50, metricBox.getMinimumSize().height));
         metricBox.setPopupWidth(d.width);
 
         valueBox.setSelectedItem(settings.getScatterValueTypes()[index]);
-        metricBox.setSelectedIndex(settings.getScatterMetricIDs()[index]);
+        metricBox.setSelectedItem(settings.getScatterMetrics()[index]);
 
         ActionListener metricSelector = new ActionListener() {
             public void actionPerformed(ActionEvent evt) {
                 try {
                     settings.setScatterValueType((ValueType) valueBox.getSelectedItem(), index);
-                    settings.setScatterMetricID(metricBox.getSelectedIndex(), index);
+                    settings.setScatterMetric((Metric) metricBox.getSelectedItem(), index);
                     window.redraw();
                 } catch (Exception e) {
                     ParaProfUtils.handleException(e);
@@ -302,6 +245,54 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         return panel;
     }
 
+    private JPanel createCallGraphPanel() {
+        JPanel panel = new JPanel();
+        panel.setLayout(new GridBagLayout());
+        panel.setBorder(BorderFactory.createRaisedBevelBorder());
+
+        GridBagConstraints gbc = new GridBagConstraints();
+        gbc.insets = new Insets(5, 5, 5, 5);
+
+        addCompItem(panel, new JLabel("Thread"), gbc, 0, 0, 1, 1);
+
+        List threadList = new ArrayList();
+
+        threadList.add(ppTrial.getDataSource().getMeanData());
+        threadList.add(ppTrial.getDataSource().getStdDevData());
+
+        threadList.addAll(ppTrial.getDataSource().getAllThreads());
+
+        final SliderComboBox threadComboBox = new SliderComboBox(threadList.toArray());
+
+        threadComboBox.addActionListener(new ActionListener() {
+
+            public void actionPerformed(ActionEvent e) {
+                // TODO Auto-generated method stub
+                settings.setSelectedThread((Thread) threadComboBox.getSelectedItem());
+                System.out.println("bargle");
+                window.redraw();
+            }
+        });
+
+        addCompItem(panel, threadComboBox, gbc, 1, 0, 1, 1);
+
+        tabbedPane = new JTabbedPane();
+        Plot plot = window.getPlot();
+        tabbedPane.addTab(plot.getName(), plot.getControlPanel(visRenderer));
+        tabbedPane.addTab("ColorScale", window.getColorScale().getControlPanel(visRenderer));
+        tabbedPane.addTab("Render", visRenderer.getControlPanel());
+        tabbedPane.setMinimumSize(new Dimension(300, 160));
+        tabbedPane.setSelectedIndex(0);
+
+        gbc.fill = GridBagConstraints.BOTH;
+        gbc.weightx = 0.5;
+        gbc.weighty = 0.5;
+
+        addCompItem(panel, tabbedPane, gbc, 0, 1, 2, 1);
+        return panel;
+
+    }
+
     private JPanel createScatterPanel() {
 
         JPanel panel = new JPanel();
@@ -342,6 +333,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         tabbedPane.addTab("ColorScale", window.getColorScale().getControlPanel(visRenderer));
         tabbedPane.addTab("Render", visRenderer.getControlPanel());
         tabbedPane.setMinimumSize(new Dimension(300, 160));
+        selectedTab = Math.min(selectedTab, tabbedPane.getTabCount()-1);
         tabbedPane.setSelectedIndex(selectedTab);
 
         gbc.fill = GridBagConstraints.BOTH;
@@ -362,6 +354,9 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         GridBagConstraints gbc = new GridBagConstraints();
         gbc.insets = new Insets(5, 5, 5, 5);
 
+        settings.getSelections()[index] = Math.min(settings.getSelections()[index], max);
+        settings.getSelections()[index] = Math.max(settings.getSelections()[index], min);
+
         final JScrollBar scrollBar = new JScrollBar(JScrollBar.HORIZONTAL, settings.getSelections()[index], 1, min, max);
         scrollBar.setBlockIncrement((max - min) / 10);
 
@@ -392,6 +387,9 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                 heightValueField.setText(window.getSelectedHeightValue());
                 colorValueField.setText(window.getSelectedColorValue());
 
+                scalePanel.setPosition(0, window.getSelectedHeightRatio());
+                scalePanel.setPosition(1, window.getSelectedColorRatio());
+
                 window.redraw();
             }
         });
@@ -401,12 +399,47 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         gbc.weightx = 0.5;
         gbc.weighty = 0.5;
 
-        ParaProfUtils.addCompItem(panel, textField, gbc, 1, 0, 1, 1);
-        ParaProfUtils.addCompItem(panel, scrollBar, gbc, 1, 1, 1, 1);
+        Utility.addCompItem(panel, textField, gbc, 1, 0, 1, 1);
+        Utility.addCompItem(panel, scrollBar, gbc, 1, 1, 1, 1);
 
         return panel;
     }
 
+    private String getScaleString(double value, Metric metric, ValueType valueType) {
+        int units = window.getUnits();
+
+        if (!metric.isTimeMetric() || !ValueType.isTimeUnits(valueType)) {
+            units = 0;
+        }
+        return UtilFncs.getOutputString(units, value, 6, metric.isTimeDenominator()).trim();
+    }
+
+    private void updateScalePanel() {
+        String mins[] = new String[2];
+        String maxs[] = new String[2];
+
+        mins[0] = "0";
+        mins[1] = "0";
+        maxs[0] = getScaleString(window.getMaxHeightValue(), settings.getHeightMetric(), settings.getHeightValue());
+        maxs[1] = getScaleString(window.getMaxColorValue(), settings.getColorMetric(), settings.getColorValue());
+
+        String labels[] = { "height", "color" };
+        String unitLabels[] = { window.getHeightUnitLabel(), window.getColorUnitLabel() };
+        scalePanel.setRanges(mins, maxs, labels, unitLabels);
+
+        scalePanel.setPosition(0, window.getSelectedHeightRatio());
+        scalePanel.setPosition(1, window.getSelectedColorRatio());
+
+    }
+
+    private JPanel createScalePanel() {
+        if (scalePanel == null) {
+            scalePanel = ThreeDeeScalePanel.CreateScalePanel();
+            updateScalePanel();
+        }
+        return scalePanel.getJPanel();
+    }
+
     private JPanel createFullDataPanel() {
 
         JPanel regularPanel = new JPanel();
@@ -422,8 +455,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
                     Plot plot = window.getPlot();
 
-                    settings.setHeightMetricID(heightMetricBox.getSelectedIndex());
-                    settings.setColorMetricID(colorMetricBox.getSelectedIndex());
+                    settings.setHeightMetric((Metric) heightMetricBox.getSelectedItem());
+                    settings.setColorMetric((Metric) colorMetricBox.getSelectedItem());
                     settings.setHeightValue((ValueType) heightValueBox.getSelectedItem());
                     settings.setColorValue((ValueType) colorValueBox.getSelectedItem());
 
@@ -436,6 +469,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
                     window.redraw();
 
+                    updateScalePanel();
+
                 } catch (Exception e) {
                     ParaProfUtils.handleException(e);
                 }
@@ -459,34 +494,28 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
         colorValueBox.setSelectedItem(settings.getColorValue());
         colorValueBox.addActionListener(metricChanger);
 
-        String[] items = new String[ppTrial.getNumberOfMetrics()];
-        for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
-            items[i] = ppTrial.getMetric(i).getName();
-        }
-
-        heightMetricBox = new SteppedComboBox(items);
+        heightMetricBox = new SteppedComboBox(ppTrial.getMetricArray());
         d = heightMetricBox.getPreferredSize();
         heightMetricBox.setMinimumSize(new Dimension(50, heightMetricBox.getMinimumSize().height));
         heightMetricBox.setPopupWidth(d.width);
-        heightMetricBox.setSelectedIndex(settings.getHeightMetricID());
+        heightMetricBox.setSelectedItem(settings.getHeightMetric());
         heightMetricBox.addActionListener(metricChanger);
 
-        items = new String[ppTrial.getNumberOfMetrics()];
-        for (int i = 0; i < ppTrial.getNumberOfMetrics(); i++) {
-            items[i] = ppTrial.getMetric(i).getName();
-        }
-        colorMetricBox = new SteppedComboBox(items);
+        colorMetricBox = new SteppedComboBox(ppTrial.getMetricArray());
         d = colorMetricBox.getPreferredSize();
         colorMetricBox.setMinimumSize(new Dimension(50, colorMetricBox.getMinimumSize().height));
         colorMetricBox.setPopupWidth(d.width);
-        colorMetricBox.setSelectedIndex(settings.getColorMetricID());
+        colorMetricBox.setSelectedItem(settings.getColorMetric());
+        colorMetricBox.addActionListener(metricChanger);
 
         tabbedPane = new JTabbedPane();
         Plot plot = window.getPlot();
         tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);
-        tabbedPane.addTab(plot.getName(), plot.getControlPanel(visRenderer));
+        tabbedPane.addTab("Scales", createScalePanel());
+        //        tabbedPane.addTab(plot.getName(), plot.getControlPanel(visRenderer));
+        tabbedPane.addTab("Plot", plot.getControlPanel(visRenderer));
         tabbedPane.addTab("Axes", plot.getAxes().getControlPanel(visRenderer));
-        tabbedPane.addTab("ColorScale", window.getColorScale().getControlPanel(visRenderer));
+        tabbedPane.addTab("Color", window.getColorScale().getControlPanel(visRenderer));
         tabbedPane.addTab("Render", visRenderer.getControlPanel());
         tabbedPane.setMinimumSize(new Dimension(290, 160));
         tabbedPane.setSelectedIndex(selectedTab);
@@ -547,6 +576,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
             if (EventSrc instanceof JRadioButton) {
 
                 selectedTab = tabbedPane.getSelectedIndex();
+                selectedTab = 0; // they don't match anymore, so always reset to 0
 
                 String arg = evt.getActionCommand();
                 Plot plot = window.getPlot();
@@ -567,6 +597,8 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
                     settings.setVisType(VisType.TRIANGLE_MESH_PLOT);
                 } else if (arg.equals(VisType.SCATTER_PLOT.toString())) {
                     settings.setVisType(VisType.SCATTER_PLOT);
+                } else if (arg.equals(VisType.CALLGRAPH.toString())) {
+                    settings.setVisType(VisType.CALLGRAPH);
                 }
 
                 window.resetSplitPane();
@@ -579,6 +611,7 @@ public class ThreeDeeControlPanel extends JPanel implements ActionListener {
 
     public void dataChanged() {
         window.redraw();
+        updateScalePanel();
         createSubPanel();
         heightValueField.setText(window.getSelectedHeightValue());
         colorValueField.setText(window.getSelectedColorValue());
