@@ -14,15 +14,19 @@ import javax.swing.*;
 import edu.uoregon.tau.paraprof.enums.SortType;
 import edu.uoregon.tau.paraprof.enums.ValueType;
 import edu.uoregon.tau.paraprof.enums.VisType;
+import edu.uoregon.tau.paraprof.graph.Layout;
+import edu.uoregon.tau.paraprof.graph.Vertex;
+import edu.uoregon.tau.paraprof.graph.Vertex.BackEdge;
 import edu.uoregon.tau.paraprof.interfaces.ParaProfWindow;
 import edu.uoregon.tau.paraprof.interfaces.SortListener;
 import edu.uoregon.tau.paraprof.interfaces.UnitListener;
 import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.Thread;
 import edu.uoregon.tau.vis.*;
+import edu.uoregon.tau.vis.XmasTree.Ornament;
 
 public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListener, Observer, Printable, ParaProfWindow,
-        UnitListener, SortListener {
+        UnitListener, SortListener, VisCanvasListener, ThreeDeeImageProvider {
 
     private final int defaultToScatter = 4000;
 
@@ -58,6 +62,8 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
     private int units = ParaProf.preferences.getUnits();
 
+    float minHeightValue = 0;
+    float minColorValue = 0;
     float maxHeightValue = 0;
     float maxColorValue = 0;
 
@@ -73,12 +79,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         });
         this.ppTrial = ppTrial;
 
-        settings.setColorMetricID(ppTrial.getDefaultMetricID());
-        settings.setHeightMetricID(ppTrial.getDefaultMetricID());
-        settings.setScatterMetricID(ppTrial.getDefaultMetricID(), 0);
-        settings.setScatterMetricID(ppTrial.getDefaultMetricID(), 1);
-        settings.setScatterMetricID(ppTrial.getDefaultMetricID(), 2);
-        settings.setScatterMetricID(ppTrial.getDefaultMetricID(), 3);
+        settings.setColorMetric(ppTrial.getDefaultMetric());
+        settings.setHeightMetric(ppTrial.getDefaultMetric());
+        settings.setScatterMetric(ppTrial.getDefaultMetric(), 0);
+        settings.setScatterMetric(ppTrial.getDefaultMetric(), 1);
+        settings.setScatterMetric(ppTrial.getDefaultMetric(), 2);
+        settings.setScatterMetric(ppTrial.getDefaultMetric(), 3);
 
         dataSorter = new DataSorter(ppTrial);
         dataSorter.setSortType(SortType.NAME);
@@ -111,7 +117,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         // initialize the scatterplot functions to the 4 most varying functions
         // we just get the first four stddev functions
         DataSorter dataSorter = new DataSorter(ppTrial);
-        dataSorter.setSelectedMetricID(ppTrial.getDefaultMetricID());
+        dataSorter.setSelectedMetric(ppTrial.getDefaultMetric());
         dataSorter.setDescendingOrder(true);
         List stdDevList = dataSorter.getFunctionProfiles(dataSource.getStdDevData());
         int count = 0;
@@ -134,6 +140,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
         visRenderer.addShape(plot);
         visRenderer.addShape(colorScale);
+        visRenderer.setVisCanvasListener(this);
         visCanvas = new VisCanvas(visRenderer);
 
         visCanvas.addKeyListener(this);
@@ -197,7 +204,7 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         Function[] scatterFunctions = settings.getScatterFunctions();
 
         ValueType[] scatterValueTypes = settings.getScatterValueTypes();
-        int[] scatterMetricIDs = settings.getScatterMetricIDs();
+        Metric[] scatterMetricIDs = settings.getScatterMetrics();
 
         DataSource dataSource = ppTrial.getDataSource();
         int numThreads = dataSource.getNumThreads();
@@ -252,8 +259,276 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         plot = scatterPlot;
     }
 
+    private List createGraph(DataSource dataSource, ThreeDeeSettings settings) {
+        List backEdges;
+        Map vertexMap;
+
+        vertexMap = new HashMap();
+        backEdges = new ArrayList();
+
+        Thread thread = settings.getSelectedThread();
+        if (thread == null) {
+            thread = dataSource.getMeanData();
+        }
+
+        CallPathUtilFuncs.buildThreadRelations(dataSource, thread);
+        List functionProfileList = thread.getFunctionProfiles();
+
+        for (int i = 0; i < functionProfileList.size(); i++) {
+            FunctionProfile fp = (FunctionProfile) functionProfileList.get(i);
+            if (fp == null) // skip it if this thread didn't call this function
+                continue;
+
+            if (!fp.isCallPathFunction()) { // skip callpath functions (we only want the actual functions)
+
+                Vertex v = new Vertex(fp, 1, 1);
+                v.setColorRatio(1);
+                vertexMap.put(fp, v);
+            }
+        }
+
+        // now we follow the call paths and eliminate back edges
+        Stack toVisit = new Stack();
+        Stack currentPath = new Stack();
+
+        for (int i = 0; i < functionProfileList.size(); i++) {
+            FunctionProfile fp = (FunctionProfile) functionProfileList.get(i);
+            if (fp == null) // skip it if this thread didn't call this function
+                continue;
+
+            if (!fp.isCallPathFunction()) { // skip callpath functions (we only want the actual functions)
+
+                // get the vertex for this FunctionProfile 
+                Vertex root = (Vertex) vertexMap.get(fp);
+
+                if (!root.getVisited()) {
+
+                    currentPath.add(fp);
+                    toVisit.add(null); // null in the toVisit stack marks the end of a set of children (they must get pushed into the stack prior to the children)
+
+                    // add all the children to the toVisit list
+                    for (Iterator it = fp.getChildProfiles(); it.hasNext();) {
+                        FunctionProfile childFp = (FunctionProfile) it.next();
+                        toVisit.add(childFp);
+                    }
+
+                    while (!toVisit.empty()) {
+                        FunctionProfile childFp = (FunctionProfile) toVisit.pop();
+
+                        if (childFp == null) {
+                            // this marks the end of a set of children, so pop the current path
+                            // and move on to the next one in toVisit
+                            currentPath.pop();
+                            continue;
+                        }
+
+                        Vertex child = (Vertex) vertexMap.get(childFp);
+                        FunctionProfile parentFp = (FunctionProfile) currentPath.peek();
+
+                        Vertex parent = (Vertex) vertexMap.get(parentFp);
+
+                        // run through the currentPath and see if childFp is in it, if so, this is a backedge
+                        boolean back = false;
+                        for (Iterator it = currentPath.iterator(); it.hasNext();) {
+                            if ((FunctionProfile) it.next() == childFp) {
+                                back = true;
+                                break;
+                            }
+                        }
+
+                        if (back) {
+                            backEdges.add(new BackEdge(parent, child));
+                        } else {
+
+                            boolean found = false;
+                            for (int j = 0; j < parent.getChildren().size(); j++) {
+                                if (parent.getChildren().get(j) == child)
+                                    found = true;
+                            }
+                            if (!found)
+                                parent.getChildren().add(child);
+
+                            found = false;
+                            for (int j = 0; j < child.getParents().size(); j++) {
+                                if (child.getParents().get(j) == parent)
+                                    found = true;
+                            }
+                            if (!found)
+                                child.getParents().add(parent);
+
+                            if (child.getVisited() == false) {
+
+                                child.setVisited(true);
+
+                                currentPath.add(childFp);
+
+                                toVisit.add(null);
+                                for (Iterator it = childFp.getChildProfiles(); it.hasNext();) {
+                                    FunctionProfile grandChildFunction = (FunctionProfile) it.next();
+
+                                    toVisit.add(grandChildFunction);
+                                }
+                            }
+
+                        }
+                    }
+                }
+            }
+        }
+        // now we should have a DAG, now find the roots
+
+        // Find Roots
+        List roots = Layout.findRoots(vertexMap);
+
+        // Assigning Levels
+        for (int i = 0; i < functionProfileList.size(); i++) {
+            FunctionProfile fp = (FunctionProfile) functionProfileList.get(i);
+            if (fp == null)
+                continue;
+
+            if (!fp.isCallPathFunction()) {
+                Vertex vertex = (Vertex) vertexMap.get(fp);
+
+                if (vertex.getLevel() == -1) {
+                    Layout.assignLevel(vertex);
+                }
+            }
+
+        }
+
+        // Insert Dummies
+        for (int i = 0; i < functionProfileList.size(); i++) {
+            FunctionProfile fp = (FunctionProfile) functionProfileList.get(i);
+            if (fp == null)
+                continue;
+
+            if (!fp.isCallPathFunction()) {
+                Vertex vertex = (Vertex) vertexMap.get(fp);
+                Layout.insertDummies(vertex);
+            }
+
+        }
+
+        // fill level lists
+        for (int i = 0; i < functionProfileList.size(); i++) {
+            FunctionProfile fp = (FunctionProfile) functionProfileList.get(i);
+            if (fp == null)
+                continue;
+
+            if (!fp.isCallPathFunction()) {
+                Vertex vertex = (Vertex) vertexMap.get(fp);
+                vertex.setVisited(false);
+            }
+
+        }
+
+        List levels = new ArrayList();
+
+        // Fill Levels
+        for (int i = 0; i < roots.size(); i++) {
+            Vertex root = (Vertex) roots.get(i);
+            Layout.fillLevels(root, levels, 0);
+        }
+
+        // Order Levels
+        Layout.runSugiyama(levels);
+        Layout.assignPositions(levels);
+        return levels;
+    }
+
+    private List decorateTree(List graphLevels, DataSource dataSource, ThreeDeeSettings settings) {
+        Map omap = new HashMap();
+        List treeLevels = new ArrayList();
+
+        Thread thread = settings.getSelectedThread();
+        if (thread == null) {
+            thread = dataSource.getMeanData();
+        }
+
+        for (int i = 0; i < graphLevels.size(); i++) {
+            List level = (List) graphLevels.get(i);
+            List treeLevel = new ArrayList();
+            int count = 0;
+
+            for (int j = 0; j < level.size(); j++) {
+                Vertex v = (Vertex) level.get(j);
+                if (v.getUserObject() != null) {
+                    count++;
+                }
+            }
+
+            //System.out.println("count = " + count + ", level.size() = " + level.size());
+
+            int c = 0;
+            for (int j = 0; j < level.size(); j++) {
+                Vertex v = (Vertex) level.get(j);
+
+                if (v.getUserObject() != null) {
+                    FunctionProfile fp = (FunctionProfile) v.getUserObject();
+                    Ornament o = new Ornament(fp.getName(), v);
+                    float size = (float) (fp.getInclusive(0) / thread.getMaxInclusive(0, 0));
+                    float color = (float) (fp.getExclusive(0) / thread.getMaxExclusive(0, 0));
+                    //float color = (float) (fp.getInclusive(0) / thread.getMaxInclusive(0, 0));
+                    //float size = (float) (fp.getExclusive(0) / thread.getMaxExclusive(0, 0));
+                    o.setSize(size);
+                    o.setColor(color);
+                    v.setGraphObject(o);
+                    omap.put(v, o);
+                    o.setPosition((float) c++ / count);
+                    treeLevel.add(o);
+                } else {
+                    // dummy node, don't make a graph cell
+                }
+
+            }
+            treeLevels.add(treeLevel);
+        }
+
+        for (int i = 0; i < graphLevels.size(); i++) {
+            List level = (List) graphLevels.get(i);
+
+            for (int j = 0; j < level.size(); j++) {
+                Vertex v = (Vertex) level.get(j);
+
+                if (v.getUserObject() != null) {
+                    Ornament a = (Ornament) v.getGraphObject();
+                    for (Iterator it = v.getChildren().iterator(); it.hasNext();) {
+                        Vertex child = (Vertex) it.next();
+                        Ornament b = (Ornament) child.getGraphObject();
+                        if (b != null && b.getUserObject() != null) {
+                            a.addChild(b);
+                        } else {
+                            //                            while (child.getGraphObject() == null) {
+                            //                                child = (Vertex) child.getChildren().get(0);
+                            //                            }
+                            //                            Ornament c = (Ornament) child.getGraphObject();
+                            //                            a.addChild(c);
+                        }
+                    }
+                } else {
+                    // dummy node, don't make a graph cell
+                }
+
+            }
+        }
+
+        return treeLevels;
+    }
+
+    private void generateCallGraph(boolean autoSize, ThreeDeeSettings settings) {
+        if (plot != null) {
+            plot.cleanUp();
+        }
+        List levels = createGraph(ppTrial.getDataSource(), settings);
+        List treeLevels = decorateTree(levels, ppTrial.getDataSource(), settings);
+        XmasTree xmasTree = new XmasTree(treeLevels);
+        xmasTree.setColorScale(colorScale);
+        plot = xmasTree;
+    }
+
     private void generate3dModel(boolean autoSize, ThreeDeeSettings settings) {
 
+        visRenderer.setCameraMode(VisRenderer.CAMERA_PLOT);
         if (plot != null) {
             plot.cleanUp();
         }
@@ -263,6 +538,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             return;
         }
 
+        if (settings.getVisType() == VisType.CALLGRAPH) {
+            generateCallGraph(autoSize, settings);
+            visRenderer.setCameraMode(VisRenderer.CAMERA_STICK);
+            return;
+        }
+
         if (triangleMeshPlot == null && barPlot == null) {
             autoSize = true;
         }
@@ -288,18 +569,14 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         }
 
         if (threadNames == null) {
-            threadNames = new ArrayList();
-            threads = new ArrayList();
-
-            for (Iterator it = ppTrial.getDataSource().getAllThreads().iterator(); it.hasNext();) {
-                Thread thread = (Thread) it.next();
-                threadNames.add(thread.getNodeID() + ":" + thread.getContextID() + ":" + thread.getThreadID());
-                threads.add(thread);
-            }
+            threadNames = ppTrial.getThreadNames();
+            threads = ppTrial.getThreads();
         }
 
         maxHeightValue = 0;
         maxColorValue = 0;
+        minHeightValue = Float.MAX_VALUE;
+        minColorValue = Float.MAX_VALUE;
 
         int funcIndex = 0;
         for (int i = 0; i < list.size(); i++) {
@@ -324,12 +601,14 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
                 if (functionProfile != null) {
                     heightValues[funcIndex][threadIndex] = (float) settings.getHeightValue().getValue(functionProfile,
-                            settings.getHeightMetricID(), ppTrial.getSelectedSnapshot());
+                            settings.getHeightMetric().getID(), ppTrial.getSelectedSnapshot());
                     colorValues[funcIndex][threadIndex] = (float) settings.getColorValue().getValue(functionProfile,
-                            settings.getColorMetricID(), ppTrial.getSelectedSnapshot());
+                            settings.getColorMetric().getID(), ppTrial.getSelectedSnapshot());
 
                     maxHeightValue = Math.max(maxHeightValue, heightValues[funcIndex][threadIndex]);
                     maxColorValue = Math.max(maxColorValue, colorValues[funcIndex][threadIndex]);
+                    minHeightValue = Math.min(minHeightValue, heightValues[funcIndex][threadIndex]);
+                    minColorValue = Math.min(minColorValue, colorValues[funcIndex][threadIndex]);
                 }
                 threadIndex++;
             }
@@ -394,16 +673,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
             axes.setOnEdge(true);
 
             if (barPlot == null) {
-
-                barPlot = new BarPlot();
-                barPlot.initialize(axes, settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(),
-                        heightValues, colorValues, colorScale);
-                plot = barPlot;
-            } else {
-                barPlot.setValues(settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(), heightValues,
-                        colorValues);
-                plot = barPlot;
+                barPlot = new BarPlot(axes, colorScale);
             }
+            barPlot.setValues(settings.getPlotWidth(), settings.getPlotDepth(), settings.getPlotHeight(), heightValues,
+                    colorValues);
+            plot = barPlot;
+
         }
     }
 
@@ -442,10 +717,10 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
                 settings.setSize((int) plot.getWidth(), (int) plot.getDepth(), (int) plot.getHeight());
 
-                if (oldSettings.getHeightMetricID() != newSettings.getHeightMetricID()
+                if (oldSettings.getHeightMetric() != newSettings.getHeightMetric()
                         || oldSettings.getHeightValue() != newSettings.getHeightValue()
                         || oldSettings.getColorValue() != newSettings.getColorValue()
-                        || oldSettings.getColorMetricID() != newSettings.getColorMetricID()) {
+                        || oldSettings.getColorMetric() != newSettings.getColorMetric()) {
                     generate3dModel(false, newSettings);
                 } else {
 
@@ -457,6 +732,12 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
                     plot.setSelectedCol(newSettings.getSelections()[1]);
                     plot.setSelectedRow(newSettings.getSelections()[0]);
                 }
+            } else if (newSettings.getVisType() == VisType.CALLGRAPH) {
+                visRenderer.removeShape(plot);
+                visRenderer.removeShape(colorScale);
+                generate3dModel(false, newSettings);
+                visRenderer.addShape(plot);
+                visRenderer.addShape(colorScale);
             }
         }
 
@@ -602,8 +883,9 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
     public void help(boolean display) {
         //Show the ParaProf help window.
         ParaProf.getHelpWindow().clearText();
-        if (display)
-            ParaProf.getHelpWindow().show();
+        if (display) {
+            ParaProf.getHelpWindow().setVisible(true);
+        }
         ParaProf.getHelpWindow().writeText("This is the 3D Window");
         ParaProf.getHelpWindow().writeText("");
         ParaProf.getHelpWindow().writeText(
@@ -711,28 +993,116 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
     }
 
     public String getFunctionName(int index) {
-        if (functionNames == null)
+        if (functionNames == null) {
             return null;
+        }
         return (String) functionNames.get(index);
     }
 
     public String getThreadName(int index) {
-        if (threadNames == null)
+        if (threadNames == null) {
             return null;
+        }
         return (String) threadNames.get(index);
     }
 
-    public String getSelectedHeightValue() {
-        if (threads == null || functionNames == null)
-            return "";
+    public double getMaxHeightValue() {
+        return maxHeightValue;
+    }
 
-        if (settings.getSelections()[1] < 0 || settings.getSelections()[0] < 0)
-            return "";
+    public double getMaxColorValue() {
+        return maxColorValue;
+    }
+
+    public double getMinHeightValue() {
+        return minHeightValue;
+    }
+
+    public double getMinColorValue() {
+        return minColorValue;
+    }
+
+    public String getHeightUnitLabel() {
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getHeightMetric();
+        return getUnitsString(units, settings.getHeightValue(), ppMetric);
+    }
+    
+    public String getColorUnitLabel() {
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getColorMetric();
+        return getUnitsString(units, settings.getColorValue(), ppMetric);
+    }
+    
+    /**
+     * @return a value between 0 and 1 representing the selected height value in the range [min to max]
+     */
+    public float getSelectedHeightRatio() {
+        if (threads == null || functionNames == null) {
+            return -1;
+        }
+
+        if (settings.getSelections()[1] < 0 || settings.getSelections()[0] < 0) {
+            return -1;
+        }
 
         Thread thread = (Thread) threads.get(settings.getSelections()[1]);
-
         Function function = (Function) functions.get(settings.getSelections()[0]);
+        FunctionProfile fp = thread.getFunctionProfile(function);
 
+        if (fp == null) {
+            return -1;
+        }
+
+        int units = this.units;
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getHeightMetric();
+        if (!ppMetric.isTimeMetric() || !ValueType.isTimeUnits(settings.getHeightValue())) {
+            units = 0;
+        }
+
+        double value = settings.getHeightValue().getValue(fp, settings.getHeightMetric().getID());
+        return (float)(value / maxHeightValue);
+    }
+    
+    /**
+     * @return a value between 0 and 1 representing the selected color value in the range [min to max]
+     */
+    public float getSelectedColorRatio() {
+        if (threads == null || functionNames == null) {
+            return -1;
+        }
+
+        if (settings.getSelections()[1] < 0 || settings.getSelections()[0] < 0) {
+            return -1;
+        }
+
+        Thread thread = (Thread) threads.get(settings.getSelections()[1]);
+        Function function = (Function) functions.get(settings.getSelections()[0]);
+        FunctionProfile fp = thread.getFunctionProfile(function);
+
+        if (fp == null) {
+            return -1;
+        }
+
+        int units = this.units;
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getColorMetric();
+        if (!ppMetric.isTimeMetric() || !ValueType.isTimeUnits(settings.getColorValue())) {
+            units = 0;
+        }
+
+        double value = settings.getColorValue().getValue(fp, settings.getColorMetric().getID());
+        return (float)(value / maxColorValue);
+    }
+    
+    public String getSelectedHeightValue() {
+        if (threads == null || functionNames == null) {
+            return "";
+        }
+
+        if (settings.getSelections()[1] < 0 || settings.getSelections()[0] < 0) {
+            return "";
+        }
+
+        Thread thread = (Thread) threads.get(settings.getSelections()[1]);
+        Function function = (Function) functions.get(settings.getSelections()[0]);
         FunctionProfile fp = thread.getFunctionProfile(function);
 
         if (fp == null) {
@@ -740,16 +1110,15 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         }
 
         int units = this.units;
-        ParaProfMetric ppMetric = ppTrial.getMetric(settings.getHeightMetricID());
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getHeightMetric();
         if (!ppMetric.isTimeMetric() || !ValueType.isTimeUnits(settings.getHeightValue())) {
             units = 0;
         }
 
-        return UtilFncs.getOutputString(units, settings.getHeightValue().getValue(fp, settings.getHeightMetricID()), 6).trim()
+        String retval = UtilFncs.getOutputString(units,
+                settings.getHeightValue().getValue(fp, settings.getHeightMetric().getID()), 6, ppMetric.isTimeDenominator()).trim()
                 + getUnitsString(units, settings.getHeightValue(), ppMetric);
-
-        //Double.toString(settings.getHeightValue().getValue(fp, settings.getHeightMetricID()));
-
+        return retval;
     }
 
     public String getSelectedColorValue() {
@@ -769,12 +1138,13 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         }
 
         int units = this.units;
-        ParaProfMetric ppMetric = ppTrial.getMetric(settings.getColorMetricID());
+        ParaProfMetric ppMetric = (ParaProfMetric) settings.getColorMetric();
         if (!ppMetric.isTimeMetric() || !ValueType.isTimeUnits(settings.getColorValue())) {
             units = 0;
         }
 
-        return UtilFncs.getOutputString(units, settings.getColorValue().getValue(fp, settings.getColorMetricID()), 6).trim()
+        return UtilFncs.getOutputString(units, settings.getColorValue().getValue(fp, settings.getColorMetric().getID()), 6,
+                ppMetric.isTimeDenominator()).trim()
                 + getUnitsString(units, settings.getColorValue(), ppMetric);
 
         //return Double.toString(settings.getColorValue().getValue(fp, settings.getColorMetricID()));
@@ -791,18 +1161,21 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
             Function[] scatterFunctions = settings.getScatterFunctions();
             ValueType[] scatterValueTypes = settings.getScatterValueTypes();
-            int[] scatterMetricIDs = settings.getScatterMetricIDs();
+            Metric[] scatterMetricIDs = settings.getScatterMetrics();
 
             List axisNames = new ArrayList();
             for (int f = 0; f < scatterFunctions.length; f++) {
                 if (scatterFunctions[f] != null) {
+                    String toDisplay = ParaProfUtils.getDisplayName(scatterFunctions[f]);
+                    if (toDisplay.length() > 30) {
+                        toDisplay = toDisplay.substring(0, 30) + "...";
+                    }
                     // e.g. "MPI_Recv()\n(Exclusive, Time)"
                     if (scatterValueTypes[f] == ValueType.NUMCALLS || scatterValueTypes[f] == ValueType.NUMSUBR) {
-                        axisNames.add(ParaProfUtils.getDisplayName(scatterFunctions[f]) + "\n(" + scatterValueTypes[f].toString()
-                                + ")");
+                        axisNames.add(toDisplay + "\n(" + scatterValueTypes[f].toString() + ")");
                     } else {
-                        axisNames.add(ParaProfUtils.getDisplayName(scatterFunctions[f]) + "\n(" + scatterValueTypes[f].toString()
-                                + ", " + ppTrial.getMetricName(scatterMetricIDs[f]) + ")");
+                        axisNames.add(toDisplay + "\n(" + scatterValueTypes[f].toString() + ", " + scatterMetricIDs[f].getName()
+                                + ")");
                     }
                 } else {
                     axisNames.add("none");
@@ -816,27 +1189,28 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
                     minScatterValues[i] = 0;
                 }
 
-                ParaProfMetric ppMetric = ppTrial.getMetric(scatterMetricIDs[i]);
+                ParaProfMetric ppMetric = (ParaProfMetric) scatterMetricIDs[i];
 
                 int units = scatterValueTypes[i].getUnits(this.units, ppMetric);
 
                 axisStrings[i] = new ArrayList();
-                axisStrings[i].add(UtilFncs.getOutputString(units, minScatterValues[i], 6).trim());
+                axisStrings[i].add(UtilFncs.getOutputString(units, minScatterValues[i], 6, ppMetric.isTimeDenominator()).trim());
                 axisStrings[i].add(UtilFncs.getOutputString(units,
-                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .25, 6).trim());
+                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .25, 6, ppMetric.isTimeDenominator()).trim());
                 axisStrings[i].add(UtilFncs.getOutputString(units,
-                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .50, 6).trim());
+                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .50, 6, ppMetric.isTimeDenominator()).trim());
                 axisStrings[i].add(UtilFncs.getOutputString(units,
-                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .75, 6).trim());
+                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]) * .75, 6, ppMetric.isTimeDenominator()).trim());
                 axisStrings[i].add(UtilFncs.getOutputString(units,
-                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]), 6).trim());
+                        minScatterValues[i] + (maxScatterValues[i] - minScatterValues[i]), 6, ppMetric.isTimeDenominator()).trim());
             }
 
-            ParaProfMetric ppMetric = ppTrial.getMetric(scatterMetricIDs[3]);
+            ParaProfMetric ppMetric = (ParaProfMetric) scatterMetricIDs[3];
             int units = scatterValueTypes[3].getUnits(this.units, ppMetric);
 
-            colorScale.setStrings(UtilFncs.getOutputString(units, minScatterValues[3], 6).trim(), UtilFncs.getOutputString(units,
-                    maxScatterValues[3], 6).trim(), (String) axisNames.get(3));
+            colorScale.setStrings(UtilFncs.getOutputString(units, minScatterValues[3], 6, ppMetric.isTimeDenominator()).trim(),
+                    UtilFncs.getOutputString(units, maxScatterValues[3], 6, ppMetric.isTimeDenominator()).trim(),
+                    (String) axisNames.get(3));
 
             scatterPlotAxes.setStrings((String) axisNames.get(0), (String) axisNames.get(1), (String) axisNames.get(2),
                     axisStrings[0], axisStrings[1], axisStrings[2]);
@@ -848,27 +1222,37 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
 
             int units;
 
-            ParaProfMetric ppMetric = ppTrial.getMetric(settings.getHeightMetricID());
+            ParaProfMetric ppMetric = (ParaProfMetric) settings.getHeightMetric();
             units = settings.getHeightValue().getUnits(this.units, ppMetric);
 
-            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.25, 6).trim());
-            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.50, 6).trim());
-            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.75, 6).trim());
-            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue, 6).trim());
+            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.25, 6, ppMetric.isTimeDenominator()).trim());
+            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.50, 6, ppMetric.isTimeDenominator()).trim());
+            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue * 0.75, 6, ppMetric.isTimeDenominator()).trim());
+            zStrings.add(UtilFncs.getOutputString(units, maxHeightValue, 6, ppMetric.isTimeDenominator()).trim());
 
             String zAxisLabel = settings.getHeightValue().getSuffix(units, ppMetric);
 
-            ppMetric = ppTrial.getMetric(settings.getColorMetricID());
+            ppMetric = (ParaProfMetric) settings.getColorMetric();
             units = settings.getColorValue().getUnits(this.units, ppMetric);
 
             String colorAxisLabel = settings.getColorValue().getSuffix(units, ppMetric);
 
-            colorScale.setStrings("0", UtilFncs.getOutputString(units, maxColorValue, 6).trim(), colorAxisLabel);
+            colorScale.setStrings("0", UtilFncs.getOutputString(units, maxColorValue, 6, ppMetric.isTimeDenominator()).trim(),
+                    colorAxisLabel);
 
             //String zAxisLabel = settings.getHeightValue().toString() + ", " + ppTrial.getMetricName(settings.getHeightMetricID());
             //String zAxisLabel = "";
 
-            fullDataPlotAxes.setStrings("Threads", "Functions", zAxisLabel, threadNames, functionNames, zStrings);
+            String threadLabel = "Thread";
+            if (ppTrial.getDataSource().getExecutionType() == DataSource.EXEC_TYPE_MPI) {
+                threadLabel = "MPI Rank";
+            }
+
+            if (ppTrial.getDataSource().getExecutionType() == DataSource.EXEC_TYPE_HYBRID) {
+                threadLabel = "MPI Rank, Thread";
+            }
+
+            fullDataPlotAxes.setStrings(threadLabel, "Function", zAxisLabel, threadNames, functionNames, zStrings);
         }
     }
 
@@ -917,4 +1301,42 @@ public class ThreeDeeWindow extends JFrame implements ActionListener, KeyListene
         sortLocalData();
     }
 
+    public JFrame getFrame() {
+        return this;
+    }
+
+    public void createNewCanvas() {
+        // TODO Auto-generated method stub
+        visCanvas = new VisCanvas(visRenderer);
+
+        visCanvas.addKeyListener(this);
+
+        JPanel panel = new JPanel() {
+            public Dimension getMinimumSize() {
+                return new Dimension(10, 10);
+            }
+        };
+
+        panel.addKeyListener(this);
+        panel.setLayout(new GridBagLayout());
+        GridBagConstraints gbc = new GridBagConstraints();
+
+        gbc.fill = GridBagConstraints.BOTH;
+        gbc.anchor = GridBagConstraints.WEST;
+        gbc.weightx = 1.0;
+        gbc.weighty = 1.0;
+        gbc.gridx = 0;
+        gbc.gridy = 0;
+        gbc.gridwidth = 1;
+        gbc.gridheight = 1;
+        panel.add(visCanvas.getActualCanvas(), gbc);
+        panel.setPreferredSize(new Dimension(5, 5));
+
+        jSplitPane.setLeftComponent(panel);
+    }
+
+    public Component getComponent() {
+        return this;
+    }
+
 }
