@@ -21,12 +21,13 @@ import edu.uoregon.tau.perfdmf.UserEvent;
 /**
  * The UserEventWindow shows one User Event over all threads.
  * 
- * <P>CVS $Id: UserEventWindow.java,v 1.28 2007/05/25 21:55:45 amorris Exp $</P>
+ * <P>CVS $Id: UserEventWindow.java,v 1.32 2009/09/10 00:13:49 amorris Exp $</P>
  * @author  Alan Morris, Robert Bell
- * @version $Revision: 1.28 $
+ * @version $Revision: 1.32 $
  * @see GlobalBarChartModel
  */
-public class UserEventWindow extends JFrame implements ActionListener, KeyListener, Observer, ChangeListener, ParaProfWindow, SearchableOwner {
+public class UserEventWindow extends JFrame implements ActionListener, KeyListener, Observer, ChangeListener, ParaProfWindow,
+        SearchableOwner {
 
     private ParaProfTrial ppTrial;
     private DataSorter dataSorter;
@@ -64,7 +65,7 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
         ppTrial.addObserver(this);
 
         this.dataSorter = new DataSorter(ppTrial);
-        
+
         int windowWidth = 750;
         int windowHeight = 650;
 
@@ -72,10 +73,10 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
         setLocation(WindowPlacer.getNewLocation(this, invoker));
 
         //Now set the title.
-        this.setTitle("TAU: ParaProf: User Event Window: " + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
+        this.setTitle("TAU: ParaProf: User Event Window: "
+                + ppTrial.getTrialIdentifier(ParaProf.preferences.getShowPathTitleInReverse()));
         ParaProfUtils.setFrameIcon(this);
 
-
         //Add some window listener code
         addWindowListener(new java.awt.event.WindowAdapter() {
             public void windowClosing(java.awt.event.WindowEvent evt) {
@@ -90,7 +91,6 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
 
         model = new UserEventThreadBarChartModel(this, dataSorter, thread);
 
-
         panel = new BarChartPanel(model, null);
 
         panel.getBarChart().setBarLength(barLengthSlider.getValue());
@@ -159,13 +159,11 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
 
         this.addKeyListener(this);
 
-
         panel = new BarChartPanel(model, null);
 
         panel.getBarChart().setBarLength(barLengthSlider.getValue());
         setupMenus();
 
-
         //Setting up the layout system for the main window.
         gbl = new GridBagLayout();
         getContentPane().setLayout(gbl);
@@ -233,7 +231,7 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
             sortByName.addActionListener(this);
             optionsMenu.add(sortByName);
         }
-        
+
         //Set the value type options.
         subMenu = new JMenu("Select Value Type");
         group = new ButtonGroup();
@@ -346,11 +344,9 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
         }
     }
 
-    public void menuDeselected(MenuEvent evt) {
-    }
+    public void menuDeselected(MenuEvent evt) {}
 
-    public void menuCanceled(MenuEvent evt) {
-    }
+    public void menuCanceled(MenuEvent evt) {}
 
     public void update(Observable o, Object arg) {
         String tmpString = (String) arg;
@@ -368,8 +364,9 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
     public void help(boolean display) {
         //Show the ParaProf help window.
         ParaProf.getHelpWindow().clearText();
-        if (display)
-            ParaProf.getHelpWindow().show();
+        if (display) {
+            ParaProf.getHelpWindow().setVisible(true);
+        }
         ParaProf.getHelpWindow().writeText("This is the userevent data window for:");
         ParaProf.getHelpWindow().writeText(userEvent.getName());
         ParaProf.getHelpWindow().writeText("");
@@ -388,7 +385,7 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
 
     public void sortLocalData() {
         dataSorter.setDescendingOrder(descendingOrder.isSelected());
-        
+
         if (sortByNCT != null && sortByNCT.isSelected()) {
             dataSorter.setSortType(SortType.NCT);
         } else {
@@ -399,7 +396,6 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
             dataSorter.setSortType(SortType.NAME);
         }
 
-        
         dataSorter.setUserEventValueType(userEventValueType);
         model.reloadData();
     }
@@ -450,7 +446,7 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
         if (userEvent != null) {
             return "Name: " + userEvent.getName() + "\n" + "Value Type: " + userEventValueType.toString() + "\n";
         } else {
-            return "Thread: " + ParaProfUtils.getThreadIdentifier(thread) + "\n" + "Value Type: " + userEventValueType.toString()
+            return "Thread: " + ParaProfUtils.getThreadLabel(thread) + "\n" + "Value Type: " + userEventValueType.toString()
                     + "\n";
         }
     }
@@ -531,7 +527,7 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
         showFindPanelBox.setSelected(show);
         validate();
     }
-    
+
     public void keyPressed(KeyEvent e) {
         if (e.isControlDown() && e.getKeyCode() == KeyEvent.VK_F) {
             showSearchPanel(true);
@@ -539,13 +535,16 @@ public class UserEventWindow extends JFrame implements ActionListener, KeyListen
     }
 
     public void keyReleased(KeyEvent e) {
-        // TODO Auto-generated method stub
-        
+    // TODO Auto-generated method stub
+
     }
 
     public void keyTyped(KeyEvent e) {
-        // TODO Auto-generated method stub
-        
+    // TODO Auto-generated method stub
+
     }
 
+    public JFrame getFrame() {
+        return this;
+    }
 }
\ No newline at end of file
