@@ -1,6 +1,8 @@
 package edu.uoregon.tau.paraprof.barchart;
 
 import java.awt.*;
+import java.awt.event.ComponentEvent;
+import java.awt.event.ComponentListener;
 import java.awt.event.MouseEvent;
 import java.awt.print.PageFormat;
 import java.awt.print.Printable;
@@ -8,18 +10,18 @@ import java.awt.print.PrinterException;
 
 import javax.swing.*;
 
+import edu.uoregon.tau.common.ImageExport;
+import edu.uoregon.tau.paraprof.DataSorter;
 import edu.uoregon.tau.paraprof.ParaProf;
-import edu.uoregon.tau.paraprof.ParaProfErrorDialog;
 import edu.uoregon.tau.paraprof.ParaProfUtils;
-import edu.uoregon.tau.paraprof.interfaces.ImageExport;
 import edu.uoregon.tau.paraprof.interfaces.ScrollBarController;
 
 /**
  * Adds scroll ability, and handles image export/printing with header support.
  * 
- * <P>CVS $Id: BarChartPanel.java,v 1.2 2005/10/18 22:50:34 amorris Exp $</P>
+ * <P>CVS $Id: BarChartPanel.java,v 1.9 2009/11/05 09:43:32 khuck Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.2 $
+ * @version $Revision: 1.9 $
  */
 public class BarChartPanel extends JScrollPane implements Printable, ImageExport, ScrollBarController {
 
@@ -27,11 +29,31 @@ public class BarChartPanel extends JScrollPane implements Printable, ImageExport
 
     //BarChartHeader barChartHeader;
 
+    public BarChartPanel(BarChartModel barChartModel) {
+        this(barChartModel, null);
+    }
+    
     public BarChartPanel(BarChartModel barChartModel, JComponent header) {
         barChart = new BarChart(barChartModel, this);
         this.setViewportView(barChart);
         this.setColumnHeaderView(header);
         this.getVerticalScrollBar().setUnitIncrement(35);
+            
+        addComponentListener(new ComponentListener() {
+
+            public void componentHidden(ComponentEvent e) {
+            }
+
+            public void componentMoved(ComponentEvent e) {
+            }
+
+            public void componentResized(ComponentEvent e) {
+                barChart.sizeChanged();
+            }
+
+            public void componentShown(ComponentEvent e) {
+            }
+        });
     }
 
    
@@ -50,7 +72,7 @@ public class BarChartPanel extends JScrollPane implements Printable, ImageExport
             return Printable.PAGE_EXISTS;
 
         } catch (Exception e) {
-            new ParaProfErrorDialog(e);
+            ParaProfUtils.handleException(e);
             return NO_SUCH_PAGE;
         }
 
@@ -76,6 +98,7 @@ public class BarChartPanel extends JScrollPane implements Printable, ImageExport
         if (header) {
             Dimension d = this.getColumnHeader().getSize();
 
+            
             Dimension chart;
             if (fullScreen) {
                 chart = barChart.getSize();
@@ -110,7 +133,7 @@ public class BarChartPanel extends JScrollPane implements Printable, ImageExport
     
     public static void main(String[] args) {
         final ParaProf paraProf = new ParaProf();
-        paraProf.startSystem();
+        ParaProf.initialize();
 
         
         JFrame frame = new JFrame("Bar Chart Test");
@@ -181,6 +204,10 @@ public class BarChartPanel extends JScrollPane implements Printable, ImageExport
                 // TODO Auto-generated method stub
                 
             }
+            
+            public DataSorter getDataSorter() {
+            	return null;
+            }
         };
         
         
