@@ -7,6 +7,7 @@ import java.util.List;
 import javax.swing.JComponent;
 import javax.swing.JPopupMenu;
 
+import edu.uoregon.tau.common.Common;
 import edu.uoregon.tau.paraprof.*;
 import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.Thread;
@@ -14,9 +15,9 @@ import edu.uoregon.tau.perfdmf.Thread;
 /**
  * A BarChartModel for doing the GlobalDataWindow
  * 
- * <P>CVS $Id: GlobalBarChartModel.java,v 1.11 2007/05/04 01:44:34 amorris Exp $</P>
+ * <P>CVS $Id: GlobalBarChartModel.java,v 1.17 2009/11/05 09:43:32 khuck Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.11 $
+ * @version $Revision: 1.17 $
  */
 
 public class GlobalBarChartModel extends AbstractBarChartModel {
@@ -58,15 +59,7 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
     }
 
     private static String getName(Thread thread) {
-        if (thread.getNodeID() == -1) {
-            return "mean";
-        } else if (thread.getNodeID() == -2) {
-            return "total";
-        } else if (thread.getNodeID() == -3) {
-            return "std. dev.";
-        } else {
-            return "n,c,t " + (thread.getNodeID()) + "," + (thread.getContextID()) + "," + (thread.getThreadID());
-        }
+        return ParaProfUtils.getThreadLabel(thread);
     }
 
     //    public String getRowLabel(int row) {
@@ -109,7 +102,7 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
         if (fp == null) {
             return -1;
         } else {
-            return dataSorter.getValueType().getValue(fp, dataSorter.getSelectedMetricID(), dataSorter.getSelectedSnapshot());
+            return dataSorter.getValueType().getValue(fp, dataSorter.getSelectedMetric(), dataSorter.getSelectedSnapshot());
         }
     }
 
@@ -224,16 +217,23 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
             name = getDisplayName(fp);
         }
 
-        String metricName = ppTrial.getMetricName(dataSorter.getSelectedMetricID());
+        String metricName = dataSorter.getSelectedMetric().getName();
 
-        int metricID = dataSorter.getSelectedMetricID();
+        Metric metric = dataSorter.getSelectedMetric();
 
-        String exclusive = "<br>Exclusive " + metricName + ": " + fp.getExclusive(metricID);
-        String inclusive = "<br>Inclusive " + metricName + ": " + fp.getInclusive(metricID);
+        String unitsString = UtilFncs.getUnitsString(window.units(), dataSorter.isTimeMetric(), false);
+
+        String exclusiveValue = UtilFncs.getOutputString(window.units(), fp.getExclusive(metric.getID()),
+                ParaProf.defaultNumberPrecision, metric.isTimeDenominator());
+        String inclusiveValue = UtilFncs.getOutputString(window.units(), fp.getInclusive(metric.getID()),
+                ParaProf.defaultNumberPrecision, metric.isTimeDenominator());
+
+        String exclusive = "<br>Exclusive " + metricName + ": " + exclusiveValue + " " + unitsString;
+        String inclusive = "<br>Inclusive " + metricName + ": " + inclusiveValue + " " + unitsString;
         String calls = "<br>Calls: " + fp.getNumCalls();
         String subr = "<br>SubCalls: " + fp.getNumSubr();
 
-        return "<html>" + name + exclusive + inclusive + calls + subr + "</html>";
+        return "<html>" + Common.HTMLEntityEncode(name) + exclusive + inclusive + calls + subr + "</html>";
 
     }
 
@@ -318,4 +318,8 @@ public class GlobalBarChartModel extends AbstractBarChartModel {
         return theThreads;
     }
 
+
+	public DataSorter getDataSorter() {
+		return dataSorter;
+	}
 }
