@@ -13,9 +13,9 @@ import edu.uoregon.tau.perfdmf.UtilFncs;
 /**
  * A BarChartModel for displaying one User Event over all threads.
  * 
- * <P>CVS $Id: UserEventBarChartModel.java,v 1.3 2007/05/16 23:34:38 amorris Exp $</P>
+ * <P>CVS $Id: UserEventBarChartModel.java,v 1.7 2009/11/05 09:43:33 khuck Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.3 $
+ * @version $Revision: 1.7 $
  */
 public class UserEventBarChartModel extends AbstractBarChartModel {
 
@@ -38,7 +38,7 @@ public class UserEventBarChartModel extends AbstractBarChartModel {
 
     public String getRowLabel(int row) {
         PPUserEventProfile ppUserEventProfile = (PPUserEventProfile) list.get(row);
-        return ParaProfUtils.getThreadIdentifier(ppUserEventProfile.getThread());
+        return ParaProfUtils.getThreadLabel(ppUserEventProfile.getThread());
     }
 
     public String getValueLabel(int row, int subIndex) {
@@ -46,7 +46,7 @@ public class UserEventBarChartModel extends AbstractBarChartModel {
 
         double value = window.getValueType().getValue(ppUserEventProfile.getUserEventProfile(),dataSorter.getSelectedSnapshot());
 
-        return UtilFncs.getOutputString(0, value, ParaProf.defaultNumberPrecision);
+        return UtilFncs.getOutputString(0, value, ParaProf.defaultNumberPrecision, false);
     }
 
     public double getValue(int row, int subIndex) {
@@ -80,7 +80,7 @@ public class UserEventBarChartModel extends AbstractBarChartModel {
         } else {
             FunctionBarChartWindow threadDataWindow = new FunctionBarChartWindow(window.getPpTrial(),
                     ppUserEventProfile.getThread(), null, owner);
-            threadDataWindow.show();
+            threadDataWindow.setVisible(true);
         }
 
     }
@@ -102,4 +102,8 @@ public class UserEventBarChartModel extends AbstractBarChartModel {
         fireModelChanged();
     }
 
+
+	public DataSorter getDataSorter() {
+		return dataSorter;
+	}
 }
