@@ -8,34 +8,34 @@ import edu.uoregon.tau.paraprof.DataSorter;
 import edu.uoregon.tau.paraprof.PPFunctionProfile;
 import edu.uoregon.tau.paraprof.ParaProfTrial;
 import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.perfdmf.Thread;
 
 /**
  * Data model for treetable using callpaths
  *    
  * TODO : ...
  *
- * <P>CVS $Id: CallPathModel.java,v 1.6 2007/03/02 20:10:54 amorris Exp $</P>
+ * <P>CVS $Id: CallPathModel.java,v 1.11 2009/09/10 00:13:52 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.6 $
+ * @version $Revision: 1.11 $
  */
 public class CallPathModel extends AbstractTreeTableModel {
 
     private List roots;
-    private edu.uoregon.tau.perfdmf.Thread thread;
+    private Thread thread;
     private DataSource dataSource;
     private ParaProfTrial ppTrial;
     private double[] maxValues;
 
     private int sortColumn;
-    private int colorMetric;
     private boolean sortAscending;
+    private int colorMetric;
 
     private boolean reversedCallPaths = true;
 
     private TreeTableWindow window;
 
-    public CallPathModel(TreeTableWindow window, ParaProfTrial ppTrial, edu.uoregon.tau.perfdmf.Thread thread,
-            boolean reversedCallPaths) {
+    public CallPathModel(TreeTableWindow window, ParaProfTrial ppTrial, Thread thread, boolean reversedCallPaths) {
         super(null);
         this.window = window;
         root = new TreeTableNode(null, this, "root");
@@ -122,22 +122,14 @@ public class CallPathModel extends AbstractTreeTableModel {
     public void computeMaximum() {
         int numMetrics = window.getPPTrial().getNumberOfMetrics();
         maxValues = new double[numMetrics];
-        for (Iterator it = roots.iterator(); it.hasNext();) {
-            TreeTableNode treeTableNode = (TreeTableNode) it.next();
 
-            if (treeTableNode.getFunctionProfile() != null) {
+        for (int i = 0; i < numMetrics; i++) {
+            // there are two ways to do this (cube brings up a window to ask you which way you 
+            // want to compute the max value for the color)
 
-                for (int i = 0; i < numMetrics; i++) {
-                    // there are two ways to do this (cube brings up a window to ask you which way you 
-                    // want to compute the max value for the color)
-
-                    //maxValue[i] = Math.max(maxValue[i], fp.getInclusive(i));
-                    maxValues[i] += treeTableNode.getFunctionProfile().getInclusive(i);
-
-                }
-            }
+            //maxValue[i] = Math.max(maxValue[i], fp.getInclusive(i));
+            maxValues[i] += thread.getMaxInclusive(i, ppTrial.getSelectedSnapshot());
         }
-
     }
 
     public int getColumnCount() {
@@ -194,7 +186,7 @@ public class CallPathModel extends AbstractTreeTableModel {
         return node.getChildren().get(index);
     }
 
-    public edu.uoregon.tau.perfdmf.Thread getThread() {
+    public Thread getThread() {
         return thread;
     }
 
@@ -217,8 +209,8 @@ public class CallPathModel extends AbstractTreeTableModel {
 
         Collections.sort(roots);
         for (Iterator it = roots.iterator(); it.hasNext();) {
-            TreeTableNode treeTableNode = (TreeTableNode) it.next();
-            treeTableNode.sortChildren();
+            TreeTableNode node = (TreeTableNode) it.next();
+            node.sortChildren();
         }
 
     }
