@@ -4,15 +4,8 @@ import jargs.gnu.CmdLineParser;
 
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import java.io.File;
-import java.io.FileInputStream;
-import java.io.FileNotFoundException;
-import java.io.FileOutputStream;
-import java.io.IOException;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-import java.net.URL;
-import java.net.URLDecoder;
+import java.io.*;
+import java.text.DecimalFormat;
 import java.util.ArrayList;
 import java.util.Iterator;
 import java.util.List;
@@ -23,28 +16,24 @@ import edu.uoregon.tau.common.TauScripter;
 import edu.uoregon.tau.paraprof.interfaces.EclipseHandler;
 import edu.uoregon.tau.paraprof.script.ParaProfScript;
 import edu.uoregon.tau.paraprof.sourceview.SourceManager;
-import edu.uoregon.tau.perfdmf.DataSource;
-import edu.uoregon.tau.perfdmf.DataSourceExport;
-import edu.uoregon.tau.perfdmf.FileList;
-import edu.uoregon.tau.perfdmf.UtilFncs;
+import edu.uoregon.tau.perfdmf.*;
 
 /**
  * ParaProf This is the 'main' for paraprof
  * 
  * <P>
- * CVS $Id: ParaProf.java,v 1.18 2007/03/26 18:19:39 amorris Exp $
+ * CVS $Id: ParaProf.java,v 1.35 2009/11/09 21:34:30 wspear Exp $
  * </P>
  * 
  * @author Robert Bell, Alan Morris
- * @version $Revision: 1.18 $
+ * @version $Revision: 1.35 $
  */
 public class ParaProf implements ActionListener {
 
     // This class handles uncaught throwables on the AWT-EventQueue thread
     static public class XThrowableHandler {
 
-        public XThrowableHandler() {
-        }
+        public XThrowableHandler() {}
 
         public void handle(Throwable t) throws Throwable {
             if (t instanceof Exception) {
@@ -55,7 +44,7 @@ public class ParaProf implements ActionListener {
         }
     }
 
-    private final static String VERSION = "Mon Mar 26 11:19:01 PDT 2007";
+    private final static String VERSION = "Mon Nov  9 13:33:15 PST 2009";
 
     public static int defaultNumberPrecision = 6;
 
@@ -71,13 +60,11 @@ public class ParaProf implements ActionListener {
     public static Runtime runtime;
     private static int numWindowsOpen = 0;
 
-    //Command line options related.
-    private static int fileType = 0; //0:profile, 1:pprof, 2:dynaprof, 3:mpip, 4:hpmtoolkit, 5:gprof, 6:psrun, 7:ppk, 8:cube
+    private static int fileType = DataSource.TAUPROFILE;
     private static File sourceFiles[] = new File[0];
     private static boolean fixNames = false;
     private static boolean monitorProfiles;
-    private static String args[];
-    //End - Command line options related.
+    private static String configFile;
 
     public static boolean demoMode;
     public static boolean usePathNameInTrial = false;
@@ -85,11 +72,13 @@ public class ParaProf implements ActionListener {
     public static boolean JNLP = false;
     public static List scripts = new ArrayList();
     public static String scriptFile;
-    
+
     public static boolean insideEclipse;
     public static EclipseHandler eclipseHandler;
     public static SourceManager directoryManager;
-    
+    public static String jarLocation;
+    public static String schemaLocation;
+
     // static initializer block
     static {
         ParaProf.runtime = Runtime.getRuntime();
@@ -101,23 +90,22 @@ public class ParaProf implements ActionListener {
         }
         return helpWindow;
     }
-    
+
     public static SourceManager getDirectoryManager() {
-    	if (directoryManager == null) {
-    		directoryManager = new SourceManager(ParaProf.preferences.getSourceLocations());
-    	}
-    	return directoryManager;
+        if (directoryManager == null) {
+            directoryManager = new SourceManager(ParaProf.preferences.getSourceLocations());
+        }
+        return directoryManager;
     }
-    
+
     public static void registerScript(ParaProfScript pps) {
         scripts.add(pps);
     }
 
-    public ParaProf() {
-    }
+    public ParaProf() {}
 
     private static void usage() {
-        System.err.println("Usage: paraprof [--pack <file>] [--dump] [-p] [-m] [-i] [-f <filetype>] <files/directory>\n\n"
+        System.err.println("Usage: paraprof [--pack <file>] [--dump] [--dumprank <rank>] [-p] [-m] [-i] [-f <filetype>] <files/directory>\n\n"
                 + "try `paraprof --help` for more information");
     }
 
@@ -125,15 +113,18 @@ public class ParaProf implements ActionListener {
         System.err.println("Usage: paraprof [options] <files/directory> \n\n" + "Options:\n\n"
                 + "  -f, --filetype <filetype>       Specify type of performance data, options are:\n"
                 + "                                    profiles (default), pprof, dynaprof, mpip,\n"
-                + "                                    gprof, psrun, hpm, packed, cube, hpc\n" + "\n"
-                + "  -h, --help                      Display this help message\n"
-                + "  -p                              Use `pprof` to compute derived data\n"
-                + "  -i, --fixnames                  Use the fixnames option for gprof\n" + "\n"
+                + "                                    gprof, psrun, hpm, packed, cube, hpc, ompp\n"
+                + "                                    snap, perixml, gptl, ipm\n"
+                + "  -h, --help                      Display this help message\n" + "\n"
+                + "The following options will run only from the console (no GUI will launch):\n" + "\n"
+                + "  --merge <file.gz>               Merges snapshot profiles\n"
                 + "  --pack <file>                   Pack the data into packed (.ppk) format\n"
-                + "                                    (does not launch ParaProf GUI)\n"
                 + "  --dump                          Dump profile data to TAU profile format\n"
-                + "                                    (does not launch ParaProf GUI)\n" + "\n" + "Notes:\n"
-                + "  -m, --monitor                   Perform runtime monitoring of profile data\n" + "\n"
+                + "  --dumprank <rank>               Dump profile data for <rank> to TAU profile format\n"
+                + "  --overwrite                     Allow overwriting of profiles\n"
+                + "  -o, --oss                       Print profile data in OSS style text output\n"
+                + "  -s, --summary                   Print only summary statistics\n"
+                + "                                    (only applies to OSS output)\n" + "\n" + "Notes:\n"
                 + "  For the TAU profiles type, you can specify either a specific set of profile\n"
                 + "files on the commandline, or you can specify a directory (by default the current\n"
                 + "directory).  The specified directory will be searched for profile.*.*.* files,\n"
@@ -167,7 +158,7 @@ public class ParaProf implements ActionListener {
 
         try {
 
-            if (fileType == 7) {
+            if (fileType == DataSource.PPK) {
                 for (int i = 0; i < sourceFiles.length; i++) {
                     File files[] = new File[1];
                     files[0] = sourceFiles[i];
@@ -206,7 +197,6 @@ public class ParaProf implements ActionListener {
 
                     ObjectInputStream inSavedPreferencesOIS = new ObjectInputStream(savedPreferenceFIS);
                     ParaProf.preferences = (Preferences) inSavedPreferencesOIS.readObject();
-                    ParaProf.preferences.setLoaded(true);
                     colorChooser = new ColorChooser(ParaProf.preferences);
                 } catch (Exception e) {
                     if (e instanceof FileNotFoundException) {
@@ -224,15 +214,11 @@ public class ParaProf implements ActionListener {
                 // try to load perfdmf.cfg.
                 File perfDMFcfg = new File(ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
                 if (perfDMFcfg.exists()) {
-                    //System.out.println("Found db configuration file: "
-                    //        + ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
                     ParaProf.preferences.setDatabaseConfigurationFile(ParaProf.paraProfHomeDirectory.getPath() + "/perfdmf.cfg");
-                } else {
-                    System.out.println("Did not find db configuration file ... load manually");
                 }
 
             } else {
-                System.out.println("Did not find ParaProf home directory ... creating ...");
+                System.out.println("Did not find ParaProf home directory... creating...");
                 paraProfHomeDirectory.mkdir();
                 System.out.println("Done creating ParaProf home directory!");
             }
@@ -241,9 +227,10 @@ public class ParaProf implements ActionListener {
             //URL url = ParaProf.class.getResource("/perfdmf.cfg");
             //throw new ParaProfException("URL = " + url);
 
-            URL url = ParaProf.class.getResource("/perfdmf.cfg");
-            String path = URLDecoder.decode(url.getPath());
-            ParaProf.preferences.setDatabaseConfigurationFile(path);
+            //            URL url = ParaProf.class.getResource("/perfdmf.cfg");
+            //            String path = URLDecoder.decode(url.getPath());
+            //            ParaProf.preferences.setDatabaseConfigurationFile(path);
+            preferences = new Preferences();
         }
 
         if (colorChooser == null) {
@@ -263,19 +250,21 @@ public class ParaProf implements ActionListener {
             // running as Java Web Start without permission
         }
 
-
         // Initialize, but do not show the manager window
-        ParaProf.paraProfManagerWindow = new ParaProfManagerWindow();
+        //System.out.println("creating Manager window with: " + configFile);
+        ParaProf.paraProfManagerWindow = new ParaProfManagerWindow(configFile);
     }
 
     public static void loadScripts() {
-        ParaProf.scripts.clear();
-        ParaProf.scriptFile = System.getProperty("user.home") + "/.ParaProf/ParaProf.py";
-        if (new File(scriptFile).exists()) {
-            try {
-                TauScripter.execfile(System.getProperty("user.home") + "/.ParaProf/ParaProf.py");
-            } catch (Exception e) {
-                new ParaProfErrorDialog("Exception while executing script: ", e);
+        if (ParaProf.JNLP == false) {
+            ParaProf.scripts.clear();
+            ParaProf.scriptFile = System.getProperty("user.home") + "/.ParaProf/ParaProf.py";
+            if (new File(scriptFile).exists()) {
+                try {
+                    TauScripter.execfile(System.getProperty("user.home") + "/.ParaProf/ParaProf.py");
+                } catch (Exception e) {
+                    new ParaProfErrorDialog("Exception while executing script: ", e);
+                }
             }
         }
     }
@@ -290,9 +279,18 @@ public class ParaProf implements ActionListener {
     }
 
     public static String getInfoString() {
-        long memUsage = (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024;
+        //long memUsage = (Runtime.getRuntime().totalMemory() - Runtime.getRuntime().freeMemory()) / 1024;
 
-        return new String("ParaProf\n" + getVersionString() + "\nJVM Heap Size: " + memUsage + "kb\n" + "http://www.cs.uoregon.edu/research/tau\n");
+        DecimalFormat f = new DecimalFormat("#.## MB");
+
+        String memUsage = "Free: " + f.format(java.lang.Runtime.getRuntime().freeMemory() / 1000000.0) + "\nTotal: "
+                + f.format(java.lang.Runtime.getRuntime().totalMemory() / 1000000.0) + "\nMax: "
+                + f.format(java.lang.Runtime.getRuntime().maxMemory() / 1000000.0);
+
+        String message = "ParaProf\n" + getVersionString() + "\nJVM Memory stats:\n" + memUsage + "\n"
+                + "http://www.cs.uoregon.edu/research/tau\n";
+
+        return message;
     }
 
     public static String getVersionString() {
@@ -307,7 +305,6 @@ public class ParaProf implements ActionListener {
         //Create ObjectInputStream and try to read it in.
         ObjectInputStream inSavedPreferencesOIS = new ObjectInputStream(savedPreferenceFIS);
         ParaProf.preferences = (Preferences) inSavedPreferencesOIS.readObject();
-        ParaProf.preferences.setLoaded(true);
         colorChooser = new ColorChooser(ParaProf.preferences);
 
         ParaProf.colorMap.setMap(ParaProf.preferences.getAssignedColors());
@@ -356,7 +353,6 @@ public class ParaProf implements ActionListener {
 
     // Main entry point
     static public void main(String[] args) {
-        ParaProf.args = args;
 
         // Set the tooltip delay to 20 seconds
         ToolTipManager.sharedInstance().setDismissDelay(20000);
@@ -367,10 +363,17 @@ public class ParaProf implements ActionListener {
         CmdLineParser.Option configfileOpt = parser.addStringOption('g', "configfile");
         CmdLineParser.Option typeOpt = parser.addStringOption('f', "filetype");
         CmdLineParser.Option fixOpt = parser.addBooleanOption('i', "fixnames");
+        CmdLineParser.Option mergeOpt = parser.addStringOption('a', "merge");
         CmdLineParser.Option packOpt = parser.addStringOption('a', "pack");
         CmdLineParser.Option unpackOpt = parser.addBooleanOption('u', "dump");
+        CmdLineParser.Option unpackRankOpt = parser.addStringOption('r', "dumprank");
+        CmdLineParser.Option ossOpt = parser.addBooleanOption('o', "oss");
+        CmdLineParser.Option overwriteOpt = parser.addBooleanOption('w', "overwrite");
+        CmdLineParser.Option summaryOpt = parser.addBooleanOption('s', "summary");
         CmdLineParser.Option monitorOpt = parser.addBooleanOption('m', "monitor");
         CmdLineParser.Option demoOpt = parser.addBooleanOption('z', "demo");
+        CmdLineParser.Option jarLocationOpt = parser.addStringOption('j', "jardir");
+        CmdLineParser.Option schemaLocationOpt = parser.addStringOption('c', "schemadir");
 
         try {
             parser.parse(args);
@@ -380,16 +383,30 @@ public class ParaProf implements ActionListener {
             System.exit(-1);
         }
 
+        configFile = (String) parser.getOptionValue(configfileOpt);
         Boolean help = (Boolean) parser.getOptionValue(helpOpt);
         String fileTypeString = (String) parser.getOptionValue(typeOpt);
         Boolean fixNames = (Boolean) parser.getOptionValue(fixOpt);
+        String merge = (String) parser.getOptionValue(mergeOpt);
         String pack = (String) parser.getOptionValue(packOpt);
         Boolean unpack = (Boolean) parser.getOptionValue(unpackOpt);
+        Boolean overwrite = (Boolean) parser.getOptionValue(overwriteOpt);
+        String unpackrank = (String) parser.getOptionValue(unpackRankOpt);
+        if (unpackrank != null) {
+            unpack = Boolean.TRUE;
+        }
+        Boolean oss = (Boolean) parser.getOptionValue(ossOpt);
+        Boolean summary = (Boolean) parser.getOptionValue(summaryOpt);
         Boolean monitor = (Boolean) parser.getOptionValue(monitorOpt);
         Boolean demo = (Boolean) parser.getOptionValue(demoOpt);
+        ParaProf.jarLocation = (String) parser.getOptionValue(jarLocationOpt);
+        ParaProf.schemaLocation = (String) parser.getOptionValue(schemaLocationOpt);
 
         demoMode = demo != null && demo.booleanValue();
-
+        if (configFile != "") {
+            //System.out.println("commandline db config: " + configFile);
+            ParaProf.preferences.setDatabaseConfigurationFile(configFile);
+        }
         if (monitor != null) {
             monitorProfiles = monitor.booleanValue();
         }
@@ -415,28 +432,39 @@ public class ParaProf implements ActionListener {
 
         if (fileTypeString != null) {
             if (fileTypeString.equals("profiles")) {
-                ParaProf.fileType = 0;
+                ParaProf.fileType = DataSource.TAUPROFILE;
             } else if (fileTypeString.equals("pprof")) {
-                ParaProf.fileType = 1;
+                ParaProf.fileType = DataSource.PPROF;
             } else if (fileTypeString.equals("dynaprof")) {
-                ParaProf.fileType = 2;
+                ParaProf.fileType = DataSource.DYNAPROF;
             } else if (fileTypeString.equals("mpip")) {
-                ParaProf.fileType = 3;
+                ParaProf.fileType = DataSource.MPIP;
             } else if (fileTypeString.equals("hpm")) {
-                ParaProf.fileType = 4;
+                ParaProf.fileType = DataSource.HPM;
             } else if (fileTypeString.equals("gprof")) {
-                ParaProf.fileType = 5;
+                ParaProf.fileType = DataSource.GPROF;
             } else if (fileTypeString.equals("psrun")) {
-                ParaProf.fileType = 6;
+                ParaProf.fileType = DataSource.PSRUN;
             } else if (fileTypeString.equals("packed")) {
-                ParaProf.fileType = 7;
+                ParaProf.fileType = DataSource.PPK;
             } else if (fileTypeString.equals("cube")) {
-                ParaProf.fileType = 8;
+                ParaProf.fileType = DataSource.CUBE;
             } else if (fileTypeString.equals("hpc")) {
-                ParaProf.fileType = 9;
-// Disabled for 2.16.3 release
-//            } else if (fileTypeString.equals("snap")) {
-//                ParaProf.fileType = 10;
+                ParaProf.fileType = DataSource.HPCTOOLKIT;
+            } else if (fileTypeString.equals("snapshot")) {
+                ParaProf.fileType = DataSource.SNAP;
+            } else if (fileTypeString.equals("snap")) {
+                ParaProf.fileType = DataSource.SNAP;
+            } else if (fileTypeString.equals("ompp")) {
+                ParaProf.fileType = DataSource.OMPP;
+            } else if (fileTypeString.equals("perixml")) {
+                ParaProf.fileType = DataSource.PERIXML;
+            } else if (fileTypeString.equals("gptl")) {
+                ParaProf.fileType = DataSource.GPTL;
+            } else if (fileTypeString.equals("paraver")) {
+                ParaProf.fileType = DataSource.PARAVER;
+            } else if (fileTypeString.equals("ipm")) {
+                ParaProf.fileType = DataSource.IPM;
             } else {
                 System.err.println("Please enter a valid file type.");
                 ParaProf.usage();
@@ -444,19 +472,57 @@ public class ParaProf implements ActionListener {
             }
         } else {
             if (sourceFilenames.length >= 1) {
-                String filename = sourceFiles[0].getName();
-                if (filename.toLowerCase().endsWith(".ppk")) {
-                    ParaProf.fileType = 7;
-                }
-                if (filename.toLowerCase().endsWith(".cube")) {
-                    ParaProf.fileType = 8;
-                }
-                if (filename.toLowerCase().endsWith(".mpip")) {
-                    ParaProf.fileType = 3;
-                }
+                ParaProf.fileType = UtilFncs.identifyData(sourceFiles[0]);
             }
         }
 
+        if (merge != null) {
+            try {
+
+                if (sourceFiles.length == 0) {
+                    FileList fl = new FileList();
+                    sourceFiles = fl.helperFindSnapshots(System.getProperty("user.dir"));
+                }
+
+                if (sourceFiles.length == 0) {
+                    System.err.println("No snapshots found\n");
+                    System.exit(-1);
+                }
+
+                for (int i = 0; i < sourceFiles.length; i++) {
+                    if (DataSource.SNAP != UtilFncs.identifyData(sourceFiles[i])) {
+                        System.err.println("Error: File '" + sourceFiles[i] + "' is not a snapshot profile\n");
+                        System.exit(-1);
+                    }
+                }
+
+                // merge and write
+                UtilFncs.mergeSnapshots(sourceFiles, merge);
+
+            } catch (Exception e) {
+                e.printStackTrace();
+            }
+            System.exit(0);
+        }
+
+        if (oss != null) {
+            try {
+
+                boolean doSummary = false;
+                if (summary != null) {
+                    doSummary = true;
+                }
+
+                DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
+                dataSource.load();
+                OSSWriter.writeOSS(dataSource, doSummary);
+
+            } catch (Exception e) {
+                e.printStackTrace();
+            }
+            System.exit(0);
+        }
+
         if (pack != null) {
             try {
 
@@ -477,16 +543,25 @@ public class ParaProf implements ActionListener {
 
                 FileList fl = new FileList();
                 List v = fl.helperFindProfiles(".");
-                if (v.size() != 0) {
-                    System.err.println("Error: profiles found in current directory, please remove first");
-                    return;
+
+                if (overwrite == null) {
+                    if (v.size() != 0) {
+                        System.err.println("Error: profiles found in current directory, please remove first");
+                        return;
+                    }
                 }
 
                 DataSource dataSource = UtilFncs.initializeDataSource(sourceFiles, fileType, ParaProf.fixNames);
                 System.out.println("Loading data...");
                 dataSource.load();
                 System.out.println("Creating TAU Profile data...");
-                DataSourceExport.writeProfiles(dataSource, new File("."));
+                if (unpackrank != null) {
+                    int rank = Integer.parseInt(unpackrank);
+                    Node node = dataSource.getNode(rank);
+                    DataSourceExport.writeProfiles(dataSource, new File("."), node.getThreads());
+                } else {
+                    DataSourceExport.writeProfiles(dataSource, new File("."));
+                }
 
             } catch (Exception e) {
                 e.printStackTrace();
