@@ -1,17 +1,11 @@
 package edu.uoregon.tau.perfdmf;
 
-import edu.uoregon.tau.perfdmf.database.*;
-
-import java.sql.SQLException;
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.util.Vector;
-import java.io.Serializable;
-import java.io.ObjectInputStream;
-import java.io.ObjectOutputStream;
-import java.io.IOException;
-
+import java.io.*;
 import java.sql.*;
+import java.util.Vector;
+
+import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfdmf.database.DBConnector;
 
 /**
  * Holds all the data for an application in the database.  This 
@@ -20,7 +14,7 @@ import java.sql.*;
  * an application from which the TAU performance data has been generated.
  * An application has zero or more experiments associated with it.
  *
- * <P>CVS $Id: Application.java,v 1.8 2007/02/23 01:08:24 khuck Exp $</P>
+ * <P>CVS $Id: Application.java,v 1.15 2009/08/19 11:14:50 khuck Exp $</P>
  * @author	Kevin Huck, Robert Bell
  * @version 0.1
  * @since   0.1
@@ -29,20 +23,24 @@ import java.sql.*;
  * @see		Experiment
  */
 public class Application implements Serializable {
-    public static String fieldNames[];
-    public static int fieldTypes[];
 
     private int applicationID;
     private String name;
     private String fields[];
 
-    // numFields, the number of optional fields found in the DB
+    private Database database;
+
+    public Database getDatabase() {
+        return database;
+    }
+
+    public void setDatabase(Database database) {
+        this.database = database;
+        fields = new String[database.getAppFieldNames().length];
+    }
+
     public Application() {
-        if (Application.fieldNames == null) {
-            fields = new String[0];
-        } else {
-            this.fields = new String[Application.fieldNames.length];
-        }
+        fields = new String[0];
     }
 
     // copy constructor
@@ -50,14 +48,7 @@ public class Application implements Serializable {
         this.name = app.getName();
         this.applicationID = app.getID();
         this.fields = (String[]) app.fields.clone();
-    }
-
-    public void reallocMetaData() {
-        if (Application.fieldNames == null) {
-            this.fields = new String[0];
-        } else {
-            this.fields = new String[Application.fieldNames.length];
-        }
+        this.database = app.database;
     }
 
     public String[] getFields() {
@@ -76,7 +67,7 @@ public class Application implements Serializable {
      */
     public static String[] getFieldNames(DB db) throws DatabaseException {
         getMetaData(db);
-        return fieldNames;
+        return db.getDatabase().getAppFieldNames();
     }
 
     public static void getMetaData(DB db) {
@@ -85,6 +76,7 @@ public class Application implements Serializable {
         //            return;
 
         try {
+            Database database = db.getDatabase();
             ResultSet resultSet = null;
 
             String appFieldNames[] = null;
@@ -128,13 +120,17 @@ public class Application implements Serializable {
             }
             resultSet.close();
 
-            Application.fieldNames = new String[nameList.size()];
-            Application.fieldTypes = new int[typeList.size()];
+            String[] fieldNames = new String[nameList.size()];
+            int[] fieldTypes = new int[typeList.size()];
 
             for (int i = 0; i < typeList.size(); i++) {
-                Application.fieldNames[i] = (String) nameList.get(i);
-                Application.fieldTypes[i] = ((Integer) typeList.get(i)).intValue();
+                fieldNames[i] = (String) nameList.get(i);
+                fieldTypes[i] = ((Integer) typeList.get(i)).intValue();
             }
+
+            database.setAppFieldNames(fieldNames);
+            database.setAppFieldTypes(fieldTypes);
+
         } catch (SQLException e) {
             e.printStackTrace();
         }
@@ -145,20 +141,20 @@ public class Application implements Serializable {
     }
 
     public String getFieldName(int idx) {
-        return fieldNames[idx];
+        return database.getAppFieldNames()[idx];
     }
 
     public int getFieldType(int idx) {
-        return Application.fieldTypes[idx];
+        return database.getAppFieldTypes()[idx];
     }
 
     // These two are here to handle the copy constructor for making ParaProfApplications
     public String[] getFieldNames() {
-        return fieldNames;
+        return database.getAppFieldNames();
     }
 
     public int[] getFieldTypes() {
-        return fieldTypes;
+        return database.getAppFieldTypes();
     }
 
     /**
@@ -188,10 +184,10 @@ public class Application implements Serializable {
     }
 
     public String getField(String name) {
-        if (Application.fieldNames == null)
+        if (database.getAppFieldNames() == null)
             return null;
-        for (int i = 0; i < Application.fieldNames.length; i++) {
-            if (name.toUpperCase().equals(Application.fieldNames[i].toUpperCase())) {
+        for (int i = 0; i < database.getAppFieldNames().length; i++) {
+            if (name.toUpperCase().equals(database.getAppFieldNames()[i].toUpperCase())) {
                 if (i < fields.length)
                     return fields[i];
             }
@@ -200,7 +196,7 @@ public class Application implements Serializable {
     }
 
     public void setField(int idx, String field) {
-        if (DBConnector.isIntegerType(fieldTypes[idx]) && field != null) {
+        if (DBConnector.isIntegerType(database.getAppFieldTypes()[idx]) && field != null) {
             try {
                 int test = Integer.parseInt(field);
             } catch (java.lang.NumberFormatException e) {
@@ -208,7 +204,7 @@ public class Application implements Serializable {
             }
         }
 
-        if (DBConnector.isFloatingPointType(fieldTypes[idx]) && field != null) {
+        if (DBConnector.isFloatingPointType(database.getAppFieldTypes()[idx]) && field != null) {
             try {
                 double test = Double.parseDouble(field);
             } catch (java.lang.NumberFormatException e) {
@@ -243,6 +239,7 @@ public class Application implements Serializable {
     public static Vector getApplicationList(DB db, String whereClause) {
         StringBuffer buf = null;
         try {
+            Database database = db.getDatabase();
             Application.getMetaData(db);
 
             ResultSet resultSet = null;
@@ -250,8 +247,8 @@ public class Application implements Serializable {
 
             buf = new StringBuffer("select id, name");
 
-            for (int i = 0; i < Application.fieldNames.length; i++) {
-                buf.append(", " + Application.fieldNames[i]);
+            for (int i = 0; i < database.getAppFieldNames().length; i++) {
+                buf.append(", " + database.getAppFieldNames()[i]);
             }
 
             buf.append(" from " + db.getSchemaPrefix() + "application");
@@ -259,25 +256,26 @@ public class Application implements Serializable {
             buf.append(whereClause);
 
             if (db.getDBType().compareTo("oracle") == 0) {
-                buf.append(" order by dbms_lob.substr(name) asc");
+                buf.append(" order by dbms_lob.substr(name), id asc");
             } else if (db.getDBType().compareTo("derby") == 0) {
-                buf.append(" order by cast (name as varchar(256)) asc");
+                buf.append(" order by cast (name as varchar(256)), id asc");
             } else if (db.getDBType().compareTo("db2") == 0) {
-                buf.append(" order by cast (name as varchar(256)) asc");
+                buf.append(" order by cast (name as varchar(256)), id asc");
             } else {
-                buf.append(" order by name asc ");
+                buf.append(" order by name, id asc ");
             }
 
             resultSet = db.executeQuery(buf.toString());
             while (resultSet.next() != false) {
                 Application application = new Application();
+                application.setDatabase(database);
 
                 application.setID(resultSet.getInt(1));
                 application.setName(resultSet.getString(2));
 
                 String tmp = resultSet.getString(3);
 
-                for (int i = 0; i < Application.fieldNames.length; i++) {
+                for (int i = 0; i < database.getAppFieldNames().length; i++) {
                     application.setField(i, resultSet.getString(i + 3));
                 }
 
@@ -291,6 +289,7 @@ public class Application implements Serializable {
         } catch (SQLException e) {
             if (buf != null)
                 System.out.println(buf.toString());
+			System.err.println(e.getMessage());
             throw new DatabaseException("", e);
         }
 
@@ -406,25 +405,25 @@ public class Application implements Serializable {
     private void readObject(ObjectInputStream aInputStream) throws ClassNotFoundException, IOException {
         // always perform the default de-serialization first
         aInputStream.defaultReadObject();
-        if (fieldNames == null)
-            fieldNames = (String[]) aInputStream.readObject();
-        if (fieldTypes == null)
-            fieldTypes = (int[]) aInputStream.readObject();
+        //        if (fieldNames == null)
+        //            fieldNames = (String[]) aInputStream.readObject();
+        //        if (fieldTypes == null)
+        //            fieldTypes = (int[]) aInputStream.readObject();
     }
 
     private void writeObject(ObjectOutputStream aOutputStream) throws IOException {
         // always perform the default serialization first
         aOutputStream.defaultWriteObject();
-        aOutputStream.writeObject(fieldNames);
-        aOutputStream.writeObject(fieldTypes);
+        //        aOutputStream.writeObject(fieldNames);
+        //        aOutputStream.writeObject(fieldTypes);
     }
 
-    /**
-     *  hack - needed to delete meta so that it is reloaded each time a new database is created.
-     */
-    public void removeMetaData() {
-        fieldNames = null;
-        fieldTypes = null;
-    }
+    //    /**
+    //     *  hack - needed to delete meta so that it is reloaded each time a new database is created.
+    //     */
+    //    public void removeMetaData() {
+    //        fieldNames = null;
+    //        fieldTypes = null;
+    //    }
 
 }
