@@ -17,9 +17,9 @@ import edu.uoregon.tau.perfdmf.database.DB;
 /**
  * Reads a single trial from the database
  *  
- * <P>CVS $Id: DBDataSource.java,v 1.10 2007/10/12 22:11:24 amorris Exp $</P>
+ * <P>CVS $Id: DBDataSource.java,v 1.16 2009/09/10 00:25:02 amorris Exp $</P>
  * @author  Robert Bell, Alan Morris
- * @version $Revision: 1.10 $
+ * @version $Revision: 1.16 $
  */
 public class DBDataSource extends DataSource {
 
@@ -54,6 +54,7 @@ public class DBDataSource extends DataSource {
                     getMetaData().put(currentName, currentValue);
                 } else {
                     currentThread.getMetaData().put(currentName, currentValue);
+                    getUncommonMetaData().put(currentName, currentValue);
                 }
             }
         }
@@ -85,6 +86,10 @@ public class DBDataSource extends DataSource {
 
         StringBuffer where = new StringBuffer();
 
+        if (metricMap.size() == 0) {
+            return;
+        }
+            
         where.append(" WHERE p.metric in (");
         for (Iterator it = metricMap.keySet().iterator(); it.hasNext();) {
             int metricID = ((Integer) it.next()).intValue();
@@ -144,6 +149,7 @@ public class DBDataSource extends DataSource {
         ResultSet resultSet = db.executeQuery(buf.toString());
         time = (System.currentTimeMillis()) - time;
         //System.out.println("Query : " + time);
+        //System.out.print(time + ", ");
 
         time = System.currentTimeMillis();
         while (resultSet.next() != false) {
@@ -178,6 +184,7 @@ public class DBDataSource extends DataSource {
         }
         time = (System.currentTimeMillis()) - time;
         //System.out.println("Processing : " + time);
+        //System.out.print(time + ", ");
 
         resultSet.close();
     }
@@ -212,7 +219,7 @@ public class DBDataSource extends DataSource {
 
     public void load() throws SQLException {
 
-        // System.out.println("Processing data, please wait ......");
+        //System.out.println("Processing data, please wait ......");
         long time = System.currentTimeMillis();
 
         DB db = databaseAPI.getDb();
@@ -230,7 +237,8 @@ public class DBDataSource extends DataSource {
         while (resultSet.next() != false) {
             int id = resultSet.getInt(1);
             String name = resultSet.getString(2);
-            Metric metric = this.addMetric(name);
+            Metric metric = this.addMetricNoCheck(name);
+            metric.setDbMetricID(id);
             metricMap.put(new Integer(id), metric);
             numberOfMetrics++;
         }
@@ -284,8 +292,10 @@ public class DBDataSource extends DataSource {
 
         downloadMetaData();
 
+        databaseAPI.terminate();
         time = (System.currentTimeMillis()) - time;
         //System.out.println("Time to download file (in milliseconds): " + time);
+        //System.out.println(time);
 
         // We actually discard the mean and total values by calling this
         // But, we need to compute other statistics anyway
