@@ -20,31 +20,38 @@ import org.w3c.dom.NodeList;
  * This class represents a data source.  After loading, data is availiable through the
  * public methods.
  *  
- * <P>CVS $Id: DataSource.java,v 1.29 2007/10/19 18:33:13 khuck Exp $</P>
+ * <P>CVS $Id: DataSource.java,v 1.49 2009/10/14 13:38:46 khuck Exp $</P>
  * @author  Robert Bell, Alan Morris
- * @version $Revision: 1.29 $
+ * @version $Revision: 1.49 $
  */
 public abstract class DataSource {
 
     public static final SimpleDateFormat dateTime = new SimpleDateFormat("yyyy-MM-dd'T'HH:mm:ss'Z'");
 
-    public static final int TAUPROFILE = 0;
-    public static final int PPROF = 1;
+    public static final int PPK = 0;
+    public static final int TAUPROFILE = 1;
     public static final int DYNAPROF = 2;
     public static final int MPIP = 3;
     public static final int HPM = 4;
     public static final int GPROF = 5;
     public static final int PSRUN = 6;
-    public static final int PPK = 7;
+    public static final int PPROF = 7;
     public static final int CUBE = 8;
     public static final int HPCTOOLKIT = 9;
     public static final int SNAP = 10;
     public static final int OMPP = 11;
     public static final int PERIXML = 12;
+    public static final int GPTL = 13; // General Purpose Timing Library - Jim Rosinski
+    public static final int PARAVER = 14; // Statistical output from Paraver - Jesus LeBarta
+    public static final int IPM = 15; // Data from IPM/NERSC
     public static final int GYRO = 100;
-
-    public static String formatTypeStrings[] = { "Tau profiles", "Tau pprof.dat", "Dynaprof", "MpiP", "HPMToolkit", "Gprof",
-            "PSRun", "ParaProf Packed Profile", "Cube", "HPCToolkit", "TAU Snapshot", "ompP", "PERI-XML" };
+    public static final int GAMESS = 101; // application log data
+    public static final String FILE_TYPE_INDEX = "File Type Index";
+    public static final String FILE_TYPE_NAME = "File Type Name";
+    
+    public static String formatTypeStrings[] = { "ParaProf Packed Profile", "Tau profiles", "Dynaprof", "MpiP", "HPMToolkit",
+            "Gprof", "PSRun", "Tau pprof.dat", "Cube", "HPCToolkit", "TAU Snapshot", "ompP", "PERI-XML",
+            "General Purpose Timing Library (GPTL)", "Paraver", "IPM" };
 
     private static boolean meanIncludeNulls = true;
 
@@ -55,7 +62,7 @@ public abstract class DataSource {
     private Function topLevelPhase;
 
     // data structures
-    private List metrics = null;
+    protected List metrics = null;
     protected Thread meanData = null;
     protected Thread totalData = null;
     protected Thread stddevData = null;
@@ -88,7 +95,22 @@ public abstract class DataSource {
     private boolean wellBehavedSnapshots;
     private long avgStartTime;
 
-    public DataSource() {
+    protected boolean monitored;
+
+    protected boolean hasThreads = true;
+    protected boolean hasContexts = true;
+    protected boolean hasMPI = false;
+    private int fileType = DataSource.TAUPROFILE;
+
+    public int getFileType() {
+		return fileType;
+	}
+
+	public void setFileType(int fileType) {
+		this.fileType = fileType;
+	}
+
+	public DataSource() {
     // nothing
     }
 
@@ -131,6 +153,10 @@ public abstract class DataSource {
         }
     }
 
+    public void setMonitored(boolean monitored) {
+        this.monitored = monitored;
+    }
+
     public Thread getMeanData() {
         return meanData;
     }
@@ -229,10 +255,14 @@ public abstract class DataSource {
 
     public Group addGroup(String name) {
         name = name.trim();
+        if (name == "TAU_MPI") {
+            hasMPI = true;
+        }
         Object obj = groups.get(name);
 
-        if (obj != null)
+        if (obj != null) {
             return (Group) obj;
+        }
 
         Group group = new Group(name, groups.size() + 1);
         groups.put(name, group);
@@ -346,7 +376,6 @@ public abstract class DataSource {
      *            Name of metric to be added
      */
     public Metric addMetric(String metricName) {
-        //System.err.println("addMetric("+metricName+")");
         if (metrics != null) {
             for (Iterator it = metrics.iterator(); it.hasNext();) {
                 Metric metric = (Metric) it.next();
@@ -362,6 +391,55 @@ public abstract class DataSource {
         return metric;
     }
 
+    public Metric addMetricNoCheck(String metricName) {
+        Metric metric = new Metric();
+        metric.setName(metricName);
+        addMetric(metric);
+        return metric;
+    }
+
+    /**
+     * Adds a metric to the DataSource's metrics List (given as a String).
+     * need to create this method from DataSource.  The DataSource
+     * assumes that we are processing in this order: metric,thread,event.
+     * The GPTL data is in thread,event,metric order.
+     * 
+     * @param metric
+     *            Name of metric to be added
+     * @param thread
+     *            Current thread that is being processed
+     */
+    public Metric addMetric(String metricName, Thread thread) {
+        Metric metric = null;
+        // if the metric list exists, and has values, search for this metric
+        if (metrics == null) {
+            metrics = new ArrayList();
+        } else {
+            for (Iterator it = metrics.iterator(); it.hasNext();) {
+                Metric tmp = (Metric) it.next();
+                if (tmp.getName().equals(metricName)) {
+                    metric = tmp;
+                    break;
+                }
+            }
+        }
+
+        // did we find the metric?  If not, create a new one
+        if (metric == null) {
+            metric = new Metric();
+            metric.setName(metricName);
+            metric.setID(this.getNumberOfMetrics());
+            metrics.add(metric);
+        }
+
+        // if the current thread doesn't have this metric, add it
+        if (thread.getNumMetrics() < this.getNumberOfMetrics()) {
+            thread.addMetric();
+        }
+
+        return metric;
+    }
+
     /**
      * Get a the List of Metrics
      * 
@@ -371,6 +449,14 @@ public abstract class DataSource {
         return metrics;
     }
 
+    /**
+     * Clear the list of metrics
+     * 
+     */
+    public void clearMetrics() {
+        metrics = null;
+    }
+
     public Metric getMetric(String name) {
         for (Iterator it = metrics.iterator(); it.hasNext();) {
             Metric metric = (Metric) it.next();
@@ -440,6 +526,7 @@ public abstract class DataSource {
          *     D
          *     
          * Here, D is duplicate data, it is the sum of the other two
+         * 
          * The Bonus data is additional duplicate data:
          *     B1 => C => D
          *     B2 => C => D
@@ -492,7 +579,7 @@ public abstract class DataSource {
                         if (functionProfile != null) {
                             FunctionProfile bfp = thread.getFunctionProfile(bonusFunction);
                             if (bfp == null) {
-                                bfp = new FunctionProfile(bonusFunction, getNumberOfMetrics());
+                                bfp = new FunctionProfile(bonusFunction, getNumberOfMetrics(), thread.getNumSnapshots());
                                 thread.addFunctionProfile(bfp);
                             }
 
@@ -582,6 +669,10 @@ public abstract class DataSource {
         this.generateUserEventStatistics();
 
         finishPhaseAnalysis();
+        
+        this.getMetaData().put(FILE_TYPE_INDEX, Integer.toString(this.fileType));
+        this.getMetaData().put(FILE_TYPE_NAME, DataSource.formatTypeStrings[this.fileType]);
+
 
         //time = (System.currentTimeMillis()) - time;
         //System.out.println("Time to process (in milliseconds): " + time);
@@ -598,97 +689,92 @@ public abstract class DataSource {
     }
 
     private void generateUserEventStatistics() {
-
+        // make sure that the allThreads list is initialized;
+        this.initAllThreadsList();
+        int numThreads = allThreads.size();
         int numSnapshots = meanData.getNumSnapshots();
+
+        int numProfiles[] = new int[getNumUserEvents() + 1];
+
         for (int snapshot = 0; snapshot < numSnapshots; snapshot++) {
+
+            for (int i = 0; i < numThreads; i++) { // for each thread
+                Thread thread = (Thread) allThreads.get(i);
+                for (Iterator it = thread.getUserEventProfiles(); it.hasNext();) {
+                    UserEventProfile uep = (UserEventProfile) it.next();
+                    UserEvent ue = uep.getUserEvent();
+
+                    // get/create the userEventProfile for mean
+                    UserEventProfile meanProfile = meanData.getUserEventProfile(ue);
+                    if (meanProfile == null) {
+                        meanProfile = new UserEventProfile(ue, numSnapshots);
+                        meanData.addUserEventProfile(meanProfile);
+                    }
+
+                    // get/create the userEventProfile for total
+                    UserEventProfile totalProfile = totalData.getUserEventProfile(ue);
+                    if (totalProfile == null) {
+                        totalProfile = new UserEventProfile(ue, numSnapshots);
+                        totalData.addUserEventProfile(totalProfile);
+                    }
+
+                    // get/create the userEventProfile for stddev
+                    UserEventProfile stddevProfile = stddevData.getUserEventProfile(ue);
+                    if (stddevProfile == null) {
+                        stddevProfile = new UserEventProfile(ue, numSnapshots);
+                        stddevData.addUserEventProfile(stddevProfile);
+                    }
+
+                    numProfiles[ue.getID()]++;
+
+                    totalProfile.setNumSamples(totalProfile.getNumSamples() + uep.getNumSamples(snapshot), snapshot);
+                    totalProfile.setMaxValue(totalProfile.getMaxValue() + uep.getMaxValue(snapshot), snapshot);
+                    totalProfile.setMinValue(totalProfile.getMinValue() + uep.getMinValue(snapshot), snapshot);
+                    totalProfile.setMeanValue(totalProfile.getMeanValue() + uep.getMeanValue(snapshot), snapshot);
+                    totalProfile.setStdDev(totalProfile.getStdDev() + uep.getStdDev(snapshot), snapshot);
+
+                    stddevProfile.setNumSamples(stddevProfile.getNumSamples()
+                            + (uep.getNumSamples(snapshot) * uep.getNumSamples(snapshot)), snapshot);
+                    stddevProfile.setMaxValue(stddevProfile.getMaxValue()
+                            + (uep.getMaxValue(snapshot) * uep.getMaxValue(snapshot)), snapshot);
+                    stddevProfile.setMinValue(stddevProfile.getMinValue()
+                            + (uep.getMinValue(snapshot) * uep.getMinValue(snapshot)), snapshot);
+                    stddevProfile.setMeanValue(stddevProfile.getMeanValue()
+                            + (uep.getMeanValue(snapshot) * uep.getMeanValue(snapshot)), snapshot);
+                    stddevProfile.setStdDev(stddevProfile.getStdDev() + (uep.getStdDev(snapshot) * uep.getStdDev(snapshot)),
+                            snapshot);
+
+                }
+            }
+
             for (Iterator it = this.getUserEvents(); it.hasNext();) {
                 UserEvent ue = (UserEvent) it.next();
 
-                // get/create the userEventProfile for mean
                 UserEventProfile meanProfile = meanData.getUserEventProfile(ue);
-                if (meanProfile == null) {
-                    meanProfile = new UserEventProfile(ue, numSnapshots);
-                    meanData.addUserEventProfile(meanProfile);
-                }
-                //ue.setMeanProfile(meanProfile);
-
-                // get/create the userEventProfile for total
                 UserEventProfile totalProfile = totalData.getUserEventProfile(ue);
-                if (totalProfile == null) {
-                    totalProfile = new UserEventProfile(ue, numSnapshots);
-                    totalData.addUserEventProfile(totalProfile);
-                }
-                //ue.setTotalProfile(totalProfile);
-
-                // get/create the userEventProfile for stddev
                 UserEventProfile stddevProfile = stddevData.getUserEventProfile(ue);
-                if (stddevProfile == null) {
-                    stddevProfile = new UserEventProfile(ue, numSnapshots);
-                    stddevData.addUserEventProfile(stddevProfile);
-                }
-                //ue.setStddevProfile(stddevProfile);
-
-                int numProfiles = 0;
-                double numSampSum = 0;
-                double numSampSumSqr = 0;
-                double maxSum = 0;
-                double maxSumSqr = 0;
-                double minSum = 0;
-                double minSumSqr = 0;
-                double meanSum = 0;
-                double meanSumSqr = 0;
-                double stdDevSum = 0;
-                double stdDevSumSqr = 0;
-
-                // make sure that the allThreads list is initialized;
-                this.initAllThreadsList();
-                int numThreads = allThreads.size();
-
-                for (int i = 0; i < numThreads; i++) { // for each thread
-                    Thread thread = (Thread) allThreads.get(i);
-                    UserEventProfile uep = thread.getUserEventProfile(ue);
-                    if (uep != null) {
-                        numProfiles++;
-
-                        numSampSum += uep.getNumSamples(snapshot);
-                        numSampSumSqr += uep.getNumSamples(snapshot) * uep.getNumSamples(snapshot);
-
-                        maxSum += uep.getMaxValue(snapshot);
-                        maxSumSqr += uep.getMaxValue(snapshot) * uep.getMaxValue(snapshot);
-
-                        minSum += uep.getMinValue(snapshot);
-                        minSumSqr += uep.getMinValue(snapshot) * uep.getMinValue(snapshot);
-
-                        meanSum += uep.getMeanValue(snapshot);
-                        meanSumSqr += uep.getMeanValue(snapshot) * uep.getMeanValue(snapshot);
-
-                        stdDevSum += uep.getStdDev(snapshot);
-                        stdDevSumSqr += uep.getStdDev(snapshot) * uep.getStdDev(snapshot);
-                    }
-                }
 
+                int divider = numThreads;
                 if (!meanIncludeNulls) { // do we include null values as zeroes in the computation or not?
-                    numThreads = numProfiles;
+                    divider = numProfiles[ue.getID()];
                 }
 
-                totalProfile.setNumSamples(numSampSum, snapshot);
-                totalProfile.setMaxValue(maxSum, snapshot);
-                totalProfile.setMinValue(minSum, snapshot);
-                totalProfile.setMeanValue(maxSum, snapshot);
-                totalProfile.setStdDev(stdDevSum, snapshot);
+                meanProfile.setNumSamples((totalProfile.getNumSamples() / divider), snapshot);
+                meanProfile.setMaxValue(totalProfile.getMaxValue(snapshot) / divider, snapshot);
+                meanProfile.setMinValue(totalProfile.getMinValue(snapshot) / divider, snapshot);
+                meanProfile.setMeanValue(totalProfile.getMeanValue(snapshot) / divider, snapshot);
+                meanProfile.setStdDev(totalProfile.getStdDev(snapshot) / divider, snapshot);
 
-                meanProfile.setNumSamples((numSampSum / numThreads), snapshot);
-                meanProfile.setMaxValue(maxSum / numThreads, snapshot);
-                meanProfile.setMinValue(minSum / numThreads, snapshot);
-                meanProfile.setMeanValue(meanSum / numThreads, snapshot);
-                meanProfile.setStdDev(stdDevSum / numThreads, snapshot);
-
-                stddevProfile.setNumSamples(computeStdDev(numSampSumSqr, meanProfile.getNumSamples(snapshot), numThreads),
-                        snapshot);
-                stddevProfile.setMaxValue(computeStdDev(maxSumSqr, meanProfile.getMaxValue(snapshot), numThreads), snapshot);
-                stddevProfile.setMinValue(computeStdDev(minSumSqr, meanProfile.getMinValue(snapshot), numThreads), snapshot);
-                stddevProfile.setMeanValue(computeStdDev(meanSumSqr, meanProfile.getMeanValue(snapshot), numThreads), snapshot);
-                stddevProfile.setStdDev(computeStdDev(stdDevSumSqr, meanProfile.getStdDev(snapshot), numThreads), snapshot);
+                stddevProfile.setNumSamples(computeStdDev(stddevProfile.getNumSamples(snapshot),
+                        meanProfile.getNumSamples(snapshot), divider), snapshot);
+                stddevProfile.setMaxValue(computeStdDev(stddevProfile.getMaxValue(snapshot), meanProfile.getMaxValue(snapshot),
+                        divider), snapshot);
+                stddevProfile.setMinValue(computeStdDev(stddevProfile.getMinValue(snapshot), meanProfile.getMinValue(snapshot),
+                        divider), snapshot);
+                stddevProfile.setMeanValue(computeStdDev(stddevProfile.getMeanValue(snapshot),
+                        meanProfile.getMeanValue(snapshot), divider), snapshot);
+                stddevProfile.setStdDev(
+                        computeStdDev(stddevProfile.getStdDev(snapshot), meanProfile.getStdDev(snapshot), divider), snapshot);
             }
         }
 
@@ -729,17 +815,17 @@ public abstract class DataSource {
         int numMetrics = this.getNumberOfMetrics();
         Thread firstThread = (Thread) getAllThreads().get(0);
         if (meanData == null) {
-            meanData = new Thread(-1, -1, -1, numMetrics);
+            meanData = new Thread(-1, -1, -1, numMetrics, this);
             addDerivedSnapshots(firstThread, meanData);
         }
 
         if (totalData == null) {
-            totalData = new Thread(-2, -2, -2, numMetrics);
+            totalData = new Thread(-2, -2, -2, numMetrics, this);
             addDerivedSnapshots(firstThread, totalData);
         }
 
         if (stddevData == null) {
-            stddevData = new Thread(-3, -3, -3, numMetrics);
+            stddevData = new Thread(-3, -3, -3, numMetrics, this);
             addDerivedSnapshots(firstThread, stddevData);
         }
 
@@ -748,7 +834,6 @@ public abstract class DataSource {
         double[] exclSumSqr = new double[numMetrics];
         double[] inclSumSqr = new double[numMetrics];
 
-        double topLevelInclSum[] = new double[numMetrics];
 
         // make sure that the allThreads list is initialized;
         this.initAllThreadsList();
@@ -756,7 +841,8 @@ public abstract class DataSource {
         for (int snapshot = 0; snapshot < meanData.getNumSnapshots(); snapshot++) {
 
             // must always iterate through all metrics regardless to find the top level timers, I think???
-            for (int i = 0; i < numMetrics; i++) { // for each metric
+            for (int i = startMetric; i <= endMetric; i++) { // for each metric
+                double topLevelInclSum[] = new double[numMetrics];
                 for (Iterator it = allThreads.iterator(); it.hasNext();) { // for each thread
                     Thread thread = (Thread) it.next();
                     if (wellBehavedSnapshots) {
@@ -765,7 +851,7 @@ public abstract class DataSource {
                         topLevelInclSum[i] += thread.getMaxInclusive(i, thread.getNumSnapshots() - 1);
                     }
                 }
-                // for now, the derived 'threads' only have one snapshot
+
                 totalData.setPercentDivider(i, snapshot, topLevelInclSum[i] / 100.0);
                 meanData.setPercentDivider(i, snapshot, topLevelInclSum[i] / 100.0);
                 stddevData.setPercentDivider(i, snapshot, topLevelInclSum[i] / 100.0);
@@ -840,8 +926,9 @@ public abstract class DataSource {
                     }
                 }
 
+                int divider = numThreads;
                 if (!meanIncludeNulls) { // do we include null values as zeroes in the computation or not?
-                    numThreads = numEvents;
+                    divider = numEvents;
                 }
 
                 // we don't want to set the calls and subroutines if we're just computing mean data for a derived metric!
@@ -850,20 +937,20 @@ public abstract class DataSource {
                     totalProfile.setNumCalls(snapshot, callSum);
                     totalProfile.setNumSubr(snapshot, subrSum);
 
-                    // mean is just the total / numThreads
-                    meanProfile.setNumCalls(snapshot, (double) callSum / numThreads);
-                    meanProfile.setNumSubr(snapshot, (double) subrSum / numThreads);
+                    // mean is just the total / divider
+                    meanProfile.setNumCalls(snapshot, (double) callSum / divider);
+                    meanProfile.setNumSubr(snapshot, (double) subrSum / divider);
 
                     double stdDev = 0;
-                    if (numThreads > 1) {
-                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((callSumSqr / (numThreads))
+                    if (divider > 1) {
+                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((callSumSqr / (divider))
                                 - (meanProfile.getNumCalls(snapshot) * meanProfile.getNumCalls(snapshot))));
                     }
                     stddevProfile.setNumCalls(snapshot, stdDev);
 
                     stdDev = 0;
-                    if (numThreads > 1) {
-                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((subrSumSqr / (numThreads))
+                    if (divider > 1) {
+                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((subrSumSqr / (divider))
                                 - (meanProfile.getNumSubr(snapshot) * meanProfile.getNumSubr(snapshot))));
                     }
                     stddevProfile.setNumSubr(snapshot, stdDev);
@@ -876,25 +963,25 @@ public abstract class DataSource {
                     totalProfile.setInclusive(snapshot, m, inclSum[m]);
 
                     // mean data computed as above in comments
-                    meanProfile.setExclusive(snapshot, m, exclSum[m] / numThreads);
-                    meanProfile.setInclusive(snapshot, m, inclSum[m] / numThreads);
+                    meanProfile.setExclusive(snapshot, m, exclSum[m] / divider);
+                    meanProfile.setInclusive(snapshot, m, inclSum[m] / divider);
 
                     double stdDev = 0;
-                    if (numThreads > 1) {
+                    if (divider > 1) {
 
                         // see http://cuwu.editthispage.com/stories/storyReader$13 for why I don't multiply by n/(n-1)
 
-                        //stdDev = java.lang.Math.sqrt(((double) numThreads / (numThreads - 1))
-                        //        * java.lang.Math.abs((exclSumSqr[i] / (numThreads))
+                        //stdDev = java.lang.Math.sqrt(((double) divider / (divider - 1))
+                        //        * java.lang.Math.abs((exclSumSqr[i] / (divider))
                         //                - (meanProfile.getExclusive(i) * meanProfile.getExclusive(i))));
-                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((exclSumSqr[m] / (numThreads))
+                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((exclSumSqr[m] / (divider))
                                 - (meanProfile.getExclusive(snapshot, m) * meanProfile.getExclusive(snapshot, m))));
                     }
                     stddevProfile.setExclusive(snapshot, m, stdDev);
 
                     stdDev = 0;
-                    if (numThreads > 1) {
-                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((inclSumSqr[m] / (numThreads))
+                    if (divider > 1) {
+                        stdDev = java.lang.Math.sqrt(java.lang.Math.abs((inclSumSqr[m] / (divider))
                                 - (meanProfile.getInclusive(snapshot, m) * meanProfile.getInclusive(snapshot, m))));
                     }
                     stddevProfile.setInclusive(snapshot, m, stdDev);
@@ -902,6 +989,12 @@ public abstract class DataSource {
                 }
             }
         }
+
+        // Now we count up whether we have threads and contexts or not, this is for display purposes in ParaProf
+        int[] nct = getMaxNCTNumbers();
+        hasContexts = nct[1] > 0;
+        hasThreads = nct[2] > 0;
+
     }
 
     /**
@@ -918,11 +1011,12 @@ public abstract class DataSource {
         Object obj = nodes.get(new Integer(nodeID));
 
         // return the Node if found
-        if (obj != null)
+        if (obj != null) {
             return (Node) obj;
+        }
 
         // otherwise, add it and return it
-        Node node = new Node(nodeID);
+        Node node = new Node(nodeID, this);
         nodes.put(new Integer(nodeID), node);
         return node;
     }
@@ -1194,7 +1288,7 @@ public abstract class DataSource {
 
             Element master = null;
             List nodeProfiles = new ArrayList();
-			List profilesAdded = new ArrayList();
+            List profilesAdded = new ArrayList();
 
             // create the common attribute node
             if (metaData.size() > 0) {
@@ -1249,16 +1343,16 @@ public abstract class DataSource {
                         addit = true;
                     }
                 }
-				nodeProfiles.add(delta);
+                nodeProfiles.add(delta);
 
                 if (addit) {
                     // don't add it to the tree, unless it has items that differ from
                     // the master record
                     root.appendChild(delta);
-					profilesAdded.add(new Boolean(true));
+                    profilesAdded.add(new Boolean(true));
                 } else {
-					profilesAdded.add(new Boolean(false));
-				}
+                    profilesAdded.add(new Boolean(false));
+                }
 
             }
 
@@ -1276,7 +1370,7 @@ public abstract class DataSource {
                         && imported.getNodeName().equals("tau:metadata")) {
                     // extract it out, and add it to our tree!
                     NodeList nodes = imported.getChildNodes();
-					int nodeIndex = 0;
+                    int nodeIndex = 0;
                     for (int i = 0; i < nodes.getLength(); i++) {
                         org.w3c.dom.Node cpa = nodes.item(i);
                         // System.out.println("\t" + cpa.getNodeName());
@@ -1299,25 +1393,25 @@ public abstract class DataSource {
                             //root.appendChild(cpa);
                         } else if (cpa.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE
                                 && cpa.getNodeName().equals("tau:ProfileAttributes")) {
-							Element currentDelta = (Element)nodeProfiles.get(nodeIndex);
-							if (((Boolean)profilesAdded.get(nodeIndex)).booleanValue()) {
+                            Element currentDelta = (Element) nodeProfiles.get(nodeIndex);
+                            if (((Boolean) profilesAdded.get(nodeIndex)).booleanValue()) {
 
-                            	NodeList attrs = cpa.getChildNodes();
-                            	// System.out.println("length: " + attrs.getLength());
-                            	for (int j = 0; j < attrs.getLength(); j++) {
-                                	// System.out.println("\t\tj:" + j);
-                                	org.w3c.dom.Node tmp = attrs.item(j);
-                                	if (tmp.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE
-                                        	&& tmp.getNodeName().equals("tau:attribute")) {
-                                    	// System.out.println("\t\tname:" + tmp.getNodeName());
-                                    	currentDelta.appendChild(tmp);
-                                	}
-                                	// System.out.println("\t\tend j:" + j);
-                            	}
-							} else {
-                            	root.appendChild(cpa);
-							}
-							nodeIndex++;
+                                NodeList attrs = cpa.getChildNodes();
+                                // System.out.println("length: " + attrs.getLength());
+                                for (int j = 0; j < attrs.getLength(); j++) {
+                                    // System.out.println("\t\tj:" + j);
+                                    org.w3c.dom.Node tmp = attrs.item(j);
+                                    if (tmp.getNodeType() == org.w3c.dom.Node.ELEMENT_NODE
+                                            && tmp.getNodeName().equals("tau:attribute")) {
+                                        // System.out.println("\t\tname:" + tmp.getNodeName());
+                                        currentDelta.appendChild(tmp);
+                                    }
+                                    // System.out.println("\t\tend j:" + j);
+                                }
+                            } else {
+                                root.appendChild(cpa);
+                            }
+                            nodeIndex++;
                         }
                     }
                 } else {
@@ -1419,4 +1513,36 @@ public abstract class DataSource {
         return uncommonMetaData;
     }
 
+    public boolean getHasThreads() {
+        return hasThreads;
+    }
+
+    public boolean getHasContexts() {
+        return hasContexts;
+    }
+
+    public final static int EXEC_TYPE_SINGLE = 0;
+    public final static int EXEC_TYPE_MPI = 1;
+    public final static int EXEC_TYPE_THREADED = 2;
+    public final static int EXEC_TYPE_HYBRID = 3;
+    public final static int EXEC_TYPE_OTHER = 4;
+
+    public int getExecutionType() {
+        if (getAllThreads().size() == 1) {
+            if (hasMPI) {
+                return EXEC_TYPE_MPI;
+            } else {
+                return EXEC_TYPE_SINGLE;
+            }
+        }
+
+        if (getHasContexts() == false && getHasThreads() == false) {
+            return EXEC_TYPE_MPI;
+        }
+        if (getHasContexts() == false) {
+            return EXEC_TYPE_HYBRID;
+        }
+        return EXEC_TYPE_OTHER;
+    }
+
 }
\ No newline at end of file
