@@ -1,28 +1,39 @@
 package edu.uoregon.tau.perfdmf;
 
-import edu.uoregon.tau.perfdmf.database.*;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.Enumeration;
+import java.util.HashMap;
+import java.util.Hashtable;
+import java.util.Iterator;
+import java.util.List;
+import java.util.ListIterator;
+import java.util.Map;
+import java.util.Vector;
 
-import java.util.*;
-import java.sql.*;
+import edu.uoregon.tau.perfdmf.database.ConnectionManager;
+import edu.uoregon.tau.perfdmf.database.DB;
 
 /**
  * This is the top level class for the Database API.
  * 
  * <P>
- * CVS $Id: DatabaseAPI.java,v 1.12 2007/03/20 17:12:52 amorris Exp $
+ * CVS $Id: DatabaseAPI.java,v 1.29 2009/12/14 23:07:49 amorris Exp $
  * </P>
  * 
  * @author Kevin Huck, Robert Bell
- * @version $Revision: 1.12 $
+ * @version $Revision: 1.29 $
  */
 public class DatabaseAPI {
 
     private Application application = null;
     private Experiment experiment = null;
     private Trial trial = null;
-    private Vector nodes = null;
-    private Vector contexts = null;
-    private Vector threads = null;
+    private List nodes = null;
+    private List contexts = null;
+    private List threads = null;
     private Vector intervalEvents = null;
     private List metrics = null;
     private Vector intervalEventData = null;
@@ -38,6 +49,8 @@ public class DatabaseAPI {
 
     private boolean cancelUpload = false;
 
+    private Database database;
+
     public void cancelUpload() {
         this.cancelUpload = true;
     }
@@ -64,9 +77,7 @@ public class DatabaseAPI {
         this.experiment = experiment;
     }
 
-    public DatabaseAPI() {
-        super();
-    }
+    public DatabaseAPI() {}
 
     public DB db() {
         return db;
@@ -78,11 +89,17 @@ public class DatabaseAPI {
 
     // Initialization / termination routines
 
-    public void initialize(Object obj, boolean prompt) throws SQLException {
-        configFileName = new String((String) (obj));
-        //Initialize the connection to the database,
-        //using the configuration settings.
-        connector = new ConnectionManager(configFileName, prompt);
+    public void initialize(String configFile, boolean prompt) throws SQLException {
+        if (configFile.startsWith("http") || (new java.io.File(configFile).exists())) {
+            initialize(new Database(configFile), prompt);
+        } else {
+            System.err.println("Could not find file: " + configFile);
+        }
+    }
+
+    public void initialize(Database database, boolean prompt) throws SQLException {
+        this.database = database;
+        connector = new ConnectionManager(database, prompt);
         connector.connect();
         db = connector.getDB();
         Application.getMetaData(db);
@@ -90,8 +107,9 @@ public class DatabaseAPI {
         Trial.getMetaData(db);
     }
 
-    public void initialize(ParseConfig parser, String password) throws SQLException {
-        connector = new ConnectionManager(parser, password);
+    public void initialize(Database database, String password) throws SQLException {
+        this.database = database;
+        connector = new ConnectionManager(database, password);
         connector.connect();
         db = connector.getDB();
         Application.getMetaData(db);
@@ -99,11 +117,9 @@ public class DatabaseAPI {
         Trial.getMetaData(db);
     }
 
-    public void initialize(Object obj, String password) throws SQLException {
-        String configFileName = (String) (obj);
-        //Initialize the connection to the database,
-        //using the configuration settings.
-        connector = new ConnectionManager(configFileName, password);
+    public void initialize(Database database) throws SQLException {
+        this.database = database;
+        connector = new ConnectionManager(database);
         connector.connect();
         db = connector.getDB();
         Application.getMetaData(db);
@@ -136,14 +152,14 @@ public class DatabaseAPI {
     }
 
     // returns Vector of Trial objects
-    public List getTrialList() {
+    public List getTrialList(boolean getMetadata) {
         StringBuffer whereClause = new StringBuffer();
         if (experiment != null) {
             whereClause.append("WHERE t.experiment = " + experiment.getID());
         } else if (application != null) {
             whereClause.append("WHERE e.application = " + application.getID());
         }
-        return Trial.getTrialList(db, whereClause.toString());
+        return Trial.getTrialList(db, whereClause.toString(), getMetadata);
     }
 
     // set the Application for this session
@@ -203,11 +219,11 @@ public class DatabaseAPI {
     }
 
     // set the Trial for this session
-    public Trial setTrial(int id) {
-        return setTrial(id, true);
+    public Trial setTrial(int id, boolean getXMLMetadata) {
+        return setTrial(id, true, getXMLMetadata);
     }
 
-    private Trial setTrial(int id, boolean clearHashes) {
+    private Trial setTrial(int id, boolean clearHashes, boolean getXMLMetadata) {
         this.trial = null;
         this.metrics = null;
         if (clearHashes) {
@@ -217,7 +233,30 @@ public class DatabaseAPI {
         // create a string to hit the database
         String whereClause;
         whereClause = " WHERE t.id = " + id;
-        Vector trials = Trial.getTrialList(db, whereClause);
+        Vector trials = Trial.getTrialList(db, whereClause, getXMLMetadata);
+        if (trials.size() == 1) {
+            this.trial = (Trial) trials.elementAt(0);
+        } //else exception?
+
+        return this.trial;
+    }
+
+    // set the Trial for this session
+    public Trial setTrial(String trialName, boolean getXMLMetadata) {
+        return setTrial(trialName, true, getXMLMetadata);
+    }
+
+    private Trial setTrial(String trialName, boolean clearHashes, boolean getXMLMetadata) {
+        this.trial = null;
+        this.metrics = null;
+        if (clearHashes) {
+            this.intervalEventHash = null;
+            this.atomicEventHash = null;
+        }
+        // create a string to hit the database
+        String whereClause;
+        whereClause = " WHERE t.name = '" + trialName + "'";
+        Vector trials = Trial.getTrialList(db, whereClause, getXMLMetadata);
         if (trials.size() == 1) {
             this.trial = (Trial) trials.elementAt(0);
         } //else exception?
@@ -238,8 +277,9 @@ public class DatabaseAPI {
 
         intervalEvents = IntervalEvent.getIntervalEvents(this, db, whereClause);
 
-        if (intervalEventHash == null)
+        if (intervalEventHash == null) {
             intervalEventHash = new Hashtable();
+        }
         IntervalEvent fun;
         for (Enumeration en = intervalEvents.elements(); en.hasMoreElements();) {
             fun = (IntervalEvent) en.nextElement();
@@ -258,10 +298,11 @@ public class DatabaseAPI {
             for (Iterator en = metrics.iterator(); en.hasNext();) {
                 metric = (Metric) en.next();
                 buf.append(metric.getID());
-                if (en.hasNext())
+                if (en.hasNext()) {
                     buf.append(", ");
-                else
+                } else {
                     buf.append(") ");
+                }
             }
         }
         IntervalLocationProfile.getIntervalEventDetail(db, intervalEvent, buf.toString());
@@ -300,10 +341,11 @@ public class DatabaseAPI {
         IntervalEvent intervalEvent = null;
         this.intervalEvents = new Vector();
         intervalEvent = getIntervalEvent(id);
-        if (intervalEvent != null)
+        if (intervalEvent != null) {
             this.intervalEvents.addElement(intervalEvent);
-        // we need this to get the metric data...
-        setTrial(intervalEvent.getTrialID(), false);
+            // we need this to get the metric data...
+            setTrial(intervalEvent.getTrialID(), false);
+        }
         return intervalEvent;
     }
 
@@ -317,6 +359,36 @@ public class DatabaseAPI {
         return atomicEvent;
     }
 
+    // clears the interval event selection
+    public void clearIntervalEvents() {
+        this.intervalEvents = null;
+        return;
+    }
+
+    // sets the current node ID
+    public void setNode(int id) {
+        Integer node = new Integer(id);
+        this.nodes = new ArrayList();
+        this.nodes.add(node);
+        return;
+    }
+
+    // sets the current context ID
+    public void setContext(int id) {
+        Integer context = new Integer(id);
+        this.contexts = new ArrayList();
+        this.contexts.add(context);
+        return;
+    }
+
+    // sets the current thread ID
+    public void setThread(int id) {
+        Integer thread = new Integer(id);
+        this.threads = new ArrayList();
+        this.threads.add(thread);
+        return;
+    }
+
     public List getIntervalEventData() throws SQLException {
         // check to make sure this is a meaningful request
         if (trial == null && intervalEvents == null) {
@@ -325,9 +397,9 @@ public class DatabaseAPI {
         }
 
         // get the hash of intervalEvent names first
-        if (intervalEvents == null) {
-            getIntervalEvents();
-        }
+        //if (intervalEvents == null) {
+        //getIntervalEvents();
+        //}
 
         // get the metric count
         int metricCount = 0;
@@ -362,10 +434,10 @@ public class DatabaseAPI {
         if (nodes != null && nodes.size() > 0) {
             buf.append(" AND p.node IN (");
             Integer node;
-            for (Enumeration en = nodes.elements(); en.hasMoreElements();) {
-                node = (Integer) en.nextElement();
+            for (Iterator iter = nodes.iterator(); iter.hasNext();) {
+                node = (Integer) iter.next();
                 buf.append(node);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -374,10 +446,10 @@ public class DatabaseAPI {
         if (contexts != null && contexts.size() > 0) {
             buf.append(" AND p.context IN (");
             Integer context;
-            for (Enumeration en = contexts.elements(); en.hasMoreElements();) {
-                context = (Integer) en.nextElement();
+            for (Iterator iter = contexts.iterator(); iter.hasNext();) {
+                context = (Integer) iter.next();
                 buf.append(context);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -386,10 +458,10 @@ public class DatabaseAPI {
         if (threads != null && threads.size() > 0) {
             buf.append(" AND p.thread IN (");
             Integer thread;
-            for (Enumeration en = threads.elements(); en.hasMoreElements();) {
-                thread = (Integer) en.nextElement();
+            for (Iterator iter = threads.iterator(); iter.hasNext();) {
+                thread = (Integer) iter.next();
                 buf.append(thread);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -448,10 +520,10 @@ public class DatabaseAPI {
         if (nodes != null && nodes.size() > 0) {
             buf.append(" AND p.node IN (");
             Integer node;
-            for (Enumeration en = nodes.elements(); en.hasMoreElements();) {
-                node = (Integer) en.nextElement();
+            for (Iterator iter = nodes.iterator(); iter.hasNext();) {
+                node = (Integer) iter.next();
                 buf.append(node);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -460,10 +532,10 @@ public class DatabaseAPI {
         if (contexts != null && contexts.size() > 0) {
             buf.append(" AND p.context IN (");
             Integer context;
-            for (Enumeration en = contexts.elements(); en.hasMoreElements();) {
-                context = (Integer) en.nextElement();
+            for (Iterator iter = contexts.iterator(); iter.hasNext();) {
+                context = (Integer) iter.next();
                 buf.append(context);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -472,10 +544,10 @@ public class DatabaseAPI {
         if (threads != null && threads.size() > 0) {
             buf.append(" AND p.thread IN (");
             Integer thread;
-            for (Enumeration en = threads.elements(); en.hasMoreElements();) {
-                thread = (Integer) en.nextElement();
+            for (Iterator iter = threads.iterator(); iter.hasNext();) {
+                thread = (Integer) iter.next();
                 buf.append(thread);
-                if (en.hasMoreElements())
+                if (iter.hasNext())
                     buf.append(", ");
                 else
                     buf.append(") ");
@@ -580,16 +652,15 @@ public class DatabaseAPI {
 
     private Hashtable saveMetrics(int newTrialID, Trial trial, int saveMetricIndex) throws SQLException {
         Hashtable metricHash = new Hashtable();
-        Iterator en = trial.getDataSource().getMetrics().iterator();
-        int i = 0;
-        while (en.hasNext()) {
-            Metric metric = (Metric) en.next();
-            int newMetricID = 0;
-            if (saveMetricIndex < 0 || saveMetricIndex == i) {
+        int idx = 0;
+        for (Iterator it = trial.getDataSource().getMetrics().iterator(); it.hasNext();) {
+            Metric metric = (Metric) it.next();
+            int newMetricID = -1;
+            if (saveMetricIndex < 0 || saveMetricIndex == idx) {
                 newMetricID = saveMetric(newTrialID, metric);
             }
-            metricHash.put(new Integer(i), new Integer(newMetricID));
-            i++;
+            metricHash.put(new Integer(idx), new Integer(newMetricID));
+            idx++;
         }
         return metricHash;
     }
@@ -705,7 +776,7 @@ public class DatabaseAPI {
      * @return the database index ID of the saved trial record
      */
 
-    public synchronized int saveTrial(Trial trial, int saveMetricIndex) throws DatabaseException {
+    public synchronized int saveTrial(Trial trial, Metric saveMetric) throws DatabaseException {
         long start = System.currentTimeMillis();
 
         DataSource dataSource = trial.getDataSource();
@@ -751,8 +822,9 @@ public class DatabaseAPI {
                 StringBuffer buf = new StringBuffer();
                 if (groups != null) {
                     for (int i = 0; i < groups.size(); i++) {
-                        if (i > 0)
+                        if (i > 0) {
                             buf.append("|");
+                        }
                         buf.append(((Group) groups.get(i)).getName());
                     }
 
@@ -803,7 +875,6 @@ public class DatabaseAPI {
         for (Iterator it = trial.getDataSource().getAllThreads().iterator(); it.hasNext();) {
             edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it.next();
             List intervalEvents = thread.getFunctionProfiles();
-            List userevents = thread.getUserEventProfiles();
 
             // create interval location profiles
             for (Iterator e4 = intervalEvents.iterator(); e4.hasNext();) {
@@ -828,23 +899,22 @@ public class DatabaseAPI {
             }
 
             // create atomic events
-            if (userevents != null) {
-                for (Iterator e4 = userevents.iterator(); e4.hasNext();) {
-                    UserEventProfile uep = (UserEventProfile) e4.next();
-                    if (uep != null) {
 
-                        AtomicLocationProfile udo = new AtomicLocationProfile();
-                        udo.setAtomicEventID(uep.getUserEvent().getID());
-                        udo.setNode(thread.getNodeID());
-                        udo.setContext(thread.getContextID());
-                        udo.setThread(thread.getThreadID());
-                        udo.setSampleCount((int) uep.getNumSamples());
-                        udo.setMaximumValue(uep.getMaxValue());
-                        udo.setMinimumValue(uep.getMinValue());
-                        udo.setMeanValue(uep.getMeanValue());
-                        udo.setSumSquared(uep.getSumSquared());
-                        atomicEventData.add(udo);
-                    }
+            for (Iterator e4 = thread.getUserEventProfiles(); e4.hasNext();) {
+                UserEventProfile uep = (UserEventProfile) e4.next();
+                if (uep != null) {
+
+                    AtomicLocationProfile udo = new AtomicLocationProfile();
+                    udo.setAtomicEventID(uep.getUserEvent().getID());
+                    udo.setNode(thread.getNodeID());
+                    udo.setContext(thread.getContextID());
+                    udo.setThread(thread.getThreadID());
+                    udo.setSampleCount((int) uep.getNumSamples());
+                    udo.setMaximumValue(uep.getMaxValue());
+                    udo.setMinimumValue(uep.getMinValue());
+                    udo.setMeanValue(uep.getMeanValue());
+                    udo.setSumSquared(uep.getSumSquared());
+                    atomicEventData.add(udo);
                 }
             }
         }
@@ -861,17 +931,23 @@ public class DatabaseAPI {
 
         int newTrialID = 0;
 
+        int saveMetricIndex = -1;
+        if (saveMetric != null) {
+            saveMetricIndex = saveMetric.getID();
+        }
+
         try {
             // output the trial data, which also saves the intervalEvents,
             // intervalEvent data, user events and user event data
-            if (saveMetricIndex < 0) { // this means save the whole thing???
+
+            Hashtable metricHash = null;
+            if (saveMetric == null) { // this means save the whole thing???
                 newTrialID = trial.saveTrial(db);
                 trial.setID(newTrialID);
-                Hashtable metricHash = saveMetrics(newTrialID, trial, saveMetricIndex);
+                metricHash = saveMetrics(newTrialID, trial, saveMetricIndex);
 
                 if (intervalEvents != null && intervalEvents.size() > 0) {
                     Hashtable functionHash = saveIntervalEvents(newTrialID, metricHash, saveMetricIndex);
-
                     saveIntervalLocationProfiles(db, functionHash, intervalEventData.elements(), metricHash, saveMetricIndex);
                 }
                 if (atomicEvents != null && atomicEvents.size() > 0) {
@@ -881,19 +957,29 @@ public class DatabaseAPI {
                     }
                 }
 
-                //  System.out.println("New Trial ID: " + newTrialID);
             } else {
                 newTrialID = trial.getID();
-                //   System.out.println("\nSaving the metric...");
-                Hashtable newMetHash = saveMetrics(newTrialID, trial, saveMetricIndex);
+                metricHash = saveMetrics(newTrialID, trial, saveMetricIndex);
 
                 if (intervalEvents != null && intervalEvents.size() > 0) {
-                    Hashtable newFunHash = saveIntervalEvents(newTrialID, newMetHash, saveMetricIndex);
-                    saveIntervalLocationProfiles(db, newFunHash, intervalEventData.elements(), newMetHash, saveMetricIndex);
+                    Hashtable newFunHash = saveIntervalEvents(newTrialID, metricHash, saveMetricIndex);
+                    saveIntervalLocationProfiles(db, newFunHash, intervalEventData.elements(), metricHash, saveMetricIndex);
 
                 }
+            }
 
-                //   System.out.println("Modified Trial ID: " + newTrialID);
+            for (Iterator it = metricHash.keySet().iterator(); it.hasNext();) {
+                Integer key = (Integer) it.next();
+                int value = ((Integer) metricHash.get(key)).intValue();
+
+                for (Iterator it2 = trial.getDataSource().getMetrics().iterator(); it2.hasNext();) {
+                    Metric metric = (Metric) it2.next();
+                    if (metric.getID() == key.intValue()) {
+                        if (value != -1) {
+                            metric.setDbMetricID(value);
+                        }
+                    }
+                }
             }
 
         } catch (SQLException e) {
@@ -1071,24 +1157,25 @@ public class DatabaseAPI {
         this.itemsDone++;
 
         //stmt.addBatch();
-//        try {
-            stmt.executeUpdate();
-//        } catch (Exception e) {
-//            e.printStackTrace();
-//            System.out.println(e);
-//            System.out.println(stmt.toString());
-//            System.out.println("exclusive: " + fp.getExclusive(metricID));
-//            System.out.println("exclusive percent: " + fp.getExclusivePercent(metricID));
-//            System.out.println("inclusive: " + fp.getInclusive(metricID));
-//            System.out.println("inclusive percent: " + fp.getExclusivePercent(metricID));
-//            System.out.println("numThreads: " + numThreads);
-//            System.out.println("numcalls: " + fp.getNumCalls());
-//            System.out.println("numsubr: " + fp.getNumSubr());
-//            System.out.println("inclusivepercall: " + fp.getInclusivePerCall(metricID));
-//        }
+        //        try {
+        stmt.executeUpdate();
+        //        } catch (Exception e) {
+        //            e.printStackTrace();
+        //            System.out.println(e);
+        //            System.out.println(stmt.toString());
+        //            System.out.println("exclusive: " + fp.getExclusive(metricID));
+        //            System.out.println("exclusive percent: " + fp.getExclusivePercent(metricID));
+        //            System.out.println("inclusive: " + fp.getInclusive(metricID));
+        //            System.out.println("inclusive percent: " + fp.getExclusivePercent(metricID));
+        //            System.out.println("numThreads: " + numThreads);
+        //            System.out.println("numcalls: " + fp.getNumCalls());
+        //            System.out.println("numsubr: " + fp.getNumSubr());
+        //            System.out.println("inclusivepercall: " + fp.getInclusivePerCall(metricID));
+        //        }
     }
 
-    private void uploadFunctionProfiles(int trialID, DataSource dataSource, Map functionMap, Map metricMap) throws SQLException {
+    private void uploadFunctionProfiles(int trialID, DataSource dataSource, Map functionMap, Map metricMap, boolean summaryOnly)
+            throws SQLException {
 
         PreparedStatement totalInsertStatement = null;
         PreparedStatement meanInsertStatement = null;
@@ -1160,7 +1247,7 @@ public class DatabaseAPI {
                 addBatchFunctionProfile(meanInsertStatement, totalData, metric.getID(), dbMetricID.intValue(),
                         function.getTotalProfile(), intervalEventID.intValue(), true, dataSource.getAllThreads().size());
 
-                for (Iterator it = dataSource.getAllThreads().iterator(); it.hasNext();) {
+                for (Iterator it = dataSource.getAllThreads().iterator(); it.hasNext() && summaryOnly == false;) {
                     edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it.next();
 
                     FunctionProfile fp = thread.getFunctionProfile(function);
@@ -1195,7 +1282,7 @@ public class DatabaseAPI {
                 for (Iterator it3 = context.getThreads(); it3.hasNext();) {
                     edu.uoregon.tau.perfdmf.Thread thread = (edu.uoregon.tau.perfdmf.Thread) it3.next();
 
-                    for (Iterator it4 = thread.getUserEventProfiles().iterator(); it4.hasNext();) {
+                    for (Iterator it4 = thread.getUserEventProfiles(); it4.hasNext();) {
                         UserEventProfile uep = (UserEventProfile) it4.next();
 
                         if (this.cancelUpload)
@@ -1255,6 +1342,10 @@ public class DatabaseAPI {
     }
 
     public synchronized int uploadTrial(Trial trial) throws DatabaseException {
+        return uploadTrial(trial, false);
+    }
+
+    public synchronized int uploadTrial(Trial trial, boolean summaryOnly) throws DatabaseException {
         long start = System.currentTimeMillis();
 
         DataSource dataSource = trial.getDataSource();
@@ -1273,11 +1364,18 @@ public class DatabaseAPI {
             trial.setID(newTrialID);
 
             computeUploadSize(dataSource);
-            // upload the metrics and get a map that maps the metrics 0 -> n-1 to their unique DB IDs
+            // upload the metrics and get a map that maps the metrics 0 -> n-1 to their unique DB IDs (e.g. 83, 84)
             Map metricMap = uploadMetrics(newTrialID, dataSource);
+
+            for (Iterator it = metricMap.keySet().iterator(); it.hasNext();) {
+                Metric key = (Metric) it.next();
+                int value = ((Integer) metricMap.get(key)).intValue();
+                key.setDbMetricID(value);
+            }
+
             Map functionMap = uploadFunctions(newTrialID, dataSource);
 
-            uploadFunctionProfiles(newTrialID, dataSource, functionMap, metricMap);
+            uploadFunctionProfiles(newTrialID, dataSource, functionMap, metricMap, summaryOnly);
 
             Map userEventMap = uploadUserEvents(newTrialID, dataSource);
 
@@ -1353,7 +1451,7 @@ public class DatabaseAPI {
         tmpSession.setDB(this.db());
 
         tmpSession.setExperiment(experimentID);
-        ListIterator trials = tmpSession.getTrialList().listIterator();
+        ListIterator trials = tmpSession.getTrialList(false).listIterator();
         while (trials.hasNext()) {
             Trial trial = (Trial) trials.next();
             Trial.deleteTrial(db, trial.getID());
@@ -1376,7 +1474,7 @@ public class DatabaseAPI {
         while (experiments.hasNext()) {
             Experiment experiment = (Experiment) experiments.next();
             tmpSession.setExperiment(experiment.getID());
-            ListIterator trials = tmpSession.getTrialList().listIterator();
+            ListIterator trials = tmpSession.getTrialList(false).listIterator();
             while (trials.hasNext()) {
                 Trial trial = (Trial) trials.next();
                 Trial.deleteTrial(db, trial.getID());
@@ -1423,7 +1521,7 @@ public class DatabaseAPI {
             statement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
                     + "interval_location_profile (interval_event, node, context, thread, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
-        } else if (db.getDBType().compareTo("derby") == 0) {
+        } else if (db.getDBType().compareTo("mysql") == 0) {
             statement = db.prepareStatement("INSERT INTO "
                     + db.getSchemaPrefix()
                     + "interval_location_profile (interval_event, node, context, thread, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, `call`, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)");
@@ -1476,6 +1574,7 @@ public class DatabaseAPI {
         // didn't find one with that name
         if (create) {
             Application newApp = new Application();
+            newApp.setDatabase(database);
             newApp.setName(name);
             setApplication(newApp);
             int appId = saveApplication();
@@ -1499,6 +1598,7 @@ public class DatabaseAPI {
             Experiment newExp = new Experiment();
             newExp.setName(name);
             newExp.setApplicationID(app.getID());
+            newExp.setDatabase(database);
             setExperiment(newExp);
             int expId = saveExperiment();
             newExp.setID(expId);
@@ -1508,6 +1608,26 @@ public class DatabaseAPI {
         return null;
     }
 
+    public Trial getTrial(Application app, Experiment exp, String name) {
+        setApplication(app);
+        List exps = getExperimentList();
+        for (Iterator it = exps.iterator(); it.hasNext();) {
+            Experiment tmp = (Experiment) it.next();
+            if (tmp.getName().equals(name)) {
+                setExperiment(tmp);
+                List trials = getTrialList(false);
+                for (Iterator it2 = trials.iterator(); it2.hasNext();) {
+                    Trial trial = (Trial) it2.next();
+                    if (exp.getName().equals(name)) {
+                        return trial;
+                    }
+                }
+            }
+        }
+
+        return null;
+    }
+
     public Experiment getExperiment(String appName, String expName, boolean create) {
         Application app = getApplication(appName, create);
         if (app == null) {
