@@ -5,9 +5,9 @@ import java.util.*;
 /**
  * This class represents a single function profile on a single thread.
  *
- * <P>CVS $Id: FunctionProfile.java,v 1.8 2007/03/20 17:03:02 amorris Exp $</P>
+ * <P>CVS $Id: FunctionProfile.java,v 1.15 2010/01/14 01:29:16 amorris Exp $</P>
  * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.8 $
+ * @version	$Revision: 1.15 $
  * @see		Function
  */
 public class FunctionProfile {
@@ -43,7 +43,8 @@ public class FunctionProfile {
 
     public FunctionProfile(Function function, int numMetrics, int snapshots) {
         numMetrics = Math.max(numMetrics, 1);
-        data = new double[((numMetrics + 1) * METRIC_SIZE) * snapshots];
+        data = new double[((numMetrics + 1) * METRIC_SIZE + 2) * snapshots];
+
         this.function = function;
     }
 
@@ -67,7 +68,7 @@ public class FunctionProfile {
         return this.getDouble(metric, INCLUSIVE);
     }
 
-    public double getInclusive(int metric, int snapshot) {
+    public double getInclusive(int snapshot, int metric) {
         return this.getDouble(snapshot, metric, INCLUSIVE);
     }
 
@@ -84,66 +85,91 @@ public class FunctionProfile {
     }
 
     public double getExclusive(int metric) {
-        //        if (function.isPhase()) {
-        //            return this.getDouble(metric, 0);
-        //        } else {
-        //            return this.getDouble(metric, 1);
-        //        }
-        return this.getDouble(metric, EXCLUSIVE);
+        double value = this.getDouble(metric, EXCLUSIVE);
+        return value;
     }
 
-    public void setInclusivePercent(int metric, double value) {
-        //this.putDouble(thread.getNumSnapshots() - 1, metric, INCLUSIVE_PERCENT, value);
+    // TODO: A bunch of these methods are not using the snapshot
+    public double getInclusivePercent(int snapshot, int metric) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+        if (thread.getNodeID() >= 0) {
+            double dividend = thread.getPercentDivider(metric, snapshot);
+            if (dividend == 0) {
+                return 0;
+            }
+            return getInclusive(snapshot, metric) / dividend;
+        } else if (thread.getNodeID() == Thread.TOTAL || thread.getNodeID() == Thread.MEAN) {
+            double dividend = thread.getPercentDivider(metric, snapshot);
+            if (dividend == 0) {
+                return 0;
+            }
+            return function.getTotalProfile().getInclusive(snapshot, metric) / dividend;
+        } else if (thread.getNodeID() == Thread.STDDEV) {
+            return getInclusive(snapshot, metric) / function.getMeanInclusive(metric) * 100.0;
+        }
+        throw new RuntimeException("Bad Thread ID = " + thread);
+
     }
 
     public double getInclusivePercent(int metric) {
-        if (thread.getNodeID() >= 0) {
-            double dividend = thread.getPercentDivider(metric);
-            if (dividend == 0) {
-                return 0;
-            }
-            return getInclusive(metric) / dividend;
-        } else if (thread.getNodeID() == Thread.TOTAL || thread.getNodeID() == Thread.MEAN) {
-            double dividend = thread.getPercentDivider(metric);
-            if (dividend == 0) {
-                return 0;
-            }
-            return function.getTotalInclusive(metric) / dividend;
-        } else if (thread.getNodeID() == Thread.STDDEV) {
-            return getInclusive(metric) / function.getMeanInclusive(metric) * 100.0;
+        return getInclusivePercent(-1, metric);
+    }
+
+    // TODO: A bunch of these methods are not using the snapshot
+    public double getExclusivePercent(int snapshot, int metric) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
         }
-        throw new RuntimeException("Bad Thread ID = " + thread);
-    }
-
-    public void setExclusivePercent(int metric, double value) {
-        //this.putDouble(thread.getNumSnapshots() - 1, metric, EXCLUSIVE_PERCENT, value);
-    }
-
-    public double getExclusivePercent(int metric) {
         if (thread.getNodeID() >= 0) {
-            double dividend = thread.getPercentDivider(metric);
+            double dividend = thread.getPercentDivider(metric, snapshot);
             if (dividend == 0) {
                 return 0;
             }
-            return getExclusive(metric) / dividend;
+            return getExclusive(snapshot, metric) / dividend;
         } else if (thread.getNodeID() == Thread.TOTAL || thread.getNodeID() == Thread.MEAN) {
-            double dividend = thread.getPercentDivider(metric);
+            double dividend = thread.getPercentDivider(metric, snapshot);
             if (dividend == 0) {
                 return 0;
             }
-            return function.getTotalExclusive(metric) / dividend;
+            if (thread.getNodeID() == Thread.TOTAL) {
+                return function.getTotalProfile().getExclusive(snapshot, metric) / dividend;
+            } else {
+                dividend /= thread.getDataSource().getAllThreads().size();
+                return function.getMeanProfile().getExclusive(snapshot, metric) / dividend;
+            }
         } else if (thread.getNodeID() == Thread.STDDEV) {
             return getExclusive(metric) / function.getMeanExclusive(metric) * 100.0;
         }
         throw new RuntimeException("Bad Thread ID = " + thread);
     }
 
+    public double getExclusivePercent(int metric) {
+        return getExclusivePercent(-1, metric);
+    }
+
+    public void setNumCalls(int snapshot, double value) {
+        this.putDouble(snapshot, 0, CALLS, value);
+    }
+
     public void setNumCalls(double value) {
         this.putDouble(0, CALLS, value);
     }
 
     public double getNumCalls() {
-        return getDouble(0, CALLS);
+        return getNumCalls(-1);
+    }
+
+    public double getNumCalls(int snapshot) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+        return getDouble(snapshot, 0, CALLS);
+    }
+
+    public void setNumSubr(int snapshot, double value) {
+        this.putDouble(snapshot, 0, SUBR, value);
     }
 
     public void setNumSubr(double value) {
@@ -151,21 +177,42 @@ public class FunctionProfile {
     }
 
     public double getNumSubr() {
-        return getDouble(0, SUBR);
+        return getNumSubr(-1);
+    }
+
+    public double getNumSubr(int snapshot) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+        return getDouble(snapshot, 0, SUBR);
     }
 
     public double getInclusivePerCall(int metric) {
-        if (this.getNumCalls() == 0) {
+        return getInclusivePerCall(-1, metric);
+    }
+
+    public double getInclusivePerCall(int snapshot, int metric) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+        if (this.getNumCalls(snapshot) == 0) {
             return 0;
         }
-        return this.getInclusive(metric) / this.getNumCalls();
+        return this.getInclusive(snapshot, metric) / this.getNumCalls(snapshot);
     }
 
     public double getExclusivePerCall(int metric) {
-        if (this.getNumCalls() == 0) {
+        return getExclusivePerCall(-1, metric);
+    }
+
+    public double getExclusivePerCall(int snapshot, int metric) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+        if (this.getNumCalls(snapshot) == 0) {
             return 0;
         }
-        return this.getExclusive(metric) / this.getNumCalls();
+        return this.getExclusive(snapshot, metric) / this.getNumCalls(snapshot);
     }
 
     public void addMetric() {
@@ -176,15 +223,12 @@ public class FunctionProfile {
         double[] newArray = new double[(newMetricSize + 1) * METRIC_SIZE * numSnapshots];
 
         for (int s = 0; s < numSnapshots; s++) {
-
             int source = (s * METRIC_SIZE * (numMetrics + 1));
             int dest = (s * METRIC_SIZE * (newMetricSize + 1));
             for (int m = 0; m < METRIC_SIZE * (numMetrics + 1); m++) {
                 newArray[dest + m] = data[source + m];
             }
-
         }
-
         data = newArray;
     }
 
@@ -312,12 +356,17 @@ public class FunctionProfile {
     }
 
     private double getDouble(int snapshot, int metric, int offset) {
+        if (snapshot == -1) {
+            snapshot = thread.getNumSnapshots() - 1;
+        }
+
         int numMetrics = thread.getNumMetrics();
         int location = (snapshot * (METRIC_SIZE * (numMetrics + 1))) + (metric * METRIC_SIZE) + offset;
         return data[location];
     }
 
     private double getDouble(int metric, int offset) {
+        // use the last snapshot (final value)
         int snapshot = thread.getNumSnapshots() - 1;
         int numMetrics = thread.getNumMetrics();
         int location = (snapshot * (METRIC_SIZE * (numMetrics + 1))) + (metric * METRIC_SIZE) + offset;
