@@ -1,12 +1,16 @@
 package edu.uoregon.tau.perfdmf;
 
-import edu.uoregon.tau.perfdmf.database.DB;
-
-import java.util.Vector;
-import java.util.Hashtable;
-import java.sql.SQLException;
+import java.sql.DatabaseMetaData;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
+import java.sql.ResultSetMetaData;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.Hashtable;
+import java.util.List;
+import java.util.Vector;
+
+import edu.uoregon.tau.perfdmf.database.DB;
 
 /**
  * Holds all the data for an interval_event in the database.
@@ -25,7 +29,7 @@ import java.sql.ResultSet;
  * index of the metric in the Trial object should be used to indicate which total/mean
  * summary object to return.
  *
- * <P>CVS $Id: IntervalEvent.java,v 1.3 2006/04/10 19:55:50 khuck Exp $</P>
+ * <P>CVS $Id: IntervalEvent.java,v 1.8 2009/11/13 00:11:41 amorris Exp $</P>
  * @author	Kevin Huck, Robert Bell
  * @version	0.1
  * @since	0.1
@@ -208,7 +212,7 @@ public class IntervalEvent {
         if (db.getDBType().compareTo("oracle") == 0) {
             buf.append(" order by dbms_lob.substr(name) asc");
         } else if (db.getDBType().compareTo("derby") == 0) {
-            buf.append(" order by cast (name as varchar(256)) asc");
+            buf.append(" order by cast (name as varchar(4000)) asc");
         } else if (db.getDBType().compareTo("db2") == 0) {
             buf.append(" order by cast (name as varchar(256)) asc");
         } else {
@@ -241,7 +245,7 @@ public class IntervalEvent {
 
     public int saveIntervalEvent(DB db, int newTrialID, Hashtable newMetHash, int saveMetricIndex)
             throws SQLException {
-        int newIntervalEventID = 0;
+        int newIntervalEventID = -1;
 
         PreparedStatement statement = null;
         if (saveMetricIndex < 0) {
@@ -270,15 +274,16 @@ public class IntervalEvent {
 
             if (db.getDBType().compareTo("oracle") == 0)
                 statement = db.prepareStatement("SELECT id FROM " + db.getSchemaPrefix()
-                        + "interval_event where dbms_lob.instr(name, ?) > 0");
+                        + "interval_event where dbms_lob.instr(name, ?) > 0 and trial = ?");
             else if (db.getDBType().compareTo("derby") == 0)
                 statement = db.prepareStatement("SELECT id FROM " + db.getSchemaPrefix()
-                        + "interval_event where cast(name as varchar(256) = ?");
+                        + "interval_event where cast(name as varchar(4000)) = ? and trial = ?");
             else
                 statement = db.prepareStatement("SELECT id FROM " + db.getSchemaPrefix()
-                        + "interval_event where name = ?");
+                        + "interval_event where name = ? and trial = ?");
 
             statement.setString(1, name);
+            statement.setInt(2, newTrialID);
             ResultSet resultSet = statement.executeQuery();
             while (resultSet.next() != false) {
                 newIntervalEventID = resultSet.getInt(1);
@@ -286,6 +291,10 @@ public class IntervalEvent {
             resultSet.close();
             statement.close();
         }
+        
+        if (newIntervalEventID == -1) {
+            throw new RuntimeException("Unable to find event in database, event: " + name, null);
+        }
 
         // save the intervalEvent mean summary
         if (meanSummary != null) {
@@ -298,4 +307,79 @@ public class IntervalEvent {
         }
         return newIntervalEventID;
     }
+    
+    public static void getMetaData(DB db) {
+        // see if we've already have them
+        // need to load each time in case we are working with a new database. 
+        //        if (Trial.fieldNames != null)
+        //            return;
+
+        try {
+            ResultSet resultSet = null;
+
+            String trialFieldNames[] = null;
+            int trialFieldTypes[] = null;
+
+            DatabaseMetaData dbMeta = db.getMetaData();
+
+            if ((db.getDBType().compareTo("oracle") == 0) || (db.getDBType().compareTo("derby") == 0)
+                    || (db.getDBType().compareTo("db2") == 0)) {
+                resultSet = dbMeta.getColumns(null, null, "INTERVAL_EVENT", "%");
+            } else {
+                resultSet = dbMeta.getColumns(null, null, "interval_event", "%");
+            }
+
+            Vector nameList = new Vector();
+            Vector typeList = new Vector();
+            List typeNames = new ArrayList();
+            List columnSizes = new ArrayList();
+            boolean seenID = false;
+
+            ResultSetMetaData md = resultSet.getMetaData();
+            for (int i = 0 ; i < md.getColumnCount() ; i++) {
+            	//System.out.println(md.getColumnName(i));
+            }
+
+            while (resultSet.next() != false) {
+
+                int ctype = resultSet.getInt("DATA_TYPE");
+                String cname = resultSet.getString("COLUMN_NAME");
+                String typename = resultSet.getString("TYPE_NAME");
+                Integer size = new Integer(resultSet.getInt("COLUMN_SIZE"));
+
+                // this code is because of a bug in derby...
+                if (cname.equals("ID")) {
+                    if (!seenID)
+                        seenID = true;
+                    else
+                        break;
+                }
+
+                nameList.add(resultSet.getString("COLUMN_NAME"));
+                typeList.add(new Integer(ctype));
+                typeNames.add(typename);
+                columnSizes.add(size);
+            }
+            resultSet.close();
+
+            String[] fieldNames = new String[nameList.size()];
+            int[] fieldTypes = new int[typeList.size()];
+            String[] fieldTypeNames = new String[typeList.size()];
+            for (int i = 0; i < typeList.size(); i++) {
+                fieldNames[i] = (String) nameList.get(i);
+                fieldTypes[i] = ((Integer) typeList.get(i)).intValue();
+                if (((Integer)columnSizes.get(i)).intValue() > 255) {
+                    fieldTypeNames[i] = (String) typeNames.get(i) + "(" + columnSizes.get(i).toString() + ")";
+                } else {
+                    fieldTypeNames[i] = (String) typeNames.get(i);
+                }
+            }
+
+            db.getDatabase().setIntervalEventFieldNames(fieldNames);
+            db.getDatabase().setIntervalEventFieldTypeNames(fieldTypeNames);
+        } catch (SQLException e) {
+            e.printStackTrace();
+        }
+    }
+
 }
