@@ -1,13 +1,12 @@
 package edu.uoregon.tau.perfdmf;
 
-import edu.uoregon.tau.perfdmf.database.DB;
-
-import java.sql.SQLException;
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
-import java.util.Vector;
+import java.sql.SQLException;
 import java.util.Hashtable;
-import java.util.Enumeration;
+import java.util.Vector;
+
+import edu.uoregon.tau.perfdmf.database.DB;
 
 /**
  * Holds all the data for a interval location profile in the database.
@@ -26,7 +25,7 @@ import java.util.Enumeration;
  * passed in to get data for a particular metric.  If there is only one metric, then no metric
  * index need be passed in.
  *
- * <P>CVS $Id: IntervalLocationProfile.java,v 1.4 2006/12/28 03:05:59 amorris Exp $</P>
+ * <P>CVS $Id: IntervalLocationProfile.java,v 1.6 2009/02/19 20:53:44 amorris Exp $</P>
  * @author	Kevin Huck, Robert Bell
  * @version	0.1
  * @since	0.1
@@ -531,7 +530,7 @@ public class IntervalLocationProfile extends Object {
                 } else if (db.getDBType().compareTo("derby") == 0) {
                     statement = db.prepareStatement("INSERT INTO "
                             + db.getSchemaPrefix()
-                            + "interval_mean_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, excl, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+                            + "interval_mean_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
                 } else if (db.getDBType().compareTo("mysql") == 0) {
                     statement = db.prepareStatement("INSERT INTO "
                             + db.getSchemaPrefix()
@@ -574,7 +573,7 @@ public class IntervalLocationProfile extends Object {
                 } else if (db.getDBType().compareTo("derby") == 0) {
                     statement = db.prepareStatement("INSERT INTO "
                             + db.getSchemaPrefix()
-                            + "interval_total_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, excl, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
+                            + "interval_total_summary (interval_event, metric, inclusive_percentage, inclusive, exclusive_percentage, exclusive, num_calls, subroutines, inclusive_per_call) VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?)");
                 } else if (db.getDBType().compareTo("mysql") == 0) {
                     statement = db.prepareStatement("INSERT INTO "
                             + db.getSchemaPrefix()
