@@ -1,15 +1,13 @@
 package edu.uoregon.tau.perfdmf;
 
-import java.util.Iterator;
-import java.util.Map;
-import java.util.TreeMap;
+import java.util.*;
 
 /**
  * This class represents a Node.  It contains a set of Contexts and an ID.
  *  
- * <P>CVS $Id: Node.java,v 1.2 2007/05/02 19:43:28 amorris Exp $</P>
+ * <P>CVS $Id: Node.java,v 1.4 2009/06/01 18:38:26 amorris Exp $</P>
  * @author	Robert Bell, Alan Morris
- * @version	$Revision: 1.2 $
+ * @version	$Revision: 1.4 $
  * @see		DataSource
  * @see		Context
  */
@@ -17,14 +15,16 @@ public class Node implements Comparable {
 
     private int nodeID = -1;
     private Map contexts = new TreeMap();
+    private DataSource dataSource;
 
     /**
      * Creates a Node with the given ID.  This constructor is not public because Nodes should 
      * only be created by DataSource.addNode(...)
      * @param 	nodeID		ID of this Node
      */
-    Node(int nodeID) {
+    Node(int nodeID, DataSource dataSource) {
         this.nodeID = nodeID;
+        this.dataSource = dataSource;
     }
 
     /**
@@ -49,7 +49,7 @@ public class Node implements Comparable {
             return (Context) obj;
 
         // otherwise, add it and return it
-        Context context = new Context(this.nodeID, contextID);
+        Context context = new Context(this.nodeID, contextID, dataSource);
         contexts.put(new Integer(contextID), context);
         return context;
     }
@@ -85,4 +85,17 @@ public class Node implements Comparable {
     public int compareTo(Object obj) {
         return nodeID - ((Node) obj).getNodeID();
     }
+
+    public List getThreads() {
+        List list = new ArrayList();
+        for (Iterator it = getContexts(); it.hasNext();) {
+            Context context = (Context) it.next();
+            for (Iterator it2 = context.getThreads(); it2.hasNext();) {
+                Thread thread = (Thread) it2.next();
+                list.add(thread);
+            }
+        }
+        return list;
+    }
+
 }
