@@ -4,19 +4,18 @@ import java.util.*;
 
 import org.xml.sax.Attributes;
 import org.xml.sax.SAXException;
+import org.xml.sax.SAXParseException;
 import org.xml.sax.helpers.DefaultHandler;
 
 /**
  * XML Handler for snapshot profiles, this is where all the work is done
  *
- * <P>CVS $Id: SnapshotXMLHandler.java,v 1.9 2007/03/26 21:04:11 amorris Exp $</P>
+ * <P>CVS $Id: SnapshotXMLHandler.java,v 1.13 2009/05/06 19:50:59 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.9 $
+ * @version $Revision: 1.13 $
  */
 public class SnapshotXMLHandler extends DefaultHandler {
 
-   
-
     private SnapshotDataSource dataSource;
 
     private Map threadMap = new HashMap();
@@ -34,24 +33,29 @@ public class SnapshotXMLHandler extends DefaultHandler {
     private String currentValue;
     private String currentGroup;
     private long currentTimestamp;
-    
+
     private static class ThreadData {
         public Thread thread;
         public Map metricMap = new HashMap();
         public Map eventMap = new HashMap();
+        public Map userEventMap = new HashMap();
     }
 
     public SnapshotXMLHandler(SnapshotDataSource source) {
         this.dataSource = source;
     }
 
-    public void startDocument() throws SAXException {
-        //System.out.println("startDocument");
+    public void warning(SAXParseException e) throws SAXException {}
+
+    public void error(SAXParseException e) throws SAXException {}
+
+    public void fatalError(SAXParseException e) throws SAXException {
+        throw e;
     }
 
-    public void endDocument() throws SAXException {
-        //System.out.println("endDocument");
-    }
+    public void startDocument() throws SAXException {}
+
+    public void endDocument() throws SAXException {}
 
     private void handleMetric(String name) {
         Metric metric = dataSource.addMetric(name);
@@ -66,6 +70,12 @@ public class SnapshotXMLHandler extends DefaultHandler {
         currentThread.eventMap.put(new Integer(id), function);
     }
 
+    private void handleUserEvent(String name) {
+        int id = currentId;
+        UserEvent userEvent = dataSource.addUserEvent(name);
+        currentThread.userEventMap.put(new Integer(id), userEvent);
+    }
+
     private void handleThread(Attributes attributes) {
         String threadName = attributes.getValue("id");
         int nodeID = Integer.parseInt(attributes.getValue("node"));
@@ -102,6 +112,34 @@ public class SnapshotXMLHandler extends DefaultHandler {
         }
     }
 
+    private void handleAtomicDataEnd() {
+        String data = accumulator.toString();
+
+        StringTokenizer tokenizer = new StringTokenizer(data, " \t\n\r");
+
+        while (tokenizer.hasMoreTokens()) {
+            int eventID = Integer.parseInt(tokenizer.nextToken());
+            UserEvent userEvent = (UserEvent) currentThread.userEventMap.get(new Integer(eventID));
+
+            UserEventProfile uep = currentThread.thread.getUserEventProfile(userEvent);
+            if (uep == null) {
+                uep = new UserEventProfile(userEvent, currentThread.thread.getSnapshots().size());
+                currentThread.thread.addUserEventProfile(uep);
+            }
+
+            double numSamples = Double.parseDouble(tokenizer.nextToken());
+            double max = Double.parseDouble(tokenizer.nextToken());
+            double min = Double.parseDouble(tokenizer.nextToken());
+            double mean = Double.parseDouble(tokenizer.nextToken());
+            double sumSqr = Double.parseDouble(tokenizer.nextToken());
+            uep.setNumSamples(numSamples);
+            uep.setMaxValue(max);
+            uep.setMinValue(min);
+            uep.setMeanValue(mean);
+            uep.setSumSquared(sumSqr);
+        }
+    }
+
     private void handleIntervalDataEnd() {
         String data = accumulator.toString();
 
@@ -132,10 +170,7 @@ public class SnapshotXMLHandler extends DefaultHandler {
 
             fp.setNumCalls(numcalls);
             fp.setNumSubr(numsubr);
-
-            //System.out.println("item = "+ item);
         }
-
     }
 
     public void startElement(String uri, String localName, String qName, Attributes attributes) throws SAXException {
@@ -159,14 +194,16 @@ public class SnapshotXMLHandler extends DefaultHandler {
             currentId = Integer.parseInt(attributes.getValue("id"));
         } else if (localName.equals("event")) {
             currentId = Integer.parseInt(attributes.getValue("id"));
+        } else if (localName.equals("userevent")) {
+            currentId = Integer.parseInt(attributes.getValue("id"));
         } else if (localName.equals("profile")) {
             handleProfile(attributes);
         } else if (localName.equals("interval_data")) {
             handleIntervalData(attributes);
             accumulator = new StringBuffer();
+        } else if (localName.equals("atomic_data")) {
+            accumulator = new StringBuffer();
         }
-        
-        
 
     }
 
@@ -181,10 +218,9 @@ public class SnapshotXMLHandler extends DefaultHandler {
         } else if (localName.equals("utc_date")) {
             try {
                 currentDate = DataSource.dateTime.parse(accumulator.toString());
-            } catch (java.text.ParseException e) {
-            }
+            } catch (java.text.ParseException e) {}
         } else if (localName.equals("timestamp")) {
-            currentTimestamp = Long.parseLong(accumulator.toString()); 
+            currentTimestamp = Long.parseLong(accumulator.toString());
         } else if (localName.equals("group")) {
             currentGroup = accumulator.toString();
         } else if (localName.equals("profile")) {
@@ -194,13 +230,16 @@ public class SnapshotXMLHandler extends DefaultHandler {
             handleMetric(currentName);
         } else if (localName.equals("event")) {
             handleEvent(currentName, currentGroup);
+        } else if (localName.equals("userevent")) {
+            handleUserEvent(currentName);
         } else if (localName.equals("attribute")) {
-            currentThread.thread.getMetaData().put(currentName,currentValue);
+            currentThread.thread.getMetaData().put(currentName, currentValue);
         } else if (localName.equals("interval_data")) {
             handleIntervalDataEnd();
+        } else if (localName.equals("atomic_data")) {
+            handleAtomicDataEnd();
         }
 
-
     }
 
     public void characters(char[] ch, int start, int length) throws SAXException {
