@@ -22,25 +22,55 @@ import java.nio.channels.FileChannel;
 import java.nio.channels.FileLock;
 import java.util.*;
 
+import edu.uoregon.tau.common.LineCountBufferedReader;
+import edu.uoregon.tau.common.Utility;
+
 public class TauDataSource extends DataSource {
 
+    public class CorruptFileException extends RuntimeException {
+        private String message;
+
+        public CorruptFileException(String message) {
+            this.message = message;
+        }
+
+        public String getMessage() {
+            return message;
+        }
+    };
+
     private volatile boolean abort = false;
     private volatile int totalFiles = 0;
     private volatile int filesRead = 0;
     private boolean profileStatsPresent = false;
     private boolean groupCheck = false;
     private List dirs; // list of directories (e.g. MULTI__PAPI_FP_INS, MULTI__PAPI_L1_DCM)
+    private int currFunction = 0;
 
     private File fileToMonitor;
 
+    private File currentFile;
+    private int currentLine;
+
+    private LineCountBufferedReader br;
+
     public TauDataSource(List dirs) {
         super();
         this.dirs = dirs;
 
         if (dirs.size() > 0) {
-            File[] files = (File[]) dirs.get(0);
-            if (files.length > 0) {
+            if (dirs.get(0) instanceof File[]) {
+                File[] files = (File[]) dirs.get(0);
+                if (files.length > 0) {
+                    fileToMonitor = files[0];
+                }
+            } else {
+                this.dirs = new ArrayList();
+                File[] files = new File[1];
+                files[0] = (File) dirs.get(0);
+                this.dirs.add(files);
                 fileToMonitor = files[0];
+                //System.out.println(files[0].toString());
             }
         }
     }
@@ -67,7 +97,7 @@ public class TauDataSource extends DataSource {
 
         boolean modernJava = false;
         try {
-            Method m = FileInputStream.class.getMethod("getChannel", null);
+            Method m = FileInputStream.class.getMethod("getChannel", (Class[]) null);
             modernJava = true;
         } catch (NoSuchMethodException nsme) {
             // way to go java 1.3
@@ -88,7 +118,6 @@ public class TauDataSource extends DataSource {
         // name rather than just checking whether this is the first file set. This is because we
         // might skip that first file (for example if the name were profile.-1.0.0) and thus skip
         // setting the metric name.
-        //Reference bug08.
 
         boolean metricNameProcessed = false;
 
@@ -126,9 +155,13 @@ public class TauDataSource extends DataSource {
                         int[] nct = getNCT(files[i].getName());
 
                         if (nct == null && dirs.size() == 1 && files.length == 1) {
-                            throw new DataSourceException(
-                                    files[i].getName()
-                                            + ": This doesn't look like a TAU profile\nDid you mean do use the -f option to specify a file format?");
+                            if (!files[i].exists()) {
+                                throw new DataSourceException("Error: File '" + files[i].getName() + "' does not exist.");
+                            } else {
+                                throw new DataSourceException("Error: File '" + files[i].getName()
+                                        + "': This doesn't look like a TAU profile\n"
+                                        + "Did you mean do use the -f option to specify a file format?");
+                            }
                         }
 
                         if (nct == null) {
@@ -145,10 +178,12 @@ public class TauDataSource extends DataSource {
                         foundValidFile = true;
 
                         FileInputStream fileIn = new FileInputStream(files[i]);
+                        currentFile = files[i];
+                        currentLine = 0;
                         FileChannel channel;
                         FileLock lock = null;
 
-                        if (modernJava) {
+                        if (modernJava && monitored) {
                             channel = fileIn.getChannel();
                             try {
                                 lock = channel.lock(0, Long.MAX_VALUE, true);
@@ -157,149 +192,165 @@ public class TauDataSource extends DataSource {
                                 lock = null;
                             }
                         }
-                        InputStreamReader inReader = new InputStreamReader(fileIn);
-                        BufferedReader br = new BufferedReader(inReader);
 
-                        // First Line (e.g. "601 templated_functions")
-                        inputString = br.readLine();
-                        if (inputString == null) {
-                            throw new DataSourceException("Unexpected end of file: " + files[i].getName()
-                                    + "\nLooking for 'templated_functions' line");
-                        }
-                        genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
-
-                        // the first token is the number of functions
-                        tokenString = genericTokenizer.nextToken();
-
-                        int numFunctions;
                         try {
-                            numFunctions = Integer.parseInt(tokenString);
-                        } catch (NumberFormatException nfe) {
-                            throw new DataSourceException(files[i].getName()
-                                    + ": Couldn't read number of functions, bad TAU Profile?");
-                        }
 
-                        // grab the (possible) metric name
-                        String metricName = getMetricName(inputString);
-
-                        // Second Line (e.g. "# Name Calls Subrs Excl Incl ProfileCalls")
-                        inputString = br.readLine();
-                        if (inputString == null) {
-                            throw new DataSourceException("Unexpected end of file: " + files[i].getName()
-                                    + "\nLooking for '# Name Calls ...' line");
-                        }
-
-                        if (inputString.indexOf("<metadata>") != -1) {
-                            int start = inputString.indexOf("<metadata>");
-                            int end = inputString.indexOf("</metadata>") + 11;
-                            String metadata = inputString.substring(start, end);
-                            MetaDataParser.parse(thread.getMetaData(), metadata);
-                        }
-
-                        // there may or may not be a metric name in the metadata
-                        String metaDataMetricName = (String) thread.getMetaData().get("Metric Name");
-                        
-                        // remove it if it was there
-                        thread.getMetaData().remove("Metric Name");
-                        
-                        if (metricNameProcessed == false) {
-                            if (metaDataMetricName != null) {
-                                metricName = metaDataMetricName;
-                            }
-                            //Set the metric name.
-                            if (metricName == null) {
-                                metricName = "Time";
-                            }
-                            this.addMetric(metricName);
-                            metricNameProcessed = true;
-                        }
-
-                        if (i == 0) {
-                            //Determine if profile stats or profile calls data is present.
-                            if (inputString.indexOf("SumExclSqr") != -1)
-                                this.setProfileStatsPresent(true);
-                        }
-
-                        for (int j = 0; j < numFunctions; j++) {
+                            InputStreamReader inReader = new InputStreamReader(fileIn);
+                            br = new LineCountBufferedReader(inReader);
 
+                            // First Line (e.g. "601 templated_functions")
                             inputString = br.readLine();
                             if (inputString == null) {
-                                throw new DataSourceException("Unexpected end of file: " + files[i].getName() + "\nOnly found "
-                                        + (j - 2) + " of " + numFunctions + " Function Lines");
+                                throw new CorruptFileException("Unexpected end of file: Looking for 'templated_functions' line");
+                            }
+                            genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
+
+                            // the first token is the number of functions
+                            tokenString = genericTokenizer.nextToken();
+
+                            int numFunctions;
+                            try {
+                                numFunctions = Integer.parseInt(tokenString);
+                            } catch (NumberFormatException nfe) {
+                                throw new CorruptFileException("Couldn't read number of functions");
                             }
 
-                            this.processFunctionLine(inputString, thread, metric);
+                            // grab the (possible) metric name
+                            String metricName = getMetricName(inputString);
 
-                            // unused profile calls
-
-                            //                        //Process the appropriate number of profile call lines.
-                            //                        for (int k = 0; k < functionDataLine.i2; k++) {
-                            //                            //this.setProfileCallsPresent(true);
-                            //                            inputString = br.readLine();
-                            //                            genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
-                            //                            //Arguments are evaluated left to right.
-                            //                            functionProfile.addCall(Double.parseDouble(genericTokenizer.nextToken()),
-                            //                                    Double.parseDouble(genericTokenizer.nextToken()));
-                            //                        }
-                        }
-
-                        //Process the appropriate number of aggregate lines.
-                        inputString = br.readLine();
-
-                        //A valid profile.*.*.* will always contain this line.
-                        if (inputString == null) {
-                            throw new DataSourceException("Unexpected end of file: " + files[i].getName()
-                                    + "\nLooking for 'aggregates' line");
-                        }
-                        genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
-                        //It's first token will be the number of aggregates.
-                        tokenString = genericTokenizer.nextToken();
-
-                        numFunctions = Integer.parseInt(tokenString);
-                        for (int j = 0; j < numFunctions; j++) {
-                            //this.setAggregatesPresent(true);
+                            // Second Line (e.g. "# Name Calls Subrs Excl Incl ProfileCalls")
                             inputString = br.readLine();
-                        }
-
-                        if (metric == 0) {
-                            //Process the appropriate number of userevent lines.
-                            inputString = br.readLine();
-                            if (inputString != null) {
-                                genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
-                                //It's first token will be the number of userEvents
-                                tokenString = genericTokenizer.nextToken();
-                                int numUserEvents = Integer.parseInt(tokenString);
-
-                                //Skip the heading (e.g. "# eventname numevents max min mean sumsqr")
-                                br.readLine();
-                                for (int j = 0; j < numUserEvents; j++) {
-                                    if (j == 0) {
-                                        setUserEventsPresent(true);
-                                    }
-
-                                    inputString = br.readLine();
-                                    if (inputString == null) {
-                                        throw new DataSourceException("Unexpected end of file: " + files[i].getName()
-                                                + "\nOnly found " + (j - 2) + " of " + numUserEvents + " User Event Lines");
-                                    }
-
-                                    this.processUserEventLine(inputString, thread);
+                            if (inputString == null) {
+                                throw new CorruptFileException("Unexpected end of file: Looking for '# Name Calls ...' line");
+                            }
 
+                            if (inputString.indexOf("<metadata>") != -1) {
+                                int start = inputString.indexOf("<metadata>");
+                                int end = inputString.indexOf("</metadata>") + 11;
+                                String metadata = inputString.substring(start, end);
+                                try {
+                                    MetaDataParser.parse(thread.getMetaData(), metadata);
+                                } catch (Exception exception) {
+                                    //exception.printStackTrace();
+                                    throw new CorruptFileException("Unable to parse metadata block");
                                 }
                             }
-                        }
 
-                        if (lock != null) {
-                            lock.release();
+                            // there may or may not be a metric name in the metadata
+                            String metaDataMetricName = (String) thread.getMetaData().get("Metric Name");
+
+                            // remove it if it was there
+                            thread.getMetaData().remove("Metric Name");
+
+                            if (metricNameProcessed == false) {
+                                if (metaDataMetricName != null) {
+                                    metricName = metaDataMetricName;
+                                }
+                                //Set the metric name.
+                                if (metricName == null) {
+                                    metricName = "Time";
+                                }
+                                this.addMetric(metricName);
+                                metricNameProcessed = true;
+                            }
+
+                            if (i == 0) {
+                                //Determine if profile stats or profile calls data is present.
+                                if (inputString.indexOf("SumExclSqr") != -1)
+                                    this.setProfileStatsPresent(true);
+                            }
+
+                            for (int j = 0; j < numFunctions; j++) {
+                                this.currFunction = j;
+
+                                inputString = br.readLine();
+                                if (inputString == null) {
+                                    throw new CorruptFileException("Unexpected end of file: Only found " + (j - 2) + " of "
+                                            + numFunctions + " Function Lines");
+                                }
+
+                                this.processFunctionLine(inputString, thread, metric);
+
+                                // unused profile calls
+
+                                //                        //Process the appropriate number of profile call lines.
+                                //                        for (int k = 0; k < functionDataLine.i2; k++) {
+                                //                            //this.setProfileCallsPresent(true);
+                                //                            inputString = br.readLine();
+                                //                            genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
+                                //                            //Arguments are evaluated left to right.
+                                //                            functionProfile.addCall(Double.parseDouble(genericTokenizer.nextToken()),
+                                //                                    Double.parseDouble(genericTokenizer.nextToken()));
+                                //                        }
+                            }
+
+                            //Process the appropriate number of aggregate lines.
+                            inputString = br.readLine();
+
+                            //A valid profile.*.*.* will always contain this line.
+                            if (inputString == null) {
+                                throw new CorruptFileException("Unexpected end of file: Looking for 'aggregates' line");
+                            }
+                            genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
+                            //It's first token will be the number of aggregates.
+                            tokenString = genericTokenizer.nextToken();
+
+                            numFunctions = Integer.parseInt(tokenString);
+                            for (int j = 0; j < numFunctions; j++) {
+                                //this.setAggregatesPresent(true);
+                                inputString = br.readLine();
+                            }
+
+                            if (metric == 0) {
+                                //Process the appropriate number of userevent lines.
+                                inputString = br.readLine();
+                                if (inputString != null) {
+                                    genericTokenizer = new StringTokenizer(inputString, " \t\n\r");
+                                    //It's first token will be the number of userEvents
+                                    tokenString = genericTokenizer.nextToken();
+                                    int numUserEvents = Integer.parseInt(tokenString);
+
+                                    //Skip the heading (e.g. "# eventname numevents max min mean sumsqr")
+                                    br.readLine();
+                                    for (int j = 0; j < numUserEvents; j++) {
+                                        if (j == 0) {
+                                            setUserEventsPresent(true);
+                                        }
+
+                                        inputString = br.readLine();
+                                        if (inputString == null) {
+                                            throw new CorruptFileException("Unexpected end of file: Only found " + (j - 2)
+                                                    + " of " + numUserEvents + " User Event Lines");
+                                        }
+
+                                        this.processUserEventLine(inputString, thread);
+
+                                    }
+                                }
+                            }
+
+                            if (lock != null) {
+                                lock.release();
+                            }
+                            br.close();
+                            inReader.close();
+                            fileIn.close();
+
+                        } catch (Exception e2) {
+                            throw new CorruptFileException("Error parsing file (" + e2.getMessage() + ")");
                         }
-                        br.close();
-                        inReader.close();
-                        fileIn.close();
 
                         finished = true;
+                    } catch (CorruptFileException cfe) {
+                        System.err.println("File '" + currentFile + "' is corrupt (at line " + br.getCurrentLine() + ") : "
+                                + cfe.getMessage());
+                        // continue to the next file
+                        finished = true;
+                    } catch (DataSourceException dse) {
+                        throw dse;
                     } catch (Exception ex) {
+                        System.out.println("Error: Current File: " + currentFile + ", Current Line: " + br.getCurrentLine());
                         ex.printStackTrace();
-                        //System.out.println("ex:);
                         if (!(ex instanceof IOException || ex instanceof FileNotFoundException)) {
                             throw new RuntimeException(ex == null ? null : ex.toString());
                         }
@@ -347,17 +398,21 @@ public class TauDataSource extends DataSource {
                     "Didn't find any valid files.\nAre you sure these are TAU profiles? (e.g. profile.*.*.*)");
         }
 
-        //time = (System.currentTimeMillis()) - time;
-        //System.out.println("Time to process (in milliseconds): " + time);
-        //        time = System.currentTimeMillis();
+        long thistime = (System.currentTimeMillis()) - time;
+        //System.out.println("Time to process (in milliseconds): " + thistime);
+        //System.out.print(thistime + ", ");
 
+        
+        EBSTraceReader.processEBSTraces(this, new File(System.getProperty("user.dir")));
+        
         //Generate derived data.
         this.generateDerivedData();
         this.aggregateMetaData();
         this.buildXMLMetaData();
 
-        //time = (System.currentTimeMillis()) - time;
-        //System.out.println("Time to process (in milliseconds): " + time);
+        time = (System.currentTimeMillis()) - time;
+        //System.out.println("Total Time to process (in milliseconds): " + time);
+        //System.out.println(time + "");
     }
 
     public String toString() {
@@ -429,7 +484,7 @@ public class TauDataSource extends DataSource {
         }
 
         if (quoteCount == 0) {
-            throw new DataSourceException("Looking for function line, found '" + string + "' instead");
+            throw new CorruptFileException("Looking for function line, found '" + string + "' instead.");
         }
 
         StringTokenizer st2;
@@ -455,6 +510,8 @@ public class TauDataSource extends DataSource {
             name = string.substring(1, i - 1);
             st2 = new StringTokenizer(string.substring(i + 1), " \t\n\r");
         }
+        
+        name = Utility.removeRuns(name);
 
         numcalls = Double.parseDouble(st2.nextToken()); //Calls
         numsubr = Double.parseDouble(st2.nextToken()); //Subroutines
@@ -467,11 +524,13 @@ public class TauDataSource extends DataSource {
         profileCalls = Integer.parseInt(st2.nextToken()); //ProfileCalls
 
         if (inclusive < 0) {
-            System.err.println("Warning, negative values found in profile, ignoring! (routine: " + name + ")");
+            System.err.println("File '" + currentFile + "' is corrupt (at line " + br.getCurrentLine()
+                    + ") : Negative values found in profile, ignoring! (routine: " + name + ")");
             inclusive = 0;
         }
         if (exclusive < 0) {
-            System.err.println("Warning, negative values found in profile, ignoring! (routine: " + name + ")");
+            System.err.println("File '" + currentFile + "' is corrupt (at line " + br.getCurrentLine()
+                    + ") : Negative values found in profile, ignoring! (routine: " + name + ")");
             exclusive = 0;
         }
 
@@ -598,14 +657,29 @@ public class TauDataSource extends DataSource {
             if (userEventProfile == null) {
                 userEventProfile = new UserEventProfile(userEvent);
                 thread.addUserEventProfile(userEventProfile);
-            }
 
-            userEventProfile.setNumSamples(numSamples);
-            userEventProfile.setMaxValue(sampleMax);
-            userEventProfile.setMinValue(sampleMin);
-            userEventProfile.setMeanValue(sampleMean);
-            userEventProfile.setSumSquared(sampleSumSquared);
-            userEventProfile.updateMax();
+                userEventProfile.setNumSamples(numSamples);
+                userEventProfile.setMaxValue(sampleMax);
+                userEventProfile.setMinValue(sampleMin);
+                userEventProfile.setMeanValue(sampleMean);
+                userEventProfile.setSumSquared(sampleSumSquared);
+                userEventProfile.updateMax();
+
+            } else {
+                // we are combining two user events here, we have to recompute
+                if (userEventProfile.getNumSamples() + numSamples != 0) {
+                    double mean = (userEventProfile.getMeanValue() * userEventProfile.getNumSamples() + numSamples * sampleMean)
+                            / (userEventProfile.getNumSamples() + numSamples);
+                    userEventProfile.setMeanValue(mean);
+                } else {
+                    userEventProfile.setMeanValue(0);
+                }
+                userEventProfile.setNumSamples(numSamples + userEventProfile.getNumSamples());
+                userEventProfile.setMaxValue(Math.max(sampleMax, userEventProfile.getMaxValue()));
+                userEventProfile.setMinValue(Math.min(sampleMin, userEventProfile.getMinValue()));
+                userEventProfile.setSumSquared(sampleSumSquared + userEventProfile.getSumSquared());
+                userEventProfile.updateMax();
+            }
         }
     }
 
