@@ -1,14 +1,33 @@
 package edu.uoregon.tau.perfdmf;
 
-import java.io.*;
-import java.sql.*;
-import java.util.*;
+import java.io.ByteArrayInputStream;
+import java.io.IOException;
+import java.io.InputStream;
+import java.io.ObjectInputStream;
+import java.io.ObjectOutputStream;
+import java.io.Serializable;
+import java.sql.DatabaseMetaData;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.sql.ResultSetMetaData;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.Collections;
 import java.util.Date;
+import java.util.Enumeration;
+import java.util.List;
+import java.util.Map;
+import java.util.TreeMap;
+import java.util.Vector;
 
-import org.xml.sax.*;
+import org.xml.sax.Attributes;
+import org.xml.sax.InputSource;
+import org.xml.sax.SAXException;
+import org.xml.sax.XMLReader;
 import org.xml.sax.helpers.DefaultHandler;
 import org.xml.sax.helpers.XMLReaderFactory;
 
+import edu.uoregon.tau.common.AlphanumComparator;
 import edu.uoregon.tau.common.Gzip;
 import edu.uoregon.tau.perfdmf.database.DB;
 import edu.uoregon.tau.perfdmf.database.DBConnector;
@@ -24,7 +43,7 @@ import edu.uoregon.tau.perfdmf.database.DBConnector;
  * number of threads per context and the metrics collected during the run.
  * 
  * <P>
- * CVS $Id: Trial.java,v 1.25 2007/10/12 21:31:01 amorris Exp $
+ * CVS $Id: Trial.java,v 1.36 2009/09/15 16:48:11 amorris Exp $
  * </P>
  * 
  * @author Kevin Huck, Robert Bell
@@ -37,7 +56,11 @@ import edu.uoregon.tau.perfdmf.database.DBConnector;
  * @see IntervalEvent
  * @see AtomicEvent
  */
-public class Trial implements Serializable {
+public class Trial implements Serializable, Comparable {
+    /**
+     * 
+     */
+    private static final long serialVersionUID = 3356242487725127605L;
     public static final String XML_METADATA = "XML_METADATA";
     public static final String XML_METADATA_GZ = "XML_METADATA_GZ";
 
@@ -45,15 +68,27 @@ public class Trial implements Serializable {
     private int experimentID;
     private int applicationID;
     private String name;
-    private Vector metrics;
+    private List metrics;
     private String fields[];
 
-    protected DataSource dataSource = null;
+    protected DataSource dataSource;
 
     private Database database;
     private Map metaData = new TreeMap();
     private Map uncommonMetaData = new TreeMap();
 
+    private boolean xmlMetaDataLoaded = false;
+
+    public boolean isXmlMetaDataLoaded() {
+        return xmlMetaDataLoaded;
+    }
+
+    public void setXmlMetaDataLoaded(boolean xmlMetaDataLoaded) {
+        this.xmlMetaDataLoaded = xmlMetaDataLoaded;
+    }
+
+    private static AlphanumComparator alphanum = new AlphanumComparator();
+
     private static class XMLParser extends DefaultHandler {
         private StringBuffer accumulator = new StringBuffer();
         private String currentName = "";
@@ -120,6 +155,7 @@ public class Trial implements Serializable {
         this.metaData = trial.metaData;
         this.uncommonMetaData = trial.uncommonMetaData;
         this.database = trial.database;
+        this.dataSource = trial.dataSource;
     }
 
     ///////////////////////////////////////////////////////
@@ -137,19 +173,77 @@ public class Trial implements Serializable {
     }
 
     public String getField(int idx) {
+
         return fields[idx];
     }
 
+    public void loadXMLMetadata(DB db) throws SQLException {
+
+        if (isXmlMetaDataLoaded()) {
+            return;
+        }
+
+        StringBuffer buf = new StringBuffer();
+        buf.append("select ");
+        boolean first = true;
+        int xDex = -1;
+        int zxDex = -1;
+        for (int i = 0; i < database.getTrialFieldNames().length; i++) {
+            if (database.getTrialFieldNames()[i].toUpperCase().equals(XML_METADATA_GZ)
+                    || database.getTrialFieldNames()[i].toUpperCase().equals(XML_METADATA)) {
+                if (database.getTrialFieldNames()[i].toUpperCase().equals(XML_METADATA_GZ)) {
+                    zxDex = i;
+                }
+                if (database.getTrialFieldNames()[i].toUpperCase().equals(XML_METADATA)) {
+                    xDex = i;
+                }
+                if (!first) {
+                    buf.append(", ");
+                }
+                buf.append("t." + database.getTrialFieldNames()[i]);
+                first = false;
+            }
+        }
+
+        buf.append(" from " + db.getSchemaPrefix() + "trial t inner join " + db.getSchemaPrefix() + "experiment e ");
+        buf.append("on t.experiment = e.id ");
+        buf.append("WHERE t.experiment = " + getExperimentID() + " AND t.id = " + getID());
+
+        ResultSet resultSet = null;
+
+        resultSet = db.executeQuery(buf.toString());
+
+        if (resultSet.next() != false) {
+            setField(xDex, resultSet.getString(1));
+
+            InputStream compressedStream = resultSet.getBinaryStream(2);
+            String tmp = Gzip.decompress(compressedStream);
+
+            if (tmp != null && tmp.length() > 0) {
+                setField(XML_METADATA, tmp);
+                parseMetaData(tmp);
+            }
+
+        }
+
+        this.setXmlMetaDataLoaded(true);
+    }
+
     public String getField(String name) {
         if (database.getTrialFieldNames() == null)
             return null;
-        for (int i = 0; i < database.getTrialFieldNames().length; i++) {
+
+        String field = null;
+        int i = 0;
+        for (; i < database.getTrialFieldNames().length; i++) {
             if (name.toUpperCase().equals(database.getTrialFieldNames()[i].toUpperCase())) {
-                if (i < fields.length)
-                    return fields[i];
+                if (i < fields.length) {
+                    field = fields[i];
+                    break;
+                }
             }
         }
-        return null;
+        return field;
     }
 
     public void setField(String field, String value) {
@@ -256,10 +350,26 @@ public class Trial implements Serializable {
      * @return metric count for this trial.
      */
     public int getMetricCount() {
-        if (this.metrics == null)
+        if (getMetrics() == null) {
             return 0;
-        else
-            return this.metrics.size();
+        } else {
+            return getMetrics().size();
+        }
+    }
+
+    /**
+     * Adds a metric to this trial. <i>NOTE: This method is used by the
+     * DataSession object to initialize the object. Not currently intended for
+     * use by any other code. </i>
+     * 
+     * @param metric
+     *            Adds a metric to this trial
+     */
+    public void addMetric(Metric metric) {
+        if (metrics == null) {
+            metrics = new ArrayList();
+        }
+        metrics.add(metric);
     }
 
     /**
@@ -267,28 +377,10 @@ public class Trial implements Serializable {
      * 
      * @return metric vector
      */
-    public Vector getMetrics() {
+    public List getMetrics() {
         return this.metrics;
     }
 
-    /**
-     * Get the metric name corresponding to the given id. The DataSession object
-     * will maintain a reference to the Vector of metric values. To clear this
-     * reference, call setMetric(String) with null.
-     * 
-     * @param metricID
-     *            metric id.
-     * 
-     * @return The metric name as a String.
-     */
-    public String getMetricName(int metricID) {
-
-        //Try getting the metric name.
-        if ((this.metrics != null) && (metricID < this.metrics.size()))
-            return ((Metric) this.metrics.elementAt(metricID)).getName();
-        else
-            return null;
-    }
 
     /**
      * Sets the unique ID associated with this trial. <i>NOTE: This method is
@@ -348,19 +440,6 @@ public class Trial implements Serializable {
         this.dataSource = dataSource;
     }
 
-    /**
-     * Adds a metric to this trial. <i>NOTE: This method is used by the
-     * DataSession object to initialize the object. Not currently intended for
-     * use by any other code. </i>
-     * 
-     * @param metric
-     *            Adds a metric to this trial
-     */
-    public void addMetric(Metric metric) {
-        if (this.metrics == null)
-            this.metrics = new Vector();
-        this.metrics.addElement(metric);
-    }
 
     // gets the metric data for the trial
     private void getTrialMetrics(DB db) {
@@ -491,7 +570,15 @@ public class Trial implements Serializable {
         }
     }
 
-    public static Vector getTrialList(DB db, String whereClause) {
+    private static Vector getTrialList(DB db, String whereClause) {
+        return getTrialList(db, whereClause, true);
+    }
+
+    public static Vector getTrialListWithoutMetadata(DB db, String whereClause) {
+        return getTrialList(db, whereClause, false);
+    }
+
+    public static Vector getTrialList(DB db, String whereClause, boolean getXMLMetadata) {
 
         try {
 
@@ -504,13 +591,18 @@ public class Trial implements Serializable {
             buf.append("t.name");
 
             for (int i = 0; i < database.getTrialFieldNames().length; i++) {
-                buf.append(", t." + database.getTrialFieldNames()[i]);
+                if (!getXMLMetadata
+                        && (database.getTrialFieldNames()[i].toUpperCase().equals(XML_METADATA_GZ) || database.getTrialFieldNames()[i].toUpperCase().equals(
+                                XML_METADATA))) {
+                    continue;
+                } else
+                    buf.append(", t." + database.getTrialFieldNames()[i]);
             }
 
             buf.append(" from " + db.getSchemaPrefix() + "trial t inner join " + db.getSchemaPrefix() + "experiment e ");
             buf.append("on t.experiment = e.id ");
             buf.append(whereClause);
-            buf.append(" order by t.id ");
+            buf.append(" order by t.name, t.id ");
 
             Vector trials = new Vector();
 
@@ -525,21 +617,27 @@ public class Trial implements Serializable {
                 trial.setName(resultSet.getString(pos++));
 
                 boolean xmlSet = false;
+
                 for (int i = 0; i < database.getTrialFieldNames().length; i++) {
                     if (database.getTrialFieldNames()[i].equalsIgnoreCase(XML_METADATA_GZ)) {
-                        InputStream compressedStream = resultSet.getBinaryStream(pos++);
-                        String tmp = Gzip.decompress(compressedStream);
-                        //trial.setField(i, tmp);
-                        if (tmp != null && tmp.length() > 0) {
-                            trial.setField(XML_METADATA, tmp);
-                            trial.parseMetaData(tmp);
+                        if (getXMLMetadata) {
+                            InputStream compressedStream = resultSet.getBinaryStream(pos++);
+                            String tmp = Gzip.decompress(compressedStream);
+                            //trial.setField(i, tmp);
+                            if (tmp != null && tmp.length() > 0) {
+                                trial.setField(XML_METADATA, tmp);
+                                trial.parseMetaData(tmp);
+                            }
+                            xmlSet = true;
+                            trial.setXmlMetaDataLoaded(true);
                         }
-                        xmlSet = true;
-
                     } else {
                         if (database.getTrialFieldNames()[i].equalsIgnoreCase(XML_METADATA)) {
-                            if (xmlSet == false) {
-                                trial.setField(i, resultSet.getString(pos++));
+                            if (getXMLMetadata) {
+                                if (xmlSet == false) {
+                                    trial.setField(i, resultSet.getString(pos++));
+                                    trial.setXmlMetaDataLoaded(true);
+                                }
                             }
                         } else {
                             trial.setField(i, resultSet.getString(pos++));
@@ -559,6 +657,8 @@ public class Trial implements Serializable {
                 trial.getTrialMetrics(db);
             }
 
+            Collections.sort(trials);
+
             return trials;
 
         } catch (Exception ex) {
@@ -691,8 +791,25 @@ public class Trial implements Serializable {
                             ByteArrayInputStream in = new ByteArrayInputStream(compressed);
                             statement.setBinaryStream(pos++, in, compressed.length);
                         }
-                    } else
-                        statement.setString(pos++, this.getField(i));
+                    } else {
+                        int type = this.getFieldType(i);
+                        if (this.getField(i) == null) {
+                            statement.setNull(pos++, type);
+                        } else if (type == java.sql.Types.VARCHAR || type == java.sql.Types.CLOB
+                                || type == java.sql.Types.LONGVARCHAR) {
+                            statement.setString(pos++, this.getField(i));
+                        } else if (type == java.sql.Types.INTEGER) {
+                            statement.setInt(pos++, Integer.parseInt(this.getField(i)));
+                        } else if (type == java.sql.Types.DECIMAL || type == java.sql.Types.DOUBLE
+                                || type == java.sql.Types.FLOAT) {
+                            statement.setDouble(pos++, Double.parseDouble(this.getField(i)));
+                        } else if (type == java.sql.Types.TIME || type == java.sql.Types.TIMESTAMP) {
+                            statement.setString(pos++, this.getField(i));
+                        } else {
+                            // give up
+                            statement.setNull(pos++, type);
+                        }
+                    }
                 }
             }
 
@@ -792,20 +909,42 @@ public class Trial implements Serializable {
 
     }
 
+    private static int getDBMetric(int trialID, int metric) {
+        return 0;
+    }
+
+    public static void deleteMetric(DB db, int trialID, int metricID) throws SQLException {
+        PreparedStatement statement = null;
+
+        // delete from the interval_location_profile table
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "interval_location_profile WHERE metric = ?");
+        statement.setInt(1, metricID);
+        statement.execute();
+        //statement.close();
+
+        // delete from the interval_mean_summary table
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "interval_mean_summary WHERE metric = ?");
+        statement.setInt(1, metricID);
+        statement.execute();
+        //statement.close();
+
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "interval_total_summary WHERE metric = ?");
+        statement.setInt(1, metricID);
+        statement.execute();
+        //statement.close();
+
+        statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "metric WHERE id = ?");
+        statement.setInt(1, metricID);
+        statement.execute();
+        //statement.close();
+    }
+
     public static void deleteTrial(DB db, int trialID) throws SQLException {
-        // save this trial
         PreparedStatement statement = null;
 
         // delete from the atomic_location_profile table
         if (db.getDBType().compareTo("mysql") == 0) {
-
             Trial.deleteAtomicLocationProfilesMySQL(db, trialID);
-
-            //                statement = db.prepareStatement(" DELETE atomic_location_profile.* FROM "
-            //                        + db.getSchemaPrefix()
-            //                        + "atomic_location_profile LEFT JOIN "
-            //                        + db.getSchemaPrefix()
-            //                        + "atomic_event ON atomic_location_profile.atomic_event = atomic_event.id WHERE atomic_event.trial = ?");
         } else {
             // Postgresql, oracle, and DB2?
             statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
@@ -813,25 +952,18 @@ public class Trial implements Serializable {
                     + "atomic_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
-            statement.close();
+            //statement.close();
         }
 
         // delete the from the atomic_events table
         statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "atomic_event WHERE trial = ?");
         statement.setInt(1, trialID);
         statement.execute();
-        statement.close();
+        //statement.close();
 
         // delete from the interval_location_profile table
         if (db.getDBType().compareTo("mysql") == 0) {
-
             Trial.deleteIntervalLocationProfilesMySQL(db, trialID);
-
-            //                statement = db.prepareStatement(" DELETE interval_location_profile.* FROM "
-            //                        + db.getSchemaPrefix()
-            //                        + "interval_location_profile LEFT JOIN "
-            //                        + db.getSchemaPrefix()
-            //                        + "interval_event ON interval_location_profile.interval_event = interval_event.id WHERE interval_event.trial = ?");
         } else {
             // Postgresql and DB2?
             statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix()
@@ -839,7 +971,7 @@ public class Trial implements Serializable {
                     + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
-            statement.close();
+            //statement.close();
         }
 
         // delete from the interval_mean_summary table
@@ -852,7 +984,7 @@ public class Trial implements Serializable {
                     + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
-            statement.close();
+            //statement.close();
         }
 
         if (db.getDBType().compareTo("mysql") == 0) {
@@ -864,23 +996,23 @@ public class Trial implements Serializable {
                     + "interval_event WHERE trial = ?)");
             statement.setInt(1, trialID);
             statement.execute();
-            statement.close();
+            //statement.close();
         }
 
         statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "interval_event WHERE trial = ?");
         statement.setInt(1, trialID);
         statement.execute();
-        statement.close();
+        //statement.close();
 
         statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "metric WHERE trial = ?");
         statement.setInt(1, trialID);
         statement.execute();
-        statement.close();
+        //statement.close();
 
         statement = db.prepareStatement(" DELETE FROM " + db.getSchemaPrefix() + "trial WHERE id = ?");
         statement.setInt(1, trialID);
         statement.execute();
-        statement.close();
+        //statement.close();
     }
 
     private boolean exists(DB db) {
@@ -1022,4 +1154,8 @@ public class Trial implements Serializable {
         this.uncommonMetaData = uncommonMetaData;
     }
 
+    public int compareTo(Object arg0) {
+        return alphanum.compare(this.getName(), ((Trial) arg0).getName());
+    }
+
 }
