@@ -1,13 +1,19 @@
 package edu.uoregon.tau.perfdmf.database;
 
 import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
 import java.io.FileReader;
+import java.io.InputStream;
+import java.io.InputStreamReader;
+import java.io.Serializable;
+import java.net.URL;
 
 import edu.uoregon.tau.common.TauRuntimeException;
 
 /* This class is intended to read in config.txt file and parse the parameters. */
 
-public class ParseConfig {
+public class ParseConfig implements Serializable {
     private String perfdmfHome;
     private String jdbcJarFile;
     private String jdbcDriver;
@@ -20,15 +26,37 @@ public class ParseConfig {
     private String dbPasswd;
     private String dbSchema;
     private String xmlSAXParser;
+    private String path;
+    private String name;
 
     public ParseConfig(String configLoc) {
 
+        path = configLoc;
+        String[] split = configLoc.split("\\.");
+        name = split[split.length - 1];
+        if (name.compareTo("cfg") == 0)
+            name = "Default";
         String inputString;
         String name;
         String value;
 
         try {
-            BufferedReader reader = new BufferedReader(new FileReader(configLoc));
+            BufferedReader reader;
+            if (configLoc.toLowerCase().startsWith("http:")) {
+                // When it gets converted from a String to a File http:// turns into http:/
+                String url_string = "";
+                if (configLoc.toLowerCase().startsWith("http://")) {
+                    url_string = "http://" + configLoc.toString().substring(7).replace('\\', '/');
+                } else if (configLoc.toLowerCase().startsWith("http:/")) {
+                    url_string = "http://" + configLoc.toString().substring(6).replace('\\', '/');
+                }
+                URL url = new URL(url_string);
+                InputStream iostream = url.openStream();
+                InputStreamReader ireader = new InputStreamReader(iostream);
+                reader = new BufferedReader(ireader);
+            } else {
+                reader = new BufferedReader(new FileReader(new File(configLoc)));
+            }
 
             // The following is for reading perfdmf.cfg out of the jar file for Java Web Start
             //        URL url = ParseConfig.class.getResource("/perfdmf.cfg");
@@ -75,10 +103,12 @@ public class ParseConfig {
                     }
                 }
             }
-
+            reader.close();
         } catch (Exception e) {
             // wrap it up in a runtime exception
-            throw new TauRuntimeException("Unable to parse \"" + configLoc + "\"", e);
+
+            e.printStackTrace();
+            //throw new TauRuntimeException("Unable to parse \"" + configLoc + "\"", e);
         }
     }
 
@@ -100,8 +130,7 @@ public class ParseConfig {
             if (getDBType().equals("oracle")) {
                 connectionString = "jdbc:oracle:thin:@//" + getDBHost() + ":" + getDBPort() + "/" + getDBName();
             } else {
-                connectionString = "jdbc:" + getDBType() + "://" + getDBHost() + ":" + getDBPort() + "/"
-                        + getDBName();
+                connectionString = "jdbc:" + getDBType() + "://" + getDBHost() + ":" + getDBPort() + "/" + getDBName();
             }
         }
         return connectionString;
@@ -164,4 +193,11 @@ public class ParseConfig {
         return xmlSAXParser;
     }
 
+    public String getName() {
+        return name;
+    }
+
+    public String getPath() {
+        return path;
+    }
 }
