@@ -7,6 +7,7 @@ import java.sql.ResultSet;
 import java.sql.SQLException;
 
 import edu.uoregon.tau.common.Common;
+import edu.uoregon.tau.common.TauRuntimeException;
 import edu.uoregon.tau.common.Wget;
 import edu.uoregon.tau.common.tar.Tar;
 import edu.uoregon.tau.perfdmf.Database;
@@ -14,7 +15,7 @@ import edu.uoregon.tau.perfdmf.DatabaseException;
 import edu.uoregon.tau.perfdmf.database.*;
 
 public class Configure {
-    private static String Usage = "Usage: configure [{-h,--help}] [{-g,--configfile} filename] [{-c --config} configuration_name] [{-t,--tauroot} path]";
+    private static String Usage = "Usage: configure [{-h,--help}] --create-default [{-g,--configfile} filename] [{-c --config} configuration_name] [{-t,--tauroot} path]";
     private static String Greeting = "\nWelcome to the configuration program for PerfDMF.\n"
             + "This program will prompt you for some information necessary to ensure\n"
             + "the desired behavior for the PerfDMF tools.\n";
@@ -22,9 +23,8 @@ public class Configure {
     // todo - remove these defaults
     // todo - consider using a hash table!
     private String configuration_name = "";
-    private String perfdmf_home = "";
-    private String tau_root = "";
-    private String arch = "";
+    private String jardir;
+    private String schemadir;
     private String jdbc_db_jarfile = "derby.jar";
     private String jdbc_db_driver = "org.apache.derby.jdbc.EmbeddedDriver";
     private String jdbc_db_type = "derby";
@@ -39,14 +39,14 @@ public class Configure {
     private String xml_parser = "xerces.jar";
     private ParseConfig parser;
     private boolean configFileFound = false;
+    private String etc = File.separator + "etc" + File.separator;
 
     private String configFileName;
 
-    public Configure(String tauroot, String arch) {
+    public Configure(String jardir, String schemadir) {
         super();
-        this.tau_root = tauroot;
-        this.arch = arch;
-        this.perfdmf_home = tauroot + "/tools/src/perfdmf";
+        this.jardir = jardir;
+        this.schemadir = schemadir;
     }
 
     public void errorPrint(String msg) {
@@ -60,9 +60,6 @@ public class Configure {
 
     public void initialize(String configFileNameIn) {
 
-        // Welcome the user to the program
-        System.out.println(Greeting);
-
         try {
             // Check to see if the configuration file exists
             configFileName = configFileNameIn;
@@ -93,7 +90,7 @@ public class Configure {
     public void parseConfigFile() throws IOException, FileNotFoundException {
         System.out.println("Parsing config file...");
         parser = new ParseConfig(configFileName);
-        perfdmf_home = parser.getPerfDMFHome();
+        //perfdmf_home = parser.getPerfDMFHome();
         jdbc_db_jarfile = parser.getJDBCJarFile();
         jdbc_db_driver = parser.getJDBCDriver();
         jdbc_db_type = parser.getDBType();
@@ -112,16 +109,33 @@ public class Configure {
      *  in the configuration file.
      **/
 
-    private String getSysDepRoot() {
+    private String getUserJarDir() {
         String os = System.getProperty("os.name").toLowerCase();
         if (os.trim().startsWith("windows")) {
-            return tau_root + "/bin/";
+            return jardir;
         } else {
-            return tau_root + "/" + arch + "/lib/";
+            String dir = System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator;
+            File file = new File(dir);
+            file.mkdirs();
+            return dir;
         }
     }
 
+    public void useDefaults() {
+        String os = System.getProperty("os.name").toLowerCase();
+        jdbc_db_jarfile = jardir + File.separator + "derby.jar";
+        db_dbname = System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "perfdmf";
+        jdbc_db_driver = "org.apache.derby.jdbc.EmbeddedDriver";
+        db_schemafile = schemadir + File.separator + "dbschema.derby.txt";
+        db_hostname = "";
+        db_portnum = "";
+        store_db_password = true;
+    }
+
     public void promptForData() {
+        // Welcome the user to the program
+        System.out.println(Greeting);
+
         BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
         String tmpString;
 
@@ -147,12 +161,7 @@ public class Configure {
                 configuration_name = "";
             }
 
-            System.out.print("Please enter the TAU root directory.\n(" + tau_root + "):");
-            tmpString = reader.readLine();
-            if (tmpString.length() > 0) {
-                tau_root = tmpString;
-                perfdmf_home = tau_root + "/tools/src/perfdmf";
-            }
+         
 
             String old_jdbc_db_type = jdbc_db_type;
             boolean valid = false;
@@ -160,13 +169,14 @@ public class Configure {
             while (!valid) {
                 // Prompt for database type
                 System.out.println("Please enter the database vendor (oracle, postgresql, mysql, db2 or derby).");
-                System.out.println("(" + jdbc_db_type + "):");
+                System.out.print("(" + jdbc_db_type + "):");
                 tmpString = reader.readLine();
                 if (tmpString.compareTo("oracle") == 0 || tmpString.compareTo("postgresql") == 0
                         || tmpString.compareTo("mysql") == 0 || tmpString.compareTo("derby") == 0
                         || tmpString.compareTo("db2") == 0 || tmpString.length() == 0) {
-                    if (tmpString.length() > 0)
+                    if (tmpString.length() > 0) {
                         jdbc_db_type = tmpString;
+                    }
                     valid = true;
                 }
             }
@@ -174,34 +184,35 @@ public class Configure {
             if (configFileFound) {
                 if (jdbc_db_type.compareTo("postgresql") == 0 && old_jdbc_db_type.compareTo("postgresql") != 0) {
                     // if the user has chosen postgresql and the config file is not already set for it
-                    jdbc_db_jarfile = getSysDepRoot() + "postgresql.jar";
+                    jdbc_db_jarfile = jardir + File.separator + "postgresql.jar";
                     jdbc_db_driver = "org.postgresql.Driver";
-                    db_schemafile = perfdmf_home + "/etc/" + "dbschema.txt";
+                    db_schemafile = schemadir + File.separator + "dbschema.txt";
                     db_portnum = "5432";
                     db_hostname = "localhost";
                     db_dbname = "perfdmf";
                 } else if (jdbc_db_type.compareTo("mysql") == 0 && old_jdbc_db_type.compareTo("mysql") != 0) {
                     // if the user has chosen mysql and the config file is not already set for it
-                    jdbc_db_jarfile = getSysDepRoot() + "mysql.jar";
+                    jdbc_db_jarfile = jardir + File.separator + "mysql.jar";
                     jdbc_db_driver = "org.gjt.mm.mysql.Driver";
-                    db_schemafile = perfdmf_home + "/etc/" + "dbschema.mysql.txt";
+                    db_schemafile = schemadir + File.separator + "dbschema.mysql.txt";
                     db_portnum = "3306";
                     db_hostname = "localhost";
                     db_dbname = "perfdmf";
                 } else if (jdbc_db_type.compareTo("oracle") == 0 && old_jdbc_db_type.compareTo("oracle") != 0) {
                     // if the user has chosen oracle and the config file is not already set for it
-                    jdbc_db_jarfile = getSysDepRoot() + "ojdbc14.jar";
+                    jdbc_db_jarfile = getUserJarDir() + "ojdbc14.jar";
                     jdbc_db_driver = "oracle.jdbc.OracleDriver";
-                    db_schemafile = perfdmf_home + "/etc/" + "dbschema.oracle.txt";
+                    db_schemafile = schemadir + File.separator + "dbschema.oracle.txt";
                     db_portnum = "1521";
                     db_hostname = "localhost";
                     db_dbname = "perfdmf";
                 } else if (jdbc_db_type.compareTo("derby") == 0 && old_jdbc_db_type.compareTo("derby") != 0) {
                     // if the user has chosen derby and the config file is not already set for it
-                    jdbc_db_jarfile = getSysDepRoot() + "derby.jar";
+                    String os = System.getProperty("os.name").toLowerCase();
+                    jdbc_db_jarfile = jardir + File.separator + "derby.jar";
                     jdbc_db_driver = "org.apache.derby.jdbc.EmbeddedDriver";
-                    db_schemafile = perfdmf_home + "/etc/" + "dbschema.derby.txt";
-                    db_dbname = tau_root + "/" + arch + "/lib/perfdmf";
+                    db_schemafile = schemadir + File.separator + "dbschema.derby.txt";
+                    db_dbname = jardir + File.separator + "perfdmf";
                     db_hostname = "";
                     db_portnum = "";
                 } else if (jdbc_db_type.compareTo("db2") == 0 && old_jdbc_db_type.compareTo("db2") != 0) {
@@ -209,16 +220,16 @@ public class Configure {
 
                     jdbc_db_jarfile = ""; // there are 3 jar files...
                     jdbc_db_driver = "com.ibm.db2.jcc.DB2Driver";
-                    db_schemafile = perfdmf_home + "/etc/" + "dbschema.db2.txt";
+                    db_schemafile = schemadir + File.separator + "dbschema.db2.txt";
                     db_dbname = "perfdmf";
                     db_schemaprefix = "perfdmf";
                     db_hostname = "localhost";
                     db_portnum = "446";
                 } else if (jdbc_db_type.compareTo("db2") == 0 && old_jdbc_db_type.compareTo("db2") == 0) {
                     // if the user has chosen db2 and the config file is already set for it
-                    int endIndex = jdbc_db_jarfile.indexOf("java/db2java.zip:");
+                    int endIndex = jdbc_db_jarfile.indexOf("java" + File.separator + "db2java.zip:");
                     if (endIndex == -1) {
-                        endIndex = jdbc_db_jarfile.indexOf("java/db2jcc.jar:");
+                        endIndex = jdbc_db_jarfile.indexOf("java" + File.separator + "db2jcc.jar:");
                         if (endIndex == -1) {
                             jdbc_db_jarfile = "";
                         } else {
@@ -257,7 +268,7 @@ public class Configure {
                     jdbc_db_jarfile = "derby.jar";
                     jdbc_db_driver = "org.apache.derby.jdbc.EmbeddedDriver";
                     db_schemafile = "dbschema.derby.txt";
-                    db_dbname = tau_root + "/" + arch + "/lib/perfdmf";
+                    db_dbname = System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "perfdmf";
                     db_hostname = "";
                     db_portnum = "";
                 } else if (jdbc_db_type.compareTo("db2") == 0) {
@@ -272,7 +283,12 @@ public class Configure {
             }
 
             if (!configFileFound) {
-                jdbc_db_jarfile = getSysDepRoot() + jdbc_db_jarfile;
+                if (jdbc_db_type.compareTo("derby") == 0) {
+                    jdbc_db_jarfile = jardir + File.separator + "derby.jar";
+                } else {
+                    //jdbc_db_jarfile = getUserJarDir() + jdbc_db_jarfile;
+                    jdbc_db_jarfile = jardir + File.separator + jdbc_db_jarfile;
+                }
             }
 
             // Prompt for JDBC jar file
@@ -286,7 +302,7 @@ public class Configure {
 
             tmpString = reader.readLine();
             if (tmpString.length() > 0) {
-                jdbc_db_jarfile = tmpString;
+                jdbc_db_jarfile = tmpString.replaceAll("~", System.getProperty("user.home"));
             }
 
             if (!new File(jdbc_db_jarfile).exists()) {
@@ -325,9 +341,9 @@ public class Configure {
 
                             if (jdbc_db_type.compareToIgnoreCase("postgresql") == 0) {
                                 Wget.wget("http://www.cs.uoregon.edu/research/paracomp/tau/postgresql-redirect.html",
-                                        ".perfdmf_tmp/postgresql-redirect.html", false);
+                                        ".perfdmf_tmp" + File.separator + "postgresql-redirect.html", false);
                                 BufferedReader r = new BufferedReader(new InputStreamReader(new FileInputStream(new File(
-                                        ".perfdmf_tmp/postgresql-redirect.html"))));
+                                        ".perfdmf_tmp" + File.separator + "postgresql-redirect.html"))));
 
                                 String URL = "";
                                 String line = r.readLine();
@@ -344,11 +360,11 @@ public class Configure {
                                 System.out.println(" Done");
                             }
                             if (jdbc_db_type.compareToIgnoreCase("mysql") == 0) {
-                                Wget.wget("http://www.cs.uoregon.edu/research/paracomp/tau/mysql-redirect.html",
-                                        ".perfdmf_tmp/mysql-redirect.html", false);
+                                Wget.wget("http://www.cs.uoregon.edu/research/paracomp/tau/mysql-redirect.html", ".perfdmf_tmp"
+                                        + File.separator + "mysql-redirect.html", false);
 
                                 BufferedReader r = new BufferedReader(new InputStreamReader(new FileInputStream(new File(
-                                        ".perfdmf_tmp/mysql-redirect.html"))));
+                                        ".perfdmf_tmp" + File.separator + "mysql-redirect.html"))));
 
                                 String URL = "";
                                 String FILE = "";
@@ -389,8 +405,9 @@ public class Configure {
             }
 
             if (jdbc_db_type.compareTo("db2") == 0) {
-                tmpString = jdbc_db_jarfile + "/java/db2java.zip:" + jdbc_db_jarfile + "/java/db2jcc.jar:" + jdbc_db_jarfile
-                        + "/function:" + jdbc_db_jarfile + "/java/db2jcc_license_cu.jar";
+                tmpString = jdbc_db_jarfile + File.separator + "java" + File.separator + "db2java.zip:" + jdbc_db_jarfile
+                        + File.separator + "java" + File.separator + "db2jcc.jar:" + jdbc_db_jarfile + File.separator
+                        + "function:" + jdbc_db_jarfile + File.separator + "java" + File.separator + "db2jcc_license_cu.jar";
                 jdbc_db_jarfile = tmpString;
             }
 
@@ -424,8 +441,29 @@ public class Configure {
                 System.out.print("Please enter the database name.\n(" + db_dbname + "):");
             }
             tmpString = reader.readLine();
-            if (tmpString.length() > 0)
-                db_dbname = tmpString;
+            if (tmpString.length() > 0) {
+                // if the user used the ~ shortcut, expand it to $HOME.
+                if (jdbc_db_type.compareTo("derby") == 0) {
+                    db_dbname = tmpString.replaceAll("~", System.getProperty("user.home"));
+                } else {
+                    db_dbname = tmpString;
+                }
+            }
+
+            if (jdbc_db_type.compareTo("derby") == 0) {
+                File f = new File(db_dbname);
+                if (f.exists()) {
+                    File f2 = new File(f + File.separator + "seg0");
+                    System.out.println(f2);
+                    if (!f2.exists()) {
+                        System.out.println("\n\nWarning!  Directory \""
+                                + db_dbname
+                                + "\" exists and does not appear to be a derby database.\n"
+                                + "Connection will most likely fail\n\n"
+                                + "If you are trying to create a new Derby database, please specify a path that does not exist\n\n");
+                    }
+                }
+            }
 
             if ((jdbc_db_type.compareTo("oracle") == 0) || (jdbc_db_type.compareTo("db2") == 0)) {
                 System.out.print("Please enter the database schema name, or your username if you are creating the tables now.\n("
@@ -480,15 +518,17 @@ public class Configure {
              */
 
             // Prompt for database schema file
-            if (configFileFound)
+            if (configFileFound) {
                 System.out.print("Please enter the PerfDMF schema file.\n(" + db_schemafile + "):");
-            else
-                System.out.print("Please enter the PerfDMF schema file.\n(" + perfdmf_home + "/etc/" + db_schemafile + "):");
+            } else {
+                System.out.print("Please enter the PerfDMF schema file.\n(" + schemadir + File.separator + db_schemafile + "):");
+            }
             tmpString = reader.readLine();
-            if (tmpString.length() > 0)
-                db_schemafile = tmpString;
-            else if (!configFileFound)
-                db_schemafile = perfdmf_home + "/etc/" + db_schemafile;
+            if (tmpString.length() > 0) {
+                db_schemafile = tmpString.replaceAll("~", System.getProperty("user.home"));
+            } else if (!configFileFound) {
+                db_schemafile = schemadir + File.separator + db_schemafile;
+            }
 
         } catch (IOException e) {
             // todo - get info from the exception
@@ -525,10 +565,26 @@ public class Configure {
             // Check to see if the configuration file exists
 
             File configFile;
-            if (configuration_name.length() == 0) {
-                configFile = new File(System.getProperty("user.home") + "/.ParaProf/perfdmf.cfg");
+
+            File perfdmfpath = new File(System.getProperty("user.home") + File.separator + ".ParaProf");
+            perfdmfpath.mkdirs();
+            
+            if (configuration_name.length() != 0) {
+                // configuration name was specified
+                configFile = new File(System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator
+                        + "perfdmf.cfg." + configuration_name);
             } else {
-                configFile = new File(configFileName + "." + configuration_name);
+                // I don't understand the logic below here
+                if (configFileName == null || configFileName.length() == 0) {
+                    if (configuration_name.length() == 0) {
+                        configFile = new File(System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator
+                                + "perfdmf.cfg");
+                    } else {
+                        configFile = new File(configFileName + "." + configuration_name);
+                    }
+                } else {
+                    configFile = new File(configFileName);
+                }
             }
 
             System.out.println("\nWriting configuration file: " + configFile);
@@ -543,9 +599,9 @@ public class Configure {
             configWriter.write("# DO NOT EDIT THIS FILE!  It is modified by the configure utility.\n");
             configWriter.newLine();
 
-            configWriter.write("# PerfDMF home directory\n");
-            configWriter.write("perfdmf_home:" + perfdmf_home + "\n");
-            configWriter.newLine();
+//            configWriter.write("# PerfDMF home directory\n");
+//            configWriter.write("perfdmf_home:" + perfdmf_home + "\n");
+//            configWriter.newLine();
 
             configWriter.write("# Database JDBC jar file (with path to location)\n");
             configWriter.write("jdbc_db_jarfile:" + jdbc_db_jarfile + "\n");
@@ -595,19 +651,13 @@ public class Configure {
 
             configWriter.close();
             return configFile.toString();
-        } catch (IOException e) {}
-        return null;
-    }
-
-    //Standard access methods for some of the fields.
-    public void setPerfDMFHome(String inString) {
-        perfdmf_home = inString;
-    }
-
-    public String getPerfDMFHome() {
-        return perfdmf_home;
+        } catch (IOException e) {
+            e.printStackTrace();
+            throw new TauRuntimeException(e);
+        }
     }
 
+  
     public void setJDBCJarfile(String inString) {
         jdbc_db_jarfile = inString;
     }
@@ -695,8 +745,9 @@ public class Configure {
     public String getConfigFileName() {
         return configuration_name;
     }
+
     public void savePassword() {
-    	this.store_db_password = true;
+        this.store_db_password = true;
     }
 
     /* Test that the database exists, and if it doesn't, create it! */
@@ -729,6 +780,26 @@ public class Configure {
         System.out.println("Configuration complete.");
     }
 
+    public static void createDefault(String configFile, String tauroot, String arch, String dbName) {
+        // Create a new Configure object, which will walk the user through
+        // the process of creating/editing a configuration file.
+        Configure config = new Configure(tauroot, arch);
+        config.initialize(configFile);
+        config.useDefaults();
+        config.setDBName(dbName);
+
+        // Write the configuration file to ${PerfDMF_Home}/bin/perfdmf.cfg
+        String configFilename = config.writeConfigFile();
+
+        ConfigureTest configTest = new ConfigureTest();
+        configTest.initialize(configFilename);
+        try {
+            configTest.createDB(false);
+        } catch (DatabaseConfigurationException e) {
+            e.printStackTrace();
+        }
+    }
+
     /*** Beginning of main program. ***/
 
     public static void main(java.lang.String[] args) {
@@ -736,9 +807,10 @@ public class Configure {
         CmdLineParser parser = new CmdLineParser();
         CmdLineParser.Option configfileOpt = parser.addStringOption('g', "configfile");
         CmdLineParser.Option configOpt = parser.addStringOption('c', "config");
-        CmdLineParser.Option homeOpt = parser.addStringOption('t', "tauroot");
-        CmdLineParser.Option archOpt = parser.addStringOption('a', "arch");
+        CmdLineParser.Option jarLocationOpt = parser.addStringOption('j', "jardir");
+        CmdLineParser.Option schemaLocationOpt = parser.addStringOption('a', "schemadir");
         CmdLineParser.Option helpOpt = parser.addBooleanOption('h', "help");
+        CmdLineParser.Option defaultOpt = parser.addBooleanOption('d', "create-default");
         try {
             parser.parse(args);
         } catch (CmdLineParser.OptionException e) {
@@ -746,12 +818,12 @@ public class Configure {
             System.err.println(Usage);
             System.exit(-1);
         }
-
         String configFile = (String) parser.getOptionValue(configfileOpt);
         String configName = (String) parser.getOptionValue(configOpt);
-        String tauroot = (String) parser.getOptionValue(homeOpt);
-        String arch = (String) parser.getOptionValue(archOpt);
+        String jardir = (String) parser.getOptionValue(jarLocationOpt);
+        String schemadir = (String) parser.getOptionValue(schemaLocationOpt);
         Boolean help = (Boolean) parser.getOptionValue(helpOpt);
+        Boolean useDefaults = (Boolean) parser.getOptionValue(defaultOpt);
 
         if (help != null && help.booleanValue()) {
             System.err.println(Usage);
@@ -759,27 +831,30 @@ public class Configure {
         }
 
         if (configFile == null) {
-            if (configName == null)
-                configFile = System.getProperty("user.home") + "/.ParaProf/perfdmf.cfg";
-            else
-                configFile = System.getProperty("user.home") + "/.ParaProf/perfdmf.cfg." + configName;
+            if (configName == null) {
+                configFile = System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "perfdmf.cfg";
+            } else {
+                configFile = System.getProperty("user.home") + File.separator + ".ParaProf" + File.separator + "perfdmf.cfg."
+                        + configName;
+            }
 
         }
-        if (tauroot == null) {
-            tauroot = new String("");
-        }
-        if (arch == null) {
-            arch = new String("");
+
+        if (useDefaults == null) {
+            useDefaults = Boolean.FALSE;
         }
 
         // Create a new Configure object, which will walk the user through
         // the process of creating/editing a configuration file.
-        Configure config = new Configure(tauroot, arch);
+        Configure config = new Configure(jardir, schemadir);
         config.initialize(configFile);
 
-        // Give the user the ability to modify any/everything
-        config.promptForData();
-
+        if (useDefaults == Boolean.TRUE) {
+            config.useDefaults();
+        } else {
+            // Give the user the ability to modify any/everything
+            config.promptForData();
+        }
         // Test the database connection
         //config.testDBConnection();
 
@@ -789,14 +864,14 @@ public class Configure {
         // Write the configuration file to ${PerfDMF_Home}/bin/perfdmf.cfg
         String configFilename = config.writeConfigFile();
 
-        ConfigureTest configTest = new ConfigureTest(tauroot);
+        ConfigureTest configTest = new ConfigureTest();
         configTest.initialize(configFilename);
         try {
-			configTest.createDB(true);
-		} catch (DatabaseConfigurationException e) {
-		    e.printStackTrace();
+            configTest.createDB(false);
+        } catch (DatabaseConfigurationException e) {
+            e.printStackTrace();
             System.exit(0);
-		}
+        }
 
         // check to see if the database is there...
         //config.createDB();
