@@ -9,23 +9,24 @@
 
 package edu.uoregon.tau.perfdmf.loader;
 
-import edu.uoregon.tau.perfdmf.database.*;
-
-import java.io.BufferedReader;
-import java.io.File;
-import java.io.FileNotFoundException;
-import java.io.IOException;
-import java.io.InputStreamReader;
-import java.sql.*;
 import jargs.gnu.CmdLineParser;
 
+import java.io.*;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+
+import edu.uoregon.tau.perfdmf.Database;
+import edu.uoregon.tau.perfdmf.DatabaseException;
+import edu.uoregon.tau.perfdmf.database.ConnectionManager;
+import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfdmf.database.ParseConfig;
+
 public class ConfigureTest {
     private static String Usage = "Usage: configure [{-h,--help}] [{-g,--configfile} filename] [{-t,--tauroot} path]";
     private static String Greeting = "\nNow testing your database connection.\n";
 
     // todo - remove these defaults
     // todo - consider using a hash table!
-    private String perfdmf_home = "";
     private String jdbc_db_jarfile = "postgresql.jar";
     private String jdbc_db_driver = "org.postgresql.Driver";
     private String jdbc_db_type = "postgresql";
@@ -40,9 +41,8 @@ public class ConfigureTest {
 
     private String configFileName;
 
-    public ConfigureTest(String tauroot) {
+    public ConfigureTest() {
         super();
-        this.perfdmf_home = tauroot + "/tools/src/dms";
     }
 
     public void errorPrint(String msg) {
@@ -70,7 +70,7 @@ public class ConfigureTest {
                 // Parse the configuration file
                 parseConfigFile();
             } else {
-                System.out.println("Configuration file NOT found...");
+                System.out.println("Configuration file NOT found... (looking for " + configFile + ")");
                 System.out.println("a new configuration file will be created.");
                 // If it doesn't exist, explain that the program looks for the 
                 // configuration file in ${PerfDMF_Home}/data/perfdmf.cfg
@@ -90,7 +90,6 @@ public class ConfigureTest {
     public void parseConfigFile() throws IOException, FileNotFoundException {
         //System.out.println("Parsing config file...");
         parser = new ParseConfig(configFileName);
-        perfdmf_home = parser.getPerfDMFHome();
         jdbc_db_jarfile = parser.getJDBCJarFile();
         jdbc_db_driver = parser.getJDBCDriver();
         jdbc_db_type = parser.getDBType();
@@ -103,15 +102,6 @@ public class ConfigureTest {
         xml_parser = parser.getXMLSAXParser();
     }
 
-    //Standard access methods for some of the fields.
-    public void setPerfDMFHome(String inString) {
-        perfdmf_home = inString;
-    }
-
-    public String getPerfDMFHome() {
-        return perfdmf_home;
-    }
-
     public void setJDBCJarfile(String inString) {
         jdbc_db_jarfile = inString;
     }
@@ -200,78 +190,139 @@ public class ConfigureTest {
         return configFileName;
     }
 
-    /* Test that the database exists, and if it doesn't, create it! */
-    public void createDB() {
+    public boolean checkSchema() {
         ConnectionManager connector = null;
         DB db = null;
         try {
 
-			if (jdbc_db_type.equals("derby")) {
-				// check to see if the directory exists.  If not, create the database.
-		        if (!(new File(db_dbname).exists())) {
-            		if (db_password != null) {
-						connector = new ConnectionManager(configFileName, db_password);
-            		} else {
-						connector = new ConnectionManager(configFileName);
-            		}
-					connector.connectAndCreate();
-					connector.dbclose();
-					connector = null;
-				}
-			}
+            Database database = new Database(configFileName);
+            if (jdbc_db_type.equals("derby")) {
+                // check to see if the directory exists.  If not, create the database.
+                if (!(new File(db_dbname).exists())) {
+                    if (db_password != null) {
+                        connector = new ConnectionManager(database, db_password);
+                    } else {
+                        connector = new ConnectionManager(database);
+                    }
+                    connector.connectAndCreate();
+                    connector.dbclose();
+                    connector = null;
+                }
+            }
             if (db_password != null) {
-                connector = new ConnectionManager(configFileName, db_password);
+                connector = new ConnectionManager(database, db_password);
             } else {
-                connector = new ConnectionManager(configFileName);
+                connector = new ConnectionManager(database);
             }
             connector.connect();
-            System.out.println();
             db = connector.getDB();
         } catch (Exception e) {
             System.out.println("\nPlease make sure that your DBMS is configured correctly, and");
             System.out.println("the database " + db_dbname + " has been created.");
-            System.exit(0);
+            e.printStackTrace();
+            throw new DatabaseConfigurationException("Error Connecting to Database.");
         }
 
         try {
             String query = new String("SELECT * FROM " + db.getSchemaPrefix() + "application");
             ResultSet resultSet = db.executeQuery(query);
         } catch (SQLException e) {
-            e.printStackTrace();
             // this is our method of determining that no 'application' table exists
+            return false;
+        }
 
-            System.out.print("Tables not found.  Would you like to upload the schema? [y/n]: ");
+        connector.dbclose();
 
-            BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
+        return true;
+    }
 
-            String input = "";
-            try {
-                input = reader.readLine();
-            } catch (java.io.IOException ioe) {
-                ioe.printStackTrace();
-                System.exit(-1);
+    /* Test that the database exists, and if it doesn't, create it! */
+    public void createDB(boolean prompt) {
+        ConnectionManager connector = null;
+        DB db = null;
+        try {
+
+            Database database = new Database(configFileName);
+            if (jdbc_db_type.equals("derby")) {
+                // check to see if the directory exists.  If not, create the database.
+                if (!(new File(db_dbname).exists())) {
+                    if (db_password != null) {
+                        connector = new ConnectionManager(database, db_password);
+                    } else {
+                        connector = new ConnectionManager(database, prompt);
+                    }
+                    connector.connectAndCreate();
+                    connector.dbclose();
+                    connector = null;
+                }
             }
-            if (input.equals("y") || input.equals("Y")) {
+            if (db_password != null) {
+                connector = new ConnectionManager(database, db_password);
+            } else {
+                connector = new ConnectionManager(database, prompt);
+            }
+            connector.connect();
+            System.out.println();
+            db = connector.getDB();
+        } catch (SQLException e) {
+            System.out.println("\nPlease make sure that your DBMS is configured correctly, and");
+            System.out.println("the database " + db_dbname + " has been created.");
+            //throw new DatabaseConfigurationException("Error Connecting to Database.");
+            throw new DatabaseException("Error Connecting to Database.", e);
+        }
 
+        try {
+            String query = new String("SELECT * FROM " + db.getSchemaPrefix() + "application");
+            ResultSet resultSet = db.executeQuery(query);
+        } catch (SQLException e) {
+            // this is our method of determining that no 'application' table exists
+            String input = "";
+
+            boolean upload = false;
+
+            if (prompt) {
+                System.out.print("This database has not been initalized with perfdmf.\n\nWould you like to upload the schema? [y/n]: ");
+
+                BufferedReader reader = new BufferedReader(new InputStreamReader(System.in));
+
+                try {
+                    input = reader.readLine();
+                } catch (IOException ioe) {
+                    ioe.printStackTrace();
+                    System.exit(-1);
+                }
+
+                if (input.equals("y") || input.equals("Y")) {
+                    upload = true;
+                } else {
+                    System.exit(0);
+                }
+
+            } else {
+                upload = true;
+            }
+
+            if (upload) {
                 System.out.println("Uploading Schema: " + db_schemafile);
                 if (connector.genParentSchema(db_schemafile) == 0) {
                     System.out.println("Successfully uploaded schema\n");
                 } else {
                     System.out.println("Error uploading schema\n");
-                    System.exit(-1);
+                    throw new DatabaseConfigurationException("Error uploading schema.");
                 }
             }
+
         }
 
         try {
             if (db.checkSchema() != 0) {
                 System.out.println("\nIncompatible schema found.  Please contact us at tau-team@cs.uoregon.edu\nfor a conversion script.");
-                System.exit(0);
+                throw new DatabaseConfigurationException("Incompatible schema found.");
             }
         } catch (SQLException e) {
             System.out.println("\nError trying to confirm schema:");
             e.printStackTrace();
-            System.exit(0);
+            throw new DatabaseConfigurationException("Error trying to confirm schema.");
         }
 
         connector.dbclose();
@@ -311,7 +362,6 @@ public class ConfigureTest {
             configFile = System.getProperty("user.home") + "/.ParaProf/perfdmf.cfg";
         }
 
-        
         if (tauroot == null)
             tauroot = new String("");
         if (arch == null)
@@ -319,8 +369,13 @@ public class ConfigureTest {
 
         // Create a new Configure object, which will walk the user through
         // the process of creating/editing a configuration file.
-        ConfigureTest config = new ConfigureTest(tauroot);
+        ConfigureTest config = new ConfigureTest();
         config.initialize(configFile);
-        config.createDB();
+        try {
+            config.createDB(true);
+        } catch (DatabaseConfigurationException e) {
+            e.printStackTrace();
+            System.exit(0);
+        }
     }
 }
