@@ -1,4 +1,4 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
 import javax.swing.*;
 import java.awt.*;
@@ -8,6 +8,11 @@ import java.awt.*;
  */
 public class AnalysisManagementPane extends JScrollPane {
 
+	/**
+	 * 
+	 */
+	private static final long serialVersionUID = 7842930560904862098L;
+
 	private static AnalysisManagementPane thePane = null;
 
 	private JTable dataTable = null;
@@ -26,6 +31,8 @@ public class AnalysisManagementPane extends JScrollPane {
 	private AnalysisManagementPane (JTable dataTable) {
 		super(dataTable);
 		this.dataTable = dataTable;
+		JScrollBar jScrollBar = this.getVerticalScrollBar();
+		jScrollBar.setUnitIncrement(35);
     }
 
 	public JTable getTable () {
