@@ -1,35 +1,61 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
-import javax.swing.*;
-import java.awt.event.*;
-import java.awt.Frame;
 import java.awt.Component;
 import java.awt.Container;
-import java.util.List;
+import java.awt.Frame;
+import java.awt.event.ActionEvent;
+import java.awt.event.ActionListener;
+import java.io.FileNotFoundException;
+import java.sql.SQLException;
 import java.util.ArrayList;
+import java.util.List;
 import java.util.ListIterator;
-import edu.uoregon.tau.perfdmf.*;
+
+import javax.swing.ImageIcon;
+import javax.swing.JComponent;
+import javax.swing.JFileChooser;
+import javax.swing.JFrame;
+import javax.swing.JMenuItem;
+import javax.swing.JOptionPane;
+import javax.swing.JTree;
+import javax.swing.UIManager;
+import javax.swing.UnsupportedLookAndFeelException;
+
+import org.jfree.data.general.SeriesException;
+
 import edu.uoregon.tau.common.Utility;
-import constants.Constants;
-import common.AnalysisType;
-import common.RMIView;
-import common.RMISortableIntervalEvent;
-import common.RMIPerfExplorerModel;
-import common.TransformationType;
-import common.PerfExplorerOutput;
-import java.io.File;
 import edu.uoregon.tau.common.VectorExport;
+import edu.uoregon.tau.perfdmf.Application;
+import edu.uoregon.tau.perfdmf.DatabaseAPI;
+import edu.uoregon.tau.perfdmf.Experiment;
+import edu.uoregon.tau.perfdmf.Metric;
+import edu.uoregon.tau.perfdmf.Trial;
+import edu.uoregon.tau.perfexplorer.common.AnalysisType;
+import edu.uoregon.tau.perfexplorer.common.Console;
+import edu.uoregon.tau.perfexplorer.common.PerfExplorerOutput;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.common.RMISortableIntervalEvent;
+import edu.uoregon.tau.perfexplorer.common.RMIView;
+import edu.uoregon.tau.perfexplorer.common.ScriptThread;
+import edu.uoregon.tau.perfexplorer.common.TransformationType;
+import edu.uoregon.tau.perfexplorer.constants.Constants;
+import edu.uoregon.tau.perfexplorer.server.PerfExplorerServer;
 
 public class PerfExplorerActionListener implements ActionListener {
 
+	public static final String DERIVE_METRIC = "Open Expression Window";
+	public static final String PARSE_EXPRESSION = "Parse Expression File";
+	public static final String REPARSE_EXPRESSION = "Re-Parse Expression File";
+	public final static String DATABASE_CONFIGURATION = "Database Configuration";
 	public final static String LOADSCRIPT = "Load Analysis Script";
 	public final static String RERUNSCRIPT = "Re-run Analysis Script";
 	public final static String SAVE_MAIN = "Save Main Window As Vector Image";
-	public final static String SAVE = "Save As Vector Image";
+	public final static String CONSOLE = "Open New Console Window";
 	public final static String QUIT = "Quit PerfExplorer";
 	public final static String QUIT_SERVER = "Quit PerfExplorer (Shutdown Server)";
 	public final static String LOOK_AND_FEEL = "Set Look and Feel: ";
 	public final static String ABOUT = "About PerfExplorer";
+	public final static String LOAD_PROFILE = "Load Profile Data";
 	public final static String SEARCH = "Search for help on...";
 	// clustering menu items
 	public final static String CLUSTERING_METHOD = "Select Clustering Method";
@@ -37,9 +63,12 @@ public class PerfExplorerActionListener implements ActionListener {
 	public final static String NORMALIZATION = "Select Normalization Method";
 	public final static String NUM_CLUSTERS = "Set Maximum Number of Clusters";
 	public final static String DO_CLUSTERING = "Do Clustering";
+	public final static String DO_INC_CLUSTERING = "Do Inclusive Clustering";
 	public final static String DO_CORRELATION_ANALYSIS = "Do Correlation Analysis";
+	public final static String DO_INC_CORRELATION_ANALYSIS = "Do Inclusive Correlation Analysis";
 	public final static String DO_CORRELATION_CUBE = "Do 3D Correlation Cube";
 	public final static String DO_VARIATION_ANALYSIS = "Show Data Summary";
+	public final static String DO_LAUNCH_WEKA = "Launch Weka With This Dataset";
 	// chart menu items
 	public final static String SET_GROUPNAME = "Set Group Name";
 	public final static String SET_PROBLEM_SIZE = "Set Problem Size (Scaling)";
@@ -57,6 +86,8 @@ public class PerfExplorerActionListener implements ActionListener {
 	public final static String COMMUNICATION_CHART = "Group % of Total Runtime";
 	public final static String FRACTION_CHART = "Runtime Breakdown";
 	public final static String CORRELATION_CHART = "Correlate Events with Total Runtime";
+	public final static String STACKED_BAR_CHART = "Stacked Bar Chart";
+	public final static String ALIGNED_STACKED_BAR_CHART = "Aligned Stacked Bar Chart";
 	// phase chart menu items
 	public final static String EFFICIENCY_PHASE_CHART = "Relative Efficiency per Phase";
 	public final static String SPEEDUP_PHASE_CHART = "Relative Speedup per Phase";
@@ -68,10 +99,14 @@ public class PerfExplorerActionListener implements ActionListener {
 	public final static String DO_IQR_BOXCHART = "Create BoxChart";
 	public final static String DO_HISTOGRAM = "Create Histograms";
 	public final static String DO_PROBABILITY_PLOT = "Create Normal Probability Plot";
+	public static final String DO_CHARTS = "Open Scalability Chart Tool";
+	public static final String DO_COMMUNICATION_MATRIX = "Show Communication Matrix";
 
 	private PerfExplorerClient mainFrame;
 
-	private String scriptName;
+	private String scriptName = null;
+	private String scriptDir = null;
+	private String expressionFilename;
 
 	public PerfExplorerActionListener (PerfExplorerClient mainFrame) {
 		super();
@@ -97,7 +132,7 @@ public class PerfExplorerActionListener implements ActionListener {
 							try {
 								PerfExplorerClient frame = PerfExplorerClient.getMainFrame();
 								UIManager.setLookAndFeel(info[i].getClassName());
-								Frame[] frames = frame.getFrames();
+								Frame[] frames = Frame.getFrames();
 								for (int x = 0 ; x < frames.length ; x++) {
 									Component[] comps = frame.getComponents();
 									for (int y = 0 ; y < comps.length ; y++) {
@@ -110,16 +145,26 @@ public class PerfExplorerActionListener implements ActionListener {
 							}
 						}
 					}
+				} else if (arg.equals(DATABASE_CONFIGURATION)) {
+					databaseConfiguration();
+				} else if (arg.equals(DERIVE_METRIC)) {
+					deriveMetric();
+				} else if (arg.equals(PARSE_EXPRESSION)) {
+					parseExpression();
+				} else if (arg.equals(REPARSE_EXPRESSION)) {
+					reparseExpression();
+				} else if (arg.equals(LOAD_PROFILE)) {
+					loadProfile();
 				} else if (arg.equals(LOADSCRIPT)) {
 					loadScript();
 				} else if (arg.equals(RERUNSCRIPT)) {
 					runScript();
 				} else if (arg.equals(SAVE_MAIN)) {
 					saveMain();
-				} else if (arg.equals(SAVE)) {
-					saveThyself();
-			// help menu items
-				} else if (arg.equals(ABOUT)) {
+				} else if (arg.equals(CONSOLE)) {
+					new Console();
+				} 
+				else if (arg.equals(ABOUT)) {
 					createAboutWindow();
 				} else if (arg.equals(SEARCH)) {
 					createHelpWindow();
@@ -133,15 +178,32 @@ public class PerfExplorerActionListener implements ActionListener {
 				} else if (arg.equals(NUM_CLUSTERS)) {
 					createClusterSizeWindow();
 				} else if (arg.equals(DO_CLUSTERING)) {
-					if (validAnalysisSelection())
+					if (validAnalysisSelection()) {
+						PerfExplorerModel.getModel().setClusterValueType("exclusive");
 						createDoClusteringWindow();
+					}
+				} else if (arg.equals(DO_INC_CLUSTERING)) {
+					if (validAnalysisSelection()) {
+						createDoClusteringWindow();
+						PerfExplorerModel.getModel().setClusterValueType("inclusive");
+					}
 				} else if (arg.equals(DO_CORRELATION_ANALYSIS)) {
-					if (validCorrelationSelection())
+					if (validCorrelationSelection()) {
+						PerfExplorerModel.getModel().setClusterValueType("exclusive");
 						createDoCorrelationWindow();
+					}
+				} else if (arg.equals(DO_INC_CORRELATION_ANALYSIS)) {
+					if (validCorrelationSelection()) {
+						PerfExplorerModel.getModel().setClusterValueType("inclusive");
+						createDoCorrelationWindow();
+					}
 			// data display items
 				} else if (arg.equals(DO_CORRELATION_CUBE)) {
 					if (valid3DSelection())
 						PerfExplorerCube.doCorrelationCube();
+				} else if (arg.equals(DO_LAUNCH_WEKA)) {
+					if (valid3DSelection())
+						PerfExplorerWekaLauncher.launch();
 				} else if (arg.equals(DO_IQR_BOXCHART)) {
 					if (valid3DSelection())
 						PerfExplorerBoxChart.doIQRBoxChart();
@@ -154,7 +216,14 @@ public class PerfExplorerActionListener implements ActionListener {
 				} else if (arg.equals(DO_PROBABILITY_PLOT)) {
 					if (validDistributionSelection())
 						PerfExplorerProbabilityPlot.doProbabilityPlot();
+				} else if (arg.equals(DO_COMMUNICATION_MATRIX)) {
+					if (validCommunicationMatrixSelection()) {
+						CommunicationMatrix mat = new CommunicationMatrix();
+						mat.doCommunicationMatrix();
+					}
 			// chart items
+				} else if (arg.equals(DO_CHARTS)) {
+					createChartDialogBox();
 				} else if (arg.equals(SET_PROBLEM_SIZE)) {
 					checkAndSetProblemSize(true);
 				} else if (arg.equals(SET_GROUPNAME)) {
@@ -167,59 +236,93 @@ public class PerfExplorerActionListener implements ActionListener {
 					checkAndSetTimesteps(true);
 				} else if (arg.equals(TIMESTEPS_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetTimesteps(false))
+						if (checkAndSetTimesteps(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doTimestepsChart();
+						}
 				} else if (arg.equals(TOTAL_TIME_CHART)) {
-					if (checkAndSetMetricName(false))
+					if (checkAndSetMetricName(false)) {
+						ChartGUI.checkScaling();
 						PerfExplorerChart.doTotalTimeChart();
+					}
 				} else if (arg.equals(EFFICIENCY_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doEfficiencyChart();
+						}
 				} else if (arg.equals(EFFICIENCY_EVENTS_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
-						PerfExplorerChart.doEfficiencyEventsChart();
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
+							PerfExplorerChart.doEfficiencyEventsChart();
+						}
 				} else if (arg.equals(EFFICIENCY_ONE_EVENT_CHART)) {
 					if (checkAndSetMetricName(false))
 						if (checkAndSetEventName(false))
-						if (checkAndSetProblemSize(false))
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doEfficiencyOneEventChart();
+						}
 				} else if (arg.equals(SPEEDUP_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
-						PerfExplorerChart.doSpeedupChart();
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
+							PerfExplorerChart.doSpeedupChart();
+						}
 				} else if (arg.equals(SPEEDUP_EVENTS_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
-						PerfExplorerChart.doSpeedupEventsChart();
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
+							PerfExplorerChart.doSpeedupEventsChart();
+						}
+				} else if (arg.equals(STACKED_BAR_CHART)) {
+					if (checkAndSetMetricName(false))
+						PerfExplorerChart.doStackedBarChart();
+				} else if (arg.equals(ALIGNED_STACKED_BAR_CHART)) {
+					if (checkAndSetMetricName(false))
+						PerfExplorerChart.doAlignedStackedBarChart();
 				} else if (arg.equals(SPEEDUP_ONE_EVENT_CHART)) {
 					if (checkAndSetMetricName(false))
 						if (checkAndSetEventName(false))
-						if (checkAndSetProblemSize(false))
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doSpeedupOneEventChart();
+						}
 				} else if (arg.equals(COMMUNICATION_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetGroupName(false))
+						if (checkAndSetGroupName(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doCommunicationChart();
+						}
 				} else if (arg.equals(FRACTION_CHART)) {
-					if (checkAndSetMetricName(false))
+					if (checkAndSetMetricName(false)) {
+						ChartGUI.checkScaling();
 						PerfExplorerChart.doFractionChart();
+					}
 				} else if (arg.equals(CORRELATION_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
 							PerfExplorerChart.doCorrelationChart();
+						}
 				} else if (arg.equals(EFFICIENCY_PHASE_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
-						PerfExplorerChart.doEfficiencyPhasesChart();
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
+							PerfExplorerChart.doEfficiencyPhasesChart();
+						}
 				} else if (arg.equals(SPEEDUP_PHASE_CHART)) {
 					if (checkAndSetMetricName(false))
-						if (checkAndSetProblemSize(false))
-						PerfExplorerChart.doSpeedupPhasesChart();
+						if (checkAndSetProblemSize(false)) {
+							ChartGUI.checkScaling();
+							PerfExplorerChart.doSpeedupPhasesChart();
+						}
 				} else if (arg.equals(FRACTION_PHASE_CHART)) {
-					if (checkAndSetMetricName(false))
+					if (checkAndSetMetricName(false)) {
+						ChartGUI.checkScaling();
 						PerfExplorerChart.doFractionPhasesChart();
+					}
 			// view items
 				} else if (arg.equals(CREATE_NEW_VIEW)) {
 					int parent = 0; // no parent
@@ -232,12 +335,25 @@ public class PerfExplorerActionListener implements ActionListener {
 					System.out.println("unknown event! " + arg);
 				}
 			}
+		} catch (SeriesException e) {
+			// this shouldn't happen, but if it does, handle it gracefully.
+			StringBuilder sb = new StringBuilder();
+			sb.append("Two or more trials in this selection have the same total number of threads of execution, and an error occurred.\n");
+			sb.append("To create a scalability chart, please ensure the trials selected have different numbers of threads.\n");
+			sb.append("To create a different parametric chart, please use the custom chart interface.");			
+			JOptionPane.showMessageDialog(PerfExplorerClient.getMainFrame(), sb.toString(),
+					"Selection Warning", JOptionPane.ERROR_MESSAGE);
 		} catch (Exception e) {
 			System.err.println("actionPerformed Exception: " + e.getMessage());
 			e.printStackTrace();
 		} 
 	}
 
+	private void createChartDialogBox() {
+		System.out.println("Opening Dialog Box to create charts");
+		JFrame dialogBox = ChartGUI.getInstance(true);
+	}
+
 	private void updateAll (Container container) {
 		Component[] comps = container.getComponents();
 		for (int y = 0 ; y < comps.length ; y++) {
@@ -264,7 +380,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		long memUsage = (Runtime.getRuntime().totalMemory() -
 			Runtime.getRuntime().freeMemory()) / 1024;
 
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		//for (java.util.Enumeration e = System.getProperties().propertyNames(); e.hasMoreElements() ;) {
 		    //System.out.println(e.nextElement());
 		//}
@@ -281,7 +397,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		buf.append("\nuser.country : " + System.getProperty("user.country"));
 		buf.append("\nuser.timezone : " + System.getProperty("user.timezone"));
 		buf.append("\nuser.language : " + System.getProperty("user.language"));
-		String message = new String("PerfExplorer 1.0\n" +
+		String message = new String("PerfExplorer 2.0\n" +
 					getVersionString() + "\nJVM Heap Size: " + memUsage
 					+ "kb\n" + buf.toString());
 		ImageIcon icon = createImageIcon(Utility.getResource("tau-large.png"));
@@ -289,15 +405,67 @@ public class PerfExplorerActionListener implements ActionListener {
 			"About PerfExplorer", JOptionPane.INFORMATION_MESSAGE, icon);
 	}
 
-    /** Returns an ImageIcon, or null if the path was invalid. */
-    protected static ImageIcon createImageIcon(java.net.URL imgURL) {
-        if (imgURL != null) {
-            return new ImageIcon(imgURL);
-        } else {
-            System.err.println("Couldn't find file: " + imgURL);
-            return null;
-        }
-    }
+	public void databaseConfiguration() {
+		(new DatabaseManagerWindow(PerfExplorerClient.getMainFrame())).setVisible(true);
+	}
+	public void deriveMetric() {
+		PerfExplorerModel theModel = PerfExplorerModel.getModel();
+		PerfExplorerConnection server = PerfExplorerConnection.getConnection();
+		
+		Application app = null;
+		Experiment exp = null;
+		Trial trial = null;
+		
+		Object selection = theModel.getCurrentSelection();
+		if(selection instanceof Application){
+			app = (Application) selection;
+		}else if(selection instanceof Experiment){
+			exp = (Experiment) selection;
+			ListIterator<Application> apps = server.getApplicationList();
+			while(apps.hasNext()){
+				app = apps.next();
+				if(app.getID()==exp.getApplicationID()){
+					break;
+				}
+			}
+		}else if(selection instanceof Trial){
+			trial = (Trial)selection;
+			ListIterator<Application> apps = server.getApplicationList();
+			while(apps.hasNext()){
+				app = apps.next();
+				if(app.getID()==trial.getApplicationID()){
+					break;
+				}
+			}
+			ListIterator<Experiment> experiments = server.getExperimentList(trial.getApplicationID());
+			while(experiments.hasNext()){
+				exp = experiments.next();
+				if(exp.getID()==trial.getExperimentID()){
+					break;
+				}
+			}
+		}
+
+		if(app != null){
+
+			DerivedMetricWindow window = new DerivedMetricWindow(PerfExplorerClient.getMainFrame(),
+					app,exp,trial);
+			window.setVisible(true);
+		}else{
+			JOptionPane.showMessageDialog(mainFrame, "Please select an Application, Experiment or Trial.",
+					"Selection Error", JOptionPane.ERROR_MESSAGE);
+		}
+	}
+
+	/** Returns an ImageIcon, or null if the path was invalid. */
+	protected static ImageIcon createImageIcon(java.net.URL imgURL) {
+		if (imgURL != null) {
+			return new ImageIcon(imgURL);
+		} else {
+			System.err.println("Couldn't find file: " + imgURL);
+			return null;
+		}
+	}
 
 	public void createHelpWindow() {
 		ImageIcon icon = createImageIcon(Utility.getResource("tau-large.png"));
@@ -308,13 +476,13 @@ public class PerfExplorerActionListener implements ActionListener {
 
 	public void createMethodWindow() {
 		Object[] options = AnalysisType.getClusterMethods();
-		String reply = (String)JOptionPane.showInputDialog (mainFrame,
+		AnalysisType reply = (AnalysisType)JOptionPane.showInputDialog (mainFrame,
 			"Select a cluster method:",
 			"Cluster Method",
 			JOptionPane.PLAIN_MESSAGE,
 			null,
 			options,
-			AnalysisType.K_MEANS);
+			PerfExplorerModel.getModel().getClusterMethod());
 		PerfExplorerModel.getModel().setClusterMethod(reply);
 	}
 
@@ -329,11 +497,11 @@ public class PerfExplorerActionListener implements ActionListener {
 			TransformationType.NONE);
         TransformationType reply = (TransformationType)obj;
 		PerfExplorerModel.getModel().setDimensionReduction(reply);
-		if (PerfExplorerModel.getModel().getDimensionReduction().equals(reply)) {
+		if (reply!=TransformationType.NONE&&reply!=null&&PerfExplorerModel.getModel().getDimensionReduction().equals(reply)) {
 			String reply2 = (String)JOptionPane.showInputDialog (mainFrame,
 				"Only select events with exclusive time % greater than X:\n(where 0 <= X < 100)",
 				"Minimum Percentage", JOptionPane.PLAIN_MESSAGE);
-			if (reply != null && !reply.equals(""))
+			if (reply2 != null && !reply2.equals(""))
 				PerfExplorerModel.getModel().setXPercent(reply2);
 		}
 	}
@@ -359,6 +527,7 @@ public class PerfExplorerActionListener implements ActionListener {
 			PerfExplorerModel.getModel().setNumberOfClusters(reply);
 	}
 
+	@SuppressWarnings("unchecked") // for trial.getMetricS() call
 	public void createDoClusteringWindow() {
 		PerfExplorerModel theModel = PerfExplorerModel.getModel();
 		Object selection = theModel.getCurrentSelection();
@@ -368,18 +537,18 @@ public class PerfExplorerActionListener implements ActionListener {
 			if (reply == 1) {
 				Application application = (Application)selection;
 				PerfExplorerConnection server = PerfExplorerConnection.getConnection();
-				ListIterator experiments = server.getExperimentList(application.getID());
+				ListIterator<Experiment> experiments = server.getExperimentList(application.getID());
 				Experiment experiment = null;
 				boolean failed = false;
 				while (experiments.hasNext() && !failed) {
-					experiment = (Experiment) experiments.next();
+					experiment = experiments.next();
 					theModel.setCurrentSelection(experiment);
-					ListIterator trials = server.getTrialList(experiment.getID());
+					ListIterator<Trial> trials = server.getTrialList(experiment.getID(),false);
 					Trial trial = null;
 					while (trials.hasNext() && !failed) {
-						trial = (Trial) trials.next();
+						trial = trials.next();
 						theModel.setCurrentSelection(trial);
-						List metrics = trial.getMetrics();
+						List<Metric> metrics = trial.getMetrics();
 						for (int i = 0; i < metrics.size(); i++) {
 							Object metric = metrics.get(i);
 							theModel.setCurrentSelection(metric);
@@ -409,13 +578,13 @@ public class PerfExplorerActionListener implements ActionListener {
 			if (reply == 1) {
 				Experiment experiment = (Experiment)selection;
 				PerfExplorerConnection server = PerfExplorerConnection.getConnection();
-				ListIterator trials = server.getTrialList(experiment.getID());
+				ListIterator<Trial> trials = server.getTrialList(experiment.getID(),false);
 				Trial trial = null;
 				boolean failed = false;
 				while (trials.hasNext() && !failed) {
-					trial = (Trial) trials.next();
+					trial = trials.next();
 					theModel.setCurrentSelection(trial);
-					List metrics = trial.getMetrics();
+					List<Metric> metrics = trial.getMetrics();
 					for (int i = 0; i < metrics.size(); i++) {
 						Object metric = metrics.get(i);
 						theModel.setCurrentSelection(metric);
@@ -448,7 +617,7 @@ public class PerfExplorerActionListener implements ActionListener {
 				RMIPerfExplorerModel modelCopy = theModel.copy();
 				status = server.requestAnalysis(modelCopy, true);
 				Trial trial = (Trial)selection;
-				List metrics = trial.getMetrics();
+				List<Metric> metrics = trial.getMetrics();
 				for (int i = 0; i < metrics.size(); i++) {
 					modelCopy = theModel.copy();
 					Object metric = metrics.get(i);
@@ -511,7 +680,7 @@ public class PerfExplorerActionListener implements ActionListener {
 
 	private int getConfirmation(RMIPerfExplorerModel theModel) {
 		Object [] options = { "No, not yet" , "Yes, do analysis" };
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		buf.append("Analysis method: " + theModel.getClusterMethod());
 		buf.append("\nDimension Reduction: " + theModel.getDimensionReduction());
 		if (theModel.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) 
@@ -520,7 +689,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		if (!theModel.getClusterMethod().equals(AnalysisType.CORRELATION_ANALYSIS))
 			buf.append("\nMax Clusters: " + theModel.getNumberOfClusters());
 		buf.append("\nTrial: " + theModel.toString());
-		buf.append("\n\nPerform clustering with the these options?");
+		buf.append("\n\nPerform " + theModel.getClusterMethod() + " with the these options?");
 		int reply = JOptionPane.showOptionDialog(mainFrame, buf.toString(),
 			"Confirm Analysis",
 			JOptionPane.YES_NO_OPTION, 
@@ -604,6 +773,19 @@ public class PerfExplorerActionListener implements ActionListener {
 		return true;
 	}
 
+	private boolean validCommunicationMatrixSelection () {
+		PerfExplorerModel theModel = PerfExplorerModel.getModel();
+		Object selection = theModel.getCurrentSelection();
+		// allow only Trials
+		if ((selection == null) || !(selection instanceof Trial)) {
+			JOptionPane.showMessageDialog(mainFrame, 
+				"Please select one Trial.",
+				"Selection Error", JOptionPane.ERROR_MESSAGE);
+			return false;
+		}
+		return true;
+	}
+
 	private boolean checkAndSetMetricName (boolean forceIt) {
 		//TODO - MAKE SURE THE METRIC EXISTS IN THE SELECTED TRIALS!
 		PerfExplorerModel theModel = PerfExplorerModel.getModel();
@@ -612,7 +794,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		String metric = theModel.getMetricName();
 		if (forceIt || (metric == null)) {
 			PerfExplorerConnection server = PerfExplorerConnection.getConnection();
-			List metrics = server.getPotentialMetrics(theModel);
+			List<String> metrics = server.getPotentialMetrics(theModel);
 			Object[] options = metrics.toArray();
 			if (options.length > 0) {
 				if (options.length == 1) {
@@ -635,7 +817,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		String group = theModel.getGroupName();
 		if (forceIt || group == null) {
 			PerfExplorerConnection server = PerfExplorerConnection.getConnection();
-			List metrics = server.getPotentialGroups(theModel);
+			List<String> metrics = server.getPotentialGroups(theModel);
 			Object[] options = metrics.toArray();
 			if (options.length > 0) {
 				if (options.length == 1) {
@@ -658,7 +840,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		String event = theModel.getEventName();
 		if (forceIt || event == null) {
 			PerfExplorerConnection server = PerfExplorerConnection.getConnection();
-			List metrics = server.getPotentialEvents(theModel);
+			List<String> metrics = server.getPotentialEvents(theModel);
 			Object[] options = metrics.toArray();
 			if (options.length > 0) {
 				if (options.length == 1) {
@@ -691,7 +873,7 @@ public class PerfExplorerActionListener implements ActionListener {
 		PerfExplorerModel theModel = PerfExplorerModel.getModel();
 		Boolean constantProblem = theModel.getConstantProblem();
 		if (forceIt || constantProblem == null) {
-			List answers = new ArrayList();
+			List<String> answers = new ArrayList<String>();
 			answers.add("The problem size remains constant. (strong scaling)");
 			answers.add("The problem size increases as the processor count increases. (weak scaling)");
 			Object[] options = answers.toArray();
@@ -707,16 +889,137 @@ public class PerfExplorerActionListener implements ActionListener {
 		return (!forceIt && constantProblem == null) ? false : true;
 	}
 
-	private boolean loadScript () {
-		// open a file chooser dialog
-		JFileChooser fc = new JFileChooser(System.getProperty("user.dir"));
-		int returnVal = fc.showOpenDialog(mainFrame);
-		scriptName = fc.getSelectedFile().getAbsolutePath();
-		runScript();
+	public static int setSession (String name) {
+		try {
+			PerfExplorerConnection server = PerfExplorerConnection.getConnection();
+			List<String> configNames = server.getConfigNames();
+			int i = 0;
+			for (String configName : configNames) {
+//				System.out.println(server.getConnectionString());
+				if (configName.equals(name)) {
+					server.setConnectionIndex(i);
+					return i;
+				}
+				i++;
+			}
+		} catch (Exception e) {}
+		return 0;
+	}
+	
+	private boolean loadProfile () {
+		// the default session is the last session, which is the perfexplorer working database.
+//		setSession(Constants.PERFEXPLORER_WORKING_CONFIG);
+
+		Application app = new Application();
+		Experiment exp = new Experiment();
+		// create new application, experiment
+		try {
+			app.setName("New Application");
+			app.setID(app.saveApplication(PerfExplorerServer.getServer().getDB()));
+			PerfExplorerModel.getModel().setCurrentSelection(app);
+			exp.setApplicationID(app.getID());
+			exp.setName("New Experiment");
+			exp.setID(exp.saveExperiment(PerfExplorerServer.getServer().getDB()));
+			PerfExplorerModel.getModel().setCurrentSelection(exp);
+		} catch (SQLException e) {
+			// TODO Auto-generated catch block
+			e.printStackTrace();
+		}
+		LoadTrialWindow ltw = new LoadTrialWindow(mainFrame, this, app, exp, true, true);
+		ltw.setVisible(true);
+
 		return true;
 	}
 
-	private boolean runScript() {
+	private boolean loadScript () {
+		JFileChooser fc = null;
+		// open a file chooser dialog
+		if (scriptDir == null) {
+			fc = new JFileChooser(System.getProperty("user.dir"));
+		} else {
+			fc = new JFileChooser(scriptDir);
+		}
+		int returnVal = fc.showOpenDialog(mainFrame);
+		if (returnVal == JFileChooser.APPROVE_OPTION) {
+			scriptName = fc.getSelectedFile().getAbsolutePath();
+			// save where we were
+			scriptDir = fc.getSelectedFile().getParent();
+			runScript();
+			return true;
+		}
+		return false;
+	}
+
+
+
+	private boolean parseExpression() {
+		PerfExplorerModel theModel = PerfExplorerModel.getModel();
+		Object selection = theModel.getCurrentSelection();
+		if(selection==null){
+			JOptionPane.showMessageDialog(mainFrame, "Please select an Application, Experiment or Trial.",
+					"Selection Error", JOptionPane.ERROR_MESSAGE);
+			return false;
+		}
+		
+		JFileChooser fc = null;
+		// open a file chooser dialog
+		if (scriptDir == null) {
+			fc = new JFileChooser(System.getProperty("user.dir"));
+		} else {
+			fc = new JFileChooser(scriptDir);
+		}
+		int returnVal = fc.showOpenDialog(mainFrame);
+		if (returnVal == JFileChooser.APPROVE_OPTION) {
+			expressionFilename = fc.getSelectedFile().getAbsolutePath();
+			// save where we were
+			scriptDir = fc.getSelectedFile().getParent();
+			reparseExpression();
+			return true;
+		}
+		return false;
+
+	}
+	public boolean reparseExpression() {
+		
+		PerfExplorerModel theModel = PerfExplorerModel.getModel();
+		Application a = theModel.getApplication();
+		Experiment ex = theModel.getExperiment();
+		Trial t = theModel.getTrial();
+		String app =null,exp=null,trial=null;
+		if(a!=null)app = a.getName();
+		if(ex!=null)exp = ex.getName();
+		if(t!=null)trial = t.getName();
+		if (expressionFilename == null) {
+			// make sure a script file has been loaded first
+			JOptionPane.showMessageDialog(mainFrame, 
+					"Please load an expression file first.",
+					"Expression File Not Found", JOptionPane.ERROR_MESSAGE);
+			return false;
+		} else {
+			if(app != null){
+			String script;
+			try {
+				script = new PerfExplorerExpression().getScriptFromFile(app,exp,trial,expressionFilename);
+				ScriptThread runner = new ScriptThread(script,true);
+			} catch (FileNotFoundException e) {
+				JOptionPane.showMessageDialog(mainFrame, 
+						"",
+						"Script File Not Found", JOptionPane.ERROR_MESSAGE);
+			} catch (ParsingException e) {
+				JOptionPane.showMessageDialog(mainFrame, 
+						"",
+						"Unable to Parse Expression", JOptionPane.ERROR_MESSAGE);
+			}
+			return true;
+			}else{
+				JOptionPane.showMessageDialog(mainFrame, "Please select an Application, Experiment or Trial.",
+						"Selection Error", JOptionPane.ERROR_MESSAGE);
+				return false;
+			}
+		}
+	}
+
+	public boolean runScript() {
 		if (scriptName == null) {
 			// make sure a script file has been loaded first
 			JOptionPane.showMessageDialog(mainFrame, 
@@ -725,22 +1028,12 @@ public class PerfExplorerActionListener implements ActionListener {
 			return false;
 		} else {
 			// run the script
-			edu.uoregon.tau.common.TauScripter.execfile(scriptName);
+			//PythonInterpreterFactory.defaultfactory.getPythonInterpreter().execfile(scriptName);
+			ScriptThread runner = new ScriptThread(scriptName);
 			return true;
 		}
 	}
 
-    public void saveThyself() {
-        //System.out.println("Daemon come out!");
-        try {
-            VectorExport.promptForVectorExport (ChartPane.getPane(), "PerfExplorer");
-        } catch (Exception e) {
-            System.out.println("File Export Failed!");
-        }
-        return;
-    }
-
-
     public void saveMain() {
         //System.out.println("Daemon come out!");
         try {
@@ -751,4 +1044,45 @@ public class PerfExplorerActionListener implements ActionListener {
         return;
     }
 
+	/**
+	 * @return the scriptName
+	 */
+	public String getScriptName() {
+		return scriptName;
+	}
+
+	/**
+	 * @param scriptName the scriptName to set
+	 */
+	public void setScriptName(String scriptName) {
+		this.scriptName = scriptName;
+	}
+
+    public void handleDelete(Object clickedOnObject) {
+    	try {
+            DatabaseAPI databaseAPI = PerfExplorerServer.getServer().getSession();
+            if (clickedOnObject instanceof Application) {
+	            Application application = (Application) clickedOnObject;
+	            if (databaseAPI != null) {
+	                databaseAPI.deleteApplication(application.getID());
+	                //treeModel.removeNodeFromParent(application.getDMTN());
+	            }
+	        } else if (clickedOnObject instanceof Experiment) {
+	            Experiment experiment = (Experiment) clickedOnObject;
+	            if (databaseAPI != null) {
+	                databaseAPI.deleteExperiment(experiment.getID());
+                    //treeModel.removeNodeFromParent(experiment.getDMTN());
+	            }	
+	        } else if (clickedOnObject instanceof Trial) {
+	        	Trial ppTrial = (Trial) clickedOnObject;
+                if (databaseAPI != null) {
+                    databaseAPI.deleteTrial(ppTrial.getID());
+                    //treeModel.removeNodeFromParent(ppTrial.getDMTN());
+                }
+	        }
+        } catch (Exception e) {
+        	System.out.print(e.getMessage());
+        	e.printStackTrace();
+        }
+    }
 }
