@@ -1,4 +1,4 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
 import javax.swing.*;
 import java.awt.event.*;
