@@ -1,19 +1,38 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
-import java.rmi.*;
-import common.*;
-import server.PerfExplorerServer;
-import java.util.*;
-import javax.swing.*;
+import java.rmi.Naming;
+import java.rmi.RMISecurityManager;
+import java.rmi.RemoteException;
+import java.util.List;
+import java.util.ListIterator;
+import java.util.Map;
+
+import javax.swing.JOptionPane;
+
+import edu.uoregon.tau.perfdmf.Application;
+import edu.uoregon.tau.perfdmf.Experiment;
+import edu.uoregon.tau.perfdmf.Trial;
+import edu.uoregon.tau.perfexplorer.common.ChartDataType;
+import edu.uoregon.tau.perfexplorer.common.RMIChartData;
+import edu.uoregon.tau.perfexplorer.common.RMICubeData;
+import edu.uoregon.tau.perfexplorer.common.RMIGeneralChartData;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorer;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.common.RMIPerformanceResults;
+import edu.uoregon.tau.perfexplorer.common.RMISortableIntervalEvent;
+import edu.uoregon.tau.perfexplorer.common.RMIVarianceData;
+import edu.uoregon.tau.perfexplorer.common.RMIView;
+import edu.uoregon.tau.perfexplorer.server.PerfExplorerServer;
 
 public class PerfExplorerConnection {
 
     private static PerfExplorerConnection theConnection = null;
 
     private RMIPerfExplorer server = null;
-    private static boolean standalone = false;
+    private static boolean standalone = true;
     private static String configFile = null;
-    private static EngineType analysisEngine = EngineType.WEKA;
+    private static String tauHome = null;
+    private static String tauArch = null;
     private int connectionIndex = 0;
 
     private PerfExplorerConnection () {
@@ -22,7 +41,7 @@ public class PerfExplorerConnection {
 
 	private void makeConnection() {
 		if (standalone) {
-	    	server = PerfExplorerServer.getServer(configFile, analysisEngine);
+	    	server = PerfExplorerServer.getServer(configFile, tauHome, tauArch);
 		} else {
 	    	if (System.getSecurityManager() == null) {
 				System.setSecurityManager(new RMISecurityManager());
@@ -49,11 +68,15 @@ public class PerfExplorerConnection {
     public static void setConfigFile (String configFile) {
 	PerfExplorerConnection.configFile = configFile;
     }
-	
-    public static void setAnalysisEngine(EngineType analysisEngine) {
-	PerfExplorerConnection.analysisEngine = analysisEngine;
-    }
 
+    public static void setTauHome (String tauHome) {
+	PerfExplorerConnection.tauHome = tauHome;
+    }
+	
+    public static void setTauArch (String tauArch) {
+    	PerfExplorerConnection.tauArch = tauArch;
+    }
+    	
     public static PerfExplorerConnection getConnection() {
 	if (theConnection == null) {
 	    theConnection = new PerfExplorerConnection();
@@ -97,8 +120,8 @@ public class PerfExplorerConnection {
     }
 
     
-    public ListIterator getApplicationList() {
-	ListIterator tmpIterator = null;
+    public ListIterator<Application> getApplicationList() {
+	ListIterator<Application> tmpIterator = null;
 	try {
 	    tmpIterator = server.getApplicationList().listIterator();
 	} catch (RemoteException e) {
@@ -107,8 +130,8 @@ public class PerfExplorerConnection {
 	return tmpIterator;
     }
 
-    public ListIterator getExperimentList(int applicationID) {
-	ListIterator tmpIterator = null;
+    public ListIterator<Experiment> getExperimentList(int applicationID) {
+	ListIterator<Experiment> tmpIterator = null;
 	try {
 	    tmpIterator =
 		server.getExperimentList(applicationID).listIterator();
@@ -118,11 +141,11 @@ public class PerfExplorerConnection {
 	return tmpIterator;
     }
 
-    public ListIterator getTrialList(int experimentID) {
-	ListIterator tmpIterator = null;
+    public ListIterator<Trial> getTrialList(int experimentID, boolean getXMLMetadata) {
+	ListIterator<Trial> tmpIterator = null;
 	try {
 	    tmpIterator =
-		server.getTrialList(experimentID).listIterator();
+		server.getTrialList(experimentID,getXMLMetadata).listIterator();
 	} catch (RemoteException e) {
 	    handleError(e, "getTrialList(" + experimentID + ")");
 	}
@@ -187,8 +210,8 @@ public class PerfExplorerConnection {
 	return data;
     }
 
-    public List getPotentialGroups(PerfExplorerModel model) {
-	List groups = null;
+    public List<String> getPotentialGroups(PerfExplorerModel model) {
+	List<String> groups = null;
 	try {
 	    groups = server.getPotentialGroups(model);
 	} catch (RemoteException e) {
@@ -197,8 +220,8 @@ public class PerfExplorerConnection {
 	return groups;
     }
 
-    public List getPotentialMetrics(PerfExplorerModel model) {
-	List metrics = null;
+    public List<String> getPotentialMetrics(PerfExplorerModel model) {
+	List<String> metrics = null;
 	try {
 	    metrics = server.getPotentialMetrics(model);
 	} catch (RemoteException e) {
@@ -207,8 +230,8 @@ public class PerfExplorerConnection {
 	return metrics;
     }
 
-    public List getPotentialEvents(PerfExplorerModel model) {
-	List events = null;
+    public List<String> getPotentialEvents(PerfExplorerModel model) {
+	List<String> events = null;
 	try {
 	    events = server.getPotentialEvents(model);
 	} catch (RemoteException e) {
@@ -227,8 +250,8 @@ public class PerfExplorerConnection {
 	return columns;
     }
 
-    public List getPossibleValues(String tableName, String columnName) {
-	List values = null;
+    public List<String> getPossibleValues(String tableName, String columnName) {
+	List<String> values = null;
 	try {
 	    values = server.getPossibleValues(tableName, columnName);
 	} catch (RemoteException e) {
@@ -255,8 +278,8 @@ public class PerfExplorerConnection {
 		}
     }
 
-    public List getViews(int parent) {
-	List views = null;
+    public List<RMIView> getViews(int parent) {
+	List<RMIView> views = null;
 	try {
 	    views = server.getViews(parent);
 	} catch (RemoteException e) {
@@ -265,10 +288,10 @@ public class PerfExplorerConnection {
 	return views;
     }
 
-    public ListIterator getTrialsForView(List views) {
-	ListIterator trials = null;
+    public ListIterator<Trial> getTrialsForView(List<RMIView> views, boolean getXMLMetadata) {
+	ListIterator<Trial> trials = null;
 	try {
-	    trials = server.getTrialsForView(views).listIterator();
+	    trials = server.getTrialsForView(views,getXMLMetadata).listIterator();
 	} catch (RemoteException e) {
 	    handleError(e, "getTrialsForView(" + views + ")");
 	}
@@ -285,10 +308,10 @@ public class PerfExplorerConnection {
 	return results;
     }
 
-    public RMICubeData requestCubeData(PerfExplorerModel model) {
+    public RMICubeData requestCubeData(PerfExplorerModel model, int numFunctions) {
 	RMICubeData results = null;
 	try {
-	    results = server.getCubeData(model);
+	    results = server.getCubeData(model, numFunctions);
 	} catch (RemoteException e) {
 	    handleError(e, "getCubeData(" + model.toString() + ")");
 	}
@@ -305,8 +328,8 @@ public class PerfExplorerConnection {
 		return conn;
 	}
 
-	public List getConnectionStrings() {
-		List conns = null;
+	public List<String> getConnectionStrings() {
+		List<String> conns = null;
 		try {
 	    	conns = server.getConnectionStrings();
 		} catch (RemoteException e) {
@@ -315,8 +338,18 @@ public class PerfExplorerConnection {
 		return conns;
 	}
 
-    public ListIterator getEventList(int trialID, int metricIndex) {
-	ListIterator tmpIterator = null;
+	public List<String> getConfigNames() {
+		List<String> conns = null;
+		try {
+	    	conns = server.getConfigNames();
+		} catch (RemoteException e) {
+	    	handleError(e, "getConfigNames()");
+		}
+		return conns;
+	}
+
+    public ListIterator<RMISortableIntervalEvent> getEventList(int trialID, int metricIndex) {
+	ListIterator<RMISortableIntervalEvent> tmpIterator = null;
 	try {
 	    tmpIterator = server.getEventList(trialID, metricIndex).listIterator();
 	} catch (RemoteException e) {
@@ -325,18 +358,18 @@ public class PerfExplorerConnection {
 	return tmpIterator;
     }
 
-	public List getTrialList(String criteria) {
-		List list = null;
+	public List<Trial> getTrialList(String criteria, boolean getXMLMetadata) {
+		List<Trial> list = null;
 		try {
-			list = server.getTrialList(criteria);
+			list = server.getTrialList(criteria,getXMLMetadata);
 		} catch (RemoteException e) {
 	    	handleError(e, "getTrialList(" + criteria + ")");
 		}
 		return list;
 	}
 
-	public List getChartFieldNames() {
-		List list = null;
+	public List<String> getChartFieldNames() {
+		List<String> list = null;
 		try {
 			list = server.getChartFieldNames();
 		} catch (RemoteException e) {
@@ -345,8 +378,8 @@ public class PerfExplorerConnection {
 		return list;
 	}
 
-    public List getXMLFields(PerfExplorerModel model) {
-		List results = null;
+    public List<String> getXMLFields(PerfExplorerModel model) {
+		List<String> results = null;
 		try {
 	    	results = server.getXMLFields(model);
 		} catch (RemoteException e) {
@@ -375,4 +408,43 @@ public class PerfExplorerConnection {
 		}
 	}
 
+	public void resetServer() {
+		try {
+			server.resetServer();
+		} catch (RemoteException e) {
+	    	handleError(e, "setConnectionIndex(" + connectionIndex + ")");
+		}
+		
+	}
+
+	public List<String> getPotentialAtomicEvents(PerfExplorerModel model) {
+		List<String> events = null;
+		try {
+		    events = server.getPotentialAtomicEvents(model);
+		} catch (RemoteException e) {
+		    handleError(e, "getPotentialEvents(" + model.toString() + ")");
+		}
+		return events;
+    }
+
+	public Map<String, Integer> checkScalabilityChartData(RMIPerfExplorerModel model) {
+		Map<String, Integer> counts = null;
+		try {
+		    counts = server.checkScalabilityChartData(model);
+		} catch (RemoteException e) {
+		    handleError(e, "checkScalabilityChartData(" + model.toString() + ")");
+		}
+		return counts;
+	}
+
+	public Map<String, double[][]> getUserEventData(RMIPerfExplorerModel model) {
+		Map<String, double[][]> userEvents = null;
+		try {
+		    userEvents = server.getUserEventData(model);
+		} catch (RemoteException e) {
+		    handleError(e, "getUserEventData(" + model.toString() + ")");
+		}
+		return userEvents;
+	}
+
 }
