@@ -1,4 +1,4 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
 import javax.swing.*;
 import java.awt.*;
@@ -6,15 +6,15 @@ import java.util.List;
 import java.util.Hashtable;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
-import common.*;
 import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.perfexplorer.common.*;
 
 public class PerfExplorerCorrelationPane extends JScrollPane implements ActionListener {
 
 	private static PerfExplorerCorrelationPane thePane = null;
 
 	private JPanel imagePanel = null;
-	private Hashtable resultsHash = null;
+	private Hashtable<String, RMIPerformanceResults> resultsHash = null;
 	private RMIPerformanceResults results = null;
 
 	public static PerfExplorerCorrelationPane getPane () {
@@ -30,7 +30,7 @@ public class PerfExplorerCorrelationPane extends JScrollPane implements ActionLi
 	private PerfExplorerCorrelationPane (JPanel imagePanel) {
 		super(imagePanel);
 		this.imagePanel = imagePanel;
-		this.resultsHash = new Hashtable();
+		this.resultsHash = new Hashtable<String, RMIPerformanceResults>();
 		JScrollBar jScrollBar = this.getVerticalScrollBar();
 		jScrollBar.setUnitIncrement(35);
 	}
