@@ -5,35 +5,41 @@
  *
  */
 
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
-import common.*;
-import java.io.IOException;
+import java.awt.Toolkit;
+import java.net.URL;
 import java.util.List;
+
 import javax.swing.JFrame;
-import java.lang.Math;
-import org.jfree.data.Range;
-import org.jfree.chart.axis.NumberAxis;
+
 import org.jfree.chart.ChartFactory;
 import org.jfree.chart.ChartPanel;
 import org.jfree.chart.JFreeChart;
+import org.jfree.chart.axis.NumberAxis;
+import org.jfree.chart.labels.XYToolTipGenerator;
 import org.jfree.chart.plot.PlotOrientation;
+import org.jfree.chart.renderer.xy.XYItemRenderer;
+import org.jfree.data.Range;
 import org.jfree.data.statistics.HistogramDataset;
 import org.jfree.data.xy.IntervalXYDataset;
-import org.jfree.ui.ApplicationFrame;
-import org.jfree.ui.RefineryUtilities;
-import org.jfree.chart.renderer.xy.XYItemRenderer;
-import org.jfree.chart.labels.XYToolTipGenerator;
 import org.jfree.data.xy.XYDataset;
-import java.net.URL;
+
 import edu.uoregon.tau.common.Utility;
-import java.awt.Toolkit;
+import edu.uoregon.tau.perfexplorer.common.ChartDataType;
+import edu.uoregon.tau.perfexplorer.common.RMIChartData;
+import edu.uoregon.tau.perfexplorer.common.RMISortableIntervalEvent;
 
 /**
  * A demo of the {@link HistogramDataset} class.
  */
 public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
 
+	/**
+	 * 
+	 */
+	private static final long serialVersionUID = -6247716923954756815L;
+
 	public PerfExplorerHistogramChart(JFreeChart chart, String name) {
 		super(chart, name);
 	}
@@ -58,9 +64,9 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
 		ChartPanel panel = new ChartPanel(chart);
 		panel.setDisplayToolTips(true);
 		XYItemRenderer renderer = chart.getXYPlot().getRenderer();
-        renderer.setToolTipGenerator(new XYToolTipGenerator() {
+        renderer.setBaseToolTipGenerator(new XYToolTipGenerator() {
             public String generateToolTip(XYDataset dataset, int arg1, int arg2) {
-                return "<html>Event: " + dataset.getSeriesName(arg1) + 
+                return "<html>Event: " + dataset.getSeriesKey(arg1) + 
                 "<BR>Count: " + dataset.getYValue(arg1, arg2) + "</html>";
             }
         });
@@ -73,6 +79,28 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
 		return window;
 	}
 
+	public static JFrame doHistogram(RMIChartData data, String title) {
+		// build the chart
+        IntervalXYDataset dataset = createDataset(data);
+        JFreeChart chart = createChart(dataset, title);
+        
+		ChartPanel panel = new ChartPanel(chart);
+		panel.setDisplayToolTips(true);
+		XYItemRenderer renderer = chart.getXYPlot().getRenderer();
+        renderer.setBaseToolTipGenerator(new XYToolTipGenerator() {
+            public String generateToolTip(XYDataset dataset, int arg1, int arg2) {
+                return "<html>Value: " + dataset.getSeriesKey(arg1) + 
+                "<BR>Count: " + dataset.getYValue(arg1, arg2) + "</html>";
+            }
+        });
+
+		JFrame window = new PerfExplorerHistogramChart (chart, "Distributions");
+        URL url = Utility.getResource("tau32x32.gif");
+		if (url != null) 
+			window.setIconImage(Toolkit.getDefaultToolkit().getImage(url));
+		return window;
+	}
+
     private static IntervalXYDataset createDataset(RMIChartData inData) {
         HistogramDataset dataset = new HistogramDataset();
         List names = inData.getRowLabels();
@@ -91,7 +119,8 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
     		double range = max - min;
     		//System.out.println("Min: " + min + ", Max: " + max + ", Range: " + range);
     		for (int j = 0; j < doubles.size(); j++) {
-    			values[j] = (((double[])(doubles.get(j)))[1]-min)/range;   
+    			//values[j] = (((double[])(doubles.get(j)))[1]-min)/range;   
+    			values[j] = ((double[])(doubles.get(j)))[1];   
     		}
 			int bins = 10;
 			if (doubles.size() >= 2098)
@@ -142,5 +171,26 @@ public class PerfExplorerHistogramChart extends PerfExplorerChartWindow {
         chart.getXYPlot().setForegroundAlpha(0.75f);
         return chart;
     }
+
+     /**
+     * Creates a chart.
+     * 
+     * @param dataset  a dataset.
+     * 
+     * @return The chart.
+     */
+    private static JFreeChart createChart(IntervalXYDataset dataset, String title) {
+        JFreeChart chart = ChartFactory.createHistogram(
+            title,
+            "Size", 
+            "Count", 
+            dataset, 
+            PlotOrientation.VERTICAL, 
+            true, 
+            false, 
+            false
+        );
+        return chart;
+    }
         
 }
