@@ -1,4 +1,4 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
 import javax.swing.*;
 
@@ -21,6 +21,30 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		JMenu fileMenu = new JMenu("File");
 		fileMenu.setMnemonic(KeyEvent.VK_F);
 
+		//Add a menu item.
+		JMenuItem profileItem = new JMenuItem(
+				PerfExplorerActionListener.LOAD_PROFILE);
+		profileItem.addActionListener(listener);
+		fileMenu.add(profileItem);
+	
+		//Add a menu item.
+		JMenuItem deriveMetricItem = new JMenuItem(
+				PerfExplorerActionListener.DERIVE_METRIC);
+		deriveMetricItem.addActionListener(listener);
+		fileMenu.add(deriveMetricItem);
+		
+		//Add a menu item.
+		JMenuItem parseItem = new JMenuItem(
+				PerfExplorerActionListener.PARSE_EXPRESSION);
+		parseItem.addActionListener(listener);
+		fileMenu.add(parseItem);
+		
+		//Add a menu item.
+		JMenuItem reparseItem = new JMenuItem(
+				PerfExplorerActionListener.REPARSE_EXPRESSION);
+		reparseItem.addActionListener(listener);
+		fileMenu.add(reparseItem);
+
 		//Add a menu item.
 		JMenuItem scriptItem = new JMenuItem(
 				PerfExplorerActionListener.LOADSCRIPT);
@@ -32,21 +56,31 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 				PerfExplorerActionListener.RERUNSCRIPT);
 		rerunItem.addActionListener(listener);
 		fileMenu.add(rerunItem);
+		
+		fileMenu.add(new JSeparator());
+		
+		//Add a menu item.
+		JMenuItem databaseItem = new JMenuItem(
+				PerfExplorerActionListener.DATABASE_CONFIGURATION);
+		databaseItem.addActionListener(listener);
+		fileMenu.add(databaseItem);
 
 		//Add a menu item.
-		JMenuItem saveItem = new JMenuItem(
-				PerfExplorerActionListener.SAVE);
-		saveItem.addActionListener(listener);
-		fileMenu.add(saveItem);
+		JMenuItem consoleItem = new JMenuItem(
+				PerfExplorerActionListener.CONSOLE);
+		consoleItem.addActionListener(listener);
+		fileMenu.add(consoleItem);
 
 		//Add a menu item.
 		JMenuItem quitItem = new JMenuItem(
 				PerfExplorerActionListener.QUIT);
 		quitItem.setAccelerator(KeyStroke.getKeyStroke(
-			KeyEvent.VK_Q, ActionEvent.SHIFT_MASK & ActionEvent.ALT_MASK));
+			//KeyEvent.VK_Q, ActionEvent.SHIFT_MASK & ActionEvent.ALT_MASK));
+			KeyEvent.VK_Q, ActionEvent.ALT_MASK));
 		quitItem.addActionListener(listener);
 		fileMenu.add(quitItem);
 
+/*
 		//Add a menu item.
 		JMenuItem quitServerItem = new JMenuItem(
 				PerfExplorerActionListener.QUIT_SERVER);
@@ -54,6 +88,7 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 			KeyEvent.VK_Q, ActionEvent.ALT_MASK));
 		quitServerItem.addActionListener(listener);
 		fileMenu.add(quitServerItem);
+*/
 
 /*
 		try {
@@ -129,6 +164,18 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		probabilityPlotItem.addActionListener(listener);
 		visualizationMenu.add(probabilityPlotItem);
 		
+		//Add a menu item.
+		JMenuItem commMatrixItem = new JMenuItem(
+				PerfExplorerActionListener.DO_COMMUNICATION_MATRIX);
+		commMatrixItem.addActionListener(listener);
+		visualizationMenu.add(commMatrixItem);
+		
+		//Add a menu item.
+		JMenuItem wekaItem = new JMenuItem(
+				PerfExplorerActionListener.DO_LAUNCH_WEKA);
+		wekaItem.addActionListener(listener);
+		visualizationMenu.add(wekaItem);
+		
 		this.add(visualizationMenu);
 	}
 
@@ -141,7 +188,7 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		JMenuItem clusteringItem = new JMenuItem(
 				PerfExplorerActionListener.CLUSTERING_METHOD);
 		clusteringItem.addActionListener(listener);
-		clusteringItem.setEnabled(false);
+//		clusteringItem.setEnabled(false);
 		analysisMenu.add(clusteringItem);
 
 		//Add a menu item.
@@ -170,6 +217,12 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 				PerfExplorerActionListener.DO_CLUSTERING);
 		doItem.addActionListener(listener);
 		analysisMenu.add(doItem);
+		
+		//Add a menu item.
+		JMenuItem doIncItem = new JMenuItem(
+				PerfExplorerActionListener.DO_INC_CLUSTERING);
+		doIncItem.addActionListener(listener);
+		analysisMenu.add(doIncItem);
 
 		//Add a menu item.
 		JMenuItem correlateItem = new JMenuItem(
@@ -177,6 +230,12 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		correlateItem.addActionListener(listener);
 		analysisMenu.add(correlateItem);
 
+		//Add a menu item.
+		JMenuItem correlateIncItem = new JMenuItem(
+				PerfExplorerActionListener.DO_INC_CORRELATION_ANALYSIS);
+		correlateIncItem.addActionListener(listener);
+		analysisMenu.add(correlateIncItem);
+
 		this.add(analysisMenu);
 
 
@@ -187,6 +246,12 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		JMenu chartMenu = new JMenu("Charts");
 		chartMenu.setMnemonic(KeyEvent.VK_C);
 
+		//Add a menu item.
+		JMenuItem allInOneItem = new JMenuItem(
+			PerfExplorerActionListener.DO_CHARTS);
+		allInOneItem.addActionListener(listener);
+		chartMenu.add(allInOneItem);
+
 		//Add a menu item.
 		JMenuItem groupNameItem = new JMenuItem(
 			PerfExplorerActionListener.SET_GROUPNAME);
@@ -217,6 +282,18 @@ public class PerfExplorerMainJMenuBar extends JMenuBar {
 		problemItem.addActionListener(listener);
 		chartMenu.add(problemItem);
 
+		//Add a menu item.
+		JMenuItem stackedBar = new JMenuItem(
+			PerfExplorerActionListener.STACKED_BAR_CHART);
+		stackedBar.addActionListener(listener);
+		chartMenu.add(stackedBar);
+		
+		//Add a menu item.
+		JMenuItem aStackedBar = new JMenuItem(
+			PerfExplorerActionListener.ALIGNED_STACKED_BAR_CHART);
+		aStackedBar.addActionListener(listener);
+		chartMenu.add(aStackedBar);
+
 		//Add a menu item.
 		JMenuItem totalTime = new JMenuItem(
 			PerfExplorerActionListener.TOTAL_TIME_CHART);
