@@ -1,6 +1,5 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
-import common.RMIView;
 import javax.swing.*;
 import java.util.List;
 import java.util.ArrayList;
@@ -8,6 +7,8 @@ import java.util.Hashtable;
 import java.util.Iterator;
 import javax.swing.tree.DefaultTreeModel;
 
+import edu.uoregon.tau.perfexplorer.common.RMIView;
+
 public class PerfExplorerViews {
 
 	public static void createNewView (JFrame mainFrame, int parent) {
@@ -60,7 +61,7 @@ public class PerfExplorerViews {
 			return;
 
 		// get the value they want to search on
-		List values = server.getPossibleValues(table, column);
+		List<String> values = server.getPossibleValues(table, column);
 		if (values.size() < 2) {
 			JOptionPane.showMessageDialog(mainFrame, "The column " + 
 				column.toUpperCase() + " in table " + table.toUpperCase() + 
@@ -96,7 +97,7 @@ public class PerfExplorerViews {
 		if (value.equals(all)) {
 			// create views for each value
 			for (int i = 0 ; i < values.size()-1 ; i++) {
-				value = (String) values.get(i);
+				value = values.get(i);
 				String name = table + ":" + column + " = " + value;
 				if (value != null)
 					server.createNewView(name, parent, table, column, oper, value);
@@ -126,8 +127,8 @@ public class PerfExplorerViews {
 		List views = server.getViews(parent);
 		
 		Iterator e = views.iterator();
-		List options = new ArrayList();
-		Hashtable myViews = new Hashtable();
+		List<String> options = new ArrayList<String>();
+		Hashtable<String, String> myViews = new Hashtable<String, String>();
 		while (e.hasNext()) {
 			RMIView view = (RMIView) e.next();
 			options.add(view.getField("NAME"));
@@ -141,7 +142,7 @@ public class PerfExplorerViews {
 			options.toArray(), null );
 
 		if (viewName != null) {
-			String viewID = (String) myViews.get(viewName);
+			String viewID = myViews.get(viewName);
 			createNewView(mainFrame, Integer.parseInt(viewID));
 		}
 	}
