@@ -40,14 +40,19 @@
 
  */
 
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
+import java.awt.BorderLayout;
 import java.io.IOException;
 import java.io.StringReader;
 import java.util.HashMap;
 import java.util.Iterator;
-import java.util.Enumeration;
 import java.util.Map;
+
+import javax.swing.JFrame;
+import javax.swing.JScrollPane;
+import javax.swing.tree.DefaultMutableTreeNode;
+
 import org.xml.sax.Attributes;
 import org.xml.sax.ContentHandler;
 import org.xml.sax.ErrorHandler;
@@ -56,17 +61,11 @@ import org.xml.sax.Locator;
 import org.xml.sax.SAXException;
 import org.xml.sax.SAXParseException;
 import org.xml.sax.XMLReader;
-import org.xml.sax.helpers.XMLReaderFactory;
 import org.xml.sax.ext.LexicalHandler;
+import org.xml.sax.helpers.XMLReaderFactory;
 
 import edu.uoregon.tau.common.treetable.AbstractTreeTableModel;
 import edu.uoregon.tau.common.treetable.JTreeTable;
-import edu.uoregon.tau.common.treetable.TreeTableModel;
-
-// This is an XML book - no need for explicit Swing imports
-import java.awt.*;
-import javax.swing.*;
-import javax.swing.tree.*;
 
 /**
  * <b><code>SAXTreeViewer</code></b> uses Swing to graphically
@@ -74,7 +73,12 @@ import javax.swing.tree.*;
  */
 public class SAXTreeViewer extends JFrame {
 
-    /** Default parser to use */
+    /**
+	 * 
+	 */
+	private static final long serialVersionUID = -176395314854266217L;
+
+	/** Default parser to use */
     private String vendorParserClass = 
         "org.apache.xerces.parsers.SAXParser";
 
@@ -99,8 +103,7 @@ public class SAXTreeViewer extends JFrame {
      * @param filename <code>String</code> path to XML document.
      */
     public void init(String xml) throws IOException, SAXException {
-        DefaultMutableTreeNode base = 
-            new DefaultMutableTreeNode("XML String");
+        DefaultMutableTreeNode base = new DefaultMutableTreeNode("XML String");
         
         // Construct the tree hierarchy
         buildTree(xml);
@@ -135,8 +138,53 @@ public class SAXTreeViewer extends JFrame {
         throws IOException, SAXException {
 
         // Create instances needed for parsing
-        XMLReader reader = 
-            XMLReaderFactory.createXMLReader(vendorParserClass);
+    	
+    	XMLReader reader; //= XMLReaderFactory.createXMLReader(vendorParserClass);
+    	
+    	 try { // Xerces
+    		 reader = XMLReaderFactory.createXMLReader(
+    				 vendorParserClass
+    		    );
+    		  }
+    		  catch (SAXException e1) {
+    		    try { // Crimson
+    		    	 reader = XMLReaderFactory.createXMLReader(
+    		       "org.apache.crimson.parser.XMLReaderImpl"
+    		      );
+    		    }
+    		    catch (SAXException e2) { 
+    		      try { // Ælfred
+    		    	  reader= XMLReaderFactory.createXMLReader(
+    		         "gnu.xml.aelfred2.XmlReader"
+    		        );
+    		      }
+    		      catch (SAXException e3) {
+    		        try { // Piccolo
+    		          reader = XMLReaderFactory.createXMLReader(
+    		            "com.bluecast.xml.Piccolo"
+    		          );
+    		        }
+    		        catch (SAXException e4) {
+    		          try { // Oracle
+    		        	  reader = XMLReaderFactory.createXMLReader(
+    		              "oracle.xml.parser.v2.SAXParser"
+    		            );
+    		          }
+    		          catch (SAXException e5) {
+    		            try { // default
+    		            	 reader = XMLReaderFactory.createXMLReader();
+    		            }
+    		            catch (SAXException e6) {
+    		              throw new NoClassDefFoundError(
+    		                "No SAX parser is available");
+    		            }
+    		          }
+    		        }
+    		      }
+    		    } 
+    		  }
+    	
+        
         JTreeContentHandler jTreeTableContentHandler = new JTreeContentHandler();
         ErrorHandler jTreeTableErrorHandler = new JTreeErrorHandler();
         JTreeLexicalHandler lexicalHandler = new JTreeLexicalHandler(jTreeTableContentHandler);
@@ -195,7 +243,7 @@ class JTreeContentHandler implements ContentHandler {
     private Locator locator;
 
     /** Store URI to prefix mappings */
-    private Map namespaceMappings;
+    private Map<String, String> namespaceMappings;
 
     /** Current node to add sub-nodes to */
     private XMLNode current = null;
@@ -211,7 +259,7 @@ class JTreeContentHandler implements ContentHandler {
      * @param base node to start adding sub-nodes to.
      */
     public JTreeContentHandler() {
-        this.namespaceMappings = new HashMap();
+        this.namespaceMappings = new HashMap<String, String>();
         this.current = new XMLNode("XML Document");
         this.root = this.current;
     }
@@ -313,11 +361,11 @@ class JTreeContentHandler implements ContentHandler {
      */
     public void endPrefixMapping(String prefix) {
         // No visual events occur here.
-        for (Iterator i = namespaceMappings.keySet().iterator(); 
+        for (Iterator<String> i = namespaceMappings.keySet().iterator(); 
              i.hasNext(); ) {
 
-            String uri = (String)i.next();
-            String thisPrefix = (String)namespaceMappings.get(uri);
+            String uri = i.next();
+            String thisPrefix = namespaceMappings.get(uri);
             if (prefix.equals(thisPrefix)) {
                 namespaceMappings.remove(uri);
                 break;
@@ -350,7 +398,7 @@ class JTreeContentHandler implements ContentHandler {
     	String prefix = "";
         // Determine namespace
         if (namespaceURI.length() > 0) {
-            prefix = (String)namespaceMappings.get(namespaceURI);
+            prefix = namespaceMappings.get(namespaceURI);
         }
 
         XMLElementNode element = null;
