@@ -1,28 +1,26 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
-import common.*;
 
 import jargs.gnu.CmdLineParser;
-import edu.uoregon.tau.perfdmf.*;
 
-import javax.swing.*;
-import java.io.FileInputStream;
-import java.awt.*;
-import java.awt.event.*;
 import java.util.*;
 
+import javax.swing.JFrame;
+import javax.swing.ToolTipManager;
+
+import edu.uoregon.tau.perfdmf.*;
+import edu.uoregon.tau.perfexplorer.common.*;
+
 public class TestHarness {
-	private static String USAGE = "Usage: TestHarness [{-h,--help}] {-c,--configfile}=<config_file> [{-s,--standalone}] [{-e,--engine}=<analysis_engine>] [{-t,--test}=<test_type>]\n  where analysis_engine = R or Weka and test_type = charts, cluster, correlation, viz, script or all ";
+	private static String USAGE = "Usage: TestHarness [{-h,--help}] {-c,--configfile}=<config_file> [{-s,--standalone}] [{-t,--test}=<test_type>]\n  where test_type = charts, cluster, correlation, viz, script or all ";
 	private PerfExplorerConnection connection = null;
 	private RMIPerfExplorerModel model = null;
 	private Object[] viewList = null;
 	private boolean foundView = false;
 
-	private TestHarness (boolean standalone, String configFile,
-		EngineType analysisEngine, boolean quiet, TestType test) {
+	private TestHarness (boolean standalone, String configFile, boolean quiet, TestType test) {
 
-		JFrame frame = new PerfExplorerClient(standalone, configFile, 
-			analysisEngine, quiet);
+		JFrame frame = new PerfExplorerClient(standalone, configFile, quiet);
 		frame.pack();
 		frame.setVisible(true);
 
@@ -92,7 +90,7 @@ public class TestHarness {
 				System.out.println("Testing views...");
 				viewList = null;
 				foundView = false;
-				getViews("0", new ArrayList());
+				getViews("0", new ArrayList<RMIView>());
 				System.out.print("Selecting: ");
 				for(int i = 0 ; i < viewList.length ; i++) 
 					System.out.print(viewList[i] + ": ");
@@ -173,9 +171,9 @@ public class TestHarness {
 	private Application setApplication(String name) {
 		Application app = null;
 		System.out.println("******** Applications *********");
-		for (ListIterator apps = connection.getApplicationList(); 
+		for (ListIterator<Application> apps = connection.getApplicationList(); 
 			apps.hasNext() ; ) {
-			app = (Application)apps.next();
+			app = apps.next();
 			System.out.println("\t" + app.getName());
 			if (app.getName().equalsIgnoreCase(name)) {
 				break;
@@ -187,9 +185,9 @@ public class TestHarness {
 	private Experiment setExperiment(Application app, String name) {
 		Experiment exp = null;
 		System.out.println("******** Experiments *********");
-		for (ListIterator exps = connection.getExperimentList(app.getID()); 
+		for (ListIterator<Experiment> exps = connection.getExperimentList(app.getID()); 
 			exps.hasNext() ; ) {
-			exp = (Experiment)exps.next();
+			exp = exps.next();
 			System.out.println("\t" + exp.getName());
 			if (exp.getName().equalsIgnoreCase(name)) {
 				break;
@@ -201,9 +199,9 @@ public class TestHarness {
 	private Trial setTrial(Experiment exp, String name) {
 		Trial trial = null;
 		System.out.println("******** Trials *********");
-		for (ListIterator trials = connection.getTrialList(exp.getID()); 
+		for (ListIterator<Trial> trials = connection.getTrialList(exp.getID(),false); 
 			trials.hasNext() ; ) {
-			trial = (Trial)trials.next();
+			trial = trials.next();
 			System.out.println("\t" + trial.getName());
 			if (trial.getName().trim().equalsIgnoreCase(name)) {
 				break;
@@ -215,9 +213,10 @@ public class TestHarness {
 	private Metric setMetric(Trial trial, String name) {
 		Metric metric = null;
 		System.out.println("******** Metrics *********");
-		Vector metrics = trial.getMetrics();
+		@SuppressWarnings("unchecked")
+		List<Metric> metrics = trial.getMetrics();
 		for (int i = 0 ; i < metrics.size() ; i++) {
-			metric = (Metric)metrics.elementAt(i);
+			metric = metrics.get(i);
 			System.out.println("\t" + metric.getName());
 			if (metric.getName().equalsIgnoreCase(name)) {
 				break;
@@ -311,17 +310,17 @@ public class TestHarness {
 //public int createNewView(String name, int parent, String tableName, String columnName, String oper, String value) throws RemoteException;
 	}
 
-	public void getViews(String parent, ArrayList list) throws Exception {
-        java.util.List viewVector = connection.getViews(Integer.parseInt(parent));
-        Iterator views = viewVector.iterator();
+	public void getViews(String parent, ArrayList<RMIView> list) throws Exception {
+        java.util.List<RMIView> viewVector = connection.getViews(Integer.parseInt(parent));
+        Iterator<RMIView> views = viewVector.iterator();
         while (views.hasNext()) {
-            RMIView view = (RMIView) views.next();
+            RMIView view = views.next();
 			System.out.println("VIEW: " + view.getField("NAME"));
 			//if (view.getField("VALUE").equals("gyro-b1"))
 			//if (view.getField("VALUE").equals("B1-std.tg"))
 			if (view.getField("VALUE").equals("gyro.B1-std.HPM"))
 				foundView = true;
-			ArrayList newList = new ArrayList(list);
+			ArrayList<RMIView> newList = new ArrayList<RMIView>(list);
 			newList.add(view);
             getViews(view.getField("ID"), newList);
         }
@@ -329,11 +328,11 @@ public class TestHarness {
 			if (viewList == null && foundView) {
 				viewList = list.toArray();
 			}
-            ListIterator trials = connection.getTrialsForView(list);
+            ListIterator<Trial> trials = connection.getTrialsForView(list,false);
             Trial trial = null;
             while(trials.hasNext())
             {
-                trial = (Trial) trials.next();
+                trial = trials.next();
 				System.out.println("TRIAL: " + trial.getName());
             }
         }
@@ -351,8 +350,9 @@ public class TestHarness {
 	}
 
 	public void testRules() throws Exception {
-		//RuleHarness rules = new RuleHarness();
-		//RuleHarness.processRules();
+		//RuleHarnessOld rules = new RuleHarnessOld();
+		System.out.println("USE A SCRIPT!");
+		//RuleHarnessOld.processRules();
 	}
 
 	public void testCorrelation() throws Exception {
@@ -393,7 +393,10 @@ public class TestHarness {
 		//edu.uoregon.tau.common.TauScripter.execfile("/Users/khuck/tau2/tools/src/perfexplorer/etc/generalcharttest.py");
 		//edu.uoregon.tau.common.TauScripter.execfile("etc/generalcharttest.py");
 		//edu.uoregon.tau.common.TauScripter.execfile("etc/lu-interactive.py");
-		edu.uoregon.tau.common.TauScripter.execfile("etc/gtc.py");
+		//edu.uoregon.tau.common.TauScripter.execfile("etc/gtc.py");
+		//edu.uoregon.tau.common.TauScripter.execfile("etc/rules.py");
+		//edu.uoregon.tau.common.TauScripter.execfile("etc/correlation.py");
+		edu.uoregon.tau.common.TauScripter.execfile("etc/longrun.py");
 	}
 		
 
@@ -419,7 +422,6 @@ public RMICubeData getCubeData(RMIPerfExplorerModel model) throws RemoteExceptio
 		CmdLineParser.Option helpOpt = parser.addBooleanOption('h',"help");
 		CmdLineParser.Option standaloneOpt = parser.addBooleanOption('s',"standalone");
 		CmdLineParser.Option configfileOpt = parser.addStringOption('c',"configfile");
-		CmdLineParser.Option engineOpt = parser.addStringOption('e',"engine");
 		CmdLineParser.Option quietOpt = parser.addBooleanOption('q',"quiet");
 		CmdLineParser.Option testOpt = parser.addStringOption('t',"test");
 
@@ -434,12 +436,9 @@ public RMICubeData getCubeData(RMIPerfExplorerModel model) throws RemoteExceptio
 		Boolean help = (Boolean) parser.getOptionValue(helpOpt);
 		Boolean standalone = (Boolean) parser.getOptionValue(standaloneOpt);
 		String configFile = (String) parser.getOptionValue(configfileOpt);
-		String engine = (String) parser.getOptionValue(engineOpt);
 		Boolean quiet = (Boolean) parser.getOptionValue(quietOpt);
 		String test = (String) parser.getOptionValue(testOpt);
 
-		EngineType analysisEngine = EngineType.WEKA;
-
 		if (help != null && help.booleanValue()) {
 			System.err.println(USAGE);
 			System.exit(-1);
@@ -462,13 +461,6 @@ public RMICubeData getCubeData(RMIPerfExplorerModel model) throws RemoteExceptio
 				System.err.println(USAGE);
 				System.exit(-1);
 			}
-			try {
-				analysisEngine = EngineType.getType(engine);
-			} catch (Exception e) {
-				System.err.println("Please enter a valid engine type.");
-				System.err.println(USAGE);
-				System.exit(-1);
-			}
 		}
 
 		TestType testType = null;
@@ -489,7 +481,7 @@ public RMICubeData getCubeData(RMIPerfExplorerModel model) throws RemoteExceptio
 	*/
 
 		TestHarness harness = new TestHarness(standalone.booleanValue(),
-			configFile, analysisEngine, quiet.booleanValue(), testType);
+			configFile, quiet.booleanValue(), testType);
 	}
 
 }
