@@ -1,4 +1,4 @@
-package client;
+package edu.uoregon.tau.perfexplorer.client;
 
 import java.awt.Color;
 import java.awt.Component;
@@ -12,18 +12,25 @@ import org.jfree.chart.ChartColor;
  *    
  * TODO : ...
  *
- * <P>CVS $Id: TreePortionCellRenderer.java,v 1.1 2007/02/01 04:31:42 khuck Exp $</P>
+ * <P>CVS $Id: TreePortionCellRenderer.java,v 1.4 2009/02/26 00:41:16 wspear Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.1 $
+ * @version $Revision: 1.4 $
  */
 public class TreePortionCellRenderer extends DefaultTreeCellRenderer {
 
-    public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf,
+    /**
+	 * 
+	 */
+	private static final long serialVersionUID = -3707397471318701237L;
+
+	public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf,
             int row, boolean hasFocus) {
         super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
 
         // set the text color?
-        if (value instanceof XMLElementNode) {
+        if (value instanceof XMLTAUAttributeElementNode) {
+            this.setForeground(ChartColor.VERY_DARK_GREEN);
+        } else if (value instanceof XMLElementNode) {
             this.setForeground(ChartColor.DARK_BLUE);
         } else if (value instanceof XMLCommentNode) {
             this.setForeground(ChartColor.VERY_DARK_RED);
