@@ -2,7 +2,7 @@
  * Created on Mar 16, 2005
  *
  */
-package clustering;
+package edu.uoregon.tau.perfexplorer.clustering;
 
 import java.util.LinkedHashMap;
 import java.util.Iterator;
@@ -11,7 +11,7 @@ import java.util.Iterator;
  * This class is used as a list of names and values to describe 
  * a cluster created during some type of clustering operation.
  * 
- * <P>CVS $Id: ClusterDescription.java,v 1.2 2007/01/04 21:20:01 khuck Exp $</P>
+ * <P>CVS $Id: ClusterDescription.java,v 1.3 2009/02/24 00:53:34 khuck Exp $</P>
  * @author khuck
  *
  */
