@@ -2,7 +2,7 @@
  * Created on Mar 17, 2005
  *
  */
-package clustering.weka;
+package edu.uoregon.tau.perfexplorer.clustering.weka;
 
 
 import weka.core.Instances;
@@ -14,14 +14,15 @@ import java.util.List;
 import java.util.ArrayList;
 import java.util.Enumeration;
 
-import clustering.RawDataInterface;
 
 import java.io.Serializable;
 
+import edu.uoregon.tau.perfexplorer.clustering.RawDataInterface;
+
 /**
  * Implementation of the RawData interface for Weka data.
  *
- * <P>CVS $Id: WekaRawData.java,v 1.9 2007/01/23 22:57:02 khuck Exp $</P>
+ * <P>CVS $Id: WekaRawData.java,v 1.13 2009/02/27 00:45:08 khuck Exp $</P>
  * @author khuck
  * @version 0.1
  * @since   0.1
@@ -42,24 +43,46 @@ public class WekaRawData implements RawDataInterface, Serializable {
      * @param vectors
      * @param dimensions
      */
-	WekaRawData (String name, List attributes, int vectors, int dimensions) {
+	WekaRawData (String name, List<String> attributes, int vectors, int dimensions, List<String> classAttributes) {
 		this.vectors = vectors;
 		this.dimensions = dimensions;
 		FastVector fastAttributes = new FastVector(attributes.size());
 		for (int i = 0 ; i < attributes.size() ; i++) {
-			String attr = (String) attributes.get(i);
+			String attr = attributes.get(i);
 			fastAttributes.addElement(new Attribute(attr));
 		}
-		instances = new Instances(name, fastAttributes, vectors);
-		
-		for (int i = 0 ; i < vectors ; i++) {
-			double[] values = new double[dimensions];
-			for (int j = 0 ; j < dimensions ; j++) {
-				values[j] = 0.0;
+		Attribute tmp = null;
+		if (classAttributes != null) {
+			String attr = "class";
+			FastVector vect = new FastVector(classAttributes.size());
+			for (String tmpClass : classAttributes) {
+				vect.addElement(tmpClass);
 			}
-			instances.add(new Instance(1.0, values));
+			tmp = new Attribute(attr, vect);
+			fastAttributes.addElement(tmp);
 		}
 		
+		instances = new Instances(name, fastAttributes, vectors);
+		
+		if (classAttributes != null) {
+			instances.setClass(tmp);
+			for (int i = 0 ; i < vectors ; i++) {
+				Instance tmpInst = new Instance(fastAttributes.size());
+				tmpInst.setDataset(instances);
+				for (int j = 0 ; j < dimensions ; j++) {
+					tmpInst.setValue(j, 0.0);
+				}
+				instances.add(tmpInst);
+			}
+		} else {
+			for (int i = 0 ; i < vectors ; i++) {
+				double[] values = new double[dimensions];
+				for (int j = 0 ; j < dimensions ; j++) {
+					values[j] = 0.0;
+				}
+				instances.add(new Instance(1.0, values));
+			}
+		}
 	}
 	
     /**
@@ -141,9 +164,9 @@ public class WekaRawData implements RawDataInterface, Serializable {
 	/* (non-Javadoc)
 	 * @see clustering.RawDataInterface#getEventNames()
 	 */
-	public List getEventNames() {
+	public List<String> getEventNames() {
 		Enumeration e = instances.enumerateAttributes();
-		List names = new ArrayList (instances.numDistinctValues(0));
+		List<String> names = new ArrayList<String> (instances.numDistinctValues(0));
 		while (e.hasMoreElements()) {
 			Attribute tmp = (Attribute) e.nextElement();
 			names.add(tmp.name());
@@ -262,4 +285,9 @@ public class WekaRawData implements RawDataInterface, Serializable {
 		String name = new String("");
 		return name;
 	}
+
+	public void addValue(int vectorIndex, int dimensionIndex, String value) {
+		Instance i = instances.instance(vectorIndex);
+		i.setValue(dimensionIndex, value);
+	}
 }
