@@ -1,15 +1,20 @@
-package common;
+package edu.uoregon.tau.perfexplorer.common;
 
 import java.rmi.Remote;
 import java.rmi.RemoteException;
 import java.util.List;
+import java.util.Map;
+
+import edu.uoregon.tau.perfdmf.Application;
+import edu.uoregon.tau.perfdmf.Experiment;
+import edu.uoregon.tau.perfdmf.Trial;
 
 /**
  * This is the main RMI object which is used to send requests to the 
  * PerfExplorerServer object.  This interface defines the API for
  * passing requests to the server.
  * 
- * <P>CVS $Id: RMIPerfExplorer.java,v 1.10 2007/06/27 23:48:48 khuck Exp $</P>
+ * <P>CVS $Id: RMIPerfExplorer.java,v 1.23 2009/11/19 15:53:33 khuck Exp $</P>
  * @author khuck
  * @version 0.1
  * @since   0.1
@@ -30,7 +35,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getApplicationList() throws RemoteException;
+    public List<Application> getApplicationList() throws RemoteException;
 
     /**
      * Returns the full list of experiments in the database for the
@@ -40,7 +45,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getExperimentList(int applicationID) throws RemoteException;
+    public List<Experiment> getExperimentList(int applicationID) throws RemoteException;
 
     /**
      * Returns the full list of trials in the database for the specified
@@ -50,7 +55,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getTrialList(int experimentID) throws RemoteException;
+    public List<Trial> getTrialList(int experimentID, boolean getXMLMetadata) throws RemoteException;
 
     /**
      * Requests analysis using the settings specified in the model.
@@ -110,7 +115,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getPotentialGroups(RMIPerfExplorerModel model)
+    public List<String> getPotentialGroups(RMIPerfExplorerModel model)
         throws RemoteException;
 
     /**
@@ -121,7 +126,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getPotentialMetrics(RMIPerfExplorerModel model)
+    public List<String> getPotentialMetrics(RMIPerfExplorerModel model)
         throws RemoteException;
 
     /**
@@ -130,7 +135,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getPotentialEvents(RMIPerfExplorerModel model)
+    public List<String> getPotentialEvents(RMIPerfExplorerModel model)
         throws RemoteException;
 
     /**
@@ -150,7 +155,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getPossibleValues(String tableName, String columnName)
+    public List<String> getPossibleValues(String tableName, String columnName)
         throws RemoteException;
 
     /**
@@ -175,7 +180,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getViews(int parent) throws RemoteException;
+    public List<RMIView> getViews(int parent) throws RemoteException;
 
     /**
      * Get the trials which are filtered by the specifed views.
@@ -184,7 +189,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getTrialsForView(List views) throws RemoteException;
+    public List<Trial> getTrialsForView(List<RMIView> views, boolean getXMLMetadata) throws RemoteException;
 
     /**
      * Gets the correlation results for the specified model.
@@ -212,7 +217,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public RMICubeData getCubeData(RMIPerfExplorerModel model)
+    public RMICubeData getCubeData(RMIPerfExplorerModel model, int numFunctions)
         throws RemoteException;
 
     /**
@@ -229,7 +234,15 @@ public interface RMIPerfExplorer extends Remote {
      * @return The connection URLs
      * @throws RemoteException
      */
-    public List getConnectionStrings() throws RemoteException;
+    public List<String> getConnectionStrings() throws RemoteException;
+
+    /**
+     * Gets the configuration names for the JDBC connections.
+     *
+     * @return The configuration names
+     * @throws RemoteException
+     */
+    public List<String> getConfigNames() throws RemoteException;
 
     /**
      * Gets the list of events for the specified trial, metric
@@ -239,7 +252,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return a list of event names
      * @throws RemoteException
      */
-	public List getEventList(int trialID, int metricIndex) throws RemoteException; 
+	public List<RMISortableIntervalEvent> getEventList(int trialID, int metricIndex) throws RemoteException; 
 
     /**
      * Returns the full list of trials in the database for the specified
@@ -249,7 +262,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return
      * @throws RemoteException
      */
-    public List getTrialList(String criteria) throws RemoteException;
+    public List<Trial> getTrialList(String criteria, boolean getXMLMetadata) throws RemoteException;
 
     /**
      * Deletes a view from the view hierarchy.
@@ -266,7 +279,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return a list of table.column names
      * @throws RemoteException
      */
-    public List getChartFieldNames() throws RemoteException;
+    public List<String> getChartFieldNames() throws RemoteException;
 
     /**
      * Requests data for generating comparison charts for the specified model.
@@ -275,7 +288,7 @@ public interface RMIPerfExplorer extends Remote {
      * @return list of XML fields
      * @throws RemoteException
      */
-    public List getXMLFields(RMIPerfExplorerModel model) throws RemoteException;
+    public List<String> getXMLFields(RMIPerfExplorerModel model) throws RemoteException;
 
     /**
      * Because PerfExplorer now supports multiple database connections, specify
@@ -287,5 +300,35 @@ public interface RMIPerfExplorer extends Remote {
      */
     public void setConnectionIndex(int connectionIndex) throws RemoteException;
 	
+    /**
+     * Because PerfExplorer now supports multiple database connections, allow
+     * for dropping all connections and restarting
+     *
+     * @return 
+     * @throws RemoteException
+     */
+    public void resetServer() throws RemoteException;
+
+    /**
+    *
+    * @param model
+    * @return
+    * @throws RemoteException
+    */
+   public List<String> getPotentialAtomicEvents(RMIPerfExplorerModel model)
+       throws RemoteException;
+
+   /**
+   * Get the counts for each thread count value for all the selected trials
+   * 
+   * @param model
+   * @return
+   * @throws RemoteException
+   */
+   public Map<String, Integer> checkScalabilityChartData(RMIPerfExplorerModel model)
+   	throws RemoteException;
+
+   public Map<String, double[][]> getUserEventData(RMIPerfExplorerModel model)
+   	throws RemoteException;
 }
 
