@@ -1,4 +1,4 @@
-package common;
+package edu.uoregon.tau.perfexplorer.common;
 
 import edu.uoregon.tau.perfdmf.Application;
 import edu.uoregon.tau.perfdmf.Experiment;
@@ -6,6 +6,8 @@ import edu.uoregon.tau.perfdmf.Trial;
 import edu.uoregon.tau.perfdmf.Metric;
 import edu.uoregon.tau.perfdmf.IntervalEvent;
 import java.io.Serializable;
+import java.util.Map;
+import java.util.HashMap;
 import java.util.List;
 import java.util.ArrayList;
 
@@ -13,7 +15,7 @@ import java.util.ArrayList;
  * This RMI object defines the state of the client model when an analysis
  * request is made.
  *
- * <P>CVS $Id: RMIPerfExplorerModel.java,v 1.31 2007/06/29 19:39:50 khuck Exp $</P>
+ * <P>CVS $Id: RMIPerfExplorerModel.java,v 1.39 2009/05/12 00:28:38 wspear Exp $</P>
  * @author khuck
  * @version 0.1
  * @since   0.1
@@ -23,19 +25,20 @@ public class RMIPerfExplorerModel implements Serializable {
 	public final static int MAX_CLUSTERS = 10;
 	public final static double X_PERCENT = 1.0;
 
-	protected List multiSelections = null;
+	protected List<Object> multiSelections = null;
 	protected SelectionType multiSelectionType = SelectionType.NO_MULTI;
 
 	// constants for chart parameters
-	protected List groupNames = null;
-	protected List metricNames = null;
-	protected List eventNames = null;
+	protected List<String> groupNames = null;
+	protected List<String> metricNames = null;
+	protected List<String> eventNames = null;
 	protected String totalTimesteps = null;
 	protected Boolean constantProblem = null;
 	protected boolean eventNoCallpath = true;
 	protected boolean mainEventOnly = true;
 	protected boolean eventExclusive100 = false;
 	protected String chartSeriesName = null;
+	protected boolean chartSeriesXML = false;
 	protected String chartMetadataFieldName = null;
 	protected String chartMetadataFieldValue = null;
 	protected String chartXAxisName = null;
@@ -47,8 +50,10 @@ public class RMIPerfExplorerModel implements Serializable {
 	protected boolean chartScalability = false;
 	protected boolean chartEfficiency = false;
 	protected boolean chartHorizontal = false;
+	protected boolean showZero = true;
 	protected String chartUnits = null;
 
+
 	// more cluster settings
 	protected AnalysisType clusterMethod = null;
 	protected TransformationType dimensionReduction = null;
@@ -67,6 +72,20 @@ public class RMIPerfExplorerModel implements Serializable {
 	protected Object[] fullPath = null;
 	protected int connectionIndex = 0;
 
+	protected String clusterValueType = null;
+	
+   	private boolean averageWarning = false;
+	
+    public String getClusterValueType() {
+		return clusterValueType;
+	}
+
+	public void setClusterValueType(String clusterValueType) {
+		this.clusterValueType = clusterValueType;
+	}
+
+	private Map<String,String> scriptParameters = new HashMap<String,String>();
+
 	/**
 	 * Default Constructor.
 	 *
@@ -102,6 +121,7 @@ public class RMIPerfExplorerModel implements Serializable {
 		this.chartScalability = source.chartScalability;
 		this.chartEfficiency = source.chartEfficiency;
 		this.chartHorizontal = source.chartHorizontal;
+		this.showZero = source.showZero;
 		this.chartUnits = source.chartUnits;
 		this.clusterMethod = source.clusterMethod;
 		this.dimensionReduction = source.dimensionReduction;
@@ -118,6 +138,8 @@ public class RMIPerfExplorerModel implements Serializable {
 		this.analysisID = source.analysisID;
 		this.fullPath = source.fullPath;
 		this.connectionIndex = source.connectionIndex;
+		
+		this.clusterValueType = source.clusterValueType;
 	}
 
 	/**
@@ -493,7 +515,7 @@ public class RMIPerfExplorerModel implements Serializable {
      * @param objects
      * @return
      */
-	public boolean setMultiSelection(List objects) {
+	public boolean setMultiSelection(List<Object> objects) {
 		for (int i = 0 ; i < objects.size() ; i++) {
 			if (objects.get(i) instanceof Application) {
 				if (multiSelectionType != SelectionType.APPLICATION &&
@@ -544,7 +566,7 @@ public class RMIPerfExplorerModel implements Serializable {
 		totalTimesteps = null;
 		constantProblem = null;
 		if (multiSelections == null) {
-			multiSelections = new ArrayList();
+			multiSelections = new ArrayList<Object>();
 			multiSelections.add(currentSelection);
 		}
 		multiSelections.add(obj);
@@ -556,7 +578,7 @@ public class RMIPerfExplorerModel implements Serializable {
      *
      * @return
      */
-	public List getMultiSelection() {
+	public List<Object> getMultiSelection() {
 		return multiSelections;
 	}
 
@@ -567,7 +589,7 @@ public class RMIPerfExplorerModel implements Serializable {
 	public String getGroupName () {
 		String tmp = null;
 		if (groupNames != null) {
-			tmp = (String)groupNames.get(0);
+			tmp = groupNames.get(0);
 		}
 		return tmp;
 	}
@@ -577,7 +599,7 @@ public class RMIPerfExplorerModel implements Serializable {
      *
      * @return
      */
-	public List getGroupNames () {
+	public List<String> getGroupNames () {
 		return groupNames;
 	}
 
@@ -586,7 +608,7 @@ public class RMIPerfExplorerModel implements Serializable {
      *
      * @return
      */
-	public List getEventNames () {
+	public List<String> getEventNames () {
 		return eventNames;
 	}
 
@@ -599,7 +621,7 @@ public class RMIPerfExplorerModel implements Serializable {
 		if (groupName == null) {
 			this.groupNames = null;
 		} else {
-			this.groupNames = new ArrayList();
+			this.groupNames = new ArrayList<String>();
 			this.groupNames.add(groupName);
 		}
 	}
@@ -611,7 +633,7 @@ public class RMIPerfExplorerModel implements Serializable {
      */
 	public String getMetricName () {
 		if (metricNames != null)
-			return (String)metricNames.get(0);
+			return metricNames.get(0);
 		if (currentSelection instanceof Metric) {
 			Metric met = (Metric)currentSelection;
 			return met.getName();
@@ -630,7 +652,7 @@ public class RMIPerfExplorerModel implements Serializable {
      *
      * @return
      */
-	public List getMetricNames () {
+	public List<String> getMetricNames () {
 		return metricNames;
 	}
 
@@ -657,7 +679,7 @@ public class RMIPerfExplorerModel implements Serializable {
 		if (metricName == null) {
 			this.metricNames = null;
 		} else {
-			this.metricNames = new ArrayList();
+			this.metricNames = new ArrayList<String>();
 			this.metricNames.add(metricName);
 		}
 	}
@@ -669,7 +691,7 @@ public class RMIPerfExplorerModel implements Serializable {
      */
 	public void addMetricName (String metricName) {
 		if (this.metricNames == null) {
-			this.metricNames = new ArrayList();
+			this.metricNames = new ArrayList<String>();
 		}
 		this.metricNames.add(metricName);
 	}
@@ -681,7 +703,7 @@ public class RMIPerfExplorerModel implements Serializable {
      */
 	public void addGroupName (String groupName) {
 		if (this.groupNames == null) {
-			this.groupNames = new ArrayList();
+			this.groupNames = new ArrayList<String>();
 		}
 		this.groupNames.add(groupName);
 	}
@@ -693,7 +715,7 @@ public class RMIPerfExplorerModel implements Serializable {
      */
 	public void addEventName (String eventName) {
 		if (this.eventNames == null) {
-			this.eventNames = new ArrayList();
+			this.eventNames = new ArrayList<String>();
 		}
 		this.eventNames.add(eventName);
 	}
@@ -726,7 +748,7 @@ public class RMIPerfExplorerModel implements Serializable {
 	public String getEventName () {
 		String tmp = null;
 		if (eventNames != null) {
-			tmp = (String)eventNames.get(0);
+			tmp = eventNames.get(0);
 		}
 		return tmp;
 	}
@@ -739,7 +761,7 @@ public class RMIPerfExplorerModel implements Serializable {
 		if (eventName == null) {
 			this.eventNames = null;
 		} else {
-			this.eventNames = new ArrayList();
+			this.eventNames = new ArrayList<String>();
 			this.eventNames.add(eventName);
 		}
 	}
@@ -754,7 +776,7 @@ public class RMIPerfExplorerModel implements Serializable {
      * @return
      */
 	public String getViewSelectionPath (boolean joinApp, boolean joinExp, String dbType) {
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		if (joinExp)
 			buf.append(" inner join experiment e on t.experiment = e.id ");
 		if (joinApp)
@@ -797,7 +819,7 @@ public class RMIPerfExplorerModel implements Serializable {
      * @return
      */
 	public String getViewSelectionString (String dbType) {
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		int i = fullPath.length - 1;
 		RMIView view = (RMIView) fullPath[i];
 		if (//(view.getField("operator").equalsIgnoreCase("like")) || //{
@@ -1110,6 +1132,15 @@ public class RMIPerfExplorerModel implements Serializable {
 		this.chartHorizontal = horizontal;
 	}
 
+    /**
+     * Set the show zero on y axis chart option
+     *
+     * @param seriesName
+     */
+	public void setShowZero(boolean showZero) {
+		this.showZero = showZero;
+	}
+
     /**
      * Get the horizontal chart option
      *
@@ -1119,6 +1150,15 @@ public class RMIPerfExplorerModel implements Serializable {
 		return this.chartHorizontal;
 	}
 
+    /**
+     * Get the show zero on y axis chart option
+     *
+     * @return
+     */
+	public boolean getShowZero() {
+		return this.showZero;
+	}
+
 	/**
 	 * @return the connectionIndex
 	 */
@@ -1133,4 +1173,37 @@ public class RMIPerfExplorerModel implements Serializable {
 		this.connectionIndex = connectionIndex;
 	}
 
+	public Map<String,String> getScriptParameters() {
+		return scriptParameters;
+	}
+
+	public void setScriptParameters(Map<String,String> scriptParameters) {
+		this.scriptParameters = scriptParameters;
+	}
+
+	public void addScriptParameter(String name, String value) {
+		this.scriptParameters.put(name, value);
+	}
+
+	/**
+	 * @return the averageWarning
+	 */
+	public boolean getAverageWarning() {
+		return averageWarning;
+	}
+
+	/**
+	 * @param averageWarning the averageWarning to set
+	 */
+	public void setAverageWarning(boolean averageWarning) {
+		this.averageWarning = averageWarning;
+	}
+	
+	public boolean isChartSeriesXML() {
+		return chartSeriesXML;
+	}
+
+	public void setChartSeriesXML(boolean chartSeriesXML) {
+		this.chartSeriesXML = chartSeriesXML;
+	}
 }
