@@ -2,7 +2,7 @@
  * Created on Feb 13, 2006
  *
  */
-package common;
+package edu.uoregon.tau.perfexplorer.common;
 
 import java.io.Serializable;
 
@@ -10,7 +10,7 @@ import java.io.Serializable;
 /**
  * This class is used as a typesafe enumeration.
  *
- * <P>CVS $Id: SelectionType.java,v 1.2 2007/07/03 00:18:33 khuck Exp $</P>
+ * <P>CVS $Id: SelectionType.java,v 1.5 2009/02/24 00:53:37 khuck Exp $</P>
  * @author  Kevin Huck
  * @version 0.2
  * @since   0.2
