@@ -1,18 +1,10 @@
 /**
- * 
+ *
  */
 package server;
 
 import clustering.*;
-#ifdef USE_WEKA_ENGINE
-import clustering.weka.WekaAnalysisFactory;
-#endif
-#ifdef USE_R_ENGINE
-import clustering.r.RAnalysisFactory;
-#endif
-#ifdef USE_OCTAVE_ENGINE
-import clustering.octave.RAnalysisFactory;
-#endif
+import clustering.weka.AnalysisFactory;
 import common.*;
 import edu.uoregon.tau.perfdmf.*;
 import edu.uoregon.tau.perfdmf.database.*;
@@ -53,30 +45,16 @@ import org.jfree.data.xy.XYDataset;
  * This class is intended to be a wrapper around data mining operations
  * available in Weka, R and Octave.  The orignal AnalysisTask class
  * only supported R directly.  This is intended to be an improvement...
- * 
- * <P>CVS $Id: AnalysisTaskWrapper.cpp,v 1.15 2005/11/08 03:09:00 khuck Exp $</P>
+ *
+ * <P>CVS $Id: AnalysisTaskWrapper.cpp,v 1.19 2009/05/08 23:36:38 wspear Exp $</P>
  * @author  Kevin Huck
  * @version 0.1
  * @since   0.1
  */
 public class AnalysisTaskWrapper extends TimerTask {
 
-	public static final int RPROJECT_ENGINE = 0;
-	public static final int WEKA_ENGINE = 1;
-	public static final int OCTAVE_ENGINE = 2;
-	
-	public static final int DENDROGRAM = 0;
-	public static final int HISTOGRAM = 1;
-	public static final int VIRTUAL_TOPOLOGY = 2;
-	public static final int CLUSTER_AVERAGES = 3;
-	public static final int CLUSTER_MAXIMUMS = 4;
-	public static final int CLUSTER_MINIMUMS = 5;
-	public static final int PCA_SCATTERPLOT = 6;
-	public static final int CORRELATION_SCATTERPLOT = 7;
-	
-	private int chartType = 0;
-	private AnalysisFactory factory = null;
-	
+	private ChartType chartType = ChartType.DENDROGRAM;
+
 	private RMIPerfExplorerModel modelData = null;
 	private PerfExplorerServer server = null;
 	private int analysisID = 0;
@@ -95,42 +73,19 @@ public class AnalysisTaskWrapper extends TimerTask {
 
 	/**
 	 * Constructor.  The engine parameter passed in specifies which analysis
-	 * engine to use. 
-	 * 
+	 * engine to use.
+	 *
 	 * @param server
 	 * @param engine
 	 */
-	public AnalysisTaskWrapper (PerfExplorerServer server, int engine) {
+	public AnalysisTaskWrapper (PerfExplorerServer server) {
 		super();
 		this.server = server;
-		switch (engine) {
-#ifdef USE_WEKA_ENGINE
-			case WEKA_ENGINE:
-				factory = WekaAnalysisFactory.getFactory();
-				break;
-#endif
-#ifdef USE_R_ENGINE
-			case RPROJECT_ENGINE:
-				//rInterpreter = server.getRInterpreter();
-				//rEvaluator = server.getREvaluator();
-				factory = RAnalysisFactory.getFactory();
-				break;
-#endif
-#ifdef USE_OCTAVE_ENGINE
-			case OCTAVE_ENGINE:
-				factory = OctaveAnalysisFactory.getFactory();
-				break;
-#endif
-			default:
-				System.out.println("Undefined analysis engine type.");
-				System.exit(1);
-				break;
-		}
 	}
 
 	/**
 	 * This method saves the analysis results in the database.
-	 * 
+	 *
 	 * @param description
 	 * @param fileName
 	 * @param thumbnail
@@ -157,7 +112,7 @@ public class AnalysisTaskWrapper extends TimerTask {
        		statement.setInt(4,(int)v_file.length());
        		statement.setBinaryStream(5,v_tis,(int)v_thumb.length());
        		statement.setBinaryStream(6,v_fis,(int)v_file.length());
-			statement.setInt(7, chartType);
+			statement.setString(7, chartType.toString());
        		statement.executeUpdate();
 			//db.commit();
        		v_fis.close();
@@ -176,7 +131,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 
 	/**
 	 * This method saves the analysis results in the database.
-	 * 
+	 *
 	 * @param centroids
 	 * @param deviations
 	 * @param thumbnail
@@ -192,7 +147,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			PreparedStatement statement = null;
 			// for each centroid, save the data
 			// TODO - MAKE THIS A GENERAL USE LATER!
-			StringBuffer buf = new StringBuffer();
+			StringBuilder buf = new StringBuilder();
 			buf.append("insert into analysis_result ");
 			buf.append(" (analysis_settings, description, thumbnail_size, thumbnail, image_size, image, result_type) values (?, ?, ?, ?, ?, ?, ?)");
 			statement = db.prepareStatement(buf.toString());
@@ -204,7 +159,7 @@ public class AnalysisTaskWrapper extends TimerTask {
        		FileInputStream inStream2 = new FileInputStream(outfile);
        		statement.setInt(5,(int)outfile.length());
        		statement.setBinaryStream(6,inStream2,(int)outfile.length());
-			statement.setInt(7, chartType);
+			statement.setString(7, chartType.toString());
 			//System.out.println(statement.toString());
        		statement.executeUpdate();
        		statement.close();
@@ -216,12 +171,12 @@ public class AnalysisTaskWrapper extends TimerTask {
 		   	tmpStr = "select IDENTITY_VAL_LOCAL() FROM analysis_result";
 			} else if (db.getDBType().compareTo("oracle") == 0) {
 		   	tmpStr = "SELECT ar_id_seq.currval FROM DUAL";
-			} else { // postgresql 
+			} else { // postgresql
 		   	tmpStr = "select currval('analysis_result_id_seq');";
 			}
 			int analysisResultID = Integer.parseInt(db.getDataItem(tmpStr));
-			
-			buf = new StringBuffer();
+
+			buf = new StringBuilder();
 			buf.append("insert into analysis_result_data ");
 			buf.append(" (interval_event, metric, value, data_type, analysis_result, cluster_index)");
 			buf.append(" values (?, ?, ?, ?, ?, ?)");
@@ -253,9 +208,9 @@ public class AnalysisTaskWrapper extends TimerTask {
 
 	/**
 	 * This method gets the constant values for the data.  In order to retrieve
-	 * the data correctly, it is necessary to get the number of events, metrics, 
+	 * the data correctly, it is necessary to get the number of events, metrics,
 	 * threads, and the total number of rows of data we are expecting.
-	 * 
+	 *
 	 * @throws PerfExplorerException
 	 */
 	private void getConstants () throws PerfExplorerException {
@@ -264,7 +219,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			DB db = PerfExplorerServer.getServer().getDB();
 			PreparedStatement statement = null;
 			// First, get the total number of rows we are expecting
-			StringBuffer sql = new StringBuffer();
+			StringBuilder sql = new StringBuilder();
 
             if (db.getDBType().compareTo("oracle") == 0) {
                 sql.append("select count(p.excl) ");
@@ -275,16 +230,16 @@ public class AnalysisTaskWrapper extends TimerTask {
 			sql.append("from interval_event e ");
 			sql.append("left outer join interval_location_profile p ");
 			sql.append("on e.id = p.interval_event ");
-			if (modelData.getDimensionReduction().equals(RMIPerfExplorerModel.OVER_X_PERCENT)) {
+			if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 				sql.append("inner join interval_mean_summary s ");
 				sql.append("on e.id = s.interval_event and s.metric = p.metric ");
 				sql.append("and s.exclusive_percentage > ");
-				sql.append("" + modelData.getXPercent() + "");	
+				sql.append("" + modelData.getXPercent() + "");
 			//} else if (modelData.getCurrentSelection() instanceof Metric) {
 				//sql.append("inner join interval_mean_summary s ");
 				//sql.append("on e.id = s.interval_event and s.metric = p.metric ");
 			}
-			sql.append("where e.trial = ?");
+			sql.append(" where e.trial = ?");
 			sql.append(" and e.group_name not like '%TAU_CALLPATH%' ");
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				sql.append(" and p.metric = ?");
@@ -307,9 +262,9 @@ public class AnalysisTaskWrapper extends TimerTask {
 
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				// Next, get the event names, and count them
-				sql = new StringBuffer();
+				sql = new StringBuilder();
 				sql.append("select e.id, e.name from interval_event e ");
-				if (modelData.getDimensionReduction().equals(RMIPerfExplorerModel.OVER_X_PERCENT)) {
+				if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 					sql.append("inner join interval_mean_summary s on ");
 					sql.append("e.id = s.interval_event ");
 					sql.append("and s.exclusive_percentage > ");
@@ -326,7 +281,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 				PerfExplorerServer.getServer().getControl().WAIT("getConstants");
 				statement = db.prepareStatement(sql.toString());
 				statement.setInt(1, modelData.getTrial().getID());
-				if (modelData.getDimensionReduction().equals(RMIPerfExplorerModel.OVER_X_PERCENT)) {
+				if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 					if (modelData.getCurrentSelection() instanceof Metric) {
 						statement.setInt(2, ((Metric)(modelData.getCurrentSelection())).getID());
 					}
@@ -346,7 +301,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			} else {
 
 				// Next, get the metric names, and count them
-				sql = new StringBuffer();
+				sql = new StringBuilder();
 				sql.append("select m.id, m.name from metric m ");
 				sql.append("where m.trial = ?");
 				sql.append(" order by 1");
@@ -368,7 +323,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			}
 
 			// get the number of threads
-			sql = new StringBuffer();
+			sql = new StringBuilder();
 			sql.append("select max(node), max(context), max(thread) ");
 			sql.append("from interval_location_profile ");
 			sql.append("inner join interval_event ");
@@ -415,12 +370,12 @@ public class AnalysisTaskWrapper extends TimerTask {
 
 	/**
 	 * This method gets the raw performance data from the database.
-	 * 
+	 *
 	 * @throws PerfExplorerException
 	 */
 	private void getRawData () throws PerfExplorerException {
 		System.out.print("Getting raw data...");
-		rawData = factory.createRawData("Cluster Test", eventIDs, numTotalThreads, numEvents);
+		rawData = AnalysisFactory.createRawData("Cluster Test", eventIDs, numTotalThreads, numEvents);
 		ResultSet results = null;
 		int currentFunction = 0;
 		int functionIndex = -1;
@@ -431,13 +386,13 @@ public class AnalysisTaskWrapper extends TimerTask {
 			PerfExplorerServer.getServer().getControl().WAIT("getRawData");
 			DB db = PerfExplorerServer.getServer().getDB();
 			PreparedStatement statement = null;
-			StringBuffer sql = new StringBuffer();
-			if (modelData.getDimensionReduction().equals(RMIPerfExplorerModel.OVER_X_PERCENT)) {
+			StringBuilder sql = new StringBuilder();
+			if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 				sql.append("select e.id, (p.node*");
 				sql.append(contexts * threads);
 				sql.append(") + (p.context*");
 				sql.append(threads);
-                
+
                 if (db.getDBType().compareTo("oracle") == 0) {
                     sql.append(") + p.thread as thread, p.metric as metric, p.excl/1000000, ");
                 } else {
@@ -448,14 +403,14 @@ public class AnalysisTaskWrapper extends TimerTask {
 				sql.append("inner join interval_mean_summary s ");
 				sql.append("on e.id = s.interval_event and (s.exclusive_percentage > ");
 				sql.append(modelData.getXPercent());
-				sql.append("or s.inclusive_percentage = 100.0) ");
+				sql.append(" or s.inclusive_percentage = 100.0) ");
 				sql.append(" left outer join interval_location_profile p ");
 				sql.append("on e.id = p.interval_event ");
 				sql.append("and p.metric = s.metric where e.trial = ? ");
 			} else {
 				sql.append("select e.id, (p.node*" + (contexts * threads) + "");
 				sql.append(") + (p.context*" + threads + "");
-                
+
                 if (db.getDBType().compareTo("oracle") == 0) {
                     sql.append(") + p.thread as thread, p.metric as metric, p.excl, ");
                 } else {
@@ -487,7 +442,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			// get the rows
 			while (results.next() != false) {
 				if (!(modelData.getDimensionReduction().equals(
-					RMIPerfExplorerModel.OVER_X_PERCENT)) || 
+					TransformationType.OVER_X_PERCENT)) ||
 					(results.getDouble(7) > modelData.getXPercent())) {
 					if (currentFunction != results.getInt(importantIndex)) {
 						functionIndex++;
@@ -497,7 +452,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 					rawData.addValue(threadIndex, functionIndex, results.getDouble(4));
 					if (maximum < results.getDouble(4))
 						maximum = results.getDouble(4);
-				} 
+				}
 				// if this is the main method, save its values
 				if (results.getDouble(6) == 100.0) {
 					rawData.addMainValue(threadIndex, functionIndex, results.getDouble(5));
@@ -528,7 +483,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 	/**
 	 * This method creates the dendrogram tree which represents the results
 	 * of the hierarchical clustering, if it was done.
-	 * 
+	 *
 	 * @param merge
 	 * @param height
 	 * @return
@@ -567,7 +522,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 		// for now, just return the original data.
 		return rawData;
 	}
-	
+
 	/* (non-Javadoc)
 	 * @see java.lang.Runnable#run()
 	 */
@@ -586,12 +541,12 @@ public class AnalysisTaskWrapper extends TimerTask {
 				// find good initial centers
 				//makeDendrogram(reducedData);
 				// do it!
-				if (modelData.getClusterMethod().equals(RMIPerfExplorerModel.K_MEANS)) {
+				if (modelData.getClusterMethod().equals(AnalysisType.K_MEANS)) {
 					int maxClusters = (numTotalThreads <= modelData.getNumberOfClusters()) ? (numTotalThreads-1) : modelData.getNumberOfClusters();
 					for (int i = 2 ; i <= maxClusters ; i++) {
 						System.out.println("Doing " + i + " clusters:" + modelData.toString());
 						// create a cluster engine
-						KMeansClusterInterface clusterer = factory.createKMeansEngine();
+						KMeansClusterInterface clusterer = AnalysisFactory.createKMeansEngine();
 						clusterer.setInputData(reducedData);
 						clusterer.setK(i);
 						clusterer.findClusters();
@@ -603,13 +558,13 @@ public class AnalysisTaskWrapper extends TimerTask {
 						File thumbnail = generateThumbnail(clusterSizes, eventIDs);
 						File chart = generateImage(clusterSizes, eventIDs);
 						// TODO - fix this to save the cluster sizes, too!
-						chartType = HISTOGRAM;
+						chartType = ChartType.HISTOGRAM;
 						saveAnalysisResult(centroids, deviations, thumbnail, chart);
-						
+
 						if (modelData.getCurrentSelection() instanceof Metric) {
 						// do PCA breakdown
 						PrincipalComponentsAnalysisInterface pca =
-						factory.createPCAEngine(server.getCubeData(modelData));
+						AnalysisFactory.createPCAEngine(server.getCubeData(modelData));
 						pca.setInputData(reducedData);
 						pca.doPCA();
 						// get the components
@@ -618,7 +573,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 						RawDataInterface[] clusters = pca.getClusters();
 						// do a scatterplot
 						rCorrelation = 0.0;
-						chartType = AnalysisTaskWrapper.PCA_SCATTERPLOT;
+						chartType = ChartType.PCA_SCATTERPLOT;
 						//for (int m = 0 ; m < i ; m++)
 							//clusters[m].normalizeData(true);
 						//System.out.println("PCA Dimensions: " + components.numDimensions());
@@ -626,25 +581,25 @@ public class AnalysisTaskWrapper extends TimerTask {
 						chart = generateImage(components, clusters);
 						saveAnalysisResult(components, components, thumbnail, chart);
 						}
-						
+
 						// do virtual topology
 						VirtualTopology vt = new VirtualTopology(modelData, clusterer);
 						String filename = vt.getImage();
 						String nail = vt.getThumbnail();
 						saveAnalysisResult("Virtual Topology", filename, nail, false);
-						
+
 						// do mins
-						chartType = CLUSTER_MINIMUMS;
+						chartType = ChartType.CLUSTER_MINIMUMS;
 						thumbnail = generateThumbnail(clusterer.getClusterMinimums(), deviations, eventIDs);
 						chart = generateImage(clusterer.getClusterMinimums(), deviations, eventIDs);
 						saveAnalysisResult(clusterer.getClusterMinimums(), deviations, thumbnail, chart);
 						// do averages
-						chartType = CLUSTER_AVERAGES;
+						chartType = ChartType.CLUSTER_AVERAGES;
 						thumbnail = generateThumbnail(centroids, deviations, eventIDs);
 						chart = generateImage(centroids, deviations, eventIDs);
 						saveAnalysisResult(centroids, deviations, thumbnail, chart);
 						// do maxes
-						chartType = CLUSTER_MAXIMUMS;
+						chartType = ChartType.CLUSTER_MAXIMUMS;
 						thumbnail = generateThumbnail(clusterer.getClusterMaximums(), deviations, eventIDs);
 						chart = generateImage(clusterer.getClusterMaximums(), deviations, eventIDs);
 						saveAnalysisResult(clusterer.getClusterMaximums(), deviations, thumbnail, chart);
@@ -653,14 +608,14 @@ public class AnalysisTaskWrapper extends TimerTask {
 					System.out.println("Doing Correlation Analysis...");
 					// get the inclusive data
 					//reducedData;
-					chartType = AnalysisTaskWrapper.CORRELATION_SCATTERPLOT;
+					chartType = ChartType.CORRELATION_SCATTERPLOT;
 					/*
 					for (int i = 0 ; i < reducedData.numDimensions() ; i++) {
 						correlateToMain = true;
 						rCorrelation = reducedData.getCorrelation(i,i);
 						File thumbnail = generateThumbnail(reducedData, i, i);
 						File chart = generateImage(reducedData, i, i);
-						saveAnalysisResult(reducedData, reducedData, thumbnail, chart);	
+						saveAnalysisResult(reducedData, reducedData, thumbnail, chart);
 						correlateToMain = false;
 					}
 					*/
@@ -669,7 +624,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 							rCorrelation = reducedData.getCorrelation(i,j);
 							File thumbnail = generateThumbnail(reducedData, i, j);
 							File chart = generateImage(reducedData, i, j);
-							saveAnalysisResult(reducedData, reducedData, thumbnail, chart);	
+							saveAnalysisResult(reducedData, reducedData, thumbnail, chart);
 						}
 						//System.out.println("Finished: " + (i+1) + " of " + reducedData.numDimensions());
 					}
@@ -680,13 +635,13 @@ public class AnalysisTaskWrapper extends TimerTask {
 			// let the server (main thread) know we are done
 			server.taskFinished();
 			modelData = null;
-		} // else 
+		} // else
 			//System.out.println("nothing to do... ");
-			
+
 	}
-	
+
 	/**
-	 * 
+	 *
 	 * @param centroids
 	 * @param deviations
 	 * @param rowLabels
@@ -751,7 +706,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 	 */
 	public File generateThumbnail(RawDataInterface[] clusters) {
 		File outfile = null;
-		if (chartType == PCA_SCATTERPLOT) {
+		if (chartType == ChartType.PCA_SCATTERPLOT) {
 	        XYDataset data = new PCAPlotDataset(clusters);
 	        JFreeChart chart = ChartFactory.createScatterPlot(
 	            null, null, null, data, PlotOrientation.VERTICAL, false, false, false);
@@ -771,7 +726,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 	 */
 	public File generateImage(RawDataInterface pcaData, RawDataInterface[] clusters) {
 		File outfile = null;
-		if (chartType == PCA_SCATTERPLOT) {
+		if (chartType == ChartType.PCA_SCATTERPLOT) {
 		/*
 			int max = pcaData.numDimensions();
 			int x = max - 1;
@@ -808,7 +763,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 	 */
 	public File generateThumbnail(RawDataInterface pcaData, int i, int j) {
 		File outfile = null;
-		if (chartType == CORRELATION_SCATTERPLOT) {
+		if (chartType == ChartType.CORRELATION_SCATTERPLOT) {
 			pcaData.normalizeData(true);
 	        XYDataset data = new ScatterPlotDataset(pcaData,
 			modelData.toString(), i, j, correlateToMain);
@@ -830,7 +785,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 	 */
 	public File generateImage(RawDataInterface pcaData, int i, int j) {
 		File outfile = null;
-		if (chartType == CORRELATION_SCATTERPLOT) {
+		if (chartType == ChartType.CORRELATION_SCATTERPLOT) {
 			pcaData.normalizeData(true);
 	        XYDataset data = new ScatterPlotDataset(pcaData,
 			modelData.toString(), i, j, correlateToMain);
@@ -854,7 +809,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			Function2D curve = new LineFunction2D(coefficients[0], coefficients[1]);
 			Range range = DatasetUtilities.findDomainExtent(data);
 			XYDataset regressionData = DatasetUtilities.sampleFunction2D(
-				curve, range.getLowerBound(), range.getUpperBound(), 
+				curve, range.getLowerBound(), range.getUpperBound(),
 				100, "Fitted Linear Regression Line");
 			plot.setDataset(1, regressionData);
 			XYItemRenderer lineRenderer = new DefaultXYItemRenderer();
@@ -865,7 +820,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			double[] powerCoefficients = Regression.getPowerRegression(data, 0);
 			Function2D powerCurve = new PowerFunction2D(powerCoefficients[0], powerCoefficients[1]);
 			XYDataset powerRegressionData = DatasetUtilities.sampleFunction2D(
-				powerCurve, range.getLowerBound(), range.getUpperBound(), 
+				powerCurve, range.getLowerBound(), range.getUpperBound(),
 				100, "Fitted Power Regression Line");
 			plot.setDataset(2, powerRegressionData);
 			XYItemRenderer powerLineRenderer = new DefaultXYItemRenderer();
@@ -875,7 +830,7 @@ public class AnalysisTaskWrapper extends TimerTask {
 			plot.getDomainAxis().setRange(range);
 			plot.getRangeAxis().setRange(range);
 
-			JFreeChart chart = new JFreeChart("Correlation Results: r = " + 
+			JFreeChart chart = new JFreeChart("Correlation Results: r = " +
 				rCorrelation, JFreeChart.DEFAULT_TITLE_FONT, plot, true);
 
 
@@ -932,13 +887,13 @@ public class AnalysisTaskWrapper extends TimerTask {
 			}
 		}
 		String chartTitle = modelData.toString();
-		if (chartType == CLUSTER_AVERAGES) {
+		if (chartType == ChartType.CLUSTER_AVERAGES) {
             chartTitle = chartTitle + " Average Values";
 		}
-		if (chartType == CLUSTER_MAXIMUMS) {
+		if (chartType == ChartType.CLUSTER_MAXIMUMS) {
             chartTitle = chartTitle + " Maximum Values";
 		}
-		if (chartType == CLUSTER_MINIMUMS) {
+		if (chartType == ChartType.CLUSTER_MINIMUMS) {
             chartTitle = chartTitle + " Minimum Values";
 		}
         JFreeChart chart = ChartFactory.createStackedBarChart(
