@@ -1,34 +1,27 @@
-package server;
+package edu.uoregon.tau.perfexplorer.server;
 
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
-import common.*;
-
-import common.ChartDataType;
-import common.PerfExplorerOutput;
-import common.RMIChartData;
-import common.RMIPerfExplorerModel;
-import common.RMIView;
-
-import edu.uoregon.tau.perfdmf.database.DB;
-import edu.uoregon.tau.perfdmf.Experiment;
-import edu.uoregon.tau.perfdmf.Metric;
-import edu.uoregon.tau.perfdmf.IntervalEvent;
-
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.sql.SQLException;
-
 import java.util.ArrayList;
 import java.util.List;
 
+import edu.uoregon.tau.perfdmf.Experiment;
+import edu.uoregon.tau.perfdmf.IntervalEvent;
+import edu.uoregon.tau.perfdmf.Metric;
+import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfexplorer.common.ChartDataType;
+import edu.uoregon.tau.perfexplorer.common.PerfExplorerOutput;
+import edu.uoregon.tau.perfexplorer.common.RMIChartData;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.common.RMIView;
+
 /**
  * The ChartData class is used to select data from the database which 
  * represents the performance profile of the selected trials, and return them
  * in a format for JFreeChart to display them.
  *
- * <P>CVS $Id: ChartData.java,v 1.43 2007/05/30 03:23:18 khuck Exp $</P>
+ * <P>CVS $Id: ChartData.java,v 1.56 2009/08/17 14:43:26 khuck Exp $</P>
  * @author  Kevin Huck
  * @version 0.1
  * @since   0.1
@@ -40,8 +33,8 @@ public class ChartData extends RMIChartData {
 	private String groupName = null;
 	private String eventName = null;
 	private String groupByColumn = null;
-	private List columnValues = null;
-	private StringBuffer buf = null;
+	private List<Double> columnValues = null;
+	private StringBuilder buf = null;
 	
 	/**
 	 * Constructor
@@ -94,7 +87,7 @@ public class ChartData extends RMIChartData {
 			String currentExperiment = "";
 			int experimentIndex = -1;
 			if (dataType == ChartDataType.CORRELATION_DATA) {
-				columnValues = new ArrayList();
+				columnValues = new ArrayList<Double>();
 				// do a pre-query to get the event with inclusive value
 				// of 100.0.
 				statement = buildPreQueryStatement();
@@ -140,7 +133,11 @@ public class ChartData extends RMIChartData {
 					value = results.getDouble(5);
 				}
 				if ((metricName.toLowerCase().indexOf("time") != -1) 
-					&& (dataType != ChartDataType.FRACTION_OF_TOTAL))
+					&& (dataType != ChartDataType.FRACTION_OF_TOTAL) &&
+					!metricName.contains("+") &&
+					!metricName.contains("-") &&
+					!metricName.contains("*") &&
+					!metricName.contains("/"))
 					value = value/1000000;
 
 				if (!currentExperiment.equals(groupingName)) {
@@ -153,11 +150,11 @@ public class ChartData extends RMIChartData {
 				// base case - MPI methods, for example
 				// add 0 values for those processor counts.
 				if (dataType == ChartDataType.CORRELATION_DATA) {
-					Double mainThreads = (Double)columnValues.get(columnCounter);
+					Double mainThreads = columnValues.get(columnCounter);
 					while (mainThreads.doubleValue() < numThreads) {
 						addColumn(experimentIndex, mainThreads.doubleValue(), 0.0);
 						columnCounter++;
-						mainThreads = (Double)columnValues.get(columnCounter);
+						mainThreads = columnValues.get(columnCounter);
 					}
 					columnCounter++;
 				}
@@ -234,18 +231,21 @@ public class ChartData extends RMIChartData {
 			DB db = PerfExplorerServer.getServer().getDB();
 
 		PreparedStatement statement = null;
-		buf = new StringBuffer();
+		buf = new StringBuilder();
 		Object object = model.getCurrentSelection();
 		if (dataType == ChartDataType.FRACTION_OF_TOTAL) {
 			// The user wants to know the runtime breakdown by events of one 
 			// experiment as the number of threads of execution increases.
+			StringBuilder tmpBuf = new StringBuilder();
+			buf.append("select ");
 			if (db.getDBType().compareTo("db2") == 0) {
-				buf.append("select cast (ie.name as varchar(256)), ");
+				tmpBuf.append(" cast (ie.name as varchar(256)), ");
 			} else {
-				buf.append("select ie.name, ");
+				tmpBuf.append(" ie.name, ");
 			}
+			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
-			buf.append(" ims.exclusive_percentage ");
+			buf.append(" avg(ims.exclusive_percentage), max(ims.inclusive) ");  // this ensures we get the main routine?
 			buf.append("from interval_mean_summary ims ");
 			buf.append("inner join interval_event ie ");
 			buf.append("on ims.interval_event = ie.id ");
@@ -266,8 +266,12 @@ public class ChartData extends RMIChartData {
 			buf.append(model.getXPercent());
 			buf.append(" and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%')");
 			buf.append("or ims.exclusive_percentage = 100.0) ");
+			buf.append("group by ");
+			buf.append(tmpBuf.toString());
+			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
 			buf.append("order by 1, 2, 3, 4");
 			statement = db.prepareStatement(buf.toString());
 			statement.setString(1, metricName);
@@ -277,7 +281,7 @@ public class ChartData extends RMIChartData {
 			// execution increases.
 			List selections = model.getMultiSelection();
 			buf.append("select ");
-			StringBuffer tmpBuf = new StringBuffer();
+			StringBuilder tmpBuf = new StringBuilder();
 			if (object instanceof RMIView) {
 				if (isLeafView()) {
 					tmpBuf.append(model.getViewSelectionString(db.getDBType()));
@@ -293,7 +297,7 @@ public class ChartData extends RMIChartData {
 			}
 			buf.append(" " + tmpBuf.toString() + ", ");
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
-			buf.append("avg(ims.inclusive) from interval_mean_summary ims ");
+			buf.append("max(ims.inclusive) from interval_mean_summary ims ");
 			buf.append("inner join interval_event ie on ims.interval_event = ie.id ");
 			buf.append("inner join trial t on ie.trial = t.id ");
 			buf.append("inner join metric m on m.id = ims.metric ");
@@ -318,15 +322,13 @@ public class ChartData extends RMIChartData {
 
 			// this sucks - can't use 100.0, because of rounding errors. Bah.
 			if (db.getDBType().compareTo("db2") == 0) {
-				buf.append(" and m.name like ? and ims.inclusive_percentage >= 99.99999 ");
+				buf.append(" and m.name like ?");
 			} else {
-				buf.append(" and m.name = ? and ims.inclusive_percentage >= 99.99999 ");
+				buf.append(" and m.name = ?");
 			}
 			buf.append(" group by ");
-			if (!(object instanceof RMIView)) {
-				buf.append(tmpBuf.toString());
-				buf.append(", ");
-			}
+			buf.append(tmpBuf.toString());
+			buf.append(", ");
 			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
 			buf.append("order by 1, 2, 3, 4");
 			//PerfExplorerOutput.println(buf.toString());
@@ -339,7 +341,7 @@ public class ChartData extends RMIChartData {
 			// one or more experiments, as the number of threads of 
 			// execution increases.
 			buf.append("select ");
-			StringBuffer tmpBuf = new StringBuffer();
+			StringBuilder tmpBuf = new StringBuilder();
 			if (object instanceof RMIView) {
 				if (isLeafView()) {
 					tmpBuf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
@@ -347,11 +349,11 @@ public class ChartData extends RMIChartData {
 					tmpBuf.append(" " + groupByColumn + ", ");
 				}
 			} else {
-			if (db.getDBType().compareTo("db2") == 0) {
-				tmpBuf.append(" cast (e.name as varchar(256)), ");
-			} else {
-				tmpBuf.append(" e.name, ");
-			}
+				if (db.getDBType().compareTo("db2") == 0) {
+					tmpBuf.append(" cast (e.name as varchar(256)), ");
+				} else {
+					tmpBuf.append(" e.name, ");
+				}
 			}
 			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
@@ -437,7 +439,7 @@ public class ChartData extends RMIChartData {
 				e.printStackTrace();
 			}
 
-			buf = new StringBuffer();
+			buf = new StringBuilder();
 			buf.append("insert into ");
 			if ((db.getDBType().compareTo("derby") == 0) ||
 			    (db.getDBType().compareTo("db2") == 0))
@@ -467,6 +469,7 @@ public class ChartData extends RMIChartData {
 			buf.append(model.getXPercent());
 			buf.append(" and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%')");
 			buf.append("or ims.exclusive_percentage = 100.0) ");
 			buf.append("and ims.inclusive_percentage < 100.0) ");
@@ -487,26 +490,28 @@ public class ChartData extends RMIChartData {
 				e.printStackTrace();
 			}
 
-			buf = new StringBuffer();
+			buf = new StringBuilder();
 			buf.append("select distinct ");
+			StringBuilder tmpBuf = new StringBuilder();
 			if (db.getDBType().compareTo("db2") == 0) {
-				buf.append(" cast (ie.name as varchar(256)), ");
+				tmpBuf.append(" cast (ie.name as varchar(256)), ");
 			} else {
-				buf.append(" ie.name, ");
+				tmpBuf.append(" ie.name, ");
 			}
+			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
 
 			if (dataType == ChartDataType.CORRELATION_DATA) {
 				if (db.getDBType().compareTo("oracle") == 0) {
-					buf.append("ims.excl from interval_mean_summary ims ");
+					buf.append("avg(ims.excl) from interval_mean_summary ims ");
 				} else {
-					buf.append("ims.exclusive from interval_mean_summary ims ");
+					buf.append("avg(ims.exclusive) from interval_mean_summary ims ");
 				}
 			} else {
 				if (db.getDBType().compareTo("oracle") == 0) {
-					buf.append("ims.excl from interval_mean_summary ims ");
+					buf.append("avg(ims.excl) from interval_mean_summary ims ");
 				} else {
-					buf.append("ims.exclusive from interval_mean_summary ims ");
+					buf.append("avg(ims.exclusive) from interval_mean_summary ims ");
 				}
 			}
 			buf.append("inner join interval_event ie on ims.interval_event = ie.id ");
@@ -532,6 +537,11 @@ public class ChartData extends RMIChartData {
 			} else {
 				buf.append(" and m.name = ? ");
 			}
+
+			buf.append(" group by ");
+			buf.append(tmpBuf.toString());
+			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
+
 			buf.append(" order by 1, 2, 3, 4 ");
 			//PerfExplorerOutput.println(buf.toString());
 			statement = db.prepareStatement(buf.toString());
@@ -541,21 +551,27 @@ public class ChartData extends RMIChartData {
 			// of one event for one or more experiments, as the number of
 			// threads of execution increases.
 			buf.append("select ");
+			StringBuilder tmpBuf = new StringBuilder();
 			if (object instanceof RMIView) {
 				if (isLeafView()) {
-					buf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
+					tmpBuf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
 				} else {
-					buf.append(" " + groupByColumn + ", ");
+					tmpBuf.append(" " + groupByColumn + ", ");
 				}
 			} else {
 				if (db.getDBType().compareTo("db2") == 0) {
-					buf.append(" cast (e.name as varchar(256)), ");
+					tmpBuf.append(" cast (e.name as varchar(256)), ");
 				} else {
-					buf.append(" e.name, ");
+					tmpBuf.append(" e.name, ");
 				}
 			}
+			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
-			buf.append("exclusive from interval_mean_summary ims ");
+			if (db.getDBType().compareTo("oracle") == 0) {
+				buf.append("avg(ims.excl) from interval_mean_summary ims ");
+			} else {
+				buf.append("avg(ims.exclusive) from interval_mean_summary ims ");
+			}
 			buf.append("inner join interval_event ie on ims.interval_event = ie.id ");
 			buf.append("inner join trial t on ie.trial = t.id ");
 			buf.append("inner join metric m on m.id = ims.metric ");
@@ -581,11 +597,17 @@ public class ChartData extends RMIChartData {
 
 			if (db.getDBType().compareTo("db2") == 0) {
 				buf.append(" and m.name like ? ");
-				buf.append("and ie.name like ? order by 1, 2, 3, 4");
+				buf.append("and ie.name like ? ");
 			} else {
 				buf.append(" and m.name = ? ");
-				buf.append("and ie.name = ? order by 1, 2, 3, 4");
+				buf.append("and ie.name = ? ");
 			}
+			
+			buf.append(" group by ");
+			buf.append(tmpBuf.toString());
+			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
+
+			buf.append(" order by 1, 2, 3, 4");
 			statement = db.prepareStatement(buf.toString());
 			statement.setString(1, metricName);
 			statement.setString(2, eventName);
@@ -594,26 +616,28 @@ public class ChartData extends RMIChartData {
 			// of all the phases for one experiment, as the number of threads of 
 			// execution increases.
 			buf.append("select ");
+			StringBuilder tmpBuf = new StringBuilder();
 			if (object instanceof RMIView) {
 				if (isLeafView()) {
-					//buf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
+					//tmpBuf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
 					if (db.getDBType().compareTo("db2") == 0) {
-						buf.append(" cast (ie.name as varchar(256)), ");
+						tmpBuf.append(" cast (ie.name as varchar(256)), ");
 					} else {
-						buf.append(" ie.name, ");
+						tmpBuf.append(" ie.name, ");
 					}
 				} else {
-					buf.append(" " + groupByColumn + ", ");
+					tmpBuf.append(" " + groupByColumn + ", ");
 				}
 			} else {
 				if (db.getDBType().compareTo("db2") == 0) {
-					buf.append(" cast (ie.name as varchar(256)), ");
+					tmpBuf.append(" cast (ie.name as varchar(256)), ");
 				} else {
-					buf.append(" ie.name, ");
+					tmpBuf.append(" ie.name, ");
 				}
 			}
+			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
-			buf.append("ims.inclusive from interval_mean_summary ims ");
+			buf.append("avg(ims.inclusive) from interval_mean_summary ims ");
 			buf.append("inner join interval_event ie on ims.interval_event = ie.id ");
 			buf.append("inner join trial t on ie.trial = t.id ");
 			buf.append("inner join metric m on m.id = ims.metric ");
@@ -629,7 +653,16 @@ public class ChartData extends RMIChartData {
 			}
 
 //			buf.append("and ims.inclusive_percentage < 100.0 ");
-			buf.append(" and ie.group_name like '%TAU_PHASE%' order by 1, 2, 3, 4");
+			buf.append(" and ie.group_name like '%TAU_PHASE%' ");
+			buf.append(" and ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append(" and ie.group_name not like '%TAU_PARAM%' ");
+
+			buf.append(" group by ");
+			buf.append(tmpBuf.toString());
+			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
+
+			buf.append(" order by 1, 2, 3, 4");
+			
 			statement = db.prepareStatement(buf.toString());
 			if (object instanceof RMIView) {
 				statement.setString(1, metricName);
@@ -643,26 +676,28 @@ public class ChartData extends RMIChartData {
 			// of one experiment as the number of threads of execution
 			// increases.
 			buf.append("select ");
+			StringBuilder tmpBuf = new StringBuilder();
 			if (object instanceof RMIView) {
 				if (isLeafView()) {
-					//buf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
+					//tmpBuf.append(" " + model.getViewSelectionString(db.getDBType()) + ", ");
 					if (db.getDBType().compareTo("db2") == 0) {
-						buf.append(" cast (ie.name as varchar(256)), ");
+						tmpBuf.append(" cast (ie.name as varchar(256)), ");
 					} else {
-						buf.append(" ie.name, ");
+						tmpBuf.append(" ie.name, ");
 					}
 				} else {
-					buf.append(" " + groupByColumn + ", ");
+					tmpBuf.append(" " + groupByColumn + ", ");
 				}
 			} else {
 				if (db.getDBType().compareTo("db2") == 0) {
-					buf.append(" cast (ie.name as varchar(256)), ");
+					tmpBuf.append(" cast (ie.name as varchar(256)), ");
 				} else {
-					buf.append(" ie.name, ");
+					tmpBuf.append(" ie.name, ");
 				}
 			}
+			buf.append(tmpBuf.toString());
 			buf.append("t.node_count, t.contexts_per_node, t.threads_per_context, ");
-			buf.append("ims.inclusive_percentage from interval_mean_summary ims ");
+			buf.append("avg(ims.inclusive_percentage) from interval_mean_summary ims ");
 			buf.append("inner join interval_event ie on ims.interval_event = ie.id ");
 			buf.append("inner join trial t on ie.trial = t.id ");
 			buf.append("inner join metric m on m.id = ims.metric ");
@@ -681,7 +716,15 @@ public class ChartData extends RMIChartData {
 			}
 
 			buf.append("and ims.inclusive_percentage < 100.0 ");
-			buf.append("and ie.group_name like '%TAU_PHASE%' order by 1, 2, 3, 4");
+			buf.append("and ie.group_name like '%TAU_PHASE%' ");
+			buf.append("and ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
+			buf.append(" group by ");
+
+			buf.append(tmpBuf.toString());
+			buf.append(" t.node_count, t.contexts_per_node, t.threads_per_context ");
+
+			buf.append("order by 1, 2, 3, 4");
 
 			statement = db.prepareStatement(buf.toString());
 			statement.setString(1, metricName);
@@ -716,6 +759,7 @@ public class ChartData extends RMIChartData {
 			buf.append("and p.metric = ? ");
 			buf.append("and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%')");
 			buf.append("or s.exclusive_percentage = 100.0) ");
 			buf.append(" order by 1,2 ");
@@ -761,6 +805,7 @@ public class ChartData extends RMIChartData {
 			statement = db.prepareStatement(buf.toString());
 			statement.setInt(1, model.getTrial().getID());
 			statement.setInt(2, model.getMetric().getID());
+			System.out.println(statement.toString());
 		}
 		return statement;
 	}
@@ -769,7 +814,7 @@ public class ChartData extends RMIChartData {
 			DB db = PerfExplorerServer.getServer().getDB();
 
 		PreparedStatement statement = null;
-		buf = new StringBuffer();
+		buf = new StringBuilder();
 		Object object = model.getCurrentSelection();
 		if (dataType == ChartDataType.FRACTION_OF_TOTAL) {
 			// The user wants to know the runtime breakdown by events of one 
@@ -797,6 +842,7 @@ public class ChartData extends RMIChartData {
 			buf.append(model.getXPercent());
 			buf.append(" and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%')");
 			buf.append("or ims.exclusive_percentage = 100.0) group by t.node_count, t.contexts_per_node, t.threads_per_context order by 1, 2, 3");
 			
@@ -825,6 +871,7 @@ public class ChartData extends RMIChartData {
 			buf.append(model.getXPercent());
 			buf.append(" and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%'));");
 			*/
 
@@ -874,7 +921,7 @@ public class ChartData extends RMIChartData {
 	private PreparedStatement buildPreQueryStatement () throws SQLException {
 		DB db = PerfExplorerServer.getServer().getDB();
 		PreparedStatement statement = null;
-		buf = new StringBuffer();
+		buf = new StringBuilder();
 		Object object = model.getCurrentSelection();
 		if (dataType == ChartDataType.CORRELATION_DATA) {
 			// The user wants to know the runtime breakdown by events of one 
@@ -902,6 +949,7 @@ public class ChartData extends RMIChartData {
 			//buf.append(model.getXPercent());
 			buf.append(" and (ie.group_name is null or (");
 			buf.append("ie.group_name not like '%TAU_CALLPATH%' ");
+			buf.append("and ie.group_name not like '%TAU_PARAM%' ");
 			buf.append("and ie.group_name not like '%TAU_PHASE%')");
 			buf.append("or ims.exclusive_percentage = 100.0) ");
 			buf.append("group by t.node_count, t.contexts_per_node, t.threads_per_context order by 2, 3, 4 ");
