@@ -1,21 +1,23 @@
-package server;
+package edu.uoregon.tau.perfexplorer.server;
 
 import java.sql.PreparedStatement;
 import java.sql.ResultSet;
 import java.sql.SQLException;
 import java.util.ArrayList;
 import java.util.List;
+import java.util.StringTokenizer;
+import java.util.NoSuchElementException;
 
-import common.PerfExplorerException;
-import common.PerfExplorerOutput;
-import common.TransformationType;
-import common.RMIPerfExplorerModel;
-import clustering.AnalysisFactory;
-import clustering.RawDataInterface;
 
 import edu.uoregon.tau.perfdmf.DatabaseAPI;
 import edu.uoregon.tau.perfdmf.Metric;
 import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfexplorer.clustering.RawDataInterface;
+import edu.uoregon.tau.perfexplorer.clustering.weka.AnalysisFactory;
+import edu.uoregon.tau.perfexplorer.common.PerfExplorerException;
+import edu.uoregon.tau.perfexplorer.common.PerfExplorerOutput;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.common.TransformationType;
 
 public class DataUtils {
 	/**
@@ -25,7 +27,6 @@ public class DataUtils {
 	 */
 	public static RawDataInterface getRawData (DatabaseAPI session, RMIPerfExplorerModel modelData) throws PerfExplorerException {
 		PerfExplorerServer server = PerfExplorerServer.getServer();
-		AnalysisFactory factory = server.getAnalysisFactory();
 		PerfExplorerOutput.print("Getting raw data...");
 	    int numRows = 0;
 	    int numTotalThreads = 0;
@@ -35,18 +36,32 @@ public class DataUtils {
 	    int threads = 0;
 	    RawDataInterface rawData = null;
 	    double maximum = 0.0;
-	    List eventIDs = null;
+	    List<String> eventIDs = null;
+	    String clusterType=modelData.getClusterValueType();
+		String clusterPerType="inclusive_percentage";
+		
 		PerfExplorerOutput.print("Getting constants...");
 		try {
 			DB db = session.db();
+			
+			if(clusterType.compareTo("inclusive")!=0){
+				if (db.getDBType().compareTo("oracle") == 0) {
+					clusterType="excl";
+				}
+				else{
+					clusterType="exclusive";
+				}
+				clusterPerType="exclusive_percentage";
+			}
+			
 			PreparedStatement statement = null;
 			// First, get the total number of rows we are expecting
-			StringBuffer sql = new StringBuffer();
-
+			StringBuilder sql = new StringBuilder();
+			
             if (db.getDBType().compareTo("oracle") == 0) {
                 sql.append("select count(p.excl) ");
             } else {
-                sql.append("select count(p.exclusive) ");
+                sql.append("select count(p.inclusive) ");
             }
 
 			sql.append("from interval_event e ");
@@ -62,7 +77,7 @@ public class DataUtils {
 				//sql.append("on e.id = s.interval_event and s.metric = p.metric ");
 			}
 			sql.append("where e.trial = ?");
-			sql.append(" and e.group_name not like '%TAU_CALLPATH%' ");
+			sql.append(" and (e.group_name is null or e.group_name not like '%TAU_CALLPATH%') ");
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				sql.append(" and p.metric = ?");
 			}
@@ -81,7 +96,7 @@ public class DataUtils {
 
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				// Next, get the event names, and count them
-				sql = new StringBuffer();
+				sql = new StringBuilder();
 				sql.append("select e.id, e.name from interval_event e ");
 				if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 					sql.append("inner join interval_mean_summary s on ");
@@ -95,7 +110,7 @@ public class DataUtils {
 				} else {
 					sql.append("where e.trial = ?");
 				}
-			sql.append(" and e.group_name not like '%TAU_CALLPATH%' ");
+			sql.append(" and (e.group_name is null or e.group_name not like '%TAU_CALLPATH%') ");
 				sql.append(" order by 1");
 				statement = db.prepareStatement(sql.toString());
 				statement.setInt(1, modelData.getTrial().getID());
@@ -107,17 +122,17 @@ public class DataUtils {
 				//PerfExplorerOutput.println(statement.toString());
 				results = statement.executeQuery();
 				numEvents = 0;
-				eventIDs = new ArrayList();
+				eventIDs = new ArrayList<String>();
 				while (results.next() != false) {
 					numEvents++;
-					eventIDs.add(results.getString(2));
+					eventIDs.add(shortName(results.getString(2)));
 				}
 				results.close();
 				statement.close();
 			} else {
 
 				// Next, get the metric names, and count them
-				sql = new StringBuffer();
+				sql = new StringBuilder();
 				sql.append("select m.id, m.name from metric m ");
 				sql.append("where m.trial = ?");
 				sql.append(" order by 1");
@@ -126,7 +141,7 @@ public class DataUtils {
 				//PerfExplorerOutput.println(statement.toString());
 				results = statement.executeQuery();
 				numEvents = 0;
-				eventIDs = new ArrayList();
+				eventIDs = new ArrayList<String>();
 				while (results.next() != false) {
 					numEvents++;
 					eventIDs.add(results.getString(2));
@@ -136,7 +151,7 @@ public class DataUtils {
 			}
 
 			// get the number of threads
-			sql = new StringBuffer();
+			sql = new StringBuilder();
 			sql.append("select max(node), max(context), max(thread) ");
 			sql.append("from interval_location_profile ");
 			sql.append("inner join interval_event ");
@@ -149,8 +164,8 @@ public class DataUtils {
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				statement.setInt(2, ((Metric)(modelData.getCurrentSelection())).getID());
 			}
-			results = statement.executeQuery();
 			//PerfExplorerOutput.println(statement.toString());
+			results = statement.executeQuery();
 			if (results.next() != false) {
 				nodes = results.getInt(1) + 1;
 				contexts = results.getInt(2) + 1;
@@ -176,7 +191,7 @@ public class DataUtils {
 		PerfExplorerOutput.println("numEvents: " + numEvents);
 		PerfExplorerOutput.println(" Done!");
 		*/
-		rawData = factory.createRawData("Cluster Test", eventIDs, numTotalThreads, numEvents);
+		rawData = AnalysisFactory.createRawData("Cluster Test", eventIDs, numTotalThreads, numEvents, null);
 		ResultSet results = null;
 		int currentFunction = 0;
 		int functionIndex = -1;
@@ -186,18 +201,27 @@ public class DataUtils {
 		try {
 			DB db = session.db();
 			PreparedStatement statement = null;
-			StringBuffer sql = new StringBuffer();
+			StringBuilder sql = new StringBuilder();
 			if (modelData.getDimensionReduction().equals(TransformationType.OVER_X_PERCENT)) {
 				sql.append("select e.id, (p.node*");
 				sql.append(contexts * threads);
 				sql.append(") + (p.context*");
 				sql.append(threads);
                 
-                if (db.getDBType().compareTo("oracle") == 0) {
-                    sql.append(") + p.thread as thread, p.metric as metric, p.excl/1000000, ");
-                } else {
-                    sql.append(") + p.thread as thread, p.metric as metric, p.exclusive/1000000, ");
-                }
+				//TODO: This will request the data that gets used for clustering
+//				if(clusterType.compareTo("inclusive")==0)
+//				{
+//					sql.append(") + p.thread as thread, p.metric as metric, p.inclusive/1000000, ");
+//				}
+//				else{
+//                if (db.getDBType().compareTo("oracle") == 0) {
+//                    sql.append(") + p.thread as thread, p.metric as metric, p.excl/1000000, ");
+//                } else {
+//                    sql.append(") + p.thread as thread, p.metric as metric, p.exclusive/1000000, ");
+//                }}
+				
+				sql.append(") + p.thread as thread, p.metric as metric, p."+clusterType+"/1000000, ");
+				
 				sql.append("p.inclusive/1000000, s.inclusive_percentage, s.exclusive_percentage ");
 				sql.append("from interval_event e ");
 				sql.append("inner join interval_mean_summary s ");
@@ -211,12 +235,20 @@ public class DataUtils {
 				sql.append("select e.id, (p.node*" + (contexts * threads) + "");
 				sql.append(") + (p.context*" + threads + "");
                 
-                if (db.getDBType().compareTo("oracle") == 0) {
-                    sql.append(") + p.thread as thread, p.metric as metric, p.excl, ");
-                } else {
-                    sql.append(") + p.thread as thread, p.metric as metric, p.exclusive, ");
-                }
-
+				//TODO: This will request the data that gets used for clustering
+//				if(clusterType.compareTo("inclusive")==0)
+//				{
+//					sql.append(") + p.thread as thread, p.metric as metric, p.inclusive, ");
+//				}
+//				else{
+//					if (db.getDBType().compareTo("oracle") == 0) {
+//						sql.append(") + p.thread as thread, p.metric as metric, p.excl, ");
+//					} else {
+//						sql.append(") + p.thread as thread, p.metric as metric, p.exclusive, ");
+//					}
+//				}
+				sql.append(") + p.thread as thread, p.metric as metric, p."+clusterType+", ");
+				
 				sql.append("p.inclusive/1000000, p.inclusive_percentage ");
 				sql.append("from interval_event e ");
 				sql.append("left outer join interval_location_profile p ");
@@ -225,7 +257,7 @@ public class DataUtils {
 			if (modelData.getCurrentSelection() instanceof Metric) {
 				sql.append(" and p.metric = ? ");
 			}
-			sql.append(" and e.group_name not like '%TAU_CALLPATH%' ");
+			sql.append(" and (e.group_name is null or e.group_name not like '%TAU_CALLPATH%') ");
 			sql.append(" order by 3,1,2 ");
 			statement = db.prepareStatement(sql.toString());
 			statement.setInt(1, modelData.getTrial().getID());
@@ -279,4 +311,31 @@ public class DataUtils {
 		PerfExplorerOutput.println(" Done!");
 		return rawData;
 	}
+
+    public static String shortName(String longName) {
+        String shorter = null;
+		// strip off instrumentation info
+    	StringTokenizer st = new StringTokenizer(longName, "[");
+    	try {
+        	shorter = st.nextToken();
+        	if (shorter.length() < longName.length()) {
+            	shorter = shorter.trim() + "()";
+        	}
+    	} catch (NoSuchElementException e) {
+        	shorter = longName;
+    	}
+		// strip off parameters
+    	st = new StringTokenizer(longName, "(");
+    	try {
+        	shorter = st.nextToken();
+        	if (shorter.length() < longName.length()) {
+            	shorter = shorter.trim() + "()";
+        	}
+    	} catch (NoSuchElementException e) {
+        	shorter = longName;
+    	}
+		//System.out.println(longName + " is now: " + shorter);
+        return shorter;
+    }
+
 }
