@@ -1,52 +1,44 @@
-package server;
+package edu.uoregon.tau.perfexplorer.server;
 
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.sql.SQLException;
-import common.*;
-
-import common.ChartDataType;
-import common.PerfExplorerOutput;
-import common.RMIChartData;
-import common.RMIPerfExplorerModel;
-import common.RMIView;
-
-import edu.uoregon.tau.perfdmf.database.DB;
-import edu.uoregon.tau.perfdmf.Experiment;
-import edu.uoregon.tau.perfdmf.Application;
-import edu.uoregon.tau.perfdmf.Trial;
-import edu.uoregon.tau.perfdmf.Metric;
-import edu.uoregon.tau.perfdmf.IntervalEvent;
-import edu.uoregon.tau.common.Gzip;
-
-import java.sql.PreparedStatement;
-import java.sql.ResultSet;
-import java.sql.SQLException;
-
-import java.util.ArrayList;
-import java.util.List;
-import java.util.HashSet;
-import java.util.Collections;
-
-import javax.xml.parsers.ParserConfigurationException;
-import java.util.regex.*;
-import javax.xml.parsers.DocumentBuilderFactory;
-import javax.xml.transform.*;
-import javax.xml.transform.dom.*;
-import javax.xml.transform.stream.*;
-import javax.xml.parsers.DocumentBuilder;
-import org.w3c.dom.*;
-import org.xml.sax.*;
-import java.io.StringReader;
-import java.io.Reader;
 import java.io.InputStream;
+import java.io.Reader;
+import java.io.StringReader;
+import java.sql.PreparedStatement;
+import java.sql.ResultSet;
+import java.sql.SQLException;
+import java.util.ArrayList;
+import java.util.Collections;
+import java.util.HashSet;
+import java.util.List;
+
+import javax.xml.parsers.DocumentBuilder;
+import javax.xml.parsers.DocumentBuilderFactory;
+
+import org.w3c.dom.Document;
+import org.w3c.dom.Node;
+import org.w3c.dom.NodeList;
+import org.xml.sax.InputSource;
+
+import edu.uoregon.tau.common.Gzip;
+import edu.uoregon.tau.perfdmf.Application;
+import edu.uoregon.tau.perfdmf.AtomicEvent;
+import edu.uoregon.tau.perfdmf.Experiment;
+import edu.uoregon.tau.perfdmf.IntervalEvent;
+import edu.uoregon.tau.perfdmf.Metric;
+import edu.uoregon.tau.perfdmf.Trial;
+import edu.uoregon.tau.perfdmf.database.DB;
+import edu.uoregon.tau.perfexplorer.common.ChartDataType;
+import edu.uoregon.tau.perfexplorer.common.PerfExplorerOutput;
+import edu.uoregon.tau.perfexplorer.common.RMIGeneralChartData;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.common.TransformationType;
 
 /**
  * The GeneralChartData class is used to select data from the database which 
  * represents the performance profile of the selected trials, and return them
  * in a format for JFreeChart to display them.
  *
- * <P>CVS $Id: GeneralChartData.java,v 1.24 2007/10/08 16:35:52 khuck Exp $</P>
+ * <P>CVS $Id: GeneralChartData.java,v 1.38 2009/08/04 22:19:01 wspear Exp $</P>
  * @author  Kevin Huck
  * @version 0.2
  * @since   0.2
@@ -59,7 +51,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 	private String eventName = null;
 	private String groupByColumn = null;
 	private List columnValues = null;
-	private StringBuffer buf = null;
+	private StringBuilder buf = null;
 	
 	/**
 	 * Constructor
@@ -105,6 +97,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 	private void doQuery () {
 		// declare the statement here, so we can reference it in the catch
 		// region, if necessary
+        //System.out.println("doQuery");
 		PreparedStatement statement = null;
 		DB db = null;
 		try {
@@ -115,7 +108,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 ////////////////////////////////
 
 			// create and populate the temporary trial table
-			buf = buildCreateTableStatement("trial", "temp_trial", db, true);
+			buf = buildCreateTableStatement("trial", "temp_trial", db, true, false);
    			buf.append("(select trial.* from trial ");
 	        buf.append("inner join experiment ");
 			buf.append("on trial.experiment = experiment.id ");
@@ -123,7 +116,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 			buf.append("on experiment.application = application.id ");
 			buf.append("where ");
 			// add the where clause
-			List selections = model.getMultiSelection();
+			List<Object> selections = model.getMultiSelection();
 			if (selections == null) {
 				// just one selection
 				Object obj = model.getCurrentSelection();
@@ -198,7 +191,8 @@ public class GeneralChartData extends RMIGeneralChartData {
 			}
 			buf.append(") ");
 			statement = db.prepareStatement(buf.toString());
-			//System.out.println(statement.toString());
+			// System.out.println(buf.toString());
+			// System.out.println(statement.toString());
 			statement.execute();
 			statement.close();
 
@@ -226,13 +220,14 @@ public class GeneralChartData extends RMIGeneralChartData {
 			if (foundXML) {
 				// create and populate the temporary XML_METADATA table
 				buf = buildCreateTableStatement("xml_metadata",
-						"temp_xml_metadata", db, false);
+						"temp_xml_metadata", db, false, false);
 				if (db.getDBType().compareTo("derby") == 0) {
 					buf.append(" (trial int, metadata_name varchar(4000), metadata_value varchar(4000))");
 				} else {
 					buf.append(" (trial int, metadata_name text, metadata_value text)");
 				}
 				statement = db.prepareStatement(buf.toString());
+				// System.out.println(buf.toString());
 				//System.out.println(statement.toString());
 				statement.execute();
 				statement.close();
@@ -272,13 +267,13 @@ public class GeneralChartData extends RMIGeneralChartData {
 					NodeList names = null;
 					NodeList values = null;
 	
-					try {
-						/* this is the 1.3 through 1.4 way */
-						names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
-							"/metadata/CommonProfileAttributes/attribute/name");
-						values = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
-							"/metadata/CommonProfileAttributes/attribute/value");
-					} catch (NoClassDefFoundError e) {
+//					try {
+//						/* this is the 1.3 through 1.4 way */
+//						names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
+//							"/metadata/CommonProfileAttributes/attribute/name");
+//						values = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
+//							"/metadata/CommonProfileAttributes/attribute/value");
+//					} catch (NoClassDefFoundError e) {
 	
 						/* this is the 1.5 way */
 						// build the xpath object to jump around in that document
@@ -293,17 +288,16 @@ public class GeneralChartData extends RMIGeneralChartData {
 						values = (NodeList) 
 							xpath.evaluate("/metadata/CommonProfileAttributes/attribute/value", 
 							metadata, javax.xml.xpath.XPathConstants.NODESET);
-					}
+//					}
 			
 					for (int i = 0 ; i < names.getLength() ; i++) {
 						Node name = (Node)names.item(i).getFirstChild();
 						Node value = (Node)values.item(i).getFirstChild();
-						//System.out.println(name.getNodeValue()+" =? "+model.getChartMetadataFieldName());
 						if ((model.getChartMetadataFieldName() == null ||
 					     	model.getChartMetadataFieldName().equals(name.getNodeValue())) &&
 							(model.getChartMetadataFieldValue() == null ||
 					     	model.getChartMetadataFieldValue().equals(value.getNodeValue()))) {
-							buf = new StringBuffer();
+							buf = new StringBuilder();
 							buf.append("insert into temp_xml_metadata VALUES (?,?,?)");
 							PreparedStatement statement2 = db.prepareStatement(buf.toString());
 							statement2.setInt(1, xmlResults.getInt(1));
@@ -313,6 +307,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 							} else {
 								statement2.setString(3, value.getNodeValue());
 							}
+							// System.out.println(buf.toString());
 							//System.out.println(statement2.toString());
 							statement2.executeUpdate();
 							statement2.close();
@@ -329,45 +324,47 @@ public class GeneralChartData extends RMIGeneralChartData {
 /////////////////////////
 
 			// create and populate the temporary metric table
-			buf = buildCreateTableStatement("metric", "temp_metric", db, true);
-    		buf.append("(select metric.* from metric ");
-			buf.append("inner join temp_trial ");
-			buf.append("on metric.trial = temp_trial.id ");
-			// add the where clause
-			List metricNames = model.getMetricNames();
-			if (metricNames != null) {
-				//if (db.getDBType().compareTo("db2") == 0) {
-					buf.append("where upper(metric.name) like ? ");
-				//} else {
-					//buf.append("where metric.name = ? ");
-				//}
-				for (int i = 1 ; i < metricNames.size() ; i++) {
+			if (!model.getChartSeriesName().equals("atomic_event.name")) {
+				buf = buildCreateTableStatement("metric", "temp_metric", db, true, false);
+    			buf.append("(select metric.* from metric ");
+				buf.append("inner join temp_trial ");
+				buf.append("on metric.trial = temp_trial.id ");
+				// add the where clause
+				List<String> metricNames = model.getMetricNames();
+				if (metricNames != null) {
 					//if (db.getDBType().compareTo("db2") == 0) {
-						buf.append("or upper(metric.name) like ? ");
+						buf.append("where upper(metric.name) like ? ");
 					//} else {
-						//buf.append("or metric.name = ? ");
+						//buf.append("where metric.name = ? ");
 					//}
+					for (int i = 1 ; i < metricNames.size() ; i++) {
+						//if (db.getDBType().compareTo("db2") == 0) {
+							buf.append("or upper(metric.name) like ? ");
+						//} else {
+							//buf.append("or metric.name = ? ");
+						//}
+					}
 				}
-			}
-			buf.append(") ");
-//			System.out.println(buf.toString());
-			statement = db.prepareStatement(buf.toString());
-			if (metricNames != null) {
-				for (int i = 1 ; i <= metricNames.size() ; i++) {
-					String tmp = (String)metricNames.get(i-1);
-					statement.setString(i, tmp.toUpperCase());
+				buf.append(") ");
+				statement = db.prepareStatement(buf.toString());
+				if (metricNames != null) {
+					for (int i = 1 ; i <= metricNames.size() ; i++) {
+						String tmp = (String)metricNames.get(i-1);
+						statement.setString(i, tmp.toUpperCase());
+					}
 				}
+				// System.out.println(buf.toString());
+				//System.out.println(statement.toString());
+				statement.execute();
+				statement.close();
 			}
-			//System.out.println(statement.toString());
-			statement.execute();
-			statement.close();
 
 ////////////////////////////////
 
 			// if we only want the main event, handle that
 			// we need a sub query.  Bah.
 			if (model.getMainEventOnly()) {
-				buf = new StringBuffer();
+				buf = new StringBuilder();
 				buf.append("select ie.name from interval_event ie ");
    				buf.append("inner join interval_mean_summary ims ");
 				buf.append("on ie.id = ims.interval_event, ");
@@ -389,6 +386,7 @@ public class GeneralChartData extends RMIGeneralChartData {
    				buf.append("and ims.metric = metricid ");
    				buf.append("and ims.inclusive = maxinclusive");
 				statement = db.prepareStatement(buf.toString());
+				// System.out.println(buf.toString());
 				//System.out.println(statement.toString());
 				ResultSet results = statement.executeQuery();
 
@@ -403,106 +401,157 @@ public class GeneralChartData extends RMIGeneralChartData {
 
 ////////////////////////////////
 
-			// create and populate the temporary event table
-			buf = buildCreateTableStatement("event", "temp_event", db, true);
-			buf.append("(select interval_event.* from interval_event ");
-			buf.append("inner join temp_trial ");
-			buf.append("on interval_event.trial = temp_trial.id ");
-			buf.append("inner join interval_mean_summary ");
-   			buf.append("on interval_mean_summary.interval_event = interval_event.id ");
-			buf.append("inner join temp_metric ");
-			buf.append("on interval_mean_summary.metric = temp_metric.id ");
+			// forward declare some variables...
+			List<String> groupNames = model.getGroupNames();
+			List<String> eventNames = model.getEventNames();
 
-			// add the where clause
-			boolean didWhere = false;
+			if (model.getChartSeriesName().equals("atomic_event.name")) {
+				// create and populate the temporary event table
+				buf = buildCreateTableStatement("event", "temp_event", db, true,
+					model.getChartSeriesName().equals("atomic_event.name"));
+				buf.append("(select atomic_event.* from atomic_event ");
+				buf.append("inner join temp_trial ");
+				buf.append("on atomic_event.trial = temp_trial.id ");
 
-			// if we don't want to include callpath or phase data, handle that
-			if (model.getEventNoCallpath()) {
-				if (didWhere) {
-					buf.append("and ");
-				} else {
-					buf.append("where ");
-					didWhere = true;
-				}
-				buf.append("(interval_event.group_name is null ");
-        		buf.append("or (interval_event.group_name not like '%TAU_CALLPATH%' ");
-            	buf.append("and interval_event.group_name not like '%TAU_PHASE%')) ");
-			}
+				// add the where clause
+				boolean didWhere = false;
 
-			// if we only want to see events with > X percent of total runtime
-			if (model.getDimensionReduction() == TransformationType.OVER_X_PERCENT) {
-				if (didWhere) {
-					buf.append("and ");
-				} else {
-					buf.append("where ");
-					didWhere = true;
-				}
-				buf.append("interval_mean_summary.exclusive_percentage > ");
-				buf.append(model.getXPercent());
-				buf.append(" ");
-			}
-
-			// if we want to see the event with 100% exclusive
-			if (model.getEventExclusive100()) {
-				if (didWhere) {
-					buf.append("and ");
-				} else {
-					buf.append("where ");
-					didWhere = true;
-				}
-				buf.append("interval_mean_summary.exclusive_percentage = 100 ");
-			}
-
-			// if we want to see events in particular groups
-			List groupNames = model.getGroupNames();
-			if (groupNames != null) {
-				boolean gotOne = false;
-				for (int i = 0 ; i < groupNames.size() ; i++) {
+				// if we don't want to include callpath or phase data, handle that
+				if (model.getEventNoCallpath()) {
 					if (didWhere) {
-						if (gotOne) {
-							buf.append("or ");
-						} else {
-							buf.append("and ");
-							gotOne = true;
-						}
+						buf.append("and ");
 					} else {
 						buf.append("where ");
 						didWhere = true;
 					}
-					if (db.getDBType().compareTo("db2") == 0) {
-						buf.append("interval_event.group_name like ? ");
-					} else {
-						buf.append("interval_event.group_name = ? ");
+					buf.append("atomic_event.name not like ' : ' ");
+				}
+
+				// if we want to see particular events
+				if (eventNames != null) {
+					boolean gotOne = false;
+					for (int i = 0 ; i < eventNames.size() ; i++) {
+						if (didWhere) {
+							if (gotOne) {
+								buf.append("or ");
+							} else {
+								buf.append("and ");
+								gotOne = true;
+							}
+						} else {
+							buf.append("where ");
+							didWhere = true;
+						}
+						if (db.getDBType().compareTo("db2") == 0) {
+							buf.append("atomic_event.name like ? ");
+						} else {
+							buf.append("atomic_event.name = ? ");
+						}
 					}
 				}
-			}
+				buf.append(")");
 
-			// if we want to see particular events
-			List eventNames = model.getEventNames();
-			if (eventNames != null) {
-				boolean gotOne = false;
-				for (int i = 0 ; i < eventNames.size() ; i++) {
+			} else {
+				// create and populate the temporary event table
+				buf = buildCreateTableStatement("event", "temp_event", db, true, false);
+				buf.append("(select interval_event.* from interval_event ");
+				buf.append("inner join temp_trial ");
+				buf.append("on interval_event.trial = temp_trial.id ");
+				buf.append("inner join interval_mean_summary ");
+   				buf.append("on interval_mean_summary.interval_event = interval_event.id ");
+				buf.append("inner join temp_metric ");
+				buf.append("on interval_mean_summary.metric = temp_metric.id ");
+
+				// add the where clause
+				boolean didWhere = false;
+
+				// if we don't want to include callpath or phase data, handle that
+				if (model.getEventNoCallpath()) {
 					if (didWhere) {
-						if (gotOne) {
-							buf.append("or ");
-						} else {
-							buf.append("and ");
-							gotOne = true;
-						}
+						buf.append("and ");
 					} else {
 						buf.append("where ");
 						didWhere = true;
 					}
-					if (db.getDBType().compareTo("db2") == 0) {
-						buf.append("interval_event.name like ? ");
+					buf.append("(interval_event.group_name is null ");
+        			buf.append("or (interval_event.group_name not like '%TAU_CALLPATH%')) ");
+            		//buf.append("and interval_event.group_name not like '%TAU_PHASE%')) ");
+				}
+
+				// if we only want to see events with > X percent of total runtime
+				if (model.getDimensionReduction() == TransformationType.OVER_X_PERCENT) {
+					if (didWhere) {
+						buf.append("and ");
 					} else {
-						buf.append("interval_event.name = ? ");
+						buf.append("where ");
+						didWhere = true;
+					}
+					buf.append("interval_mean_summary.exclusive_percentage > ");
+					buf.append(model.getXPercent());
+					buf.append(" ");
+				}
+
+				// if we want to see the event with 100% exclusive
+				if (model.getEventExclusive100()) {
+					if (didWhere) {
+						buf.append("and ");
+					} else {
+						buf.append("where ");
+						didWhere = true;
+					}
+					buf.append("interval_mean_summary.exclusive_percentage = 100 ");
+				}
+
+				// if we want to see events in particular groups
+				if (groupNames != null) {
+					boolean gotOne = false;
+					for (int i = 0 ; i < groupNames.size() ; i++) {
+						if (didWhere) {
+							if (gotOne) {
+								buf.append("or ");
+							} else {
+								buf.append("and ");
+								gotOne = true;
+							}
+						} else {
+							buf.append("where ");
+							didWhere = true;
+						}
+						if (db.getDBType().compareTo("db2") == 0) {
+							buf.append("interval_event.group_name like ? ");
+						} else {
+							buf.append("interval_event.group_name = ? ");
+						}
 					}
 				}
+
+				// if we want to see particular events
+				if (eventNames != null) {
+					boolean gotOne = false;
+					for (int i = 0 ; i < eventNames.size() ; i++) {
+						if (didWhere) {
+							if (gotOne) {
+								buf.append("or ");
+							} else {
+								buf.append("and ");
+								gotOne = true;
+							}
+						} else {
+							buf.append("where ");
+							didWhere = true;
+						}
+						if (db.getDBType().compareTo("db2") == 0) {
+							buf.append("interval_event.name like ? ");
+						} else {
+							buf.append("interval_event.name = ? ");
+						}
+					}
+				}
+				buf.append(")");
 			}
-			buf.append(")");
 
 			// build the statement
+			// System.out.println(buf.toString());
 			statement = db.prepareStatement(buf.toString());
 
 			// if we put parameters in the statement, fill them.
@@ -524,50 +573,108 @@ public class GeneralChartData extends RMIGeneralChartData {
 				}
 			}
 
+			// System.out.println(buf.toString());
 			//System.out.println(statement.toString());
 			statement.execute();
 			statement.close();
 
 ////////////////////////////////
 
-			// The user wants parametric study data, with the data
-			// organized with two axes, the x and the y.
-			// unlike scalability: 
-			// NO ASSUMPTION ABOUT WHICH COLUMN IS THE SERIES NAME,
-			// Y AXIS OR X AXIS!
 			String seriesName = model.getChartSeriesName();
 			String xAxisName = model.getChartXAxisName();
 			String yAxisName = model.getChartYAxisName();
+			buf = new StringBuilder();
 			String tableName = "interval_mean_summary";
-			if (yAxisName.indexOf("interval_mean_summary") >= 0) {
-				tableName = "interval_mean_summary";
-			} else if (yAxisName.indexOf("interval_total_summary") >= 0) {
-				tableName = "interval_total_summary";
-			}
 
-			buf = new StringBuffer();
-			buf.append("select ");
-			if (db.getDBType().compareTo("derby") == 0) {
-				// first item - the series name
-				buf.append(fixClause(seriesName, db) + ", ");
-				// second item - the x axis
-				buf.append(fixClause(xAxisName, db) + ", ");
-				// second item - the y axis
-				buf.append(fixClause(yAxisName, db) + " ");
+			if (model.getChartSeriesName().equals("atomic_event.name")) {
+				tableName = "atomic_location_profile";
+				buf.append("select ");
+				if (db.getDBType().compareTo("derby") == 0) {
+					// first item - the series name
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(fixClause(seriesName, db));
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(", ");
+					// second item - the x axis
+					buf.append(fixClause(xAxisName, db) + ", ");
+					// second item - the y axis
+					buf.append(fixClause(yAxisName, db) + " ");
+				} else {
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					
+					// first item - the series name
+					buf.append(fixClause(seriesName, db));
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(" as series_name, ");
+					// second item - the x axis
+					buf.append(fixClause(xAxisName, db) + " as xaxis_value, ");
+					// second item - the y axis
+					buf.append(fixClause(yAxisName, db) + " as yaxis_value ");
+				}
+				// add the tables
+				buf.append("from " + tableName + " ");
+				buf.append("inner join temp_event ");
+				buf.append("on " + tableName + ".atomic_event = temp_event.id ");
 			} else {
-				// first item - the series name
-				buf.append(fixClause(seriesName, db) + " as series_name, ");
-				// second item - the x axis
-				buf.append(fixClause(xAxisName, db) + " as xaxis_value, ");
-				// second item - the y axis
-				buf.append(fixClause(yAxisName, db) + " as yaxis_value ");
+				// The user wants parametric study data, with the data
+				// organized with two axes, the x and the y.
+				// unlike scalability: 
+				// NO ASSUMPTION ABOUT WHICH COLUMN IS THE SERIES NAME,
+				// Y AXIS OR X AXIS!
+				if (yAxisName.indexOf("interval_mean_summary") >= 0) {
+					tableName = "interval_mean_summary";
+				} else if (yAxisName.indexOf("interval_total_summary") >= 0) {
+					tableName = "interval_total_summary";
+				}else if(yAxisName.indexOf("interval_location_profile") >=0){
+					tableName="interval_location_profile";
+				}
+				buf.append("select ");
+
+				if (db.getDBType().compareTo("derby") == 0) {
+					// first item - the series name
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(fixClause(seriesName, db));
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(", ");
+					// second item - the x axis
+					buf.append(fixClause(xAxisName, db) + ", ");
+					// second item - the y axis
+					buf.append(fixClause(yAxisName, db) + " ");
+				} else {
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					
+					// first item - the series name
+					buf.append(fixClause(seriesName, db));
+					if(model.isChartSeriesXML()){
+						buf.append("'");
+					}
+					buf.append(" as series_name, ");
+					// second item - the x axis
+					buf.append(fixClause(xAxisName, db) + " as xaxis_value, ");
+					// second item - the y axis
+					buf.append(fixClause(yAxisName, db) + " as yaxis_value ");
+				}
+				// add the tables
+				buf.append("from " + tableName + " ");
+				buf.append("inner join temp_metric ");
+				buf.append("on " + tableName + ".metric = temp_metric.id ");
+				buf.append("inner join temp_event ");
+				buf.append("on " + tableName + ".interval_event = temp_event.id ");
 			}
-			// add the tables
-			buf.append("from " + tableName + " ");
-			buf.append("inner join temp_metric ");
-			buf.append("on " + tableName + ".metric = temp_metric.id ");
-			buf.append("inner join temp_event ");
-			buf.append("on " + tableName + ".interval_event = temp_event.id ");
 			buf.append("inner join temp_trial ");
 			buf.append("on temp_event.trial = temp_trial.id ");
 			if (gotXMLData) {
@@ -583,9 +690,12 @@ public class GeneralChartData extends RMIGeneralChartData {
 			//buf.append("group by " + fixClause(seriesName, db));
 			//buf.append(", " + fixClause(xAxisName, db) + " " );
 			if (db.getDBType().compareTo("derby") == 0) {
-				buf.append("group by " + fixClause(seriesName, db) + ", " + fixClause(xAxisName, db));
+				buf.append("group by " + fixClause(seriesName, db));
+				if(!seriesName.equals(xAxisName)){
+					 buf.append(", " + fixClause(xAxisName, db));
+				}
 				// add the order by clause
-				if (seriesName.startsWith("temp_trial.node_count") || xAxisName.startsWith("temp_trial.node_count")) {
+				if (seriesName.startsWith("trial.node_count") || xAxisName.startsWith("trial.node_count")) {
 					buf.append(" order by 1, 2, 3, 4 ");
 				} else {
 					buf.append(" order by 1, 2 ");					
@@ -595,7 +705,11 @@ public class GeneralChartData extends RMIGeneralChartData {
 				// add the order by clause
 				buf.append("order by 1, 2 ");
 			}
+			
+			//System.out.println(buf.toString());
+			
 			statement = db.prepareStatement(buf.toString());
+
 			//System.out.println(statement.toString());
 			ResultSet results = statement.executeQuery();
 
@@ -604,9 +718,9 @@ public class GeneralChartData extends RMIGeneralChartData {
 				// System.out.print(results.getString(2) + ", " );
 				// System.out.println(results.getDouble(3));
 				if (db.getDBType().compareTo("derby") == 0) {
-					if (seriesName.startsWith("temp_trial.node_count")) {
+					if (seriesName.startsWith("trial.node_count")) {
 						addRow(Integer.toString(results.getInt(1) * results.getInt(2) * results.getInt(3)), results.getString(4), results.getDouble(5));
-					} else if (xAxisName.startsWith("temp_trial.node_count")) {
+					} else if (xAxisName.startsWith("trial.node_count")) {
 						addRow(results.getString(1), Integer.toString(results.getInt(2) * results.getInt(3) * results.getInt(4)), results.getDouble(5));
 					} else {
 						addRow(results.getString(1), results.getString(2), results.getDouble(3));
@@ -629,20 +743,22 @@ public class GeneralChartData extends RMIGeneralChartData {
 			try {
 				db.setAutoCommit(true);
 				dropTable(db, "temp_event");
-				dropTable(db, "temp_metric");
+				if (!model.getChartSeriesName().equals("atomic_event.name")) {
+					dropTable(db, "temp_metric");
+				}
 				dropTable(db, "temp_xml_metadata");
 				dropTable(db, "temp_trial");
 			} catch (Exception e) {}
 		}
 	}
 
-	private static StringBuffer buildCreateTableStatement (String oldTableName, String tableName, DB db, boolean appendAs) {
+	private static StringBuilder buildCreateTableStatement (String oldTableName, String tableName, DB db, boolean appendAs, boolean doAtomic) {
 		// just in case, drop the table in case it is still hanging around.
 		// This sometimes happens with Derby.
 		// Have I ever mentioned that Derby sucks?
 		dropTable(db, tableName);
 
-		StringBuffer buf = new StringBuffer();
+		StringBuilder buf = new StringBuilder();
 		if (db.getDBType().equalsIgnoreCase("oracle")) {
 			buf.append("create global temporary table ");
 		} else if ((db.getDBType().equalsIgnoreCase("derby")) ||
@@ -667,9 +783,15 @@ public class GeneralChartData extends RMIGeneralChartData {
 					names = db.getDatabase().getMetricFieldNames();
 					types = db.getDatabase().getMetricFieldTypeNames();
 				} else if (oldTableName.equalsIgnoreCase("event")) {
-					IntervalEvent.getMetaData(db);
-					names = db.getDatabase().getIntervalEventFieldNames();
-					types = db.getDatabase().getIntervalEventFieldTypeNames();
+					if (doAtomic) {
+						AtomicEvent.getMetaData(db);
+						names = db.getDatabase().getAtomicEventFieldNames();
+						types = db.getDatabase().getAtomicEventFieldTypeNames();
+					} else {
+						IntervalEvent.getMetaData(db);
+						names = db.getDatabase().getIntervalEventFieldNames();
+						types = db.getDatabase().getIntervalEventFieldTypeNames();
+					}
 				}
 				buf.append(" (");
 				for (int i = 0 ; i < java.lang.reflect.Array.getLength(names) ; i++) {
@@ -689,7 +811,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 					System.err.println(e.getMessage());
 					e.printStackTrace(System.err);
 				}
-				buf = new StringBuffer();
+				buf = new StringBuilder();
 				buf.append(" insert into " + tableName + " ");
 			} else {
 				buf.append("as ");
@@ -700,7 +822,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 
 	private String fixClause (String inString, DB db) {
 		// change the table names
-		String outString = inString.replaceAll("trial.", "temp_trial.").replaceAll("metric.", "temp_metric.").replaceAll("interval_event.", "temp_event.");
+		String outString = inString.replaceAll("trial.", "temp_trial.").replaceAll("metric.", "temp_metric.").replaceAll("interval_event.", "temp_event.").replaceAll("atomic_event.", "temp_event.");
 		// fix the oracle specific stuff
 		if (db.getDBType().equalsIgnoreCase("oracle")) {
 			outString = outString.replaceAll("exclusive", "exec");
@@ -713,12 +835,12 @@ public class GeneralChartData extends RMIGeneralChartData {
 		return outString;
 	}
 
-	public static List getXMLFields (RMIPerfExplorerModel model) {
+	public static List<String> getXMLFields (RMIPerfExplorerModel model) {
 		// declare the statement here, so we can reference it in the catch
 		// region, if necessary
-		StringBuffer buf = null;
+		StringBuilder buf = null;
 		PreparedStatement statement = null;
-		HashSet set = new HashSet();
+		HashSet<String> set = new HashSet<String>();
 		DB db = null;
 		try {
 			db = PerfExplorerServer.getServer().getDB();
@@ -739,12 +861,12 @@ public class GeneralChartData extends RMIGeneralChartData {
 			}
 			if (!foundXML) {
 				// return an empty list
-				return new ArrayList();
+				return new ArrayList<String>();
 			}
 	////////////////////////////////
 
 			// create and populate the temporary trial table
-			buf = buildCreateTableStatement("trial", "temp_trial", db, true);
+			buf = buildCreateTableStatement("trial", "temp_trial", db, true, false);
    			buf.append("(select trial.* from trial ");
 	        buf.append("inner join experiment ");
 			buf.append("on trial.experiment = experiment.id ");
@@ -752,7 +874,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 			buf.append("on experiment.application = application.id ");
 			buf.append("where ");
 			// add the where clause
-			List selections = model.getMultiSelection();
+			List<Object> selections = model.getMultiSelection();
 			if (selections == null) {
 				// just one selection
 				Object obj = model.getCurrentSelection();
@@ -827,6 +949,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 			}
 			buf.append(") ");
 			statement = db.prepareStatement(buf.toString());
+			// System.out.println(buf.toString());
 			//System.out.println(statement.toString());
 			statement.execute();
 			statement.close();
@@ -877,22 +1000,22 @@ public class GeneralChartData extends RMIGeneralChartData {
 
 				NodeList names = null;
 
-				try {
-					/* this is the 1.3 through 1.4 way */
-					names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
-						"/metadata/CommonProfileAttributes/attribute/name");
-					for (int i = 0 ; i < names.getLength() ; i++) {
-						Node name = (Node)names.item(i).getFirstChild();
-						set.add(name.getNodeValue());
-					}
-					names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
-						"/metadata/ProfileAttributes/attribute/name");
-				
-					for (int i = 0 ; i < names.getLength() ; i++) {
-						Node name = (Node)names.item(i).getFirstChild();
-						set.add(name.getNodeValue());
-					}
-				} catch (NoClassDefFoundError e) {
+//				try {
+//					/* this is the 1.3 through 1.4 way */
+//					names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
+//						"/metadata/CommonProfileAttributes/attribute/name");
+//					for (int i = 0 ; i < names.getLength() ; i++) {
+//						Node name = (Node)names.item(i).getFirstChild();
+//						set.add(name.getNodeValue());
+//					}
+//					names = org.apache.xpath.XPathAPI.selectNodeList(metadata, 
+//						"/metadata/ProfileAttributes/attribute/name");
+//				
+//					for (int i = 0 ; i < names.getLength() ; i++) {
+//						Node name = (Node)names.item(i).getFirstChild();
+//						set.add(name.getNodeValue());
+//					}
+//				} catch (NoClassDefFoundError e) {
 					/* this is the 1.5 way */
 					// build the xpath object to jump around in that document
 					javax.xml.xpath.XPath xpath = javax.xml.xpath.XPathFactory.newInstance().newXPath();
@@ -916,7 +1039,7 @@ public class GeneralChartData extends RMIGeneralChartData {
 						Node name = (Node)names.item(i).getFirstChild();
 						set.add(name.getNodeValue());
 					}
-				}
+//				}
 
 			} 
 			xmlResults.close();
@@ -948,26 +1071,26 @@ public class GeneralChartData extends RMIGeneralChartData {
 			dropTable(db, "temp_trial");
 		}
 
-		List list = new ArrayList(set);
+		List<String> list = new ArrayList<String>(set);
 		Collections.sort(list, String.CASE_INSENSITIVE_ORDER);
 		return list;
 	}
 
-    private static void dropTable(DB db, String name) {
-        PreparedStatement statement = null;
-        try {
-            if (db.getDBType().compareTo("oracle") == 0) {
-                statement = db.prepareStatement("truncate table " + name);
-                //System.out.println(statement.toString());
-                statement.execute();
-                statement.close();
-            }
-            statement = db.prepareStatement("drop table " + name);
-            //System.out.println(statement.toString());
-            statement.execute();
-            statement.close();
-        } catch (Exception e) {
-            // do nothing, it's ok
+	private static void dropTable(DB db, String name) {
+		PreparedStatement statement = null;
+		try {
+			if (db.getDBType().compareTo("oracle") == 0) {
+				statement = db.prepareStatement("truncate table " + name);
+				//System.out.println(statement.toString());
+				statement.execute();
+				statement.close();
+			}
+			statement = db.prepareStatement("drop table " + name);
+			//System.out.println(statement.toString());
+			statement.execute();
+			statement.close();
+		} catch (Exception e) {
+			// do nothing, it's ok
             // System.err.println("***************************************");
             // System.err.println(e.getMessage());
             // System.err.println("***************************************");
@@ -981,9 +1104,8 @@ public class GeneralChartData extends RMIGeneralChartData {
             	//System.err.println(e2.getMessage());
             	//System.err.println("***************************************");
 			}
-        }
-    }
-
+		}
+	}
 
 }
 
