@@ -1,4 +1,4 @@
-package server;
+package edu.uoregon.tau.perfexplorer.server;
 
 import java.util.ArrayList;
 import java.util.List;
@@ -12,9 +12,10 @@ public class QueryManager {
 	 * 
 	 * @throws PerfExplorerException
 	 */
-	public static List getTrialList (String criteria) {
+	@SuppressWarnings("unchecked")  // for Trial.getTrialList() call
+	public static List<Trial> getTrialList (String criteria, boolean getXMLMetadata) {
 		PerfExplorerServer server = PerfExplorerServer.getServer();
-		List list = null;
+		List<Trial> list = null;
 		try {
 			// get the database connection
 			DB db = server.getDB();
@@ -22,7 +23,7 @@ public class QueryManager {
 			// changes in our criteria
 			String whereClause = " where " + fixClause(criteria, db);
 			// ask the API for the trials
-			list = Trial.getTrialList(db, whereClause);
+			list = Trial.getTrialList(db, whereClause, getXMLMetadata);
 		} catch (Exception e) {
 			System.err.println("ERROR: Couldn't the list of trials!");
             System.err.println(e.getMessage());
@@ -30,7 +31,7 @@ public class QueryManager {
 		}
 		if (list == null) {
 			// return an empty list
-			list = new ArrayList();
+			list = new ArrayList<Trial>();
 		}
 		return list;
 	}
