@@ -2,10 +2,8 @@
  * Created on Jun 29, 2005
  *
  */
-package server;
+package edu.uoregon.tau.perfexplorer.server;
 
-import clustering.KMeansClusterInterface;
-import common.RMIPerfExplorerModel;
 import java.awt.Color;
 import java.awt.image.BufferedImage;
 import java.io.File;
@@ -13,11 +11,15 @@ import java.io.IOException;
 import javax.imageio.ImageIO;
 import javax.swing.JPanel;
 
+import edu.uoregon.tau.perfexplorer.clustering.ClusterInterface;
+import edu.uoregon.tau.perfexplorer.common.RMIPerfExplorerModel;
+import edu.uoregon.tau.perfexplorer.constants.Constants;
+
 /**
  * This class takes the Weka or R cluster results, and creates a virtual
  * topology image showing which cluster each thread of execution belongs to.
  *
- * <P>CVS $Id: VirtualTopology.java,v 1.2 2007/01/04 21:20:04 khuck Exp $</P>
+ * <P>CVS $Id: VirtualTopology.java,v 1.9 2009/11/25 09:15:36 khuck Exp $</P>
  * @author khuck
  * @version 0.1
  * @since   0.1
@@ -25,21 +27,21 @@ import javax.swing.JPanel;
  */
 public class VirtualTopology extends JPanel {
 	RMIPerfExplorerModel modelData = null;
-	KMeansClusterInterface clusterer = null;
+	ClusterInterface clusterer = null;
 	int[] pixels = null;
 	BufferedImage img = null;
-	StringBuffer description = null;
+	StringBuilder description = null;
 	private static final int idealSize = 128;
 	private static final Color[] colors = PEChartColor.createDefaultColorArray();
 	
 	
-	public VirtualTopology(RMIPerfExplorerModel modelData, KMeansClusterInterface clusterer) {
+	public VirtualTopology(RMIPerfExplorerModel modelData, ClusterInterface clusterer) {
 		this.modelData = modelData;
 		this.clusterer = clusterer;
 		
-		description = new StringBuffer();
+		description = new StringBuilder();
 		description.append(modelData.toShortString() + ".");
-		description.append(clusterer.getK() + "_clusters");
+		description.append(clusterer.getClusterSizes().length + "_clusters");
 	
 		// build the image data from the cluster results
 		pixels = new int[clusterer.getNumInstances()];
@@ -47,7 +49,7 @@ public class VirtualTopology extends JPanel {
 		// get the size of the image...
 		int width = 1;
 		int height = clusterer.getNumInstances();
-		while (width < height) {
+		while (width < (height/2)) {
 			height = height / 2;
 			width = width * 2;
 		}
@@ -58,7 +60,12 @@ public class VirtualTopology extends JPanel {
 			int i = 0;
 			for (int x = 0 ; x < width ; x++) {
 				for (int y = 0 ; y < height ; y++) {
-					img.setRGB(x, y, colors[clusterer.clusterInstance(i)].getRGB());
+					int index = clusterer.clusterInstance(i);
+					if (index >= 0) {
+						img.setRGB(x, y, colors[index].getRGB());
+					} else {
+						img.setRGB(x, y, Color.black.getRGB());
+					}
 					i++;
 				}
 			}
@@ -72,9 +79,14 @@ public class VirtualTopology extends JPanel {
 			int i = 0;
 			for (int x = 0 ; x < width ; x++) {
 				for (int y = 0 ; y < height ; y++) {
+					int index = clusterer.clusterInstance(i);
+					int value = Color.black.getRGB();
+					if (index >= 0) {
+						value = colors[index].getRGB();
+					}
 					for (int cellX = x * cellWidth ; cellX < (x+1) * cellWidth ; cellX++) {
 						for (int cellY = y * cellHeight ; cellY < (y+1) * cellHeight ; cellY++) {
-							img.setRGB(cellX, cellY, colors[clusterer.clusterInstance(i)].getRGB());
+							img.setRGB(cellX, cellY, value);
 						}
 					}
 					i++;
@@ -84,7 +96,7 @@ public class VirtualTopology extends JPanel {
 	}
 
 	public String getImage() {
-		String filename = "/tmp/clusterImage." + description + ".png";
+		String filename = Constants.TMPDIR + "clusterImage." + description + ".png";
 		File outFile = new File(filename);
 		try {
 			ImageIO.write(img, "PNG", outFile);
@@ -98,7 +110,7 @@ public class VirtualTopology extends JPanel {
 	}
 
 	public String getThumbnail() {
-		String filename = "/tmp/clusterImage.thumb." + description + ".png";
+		String filename = Constants.TMPDIR + "clusterImage.thumb." + description + ".png";
 		return filename;
 	}
 }
