@@ -11,16 +11,20 @@ import java.io.DataOutputStream;
 import java.util.ArrayList;
 import java.util.EmptyStackException;
 import java.util.HashMap;
-import java.util.Iterator;
+import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.Random;
-import java.util.HashSet;
-//import java.util.Map;
 import java.util.Set;
-//import java.util.TreeMap;
-import base.drawable.*;
+
+import base.drawable.Category;
+import base.drawable.Composite;
+import base.drawable.Kind;
+import base.drawable.Primitive;
+import base.drawable.Topology;
+import base.drawable.YCoordMap;
 import base.io.BufArrayOutputStream;
-import edu.uoregon.tau.trace.*;
+import edu.uoregon.tau.trace.TraceReader;
+import edu.uoregon.tau.trace.TraceReaderCallbacks;
 
 
 class MessageEvent{
@@ -134,6 +138,10 @@ public class InputLog implements base.drawable.InputAPI
 	
 	private static int maxEvtId;
 	
+	private long arch_read;
+	private long count_read=0;
+	private long stepsize=0;
+	
 	private static int GlobalID(int tid, int nid){
 		//System.out.println("n: "+nid+ " t: "+tid);
 		return (nid << 16)+tid;
@@ -214,19 +222,24 @@ public class InputLog implements base.drawable.InputAPI
 		noMonEvents=new HashSet();
 		
 		TraceReaderCallbacks def_cb = new TAUReaderInit();
-		TraceReader tFileDefRead=TraceFactory.OpenFileForInput(tautrc,tauedf);
+		TraceReader tFileDefRead=new TraceReader(tautrc,tauedf);
 		//System.out.println()
-		tFileDefRead.setSubtractFirstTimestamp(false);
+		tFileDefRead.setSubtractFirstTimestamp(true);
 		tFileDefRead.setDefsOnly(true);
 		int recs_read=0;
-		int arch_read=0;
 		do{
 			recs_read=tFileDefRead.readNumEvents(def_cb, 1024,null);
-			arch_read+=recs_read;
 			//if(recs_read>0)
 			//System.out.println("Read "+recs_read+" records");
 		}while(recs_read!=0);//&&((Integer)tb.UserData).intValue()!=0
 		tFileDefRead.closeTrace();
+		arch_read=tFileDefRead.getNumRecords();
+		System.out.println(arch_read+" records initialized.  Processing.");
+		
+		stepsize=arch_read/50;
+		if(stepsize==0){
+			stepsize=1;
+		}
 		
 		/*if(maxthread>0)
 		{
@@ -244,8 +257,9 @@ public class InputLog implements base.drawable.InputAPI
 		noMonEventCycle = new HashMap();//[maxnode+1][maxthread+1];
 		
 		ev_cb = new TAUReader();
-		tFileEvRead=TraceFactory.OpenFileForInput(tautrc,tauedf);
-		tFileDefRead.setSubtractFirstTimestamp(false);
+		tFileEvRead=new TraceReader(tautrc,tauedf);
+		tFileEvRead.setSubtractFirstTimestamp(true);
+		//tFileDefRead.setSubtractFirstTimestamp(false);
 		return true;
 	}
 
@@ -263,6 +277,7 @@ public class InputLog implements base.drawable.InputAPI
 			//maxcats++;
 			return Kind.CATEGORY_ID;
 		}
+
 		if(!doneReading)
 		{
 			while(!eventReady)
@@ -278,6 +293,12 @@ public class InputLog implements base.drawable.InputAPI
 					else
 						return Kind.EOF_ID;
 				}
+				else {
+					count_read++;
+					if(count_read%stepsize==0){
+						System.out.println(count_read+" Records read. "+(int)(100*((double)count_read/(double)arch_read))+"% converted");
+					}
+				}
 			}
 			eventReady=false;
 			//System.out.println("returning prime");
@@ -380,6 +401,22 @@ public class InputLog implements base.drawable.InputAPI
 		return bary_outs.getByteArrayBuf();
 	}	
 	
+	protected static byte[] getInfoVals(double[] vals)
+	{
+		BufArrayOutputStream bary_outs  = new BufArrayOutputStream( vals.length*8 );//8
+		DataOutputStream     data_outs  = new DataOutputStream( bary_outs);
+		try {
+			for(int i=0; i< vals.length;i++)
+			{
+				data_outs.writeDouble(vals[i]);
+			}
+		} catch ( java.io.IOException ioerr ) {
+			ioerr.printStackTrace();
+			System.exit( 1 );
+		}
+		return bary_outs.getByteArrayBuf();
+	}	
+	
 	private static class TAUReaderInit implements TraceReaderCallbacks{
 		
 		public int defClkPeriod(Object userData, double clkPeriod) {
@@ -423,7 +460,7 @@ public class InputLog implements base.drawable.InputAPI
 				name = name.substring(1,name.length()-1);
 			
 			logformat.trace.DobjDef newobj= new logformat.trace.DobjDef(userEventToken, name, Topology.EVENT_ID, 
-					getcolors.nextInt(256),getcolors.nextInt(256),getcolors.nextInt(256), 255, 20, "Count=%d", null);
+					getcolors.nextInt(256),getcolors.nextInt(256),getcolors.nextInt(256), 255, 20, "Count=%E", null);
 			categories.add(newobj);//put(new Integer(numcats), newobj);
 			//numcats++;
 			if(userEventToken>maxEvtId)
@@ -481,17 +518,19 @@ public class InputLog implements base.drawable.InputAPI
 			return 0;
 		}
 		
-		private static boolean compareEvts(MessageEvent a, MessageEvent b){
-			
-			if(a.tag==b.tag&&a.src==b.src&&a.dst==b.dst&&a.siz==b.siz&&a.com==b.com&&a.time!=b.time)
-				return true;
-			
-			return false;
-		}
+		//TODO: Use event comparison
+//		private static boolean compareEvts(MessageEvent a, MessageEvent b){
+//			
+//			if(a.tag==b.tag&&a.src==b.src&&a.dst==b.dst&&a.siz==b.siz&&a.com==b.com&&a.time!=b.time)
+//				return true;
+//			
+//			return false;
+//		}
 		
 		public int sendMessage(Object userData, long time, int sourceNodeToken, int sourceThreadToken, 
 				int destinationNodeToken, int destinationThreadToken, int messageSize, int messageTag, int messageComm){
 
+			destinationThreadToken=0;
 			/*
 			 * The 'tag space' and 'eventId space' may overlap and overwrite each other
 			 * So add the highest tag ID+1, to be sure all IDs sent to jumpshot are unique.
@@ -551,6 +590,7 @@ public class InputLog implements base.drawable.InputAPI
 				int destinationNodeToken, int destinationThreadToken, int messageSize, int messageTag, int messageComm){
 			
 			int messageTagShift=messageTag+maxEvtId+1;
+			sourceThreadToken=0;
 			int sourceGlob = GlobalID(sourceNodeToken,sourceThreadToken);
 			int destGlob = GlobalID(destinationNodeToken,destinationThreadToken);
 			Long srcDst=new Long(SourceDest(sourceGlob,destGlob));
@@ -604,12 +644,13 @@ public class InputLog implements base.drawable.InputAPI
 					return 0;
 				}
 			}
+			//System.out.println("Test Evt Double: "+userEventValue+" vs: "+(long)userEventValue);
 			prime = new base.drawable.Primitive(userEventToken,time*clockP,time*clockP,
 					new double[]{time*clockP} ,new int[]{
 					GlobalID(nodeToken,threadToken)
 					//((Integer)global.get(new Point(nodeToken,threadToken))).intValue()
 					},
-					getInfoVals(new int[]{(int)userEventValue}));
+					getInfoVals(new double[]{userEventValue}));
 			eventReady=true;
 			return 0;
 		}
