@@ -1,5 +1,10 @@
 #!/bin/sh
 
+# If an outside makefile has specified -w, our wrapper will generate extra/incorrect 
+# output unless we remove the flags
+MAKEFLAGS=
+MFLAGS=
+
 # Define variables 
 makefile_specified=no
 options_specified=no
@@ -11,16 +16,37 @@ invoke_with_tau=yes
 DEFAULT_MAKEFILE=
 
 if [ $# = 0 ] ; then
-  echo "Usage $0 [-tau_makefile=<tau_stub_makefile>] [-tau_options=<tau_compiler_opts>] <opts> <file>"
-  echo "If -tau_makefile option is not used, "
-  echo "TAU uses the file specified in the TAU_MAKEFILE environment variable"
-  echo "e.g., "
-  echo "% $0 -tau_makefile=/usr/local/tau-2.x/ia64/lib/Makefile.tau-mpi-pdt  -tau_options=-optVerbose -c foo.c"
-  echo " 	or"
-  echo "% setenv TAU_MAKEFILE /usr/local/tau-2.x/include/Makefile"
-  echo "% setenv TAU_OPTIONS -optVerbose -optTauSelectFile=select.tau"
-  echo "% $0 -c foo.c"
-  exit 1
+    cmd=`basename $0`
+    echo ""
+    echo " $cmd - C compiler wrapper for TAU"
+    echo ""
+    echo " Usage: $cmd [-tau_makefile=<tau_stub_makefile>]"
+    echo "             [-tau_options=<tau_compiler_opts>] <opts> <file>"
+    echo ""
+    echo " If -tau_makefile option is not used, "
+    echo " TAU uses the file specified in the TAU_MAKEFILE environment variable"
+    echo " e.g., "
+    echo " % $cmd -tau_makefile=/usr/local/tau-2.x/ia64/lib/Makefile.tau-mpi-pdt "
+    echo "        -tau_options=-optVerbose -c foo.c"
+    echo "    or"
+    echo " % setenv TAU_MAKEFILE /usr/local/tau-2.x/include/Makefile"
+    echo " % setenv TAU_OPTIONS -optVerbose -optTauSelectFile=select.tau"
+    echo " % $cmd -c foo.c"
+    echo ""
+    echo "TAU_OPTIONS:"
+    echo ""
+    echo -e "  -optVerbose\t\t\tTurn on verbose debugging message"
+    echo -e "  -optDetectMemoryLeaks\t\tTrack mallocs/frees using TAU's memory wrapper"
+    echo -e "  -optPdtGnuFortranParser\tSpecify the GNU gfortran PDT parser gfparse instead of f95parse"
+    echo -e "  -optPdtCleanscapeParser\tSpecify the Cleanscape Fortran parser"
+    echo -e "  -optTauSelectFile=\"\"\t\tSpecify selective instrumentation file for tau_instrumentor"
+    echo -e "  -optPreProcess\t\tPreprocess the source code before parsing. Uses /usr/bin/cpp -P by default."
+    echo -e "  -optKeepFiles\t\t\tDoes not remove intermediate .pdb and .inst.* files" 
+    echo -e "  -optShared\t\t\tUse shared library version of TAU."
+    echo -e "  -optCompInst\t\t\tUse compiler-based instrumentation."
+    echo -e "  -optPDTInst\t\t\tUse PDT-based instrumentation."
+    echo ""
+    exit 1
 fi
 
 TAUARGS=
@@ -92,7 +118,7 @@ if [ $makefile_specified = no ] ; then
 	    echo "ERROR: environment variable TAU_MAKEFILE is set but the file is not readable"
 	    exit 1
         fi
-    elif [ "x$DEFAULT_MAKEFILE=" != "x" ] ; then
+    elif [ "x$DEFAULT_MAKEFILE" != "x" ] ; then
 	MAKEFILE=$DEFAULT_MAKEFILE
     else
 	echo $0: "ERROR: please set the environment variable TAU_MAKEFILE"
@@ -107,12 +133,14 @@ if [ $options_specified = no ] ; then
     fi
 fi
 
+TAUCOMPILER_OPTIONS="$TAUCOMPILER_OPTIONS -optDefaultParser=cparse -optTau=-c"
+
 if [ $invoke_without_tau = yes ] ; then
 cat <<EOF > /tmp/makefile.tau.$USER.$$
-  include $MAKEFILE
-  all:
-	@\$(TAU_RUN_CC) $NON_TAUARGS
-  show:
+include $MAKEFILE
+all:
+	@\$(TAU_RUN_CC) \$(TAU_MPI_INCLUDE) $NON_TAUARGS
+show:
 	@echo \$(TAU_RUN_CC) \$(TAU_MPI_FLIBS) \$(TAU_LIBS) \$(TAU_LDFLAGS) \$(TAU_CXXLIBS)
 EOF
 make -s -f /tmp/makefile.tau.$USER.$$ $SHOW
