@@ -6,10 +6,7 @@
  */
 package edu.uoregon.tau.vis;
 
-import java.awt.Color;
-import java.awt.GridBagConstraints;
-import java.awt.GridBagLayout;
-import java.awt.Insets;
+import java.awt.*;
 import java.awt.event.ActionEvent;
 import java.awt.event.ActionListener;
 import java.util.ArrayList;
@@ -18,22 +15,18 @@ import java.util.StringTokenizer;
 
 import javax.media.opengl.GL;
 import javax.media.opengl.GLAutoDrawable;
-import javax.swing.BorderFactory;
-import javax.swing.ButtonGroup;
-import javax.swing.JButton;
-import javax.swing.JCheckBox;
-import javax.swing.JLabel;
-import javax.swing.JPanel;
-import javax.swing.JRadioButton;
+import javax.swing.*;
+import javax.swing.event.ChangeEvent;
+import javax.swing.event.ChangeListener;
 
 import com.sun.opengl.util.GLUT;
 
 /**
  * Draws axes with labels.
  *
- * <P>CVS $Id: Axes.java,v 1.6 2007/02/01 03:43:10 amorris Exp $</P>
+ * <P>CVS $Id: Axes.java,v 1.10 2009/08/21 23:57:59 amorris Exp $</P>
  * @author  Alan Morris
- * @version $Revision: 1.6 $
+ * @version $Revision: 1.10 $
  */
 public class Axes implements Shape {
 
@@ -58,6 +51,7 @@ public class Axes implements Shape {
 
     private float stringSize = 3;
     private float labelSize = 8;
+    private float fontScale = 1200;
 
     private boolean enabled = true;
     private Color highlightColor = new Color(1, 0, 0);
@@ -69,18 +63,17 @@ public class Axes implements Shape {
 
     private Color textColor = Color.white;
     private Color majorColor = Color.white;
-    private Color minorColor = new Color(0.5f,0.5f,0.5f);
-
+    private Color minorColor = new Color(0.5f, 0.5f, 0.5f);
 
     // until I add a proper event interface, each component will just keep track of the old values
     // and check on each render if it needs to recreate it's display lists
-    
+
     // this is to keep track of the old reverseVideo value
     // I need to come up with a better way of tracking the settings
     // we have to know whether to recreate the display list or not
     private boolean oldReverseVideo;
     private boolean oldAntiAlias;
-    
+
     /**
      * Typesafe enum for axes orientation
      */
@@ -115,7 +108,6 @@ public class Axes implements Shape {
 
     }
 
-
     /**
      * Returns whether or not this <tt>Axes</tt> instance is visible.
      * @return whether or not this <tt>Axes</tt> instance is visible.
@@ -142,8 +134,7 @@ public class Axes implements Shape {
      * @param yStrings List of strings for the y axis
      * @param zStrings List of strings for the z axis
      */
-    public void setStrings(String xlabel, String ylabel, String zlabel, List xStrings, List yStrings,
-            List zStrings) {
+    public void setStrings(String xlabel, String ylabel, String zlabel, List xStrings, List yStrings, List zStrings) {
         this.xlabel = xlabel;
         this.ylabel = ylabel;
         this.zlabel = zlabel;
@@ -193,8 +184,9 @@ public class Axes implements Shape {
         this.xSize = xSize;
         this.ySize = ySize;
         this.zSize = zSize;
-        if (this.autoSkip)
+        if (this.autoSkip) {
             setAutoTickSkip();
+        }
         this.dirty = true;
     }
 
@@ -282,6 +274,22 @@ public class Axes implements Shape {
             }
         };
 
+        
+        final JSlider fontScaleSlider = new JSlider(0,4000,4600-(int)getFontScale());
+        fontScaleSlider.addChangeListener(new ChangeListener() {
+            public void stateChanged(ChangeEvent event) {
+                try {
+                    Axes.this.setFontScale(4600-(float)(fontScaleSlider.getValue()));
+                    visRenderer.redraw();
+                } catch (Exception e) {
+                    VisTools.handleException(e);
+                }
+            }
+        });    
+
+        
+
+        
         ButtonGroup group = new ButtonGroup();
 
         JRadioButton nw = new JRadioButton("NW", orientation == Orientation.NW);
@@ -306,21 +314,27 @@ public class Axes implements Shape {
         gbc.fill = GridBagConstraints.NONE;
         gbc.weightx = 0.2;
         gbc.weighty = 0.2;
-        VisTools.addCompItem(axesPanel, enabledCheckBox, gbc, 0, 0, 1, 3);
+        VisTools.addCompItem(axesPanel, enabledCheckBox, gbc, 0, 0, 2, 2);
         gbc.anchor = GridBagConstraints.CENTER;
         gbc.fill = GridBagConstraints.NONE;
-        VisTools.addCompItem(axesPanel, new JLabel("Orientation"), gbc, 1, 0, 2, 1);
+        VisTools.addCompItem(axesPanel, new JLabel("Orientation"), gbc, 2, 0, 2, 1);
 
         gbc.anchor = GridBagConstraints.EAST;
         gbc.fill = GridBagConstraints.NONE;
 
-        VisTools.addCompItem(axesPanel, nw, gbc, 1, 1, 1, 1);
-        VisTools.addCompItem(axesPanel, sw, gbc, 1, 2, 1, 1);
+        VisTools.addCompItem(axesPanel, nw, gbc, 2, 1, 1, 1);
+        VisTools.addCompItem(axesPanel, sw, gbc, 2, 2, 1, 1);
 
         gbc.anchor = GridBagConstraints.WEST;
-        VisTools.addCompItem(axesPanel, ne, gbc, 2, 1, 1, 1);
-        VisTools.addCompItem(axesPanel, se, gbc, 2, 2, 1, 1);
+        VisTools.addCompItem(axesPanel, ne, gbc, 3, 1, 1, 1);
+        VisTools.addCompItem(axesPanel, se, gbc, 3, 2, 1, 1);
 
+        
+        gbc.anchor = GridBagConstraints.CENTER;
+        VisTools.addCompItem(axesPanel, new JLabel("Font Size"),gbc,0,2,1,1);
+        gbc.fill = GridBagConstraints.HORIZONTAL;
+        VisTools.addCompItem(axesPanel, fontScaleSlider,gbc,0,3,1,1);
+        
         return axesPanel;
     }
 
@@ -366,19 +380,23 @@ public class Axes implements Shape {
             dirty = true;
         }
         oldReverseVideo = visRenderer.getReverseVideo();
-        
+
         if (oldAntiAlias != visRenderer.getAntiAliasedLines()) {
             dirty = true;
         }
         oldAntiAlias = visRenderer.getAntiAliasedLines();
+
         
-        if (!enabled)
+        if (!enabled) {
             return;
+        }
 
         GL gl = glDrawable.getGL();
 
         if (dirty || displayList == 0) {
-            displayList = gl.glGenLists(1);
+            if (displayList == 0) {
+                displayList = gl.glGenLists(1);
+            }
             gl.glNewList(displayList, GL.GL_COMPILE);
             privateRender(visRenderer);
             gl.glEndList();
@@ -387,8 +405,7 @@ public class Axes implements Shape {
         gl.glCallList(displayList);
     }
 
-    private void setTickSkipping(int xTicSkip, int yTicSkip, int zTicSkip, int xLabelSkip, int yLabelSkip,
-            int zLabelSkip) {
+    private void setTickSkipping(int xTicSkip, int yTicSkip, int zTicSkip, int xLabelSkip, int yLabelSkip, int zLabelSkip) {
 
         this.xTickSkip = xTicSkip;
         this.yTickSkip = yTicSkip;
@@ -400,6 +417,9 @@ public class Axes implements Shape {
     }
 
     private void setAutoTickSkip() {
+        if (xStrings == null) {
+            return;
+        }
         this.xLabelSkip = (int) (xStrings.size() / (xSize * 2));
         this.yLabelSkip = (int) (yStrings.size() / (ySize * 2));
         this.zLabelSkip = (int) (zStrings.size() / (zSize * 2));
@@ -423,7 +443,6 @@ public class Axes implements Shape {
         }
 
     }
-    
 
     private void applyMajorColor(VisRenderer visRenderer) {
         //gl.glColor3f(0.9f, 0.9f, 0.9f);
@@ -514,19 +533,20 @@ public class Axes implements Shape {
         int zOffset = 0;
 
         gl.glDisable(GL.GL_LIGHTING);
-        
+
         if (visRenderer.getAntiAliasedLines()) {
             gl.glEnable(GL.GL_LINE_SMOOTH);
-            gl.glEnable(GL.GL_BLEND);   
+            gl.glEnable(GL.GL_BLEND);
             gl.glBlendFunc(GL.GL_SRC_ALPHA, GL.GL_ONE_MINUS_SRC_ALPHA);
             gl.glHint(GL.GL_LINE_SMOOTH_HINT, GL.GL_NICEST);
         } else {
             gl.glDisable(GL.GL_LINE_SMOOTH);
-            gl.glDisable(GL.GL_BLEND);   
-        }                
-                
+            gl.glDisable(GL.GL_BLEND);
+        }
+
         gl.glLineWidth(1.0f);
 
+
         // grid for x-y plane
         drawGrid(visRenderer, numx, numy, xSize, ySize, this.xLabelSkip, this.yLabelSkip);
 
@@ -642,7 +662,6 @@ public class Axes implements Shape {
         gl.glPopMatrix();
 
         gl.glEnable(GL.GL_LIGHTING);
-
     }
 
     private void drawLabels(VisRenderer visRenderer, String label, List strings, float increment, int labelSkip, int tickSkip,
@@ -678,15 +697,17 @@ public class Axes implements Shape {
                 }
                 gl.glEnd();
 
+
+                
                 if (leftJustified) {
                     gl.glTranslatef(1.5f, 0.0f, 0.0f);
-                    gl.glScalef(stringSize / 1000, stringSize / 1000, stringSize / 1000);
+                    gl.glScalef(stringSize / fontScale, stringSize / fontScale, stringSize / fontScale);
                 } else {
                     gl.glTranslatef(-1.5f, 0.0f, 0.0f);
-                    gl.glScalef(stringSize / 1000, stringSize / 1000, stringSize / 1000);
+                    gl.glScalef(stringSize / fontScale, stringSize / fontScale, stringSize / fontScale);
                     gl.glTranslated(-width, 0.0, 0.0);
                 }
-                
+
                 // keep track of the widest width to determine where to draw the label for this axis
                 maxPoint = Math.max(maxPoint, width);
 
@@ -728,7 +749,8 @@ public class Axes implements Shape {
         if (leftJustified) {
             gl.glTranslated(maxPoint + 3.0f, -increment * (strings.size() + 1) / 2, 0);
             gl.glRotatef(90, 0, 0, 1);
-            gl.glScalef(labelSize / 1000, labelSize / 1000, labelSize / 1000);
+            gl.glScalef(labelSize / fontScale, labelSize / fontScale, labelSize / fontScale);
+
 
             StringTokenizer st = new StringTokenizer(label, "\n");
             while (st.hasMoreTokens()) {
@@ -749,8 +771,8 @@ public class Axes implements Shape {
         } else {
             gl.glTranslated(-(maxPoint + 3.0f), -increment * (strings.size() + 1) / 2, 0);
             gl.glRotatef(-90, 0, 0, 1);
-
-            gl.glScalef(labelSize / 1000, labelSize / 1000, labelSize / 1000);
+            gl.glScalef(labelSize / fontScale, labelSize / fontScale, labelSize / fontScale);
+           
 
             StringTokenizer st = new StringTokenizer(label, "\n");
             while (st.hasMoreTokens()) {
@@ -773,9 +795,6 @@ public class Axes implements Shape {
         gl.glPopMatrix();
 
     }
-    
-    
-    
 
     private void renderStrokeString(GL gl, int font, String string) {
         // Center Our Text On The Screen
@@ -859,4 +878,18 @@ public class Axes implements Shape {
         this.textColor = textColor;
         this.dirty = true;
     }
+
+    public float getFontScale() {
+        return fontScale;
+    }
+
+    public void setFontScale(float fontScale) {
+        this.fontScale = fontScale;
+        this.dirty = true;
+    }
+
+    public void resetCanvas() {
+        dirty = true;
+        displayList = 0;
+    }
 }
