@@ -30,16 +30,12 @@ import javax.swing.event.ChangeListener;
 
 import com.sun.opengl.util.GLUT;
 
-
-
-
-
 /**
  * Draws a 3d bar plot.
  *
- * <P>CVS $Id: BarPlot.java,v 1.5 2007/02/01 03:43:10 amorris Exp $</P>
+ * <P>CVS $Id: BarPlot.java,v 1.9 2009/08/21 23:58:00 amorris Exp $</P>
  * @author	Alan Morris
- * @version	$Revision: 1.5 $
+ * @version	$Revision: 1.9 $
  */
 public class BarPlot implements Plot {
 
@@ -73,15 +69,23 @@ public class BarPlot implements Plot {
     private int translucentDisplayListsXsize;
     private int translucentDisplayListsYsize;
 
+    private float scaleZ = 1.0f;
+
     private GL gl;
+    
+    float maxHeightValue;
+    float maxColorValue;
 
     /**
-     * The BarPlot constructor is empty.  Use initialize to set initial values.  
-     * This is done so that the object may be easily reused with reseting everthing.
-     * @see #initialize(Axes, float, float, float, float[][], float[][], ColorScale)
+     * Create a BarPlot with a given Axes and ColorScale
      */
-    public BarPlot() {
+    public BarPlot(Axes axes, ColorScale colorScale) {
+        this.axes = axes;
+        setColorScale(colorScale);
+    }
 
+    public String getName() {
+        return "Bar Plot";
     }
 
     /**
@@ -94,29 +98,6 @@ public class BarPlot implements Plot {
      * @param colorValues the color values to use.
      * @param colorScale ColorScale to use for this plot.
      */
-    public void initialize(Axes axes, float xSize, float ySize, float zSize, float heightValues[][],
-            float colorValues[][], ColorScale colorScale) {
-
-        setColorScale(colorScale);
-        this.nrows = heightValues.length;
-        this.ncols = heightValues[0].length;
-        this.heightValues = heightValues;
-        this.colorValues = colorValues;
-
-        this.xSize = xSize;
-        this.ySize = ySize;
-        this.zSize = zSize;
-
-        this.axes = axes;
-        axes.setSize(xSize, ySize, zSize);
-
-        processValues();
-    }
-
-    public String getName() {
-        return "Bar Plot";
-    }
-
     public void setValues(float xSize, float ySize, float zSize, float heightValues[][], float colorValues[][]) {
         this.nrows = heightValues.length;
         if (heightValues.length > 0) {
@@ -135,6 +116,29 @@ public class BarPlot implements Plot {
         this.dirty = true;
     }
 
+    /**
+     * Initializes this BarPlot with the given values.
+     * @param axes Axes to use for this plot.
+     * @param xSize size in x direction.
+     * @param ySize size in y direction.
+     * @param zSize size in z direction.
+     * @param heightValues the height values to use.
+     * @param colorValues the color values to use.
+     * @param colorScale ColorScale to use for this plot.
+     */
+    public void setValues(float heightValues[][], float colorValues[][]) {
+        this.nrows = heightValues.length;
+        if (heightValues.length > 0) {
+            this.ncols = heightValues[0].length;
+        } else {
+            this.ncols = 0;
+        }
+        this.heightValues = heightValues;
+        this.colorValues = colorValues;
+        processValues();
+        this.dirty = true;
+    }
+
     /**
      * Returns the current barSize.
      * @return the current barSize (0..1).
@@ -158,8 +162,8 @@ public class BarPlot implements Plot {
     }
 
     private void processValues() {
-        float maxHeightValue = Float.MIN_VALUE;
-        float maxColorValue = Float.MIN_VALUE;
+        maxHeightValue = -Float.MAX_VALUE;
+        maxColorValue = -Float.MAX_VALUE;
         for (int y = 0; y < nrows; y++) {
             for (int x = 0; x < ncols; x++) {
                 float heightValue = heightValues[y][x];
@@ -169,15 +173,38 @@ public class BarPlot implements Plot {
             }
         }
 
-        for (int y = 0; y < nrows; y++) {
-            for (int x = 0; x < ncols; x++) {
-                float heightValue = heightValues[y][x];
-                float colorValue = colorValues[y][x];
-                heightValues[y][x] = heightValue / maxHeightValue * zSize;
-                colorValues[y][x] = colorValue / maxColorValue;
-            }
-        }
+//        for (int y = 0; y < nrows; y++) {
+//            for (int x = 0; x < ncols; x++) {
+//                float heightValue = heightValues[y][x];
+//                float colorValue = colorValues[y][x];
+//                heightValues[y][x] = heightValue / maxHeightValue * zSize;
+//                if (maxHeightValue == 0 ) {
+//                    heightValues[y][x] = 0.0f;
+//                }
+//                colorValues[y][x] = colorValue / maxColorValue;
+//                if (maxColorValue == 0) {
+//                    colorValues[y][x] = 1.0f;
+//                }
+//            }
+//        }
     }
+    
+    
+    private float getHeight(int x, int y) {
+        if (maxHeightValue == 0) {
+            return 0;
+        }
+        return heightValues[y][x] / maxHeightValue * zSize; 
+    }
+
+    private float getColor(int x, int y) {
+        if (maxColorValue == 0) {
+            return 0;
+        }
+        return colorValues[y][x] / maxColorValue; 
+    }
+
+    
 
     public void setSize(float xSize, float ySize, float zSize) {
         this.xSize = xSize;
@@ -249,16 +276,14 @@ public class BarPlot implements Plot {
                 float xPosition = x * xIncrement;
                 float yPosition = y * yIncrement;
 
-                Color color = colorScale.getColor(colorValues[y][x]);
+                Color color = colorScale.getColor(getColor(x,y));
 
-                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f,
-                        translucency);
+                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f, translucency);
 
                 Vec min = new Vec(xPosition, yPosition, 0);
-                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                        heightValues[y][x]);
+                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
-                doBox(min, max, heightValues[y][x]);
+                doBox(min, max, getHeight(x,y));
             }
             gl.glEnd();
             gl.glEndList();
@@ -270,16 +295,14 @@ public class BarPlot implements Plot {
                 float xPosition = x * xIncrement;
                 float yPosition = y * yIncrement;
 
-                Color color = colorScale.getColor(colorValues[y][x]);
+                Color color = colorScale.getColor(getColor(x,y));
 
-                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f,
-                        translucency);
+                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f, translucency);
 
                 Vec min = new Vec(xPosition, yPosition, 0);
-                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                        heightValues[y][x]);
+                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
-                doBox(min, max, heightValues[y][x]);
+                doBox(min, max, getHeight(x,y));
             }
             gl.glEnd();
             gl.glEndList();
@@ -294,16 +317,14 @@ public class BarPlot implements Plot {
                 float xPosition = x * xIncrement;
                 float yPosition = y * yIncrement;
 
-                Color color = colorScale.getColor(colorValues[y][x]);
+                Color color = colorScale.getColor(getColor(x,y));
 
-                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f,
-                        translucency);
+                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f, translucency);
 
                 Vec min = new Vec(xPosition, yPosition, 0);
-                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                        heightValues[y][x]);
+                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
-                doBox(min, max, heightValues[y][x]);
+                doBox(min, max, getHeight(x,y));
             }
             gl.glEnd();
             gl.glEndList();
@@ -315,16 +336,14 @@ public class BarPlot implements Plot {
                 float xPosition = x * xIncrement;
                 float yPosition = y * yIncrement;
 
-                Color color = colorScale.getColor(colorValues[y][x]);
+                Color color = colorScale.getColor(getColor(x,y));
 
-                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f,
-                        translucency);
+                gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f, translucency);
 
                 Vec min = new Vec(xPosition, yPosition, 0);
-                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                        heightValues[y][x]);
+                Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
-                doBox(min, max, heightValues[y][x]);
+                doBox(min, max, getHeight(x,y));
             }
             gl.glEnd();
             gl.glEndList();
@@ -385,8 +404,6 @@ public class BarPlot implements Plot {
 
     }
 
-   
-
     public void cleanUp() {
         VisTools.vout(this, "Cleaning up!");
         // delete displaylists
@@ -394,11 +411,13 @@ public class BarPlot implements Plot {
             for (int i = 0; i < displayLists.size(); i++) {
                 gl.glDeleteLists(((Integer) displayLists.get(i)).intValue(), 1);
             }
-            displayLists = new ArrayList();
+            displayLists.clear();
+            displayLists = null;
         }
     }
 
     private void renderOpaque(GLAutoDrawable glDrawable) {
+
         GL gl = glDrawable.getGL();
 
         if (dirty || displayLists == null) {
@@ -421,17 +440,23 @@ public class BarPlot implements Plot {
             gl.glEnable(GL.GL_LIGHTING);
             gl.glFrontFace(GL.GL_CCW);
             gl.glEnable(GL.GL_CULL_FACE);
-            gl.glShadeModel(GL.GL_FLAT);
+            //gl.glShadeModel(GL.GL_FLAT);
+            gl.glShadeModel(GL.GL_SMOOTH);
             gl.glEnable(GL.GL_DEPTH_TEST);
             gl.glDisable(GL.GL_BLEND);
             gl.glBlendFunc(GL.GL_ONE, GL.GL_ZERO);
 
+            // Experimental
+            //gl.glDisable(GL.GL_FOG);
+
+            //gl.glEnable(GL.GL_FOG);
+
             float xIncrement = xSize / (ncols + 1);
             float yIncrement = ySize / (nrows + 1);
 
             gl.glPushMatrix();
-            gl.glTranslatef(xIncrement - (xIncrement * barSize / 2), yIncrement - (yIncrement * barSize / 2),
-                    0.05f);
+            gl.glTranslatef(xIncrement - (xIncrement * barSize / 2), yIncrement - (yIncrement * barSize / 2), 0.05f);
+            gl.glDepthFunc(GL.GL_LESS);
 
             gl.glBegin(GL.GL_QUADS);
 
@@ -451,7 +476,7 @@ public class BarPlot implements Plot {
                     float xPosition = xValue * xIncrement;
                     float yPosition = yValue * yIncrement;
 
-                    Color color = colorScale.getColor(colorValues[yValue][xValue]);
+                    Color color = colorScale.getColor(getColor(xValue,yValue));
 
                     float r = color.getRed() / 255.0f;
                     float g = color.getGreen() / 255.0f;
@@ -461,8 +486,7 @@ public class BarPlot implements Plot {
                             && VisTools.isSufficientlyEqual(b, lastBlue)) {
                         colorsSaved++;
                     } else {
-                        gl.glColor3f(color.getRed() / 255.0f, color.getGreen() / 255.0f,
-                                color.getBlue() / 255.0f);
+                        gl.glColor3f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f);
                         lastRed = r;
                         lastBlue = b;
                         lastGreen = g;
@@ -470,9 +494,9 @@ public class BarPlot implements Plot {
                     }
                     Vec min = new Vec(xPosition, yPosition, 0);
                     Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                            heightValues[yValue][xValue]);
+                            getHeight(xValue,yValue));
 
-                    if (heightValues[yValue][xValue] >= threshold) {
+                    if (getHeight(xValue,yValue) >= threshold) {
 
                         // top
                         gl.glNormal3f(0, 0, 1);
@@ -589,12 +613,18 @@ public class BarPlot implements Plot {
 
         this.gl = gl;
 
+        gl.glPushMatrix();
+
+        gl.glScalef(1.0f, 1.0f, scaleZ);
+
         if (translucent) {
             renderTranslucent(glDrawable, direction);
         } else {
             renderOpaque(glDrawable);
         }
 
+        gl.glPopMatrix();
+
         dirty = false;
 
     }
@@ -627,14 +657,13 @@ public class BarPlot implements Plot {
             float xPosition = x * xIncrement;
             float yPosition = y * yIncrement;
 
-            Color color = colorScale.getColor(colorValues[y][x]);
+            Color color = colorScale.getColor(getColor(x,y));
 
-            gl.glColor4f(color.getRed() / 255.0f + 0.25f, color.getGreen() / 255.0f + 0.25f,
-                    color.getBlue() / 255.0f + 0.25f, 1.0f);
+            gl.glColor4f(color.getRed() / 255.0f + 0.25f, color.getGreen() / 255.0f + 0.25f, color.getBlue() / 255.0f + 0.25f,
+                    1.0f);
 
             Vec min = new Vec(xPosition, yPosition, 0);
-            Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                    heightValues[y][x]);
+            Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
             // top
             gl.glNormal3f(0, 0, 1);
@@ -686,15 +715,14 @@ public class BarPlot implements Plot {
             float xPosition = x * xIncrement;
             float yPosition = y * yIncrement;
 
-            Color color = colorScale.getColor(colorValues[y][x]);
+            Color color = colorScale.getColor(getColor(x,y));
 
             //            gl.glColor4f(color.getRed() / 255.0f, color.getGreen() / 255.0f, color.getBlue() / 255.0f, 1.0f);
-            gl.glColor4f(color.getRed() / 255.0f + 0.25f, color.getGreen() / 255.0f + 0.25f,
-                    color.getBlue() / 255.0f + 0.25f, 1.0f);
+            gl.glColor4f(color.getRed() / 255.0f + 0.25f, color.getGreen() / 255.0f + 0.25f, color.getBlue() / 255.0f + 0.25f,
+                    1.0f);
 
             Vec min = new Vec(xPosition, yPosition, 0);
-            Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize,
-                    heightValues[y][x]);
+            Vec max = new Vec(xPosition + xIncrement * barSize, yPosition + yIncrement * barSize, getHeight(x,y));
 
             // front
             gl.glNormal3f(0, 0, 1);
@@ -766,7 +794,7 @@ public class BarPlot implements Plot {
         gl.glDepthFunc(GL.GL_LEQUAL);
 
         gl.glDisable(GL.GL_CULL_FACE);
-        float height = heightValues[selectedRow][selectedCol];
+        float height = getHeight(selectedCol,selectedRow);
 
         gl.glTranslatef(0, 0, 0.055f);
 
@@ -839,8 +867,7 @@ public class BarPlot implements Plot {
         ChangeListener chageListener = new ChangeListener() {
             public void stateChanged(ChangeEvent event) {
                 try {
-                    BarPlot.this.setSize(plotWidthSlider.getValue(), plotDepthSlider.getValue(),
-                            plotHeightSlider.getValue());
+                    BarPlot.this.setSize(plotWidthSlider.getValue(), plotDepthSlider.getValue(), plotHeightSlider.getValue());
                     visRenderer.redraw();
                 } catch (Exception e) {
                     VisTools.handleException(e);
@@ -987,4 +1014,21 @@ public class BarPlot implements Plot {
         this.translucency = translucency;
         this.dirty = true;
     }
+
+    public float getScaleZ() {
+        return scaleZ;
+    }
+
+    public void setScaleZ(float scaleZ) {
+        this.scaleZ = scaleZ;
+    }
+
+    public void resetCanvas() {
+        dirty = true;
+        displayLists.clear();
+        displayLists = null;
+        if (axes != null) {
+            axes.resetCanvas();
+        }
+    }
 }
