@@ -26,14 +26,16 @@ import com.sun.opengl.util.BufferUtil;
 /**
  * This object manages the JOGL interface.
  *    
- * <P>CVS $Id: VisRenderer.java,v 1.8 2007/05/11 01:54:17 amorris Exp $</P>
+ * <P>CVS $Id: VisRenderer.java,v 1.11 2009/08/20 23:11:24 amorris Exp $</P>
  * @author	Alan Morris
- * @version	$Revision: 1.8 $
+ * @version	$Revision: 1.11 $
  */
 public class VisRenderer implements GLEventListener, MouseListener, MouseMotionListener, MouseWheelListener {
 
     private class VisAnimator extends Thread {
 
+        private volatile boolean stop = false;
+
         public void run() {
             stop = false;
             while (!stop) {
@@ -49,45 +51,42 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
             }
         }
 
-        private volatile boolean stop = false;
-
         public void end() {
             stop = true;
         }
-
     }
 
-    private int prevMouseX, prevMouseY;
-    private boolean mouseRButtonDown = false;
-
+    // OpenGL objects
     private GL gl;
     private GLU glu;
     private GLAutoDrawable glDrawable;
 
+    // conversion to/from degress/radians
+    private final static float DTOR = 0.0174532925f;
+    private final static float RTOD = 57.2957795f;
+
+    // sensitivity settings
+    private final static float lateralSense = 1 * DTOR;
+    private final static float verticalSense = 1 * DTOR;
+
+    // camera information
+    private double viewAltitude = -30 * DTOR; // The angle from the x-y plane that the eye is placed 
+    private double viewAzimuth = -135 * DTOR; // The angle on the x-y plane that the eye is placed
+    private double viewDistance = 50.0; // The distance from the eye to the aim
     private Vec eye; // The location of the eye
     private Vec aim = new Vec(0, 0, 0); // Where the eye is focused at
     private Vec vup; // The canonical V-up vector
-
-    private Vec viewDirection;
-
-    final static private float rad = (float) (3.14 / 180);
-    final static private float lateralSense = 1 * rad;
-    final static private float verticalSense = 1 * rad;
-
-    private double viewAltitude = -30 * rad; // The angle from the x-y plane that the eye is placed 
-    private double viewAzimuth = -135 * rad; // The angle on the x-y plane that the eye is placed
-    private double viewDistance = 50.0; // The distance from the eye to the aim
-
-    private boolean reverseVideo = false;
-
-    private Color backColor = Color.white;
-    private Color foreColor = Color.black;
+    private float camera_aperture = 45.0f;
+    private float camera_near = 1.0f;
+    private float camera_far = 500.0f;
+    private float camera_focallength = 50.0f;
 
     private List shapes = new ArrayList(); // The list of shapes to draw
 
     private float fps; // Frames per Second
     private int framesRendered;
 
+    // dimensionality of the viewing frame (pixels on the screen)
     private int width, height;
 
     // for screenshot capability
@@ -98,26 +97,41 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
     private VisAnimator visAnimator;
     private volatile float rotateSpeed = 0.5f;
 
-    private boolean antiAliasedLines = false;
-
+    // GL info
     private String glInfo_Vendor;
     private String glInfo_Renderer;
     private String glInfo_Version;
     private boolean stereo_available;
+
+    // settings
     private boolean stereo;
-
     private JCheckBox stereoCheckBox;
+    private boolean antiAliasedLines;
+    private boolean fsaa;
+    private boolean reverseVideo;
 
-    private float camera_aperture = 45.0f;
-    private float camera_near = 1.0f;
-    private float camera_far = 500.0f;
-    private float camera_focallength = 50.0f;
+    private int prevMouseX, prevMouseY;
+    private boolean mouseRButtonDown;
 
-    private float DTOR = 0.0174532925f;
-    private float RTOD = 57.2957795f;
+    public static final int CAMERA_PLOT = 0;
+    public static final int CAMERA_STICK = 1;
+
+    private int cameraMode = CAMERA_PLOT;
+
+    private VisCanvasListener visCanvasListener;
 
     public VisRenderer() {}
 
+    /**
+     * Set the handler for VisCanvas events.  When a canvas change is needed, 
+     * for example when FSAA is set/unset, the parent needs to place the new canvas
+     * into their JFrame.
+     * @param visCanvasListener
+     */
+    public void setVisCanvasListener(VisCanvasListener visCanvasListener) {
+        this.visCanvasListener = visCanvasListener;
+    }
+
     /**
      * Add a shape to the list of shapes to be drawn
      * @param shape		the shape to add
@@ -135,40 +149,49 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
     }
 
     private void setLighting() {
+        float lightPosition[] = { 0.0f, 0.0f, 1.0f, 0.0f };
 
         //float lightPosition[] = { 0.7f, 1.0f, 0.6f, 0.0f };
-        float lightPosition[] = { 0.0f, 0.0f, 1.0f, 0.0f };
         //float lightPosition[] = { 5.7f, 5.0f, 5.6f, 1.0f };
+        //float lightPosition[] = { -1.0f, 0.0f, 1.0f, 0.0f };
 
-        //        float lightPosition[] = { 0f, 10.0f, 10.0f, 1.0f };
+        //float lightPosition[] = { 0f, 10.0f, 10.0f, 1.0f };
 
-        //      float lightPosition2[] = { 0.7f, -1.0f, 0.6f, 0.0f };
+        //float lightPosition2[] = { 0.7f, -1.0f, 0.6f, 0.0f };
         float whiteLight[] = { 0.75f, 0.75f, 0.75f, 1.0f };
         //float ambientLight[] = { 0.15f, 0.15f, 0.15f, 1.0f };
         float ambientLight[] = { 0.15f, 0.15f, 0.15f, 1.0f };
 
-        //      float mat_shininess[] = { 50.0f };
-        //    float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
-
         gl.glLightfv(GL.GL_LIGHT0, GL.GL_POSITION, lightPosition, 0);
         gl.glLightfv(GL.GL_LIGHT0, GL.GL_DIFFUSE, whiteLight, 0);
-        //        gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, whiteLight);
+        //gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, whiteLight, 0);
         gl.glLightfv(GL.GL_LIGHT0, GL.GL_AMBIENT, ambientLight, 0);
 
-        //        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess);
-        //        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular);
+        //        float mat_shininess[] = { 50.0f };
+        //        float mat_specular[] = { 1.0f, 1.0f, 1.0f, 1.0f };
+        //        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SHININESS, mat_shininess, 0);
+        //        gl.glMaterialfv(GL.GL_FRONT, GL.GL_SPECULAR, mat_specular, 0);
 
-        //    gl.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, lightPosition2);
-        //    gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, whiteLight);
+        //gl.glLightfv(GL.GL_LIGHT1, GL.GL_POSITION, lightPosition2);
+        //gl.glLightfv(GL.GL_LIGHT1, GL.GL_DIFFUSE, whiteLight);
         //gl.glLightfv(GL.GL_LIGHT0, GL.GL_SPECULAR, whiteLight);
-        //   gl.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, ambientLight);
+        //gl.glLightfv(GL.GL_LIGHT1, GL.GL_AMBIENT, ambientLight);
 
         gl.glLightModelfv(GL.GL_LIGHT_MODEL_AMBIENT, ambientLight, 0);
         gl.glEnable(GL.GL_COLOR_MATERIAL);
         gl.glShadeModel(GL.GL_FLAT);
         gl.glEnable(GL.GL_LIGHTING);
         gl.glEnable(GL.GL_LIGHT0);
-        //        gl.glEnable(GL.GL_LIGHT1);
+
+        //        float fogColor[] = {0.5f, 0.5f, 0.5f, 1.0f}; //set the for color to grey
+        //        float density = 0.015f;
+        //        gl.glEnable(GL.GL_FOG);
+        //        gl.glFogi (GL.GL_FOG_MODE, GL.GL_EXP2); //set the fog mode to GL_EXP2
+        //        gl.glFogfv (GL.GL_FOG_COLOR, fogColor, 0); //set the fog color to our color chosen above
+        //        gl.glFogf (GL.GL_FOG_DENSITY, density); //set the density to the value above
+        //        gl.glHint (GL.GL_FOG_HINT, GL.GL_DONT_CARE); // set the fog to look the nicest, may slow down on older cards
+
+        //gl.glEnable(GL.GL_LIGHT1);
         //gl.glEnable(GL.GL_BLEND);
 
     }
@@ -254,6 +277,11 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         eye.sety(eye.y() * viewDistance);
         eye.setz(eye.z() * viewDistance);
 
+        if (cameraMode == CAMERA_STICK) {
+            aim.setx(0);
+            aim.sety(0);
+        }
+
         eye = eye.add(aim);
     }
 
@@ -266,28 +294,34 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         }
     }
 
+    /**
+     * Rotate the camera
+     */
     private void rotate(float x, float y) {
         final float limit = 90.0f;
 
         viewAltitude += verticalSense * y;
-        if (viewAltitude < -rad * limit)
-            viewAltitude = -rad * limit;
-        if (viewAltitude > rad * limit)
-            viewAltitude = rad * limit;
+        if (viewAltitude < -DTOR * limit)
+            viewAltitude = -DTOR * limit;
+        if (viewAltitude > DTOR * limit)
+            viewAltitude = DTOR * limit;
 
         viewAzimuth += lateralSense * x;
-        if (viewAzimuth >= 2 * 3.14f)
-            viewAzimuth -= 2 * 3.14f;
-        if (viewAzimuth >= 2 * 3.14f)
-            viewAzimuth -= 2 * 3.14f;
+        if (viewAzimuth >= 2 * Math.PI)
+            viewAzimuth -= 2 * Math.PI;
+        if (viewAzimuth >= 2 * Math.PI)
+            viewAzimuth -= 2 * Math.PI;
 
         computeEye();
         redraw();
     }
 
+    /**
+     * Translate the camera
+     */
     private void translate(float x, float y) {
 
-        double oldViewAngle = viewAltitude;
+        double oldViewAltitude = viewAltitude;
         float oldz = aim.z();
 
         viewAltitude = 45.0;
@@ -310,8 +344,6 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
 
         Matrix M = rotate.multiply(translate);
 
-        Vec tmp2 = M.transform(aim);
-
         Vec diff = eye.subtract(aim);
 
         double scaleFactor = Math.sqrt(diff.length()) / 50;
@@ -329,13 +361,19 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         aim = M.transform(aim);
 
         aim.setz(oldz);
-        viewAltitude = oldViewAngle;
+        viewAltitude = oldViewAltitude;
 
         computeEye();
 
         redraw();
     }
 
+    private void translateStick(float x, float y) {
+        aim.setz(aim.z() - (y / 3));
+        computeEye();
+        redraw();
+    }
+
     /**
      * Redraws the shapes
      */
@@ -345,6 +383,16 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         }
     }
 
+    /**
+     * Check if we are ready to draw
+     */
+    public boolean isReadyToDraw() {
+        if (glDrawable != null) {
+            return true;
+        }
+        return false;
+    }
+
     /**
      * Creates a screenshot of the current display
      * @return	the screenshot
@@ -477,8 +525,6 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
                 glu.gluLookAt(aim.x(), aim.y(), aim.z(), eye.x(), eye.y(), eye.z(), vup.x(), vup.y(), vup.z());
             }
 
-            viewDirection = eye.subtract(aim);
-
             for (int i = 0; i < shapes.size(); i++) {
                 Shape shape = (Shape) shapes.get(i);
                 shape.render(this);
@@ -538,10 +584,8 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
                 int iR = pixelsRGB.get(q++);
                 int iG = pixelsRGB.get(q++);
                 int iB = pixelsRGB.get(q++);
-
                 pixelInts[i++] = 0xFF000000 | ((iR & 0x000000FF) << 16) | ((iG & 0x000000FF) << 8) | (iB & 0x000000FF);
             }
-
         }
 
         screenShot = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
@@ -550,11 +594,15 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
 
     public void displayChanged(GLAutoDrawable drawable, boolean modeChanged, boolean deviceChanged) {}
 
-    // Methods required for the implementation of MouseListener
+    // MouseListener implementation
     public void mouseEntered(MouseEvent e) {}
 
     public void mouseExited(MouseEvent e) {}
 
+    public void mouseClicked(MouseEvent e) {}
+
+    public void mouseMoved(MouseEvent e) {}
+
     public void mousePressed(MouseEvent e) {
         prevMouseX = e.getX();
         prevMouseY = e.getY();
@@ -569,9 +617,6 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         }
     }
 
-    public void mouseClicked(MouseEvent e) {}
-
-    // Methods required for the implementation of MouseMotionListener
     public void mouseDragged(MouseEvent e) {
         int x = e.getX();
         int y = e.getY();
@@ -585,7 +630,11 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         dy = y - prevMouseY;
 
         if (mouseRButtonDown) {
-            translate(-dx, -dy);
+            if (cameraMode == CAMERA_PLOT) {
+                translate(-dx, -dy);
+            } else {
+                translateStick(-dx, -dy);
+            }
         } else {
             rotate(-dx, dy);
         }
@@ -594,8 +643,6 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         prevMouseY = y;
     }
 
-    public void mouseMoved(MouseEvent e) {}
-
     // Zoom in and out with the mouse wheel
     public void mouseWheelMoved(MouseWheelEvent e) {
         int scrollAmount = e.getWheelRotation();
@@ -742,7 +789,7 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
             }
         });
 
-        final JCheckBox antialiasCheckBox = new JCheckBox("Anti-Aliased Lines", antiAliasedLines);
+        final JCheckBox antialiasCheckBox = new JCheckBox("AA Lines", antiAliasedLines);
         antialiasCheckBox.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent evt) {
                 try {
@@ -753,6 +800,17 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
             }
         });
 
+        final JCheckBox fsaaCheckBox = new JCheckBox("Full Screen AA", fsaa);
+        fsaaCheckBox.addActionListener(new ActionListener() {
+            public void actionPerformed(ActionEvent evt) {
+                try {
+                    setFSAA(fsaaCheckBox.isSelected());
+                } catch (Exception e) {
+                    VisTools.handleException(e);
+                }
+            }
+        });
+
         stereoCheckBox = new JCheckBox("Stereo", stereo);
         stereoCheckBox.addActionListener(new ActionListener() {
             public void actionPerformed(ActionEvent evt) {
@@ -819,22 +877,23 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         gbc.fill = GridBagConstraints.HORIZONTAL;
         VisTools.addCompItem(panel, speedSlider, gbc, 2, 1, 1, 1);
 
-        VisTools.addCompItem(panel, reverseCheckBox, gbc, 0, 2, 3, 1);
-        VisTools.addCompItem(panel, antialiasCheckBox, gbc, 0, 3, 3, 1);
-        VisTools.addCompItem(panel, stereoCheckBox, gbc, 0, 4, 3, 1);
+        VisTools.addCompItem(panel, reverseCheckBox, gbc, 0, 2, 2, 1);
+        VisTools.addCompItem(panel, stereoCheckBox, gbc, 1, 2, 2, 1);
+        VisTools.addCompItem(panel, antialiasCheckBox, gbc, 0, 3, 2, 1);
+        VisTools.addCompItem(panel, fsaaCheckBox, gbc, 1, 3, 3, 1);
         gbc.weightx = 0.1;
-        VisTools.addCompItem(panel, new JLabel("Separation"), gbc, 0, 5, 1, 1);
-        VisTools.addCompItem(panel, new JLabel("Aperture"), gbc, 0, 6, 1, 1);
+        VisTools.addCompItem(panel, new JLabel("Separation"), gbc, 0, 4, 1, 1);
+        VisTools.addCompItem(panel, new JLabel("Aperture"), gbc, 0, 5, 1, 1);
         gbc.fill = GridBagConstraints.BOTH;
         gbc.weightx = 0.9;
 
         gbc.anchor = GridBagConstraints.CENTER;
-        VisTools.addCompItem(panel, focalLengthSlider, gbc, 1, 5, 2, 1);
-        VisTools.addCompItem(panel, apertureSlider, gbc, 1, 6, 2, 1);
+        VisTools.addCompItem(panel, focalLengthSlider, gbc, 1, 4, 2, 1);
+        VisTools.addCompItem(panel, apertureSlider, gbc, 1, 5, 2, 1);
 
         gbc.fill = GridBagConstraints.NONE;
         gbc.anchor = GridBagConstraints.CENTER;
-        VisTools.addCompItem(panel, glInfoButton, gbc, 0, 7, 3, 1);
+        VisTools.addCompItem(panel, glInfoButton, gbc, 0, 6, 3, 1);
 
         return panel;
     }
@@ -870,7 +929,7 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
      * @return the direction of the camera.
      */
     public Vec getViewDirection() {
-        return viewDirection;
+        return eye.subtract(aim);
     }
 
     public boolean getAntiAliasedLines() {
@@ -909,4 +968,34 @@ public class VisRenderer implements GLEventListener, MouseListener, MouseMotionL
         this.redraw();
     }
 
+    public int getCameraMode() {
+        return cameraMode;
+    }
+
+    public void setCameraMode(int cameraMode) {
+        this.cameraMode = cameraMode;
+        computeEye();
+    }
+
+    public boolean getFSAA() {
+        return fsaa;
+    }
+
+    public void setFSAA(boolean fsaa) {
+        if (fsaa != this.fsaa) {
+            this.fsaa = fsaa;
+            
+            
+            if (visCanvasListener != null) {
+                visCanvasListener.createNewCanvas();
+            }
+            
+            for (int i = 0; i < shapes.size(); i++) {
+                Shape shape = (Shape) shapes.get(i);
+                shape.resetCanvas();
+            }
+            redraw();
+        }
+    }
+
 }
\ No newline at end of file
