@@ -6,10 +6,10 @@
 #include <string>
 using std::string;
 
-enum instrumentKind_t { TAU_LOOPS, TAU_LINE, TAU_ROUTINE_ENTRY, TAU_ROUTINE_EXIT, TAU_NOT_SPECIFIED, TAU_IO, TAU_MEMORY, TAU_TIMER, TAU_PHASE};
+enum instrumentKind_t { TAU_LOOPS, TAU_LINE, TAU_ROUTINE_DECL, TAU_ROUTINE_ENTRY, TAU_ROUTINE_EXIT, TAU_ABORT, TAU_NOT_SPECIFIED, TAU_IO, TAU_MEMORY, TAU_TIMER, TAU_PHASE, TAU_INIT };
 
 /* For C instrumentation */
-enum itemKind_t { ROUTINE, BODY_BEGIN, FIRST_EXECSTMT, BODY_END, RETURN, EXIT, INSTRUMENTATION_POINT, START_TIMER, STOP_TIMER, START_DO_TIMER, GOTO_STOP_TIMER, START_LOOP_TIMER, STOP_LOOP_TIMER, ALLOCATE_STMT, DEALLOCATE_STMT, IO_STMT};
+enum itemKind_t { ROUTINE, BODY_BEGIN, FIRST_EXECSTMT, BODY_END, RETURN, EXIT, INSTRUMENTATION_POINT, START_TIMER, STOP_TIMER, START_DO_TIMER, GOTO_STOP_TIMER, START_LOOP_TIMER, STOP_LOOP_TIMER, ALLOCATE_STMT, DEALLOCATE_STMT, IO_STMT };
 enum itemAttr_t { BEFORE, AFTER, NOT_APPLICABLE};
 enum itemQualifier_t { STATIC, DYNAMIC, NOT_SPECIFIED};
 enum tau_language_t { tau_c, tau_cplusplus, tau_fortran };
@@ -40,7 +40,7 @@ struct itemRef {
 extern bool fuzzyMatch(const string& a, const string& b);
 
 /***************************************************************************
- * $RCSfile: tau_datatypes.h,v $   $Author: sameer $
- * $Revision: 1.11 $   $Date: 2007/09/16 22:04:29 $
- * VERSION_ID: $Id: tau_datatypes.h,v 1.11 2007/09/16 22:04:29 sameer Exp $
+ * $RCSfile: tau_datatypes.h,v $   $Author: geimer $
+ * $Revision: 1.13 $   $Date: 2008/12/11 16:02:22 $
+ * VERSION_ID: $Id: tau_datatypes.h,v 1.13 2008/12/11 16:02:22 geimer Exp $
  ***************************************************************************/
