@@ -44,7 +44,7 @@ enum {Standard, Verbose, Debug};
  *************************************************************/         
 class Directive
 {
-  int type;
+  const int type;
   int line;
   int col;
   int depth;
@@ -58,10 +58,23 @@ class Directive
    * int l         : the line number in the source file where the
    *                 direcitve is declared.
    */
-  Directive(int t, int l)
+  /*Directive(Directive& d) : type(d.getType())
+	{
+    line = d.getLine();
+		col =  d.getCol();
+		depth = d.getDepth();
+		block = d.getBlock();
+	}*/
+  Directive(const int t, const int l) : type(t)
   {
-    type = t;
-    line = l;
+    //type = t;
+		line = l;
+	  //printf("CONSTRCTOR (1), type: %d\n", type);
+	  if (type > 4 || type < -4)
+		{
+		  //printf("ERROR reading type.\n");
+			exit(1);
+		}
   }
   /*
    * Constructor IN: 
@@ -72,15 +85,21 @@ class Directive
    * const pdbStmt* s : The pdbStmt which follows the opening
    *                    directive - a block statement.
    */
-  Directive(Directive d, int l, const pdbStmt* s)
+  Directive(Directive d, const int l, const pdbStmt* s) : type(d.getType())
   {
-    type = d.getType();
+    //type = d.getType();
     line = s->stmtEnd().line();
     col = s->stmtEnd().col();
     if (type > 1 || type < -1 )
       col++;
     depth = l;
     block = s;
+	  //printf("CONSTRCTOR (2), type: %d\n", type);
+		if (type > 4 || type < -4)
+		{
+		  //printf("ERROR reading type.\n");
+			exit(1);
+		}
   }
   /*
    * Constructor IN: 
@@ -92,15 +111,24 @@ class Directive
    * const pdbStmt* s : The pdbStmt which follows the opening
    *                    directive - a block statement.
    */
-  Directive(Directive d, int t, int l, const pdbStmt* s)
+  Directive(Directive d, const int t, const int l, const pdbStmt* s) : type(-t)
   {
-    type = -t;
+    //type = -t;
+		//printf("hello: %d\n", s->id());
+		int k = s->kind();
+		//printf("hello: %d\n", s->id());
     line = s->stmtEnd().line();
     col = s->stmtEnd().col();
     if ((type > 1 || type < -1))
       col++;
     depth = l;
     block = s;
+	  //printf("CONSTRCTOR (3), type: %d\n", type);
+		if (type > 4 || type < -4)
+		{
+		  //printf("ERROR reading type.\n");
+			exit(1);
+		}
   }  
   /*
    * Constructor IN: 
@@ -108,13 +136,19 @@ class Directive
    * int l            : the line number in the source file where the
    *                    direcitve is declared.
    */
-  Directive(Directive d, int l)
+  Directive(Directive d, const int l) : type(d.getType())
   {
-    type = d.getType();
+    //type = d.getType();
     line = d.getLine();
     depth = l;
+	  //printf("CONSTRCTOR (4), type: %d\n", type);
+		if (type > 4 || type < -4)
+		{
+		  //printf("ERROR reading type.\n");
+			exit(1);
+		}
   }
-  int getType()
+  int getType() const 
   { return type; }
   int getCol() const
   { return col; }
@@ -122,10 +156,10 @@ class Directive
   { return line; }
   const pdbStmt* getBlock()
   { return block; }
-  int getDepth()
+  int getDepth() const 
   { return depth; }
-  void setType(int t)
-  {  type = t; }
+  /*void setType(int t)
+  {  type = t; }*/
   void setCol(int l)
   { col = l; }
   void setLine(int l)
@@ -167,12 +201,16 @@ class CompleteDirectives
   /* A list of Directives that should be inserted in this routine. */
   list<Directive> addDirectives;
   int language;
-
   /* the line number at the begining this routine. */
   int beginRoutine;
   /* the line number at the end this routine. */
   int endRoutine;
 
+  static const int STATE_CLOSED = 0;
+  static const int STATE_OPEN = 1;
+  static const int STATE_EXPECTING = 2;
+  static const int STATE_ADD = 3;
+
   public:
     
  /*
@@ -258,17 +296,22 @@ class CompleteDirectives
     
     // iterate through the pragma vector added to the list each one that appears
     // with this routine.
+		//cerr << "looking for prgmas/comments" << endl;
     for (PDB::pragmavec::iterator r = pdbHead.getPragmaVec().begin();
           r!=pdbHead.getPragmaVec().end(); r++)
-    {
-      if (verbosity == Debug)
-        cerr << "<" << (**r).prBegin().line() << "  "  << getDirectiveType(**r) << ">" << endl;
-      if (getDirectiveType(**r) != 0 && (**r).prBegin().line() > beginRoutine && (**r).prBegin().line() < endRoutine)
-      {
-        directives.push_back(Directive(getDirectiveType(**r),(**r).prBegin().line()));  
-        if (verbosity == Debug)
-          cerr << directives.back().getLine() << "  "  << directives.back().getType() << endl;
-      }
+    { 
+					//cerr << "found pragmas" << endl;
+		  if (getDirectiveType(**r) != 1 && getDirectiveType(**r) != -1)
+			{
+				if (verbosity == Debug)
+					cerr << "<" << (**r).prBegin().line() << "  "  << getDirectiveType(**r) << ">" << endl;
+				if (getDirectiveType(**r) != 0 && (**r).prBegin().line() > beginRoutine && (**r).prBegin().line() < endRoutine)
+				{
+					directives.push_back(Directive(getDirectiveType(**r),(**r).prBegin().line()));  
+					if (verbosity == Debug)
+						cerr << directives.back().getLine() << "  "  << directives.back().getType() << endl;
+				}
+			}
     }
     if (verbosity == Debug)
       cerr << "----------------" << endl;
@@ -291,33 +334,44 @@ class CompleteDirectives
   int getDirectiveType(pdbPragma p)
   {
     //Transform pragma text to lower case
+		//cerr << "in getDirectiveType." << endl;
     string text = p.text();
     std::transform (text.begin(), text.end(), text.begin(), (int(*)(int)) tolower);
     
     if (language == Fortran)
     {
-      //Match OMP PARALLEL directive
-      if (text.substr(0,17) == "!$omp parallel do")
-        return 2;
-      else if (text.substr(0,14) == "!$omp parallel")
-        return 1;
-      //Match OMP DO directive
-      else if (text.substr(0,8) == "!$omp do")
-        return 3;
-      else if (text.substr(0,9) == "!$omp for")
-        return 4;
-      //Match OMP END PARALLEL directive
-      else if (text.substr(0,21) == "!$omp end parallel do")
-        return -2;
-      else if (text.substr(0,18) == "!$omp end parallel")
-        return -1;
-      //Match OMP DO directive
-      else if (text.substr(0,12) == "!$omp end do")
-        return -3;
-      else if (text.substr(0,13) == "!$omp end for")
-        return -4;
-      else
-        return 0;
+		  int startChar = 0;
+		  startChar = text.find("$omp");
+			if (startChar != string::npos)
+			{ 
+			  /*cerr << "evaulating text=" << text << endl;
+				cerr << "starting char search at: " << startChar << endl;
+				cerr << "evaluating substr='" << text.substr(startChar,16) << "'" <<
+				endl;*/
+				//Match OMP PARALLEL directive
+				if (text.substr(startChar,16) == "$omp parallel do")
+					return 2;
+				else if (text.substr(startChar,13) == "$omp parallel")
+					return 1;
+				//Match OMP DO directive
+				else if (text.substr(startChar,7) == "$omp do")
+					return 3;
+				else if (text.substr(startChar,8) == "$omp for")
+					return 4;
+				//Match OMP END PARALLEL directive
+				else if (text.substr(startChar,20) == "$omp end parallel do")
+					return -2;
+				else if (text.substr(startChar,17) == "$omp end parallel")
+					return -1;
+				//Match OMP DO directive
+				else if (text.substr(startChar,11) == "$omp end do" ||
+				text.substr(startChar,10) == "$omp enddo")
+					return -3;
+				else if (text.substr(startChar,12) == "$omp end for")
+					return -4;
+				else
+					return 0;
+			}
     }
     else
     {
@@ -361,7 +415,8 @@ class CompleteDirectives
      * close of the routine.
      *
      *      Head statement (created) -> s1 -> ... -> s2
-     */
+    */
+		//cerr << "in findOMPStmt" << endl;
     pdbStmt head(-1);
     //pdbStmt tail(-1);
     //pdbStmt state(*s);
@@ -378,13 +433,22 @@ class CompleteDirectives
     tail.extraStmt(NULL);
     state.nextStmt(&tail);
     state.downStmt(s);*/
-    return findOMPStmt(&head,s,0, pdb);
+    static list<Directive> emptyDirectives;
+    if (directives.size() == 0 and openDirectives.size() == 0) {
+		  //cerr << "no directives to find.\n" << endl;
+      return emptyDirectives;
+    }
+		else {
+		  //cerr << "finding OMP stmts...\n" << endl;
+		  return findOMPStmt(STATE_CLOSED, &head,s,0, pdb);
+		}
   }
  /*
   * findOMPStmt     : recursivesly processes each statemet in
   *                   the routine opening and closing directives and
   *                   inserting directive when need.
   * IN:
+  * int state       : The Logical state we are in.
   * pdbStmt *s      : The statement of which to begin processing.
   * pdbStmt *block  : The statement that defines the block the s statement is
   *                   in.
@@ -395,100 +459,273 @@ class CompleteDirectives
   * list<Directive>&: A list of all the directives that need to be added to this
   *                   routine to complete each open directive.
   */
-  list<Directive>& findOMPStmt(const pdbStmt *s, const pdbStmt *block, int loop, PDB& pdb)
+  list<Directive>& findOMPStmt(const int state, const pdbStmt *s, const pdbStmt *block, int loop, PDB& pdb)
   {
+    if (verbosity == Debug)
+			printLocation(s, block, loop);
 
-      while (directives.size() != 0 && s->stmtBegin().line() >=
-                  directives.front().getLine() && (
-                  directives.front().getType() == 1 ||
-                  directives.front().getType() == -1))
-      {
-        while (directives.front().getType() == 1)
+    int currentLine = s->stmtBegin().line();
+		Directive *nextDirective = &directives.front();
+	  //int depth = loop - openDirectives.front().getDepth();
+			if (state == STATE_CLOSED)
+			{
+    		if (verbosity == Debug)
+				  printf("In state: STATE_CLOSED\n");
+			  //if we have encountered an OMP directive
+				if (directives.size() > 0)
+				{
+					if (!checkForDirective(state, s, block, loop, pdb))
+					{
+						//move to the next stmt
+						return gotoNextStmt(STATE_CLOSED, s, block, loop, pdb);
+					}
+					//printf("nothing else to do...\n");
+					return gotoNextStmt(STATE_CLOSED, s, block, loop, pdb);
+				}
+				else
+				{
+          return addDirectives;
+				}
+      }
+			else if (state == STATE_OPEN)
+			{
+				int depth = loop - openDirectives.front().getDepth();
+				if (verbosity == Debug)
+				{
+					printf("In state: STATE_OPEN\n");
+					printf("last open directive: %d type: %d\n",
+					openDirectives.front().getLine(), openDirectives.front().getType());
+					printf("directive depth: %d\n", openDirectives.front().getDepth());
+					printf("loop depth: %d\n", depth);
+        }
+    		if (openDirectives.size() == 0)
+				{
+          return findOMPStmt(STATE_CLOSED, s, block, loop, pdb);
+				}
+				if (directives.size() > 0 && (directives.front().getType() +
+				openDirectives.front().getType() == 0))
+				{
+				  //printf("checking for directives...\n");
+					checkForDirective(state, s, block, loop, pdb);
+				}
+				else
+				{
+					if (depth == 0 && openDirectives.front().getLine() <
+					(s->stmtBegin().line() - 1))
+					{
+						if (verbosity == Debug)
+					  	printf("we should see a directive.\n");
+					  //parallel do open
+						if (openDirectives.front().getType() != 1)
+						{
+							if (verbosity == Debug)
+								printf("closing do loop.\n");
+							addDirectives.splice(addDirectives.end(),findOMPStmt(STATE_EXPECTING, s, block, loop, pdb));
+							return addDirectives;
+						}
+						// end of routine or function
+						else if (followingStmt(s)->kind() == pdbStmt::ST_FRETURN ||
+						followingStmt(s)->kind() == pdbStmt::ST_FEXIT)
+						{
+							if (verbosity == Debug)
+								printf("closing parallel\n");
+							addDirectives.splice(addDirectives.end(),findOMPStmt(STATE_EXPECTING, s, block, loop, pdb));
+							//printf("back in open...\n");
+							return addDirectives;
+						}
+						// wait to end parallel directive
+						else
+						{
+						  return gotoNextStmt(STATE_OPEN, s, block, loop, pdb);
+						}  
+					}
+					else
+					{ 
+					  //if (s->nextStmt() != NULL) { printf("NOT null"); }
+						return gotoNextStmt(STATE_OPEN, s, block, loop, pdb);
+					}
+				}
+			}
+			else if (state == STATE_EXPECTING)
+			{
+    		if (verbosity == Debug)
+				  printf("In state: STATE_EXPECTING\n");
+			  
+				//is there already a directive closing the one open?
+				if (directives.front().getLine() == currentLine + 1 &&
+				directives.front().getType() + openDirectives.front().getType() == 0)
         {
-          if (verbosity == Debug)  
-            cerr << "Opening Parallel OMP Directive." << endl;
-          //openDirectives.push_front(Directive(directives.front(), loop, block));
           directives.pop_front();
-        }
-        while (directives.size() != 0 && directives.front().getType() == -1)
-        {
-          //if (openDirectives.size() == 1)
-            //cerr << "ERROR: Superfluous closing OMP Directive on line: " << directives.front().getLine() << endl;
-          if (verbosity == Debug)  
-            cerr << "Closing Parallel OMP Directive." << endl;
-          
-          directives.pop_front();
-        }
-      }
+					openDirectives.pop_front();
+					if (openDirectives.size() == 0)
+					{
+					  //printf("found.\n");
+            return gotoNextStmt(STATE_CLOSED, s, block, loop, pdb);
+					}
+					else
+					{
+					  //printf("wanting.\n");
+            return gotoNextStmt(STATE_OPEN, s, block, loop, pdb);
+					}
+				}
+				else
+				{
+					addDirectives.splice(addDirectives.end(),findOMPStmt(STATE_ADD, s, block, loop, pdb));
+					//printf("back in expecting...\n");
+					return addDirectives;
+				}  
+			}
+			else if (state == STATE_ADD)
+			{
+    		if (verbosity == Debug)
+				{
+				  printf("In state: STATE_ADD\n");
+          printf("open type: %d", openDirectives.front().getType()); 
+				}
+				addDirectives.push_front(Directive(openDirectives.front(),
+				openDirectives.front().getType(), loop, s));
+				//printf("open size: %d\n", openDirectives.size());
+				openDirectives.pop_front();
+				//printf("open size: %d\n", openDirectives.size());
+				if (openDirectives.size() < 1)
+				{
+					//printf("all directives closed.\n");
+					return gotoNextStmt(STATE_CLOSED, s, block, loop, pdb);
+				}
+				else
+				{
+					return gotoNextStmt(STATE_OPEN, s, block, loop, pdb);
+				}
+			}
+      else 
+			{
+			  return addDirectives;
+			}
+	  while (!directives.empty() and nextDirective->getLine() < s->stmtBegin().line())
+		{
+      if (verbosity == Debug)
+			  printf("passed directive, removing it.\n");
 
-      //if we have found all the directives
-      if (directives.size() == 0)
-      {
-        if (openDirectives.size() == 0)
-        {
-          if (verbosity == Debug)  
-            cerr << "all directives have been closed." << endl;
-        }
-        else
-        {
-          if (verbosity == Debug)  
-            cerr << "OMP Directive on line: " << openDirectives.front().getLine() << " has not been closed."<< endl;
-        }
-      }
+			directives.pop_front();
+		  nextDirective = &directives.front();
+		}
+			return addDirectives;
+		/*
+    while ( s->stmtBegin().line() >=
+			 directives.front().getLine() && ( openDirectives.size() > 0 ||
+			 directives.size() > 0))
+		{
+#ifdef DEBUG
+			cerr << "open type: " <<  openDirectives.front().getType() << endl;		
+#endif 
+			while (
+								directives.front().getType() == 1 ||
+								directives.front().getType() == -1)
+				{
+					while (directives.front().getType() == 1)
+					{
+#ifdef DEBUG
+					printf("h");
+#endif
+						if (verbosity == Debug)  
+							cerr << "Opening Parallel OMP Directive." << endl;
+						//openDirectives.push_front(Directive(directives.front(), loop, block));
+						directives.pop_front();
+					}
+					while (directives.size() != 0 && directives.front().getType() == -1)
+					{
+						//if (openDirectives.size() == 1)
+							//cerr << "ERROR: Superfluous closing OMP Directive on line: " << directives.front().getLine() << endl;
+						if (verbosity == Debug)  
+							cerr << "Closing Parallel OMP Directive." << endl;
+						
+						directives.pop_front();
+					}
+				}
+				//if (verbosity == Debug)  
+					//cerr << "checking whether directives have closed." << endl;
 
-      //Begin searching through statements
-      //If this statement is inside a OMP Directive
-      while (directives.size() != 0 && s->stmtBegin().line() >= 
-            directives.front().getLine() && directives.front().getType() < 0 &&
-            openDirectives.size() != 0)
-      {
-        if (openDirectives.size() != 0)
-        {
-          //atempt to close directive
-          if (openDirectives.front().getType() + directives.front().getType() == 0)
-          {
-            if (verbosity == Debug)  
-              cerr << "Closing OMP Directive type: " << directives.front().getType() << endl;
-            openDirectives.pop_front();
-            directives.pop_front();
-          }
-          else 
-          {  
-            cerr << "ERROR: Mismatched closing OMP Directive on line: " << directives.front().getType() << endl;
-            if (verbosity >= Verbose)  
-              cerr << "ERROR: mismatched closing OMP Directive on line: " <<
-              s->stmtBegin().line() << " type: " << directives.front().getType() << endl;
-            openDirectives.pop_front();
-            directives.pop_front();
-          }
-        }
-        else  
-          cerr << "ERROR: Superfluous closing OMP Directive on line: " << directives.front().getLine() << endl;
-      }
-      
-      if (openDirectives.size() != 0 && openDirectives.front().getDepth() ==
+				//if we have found all the directives
+				if (directives.size() == 0)
+				{
+					if (openDirectives.size() == 0)
+					{
+						if (verbosity == Debug)  
+							cerr << "all directives have been closed." << endl;
+					}
+					else
+					{
+						//if (verbosity == Debug)  
+							//cerr << "OMP Directive on line: " << openDirectives.front().getLine() << " has not been closed."<< endl;
+					}
+				}
+				//Begin searching through statements
+				//If this statement is inside a OMP Directive
+				//printf("directive type: %d", directives.front().getType());
+				while (directives.front().getLine() && directives.front().getType() < 0 &&
+							openDirectives.size() != 0)
+				{
+					if (openDirectives.size() != 0)
+					{
+					  printf("inside if..\n");
+						//atempt to close directive
+						if (openDirectives.front().getType() + directives.front().getType() == 0)
+						{
+							if (verbosity == Debug)  
+								cerr << "Closing OMP Directive type: " << directives.front().getType() << endl;
+							openDirectives.pop_front();
+							directives.pop_front();
+						}//if
+						else 
+						{  
+							cerr << "ERROR: Mismatched closing OMP Directive on line: " << directives.front().getType() << endl;
+							if (verbosity >= Verbose)  
+								cerr << "ERROR: mismatched closing OMP Directive on line: " <<
+								s->stmtBegin().line() << " type: " << directives.front().getType() << endl;
+							openDirectives.pop_front();
+							directives.pop_front();
+						}//else
+					}//if 
+					else  
+						cerr << "ERROR: Superfluous closing OMP Directive on line: " << directives.front().getLine() << endl;
+			  }//end while
+    
+			if (openDirectives.size() != 0 && openDirectives.front().getDepth() ==
             loop && openDirectives.front().getType() > 1 && verbosity == Debug)
         cerr << "Could be a missing directive." << endl;
       
 			int nextStmtLine = block->stmtEnd().line();
-			
+		
+      if (openDirectives.size() > 0 && openDirectives.front().getDepth() ==
+			loop)
+			  printf("shoud we consider closing directive?\n");
+
 			//Are we expecting any more pragmas to be closed before this statement?
-      while (openDirectives.size() != 0 && openDirectives.front().getDepth() ==
-      loop && openDirectives.front().getType() > 1 && ( directives.size() == 0
-      || !((nextStmtLine-1) >= directives.front().getLine() &&
-      directives.front().getType() + openDirectives.front().getType() == 0)))
+      while (openDirectives.size() != 0 && 
+				openDirectives.front().getDepth() == loop && 
+				openDirectives.front().getType() > 1 && 
+				( !((nextStmtLine-1) >= directives.front().getLine() &&
+      		directives.front().getType() + openDirectives.front().getType() == 0
+					)
+				)
+			)
       {
         if (verbosity >= Verbose)
+				{
           cerr << "We are expecting there to be a directive closing the one on line: " << openDirectives.front().getLine() << endl;
+					cerr << "...in loop " << loop << ", directive in loop " << openDirectives.front().getDepth() << endl;
+				}
         //Create a closing for/do pragma
         //createDirective(s,openDirectives.front());
         
         //if (s->downStmt() == NULL)
         //  cerr << "Directives opening a loop are only allowed immediately before the loop body." << endl;
         //else
-          addDirectives.push_front(Directive(openDirectives.front(), openDirectives.front().getType(), loop, openDirectives.front().getBlock()));
+        addDirectives.push_front(Directive(openDirectives.front(), openDirectives.front().getType(), loop, openDirectives.front().getBlock()));
         
         openDirectives.pop_front();
+				if (directives.size() == 0)
+				  return addDirectives;
       }
       
       //Are we expecting to close any parallel OMP directives.
@@ -500,7 +737,7 @@ class CompleteDirectives
         //createDirective(s,openDirectives.front());
         
         
-        
+        //printf("pop 1\n");
         openDirectives.pop_front();
       }
       
@@ -508,6 +745,7 @@ class CompleteDirectives
       directives.front().getLine() && directives.front().getType() > 1)
           
       {
+          //printf("pop 2\n");
           //open new directive        
           if (s->kind() != pdbStmt::ST_FDO && s->kind() != pdbStmt::ST_FOR)
           {  
@@ -517,12 +755,14 @@ class CompleteDirectives
           else
           {
             if (verbosity == Debug)  
-              cerr << "Opening OMP loop directive." << endl;
+              cerr << "Opening OMP loop directive, loop " << loop << endl;
             openDirectives.push_front(Directive(directives.front(), loop, s));
           }
           directives.pop_front();
       }
+      }//end while
       
+			int nextStmtLine = block->stmtEnd().line();
 
       int i = 0;
       if (verbosity == Debug) 
@@ -544,9 +784,11 @@ class CompleteDirectives
       }  
       //Move to the next statement
       //Must maintain monotomy, ie don't follow loops.
-      if (s->downStmt() != NULL && s->downStmt()->stmtBegin().line() >=
+      if ((s->downStmt() != NULL || s->kind() == pdbItem::RO_EXT) && s->downStmt()->stmtBegin().line() >=
       s->stmtBegin().line())
       {  
+        if (verbosity == Debug)
+          cerr << "recurising (a) " << endl;
         addDirectives.splice(addDirectives.end(), findOMPStmt(s->downStmt(), s, loop + 1, pdb));
         if (s->nextStmt() == NULL)
         {
@@ -559,17 +801,26 @@ class CompleteDirectives
           endBlock.nextStmt(s->nextStmt());
           endBlock.downStmt(NULL);
           endBlock.extraStmt(NULL);
+        if (verbosity == Debug)
+          cerr << "recurising (b) " << endl;
           addDirectives.splice(addDirectives.end(), findOMPStmt(&endBlock, block, loop, pdb));  
         }
       }
       if (s->extraStmt() != NULL && s->extraStmt()->stmtBegin().line() >=
-            s->stmtBegin().line())
+            s->stmtBegin().line()) 
+			{
+        if (verbosity == Debug)
+          cerr << "recurising (c) " << endl;
         addDirectives.splice(addDirectives.end(), findOMPStmt(s->extraStmt(), s, loop + 1, pdb));
+			}
 
       if (s->nextStmt() != NULL && s->nextStmt()->stmtBegin().line() >=
                   s->stmtBegin().line())
+			{
+        if (verbosity == Debug)
+          cerr << "recurising (d) " << endl;
         addDirectives.splice(addDirectives.end(), findOMPStmt(s->nextStmt(), block, loop, pdb));  
-  
+      }
       //Need to make sure that the end of this block is analyzed.
       if ((s->kind() == pdbStmt::ST_FOR || s->kind() == pdbStmt::ST_FDO)
       && (s->nextStmt() == NULL || s->nextStmt()->stmtBegin().line() == 0))
@@ -584,12 +835,253 @@ class CompleteDirectives
         closeBlock.nextStmt(NULL);
         closeBlock.downStmt(NULL);
         closeBlock.extraStmt(NULL);
+        if (verbosity == Debug)
+          cerr << "recurising (e) "<< endl;
         addDirectives.splice(addDirectives.end(), findOMPStmt(&closeBlock, block, loop, pdb));  
       }
       
   
-      return addDirectives;
-  }                            
+      return addDirectives;*/
+  }      
+  bool checkForDirective(const int state, const pdbStmt *s, const pdbStmt *block, int loop, PDB& pdb)
+	{
+    int currentLine = s->stmtBegin().line();
+		Directive nextDirective = directives.front();
+		if (currentLine >= nextDirective.getLine())
+		{
+			//if the directive is valid
+			if (//loop == nextDirective.getDepth() && 
+			    nextDirective.getType() > 0 && (directives.size() > 0 ))
+			{
+				if (verbosity == Debug)
+				{
+					cerr << "open type: " <<  directives.front().getType() << endl;		
+				}	
+        if (verbosity == Debug)  
+					cerr << "Opening OMP loop directive, loop " << loop << endl;
+
+				openDirectives.push_front(directives.front());
+        openDirectives.front().setDepth(loop);
+        openDirectives.front().setBlock(s);
+				directives.pop_front();
+				//list<Directive> *newDirectives = findOMPStmt(STATE_OPEN,s,block,loop,pdb);
+				addDirectives.splice(addDirectives.end(),findOMPStmt(STATE_OPEN,s,block,loop,pdb));
+			}
+			else
+			{
+				printf("WARNING: OMP directives at line: %d maybe be mismatched.\n", s->stmtBegin().line());
+				directives.pop_front();
+				addDirectives.splice(addDirectives.end(),findOMPStmt(state, s, block, loop, pdb));
+			}
+					//printf("back in check returning true...\n");
+			return true;
+		}
+					//printf("back in check returning false...\n");
+		return false;
+	}
+	const pdbStmt* followingStmt(const pdbStmt *s)
+	{
+		if ((s->downStmt() != NULL || s->kind() == pdbItem::RO_EXT) && s->downStmt()->stmtBegin().line() >=
+		s->stmtBegin().line())
+		{
+		  return s->downStmt();
+		}
+		else if (s->extraStmt() != NULL && s->extraStmt()->stmtBegin().line() >=
+					s->stmtBegin().line()) 
+		{
+		  return s->extraStmt();
+		}
+		else if (s->nextStmt() != NULL && s->nextStmt()->stmtBegin().line() >=
+								s->stmtBegin().line())
+    {
+      return s->nextStmt();
+		}
+		else if ((s->kind() == pdbStmt::ST_FOR || s->kind() == pdbStmt::ST_FDO)
+		&& (s->nextStmt() == NULL || s->nextStmt()->stmtBegin().line() == 0))
+		{
+      //return s->stmtEnd();
+			return NULL;
+		}
+		else
+		{
+		  //printf("Fatal Error: Cannot parse PDB file.");
+      //exit(1);
+			return NULL;
+		}
+	}
+  list<Directive>& gotoNextStmt(const int state, const pdbStmt *s, const pdbStmt *block, int loop, PDB& pdb)
+	{
+		if (verbosity == Debug)
+			printf("going to next stmt, at line: %d \n", s->stmtBegin().line());
+	  //are we at the start of a loop
+    if ((s->kind() == pdbStmt::ST_FOR || s->kind() == pdbStmt::ST_FDO ||
+		s->kind() == pdbStmt::ST_FIF) && s->downStmt() != NULL)
+    {
+			//enter the loop
+			if (verbosity == Debug)
+				printf("recurising (down) \n");
+			addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->downStmt(), s, loop + 1, pdb));
+			
+			//process the else in an if stmt.
+			if (s->extraStmt() != NULL && s->extraStmt()->stmtBegin().line() >=
+					s->stmtBegin().line())
+			{
+				if (verbosity == Debug)
+					printf("recurising (extra) \n");
+				addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->extraStmt(), s, loop + 1, pdb));
+			}
+			
+			//process the end of the loop by creating a stmt located there.
+				if (verbosity == Debug)
+					printf("ending block: %d\n", s->stmtEnd().line());
+				pdbFile f(-1);
+				pdbStmt endBlock(-1);
+				endBlock.stmtBegin(pdbLoc(&f,s->stmtEnd().line(),s->stmtEnd().col()));
+				endBlock.stmtEnd(pdbLoc(&f,s->stmtEnd().line(),s->stmtEnd().col()));
+				endBlock.nextStmt(s->nextStmt());
+				endBlock.downStmt(NULL);
+				endBlock.extraStmt(NULL);
+			if (verbosity == Debug)
+				printf("recurising (end block) \n");
+      //end of the loop
+      addDirectives.splice(addDirectives.end(), findOMPStmt(state, &endBlock, block, loop, pdb));  
+
+		  if (s->nextStmt() != NULL)
+			{
+			  if (state == STATE_OPEN)	
+				{
+					addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->nextStmt(), block, loop, pdb));  
+				}
+				else
+				{
+					const pdbStmt *last = s;
+					while (last->nextStmt() != NULL)
+					{
+						last = last->nextStmt();
+					}
+				if (verbosity == Debug)
+					printf("recurising (next after loop) \n");
+				addDirectives.splice(addDirectives.end(), findOMPStmt(state, last, block, loop, pdb));  
+				}
+			}
+		}
+		else 
+		{
+		  if (s->nextStmt() != NULL)
+			{
+				if (verbosity == Debug)
+					printf("recurising (next) \n");
+				addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->nextStmt(), block, loop, pdb));  
+			}
+			if (s->extraStmt() != NULL && s->extraStmt()->stmtBegin().line() >=
+			          s->stmtBegin().line())
+			{
+				if (verbosity == Debug)
+					printf("recurising (extra) \n");
+				addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->extraStmt(), s, loop + 1, pdb));
+			}
+			else
+			{
+				if (verbosity == Debug)
+					printf("no more statements, returning \n");
+        addDirectives;
+			}
+    }
+		return addDirectives;
+		}
+	  /*
+		if (verbosity == Debug)
+			printf("going to next stmt, at line: %d \n", s->stmtBegin().line());
+		//Move to the next statement
+		//Must maintain monotomy, ie don't follow loops.
+    //if (s->nextStmt() != NULL) { printf("[1] moving to next stmt.\n"); }
+		if ((s->downStmt() != NULL || s->kind() == pdbItem::RO_EXT) && s->downStmt()->stmtBegin().line() >=
+		s->stmtBegin().line())
+		{  
+			if (verbosity == Debug)
+				printf("recurising (a) \n");
+			addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->downStmt(), s, loop + 1, pdb));
+			if (s->nextStmt() == NULL || s->nextStmt()->stmtBegin().line() >= s->stmtBegin().line())
+			{
+				if (verbosity == Debug)
+					printf("ending block: %d\n", s->stmtEnd().line());
+				pdbFile f(-1);
+				pdbStmt endBlock(-1);
+				endBlock.stmtBegin(pdbLoc(&f,s->stmtEnd().line(),s->stmtEnd().col()));
+				endBlock.stmtEnd(pdbLoc(&f,s->stmtEnd().line(),s->stmtEnd().col()));
+				endBlock.nextStmt(s->nextStmt());
+				endBlock.downStmt(NULL);
+				endBlock.extraStmt(NULL);
+			if (verbosity == Debug)
+				printf("recurising (b) \n");
+				addDirectives.splice(addDirectives.end(), findOMPStmt(state, &endBlock, block, loop, pdb));  
+			}
+		}
+    //if (s->nextStmt() != NULL) { printf("[3] moving to next stmt.\n"); }
+		else if (s->extraStmt() != NULL && s->extraStmt()->stmtBegin().line() >=
+					s->stmtBegin().line()) 
+		{
+			if (verbosity == Debug)
+				printf("recurising (c) \n");
+			addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->extraStmt(), s, loop + 1, pdb));
+		}
+    //if (s->nextStmt() != NULL) { printf("moving to next stmt.\n"); }
+		else if (s->nextStmt() != NULL && s->nextStmt()->stmtBegin().line() >=
+								s->stmtBegin().line())
+		{
+			if (verbosity == Debug)
+				printf("recurising (d) \n");
+			addDirectives.splice(addDirectives.end(), findOMPStmt(state, s->nextStmt(), block, loop, pdb));  
+		}
+		//Need to make sure that the end of this block is analyzed.
+		else if ((s->kind() == pdbStmt::ST_FOR || s->kind() == pdbStmt::ST_FDO)
+		&& (s->nextStmt() == NULL || s->nextStmt()->stmtBegin().line() == 0))
+		{
+			if (verbosity == Debug)
+				printf("ending block\n");
+				
+			pdbFile file(-1);
+			pdbStmt closeBlock(-1);
+			closeBlock.stmtBegin(pdbLoc(&file,block->stmtEnd().line(),s->stmtEnd().col()));
+			closeBlock.stmtEnd(pdbLoc(&file,block->stmtEnd().line(),s->stmtEnd().col()));
+			closeBlock.nextStmt(NULL);
+			closeBlock.downStmt(NULL);
+			closeBlock.extraStmt(NULL);
+			if (verbosity == Debug)
+				printf("recurising (e) \n");
+			addDirectives.splice(addDirectives.end(), findOMPStmt(state, &closeBlock,
+			block, loop - 1, pdb));  
+		}
+		if (verbosity == Debug)
+			printf("stopped recurising.\n");
+		
+		return addDirectives;*/
+  void printLocation(const pdbStmt* s, const pdbStmt* block, int loop)
+	{
+      int i = 0;
+			int nextStmtLine = block->stmtEnd().line();
+      if (verbosity == Debug) 
+      {
+        while (i < loop) 
+        { printf("\t"); i++; }
+        printf( "%d(%d)",s->stmtBegin().line(), s->stmtBegin().col());
+        if (s->downStmt() != NULL)
+          printf(" |%d",(*s->downStmt()).stmtBegin().line());
+        else 
+          printf(" |NA");
+        if (s->nextStmt() != NULL)
+          printf(" -%d",(*s->nextStmt()).stmtBegin().line());
+        else
+          printf(" -NA");
+        if (s->extraStmt() != NULL)
+          printf(" *%d",(*s->extraStmt()).stmtBegin().line());
+        else
+          printf(" *NA");
+        printf(" L%d  N%d", block->stmtBegin().line(), nextStmtLine);
+				printf("  D%d",directives.size());
+        printf("  O%d  L%d\n", openDirectives.size(), loop);
+      }
+	}
 
   void createDirective(const pdbStmt* s, Directive match)
   {
@@ -768,6 +1260,8 @@ int main(int argc, char *argv[])
           for (PDB::froutinevec::iterator r = p.getFRoutineVec().begin();
             r!=p.getFRoutineVec().end(); r++)
           {
+            if (verbosity >= Verbose)
+              cerr << "preprocesing rountine: " << (*r)->name() << endl;
             if ((*r)->body() != NULL && (*r)->kind() != pdbItem::RO_EXT)
             {
               if (verbosity >= Verbose)
@@ -777,7 +1271,8 @@ int main(int argc, char *argv[])
               
               //Retrive the statement within the routines.
               const pdbStmt *v = (*r)->body();
-            directivesToBeAdded.splice(directivesToBeAdded.end(), c.findOMPStmt(v,p));
+	            //printf("hello\n");
+							directivesToBeAdded.splice(directivesToBeAdded.end(), c.findOMPStmt(v,p));
             }
           }
         //}
@@ -839,6 +1334,7 @@ int main(int argc, char *argv[])
       it++;
       *output << c;
     }
+		*output << endl;
   }
   while (getline(input, buffer) != 0)
   {
@@ -848,6 +1344,6 @@ int main(int argc, char *argv[])
 }
 /***************************************************************************
  * $RCSfile: tau_ompcheck.cpp,v $   $Author: scottb $
- * $Revision: 1.12 $   $Date: 2006/05/30 19:25:22 $
- * VERSION_ID: $Id: tau_ompcheck.cpp,v 1.12 2006/05/30 19:25:22 scottb Exp $
+ * $Revision: 1.33 $   $Date: 2009/12/19 00:40:42 $
+ * VERSION_ID: $Id: tau_ompcheck.cpp,v 1.33 2009/12/19 00:40:42 scottb Exp $
  ***************************************************************************/
