@@ -18,6 +18,7 @@
 #include "BPatch_function.h"
 #include "BPatch_thread.h"
 #include "BPatch_snippet.h" 
+#include "BPatch_statement.h" 
 
 //#include <iostream.h>
 //#include <stdio.h>
@@ -57,6 +58,74 @@ extern bool processFileForInstrumentation(const string& file_name);
 extern void printExcludeList();
 extern bool instrumentEntity(const string& function_name);
 
+/* prototypes for routines below */
+int getFunctionFileLineInfo(BPatch_image* mutateeAddressSpace, 
+      BPatch_function *f, char *newname);
+
+
+/* re-writer */
+BPatch_function *name_reg;
+BPatch_Vector<BPatch_snippet *> funcNames;
+int addName(char *name)
+{
+   static int funcID = 0;
+   BPatch_constExpr *name_param = new BPatch_constExpr(name);
+   BPatch_constExpr *id_param = new BPatch_constExpr(funcID);
+   BPatch_Vector<BPatch_snippet *> params;
+   params.push_back(name_param);
+   params.push_back(id_param);
+
+   BPatch_funcCallExpr *call = new BPatch_funcCallExpr(*name_reg, params);
+   funcNames.push_back(call);
+   return funcID++;
+}
+
+void insertTrace(BPatch_function* functionToInstrument,
+                 BPatch_addressSpace* mutatee,
+                 BPatch_function* traceEntryFunc,
+                 BPatch_function* traceExitFunc)
+{
+  char name[1024];
+  char modname[1024];
+
+  functionToInstrument->getModuleName(modname, 1024);
+  if (strstr(modname, "libdyninstAPI_RT"))
+     return;
+
+  //functionToInstrument->getName(name, 1024);
+  getFunctionFileLineInfo(mutatee->getImage(), functionToInstrument, name);
+  int id = addName(name);
+  BPatch_Vector<BPatch_snippet *> traceArgs;
+  traceArgs.push_back(new BPatch_constExpr(id));
+
+  BPatch_Vector<BPatch_point*>* funcEntry = functionToInstrument->findPoint(BPatch_entry);
+  BPatch_Vector<BPatch_point*>* funcExit = functionToInstrument->findPoint(BPatch_exit);
+
+  BPatch_funcCallExpr entryTrace(*traceEntryFunc, traceArgs);
+  BPatch_funcCallExpr exitTrace(*traceExitFunc, traceArgs);
+
+  mutatee->insertSnippet(entryTrace, *funcEntry, BPatch_callBefore, BPatch_lastSnippet);
+  mutatee->insertSnippet(exitTrace, *funcExit, BPatch_callAfter, BPatch_lastSnippet);
+}
+
+struct boundFindFunc
+{
+  boundFindFunc(BPatch_image* img, BPatch_Vector<BPatch_function*>& funcPtrs) : m_image(img), m_outputStorage(funcPtrs)
+  {
+  }
+  void operator()(const char* name)
+  {
+    m_image->findFunction(name, m_outputStorage);
+  }
+
+  BPatch_image* m_image;
+  BPatch_Vector<BPatch_function*>& m_outputStorage;
+
+};
+
+
+
+
 //
 // Error callback routine. 
 //
@@ -243,9 +312,16 @@ int moduleConstraint(char *fname){ // fname is the name of module/file
        ((fname[len-4] == '.') && (fname[len-3] == 'F') && (fname[len-2] == '9') && (fname[len-1] == '0')) || 
        ((fname[len-2] == '.') && (fname[len-1] == 'F')) || 
        ((fname[len-2] == '.') && (fname[len-1] == 'f')) || 
+       //((fname[len-3] == '.') && (fname[len-2] == 's') && (fname[len-1] == 'o'))|| 
        (strcmp(fname, "LIBRARY_MODULE") == 0)){
       /* It is ok to instrument this module. Constraint doesn't exist. */
-      return false;
+      // Wait: first check if it has libTAU* in the name!
+      if (strncmp(fname, "libTAU", 6) == 0)  {
+        return true;  /* constraint applies - do not instrument! */
+      }
+      else {
+        return false; /* ok to instrument */
+      }
     }//if
     else
       return true;
@@ -293,27 +369,142 @@ int routineConstraint(char *fname){ // fname is the function name
 // check if the application has an MPI library routine MPI_Comm_rank
 // 
 int checkIfMPI(BPatch_image * appImage, BPatch_function * & mpiinit,
-		BPatch_function * & mpiinitstub){
+		BPatch_function * & mpiinitstub, bool binaryRewrite){
 
-  mpiinit 	= tauFindFunction(appImage, "PMPI_Comm_rank");
+  mpiinit 	= tauFindFunction(appImage, "MPI_Comm_rank");
   mpiinitstub 	= tauFindFunction(appImage, "TauMPIInitStub");
 
   if (mpiinitstub == (BPatch_function *) NULL)
     printf("*** TauMPIInitStub not found! \n");
   
   if (mpiinit == (BPatch_function *) NULL) {
-    dprintf("*** PMPI_Comm_rank not found looking for MPI_Comm_rank...\n");
-    mpiinit = tauFindFunction(appImage, "MPI_Comm_rank");
+    dprintf("*** MPI_Comm_rank not found looking for PMPI_Comm_rank...\n");
+      //mpiinit = tauFindFunction(appImage, "PMPI_Comm_rank");
+    mpiinit = tauFindFunction(appImage, "mpi_comm_rank_");
   }//if
   
   if (mpiinit == (BPatch_function *) NULL) { 
-    dprintf("*** MPI_Comm_rank also not found. This is not an MPI Application! \n");
+    dprintf("*** This is not an MPI Application! \n");
     return 0;  // It is not an MPI application
   }//if
   else
     return 1;   // Yes, it is an MPI application.
 }//checkIfMPI()
 
+/* We create a new name that embeds the file and line information in the name */
+int getFunctionFileLineInfo(BPatch_image* mutateeAddressSpace, 
+      BPatch_function *f, char *newname)
+{
+  bool info1, info2;
+  unsigned long baseAddr,lastAddr;
+  char fname[1024];
+  const char *filename;
+  int row1, col1, row2, col2;
+  baseAddr = (unsigned long)(f->getBaseAddr());
+  lastAddr = baseAddr + f->getSize();
+  BPatch_Vector< BPatch_statement > lines;
+  f->getName(fname, 1024);
+  info1 = mutateeAddressSpace->getSourceLines((unsigned long) baseAddr, lines);
+
+  if (info1) {
+    filename = lines[0].fileName();
+    row1 = lines[0].lineNumber();
+    col1 = lines[0].lineOffset();
+    if (col1 < 0) col1 = 0;
+    info2 = mutateeAddressSpace->getSourceLines((unsigned long) (lastAddr -1), lines);
+    if (info2) {
+      row2 = lines[1].lineNumber();
+      col2 = lines[1].lineOffset();
+      if (col2 < 0) col2 = 0;
+      sprintf(newname, "%s [{%s} {%d,%d}-{%d,%d}]", fname, filename, row1, col1, row2, col2);
+    } else {
+      sprintf(newname, "%s [{%s} {%d,%d}]", fname, filename, row1, col1);
+    }
+  }
+  else
+    strcpy(newname, fname);
+
+}
+
+
+int tauRewriteBinary(BPatch *bpatch, const char *mutateeName, char *outfile, char* libname)
+{
+  using namespace std;
+  BPatch_function *mpiinit;
+  BPatch_function *mpiinitstub;
+
+  dprintf("Inside tauRewriteBinary, name=%s, out=%s\n", mutateeName, outfile);
+  BPatch_binaryEdit* mutateeAddressSpace = bpatch->openBinary(mutateeName, false);
+
+  bpatch->setLivenessAnalysis(false);
+  mutateeAddressSpace->allowTraps(false);
+  BPatch_image* mutateeImage = mutateeAddressSpace->getImage();
+  BPatch_Vector<BPatch_function*>* allFuncs = mutateeImage->getProcedures();
+
+  bool result = mutateeAddressSpace->loadLibrary(libname);
+  assert(result);
+  BPatch_function* entryTrace = tauFindFunction(mutateeImage, "traceEntry");
+  BPatch_function* exitTrace = tauFindFunction(mutateeImage, "traceExit");
+  BPatch_function* setupFunc = tauFindFunction(mutateeImage, "tau_dyninst_init");
+  BPatch_function* cleanupFunc = tauFindFunction(mutateeImage, "tau_dyninst_cleanup");
+  BPatch_function* mainFunc = tauFindFunction(mutateeImage, "main");
+  name_reg = tauFindFunction(mutateeImage, "trace_register_func");
+
+  if(!mainFunc)
+  {
+    fprintf(stderr, "Couldn't find main(), aborting\n");
+    return -1;
+  }
+  if(!entryTrace || !exitTrace || !setupFunc || !cleanupFunc)
+  {
+    fprintf(stderr, "Couldn't find OTF functions, aborting\n");
+    return -1;
+  }
+
+  BPatch_Vector<BPatch_point*>* mainEntry = mainFunc->findPoint(BPatch_entry);
+  assert(mainEntry);
+  assert(mainEntry->size());
+  assert((*mainEntry)[0]);
+
+  mutateeAddressSpace->beginInsertionSet();
+
+  BPatch_constExpr isMPI(checkIfMPI(mutateeImage, mpiinit, mpiinitstub, true));
+  BPatch_Vector<BPatch_snippet*> init_params;
+  init_params.push_back(&isMPI);
+  BPatch_funcCallExpr setup_call(*setupFunc, init_params);
+  funcNames.push_back(&setup_call);
+
+  for (BPatch_Vector<BPatch_function*>::iterator it=allFuncs->begin();
+  it != allFuncs->end(); it++)
+  {
+    char fname[FUNCNAMELEN];
+    (*it)->getName(fname, FUNCNAMELEN);
+    dprintf("Processing %s...\n", fname); 
+    if (!routineConstraint(fname)) { // ok to instrument
+      insertTrace(*it, mutateeAddressSpace, entryTrace, exitTrace);
+    }
+    else {
+     dprintf("Not instrumenting %s\n", fname);
+    }
+  }
+
+
+  BPatch_sequence sequence(funcNames);
+  mutateeAddressSpace->insertSnippet(sequence,
+                                     *mainEntry,
+                                     BPatch_callBefore,
+                                     BPatch_firstSnippet);
+
+  mutateeAddressSpace->finalizeInsertionSet(false, NULL);
+
+  std::string modifiedFileName(outfile);
+  chdir("result");
+  mutateeAddressSpace->writeFile(modifiedFileName.c_str());
+  return 0;
+}
+
+
+
 
 //
 // entry point 
@@ -331,7 +522,16 @@ int main(int argc, char **argv){
   BPatch_function *mpiinitstub;
   bpatch = new BPatch;                           //create a new version. 
   string functions;                              //string variable to hold function names 
+  // commandline option processing args
+  int vflag = 0;
+  char *xvalue = NULL;
+  char *fvalue = NULL;
+  char *ovalue = NULL;
+  int index;
+  int c;
 
+
+  bpatch->setTrampRecursive(true); /* enable C++ support */
   // parse the command line arguments--first, there need to be atleast two arguments,
   // the program name (tau_run) and the application it is running.  If there are not
   // at least two arguments, set the error flag.
@@ -342,39 +542,62 @@ int main(int argc, char **argv){
   //then set the errflag to report an error.  When we come to a non '-' charcter, then we must
   //be at the application name.
   else{
-    strncpy(mutname, argv[0],strlen(argv[0])+1);
-    while(argv[1][0]=='-'){
-      if( strncasecmp (argv[1], "-Xrun", 5) == 0 ){ // Load the library.
-	loadlib = true;
-	sprintf(libname,"lib%s.so", &argv[1][5]);
-	fprintf(stderr, "%s> Loading %s ...\n", mutname, libname);
-	argv++;
-      }//if
-      else if (strncasecmp (argv[1], "-f", 2) == 0){ // Load the selective instrumentation file
-	processInstrumentationRequests(argv[2]);
-	dprintf("Loading instrumentation requests file %s\n", argv[2]);
-	argv += 2;
-      }//if
-      else if (strncasecmp (argv[1], "-v", 2) == 0) { 
-        debugPrint = 1; /* Verbose option set */
-        argv++;
-      }
-      else if (strncasecmp (argv[1], "-o", 2) == 0) {
-        binaryRewrite = 1; /* binary rewrite is true */
-        strcpy(outfile, argv[2]);
-        argv += 2;
-      }
-      else{ //oops! we got an unrecognized argument!
-	errflag=1;
-      }//else
-    }//while
-  }//else
+    opterr = 0; 
+     
+    while ((c = getopt (argc, argv, "vX:o:f:")) != -1)
+      switch (c)
+      {
+        case 'v':
+          vflag = 1;
+          debugPrint = 1; /* Verbose option set */
+          break;
+        case 'X':
+          xvalue = optarg;
+	  loadlib = true; /* load an external measurement library */
+	  sprintf(libname,"lib%s.so", &xvalue[3]);
+          break;
+        case 'f':
+          fvalue = optarg; /* choose a selective instrumentation file */
+	  processInstrumentationRequests(fvalue);
+	  dprintf("Loading instrumentation requests file %s\n", fvalue);
+          break;
+        case 'o':
+          ovalue = optarg;
+          binaryRewrite = 1; /* binary rewrite is true */
+          strcpy(outfile, ovalue);
+          break;
+        case '?':
+          if (optopt == 'X' || optopt == 'f' || optopt == 'o' )
+            fprintf (stderr, "Option -%c requires an argument.\n", optopt);
+          else if (isprint (optopt))
+            fprintf (stderr, "Unknown option `-%c'.\n", optopt);
+          else
+            fprintf (stderr,
+                        "Unknown option character `\\x%x'.\n",
+                        optopt);
+	    errflag=1;
+          default:
+	    errflag=1;
+        }
+     
+      dprintf ("vflag = %d, xvalue = %s, ovalue = %s, fvalue = %s\n",
+               vflag, xvalue, ovalue, fvalue);
+     
+      strncpy(mutname, argv[optind],strlen(argv[optind])+1);
+      for (index = optind; index < argc; index++)
+        dprintf ("Non-option argument %s\n", argv[index]);
+    }
+
+  dprintf("mutatee name = %s\n", mutname);
   
   //did we load a library?  if not, load the default
   if(!loadlib){
     sprintf(libname,"libTAU.so");
     loadlib=true;
   }//if
+  else {
+    fprintf(stderr, "%s> Loading %s ...\n", mutname, libname);
+  }
 
   //has an error occured in the command line arguments?
   if(errflag){
@@ -396,10 +619,15 @@ int main(int argc, char **argv){
   // getting an assertion failure under Linux otherwise 
   // bpatch->setDebugParsing(false); 
   // removed for DyninstAPI 4.0
+
+  if (binaryRewrite) {
+    tauRewriteBinary(bpatch, mutname, outfile, (char *)libname);
+    return 0; // exit from the application 
+  }
 #ifdef TAU_DYNINST41PLUS
-  appThread = bpatch->createProcess(argv[1], (const char **)&argv[1] , NULL);
+  appThread = bpatch->createProcess(argv[optind], (const char **)&argv[optind] , NULL);
 #else
-  appThread = bpatch->createProcess(argv[1], &argv[1] , NULL);
+  appThread = bpatch->createProcess(argv[optind], &argv[optind] , NULL);
 #endif /* TAU_DYNINST41PLUS */
   dprintf("After createProcess\n");
 
@@ -474,6 +702,8 @@ int main(int argc, char **argv){
            dprintf("don't instrument %s\n", fname);
         }//if
         else{ // routines that are ok to instrument
+          // get full source information
+          getFunctionFileLineInfo(appImage, (*p)[i], fname);
 	  functions.append("|");
 	  functions.append(fname);
 	}//else
@@ -494,7 +724,7 @@ int main(int argc, char **argv){
   // way, it works for both MPI and sequential tasks. 
   
   bpatch->registerErrorCallback(errorFuncNull); // turn off error reporting
-  BPatch_constExpr isMPI(checkIfMPI(appImage, mpiinit, mpiinitstub));
+  BPatch_constExpr isMPI(checkIfMPI(appImage, mpiinit, mpiinitstub,binaryRewrite));
   bpatch->registerErrorCallback(errorFunc1); // turn it back on
 
   initArgs.push_back(&funcName);
@@ -587,7 +817,10 @@ int main(int argc, char **argv){
     }
     else
     {
-      fprintf(stderr, "Error: Binary rewriting did not work: No directory name returned\n");
+      fprintf(stderr, "Error: Binary rewriting did not work: No directory name \
+returned\n\nIf you are using Dyninst 5.2 this feature is no longer \
+supported and \
+tau_run will run the application using dynamic instrumentation....\n");
     }
     delete bpatch;
     return 0;
